/**
 * # Copyright (c) 2008 BaseSoft B.V.
 * # See LICENSE for details.
 *
 * This TestCase tests the JSONRPC class.
 */

(function() {
	"use strict";

BASESOFT.test.TestCases.push(new BASESOFT.tool.TestCase({

	name: "JSonRPC test",
	assertClean: true,

	setUp : function () {

		this.broker = new BASESOFT.util.JsonBroker('/basesoft.web.test/selenium/jsonrpc/simple/', {executeIfLoggedOut: true});
		this._garbage.push(this.broker);
		BASESOFT.env.hasOutdatedSources = false;
	},

	_tearDown: function() {
		this.broker = null;
	},

	/**
	 * Tests the multiply call which takes two argumens
	 */
	testMultiply : function () {
		this.broker.call('multiply', [2, 8],
			function(result) {
				this.resume(function() {
					YAHOO.util.Assert.areSame(16, result, "Result should be 16");
				});
			},
			function(failure) {
				this.resume(function() {
					YAHOO.util.Assert.fail('received failure');
				});
			 },
			this);

		// this tells the test to wait for a resume() call.. see up
		this.wait();
	},

	testBoolean: function() {

		this.broker.call('echo', [[true, false]],
			function(rv) {
				this.resume(function() {
					console.warn('rv', rv);

					YAHOO.util.Assert.areSame(rv[0], true);
					YAHOO.util.Assert.areSame(rv[1], false);

				});
			},
			function(failure) {
				this.resume(function() {
					YAHOO.util.Assert.fail('received failure');
				});
			},
			this);
		this.wait();

	},

	/**
	 * Tests a complex utf8 string
	 */
	testUnicode : function () {

		var utf8str  = "enqu\u00eate";

		this.broker.call('echo_unicode', [utf8str],
			function(result) {
				this.resume(function() {
					YAHOO.util.Assert.areSame(utf8str, result, "utf8 test failed");
				});
			},
			function(failure) {
				this.resume(function() {
					YAHOO.util.Assert.fail('received failure');
				});
			},
			this);
		this.wait();
	},

	/**
	 * tests a string without 'strange' chars which should arrive at the server as unicode too
	 */
	testUnicode2 : function () {

		var utf8str  = "normal string";

		this.broker.call('echo_unicode', [utf8str],
			function(result) {
				this.resume(function() {
					YAHOO.util.Assert.areSame(utf8str, result, "utf8 test failed");
				});
			},
			function(failure) {
				this.resume(function() {
					YAHOO.util.Assert.fail('received failure');
				});
			},
			this);
		this.wait();
	},

	/**
	 * test encoding a null value
	 */
	testNull: function() {
		this.broker.call('echo', [null],
			function(result) {
				this.resume(function() {
					YAHOO.util.Assert.areSame(null, result, "null encoding test failed");
				});
			},
			function(failure) {
				this.resume(function() {
					YAHOO.util.Assert.fail('received failure');
				});
			},
			this);
		this.wait();
	},

	testTypes : function() {

		var areEqualRecursive = function(expected, actual)
		{
			for (var i in expected) {
				if (typeof expected[i] == 'object')
					areEqualRecursive(expected[i], actual[i]);
				else
					YAHOO.util.Assert.areSame(actual[i], expected[i]);
			}
		};

		var d  = {
			value1: 'string1',
			value2: 'string2',
			value3: ['string3', 5, 'braat']
		};

		this.broker.call('echo', [d],
			function(result) {
				this.resume(function() {
					areEqualRecursive(d, result);
				});
			},
			function(failure) {
				this.resume(function() {
					YAHOO.util.Assert.fail('received failure');
				});
			 },
			this);

		this.wait();
	},

	/**
	 * Tests receiving an failure by calling an invalid method
	 */
	testFailureInvalidMethod: function() {
		this.broker.call('invalidmethod', [],
			function(result) {
				this.resume(function() {
					 YAHOO.util.Assert.fail('success callback should not get called');
				});
			},
			function(failure){
				this.resume(function() {
					YAHOO.util.Assert.isInstanceOf(Error, failure);
				});
			},
			this);
		this.wait();
	},

	/**
	 * Tests receiving an failure by passing too many arguments
	 */
	testFailureTooManyArguments: function() {
		this.broker.call('multiply', [1, 2, 3],
			function(result) {
				this.resume(function() {
					 YAHOO.util.Assert.fail('success callback should not get called');
				});
			},
			function(failure){
				this.resume(function() {
					YAHOO.util.Assert.isInstanceOf(Error, failure);
				});
			},
			this);
		this.wait();
	},

	/**
	 * This tests if special post vars break the decoding serverside.. this was a bug
	 */
	testPostChars: function() {

		var str  = "&anothervar=kwak";

		this.broker.call('echo', [str],
			function(result) {
				this.resume(function() {
					YAHOO.util.Assert.areSame(str, result, "testPostChars failed");
				});
			},
			function(failure) {
				this.resume(function() {
					YAHOO.util.Assert.fail('received failure');
				});
			 },
			this);

		this.wait();
	},

	/*
	 * test raising public user exceptions
	 */
	testRaisePublicUserException :function() {
		this.broker.call('raise_public_user_exception', [],
			function(result) {
				this.resume(function() {
					YAHOO.util.Assert.fail('received success');
				});
			 },
			function(failure) {
				this.resume(function() {

					console.warn('failure', failure);

						YAHOO.util.Assert.isInstanceOf(Error, failure);
						YAHOO.util.Assert.areSame(failure.message, "public user exception raised with utf-8: È");
					});
			},
			this);

		this.wait();
	},

	/*
	 * test raising public system exceptions
	 */
	testRaisePublicSystemException :function() {
		this.broker.call('raise_public_system_exception', [],
			function(result) {
				this.resume(function() {
					YAHOO.util.Assert.fail('received sucess');
				});
			 },
			function(failure) {
				this.resume(function() {
					YAHOO.util.Assert.isInstanceOf(Error, failure);
					YAHOO.util.Assert.areSame(failure.message, "public system exception raised");
				});
			},
			this);

		this.wait();
	},

	/*
	 * test raising private exceptions
	 */
	testRaisePrivateException :function() {
		this.broker.call('raise_private_exception', [],
			function(result) {
				this.resume(function() {
					YAHOO.util.Assert.fail('received success');
				});
			 },
			function(failure) {
				this.resume(function() {
					YAHOO.util.Assert.isInstanceOf(Error, failure);
					YAHOO.util.Assert.areNotSame(failure.message, "private exception");
				});
			},
			this);

		this.wait();
	},

	/*
	 * test raising outdated sources exceptions
	 */
	testRaiseOutdatedSourcesException :function() {
		BASESOFT.env.hasOutdatedSources = true;
		//fail the rpc call by providing more arguments and check that failure Cb raises OutdatedSources message
		//usually BASESOFT.env.hasOutdatedSources = true on if we have OutDatedSourcesException keepalive failure
		this.broker.call('multiply', [1, 2, 3],
			function(result) {
				this.resume(function() {
					YAHOO.util.Assert.fail('received success');
				});
			 },
			function(failure) {
				this.resume(function() {
					YAHOO.util.Assert.isInstanceOf(Error, failure);
					YAHOO.util.Assert.areSame(failure.message, "You have outdated sources, please refresh your browser and try again");
				});
			},
			this);

		this.wait();
	},

	testViewException: function() {

		var broker = new BASESOFT.util.JsonBroker('/some/non/existing/view/', {executeIfLoggedOut: true});
		broker.call('non_existing_method', [],
			function(result) {
				this.resume(function() {
					YAHOO.util.Assert.fail('received success');
				});
			 },
			function(failure) {
				this.resume(function() {
					YAHOO.util.Assert.isInstanceOf(Error, failure);
				});
			},
			this);

		this.wait();
	},


	/**
	 * tests encoding and decoding of datetime objects
	 */
	testDateTime: function() {

		var x1 = new Date(2009, 10, 5, 0, 3, 18);
		var x2 = new Date(2009, 3, 5, 0, 3, 18);
		// Date only test.
		var x3 = new Date(2009, 4, 5);

		this.broker.call('date_test', [x1, x2],
			function(result) {
				this.resume(function() {
					YAHOO.util.DateAssert.datesAreEqual(x1, result[0]);
					YAHOO.util.DateAssert.timesAreEqual(x1, result[0]);
					YAHOO.util.DateAssert.datesAreEqual(x2, result[1]);
					YAHOO.util.DateAssert.timesAreEqual(x2, result[1]);
					YAHOO.util.DateAssert.datesAreEqual(x3, result[2]);
					YAHOO.util.DateAssert.timesAreEqual(x3, result[2]);
				});
			},
			function(failure) {
				YAHOO.util.Assert.fail('received failure');
			},
			this);

		this.wait();
	},

	testDateSerialisedDb: function() {
		var x1 = new Date(2009, 10, 5, 0, 3, 18);

		this.broker.call('date_test_db', [x1],
				function(result) {
					this.resume(function() {
						YAHOO.util.DateAssert.datesAreEqual(x1, result);
						YAHOO.util.DateAssert.timesAreEqual(x1, result);
					});
				},
				function(failure) {
					YAHOO.util.Assert.fail('received failure');
				},
				this);

			this.wait();
	},

	/*
	 * Tests multiple calls
	 */
	testParallelCalls: function(){
		var i;
		var results = [];
		var onFinished = new YAHOO.util.CustomEvent("onFinished");
		var broker;

		for (i = 0; i < 10; i++) {
			// do call in a timeout so all calls go out at the same moment
			setTimeout((function(scope, i){
				return (function(){

					// do the echo call
					broker = new BASESOFT.util.JsonBroker('/basesoft.web.test/selenium/jsonrpc/simple/', {executeIfLoggedOut: true});
					broker.call('multiple_test', [i], function(result, cbI){

						// verify that the result is the same as the 'i' passed in as cbArgs
						YAHOO.util.Assert.areEqual(result, cbI);

						// add result to the final list
						results.push(result);

						// fire event when all results have been received
						if (results.length === 10)
							onFinished.fire();

					}, null, scope, 1000 * 60, i);
				});
			})(this, i), 10);
		}

		// onFinished is called when all calls have been done
		onFinished.subscribe(function(){
			this.resume(function(){

				// verify result
				// result should contain [0,1,2,.., 49] in different order
				// but always sums 1225

				YAHOO.util.Assert.areEqual(results.length, 10);
				var sum = 0;
				for (var i = 0; i < 10; i++) {
					sum += results[i];
				}
				YAHOO.util.Assert.areEqual(45, sum);
			});
		}, null, this);

		// this tells the test to wait for a resume() call.. see up
		this.wait(1000 * 60);
	},


	/**
	 * tests the .destroy() method of the rpc object. It does a async rpc call and destroys while
	 * the call is active. We expect the callback not to be fired.
	 */
	testDestroy: function() {

		var fail = false;

		var rpc = new BASESOFT.util.JsonBroker('/basesoft.web.test/selenium/jsonrpc/simple/', {executeIfLoggedOut: true});

		// sleep time must be between broker.destroy()' timeout (100ms) and wait below (500ms)
        rpc.call('sleep', [0.3], function(result){
			console.warn('testDestroy: received success', arguments);
			fail = true;
        }, function(failure){
            console.warn('testDestroy: received failure', failure);
			fail = true;
        }, this);

		rpc.destroy();

		var _this = this;
		setTimeout(function() {
	        _this.resume(function(){
				YAHOO.util.Assert.areEqual(fail, false);
	        });
		}, 500);

        this.wait();
	},

	_logout: function(func) {
		var rpc = new BASESOFT.util.JsonBroker('/basesoft.web.contrib.account/login/rpc/', {executeIfLoggedOut: true});
		rpc.call('logout', [], function(){
			this.resume(function(){
				func.call(this);
			});
		}, null, this);
		this.wait();
	},

	/**
	 * tests that when a broker method raises a LoginRequiredException that it also arrives on the client
	 */
	testLoginRequiredException: function() {

		this._logout(function() {

			this.broker.call('login_required_exception', [],
				function(result) {
					YAHOO.util.Assert.fail('received success');
				},
				function(failure) {
					this.resume(function() {
						YAHOO.util.Assert.isInstanceOf(BASESOFT.exceptions.LoginRequiredException, failure);
					});
				},
				this);

			this.wait();
		});
	},

	testLoginDecorator: function() {

		this._logout(function() {

			var rpc = new BASESOFT.util.JsonBroker('/basesoft.web.test/selenium/jsonrpc/login_required/', {executeIfLoggedOut: true});
			rpc.call('echo', ['tuut'],

			// success handler
			function(result) {
				this.resume(function() {
					YAHOO.util.Assert.fail('received failure');
				});
			},

			// failure handler
			function(failure) {
				this.resume(function() {
					YAHOO.util.Assert.isInstanceOf(BASESOFT.exceptions.LoginRequiredException, failure);
				});
			}, this);

	        this.wait();
		});
	},

	testSecurityException: function() {
		var rpc = new BASESOFT.util.JsonBroker('/basesoft.web.test/selenium/jsonrpc/simple/', {executeIfLoggedOut: true});
		rpc.call('security_exception', [], function(result) {
			this.resume(function() {
				YAHOO.util.Assert.fail('received no failure');
			});
		}, function(failure) {
			this.resume(function(){
				YAHOO.util.Assert.isInstanceOf(Error, failure);
			});
		}, this);
		this.wait();
	},

	testIE8EmptyDomValues: function() {
		// IE8 bug
		// http://blogs.msdn.com/b/jscript/archive/2009/06/23/serializing-the-value-of-empty-dom-elements-using-native-json-in-ie8.aspx
		var sandbox = YAHOO.util.Dom.get('sandbox');
		var input = document.createElement('input');
		input.type = 'text';
		input.value = '';
		sandbox.appendChild(input);
		var value = input.value;

		this.broker.call('echo', [value], function(result) {
			this.resume(function() {
				YAHOO.util.Assert.areEqual(result, "");
			});
		}, null, this);
		this.wait();
	},

	testAbort: function() {

		var connId = this.broker.call('multiply', [2, 8],

			function(result) {
				this.resume(function() {
					YAHOO.util.Assert.fail('received success');
				});

				},
			function(failure) {
				this.resume(function() {
					YAHOO.util.Assert.fail('received failure');
				});
			 },

		this);
		console.warn('Calling abort');
		this.broker.abort(connId);

		var _this = this;
		setTimeout(function() {
			_this.resume(function() {
				// test passed
			});
		}, 2000);

		// this tells the test to wait for a resume() call.. see up
		this.wait();

	},

	/**
	 * tests that even tho a timeout occurs in the middle of the queue, it should still continue with the rest
	 */
	testQueueTimeout: function() {

		var failed = false;

		this.broker.call('sleep', [0.4], function(result){}, null, this);

		// Error callback is called if aborted.
		var connId = this.broker.call('sleep', [0.6], function(result){}, function() {
			console.warn('0.6 error corben');
			failed = true;

		}, this, 500);

		this.broker.call('sleep', [0.5], function(result){

			this.resume(function() {

				YAHOO.util.Assert.areEqual(failed, true);

			});

		}, null, this);

		this.wait();
	},


	/**
	 * tests aborting of rpc call that is not yet running
	 */
	testAbortInQueue: function() {

		var failed = false;

		var connId0 = this.broker.call('sleep', [1], function(result){}, null, this);
		var connId1 = this.broker.call('sleep', [10], function(result){}, function() {}, this);

		var connId2 = this.broker.call('sleep', [0.5], function(result){
			this.resume(function() {
				// pass
			});
		}, null, this);

		this.broker.abort(connId1);

		this.wait();
	},

	/**
	 * verifies that if a callback has an error the queue still continues executing
	 */
	testErrorInQueue: function() {

		var failed = false;

		// for this set set debug to false so that errors are not caught directly
		BASESOFT.env.server.debug = false;

		this.broker.call('sleep', [1], function(result){}, null, this);

		for (var i=0; i<=10; i++) {
			this.broker.call('echo', [i], function(r){

				console.warn('corben', r);

				if (r === 5) {
					throw new Error('oops');
				}

				if (r === 10) {
					this.resume(function() {
						// pass

						// restore debug
						BASESOFT.env.server.debug = true;
					});
				}

			}, null, this);
		}

		this.wait();

	},

	testExecuteIfLoggedOut : function () {
		// make sure no user is logged in
		var rpc = new BASESOFT.util.JsonBroker('/basesoft.web.contrib.account/login/rpc/', {executeIfLoggedOut: true});
		rpc.call('logout', [], function(){
			this.resume(function(){
				var broker = new BASESOFT.util.JsonBroker('/basesoft.web.test/selenium/jsonrpc/simple/');
				var f = broker.call('multiply', [2, 8], function(result) {}, function(failure) {}, this);
				YAHOO.util.Assert.areEqual(f, null);
			});
		}, null, this);
		this.wait();
	},

	testUserLogin : function () {

		try {

			// login so we are allowed to save
			var loginrpc = new BASESOFT.util.JsonBroker('/basesoft.web.contrib.account/login/rpc/',
								{executeIfLoggedOut: true});
			loginrpc.call('login', ['user_compose', 'pass_compose'], function(res){
				this.resume(function(){

					var rpc = new BASESOFT.util.JsonBroker('/basesoft.web.test/selenium/jsonrpc/login_required/', {executeIfLoggedOut: true});
					rpc.call('echo', ['tuut'],

					// success handler
					function(result) {
						this.resume(function() {

						});
					},

					// failure handler
					function(failure) {
						this.resume(function() {
							YAHOO.util.Assert.fail('received failure');
						});
					}, this);

			        this.wait();

				});
			}, null, this);


		} catch ( e ) {

			console.error(e);

		}
		this.wait();

	},

	testDecimal: function() {

		var decimal = new BASESOFT.util.Decimal('149.95');

		BASESOFT.env.L10n.DECIMAL_SEPARATOR = ',';

		this.broker.call('echo', [decimal],
			function(rv) {

				this.resume(function() {
					console.warn('rv', rv);

					YAHOO.util.Assert.areSame(rv.format(1), '149,9');

				});
			},
			function(failure) {
				this.resume(function() {
					YAHOO.util.Assert.fail('received failure');
				});
			},
			this);
		this.wait();

	},

	testGetDecimalRemovedExponentZero : function () {
		BASESOFT.env.L10n.DECIMAL_SEPARATOR = '.';

		this.broker.call('get_decimal_removed_exponent', [true],
			function(rv) {

				this.resume(function() {
					console.warn('rv', rv);

					YAHOO.util.Assert.areSame(rv.format(1), '0.0');

				});
			},
			function(failure) {
				this.resume(function() {
					YAHOO.util.Assert.fail('received failure');
				});
			},
			this);
		this.wait();
	},

	testGetDecimalRemovedExponentNonZero : function () {
		BASESOFT.env.L10n.DECIMAL_SEPARATOR = '.';

		this.broker.call('get_decimal_removed_exponent', [false],
			function(rv) {

				this.resume(function() {
					console.warn('rv', rv);

					YAHOO.util.Assert.areSame(rv.format(1), '5,000.0');

				});
			},
			function(failure) {
				this.resume(function() {
					YAHOO.util.Assert.fail('received failure');
				});
			},
			this);
		this.wait();
	},

	testDictionaryArgs: function() {
		this.broker.call('multiply', {a: 2, b: 11},
			function(result) {
				this.resume(function() {
					YAHOO.util.Assert.areSame(22, result, "Result should be 22");
				});
			},
			function(failure) {
				this.resume(function() {
					YAHOO.util.Assert.fail('received failure');
				});
			 },
			this);

		// this tells the test to wait for a resume() call.. see up
		this.wait();
	}

}));

YAHOO.register("test_jsonrpc", null, {version: "1.0.0", build: "1"});

})();
