# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

"""
This is used to create temporary test tables for tests within basesoft.*

We also create a test user for javascript unit tests.
Please attach permissions here.
(permission validation should be done with python unit tests)
"""
from basesoft.web.contrib.license.hooks import PrivilegeObjectMapper
from basesoft.web.core.models import PrivilegeObject

import datetime

from sqlalchemy import Column, Integer, ForeignKey, SmallInteger, DateTime, String
from basesoft.sqlalchemy.table import IQTable

from basesoft.web.contrib.license import models as license_models
from basesoft.web.test.models import MockMail, MockMailing, MockPrivilegeObject, MockPrivilegeObjectGroup, MockOrder,\
    MockOrderLine, MockOrder2
from basesoft.web.contrib.order.models import Order, OrderLine

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:

        # link to define that mail_id is a mock email
        tblMockMail = IQTable('tblMockMail', metadata,
            Column('mail_id', Integer, ForeignKey('tblMail.id', ondelete='CASCADE'), primary_key=True),
            Column('fallback_locale_id', Integer, ForeignKey('tblLocale.id'), nullable=False),
        )

        tblMockPrivilegeObject = IQTable('tblMockPrivilegeObject', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('name', String(255), nullable=False),
            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblMockPrivilegeObjectGroup = IQTable('tblMockPrivilegeObjectGroup', metadata,
            Column('mock_object_id', Integer, ForeignKey('tblMockPrivilegeObject.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tables['tblMockMail'] = tblMockMail
        tables['tblMockPrivilegeObject'] = tblMockPrivilegeObject
        tables['tblMockPrivilegeObjectGroup'] = tblMockPrivilegeObjectGroup

        __tables_defined = True

def do_mappings():
    """
    This function is called by Django middleware when needed.
    """
    global __mapping_defined
    if not __mapping_defined:

        from sqlalchemy.orm import mapper, relation
        from basesoft.web.contrib.mailing import models as mailing_models
        from basesoft.web.contrib.mailing.sa_datamapper import get_tables as mail_tables
        from basesoft.web.contrib.i18n import models as i18n_models

        mapper(MockMail, tables['tblMockMail'],
            properties = {
                'fallback_locale': relation(i18n_models.Locale),
                'locales': relation(mailing_models.MailLocale, order_by=mail_tables()['tblMailLocale'].c.locale_id, passive_deletes=True)
            },
            polymorphic_identity='MockMail',
            inherits=mailing_models.Mail
        )

        mapper(MockMailing,
            polymorphic_identity='MockMailing',
            inherits=mailing_models.Mailing
        )

        mapper(MockPrivilegeObject, tables['tblMockPrivilegeObject'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'groups': relation(MockPrivilegeObjectGroup, backref='mock_object', cascade="all", passive_deletes=True)
            }
        )

        mapper(MockPrivilegeObjectGroup, tables['tblMockPrivilegeObjectGroup'],
            properties = {
                'group': relation(license_models.Group)
            }
        )

        mapper(MockOrder,
            inherits=Order,
            polymorphic_identity='MockOrder'
        )

        mapper(MockOrderLine,
            inherits=OrderLine,
            properties = {
            },
            polymorphic_identity='MockOrderLine'
        )

        mapper(MockOrder2,
            polymorphic_identity='MockOrder2',
            inherits=Order
        )

        __mapping_defined = True

def fill_data(conn):
    """
    Fill test account, license and domain stuff.
    """
    from basesoft.web.core.util.db import Session
    session = Session()

    from basesoft.web.contrib.contact import models as contact_models
    from basesoft.web.contrib.i18n import models as i18n_models
    from basesoft.web.contrib.account import models as reg_models

    domain = license_models.Domain()
    domain.name = 'Testing'

    person = contact_models.Person()
    person.first_name = u'Test'
    person.last_name = u'Account'
    person.email = u'test@basesoft.com'

    license_manager_permissions = session.query(
        license_models.Permission).filter(
        license_models.Permission.name.in_([
            # the test_account user can have all permissions listed here so feel free to add
            # if a test requires (to omit) specific permissions create a new user for that test specifically
            'view_dashboard', 'save_dashboard',
            'create_mail', 'view_mail', 'update_mail', 'delete_mail',
            'create_mock_mail', 'view_mock_mail',
            'update_mock_mail', 'delete_mock_mail',
            'view_datasource', 'create_datasource',
            'view_product', 'create_product',
            'view_datasource', 'create_datasource', 'datasource_manager',
            'create_report', 'datasource_create_cube',
            'view_contact', 'create_contact', 'update_contact', 'delete_contact', 'view_contact_list',
            'create_contact_list', 'update_contact_list', 'delete_contact_list', 'link_to_contact_list',
            'unlink_from_contact_list', 'import_contacts',
            'create_notification', 'view_notification', 'view_license'
        ])).all()

    license = license_models.License()
    license.is_default = True
    license.name = u'IQPortal Test License'
    license.domain = domain
    license.contact_person = person
    license.permissions = license_manager_permissions

    group = license_models.Group()
    group.name = u'Test Accounts'
    group.license = license
    group.is_default = True
    group.permissions = license_manager_permissions

    # pass: test_password
    account = reg_models.Account()
    account.name = u'test_account'
    account.password = '10a6e6cc8311a3e2bcc09bf6c199adecd5dd59408c343e926b129c4914f3cb01'
    account.locale = session.query(i18n_models.Locale).filter(
                                   i18n_models.Locale.locale=='en_GB').one()

    ga = reg_models.GroupAccount()
    ga.account = account
    ga.group = group
    ga.active = True
    ga.is_default = True

    session.add(ga)

    # add privilege hook for contacts & contact lists to always PRIV_DELETE.
    hook = license_models.PrivilegeObjectHook()
    hook.domain = domain
    hook.target_group = group
    hook.object = 'Contact'
    hook.group_privileges = PrivilegeObject.PRIV_DELETE
    session.add(hook)

    peterpan = license_models.PrivilegeObjectHook()
    peterpan.domain = domain
    peterpan.target_group = group
    peterpan.object = 'ContactList'
    peterpan.group_privileges = PrivilegeObject.PRIV_DELETE
    session.add(peterpan)

    session.flush()

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory('Basesoft Test')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the
    `iqsyncpermissions` command.
    """
    return [
        license_models.Permission(u'create_mock_mail', u"""
            This gives the right to create MockMails.
            """, u"Create mock mails"),
        license_models.Permission(u'view_mock_mail', u"""
            This gives the right to view MockMails.
            """, u"View mock mails"),
        license_models.Permission(u'update_mock_mail', u"""
            This gives the right to update MockMails.
            """, u"Update mock mails"),
        license_models.Permission(u'delete_mock_mail', u"""
            This gives the right to delete MockMails.
            """, u"Delete mock mails"),
        license_models.Permission(u'schedule_mock_mail', u"""
            This gives the right to schedule a MockMail.
            """, u"Schedule mock mails"),

        license_models.Permission(u'manage_mock_configs', u"""
            This gives the right to manage Mock Configs.
            """, u"Manage mock configs"),

        license_models.Permission(u'view_mock_orders', u"""
            This gives the right to view Mock Orders.
            """, u"View mock orders"),

        ]

def app_configs():
    from basesoft.web.contrib.account import models as account_models
    configs = []

    # Mock Configuration
    config = account_models.Config()
    config.name = 'basesoft.test.mock.config'
    config.type = 'bool'
    config.default = False
    config.description = u'Mock Config.'
    configs.append(config)

    # Mock Configuration 2
    config = account_models.Config()
    config.name = 'basesoft.test.mock.config.2'
    config.type = 'integer'
    config.default = 20
    config.description = u'Mock Config 2.'
    configs.append(config)

    return configs

def app_roles():
    return [{

        'name': 'Unit Test Automated Role',
        'required_permission': 'manage_iqportal_roles',
        'permissions': ['view_mock_mail', 'create_mock_mail'],
        'license_configs': []

    }]

