"""
Provides test tasks for the scheduler
"""

import time, random

from basesoft.scheduler.task import ScheduleTask
from basesoft.web.contrib.file.models import ImportFile, ImportFileType
from basesoft.web.contrib.file.schedule import AbstractImportFileTask
from basesoft.web.contrib.i18n import models as i18n_models
from basesoft.web.test.controllers import MockFileController
from basesoft.scheduler.crontab import CronTab
from sqlalchemy.sql.expression import and_


class SleepyTask(ScheduleTask):
    """
    Task that doesnt start itself (interval is 0). And when im started i sleep for a minute.
    """
    INTERVAL = 0 # task can only be started manually

    def execute(self):
        time.sleep(60)


class TickyTaskController:

    ticky = None

    def work(self, loop_validator):

        for i in range(60):
            self.ticky = i
            time.sleep(1)

            loop_validator()


class TickyTask(ScheduleTask):
    """
    Task that needs manual starting and ticks till 60, saving the index in itself which can be returned with get_status()
    """

    INTERVAL = 0

    def execute(self):

        self.tc = TickyTaskController()
        self.tc.work(self.loop_validator)

    def get_status(self):
        return self.tc.ticky

class BadTask(ScheduleTask):
    """
    task throws an exception and should go into history with failure flag set
    """
    def execute(self):
        self._do_something()

    def _do_something(self):
        raise Exception('grr!')

class DbTask(ScheduleTask):
    """
    db manipulating task
    """
    INTERVAL = 0

    def execute(self):
        # just do something.. this shouldnt error
        locale = self.sa_session.query(i18n_models.Locale).filter_by(locale='en_GB').one()
        locale.name = 'engliesh'
        self.sa_session.add(locale)

        # this value will be asserted
        return 'all good!'

class ReportingTask(ScheduleTask):
    """
    Task that returns a result every time for report debugging
    """

    INTERVAL = 5

    def execute(self):
        # return a random int from 0 to 100
        return random.randrange(0, 101)

class LongTask(ScheduleTask):
    """
    Longer running task
    """
    INTERVAL = 0

    def execute(self):
        import time
        now = time.time()
        while time.time() - now < 5:
            time.sleep(0.1)
        return 'Phew...'

class DuplicateTask(ScheduleTask):
    """
    Duplicate task that allows for different output
    """
    INTERVAL = 0
    ALLOW_DUPLICATES = True

    def execute(self):
        return self.params['stuff']

class LongDuplicateTask(ScheduleTask):
    """
    Longer running duplicate-able task
    """
    INTERVAL = 0
    ALLOW_DUPLICATES = True

    def execute(self):
        now = time.time()
        while time.time() - now < 5:
            time.sleep(0.1)
        return 'Phew...'

class DuplicateQueueableTask(ScheduleTask):
    INTERVAL = 0
    ALLOW_DUPLICATES = True
    QUEUE_IF_EXISTS = True
    MAX_COMPARABLE_TASKS_IN_QUEUE = 2

    def execute(self):
        time.sleep(5)
        return 'Yay'


class CronTask(ScheduleTask):

    # runs every minute of the day on sunday
    INTERVAL = CronTab('* * * * 0')

    TEST_COUNT = 0

    def execute(self):
        CronTask.TEST_COUNT += 1
        return CronTask.TEST_COUNT


class ImportFileTask(AbstractImportFileTask):

    controller = MockFileController
    importer = MockFileController

    def get_controller_and_files(self):
        files = self.sa_session.query(ImportFile).join(ImportFileType).filter(and_(
                    ImportFileType.name=='basesoft.web.test',
                    ImportFile.status==ImportFile.STATUS_NEW,
                    ImportFile.deleted==False,
                    ImportFile.archived==False)).all()
        return files


class DisabledTask(ScheduleTask):
    def execute(self):
        return 'I SHOULD NOT BE ENABLED'


tasks = (SleepyTask, TickyTask, DbTask, BadTask, ReportingTask,
         LongTask, DuplicateTask, LongDuplicateTask, DuplicateQueueableTask, CronTask, DisabledTask, )
