# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

"""
This provides the abstract SeleniumTest class which connects to the selenium server
"""
import os
import time
import datetime

from types import NoneType
from basesoft.web.core.util.controllers import lazy_assert

from django.utils import simplejson
from django.conf import settings

from basesoft.web.test.unit import iqportal_unittest

from basesoft import logging
from twisted.trial.unittest import SkipTest
from selenium.selenium import selenium as Selenium
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
from selenium.webdriver.remote.webdriver import WebDriver
from basesoft.web.contrib.file.controllers import UploadFileController
from django.core.files.base import File, ContentFile
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.remote.webelement import WebElement
from selenium.common.exceptions import NoSuchElementException,\
    ElementNotVisibleException, WebDriverException, UnexpectedAlertPresentException, TimeoutException
from selenium.webdriver.support.select import Select
from selenium.webdriver.common.keys import Keys

logger = logging.getLogger('basesoft.web.test.selenium')

class IQSelenium(Selenium):
    """Extends the openqa selenium class for custom methods"""

    def do_command(self, verb, args):
        """
        Extends selenium.do_command so we can see if there is an exception thrown and make a screenshot before continue
        """

        logger.debug("IQSelenium: do_command: %s: %s" % (verb, unicode(args)))
        return Selenium.do_command(self, verb, args)


_driver = None

class SeleniumTestCase(iqportal_unittest.IQPortalTestCase):

    timeout = settings.SELENIUM['timeout']

    _django_debug = True
    _selenium_debug = True

    selenium = None

    _extra_stable_time = 0

    def setUp(self):

        if not settings.FORM_VALIDATION:
            raise Exception('Cannot run test with form validation disabled')

        logger.debug('%s: setUp()' % self)

        # kill existing driver if available. Happens if 'closeBrowser' is False and running multiple tests
        global _driver
        if _driver is not None:
            _driver.quit()

        self._start_django()
        self._start_scheduler()

        capabilities = settings.SELENIUM['desired_capabilities']

        self._extra_stable_time = getattr(settings, 'SELENIUM_EXTRA_STABLE_TIME', 0)

        # IE driver changes
        # from https://code.google.com/p/selenium/issues/detail?id=2067#c93
        if capabilities['browserName'] == 'internet explorer':
            capabilities['requireWindowFocus'] = True
            capabilities['enablePersistentHover'] = True

            self._extra_stable_time += 2

        url = 'http://%s:%s/wd/hub' % (settings.SELENIUM['host'], settings.SELENIUM['port'])

        tries = 5
        for i in range(tries):
            try:
                _driver = self._driver = WebDriver(command_executor=url, desired_capabilities = capabilities)
            except Exception:
                if i == tries-1:
                    print 'WebDriver max tries reached.'
                    raise
                print 'WebDriver connect failed %s times sleeping..' % i
                time.sleep(5)
            else:
                break

        if settings.SELENIUM['maximize_window']:
            self._driver.maximize_window()

        # always start clean
        self.clear_cookies()

        #self._driver.implicitly_wait(2)
        #self._driver.set_window_size(1280, 1024)


    def clear_cookies(self):
        self._driver.get(settings.SELENIUM['browserURL'] + 'basesoft.web.core/blank/')
        self._driver.delete_all_cookies()

    def tearDown(self):
        """
        Overrides tearDown so we can make sure that selenium and django are stopped
        """

        if not self._passed:

            # print any JS errors found
            print "JavaScript Errors: %s" % self._driver.execute_script("if (typeof BASESOFT != 'undefined') return BASESOFT.read_errors()")

            # force umask to be readable for all
            os.umask(0022)

            # take a screenshot if the current test failed
            if not os.path.exists(settings.SELENIUM['screenshot_path']):
                os.makedirs(settings.SELENIUM['screenshot_path'])
            filename = os.path.join(settings.SELENIUM['screenshot_path'], 'selenium_%s.png' % time.strftime('%y%m%d%H%S'))
            try:
                self._driver.get_screenshot_as_file(filename)
                #logger.debug("IQSelenium: selenium screenshot saved to %s" % filename)

                filename = filename.replace('/var/lib/buildbot/public_html/', 'http://vm.office.basesoft.com/~buildbot/')
                print "IQSelenium: screenshot: %s" % filename

            except Exception, e2:
                logger.debug("IQSelenium: failed to take screenshot: %s" % e2)
                pass


        if settings.SELENIUM['closeBrowser']:
            global _driver
            _driver.quit()
            _driver = self._driver = None

        # now really teardown
        iqportal_unittest.IQPortalTestCase.tearDown(self)

    def _classCleanUp(self, result):
        """
        Extends _classCleanup from twisted and we add our own cleanup
        """
        iqportal_unittest.IQPortalTestCase._classCleanUp(self, result)
        self._cleanup()

    def refresh_page(self):
        """
        Refresh current page
        """
        self._driver.refresh()

        #try:
        self.wait_for_loading()
        #except UnexpectedAlertPresentException:
        #    # catch any 'do you wanna save' alerts.. then accept then and it continue
        #    alert = self._driver.switch_to_alert()
        #    alert.accept()

    def _selector_statement(self, selector, operator='===', count=1):
        """
        waits for a yui selector query on the document, asserting the exact count (default === 1)
        http://www.w3.org/TR/css3-selectors
        """

        selector = selector.replace("'", "\'")

        statement = "selenium.browserbot.getCurrentWindow().YAHOO.util"
        statement += ".Selector.query('%s').length %s %s" % (selector, operator, count)

        return statement

    def find_element_by_css_contains(self, css_selector, contains_text):
        """
        wrapper providing an alternative for :contains()
        """

        els = self._driver.find_elements_by_css_selector(css_selector)

        for el in els:
            if el.text == contains_text:
                return el
        raise Exception('no elements with contains_text "%s" not found in selector: %s' % (contains_text, css_selector))

    def get_form_element(self, *args, **kwargs):
        """
        returns a WebElement.. see fill_form()
        """
        kwargs['return_element'] = True
        return self.fill_form(*args, **kwargs)

    def wait_for_selector(self, selector, operator='===', count=1):
        """
        """
        statement = self._selector_statement(selector, operator, count)
        logger.debug("wait_for_selector: %s" % statement)

        def _check(driver):

            elements = driver.find_elements_by_css_selector(selector)

            if operator == '===':
                return len(elements) == count
            elif operator == '>':
                return len(elements) > count
            elif operator == '>=':
                return len(elements) >= count
            else:
                raise Exception('unknown operator: %s' % operator)

        try:
            wait = WebDriverWait(self._driver, self.timeout / 1000)
            wait.until( _check )
        except TimeoutException:
            raise TimeoutException('Timeout while waiting for %s %s %s' % (selector, operator, count) )

    DONE_LOADING_CMD = "\
        return BASESOFT && \
        (BASESOFT.env && BASESOFT.env.bootLoader && !BASESOFT.env.bootLoader.isBusy()) && \
        (!YAHOO.util.Dom.get('bs-loading-dialog') || YAHOO.util.Dom.get('bs-loading-dialog').style.display == 'none') && \
        !YAHOO.util.Dom.hasClass('body', 'loader-busy') && \
        !YAHOO.util.Dom.hasClass('body', 'iq-compose-loading') && \
        !YAHOO.util.Dom.hasClass('body', 'busy') \
    "

    def wait_for_loading(self, times='deprecated', stable_time=1, assert_no_errors=True):
        return self.wait_for_loading2(stable_time, assert_no_errors=assert_no_errors)

    def wait_for_loading2(self, stable_time=2, assert_no_errors=True):
        """
        The new wait_for_loading()
        It waits till the page is not loading for at least $stable_time seconds (default 2)
        """
        t1 = time.time()

        max_tries = 100
        current_try = 0

        # especially IE gets a bit more time assigned
        stable_time = stable_time + self._extra_stable_time

        while True:

            current_try += 1
            if current_try >= max_tries:
                raise Exception('max tries exceeded')

            if self._driver.execute_script(self.DONE_LOADING_CMD):

                if (time.time() - t1) > stable_time:
                    break

            else:
                t1 = time.time()

            time.sleep(0.2)

        if assert_no_errors:
            self.assert_no_growl_errors()

    def wait_for_panel_or_form(self, panel_name=None, form_name=None):
        """
        waits for the panel and/or form to be available and returns the xpath for it
        """
        css = ""

        if panel_name:
            self.wait_for_selector("div[name='%s']" % panel_name)
            css += "div[name='%s'] " % panel_name

        if form_name:
            self.wait_for_selector("form[name='%s']" % form_name)
            css += " form[name='%s'] " % form_name

        return css

    def wait_for_panel(self, panel_name):
        """
        quick method for waiting for panel
        """
        return self.wait_for_panel_or_form(panel_name=panel_name)

    def wait_for_form(self, form_name):
        """
        quick method for waiting for forms
        """
        return self.wait_for_panel_or_form(form_name=form_name)

    def close_panel(self, panel_name):
        """
        clicks on the close icon for the given panel
        """
        el = self._driver.find_element_by_css_selector("div[name='%s'] a.container-close" % panel_name)
        el.click()


    def wait_for_panel_closed(self, panel_name):
        """
        waits for the selected panel to be gone
        """
        self.wait_for_selector("div[name='%s']" % panel_name, count=0)

    def wait_for_element_present(self, element_id):
        """
        Waits for the given element_id to be present. Uses YAHOO.util.Dom till the element is available.
        """
        raise Exception('Use wait_for_selector()')

    def wait_for_cq(self, panel_name=None, cq_name=None, operator='>=', rows=0):
        """
        Waits for the custom query to be initialised
        Optionally: the number of rows can be matched using 'operator' and 'rows'

        Example:
            # wait for exactly 1 row
            self.wait_for_cq(panel_name='license-manager-panel', operator='=', rows=1)
        """

        selector = ""

        # optional: wait for panel
        if panel_name:
            self.wait_for_panel(panel_name)
            selector += "div[name='%s'] " % panel_name

        # TODO: implement cq_name

        # wait for cq to be loaded with a yui datatable in it with x rows
        selector += '.bs-customdatatable tbody.yui-dt-data tr'
        self.wait_for_selector(selector, operator, rows)

    def filter_cq(self, panel_name=None, cq_name=None, filters=None):
        """
        Applies the given filters to the cq.

        Example:
            # apply two filters. the second filter is a select
            self.filter_cq(panel_name='license-manager-panel', filters=[
                ('License', 'Equal', 'My License ' + rand_id),
                ('Type', 'Contains', ('select', 'License')),
            ])
        """
        if filters is None:
            filters = []
        assert isinstance(filters, (list, tuple) )

        self.wait_for_cq(panel_name, cq_name)

        if panel_name:
            css = self.wait_for_panel(panel_name)
        else:
            css = ""

        # TODO: implement cq_name

        # create path to filters button
        filter_open_btn = css + " div.toolbar "

        # append filter fieldset to xpath
        css = css + " div.bs-filter_fieldset "

        # todo: clear current filters

        # click on collapse to open
        # todo: maybe add check if already opened in case filter_cq is called multiple times
        el = self._driver.find_element_by_css_selector(filter_open_btn + " span.filters")
        el.click()

        def add_filter(css):
            # add the filter to the cq
            time.sleep(1)
            self.wait_for_loading()
            el = self._driver.find_element_by_css_selector(css + " select[name='sel_filters']")
            select = Select(el)
            select.select_by_visible_text(name)

            # the div containing the filter rows
            return css + " fieldset.layout-filters > div:first-child > div:last-child "

        # select filters
        for filter in filters:
            assert len(filter) == 3
            name, operator, value = filter
            lazy_assert(name, str)

            input_type = 'input_text'
            if type(value) == tuple:
                input_type, value = value

            # the div containing the filter rows
            last_row = add_filter(css)

            # select operator
            el = self._driver.find_element_by_css_selector(last_row + " select[name^='sel_operator']")
            select = Select(el)
            select.select_by_visible_text(operator)

            if input_type != 'multiple':
                value = [value]

            i = value.__iter__()
            val = i.next()

            try:
                while True:
                    if type(val) == tuple:
                        input_type, val = val
                    # put value
                    if input_type == 'input_text':
                        input_css = last_row + " input[type='text'][name^='value']"
                        el = self._driver.find_element_by_css_selector(input_css)
                        assert isinstance(el, WebElement)
                        el.clear()
                        el.send_keys(unicode(val))
                    elif input_type == 'select':
                        input_css = last_row + " select[name^='value']"
                        el = self._driver.find_element_by_css_selector(input_css)
                        select = Select(el)

                        if val.startswith('value='):
                            val = val[6:]
                            select.select_by_value(val)
                        else:
                            matched = False
                            for option in select.options:
                                if option.text == val:
                                    matched = True
                                    option.click() and not option.is_selected()
                            if not matched:
                                raise NoSuchElementException("Could not locate element with visible text: %s" % val)

                    else:
                        raise Exception('unknown input type: %s' % input_type)

                    val = i.next()
                    last_row = add_filter(css)
            except StopIteration:
                pass

        el = self.find_element_by_css_contains(css + 'button', 'Update')
        el.click()

        self.wait_for_loading()


    def assert_cq(self, panel_name=None, cq_name=None, row_index=None, data=None, is_datatable=False):
        """
        Asserts the data of the CQ for the given row_index. 'data' is an object where
        the key is the column name like defined in the CQ and the value is either a XPath
        object or anything else. If other than Xpath it will be asserted as text (not html!)
        somewhere inside the TD.

        Example:
            # assert first row in the CQ.. license must be a link
            self.assert_cq(panel_name='license-manager-panel', row_index=1, data={
                'contact': 'Paul de Graaf',
                'company_name': 'BaseSoft Solutions B.V.',
                'license_id': XPath("//a[text()='View details']"),
            })
        """

        assert isinstance(data, object)

        # loop data
        for col_name, value in data.iteritems():
            assert isinstance(col_name, str)

            # get the TD xpath for the arguments
            css = self._cq_td_xpath(panel_name, cq_name, row_index, col_name, is_datatable=is_datatable)

            # getting the td shouldnt raise an exception
            el = self._driver.find_element_by_css_selector(css)

            # the loop below scrolls horizontally 10px every loop until the element becomes visible
            # and errors if more than 1000px
            max_scroll = 1000
            left_value = 0
            while True:
                if el.is_displayed():
                    break
                left_value = left_value+20
                if left_value > max_scroll:
                    raise Exception('failed to scroll to column %s' % col_name)
                self.scroll_panel(panel_name=panel_name, left_value=left_value)

            assert el.is_displayed()

            if value is None:
                self.assertEqual(el.text, '')
            else:
                self.assertEqual(el.text, value)

    def cq_rows_length(self, panel_name, cq_name=None, is_datatable=False):
        """
        returns the number of rows

        @todo: implement cq_name
        """

        assert isinstance(panel_name, (NoneType, str))

        # optional: match panel
        css = self.wait_for_panel_or_form(panel_name=panel_name, form_name=None)

        # match cq
        if is_datatable:
            css += " .bs-customdatatable"
        else:
            css += " .bs-custom_query"

        # at this point verify that the CQ even exists
        try:
            self._driver.find_element_by_css_selector(css)
        except NoSuchElementException:
            raise Exception('Element not found: %s' % css)

        css += " tbody.yui-dt-data tr"
        return len(self._driver.find_elements_by_css_selector(css))


    def click_cq(self, panel_name=None, cq_name=None, row_index=None, col_name=None, extra_css=' a', is_datatable=False):
        """
        Clicks on a specific cell (by default any <a> in it) for the given arguments.
        Pass in another xpath to click on something else than a link (like an image).

        Example:
            self.click_cq(panel_name='license-manager-panel', row_index=1, col_name='license_id')
        """
        assert isinstance(panel_name, (NoneType, str))
        assert isinstance(row_index, int)
        assert isinstance(col_name, str)

        # get the TD xpath and glue the xpath we are looking for
        css = self._cq_td_xpath(panel_name, cq_name, row_index, col_name, is_datatable=is_datatable) + extra_css

        # clicky
        el = self._driver.find_element_by_css_selector(css)
        el.click()

    def click_toolbar_button(self, panel_name, button_class, cq_name=None):
        """
        Click the Separate/Group button on the Manager toolbar
        Usage: self.click_toolbar_button('account_manager', 'filters')
        """

        self.wait_for_cq(panel_name, cq_name)

        if panel_name:
            css = self.wait_for_panel(panel_name)
        else:
            css = ""

        # create path to filters button
        css = css + " div.toolbar "

        # click on the button
        el = self._driver.find_element_by_css_selector(css + " span." + button_class)
        el.click()

    def _cq_td_xpath(self, panel_name=None, cq_name=None, row_index=None, col_name=None, is_datatable=False):
        """
        protected method for getting the TD for a certain CQ. Used in assert_cq() and click_cq()
        TODO: implement cq_name
        """

        assert isinstance(panel_name, (NoneType, str))
        assert isinstance(row_index, int)
        assert isinstance(col_name, str)

        # optional: match panel
        css = self.wait_for_panel_or_form(panel_name=panel_name, form_name=None)

        # match cq
        if is_datatable:
            css += " .bs-customdatatable"
        else:
            css += " .bs-custom_query"

        # at this point verify that the CQ even exists
        try:
            self._driver.find_element_by_css_selector(css)
        except NoSuchElementException:
            raise Exception('Element not found: %s' % css)

        # return the xpath for the given row_index and col_name
        css += " tbody.yui-dt-data tr:nth-child(%s) > td.yui-dt-col-%s" % ( (row_index+1), col_name)

        # assert that exists
        self._driver.find_element_by_css_selector(css)

        # goodbye
        return css

    def assert_cq_empty(self, panel_name=None, cq_name=None, is_datatable=False):
        """
        asserts that the cq for the given arguments is empty
        TODO: implement cq_name
        """
        css = self.wait_for_panel_or_form(panel_name=panel_name, form_name=None)

        if is_datatable:
            css += " .bs-customdatatable tbody.yui-dt-message .yui-dt-empty"
        else:
            css += " .bs-custom_query tbody.yui-dt-message .yui-dt-empty"


        print css

        els = self._driver.find_elements_by_css_selector(css)
        self.assertEqual(len(els), 1)


    def assert_cube_table(self, row_index, value, css_selector, col_index=None, assert_header=False, has_row_headers=True):
        """
        Asserts the data of the cube table for the given row_index. 'value' is list of table values that should be
        validated to the table data provided by the css_selector. Col_index is optional if it is None the whole row is validated
        Columns usually start from index 2.
        Example:
            self.assert_cube_table(1, [5908, 5299, 2940, 1860], table.basesoft-cubetable, col_index=2)
        """

        assert isinstance(value, list)
        list_length = len(value)

        if assert_header is True:
            css = "body %s tbody tr:nth-child(%d) th" % (css_selector, row_index)
            try:
                el = self._driver.find_element_by_css_selector(css)
            except NoSuchElementException:
                raise Exception('Element not found: %s' % css)

            self.assertEqual(el.text, unicode(value[0]))
            return

        add = 1
        if has_row_headers:
            add += 1

        for i in range(0, list_length):
            if col_index is None:
                css = "body %s tbody tr:nth-child(%d) td:nth-child(%d)" % (css_selector, row_index, i+add)
            else:
                css = "body %s tbody tr:nth-child(%d) td:nth-child(%d)" % (css_selector, row_index, col_index+i)

            try:
                el = self._driver.find_element_by_css_selector(css)
            except NoSuchElementException:
                raise Exception('Element not found: %s' % css)

            self.assertEqual(el.text, unicode(value[i]))

    def save_dialog_input(self, input_text=None):
        target = self._driver.find_element_by_css_selector('input.feedback-input')
        target.clear()
        target.send_keys(unicode(input_text))

        # click Save to save the name of the report
        self.click_button(panel_name='dialog-feedback', text='OK')


    def click_menu(self, panel_name=None, form_name=None, menu_index=None, submenu=None, text=None):
        """
        Clicks on the menu for the given panel/form. Then clicks on the text for a menu item.
        For multiple menus on the same panel use menu_index=1 for the first menu etc.

        TODO; implement submenus (list) that should be opened before the text can be clicked

        Example:
            self.click_menu(panel_name='license-manager-panel', menu_index=1, text='New license wizard')
        """

        css = self.wait_for_panel_or_form(panel_name=panel_name, form_name=form_name)
        css += " div.menu-button-container button"
        self.wait_for_selector(css, '>', 0)

        # click menu
        if menu_index is not None:
            el = self._driver.find_elements_by_css_selector(css)[menu_index-1]
            el.click()
        else:
            el = self._driver.find_element_by_css_selector(css)
            el.click()

        # NOTE: the menu is not a child of the panel/form!
        menu_els = self._driver.find_elements_by_css_selector("div.iq-menu div.text")
        for el in menu_els:
            if el.text == text:
                el.click()
                return
        raise Exception('menu not found: %s' % text)

    def select_as_group(self, panel_name=None, form_name=None, el_name=None, values=None, agg=True, name='My Group'):
        assert isinstance(values, (tuple, list))

        self.scroll_top('html', 0)

        # important: dont modify this value in the loop below
        css = self.wait_for_panel_or_form(panel_name, form_name)

        container = css + " div[name='%s']" % el_name
        left_select_css = container + " select[id$='leftselect']"

        # fixme: scrolling?
        #self._scrollTo(panel_name, css=left_select_css)

        select = Select(self._driver.find_element_by_css_selector(left_select_css))

        for option_label in values:
            # TODO: make selecting by value possible with 'value=xx'
            select.select_by_visible_text(option_label)
            #self.selenium.select(left_select_css, opt_css)

        button = self.find_element_by_css_contains(container + ' button', 'Select as Group')
        button.click()

        values = {
            'name': name,
            'use_aggregate': ('checkbox', agg)
        }

        self.fill_form(panel_name='iqube-select-as-group', values=values)
        self.click_button(panel_name='iqube-select-as-group', text='Add as group')


    def fill_form(self, panel_name=None, form_name=None, rowcontrol_name='row_control', rowcontrol_row_index=None,
                  values=None, assert_=False, return_element=False):
        """
        Waits for the given panel and/or form to be available and sets the the form values.
        Values is a dictionary in the form element_name => element_value
        Element_value can also be a tuple where the type can be set in the first element, values second

        rowcontrol_row_index: index of the row within the rowcontrol, starts at 0

        If return_element is True use same syntax for values={} with element value just an empty string.

        Example:
            # fill a form with text inputs, select and fancy-select
            self.fill_form(panel_name='license-wizard-panel', values={
                'first_name': 'Johnny', 'last_name': 'Depp',
                'title': 'Pirate', 'gender': ('select', 'Male'),
                'permissions': ('fancy-select', ('Scheduler manager access', 'Create surveys'))
            })
        """
        assert isinstance(values, object)

        # important: this is needed to pass the test for linux firefox browser
        # Should be skipped in future releases of Selenium (current version: selenium-server-standalone-2.40.0.jar)
        # after Selenium fixed upgrade
        self.scroll_top('html', 0)

        # important: dont modify this value in the loop below
        css = self.wait_for_panel_or_form(panel_name, form_name)
        if rowcontrol_row_index is not None:
            css += " table.row-control-table[name='%s'] > tbody > tr:nth-child(%s)" % (rowcontrol_name, rowcontrol_row_index+1)

        container_el = self._driver.find_element_by_css_selector(css)

        for key, value in values.iteritems():

            logger.info('fill_form: filling key="%s", value="%s"' % (unicode(key), unicode(value)))

            try:

                input_type = 'input_text'
                if (type(value) == tuple):
                    input_type, value = value

                if input_type == 'input_text':

                    input_css = "%s input[name='%s']" %  (css, key)
                    logger.info('fill_form: input_css: %s' % input_css)

                    input_el = self._driver.find_element_by_css_selector(input_css)
                    assert isinstance(input_el, WebElement)

                    if return_element:
                        return input_el

                    # FIXME: scrolling?
                    #self._scrollTo(panel_name, css=input_css)

                    if assert_:
                        self.assertEquals( input_el.get_attribute("value"), value )
                    else:
                        input_el.clear()
                        input_el.send_keys(unicode(value))

                        if input_el.get_attribute("value") != unicode(value):
                            input_el.clear()
                            input_el.send_keys(unicode(value))


                elif input_type == 'select':
                    input_css = "%s select[name='%s']" %  (css, key)
                    logger.info('fill_form: select input_css: %s' % input_css)

                    el = self._driver.find_element_by_css_selector(input_css)

                    if return_element:
                        return el

                    select_el = Select(el)

                    # FIXME: scrolling still needed?
                    #self._scrollTo(panel_name, css=input_css)

                    # by default 'value' means the label except if it is prefixed with value=
                    select_by_value = False
                    if value.startswith('value='):
                        select_by_value = True
                        value = value[6:]

                    if assert_:

                        selected_options = select_el.all_selected_options

                        # fixme: assert selected multiple (check for value is array?)
                        if len(selected_options) != 1:
                            raise Exception('not implemented')

                        option = selected_options[0]
                        assert isinstance(option, WebElement)

                        if select_by_value:
                            self.assertEqual(option.get_attribute("value"), value)
                        else:
                            self.assertEqual(option.text, value)

                    else:

                        if select_by_value:
                            select_el.select_by_value(value)
                        else:
                            matched = False
                            for option in select_el.options:
                                if option.text == value:
                                    matched = True
                                    option.click() and not option.is_selected()
                                    break
                            if not matched:
                                raise NoSuchElementException("Could not locate element with visible text: %s" % value)

                elif input_type == 'checkbox' or input_type == 'checkbox2':

                    if type(value) in (str, unicode, ) and value.startswith('value='):
                        value = value[6:]
                        input_css = "%s input[name='%s']" %  (css, key)
                        els = self._driver.find_elements_by_css_selector(input_css)
                        for cb in els:
                            if cb.get_attribute("value") == value:
                                el = cb
                                break
                    else:
                        input_css = "%s input[name='%s']" %  (css, key)
                        el = self._driver.find_element_by_css_selector(input_css)

                    if return_element:
                        return el

                    if assert_:
                        if value:
                            self.assertTrue(el.is_selected())
                        else:
                            self.assertFalse(el.is_selected())

                    else:

                        if value:
                            if el.is_selected():
                                print 'warning: element %s already checked' % input_css
                            else:
                                el.click()
                        else:
                            if el.is_selected():
                                el.click()
                            else:
                                print 'warning: element %s already unchecked' % input_css

                elif input_type == 'radio':
                    input_css = "%s input[name='%s']" %  (css, key)
                    # get all radios matching the selector, then click the one with the given value
                    els = self._driver.find_elements_by_css_selector(input_css)

                    # should we return only the element with the given value?
                    if return_element:
                        return els

                    for el in els:
                        if el.get_attribute("value")  == unicode(value):
                            el.click()

                elif input_type == 'fancy-select':
                    assert isinstance(value, (tuple, list))

                    container = css + " div[name='%s']" % key
                    left_select_css = container + " select[id$='leftselect']"

                    left_select = self._driver.find_element_by_css_selector(left_select_css)

                    if return_element:
                        return left_select

                    # fixme: scrolling?
                    #self._scrollTo(panel_name, css=left_select_css)

                    if assert_:

                        right_select_css = container + " select[id$='rightselect']"

                        select_el = Select(self._driver.find_element_by_css_selector(right_select_css))
                        selected_options = select_el.options

                        self.assertEqual(len(selected_options), len(value))
                        for option in selected_options:
                            # TODO: make selecting by value possible with 'value=xx'
                            self.assertTrue(option.text in value)
                    else:
                        for option_label in value:

                            select = Select(left_select)

                            # TODO: make selecting by value possible with 'value=xx'
                            select.select_by_visible_text(option_label)
                            #self.selenium.select(left_select_css, opt_css)

                            button = self.find_element_by_css_contains(container + ' button', 'Select')
                            button.click()


                elif input_type == 'upload':

                    # Create django uploaded file and pass it to the upload file controller so
                    # we get a file hash.

                    # Then save this hash into the form and trigger the 'blur' event. This writes the hash to the
                    # form just like the yui uploader does.

                    if value.startswith('/'):
                        filename = os.path.basename(value)
                        with open(value) as f:
                            content = f.read()
                    else:
                        filename = value
                        content = 'uploaded content'

                    django_file = ContentFile(name=filename, content = content)
                    files = UploadFileController().save_temp_files({filename: django_file})

                    input_css = "%s input[name='%s']" %  (css, key)

                    self._driver.execute_script('YAHOO.util.Selector.query("%s")[0].setAttribute("type", "text")' % input_css)

                    input_el = self._driver.find_element_by_css_selector(input_css)

                    if return_element:
                        return input_el

                    input_el.clear()
                    input_el.send_keys(unicode(files[0].file_hash))

                    self._driver.execute_script('YAHOO.util.Selector.query("%s")[0].setAttribute("type", "hidden")' % input_css)

                elif input_type == 'textarea':

                    input_css = "%s textarea[name='%s']" %  (css, key)
                    input_el = self._driver.find_element_by_css_selector(input_css)

                    if return_element:
                        return input_el

                    if assert_:
                        self.assertEquals( input_el.text, value)
                    else:
                        input_el.clear()
                        input_el.send_keys(unicode(value))

                elif input_type == 'date':

                    assert isinstance(value, datetime.date)

                    input_css = "%s input[name='%s']" %  (css, key)
                    input_el = self._driver.find_element_by_css_selector(input_css)

                    if return_element:
                        return input_el

                    # use the javascript method date_short to convert a datetime to his locale
                    user_date = self._driver.execute_script("return BASESOFT.util.date_short ( new Date(%s, %s, %s) );"  % (value.year, value.month-1, value.day) )

                    if assert_:
                        self.assertEquals( input_el.get_attribute("value"), user_date )
                    else:
                        input_el.clear()
                        input_el.send_keys(unicode(user_date))

                elif input_type == 'datetime':
                    assert isinstance(value, datetime.datetime)

                    input_css = "%s input[name='%s']" %  (css, key)
                    input_el = self._driver.find_element_by_css_selector(input_css)

                    if return_element:
                        return input_el

                    # use the javascript method date_short to convert a datetime to his locale
                    user_date = self._driver.execute_script("return BASESOFT.util.date_time_short ( new Date(%s, %s, %s, %s, %s) );"  % (value.year, value.month-1, value.day, value.hour, value.second) )

                    if assert_:
                        self.assertEquals( input_el.get_attribute("value"), user_date )
                    else:
                        input_el.clear()
                        input_el.send_keys(unicode(user_date))

                elif input_type == 'autocomplete':

                    input_css = "%s input[name='%s']" %  (css, key)
                    input_el = self._driver.find_element_by_css_selector(input_css)

                    if return_element:
                        return input_el

                    if assert_:
                        self.assertEquals( input_el.get_attribute('value'), unicode(value) )
                    else:
                        input_el.clear()
                        input_el.send_keys(unicode(value))

                    # select the value from the drop-down
                    menus = self._driver.find_elements_by_css_selector('ul.ui-autocomplete')
                    menu = None

                    self.wait_for_loading()

                    for m in menus:
                        if m.is_displayed():
                            menu = m

                    options = menu.find_elements_by_tag_name('a')

                    for option in options:
                        if option.text == value:
                            option.click()
                            break
                    else:
                        raise Exception('Autocomplete menu option not found: %s ' % value)

                else:
                    raise Exception('SeleniumTestCase: invalid input type: %s' % input_type)

            except Exception:
                logger.error('Exception while working with element "%s" with value "%s"' % (key, value))
                raise

        # finally, blur the form
        # disabled cuz it errors in chromedriver
        #ac = ActionChains(self._driver)
        #ac.move_to_element_with_offset(self._driver.find_element_by_css_selector('html'), 1, 1)
        #ac.click()
        #ac.perform()

    def fill_forms(self, panel_name=None, forms_data=None, assert_=False):
        """
        quick method for filling multiple forms in the same panel
        """
        for form_name, values in forms_data.iteritems():
            self.fill_form(panel_name=panel_name, form_name=form_name, values=values, assert_=assert_)

    def assert_form(self, *args, **kwargs):
        """
        asserts the values of a form.. see fill_form()
        """
        kwargs['assert_'] = True
        self.fill_form(*args, **kwargs)

    def assert_forms(self, *args, **kwargs):
        """
        asserts the values of multiple forms.. see fill_forms()
        """
        kwargs['assert_'] = True
        self.fill_forms(*args, **kwargs)

    def click_fieldset(self, panel_name=None, form_name=None, title=None):
        """
        performs a click on the header of a fieldset to (un)collapse it
        """
        css = self.wait_for_panel_or_form(panel_name=panel_name, form_name=form_name)
        el = self.find_element_by_css_contains(css + " div.bs-fieldset-collapse-text", title)
        el.click()

    def click_tab(self, text):
        css = 'div.bs-tabview li.bs-tab'
        tab = self.find_element_by_css_contains(css, text)
        tab.click()

    def click_button(self, panel_name=None, form_name=None, text=None, btn_type='button'):
        """
        clicks on a button for the given text
        """
        assert isinstance(text, (str, unicode, ))

        css = self.wait_for_panel_or_form(panel_name, form_name)
        css += btn_type

        button = self.find_element_by_css_contains(css, text)
        button.click()

    def click_button_by_id(self, _id=None):
        css = '#'+_id

        button = self._driver.find_elements_by_css_selector(css)[0]
        button.click()

    def click_iconbar(self, icon_id=None, hover=None, title=None):
        """
        Waits for the icon to apear and then performs a click on the specified icon on the iconbar

        Example using the icon title:
            self.click_iconbar(title='Data source manager')

        Example using the element id (deprecated):
            self.click_iconbar('icon-license-manager')
        """

        assert icon_id is None or title is None, "either icon_id or title must be given"


        # to speedup test runs, disable animation on the iconbar, this avoid sleeps in logic below
        self._driver.execute_script("BASESOFT.env.Iconbar.animation_time_up = 0;")
        self._driver.execute_script("BASESOFT.env.Iconbar.animation_time_down = 0;")

        if hover is not None:

            if title is None:
                match = "div#%s" % hover
            else:
                match = "div[title='%s']" % hover

            self.wait_for_selector(match)

            target = self._driver.find_element_by_css_selector(match)

            #if not target.is_displayed():
                #raise Exception('element is not displayed: %s' % match)

            ac = ActionChains(self._driver)
            ac.move_to_element(target)
            ac.perform()
            # FIXME: even though iconbar animation is set to 0ms it still opens the stack slow enough to break a test
            time.sleep(1)

        if title is None:
            match = "div#%s" % icon_id
        else:
            match = "div[title='%s']" % title

        target = self._driver.find_element_by_css_selector(match)

        if not target.is_displayed():
            raise Exception('element is not displayed: %s' % match)

        ac = ActionChains(self._driver)
        ac.move_to_element(target)
        ac.click()

        ac.move_to_element(self._driver.find_element_by_css_selector("body"))
        ac.click()
        ac.perform()

        self.wait_for_loading()

    def click_disabled_button(self, button_index=None):

        if button_index is not None:
            el_class_name = self._driver.find_element_by_css_selector(".yui-button-disabled:nth-child(%s)" % (button_index + 1) )
            el_id = el_class_name.get_attribute("id")

            target_button = "document.getElementById('%s').className = 'yui-button yui-button-button';" % el_id
            self._driver.execute_script(target_button)

            unhide_button = "document.getElementById('%s-button').disabled = false;" % el_id
            self._driver.execute_script(unhide_button)

            self.click_button(text=u'Next \u00BB')

        self.wait_for_loading(stable_time=2)

    def click_on_body(self):
        """
        Perform a click on the body to close certain elements
        """
        ac = ActionChains(self._driver)
        ac.move_to_element(self._driver.find_element_by_css_selector("body"))
        ac.click()

    def _growl_xpath(self, type_=None):
        assert type_ in [None, 'warning', 'information', 'critical']

        typecheck = ""
        if type is not None:
            typecheck = "and contains(@class,'%s')" % type_

        return "//div[contains(@class,'bs-growl') %s]" % typecheck

    def _growl_css(self, type_=None):
        assert type_ in [None, 'warning', 'information', 'critical']

        typecheck = ""
        if type is not None:
            typecheck = ".%s" % type_

        return "div.bs-growl%s" % typecheck


    def growl_count(self, type_=None):
        """
        returns the count of the number of growls for the given type (if given)
        """
        return len(self._driver.find_elements_by_css_selector( self._growl_css(type_) ))


    def assert_no_growl_errors(self):
        """
        wrapper function for asserting that no growl errors are present
        """

        if self.growl_count('warning') > 0:
            growl_text = self._driver.find_element_by_xpath(self._growl_xpath('warning')).text
            raise self.failureException('A warning growl was detected: %s' % growl_text)

        if self.growl_count('critical') > 0:
            growl_text = self._driver.find_element_by_xpath(self._growl_xpath('critical')).text
            raise self.failureException('A critical growl was detected: %s' % growl_text)

    def assert_growl_errors_present(self):
        self.assertRaises(self.failureException, self.assert_no_growl_errors)

    def click_growl(self, _type="warning"):
        el = self._driver.find_element_by_xpath(self._growl_xpath(_type))
        el.click()

    def login(self, account, password, language='en_GB'):
        """
        logs into iqportal for the specified credentials
        """

        # logout first
        self.clear_cookies()

        self._driver.get(settings.SELENIUM['browserURL'])
        self.wait_for_loading()

        # FIXME: language select is broken cuz of new select input
        #self.switch_language(language=language)

        self.fill_form(form_name='home-login-form', values={
            'username': account,
            'password': password
        });

        self.click_button(form_name='login-button-form', text='Login')

        self.wait_for_loading(stable_time=2)

    def logout(self):
        """
        logout the current user
        """

        self._driver.get(settings.SELENIUM['browserURL']+'#m=logout')
        self.wait_for_loading()

        self.click_button(panel_name='logout-dialog', text='Logout')
        self.wait_for_loading()


    def switch_language(self, language='en_GB'):
        """
        switches the public user interface to the given language
        argument is language_code
        """

        select_xpath = "//select[@id='id_language_select']"
        select = Select(self._driver.find_element_by_xpath(select_xpath))
        current_language = select.first_selected_option().get_attribute("value")

        if current_language != language:
            select.select_by_value(language)
            self.wait_for_loading()

    def _compose_target(self, block_index=0, column_index=None, item_index=None, item_name='compose-container-items'):
        """
        gets the xpath target for the given arguments.. used as a base for other methods
        """

        target = ""

        if block_index is not None:
            target += "//div[contains(@class, 'bs-compose-block')][%s]" % (block_index + 1) # +1 cuz xpath starts at 1

        if column_index is not None:
            target += "//div[contains(@class, 'bs-compose-column')][%s]" % (column_index + 1)

        if item_index is not None:
            target += "//div[contains(@class, '%s')][%s]" % (item_name, item_index + 1)

        return target

    def _compose_target_css(self, block_index=0, column_index=None, item_index=None, item_name='bs-compose-module'):
        """
        gets the css target for the given arguments.. used as a base for other methods
        """

        target = ""

        if block_index is not None:
            target += ".bs-compose-block:nth-child(%s) " % (block_index + 2) # +2 as the nth-child counts a hidden div as well

        if column_index is not None:
            target += ".bs-compose-column:nth-child(%s) " % (column_index + 2) # +2 as the nth-child counts a hidden div as well

        if item_index is not None:
            target += ".%s:nth-child(%s) " % (item_name, item_index + 2) # +2 as the nth-child counts a hidden div as well

        return target

    def click_cube_table_icon(self, button_title, *args, **kwargs):
        """
        gets hidden menu in cube table and clicks on designated icon to apply to table results

        argument: button_title = [Transpose|Excel export|Review with filtered data|Settings]
        """
        target = self._compose_target_css(*args, **kwargs)
        menu_css = '%s table.basesoft-cubetable thead tr th' % target
        hidden_submenu_button = 'span[title="%s"]' % button_title

        menu_css = self._driver.find_element_by_css_selector(menu_css)
        hidden_submenu_button = self._driver.find_element_by_css_selector(hidden_submenu_button)

        ac = ActionChains(self._driver)
        ac.move_to_element(menu_css)
        ac.click(hidden_submenu_button)
        ac.perform()

    def drag_item_from_palette(self, item_name, *args, **kwargs):
        """
        Drags the item_name as it appears in the pallette (for example 'Single choice') to the given block/column/item
        """

        source = "//div[contains(@class,'bs-palette')]//button[text()='%s']" % item_name
        target = self._compose_target(*args, **kwargs)

        target = self._driver.find_element_by_xpath(target)
        target.click()

        #ac = ActionChains(self._driver)
        #ac.move_to_element(target)
        #ac.click(target)

        source = self._driver.find_element_by_xpath(source)

        ac = ActionChains(self._driver)
        ac.move_to_element(source)
        ac.click()
        ac.drag_and_drop(source, target)
        ac.perform()

    def drag_item_from_palette_by_offset(self, item_name, xoffset, yoffset):
        source = "//div[contains(@class,'bs-palette')]//button[text()='%s']" % item_name
        source = self._driver.find_element_by_xpath(source)

        target = self._compose_target(block_index=None, item_index=0)
        target = self._driver.find_element_by_xpath(target)

        ac = ActionChains(self._driver)
        ac.click_and_hold(source)
        ac.move_to_element_with_offset(target, xoffset, yoffset)
        ac.release()
        ac.perform()

    def connect_jsplumb_terminals(self, source_index, target_index):
        source = "//div[contains(@class, 'jsplumb-droppable')][%d]" % source_index
        source = self._driver.find_element_by_xpath(source)

        target = "//div[contains(@class, 'jsplumb-droppable')][%d]" % target_index
        target = self._driver.find_element_by_xpath(target)

        target.click()

        ac = ActionChains(self._driver)
        ac.drag_and_drop(source, target)
        ac.perform()

    def select_compose_module_floating(self, item_index=0):
        """
        'selects' the given compose module if not already selected so the action palette will become available
        """
        target = self._compose_target(block_index=None, item_index=0)+"/div[%d]/div[contains(@class, 'hd')]" % (item_index+1)

        self._driver.find_element_by_xpath(target).click()

    def wait_for_condition(self, eval_cmd):

        count = 0
        while True:
            count +=1
            time.sleep(1)
            if self._driver.execute_script("return %s" % eval_cmd):
                break
            if count > (self.timeout / 1000):
                raise Exception('timeout')
        time.sleep(1)

    def compose_edit_rte(self, text, *args, **kwargs):
        """
        Changes the text for the given path using the RTE
        """

        # open the editor by clicking the editable for this item
        target = self._compose_target(*args, **kwargs)
        self._driver.find_element_by_xpath(target + "//div[contains(@class, 'EM-editable')]").click()
        time.sleep(1)
        self._driver.switch_to_frame(self._driver.find_element_by_xpath("//iframe[contains(@class, 'yui-editor-editable')]"))

        self.wait_for_condition("document.body._rteLoaded")

        rte_body = self._driver.find_element_by_xpath("//body")
        rte_body.click()
        rte_body.clear()
        rte_body.send_keys(text)

        # back to parent frame
        self._driver.switch_to_default_content()
        self._driver.find_element_by_xpath("//body").click()

        """
        # set selenium frame to be the editor and wait for loaded
        self.selenium.select_frame("//iframe[contains(@class, 'yui-editor-editable')]")
        self.selenium.wait_for_condition("selenium.browserbot.getCurrentWindow().document.body._rteLoaded", self.timeout)

        # type text
        self.selenium.type("//body", text)

        # switch back to parent frame (normal iqportal window)
        self.selenium.select_frame("relative=parent")

        # close editor
        self.selenium.click("//body")
        """

    def compose_edit_wiki(self, text, *args, **kwargs):
        """
        Changes the text for the given path using the wiki editor
        """

        # open the editor by clicking the editable for this item
        target = self._compose_target_css(*args, **kwargs)
        self._driver.find_element_by_css_selector(target + "div[class*='EM-editable']").click()
        self.wait_for_loading()

        editor_body = self._driver.find_element_by_css_selector("textarea[name='wiki_input']")

        editor_body.send_keys(text)
        self.wait_for_loading()
        # click somewhere where there are no clickable elements to close the editor
        self._driver.find_element_by_css_selector("div[id='header-container']").click()

    def compose_edit_inline(self, texts, *args, **kwargs):
        """
        Edits the given text element locator with texts. One compose element must be given and 'texts' should
        be a list of strings where the index is the nth input found
        """

        for text_index, text in enumerate(texts):

            target = self._compose_target(*args, **kwargs)
            target += "//*[contains(@class, 'EM-editable')]"
            target = "(%s)[%s]" % (target, text_index+1)

            try:
                self._driver.find_element_by_xpath(target).click()

            # it might be that a previous text already opened the editor, in that case a click isnt needed
            # this is the kees in iquest
            except WebDriverException, e:
                if str(e).find('Element is not clickable at point') == -1:
                    raise

            # editor ready has a setTimeout in JS land.
            time.sleep(0.1)

            self._driver.find_element_by_xpath("//input[contains(@class, 'iq-editor-manager-input')]").send_keys(text + "\n")

    def click_palette_title(self, title):
        el = self._driver.find_element_by_css_selector('.bs-action-palette div[title="%s"]' % title)
        el.click()

    def click_palette(self, name, palette_index=0):
        """
        palette_index:
            index 0: dashboard actions
            index 1: current selected actions
            index 2: palette (to add new widgets)
        """

        palette = "(//div[contains(@class, 'compose-right')]/div[contains(@class, 'bs-collapse-module')])[%s]" % (palette_index + 1)

        try:
            self._driver.find_element_by_xpath(palette + "[ contains(@class, 'closed') ]")
        except NoSuchElementException:
            pass
        else:
            el = self._driver.find_element_by_xpath(palette + "//div[contains(@class, 'hd')]")
            ac = ActionChains(self._driver)
            ac.click(el)
            ac.perform()

        self._driver.find_element_by_xpath(palette + "//button[text()='%s']" % name).click()

    def expand_layout_pane(self, index=0):
        """
        expands a yui layout pane
        """

        # check if the 'click to open' element cant be found, then return
        # this way we check if the layout is already opened..
        # fixme: if the panel is closed and reopened then this element exists and this function errors
        try:
            self._driver.find_element_by_css_selector("div[class~=yui-layout-clip]")
        except NoSuchElementException:
            return

        # else open it
        css = "div[class~=yui-layout-clip] .collapse"
        el = self._driver.find_elements_by_css_selector(css)[index]
        el.click()
        time.sleep(1) # for animation

    def select_compose_module(self, *args, **kwargs):
        """
        'selects' the given compose module if not already selected so the action palette will become available
        """
        target = self._compose_target(*args, **kwargs)

        els = self._driver.find_elements_by_xpath(target + "[ contains(@class, 'selected') ]")
        if len(els) == 0:
            self._driver.find_element_by_xpath(target).click()

    def run_task(self, task_name, timeout=60):
        """
        must can be called when dashboard is opened. It clears the dashboard, adds the scheduler widget, runs the given task name
        and returns when task is completed. timeout in seconds
        """

        time.sleep(4)
        self.wait_for_loading()

        # goto compose mode
        self.expand_layout_pane(0)
        try:
            self.click_palette('Compose')
        except ElementNotVisibleException:
            self.click_palette('Mode')
            self.click_palette('Compose')
        self.wait_for_loading()

        # just to be sure clear the dashboard
        self.click_palette('Clear dashboard')
        self.click_button(panel_name='dialog-question', text='Yes')

        # add scheduler widget
        self.click_palette('Administration', palette_index=2)
        self.drag_item_from_palette('Scheduler server')
        self.wait_for_loading()
        #self.assert_no_growl_errors()

        self.click_palette('Enable all Tasks', palette_index=1)
        self.wait_for_loading()

        target = self._compose_target(block_index=0, column_index=0, item_index=0, item_name='compose-container-widgets')

        self._driver.find_element_by_xpath(target + '//input[contains(@class, "filter")]').send_keys(task_name)
        time.sleep(4)

        # start the task
        self._driver.find_element_by_xpath(target + "//button[text()='run']").click()

        # wait till the task has started
        #self.wait_for_xpath("//div[contains(@class,'bs-growl')]//p[contains(text(), 'successfully started')]")
        self.wait_for_loading(stable_time=2)

        # it should appear soon in the list.. then wait for it to be done
        self.assert_no_growl_errors()


        #self.wait_for_xpath(target + "//td//div[text()='No records found.']", timeout=timeout)

        # fixme: this could get a better path
        time.sleep(4)
        self.wait_for_selector('.widget .yui-dt-empty')



    def open_fieldset(self, panel_name=None, form_name=None, title=''):

        css = self.wait_for_panel_or_form(panel_name, form_name)
        css += " div.bs-fieldset-collapse-text"

        el = self.find_element_by_css_contains(css, title)
        el.click()

    def _scroll(self, css_selector, value, direction):
        assert direction in ('Left', 'Top')
        el = self._driver.find_element_by_css_selector(css_selector)
        assert el.is_displayed()
        command = "$('%s').scroll%s(%s)" % (css_selector.replace("'", "\\'"), direction, value)
        return self._driver.execute_script(command)

    def scroll_top(self, css_selector, value):
        self._scroll(css_selector, value, 'Top')

    def scroll_left(self, css_selector, value):
        self._scroll(css_selector, value, 'Left')

    def scroll_panel(self, panel_name, top_value=None, left_value=None, form_name=None):
        css = self.wait_for_panel_or_form(panel_name=panel_name, form_name=form_name)
        css += ' > .bd'

        if top_value is not None:
            self.scroll_top(css, top_value)
        if left_value is not None:
            self.scroll_left(css, left_value)

    # function to pick mdx designer drag n drop target for testing
    def _canvas_target(self):
        """
        gets the xpath target for the given arguments.. used as a base for other methods
        """
        import random
        # pick random int from 1 to 3
        random = random.randint(1,4)
        # find target by div title
        css_select = 'div#dropTarget.target'
        target = self._driver.find_element_by_css_selector(css_select)

        return target


    def drag_item_from_carousel(self, item_name, target_css):
        """
        Drags the item_name as it appears in the MDX Designer carousel (for example 'TopCount')
        to the given yahoo target by using css selector
        """

        source = "//div[text()='%s']" % item_name
        dropTarget = self._driver.find_element_by_css_selector(target_css)

        source = self._driver.find_element_by_xpath(source)
        #target = self._driver.find_element_by_xpath(target)

        dropTarget.click()

        ac = ActionChains(self._driver)
        ac.drag_and_drop(source, dropTarget)
        ac.perform()

    def assert_compose_textitem(self, item_text, item_css=".bs-iquest-survey-editable", block_index=0,
                                column_index=None, item_index=None, assert_failure=False):
        """
        Method for asserting compose text items. Asserts warning text of filters as well.
        """
        target = self._compose_target_css(block_index=block_index, column_index=column_index, item_index=item_index)

        if assert_failure:
            el = self._driver.find_element_by_css_selector(target + ".iq-filter-failure")
        else:
            try:
                el = self._driver.find_element_by_css_selector(target + item_css)
            except NoSuchElementException:
                raise NoSuchElementException("Element not found for: %s.%s" % (target, item_css))

        if self.assertEqual(el.text, item_text):
            return

        raise Exception('Element text is not the same as the provided: %s' % item_text)

    def switch_to_frame(self, css_class):
        self._driver.switch_to_frame(self._driver.find_element_by_css_selector("iframe[class*='%s']" % css_class))

    def _get_text_from_selector(self, selector):
        """
        returns the innerText for the given css selector
        """

        return self._driver.find_element_by_css_selector(selector).text

    def get_panel_title(self, panel_name):
        """
        returns the title for the given panel_name
        """
        panel_css = self.wait_for_panel(panel_name)
        hd = self._driver.find_element_by_css_selector('%s > .hd > .tcm' % panel_css)
        return hd.text






class IQubeSeleniumTestCase(SeleniumTestCase):
    pass


class YuiTestRunnerTestCase(SeleniumTestCase):
    """
    SeleniumTest class for testing yui tests
    """

    # override this to point selenium to the correct yui testrunner
    url = None

    # give yui 5 minutes to perform each test
    YUI_TIMEOUT = 5 * 60


    DONE_LOADING_CMD = "\
        return BASESOFT && \
        (!YAHOO.util.Dom.get('bs-loading-dialog') || YAHOO.util.Dom.get('bs-loading-dialog').style.display == 'none') && \
        !YAHOO.util.Dom.hasClass('body', 'loader-busy') && \
        !YAHOO.util.Dom.hasClass('body', 'iq-compose-loading') && \
        !YAHOO.util.Dom.hasClass('body', 'busy') \
    "

    def yui_test(self, yui_test_name):

        # open the test page
        if self.url is None:
            raise Exception('url not set')

        logger.debug('opening url: %s' % self.url)
        self._driver.get(self.url)

        # wait for the run_tests button to apear and click it for the selected test
        self.wait_for_loading()

        # set test values
        self._driver.find_element_by_css_selector('#yuitest-clear-selection').click()
        self.find_element_by_css_contains('label', yui_test_name).click()


        # if assertClean is set for settings run the test twice anyway.. but it only fails if the test also has
        # set the yui_test_assertclean flag (see below)
        if settings.SELENIUM.has_key('assertClean') and settings.SELENIUM['assertClean']:

            el = self._driver.find_element_by_css_selector('#testrunner_repeat')
            el.clear()
            el.send_keys(2)

        # click run
        self._driver.find_element_by_css_selector('#btnRun').click()

        # wait till the tests have run
        el = self._driver.find_element_by_css_selector('#yui_test_failures')
        time_passed = 0
        while True:

            if time_passed > self.YUI_TIMEOUT:
                self.fail('yui timeout (%s seconds)' % self.YUI_TIMEOUT)

            time.sleep(1)
            time_passed =+ 1

            if el.get_attribute("value") != "":
                break

        # yui test result: pretty printing (encode json data and make a pretty dump)
        result = el.get_attribute("value")
        if result != '':
            result = simplejson.loads(result)
            result = simplejson.dumps(result, sort_keys=True, indent=4)

        # assert 0 failures
        failures = self._driver.find_element_by_css_selector('#yui_test_failures').get_attribute("value")
        self.assertTrue( (int(failures) <= 0), "There are %s failing YUI tests. Test result:\n %s" % (failures, result) )

        # assert test clean (see above)
        if settings.SELENIUM['assertClean']:
            newlisteners = self._driver.find_element_by_css_selector('#yui_new_listeners').get_attribute("value")

            # fail test if test requires clean asserting
            if int(newlisteners) > 0 and self._driver.find_element_by_css_selector('#yui_test_assertclean').is_selected():
                listeners_result = self._driver.find_element_by_css_selector('#yui_test_listeners_result').get_attribute("value")
                self.fail("There are %s new listeners and test is marked as being clean: \n%s" % (newlisteners, listeners_result))

            # else just warn if there are any listeners left
            if int(newlisteners) > 0:
                print "\nWARNING: %s has '%s' listeners that need to be cleaned up\n" % (yui_test_name, newlisteners)


    def test_count(self):
        """
        Tests if all the tests specified clientside are called serverside by comparing the count of the tests
        """

        # count own test methods by looping all test methods (and -1 to exclude itself)
        python_test_count = len([i for i in dir(self) if i[0:5] == 'test_']) -1

        # bail out if nothing to test
        if python_test_count == 0:
            return

        # now start selenium and open the test page
        if self.url is None:
            raise Exception('url not set')
        self._driver.get(self.url)

        # wait for the run_tests button to apear and click it for the selected test
        self.wait_for_loading()

        # this is what yui testrunner counted as a total number available tests
        yui_test_count = int(self._driver.find_element_by_css_selector('#yui_total_test_count').get_attribute("value"))

        # assert count
        self.assertEqual(yui_test_count, python_test_count, 'yui and python testcount should be equal')


class XPath(str):
    """
    class extending string used to give an indication that the string is of xpath type
    """


class SelectionAction(object):
    pass






