# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.

import datetime
from dateutil.relativedelta import relativedelta

from basesoft.web.test.selenium.selenium_testcase import SeleniumTestCase, XPath
from basesoft.web.contrib.announce.controllers import AnnounceController
from basesoft.web.contrib.announce.models import Announcement
from basesoft.web.contrib.i18n.models import Locale
from basesoft.util.virtual_date import now, set_date
from selenium.common.exceptions import NoSuchElementException

class TestCase(SeleniumTestCase):

    def setUp(self):
        set_date(datetime.datetime.now())
        super(TestCase, self).setUp()
        self.connect_db()

        self._announcemnt_counter = 0

    def _create_announcement(self, account, publish_date, expire_date=None, recipients=[], must_read=False):

        # create announcement
        locale_en = self.session.query(Locale).filter_by(locale='en_GB').one()
        ac = AnnounceController(self.session, account_id=account.id)
        announcement = ac.save_announcement(None, {
            'announce': {
                'publish_date': publish_date,
                'expire_date': expire_date,
                'status': Announcement.STATUS_PUBLISHED,
                'label': 'test announcement %s' % self._announcemnt_counter,
                'must_read': must_read,
                'recipients': recipients,
            },
            'wiki': {
                'fallback_locale_id': locale_en.id,
                'locale_id': locale_en.id,
                'title': 'title english',
                'content': 'content english'
            }
        })

        self._announcemnt_counter += 1

        return announcement


    def test_create_announce(self):

        account = self._create_active_account()
        group = account.groups[0].group
        self._attach_permissions_to_group(group, 'view_dashboard', 'save_dashboard', 'manage_announcements', 'view_domain', 'view_license', 'mark_as_archived', 'mark_as_deleted')

        self.login(account.name, 'test_password')

        self.click_iconbar('icon-announce-manager')
        self.click_menu('announce-manager', text='Create a new Announcement')
        self.wait_for_loading()

        self.fill_form('announcement-details', values={
            'label': 'My Announcement',
            'publish_date': ('date', datetime.datetime.now()),
            'status': ('select', 'Published'),
            'fallback_locale_id': ('select', 'English'),
            'locale_id': ('select', 'English'),
            'title': 'My Title',
            'content': ('textarea', 'My Content'),
        })
        self.click_button('announcement-details', text='Save')
        self.wait_for_loading()

        self.click_cq('announce-manager', row_index=0, col_name='translate')
        self.wait_for_loading()

        self.fill_form('wiki-translate-panel', values={
            'locale_id': ('select', 'Dutch'),
        })
        self.wait_for_loading()
        self.fill_form('wiki-translate-panel', values={
            'title': 'Mijn Titel',
            'content': ('textarea', 'Mijn content'),
        })
        self.click_button('wiki-translate-panel', text='Save')
        self.wait_for_loading()


    def test_view_announcements(self):
        """
        tests various receiving of announcements with a user that has receive_announcements permission (so he also has dock/widget)
        """

        # creator
        account = self._create_active_account()
        group = account.groups[0].group
        self._attach_permissions_to_group(group, 'view_dashboard', 'save_dashboard', 'manage_announcements', 'view_domain', 'view_license')

        # recipient

        recipient_license = self._create_license('new license', domain=account.default_group.license.domain)
        account_recipient = self._create_active_account(license=recipient_license)
        self._attach_permissions_to_group(account_recipient.groups[0].group, 'view_dashboard', 'save_dashboard', 'receive_announcements')

        # make announcement with must-read
        self._create_announcement(account, publish_date=now(), recipients=['license_%s' % recipient_license.id], must_read=True)

        # make announcement without must-read
        self._create_announcement(account, publish_date=now(), recipients=['license_%s' % recipient_license.id], must_read=False)

        # create a new announcement, before the creation of the user, expecting not to see the announcement
        self._create_announcement(account, publish_date=now() - relativedelta(days=1), recipients=['license_%s' % recipient_license.id])

        # create an expired announcement, expecting not to see it
        self._create_announcement(account, publish_date=now(), expire_date=now(), recipients=['license_%s' % recipient_license.id])


        self.login(account_recipient.name, 'test_password')

        # expect must-read popup, click mark as read
        self.wait_for_panel('announce-recipient-panel')
        self.click_button('announce-recipient-panel', text='Mark as read')
        self.wait_for_loading()

        # assert custom query
        self.click_iconbar('icon-announce-recipient-manager')
        self.assert_cq('announce-recipient-manager', row_index=0, data={'label': 'test announcement 0'})
        self.assert_cq('announce-recipient-manager', row_index=1, data={'label': 'test announcement 1'})

        # assert there is no row 3, these announcements shouldnt be visible for this user
        self.assertRaises(NoSuchElementException, self.assert_cq, 'announce-recipient-manager', row_index=2, data={'label': 'test announcement 2'})

    def test_view_announcement_must_read(self):
        """
        tests receiving of an announcement for a user that does not have the receive_announcements permission, still he should see the modal announcements
        """

        account = self._create_active_account()
        group = account.groups[0].group
        self._attach_permissions_to_group(group, 'view_dashboard', 'save_dashboard', 'manage_announcements', 'view_domain', 'view_license')

        # recipient
        recipient_license = self._create_license('new license', domain=account.default_group.license.domain)
        account_recipient = self._create_active_account(license=recipient_license)
        self._attach_permissions_to_group(account_recipient.groups[0].group, 'view_dashboard', 'save_dashboard')

        # make announcement with must-read
        self._create_announcement(account, publish_date=now(), recipients=['license_%s' % recipient_license.id], must_read=True)

        self.login(account_recipient.name, 'test_password')

        # expect must-read popup, click mark as read
        self.wait_for_panel('announce-recipient-panel')
        self.click_button('announce-recipient-panel', text='Mark as read')
        self.wait_for_loading()


# below is for running the test from eclipse
if __name__ == '__main__':
    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_compose.py', 'basesoft.web.test.selenium.test_announce.TestCase.test_create_announce']
    run()
    sys.exit()
