from decimal import Decimal
import datetime
from twisted.trial.unittest import SkipTest
from basesoft.util.ordered_dict import OrderedDict
from basesoft.util.virtual_date import today, now
from basesoft.web.contrib.i18n.models import Locale
from basesoft.web.contrib.order.controllers import TaxController, GLAccountController
from basesoft.web.test.models import MockOrderLine
from basesoft.web.test.selenium.selenium_testcase import SeleniumTestCase
from basesoft.web.test.unit.test_order_controller import MockOrderController


class TestCase(SeleniumTestCase):
    """
    basesoft.web.test.selenium.test_order.TestCase
    """

    def setUp(self):

        super(TestCase, self).setUp()
        self.connect_db()

        self.merchant = self._create_license()
        self.ma = self._create_active_account(self.merchant)
        self.locale = self.session.query(Locale).filter_by(locale='en_GB').one()
        self._add_address_and_bank_account_to_company(self.merchant.company)

        self.account_license = self._create_license(domain=self.merchant.default_group.license.domain)
        self._add_address_and_bank_account_to_company(self.account_license.company)
        self.account = self._create_active_account(license=self.account_license)


        self._attach_permissions_to_group(self.account.groups[0].group,
                                          'view_dashboard', 'save_dashboard', 'view_order_rules',
                                          'manage_order_rules', 'view_mock_orders',
                                          'view_orders',
                                          'view_domain', 'view_license' # needed to see all clients in domain
                                          )

        self.order_controller = MockOrderController(self.sa_session, account_id=self.account.id)

    def _setup_tax_and_gl_account(self):
        self._attach_permissions_to_group(self.account.groups[0].group, 'manage_tax_classes', 'manage_country_tax', 'manage_gl_accounts', 'view_domain', 'view_license')

        c = TaxController(self.sa_session, account_id = self.account.id)
        tax_class = c.save_tax_class_state(None, {'name': 'Administrative Fee Tax', 'code': 'MOCK_ORDER_TAX_CLASS'})
        tcc = c.save_country_tax_state(None, {
            'country_id': 155,
            'tax_class_id': tax_class.id,
            'tax_rates': [{
                'start_date': today(),
                'rate': Decimal(21, 0)
            }]
        })

        c = GLAccountController(self.sa_session, account_id = self.account.id)
        ga = c.save_gl_account_state(None, {'merchant_id': self.merchant.id, 'code': 'MOCK_ORDER_GL_ACCOUNT', 'account_number': 'GL1234'})

        return tax_class, tcc, ga

    def _make_rule(self, name, **kwargs):

        self.order_controller.save_rule(None, {
            'order_type': kwargs.get('order_type', 'MockOrder'),
            'name': name,
            'enabled': kwargs.get('enabled', True),
            'execute_order': kwargs.get('execute_order', None),
            'description': kwargs.get('description', ''),
            'start_date': kwargs.get('start_date', now()),
            'end_date': kwargs.get('end_date', None),
            'match_all': kwargs.get('match_all', True),
            'stop_processing': kwargs.get('stop_processing', False),
            'conditions': kwargs.get('conditions', []),
            'actions': kwargs.get('actions', [])
        })

    def _make_rule_order(self, **kwargs):

        state = {
            'merchant': self.merchant,
        }
        lines = [{
            'description': 'Some product',
            'polymorphic_type': MockOrderLine,
            'quantity': kwargs.get('quantity', 1),
            'price': Decimal('99.9')
        }]
        order = self.order_controller.save_or_update_order(None, lines, **state)

        return order


    def test_order_rule(self):
        """
        basesoft.web.test.selenium.test_order.TestCase.test_order_rule
        """
        tax_class, tcc, ga = self._setup_tax_and_gl_account()

        self.login(self.account.name, 'test_password')

        self.click_iconbar(title='Order rules manager', hover='Tax class manager')

        self.click_menu(panel_name='order-rules-manager', text='Create new Order rule')
        self.wait_for_loading()

        self.fill_form(panel_name='order-rule-details', values={
            'order_type': ('select', 'MockOrder'),
            'name': 'My roel',
            'start_date': ('datetime', datetime.datetime.now())
        })
        self.wait_for_loading()

        # conditions

        self.fill_form(panel_name='order-rule-details', rowcontrol_name='conditions', rowcontrol_row_index=0, values=OrderedDict((
            ('condition', ('select', 'Merchant')),
        )))
        self.wait_for_loading()

        self.fill_form(panel_name='order-rule-details', rowcontrol_name='conditions', rowcontrol_row_index=0, values=OrderedDict((
            ('value_select', ('autocomplete', self.merchant.name)),
        )))

        self.click_button(panel_name='order-rule-details', text='Add condition')
        self.fill_form(panel_name='order-rule-details', rowcontrol_name='conditions', rowcontrol_row_index=1, values=OrderedDict((
            ('condition', ('select', 'Is resubmit')),
            ('value_bool', ('checkbox', True)),
        )))

        self.click_button(panel_name='order-rule-details', text='Add condition')
        self.fill_form(panel_name='order-rule-details', rowcontrol_name='conditions', rowcontrol_row_index=2, values=OrderedDict((
            ('condition', ('select', 'Total amount (ex VAT)')),
            ('operator', ('select', 'Greater than')),
            ('value_decimal', '99.9'),
        )))

        self.click_button(panel_name='order-rule-details', text='Add condition')
        self.fill_form(panel_name='order-rule-details', rowcontrol_name='conditions', rowcontrol_row_index=3, values=OrderedDict((
            ('condition', ('select', 'Quantity over X')),
            ('value_int', '10'),
        )))

        # actions

        self.fill_form(panel_name='order-rule-details', rowcontrol_name='actions', rowcontrol_row_index=0, values=OrderedDict((
            ('action', ('select', 'Fixed amount discount')),
            ('value_decimal', '10.12'),
            ('description', 'special price 4u'),
            ('is_persistent', ('checkbox', True)),
        )))
        self.click_button(panel_name='order-rule-details', text='Add action')

        self.fill_form(panel_name='order-rule-details', rowcontrol_name='actions', rowcontrol_row_index=1, values=OrderedDict((
            ('action', ('select', 'All free!')),
            ('description', 'u r lucky'),
            ('tax_class_id', ('select', 'Administrative Fee Tax')),
            ('gl_account_code', ('select', 'MOCK_ORDER_GL_ACCOUNT')),

        )))

        # save

        self.click_button(panel_name='order-rule-details', text='Save')
        self.wait_for_loading()

        # re-open and save again
        self.click_cq(panel_name='order-rules-manager', row_index=0, col_name='rule_name')
        self.wait_for_loading()

        self.click_button(panel_name='order-rule-details', text='Save')
        self.wait_for_loading()

    def test_order_log(self):

        self._attach_permissions_to_group(self.account.groups[0].group, 'view_order_log')

        tax_class, tcc, ga = self._setup_tax_and_gl_account()

        self._make_rule('rule 1',
            match_all = False,
            description='rule description',
            conditions =  [{
                'condition': 'merchant',
                'operator': None, 'value_text': None, 'value_bool': None,
                'value_int': self.merchant.id,
                'value_decimal': None,
            }],
            actions = [{
                'action': 'fixed_amount_discount',
                'description': 'no tax',
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': 10,
                'gl_account_code': None,
                'tax_class_id': None,
            }, {
                'action': 'fixed_amount_discount',
                'description': 'with tax',
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': 10,
                'gl_account_code': None,
                'tax_class_id': tax_class.id,
            }]
        )

        order = self._make_rule_order()
        self.order_controller.submit_order(order)

        self.login(self.account.name, 'test_password')

        self.click_iconbar(title='MockOrder manager')

        self.click_cq(panel_name='default-order-manager', row_index=0, col_name='order_log')
        self.wait_for_loading()

    def test_order_simulate(self):

        self._attach_permissions_to_group(self.account.groups[0].group, 'manage_order_simulations')

        tax_class, tcc, ga = self._setup_tax_and_gl_account()

        self._make_rule('rule 1',
            match_all = False,
            description='rule description',
            conditions = [{
                'condition': 'merchant',
                'operator': None, 'value_text': None, 'value_bool': None,
                'value_int': self.merchant.id,
                'value_decimal': None,
            }],
            actions = [{
                'action': 'fixed_amount_discount',
                'description': 'no tax',
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': 10,
                'gl_account_code': None,
                'tax_class_id': None,
            }, {
                'action': 'fixed_amount_discount',
                'description': 'with tax',
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': 10,
                'gl_account_code': None,
                'tax_class_id': tax_class.id,
            }]
        )

        self.login(self.account.name, 'test_password')

        self.click_iconbar(title='Order simulation manager', hover='Tax class manager')
        self.click_menu(panel_name='order-simulation-manager', text='Create new Order simulation')

        self.fill_form(panel_name='order-simulation-details', values={
            'name': 'Order sim1',
            'polymorphic_type': ('select', 'MockOrder'),
        })
        self.wait_for_loading()

        self.fill_form(panel_name='order-simulation-details', values={
            'merchant': ('autocomplete', self.merchant.name),
        })
        self.wait_for_loading()

        self.fill_form(panel_name='order-simulation-details', values={
            'scope': ('autocomplete', 'test_group'),
        })
        self.wait_for_loading()

        self.click_button(panel_name='order-simulation-details', text='Add line')

        self.fill_form(panel_name='order-simulation-details', rowcontrol_name='lines', rowcontrol_row_index=0, values={
            'polymorphic_type': ('select', 'Mock line'),
        })
        self.fill_form(panel_name='order-simulation-details', rowcontrol_name='lines', rowcontrol_row_index=1, values={
            'polymorphic_type': ('select', 'Mock line'),
        })

        self.wait_for_loading()

        self.fill_form(panel_name='order-simulation-details', rowcontrol_name='lines', rowcontrol_row_index=0, values={
            'description': 'Line 1',
            'quantity': '10',
            'price': '11',

        })
        self.fill_form(panel_name='order-simulation-details', rowcontrol_name='lines', rowcontrol_row_index=1, values={
            'description': 'Line 2',
            'quantity': '20',
            'price': '22',
        })

        self.click_button(panel_name='order-simulation-details', text='Save')
        self.wait_for_loading()

        # re-open and save again
        self.click_cq(panel_name='order-simulation-manager', row_index=0, col_name='order_simulation_name')
        self.wait_for_loading()

        self.click_button(panel_name='order-simulation-details', text='Save')
        self.wait_for_loading()

        # re-open and click simulate
        self.click_cq(panel_name='order-simulation-manager', row_index=0, col_name='order_simulation_name')
        self.wait_for_loading()

        self.click_button(panel_name='order-simulation-details', text='Simulate submit')
        self.wait_for_loading()

        # verify last amount golumns
        amounts = [
            '133.1000',
            '532.4000',
            '-10.0000',
            '-12.1000',
            '530.0000',
            '113.4000',
            '643.4000',
        ]
        last_tds = self._driver.find_elements_by_css_selector('div[name="order-simulation-result"] td:last-child')
        for i in range(len(last_tds)):
            td = last_tds[i]
            self.assertEqual(td.text, amounts[i])

