# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.
from basesoft.web.core.util.db import Session

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.core.util.controllers import AbstractController, \
    AccountConfigController, transaction_decorator

from basesoft.web.core.exceptions import SecurityException, PublicSystemException,\
    PermissionDeniedException
from basesoft.web.contrib.contact.models import Person
from basesoft.web.contrib.contact.models import Contact, ContactGroup
from twisted.trial.unittest import SkipTest
from sqlalchemy.orm.exc import NoResultFound
from django.http import HttpRequest

class MockController(AbstractController):
    """
    This is a mock controller to test nested transactions
    """

    PRIV_LICENSE_ONLY = 32
    PRIV_WORLD_ONLY = 64

    @transaction_decorator
    def save_person(self, raise_exception=False, nested=True):
        """
        This saves a simple person and triggers a nested save in case applicable.
        """
        p = Person()
        p.first_name = 'Johnny'
        self.sa_session.add(p)
        #self.sa_session.flush()
        if raise_exception:
            self.save_person_with_exception_thrown()
        elif nested:
            self.save_person(nested=False)
        return p

    @transaction_decorator
    def save_person_with_exception_thrown(self):
        """
        This saves a person and then decides to throw an exception
        """
        p = Person()
        p.first_name = 'Bruno'
        self.sa_session.add(p)
        #self.sa_session.flush()
        raise Exception('This is a test with an exception Raised')

    @transaction_decorator
    def save_same_level(self):
        """
        This first saves a person without error and then saves a person with
        an exception thrown.
        """
        self.save_person()
        self.save_person_with_exception_thrown()


    @transaction_decorator
    def permission_required(self):
        """
        Checks if the group who instantiated this controller has the two permissions.
        """
        self.assert_permissions('permission_a', 'permission_b')
        return "Permission Check Passed"

    @transaction_decorator
    def permission_required_fail(self):
        """
        This checks if the group who instantiated this controller has three
        permissions.
        """
        self.assert_permissions('permission_a', 'permission_b', 'permission_c')
        return "Permission Check Passed"

    @transaction_decorator
    def permission_required_fail_on_undefined_permission(self):
        """
        This uses an undefined permissions, which will throw an exception
        """
        self.assert_permissions('permission_a', 'permission_b', 'undefined_permission')
        return "Permission Check Passed"

    @transaction_decorator
    def permission_required_for_owner_update(self, obj):
        """
        This requires the PRIV_UPDATE on the supplied object.
        """
        self.assert_privileges_on_object(obj, Contact.PRIV_UPDATE)
        return "Privileges Check Passed"

    @transaction_decorator
    def permission_required_for_owner_update_and_view(self, obj):
        """
        This requires the PRIV_UPDATE and PRIV_VIEW on the supplied object.
        """
        self.assert_privileges_on_object(obj, Contact.PRIV_UPDATE, Contact.PRIV_VIEW)
        return "Privileges Check Passed"

    @transaction_decorator
    def permission_required_for_owner_delete(self, obj):
        """
        This requires the PRIV_DELETE on the supplied object.
        """
        self.assert_privileges_on_object(obj, Contact.PRIV_DELETE)
        return "Privileges Check Passed"

    @transaction_decorator
    def permission_required_for_license_only(self, obj):
        """
        This requires the PRIV_LICENSE_ONLY on the supplied object.
        """
        self.assert_privileges_on_object(obj, MockController.PRIV_LICENSE_ONLY)
        return "Privileges License Check Passed"

    @transaction_decorator
    def permission_required_for_world_only(self, obj):
        """
        This requires the PRIV_WORLD_ONLY on the supplied object.
        """
        self.assert_privileges_on_object(obj, MockController.PRIV_WORLD_ONLY)
        return "Privileges World Check Passed"

    @transaction_decorator
    def permission_required_for_view(self, obj):
        """
        This requires the PRIV_VIEW on the supplied object.
        """
        self.assert_privileges_on_object(obj, Contact.PRIV_VIEW)
        return "Privileges Check Passed"

    @transaction_decorator
    def usage_option_credit_validation(self, credit='test_credit'):
        """
        This tests credits for a usage option.
        """
        self.assert_credits(credit)
        return 'Got Credit!'

class MockRequest(HttpRequest):
    """
    Mock request object used for the logger
    """
    def __init__(self):
        self.Session = Session()
        self.session = {'user_id': 1}
        self.META = {'REMOTE_ADDR': '127.0.0.1', 'HTTP_HOST': 'iqportal.bier'}


class AbstractControllerTestCase(iqportal_unittest.IQPortalTestCase):
    """
    This tests the Abstract Controller stuff
    """
    def setUp(self):
        """
        Reset database and instantiate the MockController.
        """
        self.connect_db()

        account = self._create_active_account()

        from basesoft.web.contrib.license.models import Permission

        for perm in ['permission_a', 'permission_b', 'permission_c']:
            try:
                self.session.query(Permission).filter(Permission.name==perm).one()
            except NoResultFound:
                p = Permission(name=perm, title=perm)
                self.session.add(p)

        self.group = account.groups[0].group
        self.group.permissions = self.session.query(Permission).filter(
                                 Permission.name.in_(['permission_a', 'permission_b'])).all()

        self.session.add(self.group)
        self.session.flush()

        self.controller = MockController(sa_session=self.session,account_id=account.id)

    def __create_contact(self, group=None):
        """
        Creates a contact for testing purposes.
        """
        if group is None:
            group = self.group

        contact = Contact()
        self.__attach_group_to_contact(contact, group, (Contact.PRIV_UPDATE,))
        self.__attach_group_to_contact(contact, group, (MockController.PRIV_LICENSE_ONLY,), group_has_priv=False, license_has_priv=True)
        self.__attach_group_to_contact(contact, group, (MockController.PRIV_WORLD_ONLY,), group_has_priv=False, world_has_priv=True)
        self.__attach_group_to_contact(contact, group, (Contact.PRIV_VIEW,), group_has_priv=False, license_has_priv=True)
        self.__attach_group_to_contact(contact, group, (Contact.PRIV_DELEGATE_PRIVILEGES,), group_has_priv=True)
        self.session.add(contact)

        return contact

    def __attach_group_to_contact(self, contact, group, privs, group_has_priv=True, license_has_priv=False, domain_has_priv=False, world_has_priv=False):
        """
        """
        for i in privs:
            cg = ContactGroup()
            cg.contact = contact
            cg.group = group
            cg.privilege = i
            cg.group_has_privilege = group_has_priv
            cg.license_has_privilege = license_has_priv
            cg.domain_has_privilege = domain_has_priv
            cg.world_has_privilege = world_has_priv
            contact.groups.append(cg)
            self.session.add(cg)

        if len(privs) == 1:
            return cg

    def __add_usage_option(self, usage_option, credits):
        """
        This adds a usage option with credits to the license of the default
        user group for testing purposes.
        """
        from basesoft.web.contrib.license.models import UsageOption, \
                                                               LicenseUsage

        try:
            uo = self.session.query(UsageOption).filter(UsageOption.name==usage_option).one()
        except:
            uo = UsageOption()
            uo.name = usage_option
            self.session.add(uo)

        luo = LicenseUsage()
        luo.license = self.group.license
        luo.usage_option = uo
        luo.credits = credits

        self.session.add(luo)
        self.session.flush()


    def test_grant_revoke_privileges(self):
        """
        """
        contact = self.__create_contact()

        go = self.controller.get_group_instance_by_privilege(contact, Contact.PRIV_VIEW)
        self.assertFalse(go.group_has_privilege)
        self.assertTrue(go.license_has_privilege)

        self.controller.grant_privilege_to(contact, Contact.PRIV_VIEW, group=True)

        go = self.controller.get_group_instance_by_privilege(contact, Contact.PRIV_VIEW)
        self.assertTrue(go.group_has_privilege)
        self.assertTrue(go.license_has_privilege)

        self.controller.revoke_privilege_from(contact, Contact.PRIV_UPDATE, group=True)
        go = self.controller.get_group_instance_by_privilege(contact, Contact.PRIV_UPDATE)
        self.assertEqual(go, None)

    def test_save_contact_with_hook(self):
        """
        """
        from basesoft.web.contrib.license.models import PrivilegeObjectHook
        account = self._create_active_account(self.group.license, 'uberlord', 'my admins')
        other_group = account.groups[0].group

        # we add a hook for every Contact object that gets created for this Domain, License and Group
        # it will add All privileges on the Contact object to the "other_group"
        poh = PrivilegeObjectHook()
        poh.domain = self.group.license.domain
        poh.license = self.group.license
        poh.group = self.group
        poh.group_privileges = sum(Contact.all_privileges)
        poh.object = 'Contact'
        poh.target_group = other_group

        self.session.add(poh)
        self.session.flush()

        contact = Contact()

        self.__attach_group_to_contact(contact, self.group, (Contact.PRIV_UPDATE,))

        # add the contact
        self.session.add(contact)
        self.session.flush()
        self.session.expire(contact)

        # assert that the Contact has 5 additional groups (all privileges = 5 entries)
        self.assertTrue(len(contact.groups)==6)

        # remove this hook, we will add another one.
        self.session.delete(poh)
        self.session.flush()

        # we
        poh = PrivilegeObjectHook()
        poh.target_group = self.group
        poh.domain = self.group.license.domain
        poh.license_privileges = Contact.PRIV_DELETE
        poh.object = 'Contact'

        self.session.add(poh)
        self.session.flush()

        contact = self.__create_contact(other_group)
        self.session.flush()
        self.session.expire(contact)

        try:
            self.controller.assert_privileges_on_object(contact, Contact.PRIV_VIEW, MockController.PRIV_LICENSE_ONLY, Contact.PRIV_DELETE)
        except:
            self.fail('Asserting raised exception')

    def test_permissions_success(self):
        """
        This will simply test global permissions to which the account has rights.
        """
        self.assertEqual(self.controller.permission_required(),
                         'Permission Check Passed')

    def test_permissions_fail(self):
        """
        This will test global permissions to which the account does not have rights.
        """
        self.assertRaises(PermissionDeniedException,
                          self.controller.permission_required_fail)

    def test_permissions_fail_on_undefined_permission(self):
        """
        This will test global permissions with a list of permissions that are
        not all available in the database.
        """
        self.assertRaises(PermissionDeniedException,
                          self.controller.permission_required_fail_on_undefined_permission)

    def test_domain_group_on_object(self):
        """
        This will test the Service Manager group linked to domains directly.
        See for more info trac ticket #309 or read the Permissions wiki page.
        """
        account = self._create_active_account()
        contact = self.__create_contact()

        controller = MockController(sa_session=self.session,account_id=account.id)

        # before we are part of the Domain Group, it should raise an exception.
        self.assertRaises(PermissionDeniedException,
                          controller.permission_required_for_owner_update,
                          contact)

        # add the original domain to this newly created group
        account.groups[0].group.domains.append(self.group.license.domain)
        self.session.add(account.groups[0].group)
        self.session.flush()

        # now that the account became part of the Domain through the DomainGroup
        # we should have privileges.
        self.assertEqual(controller.permission_required_for_owner_update(contact),
                         'Privileges Check Passed')

    def test_privileges_with_object(self):
        """
        This will test privileges on a certain object.
        """
        contact = self.__create_contact()
        self.assertEqual(self.controller.permission_required_for_owner_update(contact),
                         'Privileges Check Passed')

    def test_privileges_with_object_on_group_and_license(self):
        """
        This will checks for group level privileges first followed by the
        license privileges second. Please see the setUp method.
        """
        contact = self.__create_contact()
        self.assertEqual(self.controller.permission_required_for_owner_update_and_view(contact),
                         'Privileges Check Passed')

    def test_privileges_with_object_fail(self):
        """
        Our test account doesn't have delete privileges on the object.
        Validate this.
        """
        contact = self.__create_contact()
        self.assertRaises(PermissionDeniedException,
                          self.controller.permission_required_for_owner_delete,
                          contact)

    def test_privileges_only_on_license(self):
        """
        There is a license only mockcontroller privilege, see if that works.
        """
        contact = self.__create_contact()
        self.assertEqual(self.controller.permission_required_for_license_only(contact),
                         'Privileges License Check Passed')

    def test_privileges_only_for_world(self):
        """
        """
        contact = self.__create_contact()
        self.assertEqual(self.controller.permission_required_for_world_only(contact),
                         'Privileges World Check Passed')


    def test_privileges_on_different_group(self):
        """
        Lets see if we attach another group to the created contact.
        """
        account = self._create_active_account()
        contact = self.__create_contact(account.groups[0].group)
        self.__attach_group_to_contact(contact,self.group,(Contact.PRIV_VIEW,))

        self.assertEqual(self.controller.permission_required_for_view(contact),
                         'Privileges Check Passed')

    def test_privileges_on_different_group_fail(self):
        """
        Lets see if we can make the other group flunk by adding the wrong privileges.
        """
        account = self._create_active_account()
        contact = self.__create_contact(account.groups[0].group)
        self.__attach_group_to_contact(contact,self.group,(Contact.PRIV_DELETE,))

        self.assertRaises(PermissionDeniedException,
                          self.controller.permission_required_for_view,
                          contact)

    def test_default_get_method(self):
        contact = self.__create_contact()

        c = self.controller.get_privileged_object_by_id(Contact, contact.id)
        self.assertEqual(c[0], contact)

        c = self.controller.get_privileged_object_by_id(Contact, contact.id,  Contact.PRIV_DELETE)
        self.assertTrue(len(c)==0)

    def test_get_query_filter_on_privileges(self):
        """
        """
        # this contact should be in the result
        contact1 = Contact()
        self.__attach_group_to_contact(contact1, self.group, (Contact.PRIV_UPDATE,))
        self.__attach_group_to_contact(contact1, self.group, (Contact.PRIV_VIEW,), group_has_priv=False, domain_has_priv=True)
        self.session.add(contact1)

        account = self._create_active_account(self.group.license, 'uberlord', 'my admins')
        other_group = account.groups[0].group

        # this contact shouldn't be in the result
        contact2 = Contact()
        self.__attach_group_to_contact(contact2, other_group, (Contact.PRIV_VIEW,))
        self.session.add(contact2)

        # this contact should be in the result
        contact3 = Contact()
        self.__attach_group_to_contact(contact3, other_group, (Contact.PRIV_VIEW,), license_has_priv=True)
        self.session.add(contact3)

        account = self._create_active_account()
        non_domain_group = account.groups[0].group

        # this contact shouldn't be in the result
        contact4 = Contact()
        self.__attach_group_to_contact(contact4, non_domain_group, (Contact.PRIV_VIEW,))
        self.__attach_group_to_contact(contact4, self.group, (Contact.PRIV_DELETE,))
        self.session.add(contact4)

        # this contact should be in the result
        contact5 = Contact()
        self.__attach_group_to_contact(contact5, non_domain_group, (Contact.PRIV_VIEW,), license_has_priv=True)
        self.__attach_group_to_contact(contact5, other_group, (Contact.PRIV_UPDATE,))
        self.__attach_group_to_contact(contact5, other_group, (Contact.PRIV_VIEW,), group_has_priv=False, license_has_priv=True)
        self.session.add(contact5)

        self.session.flush()

        filter, j = self.controller.get_query_filter_on_privileges(Contact, ContactGroup, Contact.PRIV_VIEW)
        result = self.session.query(Contact).select_from(j).filter(filter).all()
        self.assertTrue(contact1 in result)
        self.assertTrue(contact3 in result)
        self.assertTrue(contact5 in result)
        self.assertFalse(contact2 in result)
        self.assertFalse(contact4 in result)

    def test_credits_success(self):
        """
        This will pass the check for test_credit.
        """
        self.__add_usage_option('test_credit', 1)
        self.assertEqual(self.controller.usage_option_credit_validation(),
                         'Got Credit!')

    def test_credits_failure_invalid_usage_option(self):
        """
        This will raise an exception cuz the usage option does not exist.
        """
        self.assertRaises(PermissionDeniedException,
                          self.controller.usage_option_credit_validation,
                          self._uniquestring('random'))

    def test_credits_failure(self):
        """
        This will raise an exception cuz the license has used all credits.
        """
        self.__add_usage_option('test_credit', 0)
        self.assertRaises(PermissionDeniedException,
                          self.controller.usage_option_credit_validation)

    def test_flunked_nested_transaction(self):
        """
        Tests if it indeed flunks and has been roll-backed.
        """
        # get the original number of persons, we want to verify that nothing got
        # added due to a flunk.
        original_number_of_persons = len(self.session.query(Person).all())

        self.assertRaises(Exception, self.controller.save_person, raise_exception=True)

        # one left from the account
        self.assertEqual(len(self.session.query(Person).all()),
                         original_number_of_persons)

    def test_same_level_with_exception(self):
        """
        This tests the scenario of multiple nested transactions on the same level
        """
        original_number_of_persons = len(self.session.query(Person).all())

        self.assertRaises(Exception, self.controller.save_same_level)
        # one left from the account
        self.assertEqual(len(self.session.query(Person).all()), original_number_of_persons)

    def test_nested_transaction(self):
        """
        Test a correctly executed nested save.
        """
        original_number = len(self.session.query(Person).all())

        self.controller.save_person() # this will add two new ones.

        persons = self.session.query(Person).all()
        # two new ones plus the one from the account
        self.assertEqual(len(persons), original_number+2)

        # we test it again by doing a direct query on the database.
        from sqlalchemy.sql import select
        from basesoft.web.contrib.contact.sa_datamapper import get_tables
        s = select([get_tables()['tblPerson']]).order_by(get_tables()['tblPerson'].c.first_name)
        result = self.sa_session.connection().execute(s)
        # two new ones plus the one from the account
        self.assertEqual(len([r for r in result]), original_number+2)
        # note: removed the first name comparing cuz we test that in a different
        # test anyways. This is only a test to see if a nested transaction works
        # not a test for the Contact Controller.


class AccountControllerControllerTestCase(iqportal_unittest.IQPortalTestCase):
    """
    This tests the AccountControllerController stuff
    """
    def setUp(self):
        """
        """
        self.connect_db()

        config_name = self._uniquestring('basesoft.web.test')

        # create some test configs
        from basesoft.web.contrib.account.models import Config
        self.test_config = Config()
        self.test_config.name = config_name
        self.test_config.type = 'integer'
        self.test_config.default = '15'
        self.session.add(self.test_config)
        self.session.flush()

        self.account = self._create_active_account()

    def test_config_name_incorrect(self):
        """
        Tests trying to set/get value for an inexisting config
        """
        controller = AccountConfigController(sa_session=self.session, account_id=self.account)

        self.assertRaises(PublicSystemException,
                          controller.set,
                          "test.inexistent.config", '1')

        self.assertRaises(PublicSystemException,
                          controller.get,
                          "test.inexistent.config")

    def test_account_self_configs(self):
        """
        Tests the get/set methods for when controller is used by current account.
        """
        controller = AccountConfigController(sa_session=self.session, account_id=self.account)

        # test with values incorrect
        account_config = {self.test_config.name: '12incorrect'}
        self.assertRaises(PublicSystemException,
                          controller.set_configs,
                          account_config)

        # test with correct values
        controller.set(self.test_config.name, '12')
        self.assertEqual(controller.get(self.test_config.name), 12)

        # test set to default
        controller.set(self.test_config.name, self.test_config.default)
        # get will return the value converted to the config type, so have to assert with that
        val = int(self.test_config.default)
        self.assertEqual(controller.get_configs([self.test_config.name]), {self.test_config.name: val})

        # now lets look in the db and see if the record in AccountConfig was removed
        from basesoft.web.contrib.account.models import AccountConfig
        from sqlalchemy import and_
        q = self.session.query(AccountConfig).filter(and_(AccountConfig.account_id==self.account.id,
                                                          AccountConfig.config_id==self.test_config.id))
        self.assertEqual(q.count(), 0)

    def test_other_account_configs(self):
        """
        Tests the get/set methods for when controller is used for another account.
        """
        # create a new account that will set/get configs for self.account
        scope_account = self._create_active_account()
        controller = AccountConfigController(sa_session=self.session, account_id=self.account,
                                             scope_account_id=scope_account)

        # the scope account has no perm to config other account
        account_config = {self.test_config.name: '10'}
        self.assertRaises(PermissionDeniedException,
                          controller.set_configs,
                          account_config)

        # now set the correct permission
        self._attach_permissions_to_group(scope_account.groups[0].group, 'account_config_manager')
        controller.set(self.test_config.name, '10')
        self.assertEqual(controller.get(self.test_config.name), 10)
