# -*- coding: utf-8 -*-

# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.


import datetime
from basesoft.web.test.models import MockPrivilegeObject
from django.conf import settings

from hashlib import sha256 as sha
from basesoft.web.test.unit import iqportal_unittest

from basesoft.web.core.exceptions import PublicUserException, PermissionDeniedException, PublicSecurityException
from basesoft.web.contrib.i18n import models as i18n_models

from basesoft.web.contrib.account.schedule import AccountActivationMailTask, LostPasswordActivationMailTask
from basesoft.web.contrib.account.controllers import AccountController
from basesoft.web.contrib.account.mailing import MailController
from basesoft.web.contrib.account.models import AccountActivation, AccountActivationMail, LostPasswordMail, Account
from basesoft.web.test.unit.test_abstract_controller import MockRequest
from basesoft.web.contrib.branding.models import Site, Theme
from basesoft.web.contrib.license.models import Domain, Role, Permission,\
    RoleConfig, License
from basesoft.web.core.util.controllers import LicenseConfigController, StateManager
from basesoft.web.test.unit.test_dashboard_controller import get_state
from basesoft.web.contrib.dashboard.controllers import DashboardStateController
from basesoft.web.contrib.contact.controllers import SimpleContactController
from basesoft.web.contrib.i18n.models import Locale

class TestCase(iqportal_unittest.IQPortalTestCase):
    """
    Test the account controller.
    """
    def setUp(self):
        """
        The script for reseting the db and creating all the account tables
        """
        self.connect_db()

        self.account = self._create_active_account()
        self.group = self.account.groups[0].group

        self.locale = self.session.query(i18n_models.Locale).filter_by(locale='en_GB').one()

        self.setupMails()

        self.controller = AccountController(self.session, account_id=self.account.id)

    def setupMails(self):
        """
        Configures activation and lost-password mails.
        """

        # There can only be one currently in the db so first delete any entries that already exist
        for mail in self.session.query(AccountActivationMail).all():
            self.session.delete(mail)
        for mail in self.session.query(LostPasswordMail).all():
            self.session.delete(mail)

        # flush to commit to db
        self.session.flush()

        # mail controller
        self._attach_permissions_to_group(self.group, 'create_mail', 'create_account_mail')
        controller = MailController(self.session, self.locale, account_id=self.account.id)

        # account activation mail
        mail_state_activation = self._get_activation_mail_state()

        # lost pass mail
        mail_state_lost_pass = {
            "mail": {
                "mail_from_text": "MockMailer",
                "name": u"AccountActivationMail",
                "objectName": u"LostPasswordMail",
                "dbID": None,

                "locales": {
                    'nl_NL': {
                        'subject': u"IQPortal Account Activatie",
                        'content': u'Geachte <img  src="/basesoft.web.core/rte/make_image/Account%3AAccount%20Name" title="account.name" class="replacement-field ">, <br /><br /> \
                                  Klikkie op de volgende link svp voor activatie: <br /><br /> \
                                  <img  src="/basesoft.web.core/rte/make_image/Account%3ALink%20to%20Activation" title="account.activation.link" class="replacement-field ">'
                    },
                    'en_GB': {
                        'subject': u"test_replacement_fields",
                        'content': u'Dear <img  src="/basesoft.web.core/rte/make_image/Account%3AAccount%20Name" title="account.name" class="replacement-field ">, <br /><br /> \
                                  Click on the following link for activation: <br /><br />\
                                  <img  src="/basesoft.web.core/rte/make_image/Account%3ALink%20to%20Activation" title="account.activation.link" class="replacement-field ">'
                    },
                },

                "fallbackLocale": {'value': u'nl_NL'},
            }
        }

        owner = self.sa_session.query(License).filter(License.name==settings.SITE_OWNER_LICENSE_NAME).one()

        mail = controller.save_state(mail_state_activation, self.group)
        # these should be site wide mails
        controller.grant_privilege_to(mail, AccountActivationMail.PRIV_USE_FOR_SENDING, group=True, target_group=owner.default_group, check_own_privs=False, world=True)
        mail = controller.save_state(mail_state_lost_pass, self.group)
        # these should be site wide mails
        controller.grant_privilege_to(mail, LostPasswordMail.PRIV_USE_FOR_SENDING, group=True, target_group=owner.default_group, check_own_privs=False, world=True)

    def _get_activation_mail_state(self):
        return {
            "mail": {
                "mail_from_text": "MockMailer",
                "name": u"AccountActivationMail",
                "objectName": u"AccountActivationMail",
                "dbID": None,

                "locales": {
                    'nl_NL': {
                        'subject': u"IQPortal Account Activatie",
                        'content': u'Geachte <img  src="/basesoft.web.core/rte/make_image/Account%3AAccount%20Name" title="account.name" class="replacement-field ">, <br /><br /> \
                                  Klikkie op de volgende link svp voor activatie: <br /><br /> \
                                  <img  src="/basesoft.web.core/rte/make_image/Account%3ALink%20to%20Activation" title="account.activation.link" class="replacement-field ">'
                    },
                    'en_GB': {
                        'subject': u"test_replacement_fields",
                        'content': u'Dear <img  src="/basesoft.web.core/rte/make_image/Account%3AAccount%20Name" title="account.name" class="replacement-field ">, <br /><br /> \
                                  Click on the following link for activation: <br /><br />\
                                  <img  src="/basesoft.web.core/rte/make_image/Account%3ALink%20to%20Activation" title="account.activation.link" class="replacement-field ">'
                    },
                },

                "fallbackLocale": {'value': u'nl_NL'},
            }
        }

    def _save_account_activation_mail(self, account):

        controller = MailController(self.session, self.locale, account_id=account.id)
        mail = controller.save_state(self._get_activation_mail_state())

        return mail

    def test_create_random_account_from_person(self):
        """
        This is used as an option when creating new licenses. The last name
        of the Person object is used to create a random user name.
        """
        from basesoft.web.contrib.contact.models import Person
        p = Person()
        p.last_name = 'Mi'
        name = self.controller.generate_account_name_from_person(p)

        self.assertEqual(name[:2], p.last_name)
        self.assertEqual(len(name), 10)

        p = Person()
        p.last_name = 'TooLongTooJustAppend'
        # if the last name is longer than 6 characters, it is stripped.
        name = self.controller.generate_account_name_from_person(p)

        self.assertNotEqual(name[:7],p.last_name[:7])
        self.assertEqual(name[:6],p.last_name[:6])
        self.assertEqual(len(name), 10)

    def test_create_account_from_license_contact(self):
        """
        This creates a license and tests to see if a semi-random account name
        is added based on the contacts' last name.
        """

        self._attach_permissions_to_group(self.group, 'create_account')

        license = self._create_license('VerifyLastNameInAccount')
        from basesoft.web.contrib.license.models import Group
        g = Group()
        g.name = 'admins'
        g.license = license
        g.is_default = True
        self.session.add(g)

        account = self.controller.create_account_from_license_contact(license)
        self.assertEqual(account.name[:6],'Verify')
        self.assertEqual(len(account.name), 10)

        # there should be no account activation entry for this account
        aa = self.session.query(AccountActivation).filter(AccountActivation.account_id==account.id).all()
        self.assertEqual(len(aa), 0)

    def test_create_inactive_account(self):
        """
        This is a successful testing of the creation of an inactive account.
        """

        name = self._uniquestring('testaccountname')[0:39]
        password = 'testaccountpassword'
        email = '%s@mail-test.basesoft.com' % self._uniquestring('random')

        account = self.controller.create_inactive_account(name, password, email)
        self.assertEqual(account.name, name)
        self.assertEqual(account.password, sha(password).hexdigest())
        self.assertEqual(account.person.email, email)

        # now make sure we also have an AccountActivation entry.
        aa = self.controller.get_account_activation(account)
        self.assertEqual(aa.account, account)
        self.assertEqual(aa.activation_type, AccountActivation.ACTIVATION_TYPE_NEW_ACCOUNT)

    def test_create_inactive_account_dupe(self):
        """
        """

        name = self._uniquestring('testaccountname')[0:39]
        password = 'testaccountpassword'
        email = '%s@mail-test.basesoft.com' % self._uniquestring('random')

        self.controller.create_inactive_account(name, password, email)

        self.assertRaises(PublicUserException,
                          self.controller.create_inactive_account,
                          name=self._uniquestring('testothername')[0:39],
                          password='other_password',
                          email=email)

        # now test for duplicate account name
        self.assertRaises(PublicUserException,
                          self.controller.create_inactive_account,
                          name=name,
                          password='other_password',
                          email='%s@mail-test.basesoft.com' % self._uniquestring('random'))

    def test_activate_account(self):
        """
        This tests a successful activation of an account.
        """

        # create inactive account and verify the account is not active for the default group yet.
        account = self.controller.create_inactive_account(self._uniquestring('test'), 'test',
            '%s@mail-test.basesoft.com' % self._uniquestring('random'))
        self.assertEqual(account.groups[0].active, False)

        # run the AccountActivationMailTask, this sends out actiation mail to the user
        AccountActivationMailTask().execute()
        self.session.expire(account)

        # get activation and verify
        aa = self.controller.get_account_activation(account)
        self.assertEqual(aa.activation_stage, AccountActivation.ACTIVATION_STAGE_PENDING)

        self.assertTrue(self.controller.can_active(aa.activation_key))

        # activate account like the user would click on a link from the email
        account = self.controller.activate_account(aa.activation_key)

        # verify the account has been activated for the default group.
        self.assertEqual(account.groups[0].active, True)

        # verify there is no longer an AccountActivation entry for this account.
        self.assertEqual(None, self.controller.get_account_activation(
            account=account,
            type=AccountActivation.ACTIVATION_TYPE_NEW_ACCOUNT))

    def test_password_with_unicode_character(self):
        """
        This test unicode possibilities within passwords. #3399
        """
        acname = self._uniquestring('test')
        account = self.controller.create_inactive_account(acname, u'test要zodan',
            '%s@mail-test.basesoft.com' % self._uniquestring('random'))
        AccountActivationMailTask().execute()
        self.session.expire(account)

        aa = self.controller.get_account_activation(account)
        account = self.controller.activate_account(aa.activation_key)

        a = self.controller.validate_account(acname, u'test要zodan')


    def test_activate_account_with_person_requirement(self):
        """
        This tests the domain setting for account person details requirement: #3792
        """
        account = self.controller.create_inactive_account(self._uniquestring('test'), 'test',
            '%s@mail-test.basesoft.com' % self._uniquestring('random'))
        account.default_group.license.domain.settings |= Domain.SETTING_REQUIRE_PERSON_DETAILS_ON_ACCOUNT_ACTIVATION
        self.session.add(account)
        self.session.flush()

        AccountActivationMailTask().execute()
        self.session.expire(account)

        aa = self.controller.get_account_activation(account)

        # the person details are not yet added for this account
        self.assertRaises(PublicUserException, self.controller.activate_account, aa.activation_key)

        # provide some person details.
        account.person.last_name = 'Nguyen'
        account.person.first_name = 'Li'
        self.session.add(account.person)
        self.session.flush()

        # assert that we can activate the account now.
        self.assertTrue(self.controller.can_active(aa.activation_key))

    def test_create_account_with_default_dashboard(self):
        """
        Tests if a new account from a Site gets the correct default dashboard set. #3798
        """
        self._attach_permissions_to_group(self.group, 'save_dashboard', 'view_dashboard')
        manager = StateManager(None, self.account.id, None)
        dc = DashboardStateController(self.sa_session, manager)

        # save a dashboard
        state = get_state()
        _map = dc.save_state(state)
        dashboard = dc.get_dashboards(_map[state['dashboard']['domID']]).pop()

        name = self._uniquestring('testaccountname')[0:39]
        password = 'testaccountpassword'
        email = '%s@mail-test.basesoft.com' % self._uniquestring('random')

        # request with random hostname
        request = MockRequest()
        request.META['HTTP_HOST'] = self._uniquestring('host')

        # site links to domain
        site = Site()
        site.order = 1
        site.theme = self.sa_session.query(Theme).all()[0]
        site.host_match = request.META['HTTP_HOST']
        site.domain = self.group.license.domain
        site.home_app = 'website'
        site.trac_conn_str = 'http://'
        self.sa_session.add(site)

        role = Role()
        role.name = self._uniquestring('r')
        role.permissions = [
            self.sa_session.query(Permission).filter(Permission.name == 'view_dashboard').one()
        ]
        role.configs = []
        role.required_permission = 'manage_roles'
        role.dashboard = dashboard
        self.sa_session.add(role)
        site.domain.registrant_role = role

        self.sa_session.flush()

        # set request context on controller so it can retrieve the domain
        self.controller.request = request
        # create inactive accouint
        account = self.controller.create_inactive_account(name, password, email)

        # activate the account
        AccountActivationMailTask().execute()
        self.session.expire(account)
        aa = self.controller.get_account_activation(account)
        account = self.controller.activate_account(aa.activation_key)

        # the account should get the domain based on the site
        manager = StateManager(None, account.id, None)
        dc = DashboardStateController(self.sa_session, manager)
        self.assertEqual(len(dc.get_dashboards()), 1)


    def test_create_account_with_domain_and_role(self):
        """
        tests if a new account from a Site gets the proper Domain and Role set
        """

        name = self._uniquestring('testaccountname')[0:39]
        password = 'testaccountpassword'
        email = '%s@mail-test.basesoft.com' % self._uniquestring('random')

        # request with random hostname
        request = MockRequest()
        request.META['HTTP_HOST'] = self._uniquestring('host')

        # domain for the site
        domain = Domain()
        domain.name = self._uniquestring('d')

        # site links to domain
        site = Site()
        site.order = 1
        site.theme = self.sa_session.query(Theme).all()[0]
        site.host_match = request.META['HTTP_HOST']
        site.domain = domain
        site.home_app = 'website'
        site.trac_conn_str = 'http://'

        # role for the domain
        role = Role()
        role.name = self._uniquestring('r')
        role.required_permission = 'manage_roles'
        role.permissions = [
            self.sa_session.query(Permission).filter(Permission.name == 'view_dashboard').one()
        ]
        rlc = RoleConfig()
        rlc.set_config('basesoft.account.post_login.js_modules', 'iq_test', self.sa_session)
        role.configs = [
            rlc
        ]

        domain.registrant_role = role

        # license configs for role
        self.sa_session.add(domain)
        self.sa_session.add(role)
        self.sa_session.add(site)
        self.sa_session.flush()

        # set request context on controller so it can retrieve the domain
        self.controller.request = request

        # create inactive accouint
        account = self.controller.create_inactive_account(name, password, email)

        # the account should get the domain based on the site
        self.assertEqual(account.groups[0].group.license.domain.name, domain.name)

        # activate user
        AccountActivationMailTask().execute()
        self.session.expire(account)
        aa = self.controller.get_account_activation(account)
        account = self.controller.activate_account(aa.activation_key)

        # the account group and license should now have the permissions set
        self.assertEqual(len(account.groups[0].group.license.permissions), 1)
        self.assertEqual(account.groups[0].group.license.permissions[0].name, 'view_dashboard')
        self.assertEqual(len(account.groups[0].group.permissions), 1)
        self.assertEqual(account.groups[0].group.permissions[0].name, 'view_dashboard')

        # the license of the account should have the licenseconfig set
        lc = LicenseConfigController(self.sa_session, scope = account.groups[0].group.license)
        self.assertEqual(lc.get('basesoft.account.post_login.js_modules'), 'iq_test')


    def test_activate_account_failures(self):
        """
        This tests failures in activation of an account.
        """

        self.assertRaises(PublicUserException, self.controller.activate_account, 'bogus_key')

        account = self.controller.create_inactive_account(self._uniquestring('test'), 'test',
            '%s@mail-test.basesoft.com' % self._uniquestring('random'))

        aa = self.controller.get_account_activation(account)
        expiration_date = datetime.timedelta(days=(settings.ACTIVATION_KEY_VALID_DAYS+1))
        aa.created = datetime.datetime.now() - expiration_date
        self.session.add(aa)
        self.session.flush()

        self.assertRaises(PublicUserException, self.controller.activate_account, aa.activation_key)

    def test_activate_account_url(self):

        account1 = self.controller.create_inactive_account(self._uniquestring('test'), 'test',
            '%s@mail-test.basesoft.com' % self._uniquestring('random'))

        account2 = self.controller.create_inactive_account(self._uniquestring('test'), 'test',
            '%s@mail-test.basesoft.com' % self._uniquestring('random'))

        license = account2.groups[0].group.license
        license.http_host = 'kwek.bleh'
        self.session.add(license)
        self.session.flush()

        # run the AccountActivationMailTask, this sends out actiation mail to the user
        AccountActivationMailTask().execute()
        self.session.expire(account1)
        self.session.expire(account2)

    def test_get_accounts_with_privilege_on_instance(self):
        account = self._create_active_account()

        c = AccountController(self.sa_session, account_id=account.id)

        mpo = MockPrivilegeObject()
        mpo.name = 'm to the p to the o'
        c.save_default_privilege_object_group(mpo)

        accounts = c.get_accounts_with_privilege_on_instance(mpo, MockPrivilegeObject.PRIV_VIEW)
        self.assertEqual(len(accounts), 1)
        self.assertTrue(account in accounts)

        account2 = self._create_active_account(license=account.default_group.license, group=account.default_group)
        accounts = c.get_accounts_with_privilege_on_instance(mpo, MockPrivilegeObject.PRIV_VIEW)
        self.assertEqual(len(accounts), 2)
        self.assertTrue(account2 in accounts)

        # deactivate account 1
        self.controller.save_or_update_group_account(account.default_group, account, is_active=False)
        accounts = c.get_accounts_with_privilege_on_instance(mpo, MockPrivilegeObject.PRIV_VIEW)
        self.assertEqual(len(accounts), 1)
        self.assertFalse(account in accounts)

    def test_save_account_from_state(self):
        """
        """

        rnd_acc = self._create_active_account()
        state = {
                'person': {
                    'first_name': u'Account FN',
                    'gender': u'male',
                    'last_name': u'Account LN',
                    'email': '%s@mail-test.basesoft.com' % self._uniquestring('random'),
                    'address_data': {
                        'city': u'My City',
                        'state': u'My State',
                        'street': u'232 near the high school',
                        'postcode': u'90210',
                        'country': 204} # Spain
                },
                'account': {
                    'groups': [rnd_acc.groups[0].group.id],
                    'default_group': rnd_acc.groups[0].group.id,
                    'account_name': self._uniquestring('myaccountyo')[0:39]
                  }}

        self._attach_permissions_to_group(self.group, 'view_all_domains')
        self.assertRaises(PermissionDeniedException, self.controller.save_account_from_state, state)

        self._attach_permissions_to_group(self.group, 'create_account', 'update_account')

        account = self.controller.save_account_from_state(state)
        self.assertEqual(account.name, state['account']['account_name'])

        groups = self.controller.get_groups_for_account(account, rnd_acc.groups[0].group.id)
        self.assertEqual(len(groups),1)
        self.assertEqual(groups[0].group, rnd_acc.groups[0].group)

        # update the account state information.
        state['person']['first_name'] = u'Barry'
        state['person']['address_data']['street'] = u'around the corner'
        updated = self.controller.save_account_from_state(state, account.id)

        self.assertEqual(updated.id, account.id)
        self.assertEqual(updated.person.id, account.person.id)
        self.assertEqual(updated.person.address.id, account.person.address.id)
        self.assertEqual(updated.person.first_name, u'Barry')
        self.assertEqual(updated.person.address.street, u'around the corner')

    def test_validate_account(self):
        """
        This tests a success and a failure of the validate_account method of
        the controller.
        """
        self.assertFalse(self.controller.validate_account('fail', 'account'))
        name = self._uniquestring('success')
        account = self.controller.create_inactive_account(name, 'account', '%s@mail-test.basesoft.com' % self._uniquestring('random'))

        AccountActivationMailTask().execute()
        self.session.expire(account)

        aa = self.controller.get_account_activation(account)
        account = self.controller.activate_account(aa.activation_key)
        self.assertEqual(self.controller.validate_account(name, 'account'),
                         account)

    def test_update_last_login(self):
        """
        This tests the update_last_login method on the controller.
        """
        import time
        account = self.controller.create_inactive_account(self._uniquestring('test'), 'test',
                                                          '%s@mail-test.basesoft.com' % self._uniquestring('random'))
        current_login = account.last_login
        time.sleep(1)
        self.controller.update_last_login(account)
        self.assertNotEqual(current_login, account.last_login)

    def test_forgot_credentials(self):
        """
        This tests a failure and a successful call to the forgot_credentials
        method on the controller.
        """
        self.assertRaises(PublicUserException,
                          self.controller.forgot_credentials,
                          account_name='failure')

        email = '%s@mail-test.basesoft.com' % self._uniquestring('random')

        account = self.controller.create_inactive_account(self._uniquestring('test'), 'test', email)

        AccountActivationMailTask().execute()
        self.session.expire(account)

        aa = self.controller.get_account_activation(account)
        account = self.controller.activate_account(aa.activation_key)
        self.assertEqual(
            self.controller.forgot_credentials(email=email),
            account)

    def test_activate_change_password(self):
        """
        Test the activation of a changed password with the provided key.
        """

        # with a bogus key.
        self.assertRaises(PublicUserException,
                          self.controller.activate_change_password,
                          'boguskey',
                          'hocususer',
                          'newpasswd')
        email = '%s@mail-test.basesoft.com' % self._uniquestring('random')
        user = self._uniquestring('test')

        account = self.controller.create_inactive_account(user, 'test', email)

        AccountActivationMailTask().execute()

        self.session.expire(account)

        aa = self.controller.get_account_activation(account)
        account = self.controller.activate_account(aa.activation_key)
        self.controller.forgot_credentials(email=email)
        aa = self.controller.get_account_activation(account,
                 type=AccountActivation.ACTIVATION_TYPE_CHANGE_PASSWORD)

        LostPasswordActivationMailTask().execute()

        # with a correct key for the wrong user.
        self.assertRaises(PublicUserException,
                          self.controller.activate_change_password,
                          aa.activation_key,
                          'hocususer',
                          'newpasswd')

        # and a successful update of the password
        account = self.controller.activate_change_password(aa.activation_key, user, u'newpasswd كشميري')
        self.assertEqual(account.password, sha(u'newpasswd كشميري'.encode('utf-8')).hexdigest())

        # verify there is no longer an AccountActivation entry for this account.
        self.assertEqual(None, self.controller.get_account_activation(
            account=account,
            type=AccountActivation.ACTIVATION_TYPE_CHANGE_PASSWORD))

    def test_account_login_for_deactivated_license(self):
        """
        Try to login with an account which has no active license.
        """
        name = self._uniquestring('success')
        account = self.controller.create_inactive_account(name, 'account', '%s@mail-test.basesoft.com' % self._uniquestring('random'))

        AccountActivationMailTask().execute()
        self.session.expire(account)

        aa = self.controller.get_account_activation(account)
        account = self.controller.activate_account(aa.activation_key)
        self.assertEqual(self.controller.validate_account(name, 'account'),
                         account)

        # now we deactivate the license.
        l = account.groups[0].group.license
        l.is_active = False
        self.sa_session.add(l)
        self.sa_session.flush()

        self.assertRaises(PublicUserException, self.controller.validate_account, name, 'account')

    def test_account_activation_mailing_order(self):
        """
        This creates multiple account activation mails on different privilege and license levels to test if the correct ones are picked for sending
        """
        for mail in self.session.query(AccountActivationMail).all():
            self.session.delete(mail)
        for mail in self.session.query(LostPasswordMail).all():
            self.session.delete(mail)

        # create a new mail with license level send privs
        license_mail = self._save_account_activation_mail(self.account)
        self.controller.grant_privilege_to(license_mail, AccountActivationMail.PRIV_USE_FOR_SENDING, group=True, license=True, check_own_privs=False, target_group=self.group)

        # create a new mail with domain level send privs
        domain_mail = self._save_account_activation_mail(self.account)
        self.controller.grant_privilege_to(domain_mail, AccountActivationMail.PRIV_USE_FOR_SENDING, group=True, domain=True, check_own_privs=False, target_group=self.group)

        # create a new mail on reseller with world send privs
        reseller_mail = self._save_account_activation_mail(self.account)
        self.controller.grant_privilege_to(reseller_mail, AccountActivationMail.PRIV_USE_FOR_SENDING, group=True, world=True, check_own_privs=False, target_group=self.group)

        # create a new mail on basesoft with world send privs
        owner_license = self.sa_session.query(License).filter(License.name==settings.SITE_OWNER_LICENSE_NAME).one()
        owner_account = self._create_active_account(owner_license)
        self._attach_permissions_to_group(owner_account.default_group, 'create_mail', 'create_account_mail')
        owner_mail = self._save_account_activation_mail(owner_account)
        self.controller.grant_privilege_to(owner_mail, AccountActivationMail.PRIV_USE_FOR_SENDING, group=True, world=True, check_own_privs=False, target_group=owner_account.default_group)

        l = self.account.default_group.license
        # create a new account on same license
        my_account = self._create_active_account(l)
        ac = AccountController(self.sa_session, account_id=my_account.id)
        # check get_mail returns license level mail.
        mail = ac.get_account_activation_mail(my_account.default_group)
        self.assertEqual(mail, license_mail)

        # create a new account on different license same domain
        license = self._create_license(domain=l.domain)
        my_account = self._create_active_account(license)
        ac = AccountController(self.sa_session, account_id=my_account.id)
        # check get_mail return domain level mail.
        mail = ac.get_account_activation_mail(my_account.default_group)
        self.assertEqual(mail, domain_mail)

        # create a new account with own domain/license for reseller
        my_account = self._create_active_account()
        my_account.default_group.license.domain.reseller = l
        self.sa_session.add(my_account.default_group.license.domain)
        self.sa_session.flush()
        # check get_mail returns reseller mail.
        mail = ac.get_account_activation_mail(my_account.default_group)
        self.assertEqual(mail, reseller_mail)

        # create a new account unrelated to any of the created domains/resellers
        my_account = self._create_active_account()
        # check get_mail returns basesoft mail.
        mail = ac.get_account_activation_mail(my_account.default_group)
        self.assertEqual(mail, owner_mail)

    def test_account_preferred_locale_bubble(self):
        """
        This tests ticket #3834
        """
        # create an account with person.
        account = self._create_active_account()

        # retrieve the current account locale (empty)
        scc = SimpleContactController(self.sa_session, account_id=account.id)
        current_locale = scc.config.get('basesoft.web.contrib.locale')

        locale = self.sa_session.query(Locale).filter(Locale.locale=='en_GB').one()

        state = {
            'preferred_locale': locale.id
        }

        # update the person preferred locale
        person = scc._save_or_update_person(account.person.id, state)
        self.assertEqual(person.preferred_locale, locale)

        # verify account config for preferred locale has been altered.
        new_locale = scc.config.get('basesoft.web.contrib.locale')
        self.assertEqual(new_locale, locale.locale)
        self.assertNotEqual(current_locale, new_locale)

    def test_account_reset_activation(self):
        """
        This tests ticket #4077
        """

        # create inactive account
        account = self.controller.create_inactive_account(self._uniquestring('test'), 'test',
            '%s@mail-test.basesoft.com' % self._uniquestring('random'))
        self.assertEqual(account.groups[0].active, False)

        # reset the activation
        self.controller.reset_account_activation(account)

        # status should be pending
        aa = self.controller.get_account_activation(account)
        self.assertEqual(aa.activation_stage, AccountActivation.ACTIVATION_STAGE_NEW)
        self.session.expire(aa)

        # run task, should send mail
        AccountActivationMailTask().execute()
        self.session.expire(account)

        # get activation and verify
        aa = self.controller.get_account_activation(account)
        self.assertEqual(aa.activation_stage, AccountActivation.ACTIVATION_STAGE_PENDING)
        self.assertTrue(self.controller.can_active(aa.activation_key))

        # activate account like the user would click on a link from the email
        account = self.controller.activate_account(aa.activation_key)

        # verify the account has been activated for the default group.
        self.assertEqual(account.groups[0].active, True)

        # verify there is no longer an AccountActivation entry for this account.
        self.assertEqual(None, self.controller.get_account_activation(account=account))

    def test_account_reset_activation_invalid(self):

        # setup an account, this will make company data too
        account = self._create_active_account()

        # assert that its not possible to reset this account activation
        self.assertRaises(PublicUserException, self.controller.reset_account_activation, account)

    def test_archive_account(self):
        self._attach_permissions_to_group(self.group, 'view_license', 'view_all_domains')
        account = self._create_active_account()

        self.assertRaises(PermissionDeniedException, self.controller.archive_account, account.id)
        self._attach_permissions_to_group(self.group, 'delete_account')
        self.controller.archive_account(account.id)

        self.assertRaises(PublicUserException, self.controller.validate_account, account.name, 'test_password')

    def test_remove_own_account_failure(self):
        self.assertRaises(PermissionDeniedException, self.controller.remove_account, self.account.id)
        self._attach_permissions_to_group(self.group, 'delete_account')
        self.assertRaises(PublicSecurityException, self.controller.remove_account, self.account.id)

    def test_remove_account(self):
        self._attach_permissions_to_group(self.group, 'delete_account')

        account = self._create_active_account()

        self.assertRaises(PermissionDeniedException, self.controller.remove_account, account.id)
        self._attach_permissions_to_group(self.group, 'view_license')
        # we cannot delete the account as we do not have access to that newly created account
        self.assertRaises(PublicSecurityException, self.controller.remove_account, account.id)

        self._attach_permissions_to_group(self.group, 'view_all_domains')

        _id = account.id

        self.controller.remove_account(account.id)

        ac = self.sa_session.query(Account).filter(Account.id==_id).first()
        self.assertEqual(ac, None)

    def test_change_password(self):
        new_account = self._create_active_account()
        controller = AccountController(self.sa_session, account_id=new_account.id)

        self.assertRaises(PublicUserException, controller.change_password, 'test_password', 'some', 'no match')
        self.assertRaises(PublicSecurityException, controller.change_password, 'test_password1', 'match', 'match')

        controller.change_password('test_password', 'newpassword', 'newpassword')

        account = self.controller.validate_account(new_account.name, 'newpassword')
        self.assertEqual(account, new_account)

    def test_change_password_for_target_account(self):
        new_account = self._create_active_account()
        group = new_account.groups[0].group

        other_account = self._create_active_account(license=group.license)

        controller = AccountController(self.sa_session, account_id=new_account.id)
        self.assertRaises(PermissionDeniedException, controller.force_change_password, other_account.id, 'match', 'match')
        self._attach_permissions_to_group(group, 'manage_user_passwords', 'view_license')

        controller.force_change_password(other_account.id, 'match', 'match')

        account = self.controller.validate_account(other_account.name, 'match')
        self.assertEqual(account, other_account)

    def test_switch_account(self):
        new_account = self._create_active_account()
        group = new_account.groups[0].group

        other_account = self._create_active_account(license=group.license)
        third_account = self._create_active_account(domain=group.license.domain)

        ac = AccountController(self.sa_session, account_id=new_account.id)
        self.assertRaises(PermissionDeniedException, ac.login_as, None, new_account.name, other_account.name)

        self._attach_permissions_to_group(group, 'switch_account')
        self.assertRaises(PermissionDeniedException, ac.login_as, None, new_account.name, other_account.name)

        self._attach_permissions_to_group(group, 'view_license')
        self.assertEqual(other_account, ac.login_as(None, new_account.name, other_account.name))

        # the third account is part of the same domain, but a different license, so no access to switch.
        self.assertFalse(ac.login_as(None, new_account.name, third_account.name))


# below is for running the test from eclipse
if __name__ == '__main__':
    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_custom_query.py', 'basesoft.web.test.unit.test_account_controller.AccountControllerTestCase.test_account_reset_activation_ivalid']
    run()
    sys.exit()
