# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.test.unit.iqportal_unittest import IQPortalTestCase
from basesoft.web.contrib.announce.controllers import AnnounceController
import datetime
from basesoft.web.contrib.i18n.models import Locale
from basesoft.web.contrib.announce.models import Announcement
from basesoft.util.virtual_date import today, now, set_date
from dateutil.relativedelta import relativedelta


class TestCase(IQPortalTestCase):

    def setUp(self):
        set_date(datetime.datetime.now())
        self.connect_db()

    def test_edit_wiki(self):

        account = self._create_active_account()
        group = account.groups[0].group
        locale_en = self.session.query(Locale).filter_by(locale='en_GB').one()
        locale_nl = self.session.query(Locale).filter_by(locale='nl_NL').one()
        locale_de = self.session.query(Locale).filter_by(locale='de_DE').one()

        self._attach_permissions_to_group(group, 'manage_announcements')

        # save an announcement, does not yet have wiki text (but does create a wiki_id)
        ac = AnnounceController(self.session, account_id=account.id)
        announcement = ac.save_announcement(None, {
            'announce': {
                'publish_date': datetime.datetime.now(),
                'expire_date': None,
                'status': Announcement.STATUS_PUBLISHED,
                'label': 'test announcement',
                'must_read': True,
                'recipients': []
            },
            'wiki': {
                'fallback_locale_id': locale_en.id,
                'locale_id': locale_en.id,
                'title': 'title english',
                'content': 'content english'
            }

        })

        # get current english revision
        en_r1 = ac._get_current_wiki_content(announcement.wiki, locale_en)

        # save same english text, revision shouldnt change
        en_r2 = ac.save_wiki_revision(announcement.id, {
            'locale_id': locale_en.id,
            'title': 'title english',
            'content': 'content english'
        })
        self.assertEqual(en_r1.revision, en_r2.revision)

        # save updated english text, revision should change
        en_r2 = ac.save_wiki_revision(announcement.id, {
            'locale_id': locale_en.id,
            'title': 'title english',
            'content': 'content english updated'
        })
        self.assertNotEqual(en_r1.revision, en_r2.revision)

        # save dutch text
        nl_r1 = ac.save_wiki_revision(announcement.id, {
            'locale_id': locale_nl.id,
            'title': 'title dutch',
            'content': 'content dutch'
        })

        # get german revision, should be None
        de_r1 = ac._get_current_wiki_content(announcement.wiki, locale_de)
        self.assertEqual(de_r1, None)

        # TODO: revert to revision

    def _create_announcement(self, account, publish_date, expire_date=None, status=Announcement.STATUS_PUBLISHED, recipients=[]):

        # create announcement
        locale_en = self.session.query(Locale).filter_by(locale='en_GB').one()
        ac = AnnounceController(self.session, account_id=account.id)
        announcement = ac.save_announcement(None, {
            'announce': {
                'publish_date': publish_date,
                'expire_date': expire_date,
                'status': status,
                'label': 'test announcement',
                'must_read': False,
                'recipients': recipients,
            },
            'wiki': {
                'fallback_locale_id': locale_en.id,
                'locale_id': locale_en.id,
                'title': 'title english',
                'content': 'content english'
            }
        })

        return announcement

    def test_recipients(self):

        locale_en = self.session.query(Locale).filter_by(locale='en_GB').one()
        locale_nl = self.session.query(Locale).filter_by(locale='nl_NL').one()

        # creator
        account = self._create_active_account()
        group = account.groups[0].group
        self._attach_permissions_to_group(group, 'view_dashboard', 'manage_announcements', 'view_domain', 'view_license')

        # recipient
        recipient_license = self._create_license('new license', domain=account.default_group.license.domain)
        account_recipient = self._create_active_account(license=recipient_license)
        self._attach_permissions_to_group(account_recipient.groups[0].group, 'view_dashboard')

        # create announcement for today
        self._create_announcement(account, publish_date=now(), recipients=['license_%s' % recipient_license.id])

        # as the recipient, expect 1 unread announcement
        ac = AnnounceController(self.session, account_id=account_recipient.id)
        announcements = ac.query_unread_announcements()
        self.assertEqual(len(announcements), 1)

        # mark as read
        ac.mark_as_read(announcements[0].id)

        # now there should be 0 new announcements
        self.assertEquals(len(ac.query_unread_announcements()), 0)

        # create a new announcement, before the creation of the user, expecting not to see the announcement
        self._create_announcement(account, publish_date=now() - relativedelta(days=1), recipients=['license_%s' % recipient_license.id])
        self.assertEquals(len(AnnounceController(self.session, account_id=account_recipient.id).query_unread_announcements()), 0)

        # create an expired announcement, expecting not to see it
        self._create_announcement(account, publish_date=now(), expire_date=now(), recipients=['license_%s' % recipient_license.id])
        self.assertEquals(len(AnnounceController(self.session, account_id=account_recipient.id).query_unread_announcements()), 0)

        # create pending announcement
        self._create_announcement(account, publish_date=now(), status=Announcement.STATUS_PENDING, recipients=['license_%s' % recipient_license.id])
        self.assertEquals(len(AnnounceController(self.session, account_id=account_recipient.id).query_unread_announcements()), 0)

        # create a new announcement
        announcement = self._create_announcement(account, publish_date=now(), recipients=['license_%s' % recipient_license.id])

        # testing fallback
        self.assertEquals(ac.load_wiki_with_fallback(announcement.id, locale_en)['title'], 'title english')
        self.assertTrue(ac.load_wiki_with_fallback(announcement.id, locale_en)['content'].find('content english') != -1)
        self.assertEquals(ac.load_wiki_with_fallback(announcement.id, locale_nl)['title'], 'title english')
        self.assertTrue(ac.load_wiki_with_fallback(announcement.id, locale_en)['content'].find('content english') != -1)




# below is for running the test from eclipse
if __name__ == '__main__':
    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_compose.py', 'basesoft.web.test.unit.test_announce.TestCase.test_recipients']
    run()
    sys.exit()
