# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.test.unit import iqportal_unittest
from basesoft.util.attribute import AttributeProvider

class Bar(AttributeProvider):
    """Example class that has attributes"""
    def __init__(self, min_limit=2):
        apply(AttributeProvider.__init__,(self,))

        self.min_limit = min_limit

        f = Foo()

        self.setAttributeConfig('a1', {'value': 1, 'validator': self.valid_limit})
        self.setAttributeConfig('a2', {'value': 5, 'validator': f.valid_limit})


    def valid_limit(self, value):
        """Return True if the checked value is more or equal then min_limit"""
        return value >= self.min_limit

class Foo:
    """
    Class used to provide a validator function for Bar attributes in a different context
    """
    def __init__(self):
        self.max_limit = 10
        self.test = 0

    def valid_limit(self, value):
        """Return True if the checked value is less or equal then max_limit"""
        return value <= self.max_limit

    def change_test(self, value):
        """Modify the test variable. We'll use this to see if the change will call the proper method"""
        self.test = self.test + value

class AttributeTestCase(iqportal_unittest.IQPortalTestCase):
    """
    Test if the attributes are used correctly
    """

    def test_change_attribute(self):
        """Change the attributes and see if the validator kicks in"""
        b = Bar(4)
        self.assertRaises(Exception, b.set, 'a1', 3)
        self.assertRaises(Exception, b.set, 'a2', 11)
        self.assertTrue(b.set('a1', 5))

    def test_on_change(self):
        """Test if a method is called once the value is changed"""
        b = Bar()
        f = Foo()
        # create a new attribute and set the onChange method
        b.setAttributeConfig('a3', {'value': 5, 'onChange': f.change_test})
        self.assertEqual(f.test, 0) # the f.test variable should be 0 now
        b.set('a3', 2)
        self.assertEqual(f.test, 2) # the f.test variable should have been incremented with 2
        b.set('a3', 3)
        self.assertEqual(f.test, 5) # the f.test variable should have been incremented again, with 3

    def test_set_get(self):
        """Check if get and set work properly"""
        b = Bar()
        self.assertEqual(b.get('a1'), 1)
        self.assertEqual(b.get('a2'), 5)
        b.set('a1', 3)
        self.assertEqual(b.get('a1'), 3)

    def test_read_only(self):
        """Create a readonly attribute and see if it's really readonly"""
        b = Bar()
        b.setAttributeConfig('a3', {'value': 5, 'readOnly': True})
        self.assertEqual(b.get('a3'), 5)
        self.assertRaises(Exception, b.set, 'a3', 6)

    def test_write_once(self):
        """Create a write once attribute and see if it can only be written once"""
        b = Bar()
        b.setAttributeConfig('a3', {'value': 5, 'writeOnce': True})
        self.assertEqual(b.get('a3'), 5)
        b.set('a3', 6)
        self.assertRaises(Exception, b.set, 'a3', 7)

    def test_reset(self):
        """Change the value of an attribute then reset it"""
        b = Bar()
        self.assertEqual(b.get('a2'), 5)
        b.set('a2', 6)
        self.assertEqual(b.get('a2'), 6)
        b.resetValue('a2')
        self.assertEqual(b.get('a2'), 5)
