# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.core.exceptions import SecurityException,\
    PermissionDeniedException
from basesoft.web.contrib.branding.models import BrandingHeaderFooter, Theme,\
    LicenseTheme
from basesoft.web.contrib.branding.controllers import HeaderFooterController,\
    SiteController

class HeaderFooterTestCase(iqportal_unittest.IQPortalTestCase):
    """
    """
    def setUp(self):
        self.connect_db()

        self.account = self._create_active_account()
        self.group = self.account.groups[0].group

        self.controller = HeaderFooterController(self.session, account_id=self.account.id)

    def get_state(self):
        return {
            "dbID": None,
            "name": u"Some header",
            "content": u"Some content",
        }

    def test_load_state(self):

        self._attach_permissions_to_group(self.group, 'branding_create_header_footer')

        state = self.get_state()

        # first save a new state
        hf = self.controller.save_state(state)

        # test without permission
        self.assertRaises(PermissionDeniedException, self.controller.load_state, hf.id)

        # add permissions
        self._attach_permissions_to_group(self.group, 'branding_view_header_footer')

        # then load it again
        loaded_state = self.controller.load_state(hf.id)

        # update the id in our state
        state['dbID'] = hf.id

        # now state should be the same as the loaded_state
        self.assertEquals(state, loaded_state)

    def test_save_state(self):
        """
        Tests the updating of a header/footer using a state that has the dbID set
        """

        self._attach_permissions_to_group(self.group, 'branding_create_header_footer', 'branding_view_header_footer')

        # first save a new h/f and get the response
        # (we save it only with view privileges so we can test later)
        state = self.get_state()
        state_response = self.controller.save_state(state)

        # load the state. this has the dbID set
        loaded_state = self.controller.load_state(state_response.id)

        # verify that the canvas id is there
        self.assertNotEquals(loaded_state['dbID'], None)

        # now we change the state a bit
        loaded_state['name'] = u'Some updated header'
        loaded_state['content'] = u'Some updated content'

        # call save again, first without permission
        self.assertRaises(PermissionDeniedException, self.controller.save_state, loaded_state)

        # add the proper permission
        self._attach_permissions_to_group(self.group, 'branding_update_header_footer')

        updated_state_response = self.controller.save_state(loaded_state)

        # now query it and verify
        hf = self.session.query(BrandingHeaderFooter).get(updated_state_response.id)
        self.assertEquals(hf.name, u'Some updated header')
        self.assertEquals(hf.content, u'Some updated content')

class SiteTestCase(iqportal_unittest.IQPortalTestCase):

    def setUp(self):
        self.connect_db()

        self.account = self._create_active_account()
        self.group = self.account.groups[0].group

        self.controller = SiteController(self.session, account_id=self.account.id)

    def test_get_site(self):
        pass

    def test_save_or_update_site(self):

        self.assertRaises(PermissionDeniedException, self.controller.save_or_update_site, 'host_name', 1, 1, 1)

        self._attach_permissions_to_group(self.group, 'site_manager')

        host_name = self._uniquestring('host_match')
        self.assertRaises(SecurityException, self.controller.save_or_update_site, host_name, 1, None, 1)

        theme = self.sa_session.query(Theme).filter(Theme.id==1).first()
        lt = LicenseTheme()
        lt.license = self.group.license
        lt.theme = theme
        self.sa_session.add(lt)
        self.sa_session.flush()

        site = self.controller.save_or_update_site(host_name, 1, None, 1)
        self.assertEqual(site.theme, theme)
        self.assertEqual(site.order, 1)
        self.assertEqual(site.host_match, host_name)
        self.assertEqual(site.domain, None)

        self.assertRaises(SecurityException, self.controller.save_or_update_site, host_name, 2, None, 2)

        theme = self.sa_session.query(Theme).filter(Theme.id==2).first()
        lt = LicenseTheme()
        lt.license = self.group.license
        lt.theme = theme
        self.sa_session.add(lt)
        self.sa_session.flush()

        update = self.controller.save_or_update_site(host_name, 2, None, 2)
        self.assertEqual(update.theme, theme)
        self.assertEqual(update.order, 2)
        self.assertEqual(update.domain, None)



