# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.
import os

import datetime
import time
from datetime import date

from django.conf import settings

from basesoft.web.test.unit import iqportal_unittest

from basesoft.web.contrib.contact.models import Person, Contact, ContactList
from basesoft.web.contrib.contact.controllers import ContactController, ImportContactController, SimpleContactController
from basesoft.web.contrib.license.controllers import LicenseController
from basesoft.web.core.exceptions import SecurityException, PublicUserException, PermissionDeniedException
from basesoft.web.contrib.license.models import LicenseConfig
from basesoft.web.contrib.account.models import Config

from basesoft.web.contrib.i18n.models import Locale
from basesoft.web.contrib.contact.schedule import AutomatedImportTask, ImportContactsTask
from basesoft.web.contrib.notification.controllers import NotificationController

class ContactControllerTestCase(iqportal_unittest.IQPortalTestCase):
    """
    This will test the Contact Controller.
    All individual *public* methods should be tested separately.
    """

    def setUp(self):
        """
        The script for reseting the db and creating all the tables
        """
        self.connect_db()

        #create an account so we can check permission with it
        account = self._create_active_account()

        # create an instance of the controller. It is used in all test methods.
        self.controller = ContactController(self.session, account_id = account.id)
        self.lc = LicenseController(self.session)
        self.group = account.groups[0].group

        self.person_data = {'first_name': u'John',
                            'polymorphic_type': 'Person',
                            'middle_name': u'',
                            'last_name': u'Smith',
                            'title': u'mr',
                            'gender': Person.MALE,
                            'email': u'john@mail-test.basesoft.com',
                            'position': u'assistent',
                            'department': u'marketing',
                            'phone': u'0040123456789',
                            'mobile_phone': '',
                            'fax': '',
                            'birth_date': date(1978, 12, 23),
                            'address_data': {'country': '181',
                                             'postcode': u"400295",
                                             'street': u" Bihorului 10",
                                             'city': u"Cluj Napoca",
                                             'state': u"Cluj"
                                            }
                           }

    def test_save_or_update_contact_no_permission(self):
        """
        This tests saving and updating of contact without having the necessary permissions or privileges
        """
        #can't insert
        self.assertRaises(PermissionDeniedException,
                          self.controller.save_or_update_contact,
                          id = None, person_data = self.person_data, company_data = None)

        self._attach_permissions_to_group(self.group, 'create_contact')

        #insert
        contact = self.controller.save_or_update_contact(id = None, person_data = self.person_data, company_data = None)
        #can't update
        self.assertRaises(PermissionDeniedException,
                          self.controller.save_or_update_contact,
                          id = contact.id, person_data = self.person_data, company_data = None)

    def test_save_or_update_contact(self):
        """
        This tests saving and updating of contact
        """

        self._attach_permissions_to_group(self.group, 'create_contact', 'update_contact')

        # set contacts number limit to 1 for this license so we can test the usage options check
        self._set_license_usage(self.group.license, 'max_number_of_contacts', 1)

        # first save is ok
        contact = self.controller.save_or_update_contact(id = None, person_data = self.person_data, company_data = None)

        # second one should raise an exception cuz the maximum limit of total contacts was already reached
        self.assertRaises(PermissionDeniedException, self.controller.save_or_update_contact,
                          id = None, person_data = self.person_data, company_data = None)

        for key in self.person_data:
            if key != "address_data":
                self.assertEqual(getattr(contact.person, key), self.person_data[key])
            else:
                for k in self.person_data["address_data"]:
                    self.assertEqual(getattr(contact.person.address, k), self.person_data["address_data"][k])

        new_person_data = self.person_data
        new_person_data["first_name"] = u"Anna"
        new_person_data["gender"] = Person.FEMALE
        new_person_data["address_data"]["postcode"] = "400000"

        #update
        new_contact = self.controller.save_or_update_contact(id = contact.id, person_data = new_person_data,
                                                              company_data = None)
        for key in new_person_data:
            if key != "address_data":
                self.assertEqual(getattr(new_contact.person, key), new_person_data[key])
            else:
                for k in new_person_data["address_data"]:
                    self.assertEqual(getattr(new_contact.person.address, k), new_person_data["address_data"][k])

    def test_save_or_update_contact_with_preferred_locale(self):

        self._attach_permissions_to_group(self.group, 'create_contact')

        en_locale = self.sa_session.query(Locale).filter(Locale.locale=='en_GB').one()

        pd = self.person_data
        pd['preferred_locale'] = 'en'
        pd['address_data']['country'] = 230

        contact = self.controller.save_or_update_contact(id = None, person_data = pd, company_data = None)
        self.assertEqual(contact.person.preferred_locale, en_locale)

        pd = self.person_data
        pd['preferred_locale'] = en_locale
        pd['address_data']['country'] = 7


        contact = self.controller.save_or_update_contact(id = None, person_data = pd, company_data = None)
        self.assertEqual(contact.person.preferred_locale, en_locale)

        pd = self.person_data
        pd['preferred_locale'] = 'nl'
        pd['address_data']['country'] = 230

        contact = self.controller.save_or_update_contact(id = None, person_data = pd, company_data = None)
        self.assertEqual(contact.person.preferred_locale, None)

        pd = self.person_data
        pd['preferred_locale'] = 'nl'
        del pd['address_data']

        self.assertRaises(PublicUserException, self.controller.save_or_update_contact, id = None, person_data = pd, company_data = None)

    def test_save_or_update_contacts_from_state(self):
        """
        This tests saving and updating of a contact from state (dict with contact details)
        """
        self._attach_permissions_to_group(self.group, 'view_contact', 'create_contact', 'update_contact')

        contact_details = {'first_name': self.person_data['first_name'],
                           'last_name': self.person_data['last_name']}
        #insert
        contact =  self.controller.save_or_update_contacts_from_state(contacts_state = [contact_details])[0]

        self.assertEqual(contact.person.first_name, self.person_data['first_name'])
        self.assertEqual(contact.person.last_name, self.person_data['last_name'])

        new_contact_details = {'id': contact.id,
                               'first_name': u"NewName",
                               'email': u'email@mail-test.basesoft.com',
                               'contact': {'contact_email': u'email2@mail-test.basesoft.com'} #3166
                               }
        #update
        new_contact =  self.controller.save_or_update_contacts_from_state(contacts_state = [new_contact_details])[0]

        self.assertEqual(new_contact.person.first_name, new_contact_details['first_name'])
        self.assertEqual(new_contact.person.last_name, self.person_data['last_name'])
        self.assertEqual(new_contact.person.email, new_contact_details['email'])
        self.assertEqual(new_contact.email, new_contact_details['contact']['contact_email'])

    def test_save_or_update_contact_list_no_permission(self):
        """
        This tests saving and updating of contact list without having the necessary permissions or privileges
        """
        name = u"test"
        #can't insert
        self.assertRaises(PermissionDeniedException,
                          self.controller.save_or_update_contact_list,
                          id = None, name = name)

        self._attach_permissions_to_group(self.group, 'create_contact_list')

        #insert
        cl = self.controller.save_or_update_contact_list(id = None, name = name)
        #can't update
        self.assertRaises(PermissionDeniedException,
                          self.controller.save_or_update_contact_list,
                          id = cl.id, name = name)

    def test_save_or_update_contact_list(self):
        """
        This tests saving and updating of contact list
        """

        self._attach_permissions_to_group(self.group, 'create_contact_list', 'update_contact_list')

        name = u"test contact list"
        #insert
        cl = self.controller.save_or_update_contact_list(id = None, name = name)
        self.assertEqual(cl.name, name)

        new_name = "update contact list"

        #insert
        new_cl = self.controller.save_or_update_contact_list(id = cl.id, name = new_name)
        self.assertEqual(new_cl.name, new_name)

    def test_save_or_update_contact_lists_from_state(self):
        """
        This tests saving and updating of a contact list from state (dict with contact list details)
        """
        self._attach_permissions_to_group(self.group, 'view_contact_list', 'create_contact_list', 'update_contact_list')

        name = u"test contact list"
        contact_list_details = {'name': name}
        #insert
        cl = self.controller.save_or_update_contact_lists_from_state([contact_list_details])[0]

        self.assertEqual(cl.name, name)

        new_contact_list_details = {'id': cl.id, 'name': u"NewName"}
        #update
        cl =  self.controller.save_or_update_contact_lists_from_state(contact_lists_state = [new_contact_list_details])[0]

        self.assertEqual(cl.name, new_contact_list_details['name'])

    def test_delete_contacts_no_permission(self):
        """
        test delete contacts without having the necessary permissions or privileges
        """
        self._attach_permissions_to_group(self.group, 'create_contact', 'update_contact')
        #insert
        contact = self.controller.save_or_update_contact(id = None, person_data = self.person_data, company_data = None)
        #can't delete cuz no delete high permission
        self.assertRaises(PermissionDeniedException,
                          self.controller.delete_contacts,
                          contact)

        self._attach_permissions_to_group(self.group, 'view_contact', 'delete_contact')
        self.controller.revoke_privilege_from(contact, Contact.PRIV_DELETE, group=self.group, check_own_privs=False)

        #can't delete cuz no delete privs on object, but no exception raised, only action logged
        self.assertTrue(self.controller.delete_contacts(contact)==[contact.id])

    def test_delete_contacts(self):
        """
        test delete contacts
        """

        self._attach_permissions_to_group(self.group, 'view_contact', 'create_contact', 'delete_contact')

        #create a contact
        contact = self.controller.save_or_update_contact(id = None, person_data = self.person_data, company_data = None)
        id = contact.id
        self.controller.delete_contacts(contact)
        # nothing happened cuz delete priv is not set by default
        self.assertEqual(len(self.controller.get_contacts(id = id)), 0)
        self.assertEqual(len(self.controller.get_privileged_object_by_id(Contact, id, filter_marks=False)), 1)

    def test_delete_contact_lists_no_permission(self):
        """
        test delete contacts without having the ndecessary permissions or privileges
        """
        name = u"test"
        self._attach_permissions_to_group(self.group, 'create_contact_list', 'update_contact_list')
        #insert
        cl = self.controller.save_or_update_contact_list(id = None, name = name)
        #can't delete cuz no delete high permission
        self.assertRaises(PermissionDeniedException,
                          self.controller.delete_contact_lists,
                          cl)
        self._attach_permissions_to_group(self.group, 'view_contact_list', 'delete_contact_list')
        self.controller.delete_contact_lists(cl)
        self.assertEqual(len(self.controller.get_contact_lists(id = cl.id)), 0)

    def test_delete_contact_lists(self):
        """
        test delete contact lists
        """
        self._attach_permissions_to_group(self.group, 'view_contact_list', 'create_contact_list',
                                                      'delete_contact_list', 'update_contact_list')

        #create a contact list
        name = u"test contact list"
        cl = self.controller.save_or_update_contact_list(id = None, name = name)
        id = cl.id

        self.controller.revoke_privilege_from(cl, ContactList.PRIV_DELETE, self.group)

        self.controller.delete_contact_lists(cl)
        # nothing happened cuz delete priv was revoked
        self.assertEqual(len(self.controller.get_contact_lists(id = id)), 1)
        #add the delete priv

        self.controller.grant_privilege_to(cl, ContactList.PRIV_DELETE, self.group, check_own_privs=False)

        self.controller.delete_contact_lists(cl)
        # nothing happened cuz delete priv is not set by default
        self.assertEqual(len(self.controller.get_contact_lists(id = id)), 0)
        self.assertEqual(len(self.controller.get_privileged_object_by_id(ContactList, id, filter_marks=False)), 1)

    def test_link_contacts(self):
        """
        test link contacts to a contact list
        """
        self._attach_permissions_to_group(self.group, 'create_contact_list', 'create_contact')
        #create a contact
        contact = self.controller.save_or_update_contact(id = None, person_data = self.person_data, company_data = None)
        #create a contact list
        name = u"test contact list"
        cl = self.controller.save_or_update_contact_list(id = None, name = name)
        #can't link because no high level permissions (link on contact list and view on contact)
        self.assertRaises(PermissionDeniedException,
                          self.controller.link_contacts,
                          [contact], cl)

        self._attach_permissions_to_group(self.group, 'link_to_contact_list', 'view_contact')
        self.controller.link_contacts([contact], cl)
        self.assertEqual(contact.contact_lists[0], cl)
        self.assertEqual(cl.contacts[0], contact)

    def test_unlink_contacts(self):
        """
        test unlink contacts to a contact list
        """
        self._attach_permissions_to_group(self.group, 'create_contact_list', 'create_contact',
                                          'link_to_contact_list', 'view_contact')
        #create a contact
        contact = self.controller.save_or_update_contact(id = None, person_data = self.person_data, company_data = None)
        #create a contact list
        name = u"test contact list"
        cl = self.controller.save_or_update_contact_list(id = None, name = name)
        #link them
        self.controller.link_contacts([contact], cl)

        #can't unlink because no high level permissions unlink from contact list
        self.assertRaises(PermissionDeniedException,
                          self.controller.unlink_contacts,
                          [contact], cl)

        self._attach_permissions_to_group(self.group, 'unlink_from_contact_list')
        self.controller.unlink_contacts([contact], cl)
        self.assertEqual(len(contact.contact_lists), 0)
        self.assertEqual(len(cl.contacts), 0)

    def test_update_contact_with_no_privileges_on_contact_list(self):
        """
        this tests the saving of a contact list (as from the details panel) whereby the user does not have privileges
        to unlink from certain linked contact lists. See ticket #4326
        """

        self._attach_permissions_to_group(self.group, 'create_contact_list', 'create_contact', 'view_contact_list',
                                          'unlink_from_contact_list', 'link_to_contact_list', 'view_contact')

        # create a contact
        contact = self.controller.save_or_update_contact(id = None, person_data = self.person_data, company_data = None)

        # grant domain privs to contact
        self.controller.grant_privilege_to(contact, privileges=Contact.all_privileges, domain=True, target_group=self.group, check_own_privs=False)

        # make a second group+user.
        second_account = self._create_active_account(license=self.group.license)
        self._attach_permissions_to_group(second_account.groups[0].group, 'create_contact_list', 'create_contact', 'view_contact_list',
                                          'unlink_from_contact_list', 'link_to_contact_list', 'view_contact')

        # instantiate controller
        cc = ContactController(self.session, account_id = second_account.id)

        # create two contact lists, one for user a and one for user b.
        name = u"test contact list group A"
        cl1 = self.controller.save_or_update_contact_list(id = None, name = name)

        name = u"test contact list group B"
        cl2 = cc.save_or_update_contact_list(id = None, name = name)

        # link contact to both contact lists
        self.controller.link_contacts([contact], cl1)
        cc.link_contacts([contact], cl2)

        self.sa_session.expire(contact)
        self.controller.save_or_update_contacts_with_lists([contact], group_id=self.group.id, contact_lists=[cl1.id])

        self.sa_session.expire(contact)
        self.assertTrue(len(contact.contact_lists), 2)

    def test_check_contact_lists_email(self):
        """
        """
        self._attach_permissions_to_group(self.group, 'create_contact_list', 'create_contact',
                                          'link_to_contact_list', 'view_contact', 'update_contact')
        # remove the email from the person data
        self.person_data["email"] = None
        #create a contact
        contact = self.controller.save_or_update_contact(id = None, person_data = self.person_data,
                                                          company_data = None)
        #create a contact list
        name = u"cl no email"
        cl = self.controller.save_or_update_contact_list(id = None, name = name)

        #link contact to contact list
        self.controller.link_contacts([contact], cl)

        # it should fail, cuz the only contact linked to the cl has no email address
        self.assertRaises(PublicUserException,
                          self.controller.check_contact_lists_email,
                          cl)
        # add an email address to the contact
        self.person_data["email"] = u"testing@mail.clusternet"
        self.controller.save_or_update_contact(id = contact.id, person_data = self.person_data,
                                                company_data = None)
        self.assertTrue(self.controller.check_contact_lists_email(cl))

    def test_merge(self):
        self._attach_permissions_to_group(self.group, 'create_contact', 'view_contact', 'update_contact')

        self.person_data["email"] = 'john1@john.com'
        contact1 = self.controller.save_or_update_contact(id = None, person_data = self.person_data, company_data = None)

        self.person_data["email"] = 'john2@john.com'
        contact2 = self.controller.save_or_update_contact(id = None, person_data = self.person_data, company_data = None)

        self.person_data["email"] = 'john3@john.com'
        contact3 = self.controller.save_or_update_contact(id = None, person_data = self.person_data, company_data = None)

        cid1 = contact1.id
        cid2 = contact2.id
        cid3 = contact3.id

        pid1 = contact1.person.id
        pid2 = contact2.person.id
        pid3 = contact3.person.id

        self.controller.merge(pid1, [pid1, pid2, pid3], 'person')

        contacts = self.session.query(Contact).filter(Contact.id.in_([cid1, cid2, cid3])).all()
        self.assertEqual(contacts[0].person.id, pid1)
        self.assertEqual(contacts[1].person.id, pid1)
        self.assertEqual(contacts[2].person.id, pid1)

        persons = self.session.query(Person).filter(Person.id.in_([pid1, pid2, pid3])).all()
        self.assertEqual(len(persons), 1)
        self.assertEqual(persons[0].id, pid1)


class MockSimpleContactController(SimpleContactController):
    """
    Mock object to test protected methods of the SimpleContactController
    """
    def save_or_update_address(self, id, address_data):
        return self._save_or_update_address(id, address_data)

    def save_or_update_bank_account(self, id, bank_account_data):
        return self._save_or_update_bank_account(id, bank_account_data)

    def save_or_update_company(self, id, company_data):
        return self._save_or_update_company(id, company_data)

    def save_or_update_company_address(self, company, address, data):
        return self._save_or_update_company_address(company, address, data)

    def save_or_update_company_bank_account(self, company, bank_account, data):
        return self._save_or_update_company_bank_account(company, bank_account, data)

    def save_or_update_person(self, id, person_data):
        return self._save_or_update_person(id, person_data)

class SimpleContactControllerTestCase(iqportal_unittest.IQPortalTestCase):
    """
    This will test the Contact Controller.
    All individual *public* methods should be tested separately.
    """

    def setUp(self):
        """
        The script for reseting the db and creating all the tables
        """
        self.connect_db()

        # create an instance of the mock controller. It is used in all test methods.
        self.controller = MockSimpleContactController(self.session)

    def _get_person_data(self):
        return {'first_name': 'John',
                'middle_name': '',
                'last_name': 'Smith',
                'title': 'mr',
                'gender': Person.MALE,
                'email': 'john@mail-test.basesoft.com',
                'position': 'assistent',
                'department': 'marketing',
                'phone': '0040123456789',
                'mobile_phone': '',
                'fax': '',
                'birth_date': date(1978, 12, 31),
                'address_data': self._get_address_data()}

    def _get_address_data(self):
        return {'country': '181',
                'postcode': "400295",
                'street': " Bihorului 10",
                'city': "Cluj Napoca",
                'state': "Cluj"}

    def _get_bank_account_data(self):
        return {'number':self._uniquestring('99'),
                'iban':self._uniquestring('NL'),
                'bic':'BBBBNLNH'}

    def _get_company_data(self):
        return {'name': self._uniquestring('company'),
                'vat_no': 'NL 8104.49.493.B01',
                'website': 'http://www.basesoft.com',
                'company_type': 'Company',
                'contact_person_data': self._get_person_data()}

    def test_save_or_update_address(self):
        """
        This tests saving and updating of address
        """
        address_data = self._get_address_data()
        # Saving the address_data
        address = self.controller.save_or_update_address(id=None,
                                                          address_data=address_data)

        for key in address_data:
            self.assertEqual(getattr(address, key), address_data[key])

        # Now lets change it a bit and update it.
        new_address_data = {
            'country': 181,
            'postcode': "4002954",
            'street': " update Bihorului 10",
            'city': "Cluj N.",
            'state': "Cluj"}

        new_address = self.controller.save_or_update_address(id=address.id,
                                                              address_data=new_address_data)
        for key in new_address_data:
            self.assertEqual(getattr(new_address, key), new_address_data[key])

    def test_save_or_update_bank_account(self):
        """
        Testing of the saving and updating of Bank Account information
        """
        bank_data = self._get_bank_account_data()

        # Save the bank account information with the controller.
        account = self.controller.save_or_update_bank_account(None,bank_data)

        for key in bank_data:
            self.assertEqual(getattr(account, key), bank_data[key])

        # Now lets alter a field and check it.
        bank_data['bic'] = 'BBBBDELS'
        account = self.controller._save_or_update_bank_account(account.id,
                                                               bank_data)

        for key in bank_data:
            self.assertEqual(getattr(account, key), bank_data[key])

    def test_save_or_update_company(self):
        """
        Testing of the saving and updating of Company information
        """
        company_data = self._get_company_data()

        company = self.controller.save_or_update_company(None,company_data)

        for key in company_data:
            if key not in ["contact_person_data", "company_type"]:
                self.assertEqual(getattr(company, key),
                                 company_data[key])
                continue
            for k in company_data["contact_person_data"]:
                if(k != 'address_data'):
                    self.assertEqual(getattr(company.contact_person, k),
                        company_data["contact_person_data"][k])

    def test_save_or_update_company_address(self):
        """
        This tests the saving and updating of the link between company and
        address information.
        """
        company_data = self._get_company_data()
        company = self.controller.save_or_update_company(None,company_data)

        address_data = self._get_address_data()
        address = self.controller.save_or_update_address(None,address_data)

        ca = self.controller.save_or_update_company_address(company,address,
                 {'is_invoice': True, 'is_delivery': False})

        self.assertEqual(ca.is_invoice, True)
        self.assertEqual(ca.is_delivery, False)
        self.assertEqual(company, ca.company)
        self.assertEqual(address, ca.address)

        # Now lets test the update of this information

        ca = self.controller.save_or_update_company_address(company,address,
                 {'is_invoice': False, 'is_delivery': True})

        self.assertEqual(ca.is_invoice, False)
        self.assertEqual(ca.is_delivery, True)
        self.assertEqual(company, ca.company)
        self.assertEqual(address, ca.address)

    def test_save_or_update_company_bank_account(self):
        """
        Tests the saving and updating of the link between company and bank
        account data.
        """
        company_data = self._get_company_data()
        company = self.controller.save_or_update_company(None, company_data)

        bank_data = self._get_bank_account_data()
        account = self.controller.save_or_update_bank_account(None, bank_data)

        cba = self.controller.save_or_update_company_bank_account(company,account,
                  {'is_default': True})

        self.assertEqual(cba.is_default, True)
        self.assertEqual(company, cba.company)
        self.assertEqual(account, cba.bank_account)

        # And then we update it.
        cba = self.controller.save_or_update_company_bank_account(company,account,
                  {'is_default': False})

        self.assertEqual(cba.is_default, False)
        self.assertEqual(company, cba.company)
        self.assertEqual(account, cba.bank_account)

    def test_save_or_update_person(self):
        """
        This tests saving and updating of person
        """

        person_data = self._get_person_data()

        # Save the person data including the address stuff.
        person =  self.controller.save_or_update_person(id=None,
                                                         person_data=person_data)
        for key in person_data:
            if key != "address_data":
                self.assertEqual(getattr(person, key),
                                 person_data[key])
                continue
            for k in person_data["address_data"]:
                self.assertEqual(getattr(person.address, k),
                                 person_data["address_data"][k])

        # Now lets change a few attributes and update the person data stuff.
        new_person_data = self._get_person_data()
        new_person_data["first_name"] = u"Anna"
        new_person_data["gender"] = Person.FEMALE
        new_person_data["address_data"]["postcode"] = "400000"

        new_person =  self.controller.save_or_update_person(id=person.id,
                                                             person_data=new_person_data)
        for key in new_person_data:
            if key != "address_data":
                self.assertEqual(getattr(new_person, key),
                                 new_person_data[key])
                continue
            for k in new_person_data["address_data"]:
                self.assertEqual(getattr(new_person.address, k),
                                 new_person_data["address_data"][k])

class ContactsImportControllerTestCase(iqportal_unittest.IQPortalTestCase):
    """
    This will test the ContactImport Controller.
    All individual *public* methods should be tested separately.
    """

    def setUp(self):
        """
        The script for reseting the db and creating all the tables
        """
        self.connect_db()

        #create an account so we can check permission with it
        self.account = account = self._create_active_account()
        self.group = account.groups[0].group

        # add all required permissions since all permissions checks are tested
        # in ContactControllerTestCase
        self._attach_permissions_to_group(self.group,
            'import_contacts', 'view_contact','create_contact',
            'update_contact', 'delete_contact')

        # create the file
        self.filepointer = "/tmp/test_import_contacts.csv"
        f = open(self.filepointer, 'w')
        f.write("%s\n%s\n%s\n%s"
                % ("first name,middle name,last name,email address,phone number,country",
                   "John,,Smith,john@mail-test.basesoft.com,0040740123456,romaniaaaa",
                   "jerry,,seinfeld,jerry@mail-test.basesoft.com,,United States of America",
                   "anne,marie,franklin,anne@email,0012465889,netherlands")
                )
        f.close()

        # create an instance of the controller. It is used in all test methods.
        self.controller = ImportContactController(self.filepointer, 'test_import_contacts.csv', self.session, account_id = account.id)
        self.controller.set_file_lines()

        self.controller.field_mappings = [
            {"file_header": "first name", "column": "first_name", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "middle name", "column": "middle_name", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "last name", "column": "last_name", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "email address", "column": "email", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "phone number", "column": "phone", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "country", "column": "country_name", "target": self.controller.COLUMN_IS_PERSON_INFO}
       ]


    def test_contact_list_merge(self):
        """
        Added here to have contact lists available.
        """
        self._attach_permissions_to_group(self.group, 'create_contact_list', 'link_to_contact_list')

        # create contact list one


        headers = ['first name','email address','company_name']
        contact1 = ['John', 'b.rat@mail-test.basesoft.com', 'JOHN']
        contact2 = ['Freddie', 'f.mercury@mail-test.basesoft.com', 'FRED INC']
        contact3 = ['Shawn', 'shawnie@mail-test.basesoft.com', 'SH.AW.N']

        f = open(self.filepointer, 'w')
        f.write("%s\n%s\n%s\n%s"
                % (
                   ','.join(headers),
                   ','.join(contact1),
                   ','.join(contact2),
                   ','.join(contact3)
                ))
        f.close()
        self.controller.set_file_lines()

        self.controller.field_mappings = [
            {"file_header": "first name", "column": "first_name", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "email address", "column": "contact_email", "target": self.controller.COLUMN_IS_CONTACT},
            {"file_header": "company_name", "column": "name", "target": self.controller.COLUMN_IS_COMPANY_INFO},
        ]
        first_contacts = self.controller.import_contacts();

        cl1 = self.controller.link_contacts(first_contacts, self._uniquestring('list_one'))

        # create contact list two with overlapping companies/persons

        headers = ['first name','email address','company_name']
        contact1 = ['John', 'b.rat@mail-test.basesoft.com', 'JOHN']
        contact2 = ['Freddie', 'f.mercury@mail-test.basesoft.com', 'FRED INC']

        f = open(self.filepointer, 'w')
        f.write("%s\n%s\n%s"
                % (
                   ','.join(headers),
                   ','.join(contact1),
                   ','.join(contact2)
                ))
        f.close()
        self.controller.set_file_lines()

        self.controller.field_mappings = [
            {"file_header": "first name", "column": "first_name", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "email address", "column": "contact_email", "target": self.controller.COLUMN_IS_CONTACT},
            {"file_header": "company_name", "column": "name", "target": self.controller.COLUMN_IS_COMPANY_INFO},
        ]
        contacts = self.controller.import_contacts()
        self.assertEqual(len(contacts), 2)
        # they should be new ones.
        for contact in contacts:
            self.assertFalse(contact in first_contacts)

        cl2 = self.controller.link_contacts(contacts, self._uniquestring('list_two'))


        # merge them together with a unique identifier set for both companies and persons.
        self.controller.merge_contact_list_contacts(cl1, cl2, [
            {'target': self.controller.COLUMN_IS_PERSON_INFO, 'column': 'first_name'},
            {'target': self.controller.COLUMN_IS_COMPANY_INFO, 'column': 'name'}
        ])

        persons = []
        companies = []
        self.sa_session.refresh(cl2)
        for contact in cl1.contacts:
            persons.append(contact.person)
            companies.append(contact.company)

        for contact in cl2.contacts:
            self.assertTrue(contact.person in persons)
            self.assertTrue(contact.company in companies)


    def test_autodetermine_separator(self):
        """
        This tests autodetermining of the file columns separator
        """
        self.assertEqual(self.controller._importer.separator,",")

    def test_get_row_values(self):
        """
        Tests getting a row's values
        """
        # headers
        values = self.controller.get_row_values(row_index=0)
        self.assertEqual(len(values),6)
        vals = ["first name", "middle name", "last name", "email address","phone number","country"];
        for i in range(len(vals)):
            self.assertEqual(values[i],vals[i])

        # a simple row
        values = self.controller.get_row_values(row_index=2)
        vals = ["jerry", "", "seinfeld", "jerry@mail-test.basesoft.com","","United States of America"];
        for i in range(len(vals)):
            self.assertEqual(values[i],vals[i])


    def test_validate(self):
        """
        test file values validation
        """
        result = self.controller.validate()
        self.assertEqual(len(result),2)

        self.assertEqual(result[0]["row_index"],0)
        self.assertTrue(result[0]["errors"].has_key("country_name"))

        self.assertEqual(result[1]["row_index"],2)
        self.assertTrue(result[1]["errors"].has_key("email"))

    def test_import_contacts(self):
        """
        test importing of contacts from file
        """

        cl = self._uniquestring('cl_')
        self._attach_permissions_to_group(self.group, 'create_contact_list', 'link_to_contact_list')

        excluded_rows = [2]
        updated_rows = [
            { "row_index": 0,
              "row_values": {"first_name": "update john",
                         "middle_name": "",
                         "last_name": "smith",
                         "email": "john@mail-test.basesoft.com",
                         "phone": "",
                         "country_name":"romania"}},
        ]

        contacts = self.controller.import_contacts(updated_rows, excluded_rows)
        self.assertEqual(len(contacts),2)
        self.assertEqual(contacts[0].person.first_name, "update john")
        self.assertEqual(contacts[0].person.last_name, "smith")
        self.assertEqual(contacts[0].person.email, "john@mail-test.basesoft.com")
        self.assertEqual(contacts[0].person.address.country.name, "Romania")
        self.assertEqual(contacts[1].person.email, "jerry@mail-test.basesoft.com")

        # test import companies with address and bank info
        # need some ids to make ibans and account numbers unique
        c = [str(c.id) for c in contacts]
        f = open(self.filepointer, 'w')
        f.write("%s\n%s\n%s"
                % ("first name,middle name,last name,email address,phone number,country, name, iban, bic, number, city, contact_email, setting, list",
                   "John,,Smith,john@mail-test.basesoft.com,0040740123456,romania,C1,iban"+c[0]+",bic1, acc"+c[0]+",cluj,johnc@mail-test.basesoft.com, 1, "+cl,
                   "jerry,,seinfeld,jerry@mail-test.basesoft.com,,United States of America,C2,iban"+c[1]+",bic2, acc"+c[1]+",ams,jerryc@mail-test.basesoft.com, 0, "+cl)
                )
        f.close()
        self.controller.field_mappings = self.controller.field_mappings + [
            {"file_header": "name", "column": "name", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "iban", "column": "iban", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "bic", "column": "bic", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "number", "column": "number", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "city", "column": "company_city", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "contact_email", "column": "contact_email", "target": self.controller.COLUMN_IS_CONTACT},
            {"file_header": "setting", "column": "mail_settings", "target": self.controller.COLUMN_IS_CONTACT_LIST_CONTACT},
            {"file_header": "list", "column": "contact_list_name", "target": self.controller.COLUMN_IS_CONTACT_LIST}
        ]
        self.controller.set_file_lines()

        rv = self.controller.validate()
        self.assertEqual(len(rv), 0)

        contacts = self.controller.import_contacts(None, None)
        self.assertEqual(len(contacts), 2)

        self.assertEqual(contacts[0].email, 'johnc@mail-test.basesoft.com')
        self.assertEqual(contacts[1].email, 'jerryc@mail-test.basesoft.com')

        self.assertEqual(contacts[0].company.name, 'C1')
        self.assertEqual(contacts[1].company.name, 'C2')

        self.assertEqual(len(contacts[0].company.addresses), 1)
        self.assertEqual(len(contacts[0].company.bank_accounts), 1)
        self.assertEqual(contacts[0].company.addresses[0].address.city, 'cluj')
        self.assertEqual(contacts[0].company.bank_accounts[0].bank_account.number, "acc"+c[0])


    def test_import_contacts_wrong_mail_settings(self):
        """
        Test correct validation error for mail settings (#3833)
        """
        cl = self._uniquestring('cl_')
        self._attach_permissions_to_group(self.group, 'create_contact_list', 'link_to_contact_list')

        # test import companies with address and bank info
        # need some ids to make ibans and account numbers unique
        f = open(self.filepointer, 'w')
        f.write("%s\n%s\n%s"
                % ("first name,middle name,last name,email address,phone number,country, name, iban, bic, number, city, contact_email, setting, list",
                   "John,,Smith,john@mail-test.basesoft.com,0040740123456,romania,C1,iban1,bic1, acc1,cluj,johnc@mail-test.basesoft.com, O, "+cl,
                   "jerry,,seinfeld,jerry@mail-test.basesoft.com,,United States of America,C2,iban2,bic2, acc2,ams,jerryc@mail-test.basesoft.com, 0, "+cl)
                )
        f.close()
        self.controller.field_mappings = self.controller.field_mappings + [
            {"file_header": "name", "column": "name", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "iban", "column": "iban", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "bic", "column": "bic", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "number", "column": "number", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "city", "column": "company_city", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "contact_email", "column": "contact_email", "target": self.controller.COLUMN_IS_CONTACT},
            {"file_header": "setting", "column": "mail_settings", "target": self.controller.COLUMN_IS_CONTACT_LIST_CONTACT},
            {"file_header": "list", "column": "contact_list_name", "target": self.controller.COLUMN_IS_CONTACT_LIST}
        ]
        self.controller.set_file_lines()

        rv = self.controller.validate()
        self.assertEqual(len(rv), 1)

    def test_import_preferred_locale(self):
        headers = ['first name','email address','country', 'language']
        contact1 = ['Borat', 'b.rat@mail-test.basesoft.com', 'NL', 'nl']
        contact2 = ['Freddie', 'f.mercury@mail-test.basesoft.com', 'GB', 'en']
        contact3 = ['Ling-wu', 'wuwuwu@mail-test.basesoft.com', 'UA', 'en'] # shouldn't be found

        f = open(self.filepointer, 'w')
        f.write("%s\n%s\n%s\n%s"
                % (
                   ','.join(headers),
                   ','.join(contact1),
                   ','.join(contact2),
                   ','.join(contact3),
                ))
        f.close()
        self.controller.set_file_lines()

        self.controller.field_mappings = [
            {"file_header": "first name", "column": "first_name", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "email address", "column": "email", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "country", "column": "country_iso_3166_a2", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "language", "column": "preferred_locale", "target": self.controller.COLUMN_IS_PERSON_INFO},
        ]
        rv = self.controller.validate()
        self.assertTrue(len(rv) == 0)

        en_locale = self.sa_session.query(Locale).filter(Locale.locale=='en_GB').one()
        nl_locale = self.sa_session.query(Locale).filter(Locale.locale=='nl_NL').one()

        uf = [{'target': self.controller.COLUMN_IS_PERSON_INFO, 'column': 'email'}]
        contacts = self.controller.import_contacts(None, None, uf)
        self.assertTrue(len(contacts), 3)
        self.assertEqual(contacts[0].person.preferred_locale, nl_locale)
        self.assertEqual(contacts[1].person.preferred_locale, en_locale)
        self.assertEqual(contacts[2].person.preferred_locale, None)

    def test_import_preferred_locale_no_country(self):
        headers = ['first name','email address','country', 'language']
        contact1 = ['Borat', 'b.rat@mail-test.basesoft.com', 'NL', 'nl']
        contact2 = ['Freddie', 'f.mercury@mail-test.basesoft.com', 'GB', 'en']
        contact3 = ['Ling-wu', 'wuwuwu@mail-test.basesoft.com', 'UA', 'en'] # shouldn't be found

        f = open(self.filepointer, 'w')
        f.write("%s\n%s\n%s\n%s"
                % (
                   ','.join(headers),
                   ','.join(contact1),
                   ','.join(contact2),
                   ','.join(contact3),
                ))
        f.close()
        self.controller.set_file_lines()

        self.controller.field_mappings = [
            {"file_header": "first name", "column": "first_name", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "email address", "column": "email", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "language", "column": "preferred_locale", "target": self.controller.COLUMN_IS_PERSON_INFO},
        ]
        rv = self.controller.validate()
        self.assertTrue(len(rv)==3)

    def test_import_with_client_last_update(self):
        """
        This tests a feature to compare the contacts' last update date with the date in the file.
        See ticket #3169
        """

        # create a new file which has a last update date
        headers = ['first name','email address','last update']
        contact1 = ['Borat', 'b.rat@mail-test.basesoft.com', '29/01/2011']
        contact2 = ['Freddie', 'f.mercury@mail-test.basesoft.com', '29/01/2011']

        f = open(self.filepointer, 'w')
        f.write("%s\n%s\n%s"
                % (
                   ','.join(headers),
                   ','.join(contact1),
                   ','.join(contact2)
                ))
        f.close()
        self.controller.set_file_lines()

        self.controller.field_mappings = [
            {"file_header": "first name", "column": "first_name", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "email address", "column": "email", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "last update", "column": "client_last_update", "target": self.controller.COLUMN_IS_CONTACT},
        ]
        # validate the file
        rv = self.controller.validate()
        self.assertTrue(len(rv)==2)

        # change the config. TODO: this needs to be done with the LicenseConfigController when that is implemented
        lc = LicenseConfig()
        lc.config = self.sa_session.query(Config).filter(Config.name=='contact.client_last_update.date_format').one()
        lc.value = '%d/%m/%Y'
        lc.license = self.group.license
        self.sa_session.add(lc)
        self.sa_session.flush()

        uf = [{'target': self.controller.COLUMN_IS_PERSON_INFO, 'column': 'email'}]

        rv = self.controller.validate(None, None, uf)
        self.assertTrue(len(rv)==0)

        # import the file
        self.controller.import_contacts(None, None, uf)

        # now we upload a new file, one of the contacts has newer last update column, the other has old data.
        contact1[2] = '14/02/2011'
        contact2[2] = '01/01/2011'
        f = open(self.filepointer, 'w')
        f.write("%s\n%s\n%s"
                % (
                   ','.join(headers),
                   ','.join(contact1),
                   ','.join(contact2)
                ))
        f.close()
        self.controller.set_file_lines()

        # verify that only the one with a newer date is update and that the old data row is in the validation failure file.
        rv = self.controller.validate(None, None, uf)
        self.assertTrue(len(rv)==1)

    def test_import_with_unique_fields_on_contact_and_person(self):
        # changes made with ticket #3895 should be tested
        headers = ['first name','email address','last name']
        contact1 = ['Borat', '%s@mail-test.basesoft.com' % self._uniquestring('ln'), self._uniquestring('ln')]
        contact2 = ['Freddie', '%s@mail-test.basesoft.com' % self._uniquestring('ln'), self._uniquestring('ln')]

        f = open(self.filepointer, 'w')
        f.write("%s\n%s\n%s"
                % (
                   ','.join(headers),
                   ','.join(contact1),
                   ','.join(contact2)
                ))
        f.close()
        self.controller.set_file_lines()

        self.controller.field_mappings = [
            {"file_header": "first name", "column": "first_name", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "email address", "column": "contact_email", "target": self.controller.COLUMN_IS_CONTACT},
            {"file_header": "last name", "column": "last_name", "target": self.controller.COLUMN_IS_PERSON_INFO},
        ]

        contacts = self.session.query(Contact).filter(Contact.email.in_([contact1[1], contact2[1]])).all()
        self.assertEqual(len(contacts), 0)

        uf = [{'target': self.controller.COLUMN_IS_CONTACT, 'column': 'contact_email'}, {'target': self.controller.COLUMN_IS_PERSON_INFO, 'column': 'last_name'}]
        original = self.controller.import_contacts(None, None, uf)

        contact1 = ['Borat', '%s@mail-test.basesoft.com' % self._uniquestring('ln'), contact2[2]]
        contact2 = ['Freddie', '%s@mail-test.basesoft.com' % self._uniquestring('ln'), self._uniquestring('ln')]

        f = open(self.filepointer, 'w')
        f.write("%s\n%s\n%s"
                % (
                   ','.join(headers),
                   ','.join(contact1),
                   ','.join(contact2)
                ))
        f.close()
        self.controller.set_file_lines()

        contacts = self.controller.import_contacts(None, None, uf)
        for c in contacts:
            self.assertTrue(c not in original, 'Contact %s was found in original' % c)

        self.assertEqual(contacts[0].company, original[1].company)

    def test_import_with_unique_fields(self):

        # create a new file with emails that would be unique in the db
        uuid = str(time.time())
        emails = ["email1"+uuid+"@mail-test.basesoft.com", "email2"+uuid+"@mail-test.basesoft.com",
                  "email3"+uuid+"@mail-test.basesoft.com"]

        f = open(self.filepointer, 'w')
        f.write("%s\n%s\n%s\n%s"
                % ("first name,middle name,last name,email address,phone number,country",
                   "John,,Smith,"+emails[0]+",0040740123456,romania",
                   "jerry,,seinfeld,"+emails[1]+",,United States of America",
                   "anne,marie,franklin,"+emails[2]+",0012465889,netherlands")
                )
        f.close()
        self.controller.set_file_lines()

        # no contacts with those emails should be in the db
        contacts = self.session.query(Contact).join(Person).filter(Person.email.in_(emails)).all()
        self.assertEqual(len(contacts), 0)

        uf = [{'target': self.controller.COLUMN_IS_PERSON_INFO, 'column': 'email'}]
        contacts = self.controller.import_contacts(None, None, uf)
        # even if unique fields set, emails were not found, so new contacts are saved
        self.assertEqual(len(contacts), 3)
        cids = [c.id for c in contacts]
        pids = [c.person.id for c in contacts]
        self.assertEqual(contacts[0].person.email, emails[0])
        self.assertEqual(contacts[1].person.email, emails[1])
        self.assertEqual(contacts[2].person.email, emails[2])
        self.assertEqual(contacts[0].person.first_name, 'John')
        self.assertEqual(contacts[1].person.first_name, 'jerry')
        self.assertEqual(contacts[2].person.first_name, 'anne')

        # update the file, but keep the emails
        f = open(self.filepointer, 'w')
        f.write("%s\n%s\n%s\n%s"
                % ("first name,middle name,last name,email address,phone number,country",
                   "John update,,Smith,"+emails[0]+",0040740123456,romania",
                   "jerry update,,seinfeld,"+emails[1]+",,United States of America",
                   "anne update,marie,franklin,"+emails[2]+",0012465889,netherlands")
                )
        f.close()
        self.controller.set_file_lines()

        uf = [{'target': self.controller.COLUMN_IS_PERSON_INFO, 'column': 'email'}]
        contacts = self.controller.import_contacts(None, None, uf)
        # now we should have the same contact ids (cuz company is none for those contacts)
        self.assertEqual(len(contacts), 3)
        for c in contacts:
            self.assertTrue(c.id in cids)
            self.assertTrue(c.person.id in pids)
        self.assertEqual(contacts[0].person.email, emails[0])
        self.assertEqual(contacts[1].person.email, emails[1])
        self.assertEqual(contacts[2].person.email, emails[2])
        self.assertEqual(contacts[0].person.first_name, 'John update')
        self.assertEqual(contacts[1].person.first_name, 'jerry update')
        self.assertEqual(contacts[2].person.first_name, 'anne update')

        # now add some company info to the file (same persons)
        f = open(self.filepointer, 'w')
        f.write("%s\n%s\n%s\n%s"
                % ("first name,middle name,last name,email address,phone number,country, name",
                   "John update,,Smith,"+emails[0]+",0040740123456,romania, C1",
                   "jerry update,,seinfeld,"+emails[1]+",,United States of America, C2",
                   "anne update,marie,franklin,"+emails[2]+",0012465889,netherlands, C3")
                )
        f.close()
        self.controller.field_mappings.append(
            {"file_header": "name", "column": "name", "target": self.controller.COLUMN_IS_COMPANY_INFO}
        )
        self.controller.set_file_lines()

        uf = [{'target': self.controller.COLUMN_IS_PERSON_INFO, 'column': 'email'}]
        contacts = self.controller.import_contacts(None, None, uf)
        # now we should have new contacts but linked to same persons
        self.assertEqual(len(contacts), 3)
        for c in contacts:
            self.assertFalse(c.id in cids)
            self.assertTrue(c.person.id in pids)
        self.assertEqual(contacts[0].person.email, emails[0])
        self.assertEqual(contacts[1].person.email, emails[1])
        self.assertEqual(contacts[2].person.email, emails[2])
        self.assertEqual(contacts[0].person.first_name, 'John update')
        self.assertEqual(contacts[1].person.first_name, 'jerry update')
        self.assertEqual(contacts[2].person.first_name, 'anne update')

        self.assertEqual(contacts[0].company.name, 'C1')
        self.assertEqual(contacts[1].company.name, 'C2')
        self.assertEqual(contacts[2].company.name, 'C3')

    def test_get_available_columns(self):
        """
        test getting available table columns for the field mappings
        """
        cols = self.controller.get_available_columns()

        self.assertEqual(len(cols), 6)

        person_cols = ["first_name", "middle_name", "last_name", "title",
                       "gender", "email", "position", "department", "phone",
                       "mobile_phone", "fax", "birth_date", "postcode",
                       "street", "city", "state", "country_name", "skype_name",
                       "country_iso_3166_a2", "country_iso_3166_a3", "preferred_locale"]
        company_cols = ["name", "vat_no", "website", "number", "iban", "bic",
                       "postcode", "street", "city", "state", "country_name",
                       "country_iso_3166_a2", "country_iso_3166_a3"]

        self.assertEqual(len(cols[1]["options"]), len(person_cols))
        self.assertEqual(len(cols[2]["options"]), len(company_cols))


    def _make_xlsx(self, data):

        # http://packages.python.org/openpyxl
        from openpyxl.workbook import Workbook
        wb = Workbook(encoding="UTF-8")
        ws = wb.get_active_sheet()
        ws.title = "New Title"

        for i in range(len(data)):
            for j in range(len(data[i])):
                cell = ws.cell(row = i, column = j)
                cell.value = data[i][j]

        file_path = os.path.join(settings.WRITABLE_DIRECTORY, 'temp.xlsx')
        wb.save( file_path)

        return file_path


    def _make_xls(self, data):

        import xlwt
        wbk = xlwt.Workbook()
        sheet = wbk.add_sheet('sheet 1')

        for i in range(len(data)):
            for j in range(len(data[i])):

                v = data[i][j]

                # stupid excel dates: http://stackoverflow.com/questions/3118940/datetime-issue-with-xlrd-xlwt-python-libs
                if isinstance(v, (datetime.date, datetime.datetime)):
                    style = xlwt.XFStyle()
                    style.num_format_str = 'YYYY-MM-DD'
                    sheet.write(i, j, v, style)
                else:
                    sheet.write(i, j, v)

        file_path = os.path.join(settings.WRITABLE_DIRECTORY, 'temp.xls')
        wbk.save(file_path)

        return file_path

    def _get_excel_test_data(self):
        return [
            ['first name', 'middle name', 'last name', 'email address', 'phone number', 'country', 'last update'],
            ['John','','Smith','john@mail-test.basesoft.com','0040740123456','Romania', datetime.date(2013, 1, 1)],
            ['jerry','','seinfeld','jerry@mail-test.basesoft.com','','United States of America', datetime.date(2011, 12, 31)],
            ['anne','marie','franklin','anne@email.com','0012465889','netherlands', datetime.date(2011, 10, 1)]
        ]

    def _test_import_excel(self, file_path, file_name):

        # create an instance of the controller. It is used in all test methods.
        controller = ImportContactController(file_path, file_name, self.session, account_id = self.account.id)
        controller.set_file_lines()

        controller.field_mappings = [
            {"file_header": "first name", "column": "first_name", "target": controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "middle name", "column": "middle_name", "target": controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "last name", "column": "last_name", "target": controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "email address", "column": "email", "target": controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "phone number", "column": "phone", "target": controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "country", "column": "country_name", "target": controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "last update", "column": "client_last_update", "target": controller.COLUMN_IS_CONTACT},
       ]

        # headers
        values = controller.get_row_values(row_index=0)
        self.assertEqual(len(values), 7)
        vals = ["first name", "middle name", "last name", "email address","phone number","country", 'last update'];
        for i in range(len(vals)):
            self.assertEqual(values[i],vals[i])

        # verify a row
        values = controller.get_row_values(row_index=2)
        vals = ["jerry", "", "seinfeld", "jerry@mail-test.basesoft.com","","United States of America", datetime.datetime(2011, 12, 31, 0, 0)];
        for i in range(len(vals)):
            self.assertEqual(values[i],vals[i])

        # unique fields
        uf = [{'target': controller.COLUMN_IS_PERSON_INFO, 'column': 'email'}]

        # validate file
        rv = controller.validate(None, None, uf)
        self.assertTrue(len(rv)==0)

        # import
        controller.import_contacts(None, None, uf)


    def test_import_xlsx(self):
        file_path = self._make_xlsx(self._get_excel_test_data())
        self._test_import_excel(file_path, 'test.xlsx')

    def test_import_xls(self):
        file_path = self._make_xls(self._get_excel_test_data())
        self._test_import_excel(file_path, 'test.xls')


    def tearDown(self):
        """
        rm the file
        """
        os.remove(self.filepointer)

class ContactFieldMappingTestCase(iqportal_unittest.IQPortalTestCase):

    def setUp(self):
        self.connect_db()

        #create an account so we can check permission with it
        self.account = account = self._create_active_account()
        self.group = account.groups[0].group

        self.controller = ContactController(self.sa_session, account_id=self.account.id)

    def test_save_or_update_field_mapping(self):
        """
        Tests saving and updating field mapping presets.
        """

        mapping = [{
            'field': 'Name',
            'target': {"1": 'last_name'}
        }, {
            'field': 'Email Address',
            'target': {"1": 'email'}
        }]
        unique_fields = None

        # check exception for permission check
        self.assertRaises(PermissionDeniedException, self.controller.save_or_update_field_mapping, None, 'test1', 'Contact', mapping, unique_fields)

        # assign required permissions
        self._attach_permissions_to_group(self.group, 'use_field_mapping_presets', 'manage_field_mapping_presets')

        # check for bogus contact type to throw security exception
        self.assertRaises(SecurityException, self.controller.save_or_update_field_mapping, None, 'test1', 'BogusContactType', mapping, unique_fields)

        # save the field mapping
        fm = self.controller.save_or_update_field_mapping(None, 'test1', 'Contact', mapping, unique_fields)
        self.sa_session.flush()
        # load the field mapping for validation
        fm = self.controller.get_field_mappings(fm.id).pop()

        self.assertEqual(mapping, fm.mapping)
        self.assertEqual('Contact', fm.contact_type)
        self.assertEqual('test1', fm.name)

        id = fm.id
        self.sa_session.expire(fm)

        # now we try to add a unique field which is not in the curent mapping.
        unique_fields = ['1+first_name'] # TODO: this value is based on the grouped fancy select setup: #3717

        # assert the raised public user exception
        self.assertRaises(PublicUserException, self.controller.save_or_update_field_mapping, id, 'test1', 'Contact', mapping, unique_fields)

        unique_fields = ['1+last_name']
        self.controller.save_or_update_field_mapping(id, 'test2', 'Contact', mapping, unique_fields)

        fm = self.controller.get_field_mappings(id).pop()
        self.assertEqual('test2', fm.name)
        self.assertEqual(unique_fields, fm.unique_fields)

class ContactAutomatedImportTestCase(iqportal_unittest.IQPortalTestCase):
    """
    Tests the automated import from ticket #3678
    """

    def setUp(self):
        self.connect_db()

        #create an account so we can check permission with it
        self.account = account = self._create_active_account()
        self.group = account.groups[0].group

        self._attach_permissions_to_group(self.group, 'view_notification', 'import_contacts', 'create_contact', 'view_contact')

        d = os.path.join(settings.CONTACT_IMPORT_DIR, '%s/' % self.group.license.id)

        if not os.path.exists(d): # create the directory if it does not exist
            os.makedirs(d)

        self.filepointer = "%stest_import_contacts.csv" % d

    def test_automated_import(self):
        """
        """

        # create a file in the CONTACT_IMPORT_DIR for the created license.
        f = open(self.filepointer, 'w')
        f.write("%s\n%s\n%s\n%s"
                % ("first name,middle name,last name,email address,phone number,country, name",
                   "John,,Smith, j@mail-test.basesoft.com,0040740123456,romania, C1",
                   "jerry,,seinfeld,s@mail-test.basesoft.com,,United States of America, C2",
                   "anne,marie,franklin,f@mail-test.basesoft.com,0012465889,netherlands, C3")
                )
        f.close()

        # verify the file is not added to the import queue due to the permission not being added
        t = AutomatedImportTask()
        t.execute()

        self.assertTrue(os.path.exists(self.filepointer))

        # add the permission for automated import
        self._attach_permissions_to_group(self.group, 'automated_imports')

        nc = NotificationController(self.sa_session, account_id=self.account.id)
        cnt = len(nc.get_notifications())

        # verify notification has been sent due to no Field Mapping presets
        t.execute()

        new_cnt = cnt+1
        notifications = len(nc.get_notifications())
        self.assertEqual(new_cnt, notifications)

        # add field mapping preset
        self._attach_permissions_to_group(self.group, 'use_field_mapping_presets', 'manage_field_mapping_presets')

        mapping = [{
            'field': 'first name',
            'target': {"1": 'first_name'}
        }, {
            'field': 'middle name',
            'target': {"1": 'middle_name'}
        }, {
            'field': 'last name',
            'target': {"1": 'last_name'}
        }, {
            'field': 'email address',
            'target': {"1": 'email'}
        }]

        cc = ContactController(self.sa_session, account_id=self.account.id)

        fm = cc.save_or_update_field_mapping(None, 'test1', 'Contact', mapping, None)
        self.sa_session.flush()

        # set the created field mapping as the license default.
        c = self.sa_session.query(Config).filter(Config.name=='contact.import.default.field.mapping.id').one()
        lc = LicenseConfig()
        lc.license = self.group.license
        lc.config = c
        lc.value = fm.id
        self.sa_session.add(lc)
        self.sa_session.flush()

        t.execute()
        # no new notification should have been triggered?
        self.assertEqual(new_cnt, notifications)

        # verify file has been added to the background import queue
        it = ImportContactsTask()
        it.execute()

        # we should have 3 newly created contacts for the current license
        self.assertEqual(len(cc.get_contacts()),3)

    def tearDown(self):
        # remove the file if it still exists
        if os.path.exists(self.filepointer):
            os.remove(self.filepointer)





