# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from hashlib import sha256 as sha
import random
import os

from django.conf import settings

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.core.util.custom_query import CustomQueryRpcHandler, FILTER_EQUAL
from basesoft.web.contrib.i18n.models import Country, DomainCountry
from basesoft.web.test.custom_query import TestSQLCustomQuery, TestSQLUnionCustomQuery, TestFalseFieldCustomQuery


class TestCase(iqportal_unittest.IQPortalTestCase):

    def setUp(self):
        self.connect_db()
        self.account = self._create_active_account()
        self.group = self.account.groups[0].group

    def test_cq(self):
        rpc = CustomQueryRpcHandler('basesoft.web.test.TestMailCustomQuery', self.session, self.account.id)
        rs = rpc.get_definitions('', [], [])

        self.assertTrue(len(rs) == 1)
        rs = rs.pop()

        self.assertTrue(rs['title'] is None)

        coldef = rs['column_defs']

        # since Mail is a Privileged Object, column list will also contain 'deleted' and 'archived'
        self.assertTrue(len(coldef) == 4)
        self.assertEquals('mail_1_id', coldef[0]['key'])
        self.assertEquals('ID', coldef[0]['label'])
        self.assertEquals(False, coldef[0]['sortable'])
        self.assertEquals('mail_1_name', coldef[1]['key'])
        self.assertEquals('Name', coldef[1]['label'])
        self.assertEquals(True, coldef[1]['sortable'])

    def test_file_cq(self):
        from basesoft.web.contrib.file.controllers import ProcessFileController
        pfc = ProcessFileController(self.session, account_id=self.account.id)
        hash = sha(str(random.random())).hexdigest()[:8]

        #create a source file
        source_path = os.path.join(settings.TMP_FILE_REPOSITORY, hash)
        f = open(source_path, 'w')
        f.write("a")
        f.close()

        self._attach_permissions_to_group(self.group, 'create_file', 'view_file', 'update_file')

        fc = pfc.save_or_update_file_category(None, u'test', None)

        file = pfc.save_or_update_file(id=None,
                                       file_name='aapje_kijken',
                                       file_hash=hash,
                                       remove_source=False,
                                       category=fc)

        rpc = CustomQueryRpcHandler('basesoft.web.test.TestFileCustomQuery', self.session, self.account.id)

        # wihtout groupby
        data = rpc.get_data(None, {'dir': 'asc', 'sort': None, 'startIndex': 0, 'results': 5}, {}, [])['data']

        self.assertEquals(file.id, data[0]['file_id'])
        self.assertEquals(file.category.id, data[0]['filecategory_id'])

    def test_group_concat_separation(self):

        rpc = CustomQueryRpcHandler('basesoft.web.test.TestGroupConcatQuery', self.session, self.account.id)
        data = rpc.get_data(None, {'dir': 'asc', 'sort': None, 'startIndex': 0, 'results': 5}, {}, [])['data']
        self.assertEqual(len(data), 1)

        data = rpc.get_data(None, {'dir': 'asc', 'sort': None, 'startIndex': 0, 'results': 5}, {}, [], separated=True)['data']
        self.assertTrue(len(data) > 0)
        self.assertNotEqual(len(data), 1)

    def _fill_collins(self, account_id=None):

        if account_id is None:
            account_id = self.account.id

        from basesoft.web.contrib.contact.controllers import ContactController
        from basesoft.web.contrib.contact.models import Person

        self._attach_permissions_to_group(self.group, 'create_contact')


        controller = ContactController(self.session, account_id=account_id)

        departments = [u'Cleaning', u'Marketing', u'Sales', u'Director', u'IT']
        positions = [u'Assistent', u'Manager', U'Boss']
        genders = [Person.MALE, Person.FEMALE]

        countries = [self.session.query(Country).filter(Country.name==name).one().id for name in ('Netherlands', 'Romania', 'Germany', 'Viet Nam') ]

        person_data = {'first_name': u'John',
                        'middle_name': u'',
                        'last_name': u'Smith',
                        'title': u'mr',
                        'gender': Person.MALE,
                        'email': u'john@mail-test.basesoft.com',
                        'position': u'assistent',
                        'department': u'marketing',
                        'phone': u'0040123456789',
                        'mobile_phone': '',
                        'fax': '',
                        'birth_date': '1978-10-10',
                        'address_data': {'country': '181',
                                         'postcode': u"400295",
                                         'street': u" Bihorului 10",
                                         'city': u"Cluj Napoca",
                                         'state': u"Cluj"
                                        }
                       }


        rv = []
        for i in range(50):

            person_data['first_name'] = u'John %s' % i
            person_data['department'] = departments[ i % len(departments) ]
            person_data['position'] = positions[ i % len(positions) ]
            person_data['gender'] = genders[ i % len(genders) ]
            person_data['address_data']['country'] = countries[ i % len(countries) ]

            contact = controller.save_or_update_contact(id=None, person_data=person_data)
            rv.append({'contact_id': contact.id, 'person_id': contact.person.id, 'contact': contact})

        return rv


    def test_contact_cq(self):

        contacts = self._fill_collins()

        self._attach_permissions_to_group(self.group, 'view_contact')

        rpc = CustomQueryRpcHandler('basesoft.web.test.TestContactCustomQuery', self.session, self.account.id)
        fg = rpc.get_filters_and_group_by([])
        expected = {'group_by': {'country_name': 'Country', 'gender': 'Gender', 'person_department': 'Department'},
                    'filters': {'person_created': {'unique': False, 'type': 3, 'label': 'Created'},
                                'archived': {'unique': False, 'type': 4, 'label': u'Archived'},
                                'person_department': {'unique': False, 'type': 1,
                                                      'options': [{'value': u'Cleaning', 'label': u'Cleaning'},
                                                                  {'value': u'Director', 'label': u'Director'},
                                                                  {'value': u'IT', 'label': u'IT'},
                                                                  {'value': u'Marketing', 'label': u'Marketing'},
                                                                  {'value': u'Sales', 'label': u'Sales'}
                                                                  ],
                                                        'label': 'Department'},
                                'deleted': {'unique': False, 'type': 4, 'label': u'Deleted'},
                                'person_last_name': {'unique': False, 'type': 1, 'label': 'Last name'},
                                'contact_id': {'unique': False, 'type': 2, 'label': 'Contact ID'},
                                'gender': {'unique': False, 'type': 1, 'label': 'Gender'},
                                'country_name': {'unique': False, 'type': 1,
                                                'options': [{'value': 'Germany', 'label': 'Germany'},
                                                            {'value': 'Netherlands', 'label': 'Netherlands'},
                                                            {'value': 'Romania', 'label': 'Romania'},
                                                            {'value': 'Viet Nam', 'label': 'Viet Nam'}
                                                            ],
                                                'label': 'Country'},
                                'person_position': {'unique': False, 'type': 1,
                                                    'options': [{'value': u'Assistent', 'label': u'Assistent'},
                                                                {'value': u'Boss', 'label': u'Boss'},
                                                                {'value': u'Manager', 'label': u'Manager'}],
                                                    'label': 'Position'}
                                }
                    }
        self.assertEquals(fg, expected)

        # wihtout groupby
        defs = rpc.get_definitions('', [], [])
        expected = [{'column_defs': [{'is_hidden': False, 'sortable': True, 'label': 'Contact ID','is_sorted': False, 'key': 'contact_id', 'sorted_asc': True, 'type': 2, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'First name', 'is_sorted': False, 'key': 'person_first_name', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Last name', 'is_sorted': False, 'key': 'person_last_name', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Department', 'is_sorted': False, 'key': 'person_department', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Position', 'is_sorted': False, 'key': 'person_position', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Gender', 'is_sorted': False, 'key': 'gender', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Created', 'is_sorted': False, 'key': 'person_created', 'sorted_asc': True, 'type': 3, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': u'Deleted', 'is_sorted': False, 'key': 'deleted', 'sorted_asc': True, 'type': 4, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': u'Archived', 'is_sorted': False, 'key': 'archived', 'sorted_asc': True, 'type': 4, 'resizeable': True}],
                    'group_by_label': None,
                    'title': None
                    }]
        self.assertEquals(defs, expected)

        data = rpc.get_data(None, {'dir': 'asc', 'sort': None, 'startIndex': 0, 'results': 5}, {}, [])
        expected = {'totalRecords': 50, 'data': []}
        i = 0
        for contact in contacts:
            i = i+1
            if i == 6:
                break
            c = contact['contact']
            expected['data'].append(
                {'person_created': c.person.created, 'archived': 0, 'person_department': c.person.department, 'deleted': 0,
                 'gender': c.person.gender, 'contact_id': c.id, 'person_last_name': c.person.last_name,
                 'person_first_name': c.person.first_name, 'person_position': c.person.position})

        self.assertEquals(data, expected)

        # with groupby
        defs2 = rpc.get_definitions('person_department', [], [])
        expected = [{'column_defs': [{'is_hidden': False, 'sortable': True, 'label': 'Contact ID', 'is_sorted': False, 'key': 'contact_id', 'sorted_asc': True, 'type': 2, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'First name', 'is_sorted': False, 'key': 'person_first_name', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Last name', 'is_sorted': False, 'key': 'person_last_name', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Department', 'is_sorted': False, 'key': 'person_department', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Position', 'is_sorted': False, 'key': 'person_position', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Gender', 'is_sorted': False, 'key': 'gender', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Created', 'is_sorted': False, 'key': 'person_created', 'sorted_asc': True, 'type': 3, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': u'Deleted', 'is_sorted': False, 'key': 'deleted', 'sorted_asc': True, 'type': 4, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': u'Archived', 'is_sorted': False, 'key': 'archived', 'sorted_asc': True, 'type': 4, 'resizeable': True
                                    }],
                    'group_by_label': u'Cleaning',
                    'title': u'Cleaning'
                    },
                    {'column_defs': [{'is_hidden': False, 'sortable': True, 'label': 'Contact ID', 'is_sorted': False, 'key': 'contact_id', 'sorted_asc': True, 'type': 2, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'First name', 'is_sorted': False, 'key': 'person_first_name', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Last name', 'is_sorted': False, 'key': 'person_last_name', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Department', 'is_sorted': False, 'key': 'person_department', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Position', 'is_sorted': False, 'key': 'person_position', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Gender', 'is_sorted': False, 'key': 'gender', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Created', 'is_sorted': False, 'key': 'person_created', 'sorted_asc': True, 'type': 3, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': u'Deleted', 'is_sorted': False, 'key': 'deleted', 'sorted_asc': True, 'type': 4, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': u'Archived', 'is_sorted': False, 'key': 'archived', 'sorted_asc': True, 'type': 4, 'resizeable': True
                                    }],
                    'group_by_label': u'Director',
                    'title': u'Director'
                    },
                    {'column_defs': [{'is_hidden': False, 'sortable': True, 'label': 'Contact ID', 'is_sorted': False, 'key': 'contact_id', 'sorted_asc': True, 'type': 2, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'First name', 'is_sorted': False, 'key': 'person_first_name', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Last name', 'is_sorted': False, 'key': 'person_last_name', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Department', 'is_sorted': False, 'key': 'person_department', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Position', 'is_sorted': False, 'key': 'person_position', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Gender', 'is_sorted': False, 'key': 'gender', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Created', 'is_sorted': False, 'key': 'person_created', 'sorted_asc': True, 'type': 3, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': u'Deleted', 'is_sorted': False, 'key': 'deleted', 'sorted_asc': True, 'type': 4, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': u'Archived', 'is_sorted': False, 'key': 'archived', 'sorted_asc': True, 'type': 4, 'resizeable': True
                                    }],
                     'group_by_label': u'IT',
                     'title': u'IT'
                    },
                     {'column_defs': [{'is_hidden': False, 'sortable': True, 'label': 'Contact ID', 'is_sorted': False, 'key': 'contact_id', 'sorted_asc': True, 'type': 2, 'resizeable': True},
                                      {'is_hidden': False, 'sortable': True, 'label': 'First name', 'is_sorted': False, 'key': 'person_first_name', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                      {'is_hidden': False, 'sortable': True, 'label': 'Last name', 'is_sorted': False, 'key': 'person_last_name', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                      {'is_hidden': False, 'sortable': True, 'label': 'Department', 'is_sorted': False, 'key': 'person_department', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                      {'is_hidden': False, 'sortable': True, 'label': 'Position', 'is_sorted': False, 'key': 'person_position', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                      {'is_hidden': False, 'sortable': True, 'label': 'Gender', 'is_sorted': False, 'key': 'gender', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                      {'is_hidden': False, 'sortable': True, 'label': 'Created', 'is_sorted': False, 'key': 'person_created', 'sorted_asc': True, 'type': 3, 'resizeable': True},
                                      {'is_hidden': False, 'sortable': True, 'label': u'Deleted', 'is_sorted': False, 'key': 'deleted', 'sorted_asc': True, 'type': 4, 'resizeable': True},
                                      {'is_hidden': False, 'sortable': True, 'label': u'Archived', 'is_sorted': False, 'key': 'archived', 'sorted_asc': True, 'type': 4, 'resizeable': True
                                    }],
                    'group_by_label': u'Marketing',
                    'title': u'Marketing'
                    },
                    {'column_defs': [{'is_hidden': False, 'sortable': True, 'label': 'Contact ID', 'is_sorted': False, 'key': 'contact_id', 'sorted_asc': True, 'type': 2, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'First name', 'is_sorted': False, 'key': 'person_first_name', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Last name', 'is_sorted': False, 'key': 'person_last_name', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Department', 'is_sorted': False, 'key': 'person_department', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Position', 'is_sorted': False, 'key': 'person_position', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Gender', 'is_sorted': False, 'key': 'gender', 'sorted_asc': True, 'type': 1, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': 'Created', 'is_sorted': False, 'key': 'person_created', 'sorted_asc': True, 'type': 3, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': u'Deleted', 'is_sorted': False, 'key': 'deleted', 'sorted_asc': True, 'type': 4, 'resizeable': True},
                                     {'is_hidden': False, 'sortable': True, 'label': u'Archived', 'is_sorted': False, 'key': 'archived', 'sorted_asc': True, 'type': 4, 'resizeable': True
                                    }],
                    'group_by_label': u'Sales',
                    'title': u'Sales'}]
        self.assertEquals(defs2, expected)

        data2 = rpc.get_data(['person_department', u'Cleaning'], {'dir': 'asc', 'sort': None, 'startIndex': 0, 'results': 5}, {}, [])
        expected = {'totalRecords': 10, 'data': []}
        i = 0
        for contact in contacts:
            c = contact['contact']
            if c.person.department != u'Cleaning':
                continue
            i = i+1
            if i == 6:
                break
            expected['data'].append(
                {'person_created': c.person.created, 'archived': 0, 'person_department': c.person.department, 'deleted': 0,
                 'gender': c.person.gender, 'contact_id': c.id, 'person_last_name': c.person.last_name,
                 'person_first_name': c.person.first_name, 'person_position': c.person.position})
        self.assertEquals(data2, expected)

    def test_domain_country_cq(self):

        license = self._create_license()
        account = self._create_active_account(license=license)

        self._attach_permissions_to_group(account.groups[0].group, 'view_contact', 'create_contact')

        self._fill_collins(account.id)

        rpc = CustomQueryRpcHandler('basesoft.web.test.TestContactCustomQuery', self.session, account.id)
        filters = rpc.get_filters_and_group_by([])
        data = rpc.get_data(None, {'dir': 'asc', 'sort': None, 'startIndex': 0, 'results': 5}, {}, [])
        self.assertEquals(len(data['data']), 5)

        for filter in filters['filters']['country_name']['options']:
            self.assertTrue(filter['label'] in ('Netherlands', 'Romania', 'Germany', 'Viet Nam'))

        dc = DomainCountry()
        dc.domain = license.domain
        dc.country = self.session.query(Country).filter(Country.name=='Viet Nam').one()
        dc.active = True
        dc.name = 'Vietnam'
        self.session.add(dc)
        self.session.flush()

        rpc = CustomQueryRpcHandler('basesoft.web.test.TestContactCustomQuery', self.session, account.id)
        filters = rpc.get_filters_and_group_by([])

        for filter in filters['filters']['country_name']['options']:
            self.assertTrue(filter['label'] in ('Netherlands', 'Romania', 'Germany', 'Vietnam'), 'invalid country "%s"' % filter['label'])

        # group by
        data = rpc.get_definitions("country_name", {}, {}, True)
        data = rpc.get_data(("country_name", 'Country'), {'dir': 'asc', 'sort': None, 'startIndex': 0, 'results': 5}, {}, [])

    def test_sql_custom_query(self):

        _license = self._create_license()
        account = self._create_active_account(license=_license)

        rpc = CustomQueryRpcHandler('basesoft.web.test.TestSQLCustomQuery', self.sa_session, account.id)
        filters = rpc.get_filters_and_group_by([])
        self.assertEqual(len(filters['filters']), 1)

        cq = TestSQLCustomQuery(self.sa_session, account_id=account.id)
        sql = cq._build_query(split_by=None, sort=None, asc=None, offset=0, limit=25)
        self.assertEqual(sql, """SELECT `c`.`id` as `contact_id`, `p`.`first_name` as `first_name`, `p`.`gender` as `gender` """+
                         """\nFROM `tblContact` as `c`
INNER JOIN `tblPerson` as `p` ON `c`.`person_id` = `p`.`id`
GROUP BY `c`.`id`
LIMIT 0, 25""")

    def test_union_custom_query(self):
        _license = self._create_license()
        account = self._create_active_account(license=_license)

        cq = TestSQLUnionCustomQuery(self.sa_session, account_id=account.id)

        # test simple union query
        sql, params, count = cq._get_sql(split_by=None, sort=None, asc=None, offset=0, limit=25, applied_filters={}, static_filters=[])
        self.assertEqual(len(params), 0)
        self.assertEqual(sql, """(SELECT `c`.`id` as `union_id` \nFROM `tblContact` as `c`) UNION (SELECT `p`.`id` as `union_id` \nFROM `tblPerson` as `p`) \nLIMIT 0, 25""")

        # test union query with sort
        sql, params, count = cq._get_sql(split_by=None, sort='union_id', asc=True, offset=0, limit=25, applied_filters={}, static_filters=[])
        self.assertEqual(sql, """(SELECT `c`.`id` as `union_id` \nFROM `tblContact` as `c`) UNION (SELECT `p`.`id` as `union_id` \nFROM `tblPerson` as `p`) \nORDER BY union_id ASC \nLIMIT 0, 25""")

        # test union query with applied filters
        sql, params, count = cq._get_sql(split_by=None, sort=None, asc=None, offset=0, limit=25, applied_filters={
            'union_id': {
                'operator': FILTER_EQUAL,
                'values': [1]
            }
        }, static_filters=[])
        self.assertEqual(len(params), 2)
        self.assertEqual(sql, """(SELECT `c`.`id` as `union_id` \nFROM `tblContact` as `c`\nWHERE `c`.`id` IN (:param0)) UNION (SELECT `p`.`id` as `union_id` \nFROM `tblPerson` as `p`\nWHERE `p`.`id` IN (:param1)) \nLIMIT 0, 25""")

    def test_false_field(self):
        cq = TestFalseFieldCustomQuery(self.sa_session, account_id=self.account.id)

        # check that field is not show when not separated
        data = cq.get_data(None, None, 'asc', 0, 5, {}, {})
        self.assertFalse(data['data'][0].has_key('iso_a2'))

        # check that field exists when cq separated
        cq.set_separated(True)
        data = cq.get_data(None, None, 'asc', 0, 5, {}, {})
        self.assertTrue(data['data'][0].has_key('iso_a2'))


# below is for running the test from eclipse
if __name__ == '__main__':
    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_custom_query.py', 'basesoft.web.test.unit.test_custom_query.CustomQueryTestCase.test_domain_country_cq']
    run()
    sys.exit()
