# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.
from basesoft.web.test.unit import iqportal_unittest

from basesoft.web.core.exceptions import SecurityException,\
    PermissionDeniedException, PublicUserException
from basesoft.web.contrib.dashboard.controllers import DashboardStateController, TabStateController, ColumnStateController
from basesoft.web.contrib.dashboard.models import DashboardAccount,\
    DashboardGroup, Dashboard
from basesoft.web.core.models import PrivilegeObject
from basesoft.web.contrib.license.controllers import LicenseController
from basesoft.web.contrib.account.controllers import AccountController
from basesoft.web.core.util.controllers import StateManager

def get_state():
    return {
        'dashboard': {
            'name': u'dashboard 1',
            'objectName': u'Dashboard',
            'dbID': None,
            'domID': u'bs-dashboard-container',
            'activeIndex': 0,
            'children': {
                'tabs': [
                    {'objectName': u'DashboardTab',
                     'dbID': None,
                     'domID':u'yui-gen24',
                     'label': u'Tab 0',
                     'children': {
                         'blocks': [
                             {'objectName': u'DashboardBlock',
                              'dbID': None,
                              'domID': u'yui-gen26',
                              'children': {
                                  'columns': [
                                      {'objectName': u'DashboardColumn',
                                       'dbID': None,
                                       'domID': u'yui-gen28',
                                       'width': 50,
                                       'children': {
                                           'widgets': [
                                               {'objectName': u'TestWidget',
                                                'loaderName': 'test_dashboard',
                                                'title': u'TestWidget',
                                                'dbID': None,
                                                'domID': u'yui-gen34'}]
                                           }
                                       },
                                       {'objectName': u'DashboardColumn',
                                        'dbID': None,
                                        'domID': u'yui-gen32',
                                        'width': 50,
                                        'children': {
                                            'widgets': [
                                                {'objectName': u'TestWidget',
                                                 'loaderName': 'test_dashboard',
                                                 'title': u'TestWidget 2',
                                                 'dbID': None,
                                                 'domID': u'yui-gen37'}]
                                            }
                                        }]
                                  }
                              }]
                         }
                     }]
                }
            }
        }

class TestCase(iqportal_unittest.IQPortalTestCase):
    """
    Uber simple tests for now, cuz we currently just save the JSON encoded
    string to the account able in the database.
    """
    def setUp(self):
        """
        Create the account and a controller
        """
        self.connect_db()

        self.account = self._create_active_account()
        self.group = self.account.groups[0].group

        self._attach_permissions_to_group(self.group, 'view_dashboard')

        self.manager = StateManager(None, self.account.id, None)
        self.controller = DashboardStateController(self.session, self.manager)


    def _get_state(self):
        """
        Get a state dictionary for testing purposes...
        """
        return get_state()

    def test_save_and_load_state(self):
        """
        The loaded state should be the same as the state in the _get_state method
        that we just saved.
        """
        state = self._get_state()

        # test security exception
        self.assertRaises(PermissionDeniedException, self.controller.save_state, state)

        # attach the save_dashboard permission
        # save the state to the provided account on the controller
        self._attach_permissions_to_group(self.group, 'save_dashboard')
        self.controller.save_state(state)

        # FIXME: test loading

        # load the state for the account from the database.
        #loaded_state = self.controller.load_state()
        # assert that they are equal
        #self.assertEquals(loaded_state, state)

    def test_copy_to_dashboard(self):
        """
        Tests copy tab and widget to another dashboard
        """
        tab_controller = TabStateController(self.session, self.manager)
        column_controller = ColumnStateController(self.session, self.manager)

        self._attach_permissions_to_group(self.group, 'save_dashboard')
        state1 = self._get_state()
        id_map = self.controller.save_state(state1)
        d1_id = int(id_map[state1['dashboard']['domID']])
        d1_tab1_id = int(id_map[state1['dashboard']['children']['tabs'][0]['domID']])
        d1_widget1_id = int(id_map[state1['dashboard']['children']['tabs'][0]['children']['blocks'][0]['children']['columns'][0]['children']['widgets'][0]['domID']])

        state2 = self._get_state()
        id_map = self.controller.save_state(state2)
        d2_id = int(id_map[state2['dashboard']['domID']])

        tab_state = state1['dashboard']['children']['tabs'][0]
        tab_state['dbID'] = d1_tab1_id
        # try to copy a tab to same dashboard
        self.assertRaises(SecurityException, tab_controller.copy_tab_to_dasboard, tab_state, d1_id)
        d2 = tab_controller.copy_tab_to_dasboard(tab_state, d2_id)
        self.assertEqual(len(d2.tabs), 2)
        d2_tab2_id = d2.tabs[1].id

        widget_state = state1['dashboard']['children']['tabs'][0]['children']['blocks'][0]['children']['columns'][0]['children']['widgets'][0]
        widget_state['dbID'] = d1_widget1_id
        # try to copy a widget to same dashboard
        self.assertRaises(SecurityException, column_controller.copy_widget_to_tab, widget_state, d1_tab1_id)
        tab = column_controller.copy_widget_to_tab(widget_state, int(d2_tab2_id))
        self.assertEqual(len(tab.blocks[0].columns[0].widgets), 2)

    def test_dashboard_share_removal(self):
        self._attach_permissions_to_group(self.group, 'save_dashboard')
        self._attach_permissions_to_group(self.group, 'share_dashboard')
        self._attach_permissions_to_group(self.group, 'view_license')

        license = self.account.groups[0].group.license
        group = self.account.groups[0].group

        # save a dashboard for me
        state = self._get_state()
        map = self.controller.save_state(state)
        dashboard_id = map['bs-dashboard-container']
        dashboard = self.session.query(Dashboard).get(dashboard_id)

        c = DashboardStateController(self.session, StateManager(None, self.account.id, None))

        # scenario 1: we can't delete our own group.
        self.assertRaises(PublicUserException, c.remove_share, dashboard.id, group.id)

        # scenario 2: another user in different group gets dashboard access trough group

        # add a user in another group on the same license
        account3 = self._create_active_account(license=license)
        self._attach_permissions_to_group(account3.groups[0].group, 'view_dashboard')
        c3 = DashboardStateController(self.session, StateManager(None, account3.id, None))
        self.assertFalse(dashboard_id in [d.id for d in c3.get_dashboards()])

        c.save_sharing(dashboard, account3.groups[0].group, viewable=True, editable=False, account_ids=[])

        self.assertTrue(dashboard_id in [d.id for d in c3.get_dashboards()])

        # remove the group
        c.remove_share(dashboard.id, account3.groups[0].group.id)

        self.assertFalse(dashboard_id in [d.id for d in c3.get_dashboards()])

    def test_dashboard_sharing(self):

        self._attach_permissions_to_group(self.group, 'save_dashboard')
        self._attach_permissions_to_group(self.group, 'share_dashboard')
        self._attach_permissions_to_group(self.group, 'view_license')

        license = self.account.groups[0].group.license
        group = self.account.groups[0].group

        # save a dashboard for me
        state = self._get_state()
        map = self.controller.save_state(state)
        dashboard_id = map['bs-dashboard-container']
        dashboard = self.session.query(Dashboard).get(dashboard_id)

        # assert the user has access
        c = DashboardStateController(self.session, StateManager(None, self.account.id, None))
        self.assertTrue(dashboard_id in [d.id for d in c.get_dashboards()])

        # scenario 1: another user in same group

        # assert that another user in the same group does not have access to the dashboard
        # because its a personal dashboard
        account2 = self._create_active_account(license=license, group=group)
        c2 = DashboardStateController(self.session, StateManager(None, account2.id, None))
        self.assertFalse(dashboard_id in [d.id for d in c2.get_dashboards()])

        # assert that if this user gets the dashboard assigned personally he does have access
        c.save_sharing(dashboard, group, viewable=True, editable=True, account_ids=[account2.id])
        self.assertTrue(dashboard_id in [d.id for d in c2.get_dashboards()])

        # he can edit
        c2.assert_privileges_on_object(dashboard, PrivilegeObject.PRIV_UPDATE)

        # scenario 2: another user in different group gets dashboard access trough group

        # add a user in another group on the same license
        account3 = self._create_active_account(license=license)
        self._attach_permissions_to_group(account3.groups[0].group, 'view_dashboard')
        c3 = DashboardStateController(self.session, StateManager(None, account3.id, None))

        # he should not have access yet
        self.assertFalse(dashboard_id in [d.id for d in c3.get_dashboards()])

        # let user1 grant to the group of user3
        # and now he got access

        c.grant_privilege_to(dashboard, [PrivilegeObject.PRIV_VIEW], group=True, target_group=account3.groups[0].group)
        self.assertTrue(dashboard_id in [d.id for d in c3.get_dashboards()])

        # he shouldnt be able to edit
        self.assertRaises(PermissionDeniedException, c3.assert_privileges_on_object, dashboard, PrivilegeObject.PRIV_UPDATE)

        # other users in the same group also have access to the dashboard
        account4 = self._create_active_account(license=license, group=account3.groups[0].group)
        c4 = DashboardStateController(self.session, StateManager(None, account4.id, None))
        self.assertTrue(dashboard_id in [d.id for d in c4.get_dashboards()])

        # scenario 3: user in another group gets personal dashboard shared

        # create new account and let original owner share the dashboard to that account
        account5 = self._create_active_account(license=license)
        self._attach_permissions_to_group(account5.groups[0].group, 'view_dashboard')

        # no access
        c5 = DashboardStateController(self.session, StateManager(None, account5.id, None))
        self.assertFalse(dashboard_id in [d.id for d in c5.get_dashboards()])

        # yes access
        c.save_sharing(dashboard, account5.groups[0].group, viewable=True, editable=False, account_ids=[account5.id])
        self.assertTrue(dashboard_id in [d.id for d in c5.get_dashboards()])

        # but not edit
        self.assertRaises(PermissionDeniedException, c5.assert_privileges_on_object, dashboard, PrivilegeObject.PRIV_UPDATE)

        # reshare.. but now yes edit
        c.save_sharing(dashboard, account5.groups[0].group, viewable=True, editable=True, account_ids=[account5.id])
        c5.assert_privileges_on_object(dashboard, PrivilegeObject.PRIV_VIEW, PrivilegeObject.PRIV_UPDATE)


    def test_user_group_move(self):

        self._attach_permissions_to_group(self.group, 'save_dashboard', 'share_dashboard', 'view_license', 'create_account', 'update_account')

        account = self.account
        license = self.account.groups[0].group.license
        group = self.account.groups[0].group

        # save dashboard for user
        dc = DashboardStateController(self.session, StateManager(None, self.account.id, None))
        dc.save_state(self._get_state())

        # assert he only has one
        dashboards = dc.get_dashboards()
        self.assertEqual(len(dashboards), 1)
        dashboard = dashboards[0]

        # load account state
        ac = AccountController(self.session, None, account.id)
        account_state = ac.get_account_state(account.id)

        # make a new group
        lc = LicenseController(self.session, None, account.id)
        new_group = lc.save_or_update_group(None, 'new group', group.license)
        self._attach_permissions_to_group(new_group, 'view_dashboard')

        # assign account to new group
        del account_state['person']['dbID'] # this is needed to be able to save state again
        account_state['account']['groups'] = [new_group.id]
        account_state['account']['default_group'] = new_group.id
        ac.save_account_from_state(account_state, account_id=account.id)

        # assert new group is saved
        self.assertEqual(account.groups[0].group.id, new_group.id)

        # assert dashboard has been moved
        dc = DashboardStateController(self.session, StateManager(None, self.account.id, None))
        self.assertTrue(dashboard.id in [d.id for d in dc.get_dashboards()])


    def test_user_group_move_2(self):

        self._attach_permissions_to_group(self.group, 'save_dashboard', 'share_dashboard', 'view_license', 'create_account', 'update_account')

        license = self.account.groups[0].group.license
        group = self.account.groups[0].group

        account = self.account

        # save a dashboard for me
        state = self._get_state()
        map = self.controller.save_state(state)
        dashboard_id = map['bs-dashboard-container']
        dashboard = self.session.query(Dashboard).get(dashboard_id)

        # make a new account in the same group
        account2 = self._create_active_account(license=license, group=group)

        # share the dashboard with that new account
        c = DashboardStateController(self.session, StateManager(None, self.account.id, None))
        c.save_sharing(dashboard, group, viewable=True, editable=True, account_ids=[self.account.id, account2.id])

        # assert the dashboard is shared
        dc = DashboardStateController(self.session, StateManager(None, self.account.id, None))
        self.assertTrue(len(dc.get_dashboards()) == 1)
        self.assertTrue(dc.get_dashboards()[0].is_shared)

        # load account state
        ac = AccountController(self.session, None, account.id)
        account2_state = ac.get_account_state(account2.id)

        # make a new group
        lc = LicenseController(self.session, None, account.id)
        new_group = lc.save_or_update_group(None, 'new group', group.license)
        self._attach_permissions_to_group(new_group, 'view_dashboard')

        # move the second user to a new group
        del account2_state['person']['dbID'] # this is needed to be able to save state again
        account2_state['account']['groups'] = [new_group.id]
        account2_state['account']['default_group'] = new_group.id
        ac.save_account_from_state(account2_state, account_id=account2.id)

        # expect that first user has access to dashboard
        dc = DashboardStateController(self.session, StateManager(None, self.account.id, None))
        self.assertTrue(dashboard.id in [d.id for d in dc.get_dashboards()])

        # and also the second user
        dc = DashboardStateController(self.session, StateManager(None, account2.id, None))
        self.assertTrue(dashboard.id in [d.id for d in dc.get_dashboards()])

        # assert that the second user also does have a reference to AccountDashboard anymore
        self.assertEqual(
            self.session.query(DashboardAccount).filter(DashboardAccount.dashboard==dashboard) \
                .filter(DashboardAccount.account==account2).count(),
            1)

        # assert the dashboard is still shared
        dc = DashboardStateController(self.session, StateManager(None, self.account.id, None))
        self.assertTrue(len(dc.get_dashboards()) == 1)
        self.assertTrue(dc.get_dashboards()[0].is_shared)

    def test_mark_as_deleted_event(self):

        self._attach_permissions_to_group(self.group, 'save_dashboard')
        state = self._get_state()
        id_map = self.controller.save_state(state)
        dashboard_id = id_map['bs-dashboard-container']
        dashboard = self.session.query(Dashboard).get(dashboard_id)

        self.sa_session.expire(self.account)
        accountdb = [ad for ad in self.account.dashboards]

        self.assertTrue(dashboard in [ad.dashboard for ad in accountdb])

        self.assertRaises(PermissionDeniedException, self.controller.mark_as_deleted, Dashboard, [dashboard.id])
        self._attach_permissions_to_group(self.group, 'mark_as_deleted')
        self.controller.mark_as_deleted(Dashboard, [dashboard.id])

        self.sa_session.expire(self.account)
        accountdb = [ad for ad in self.account.dashboards]

        self.assertTrue(dashboard in [ad.dashboard for ad in accountdb])




