# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.
import os
from hashlib import sha256 as sha
import random

from basesoft.web.contrib.notification.controllers import NotificationController
from basesoft.web.test.controllers import MockFileController
from basesoft.web.test.schedule import ImportFileTask
from basesoft.web.test.unit import iqportal_unittest
from django.conf import settings

from basesoft.web.contrib.file.controllers import UploadFileController, ProcessFileController
from basesoft.web.contrib.file.models import ExcelFile, ImportFileType, ImportFile, File
from basesoft.web.core.exceptions import PermissionDeniedException

from django.core.files.uploadedfile import SimpleUploadedFile
from twisted.trial.unittest import SkipTest

from sqlalchemy.sql.expression import and_


class UploadFileControllerTestCase(iqportal_unittest.IQPortalTestCase):
    """
    This will test the File Controller - upload part.
    All individual *public* methods are tested separately.
    """

    def setUp(self):
        """
        The script for reseting the db and creating all the tables
        """
        self.connect_db()

        # create the file
        self.filename = "test_file_upload.txt"
        self.file = {'filedata': SimpleUploadedFile(self.filename, 'test')}
        # create an instance of the controller. It is used in all test methods.
        self.controller = UploadFileController(self.session, None)

    def test_save_temp_files(self):
        """
        tests saving an uploaded file in the temp db and temp destination
        """

        file = self.controller.save_temp_files(self.file)[0]

        self.assertEqual(file.file_name, self.filename)
        # lets see if the file is in the right destination
        self.assertTrue(os.path.exists(os.path.join(settings.TMP_FILE_REPOSITORY,
                                                         file.file_hash)))

        # i remove here the uploaded file from the destination cuz tearDown
        # won't know about its hash
        os.remove(os.path.join(settings.TMP_FILE_REPOSITORY, file.file_hash))

    def test_delete_temp_files(self):
        """
        test removing a temp file
        """
        file = self.controller.save_temp_files(self.file)[0]
        self.controller.delete_temp_files([file], True)
        # test if removed from db
        self.assertEqual(len(self.controller.get_temp_files(id = file.id)), 0)
        # test if removed from the destination
        self.assertFalse(os.path.exists(os.path.join(settings.TMP_FILE_REPOSITORY, file.file_hash)))

class ProcessFileControllerTestCase(iqportal_unittest.IQPortalTestCase):
    """
    This will test the File Controller - upload part.
    All individual *public* methods are tested separately.
    """

    def setUp(self):
        """
        The script for reseting the db and creating all the tables
        """
        self.connect_db()

        #create an account so we can check permission with it
        account = self._create_active_account()
        self.group = account.groups[0].group

        # create the file
        self.filename = "test_file_upload.txt"

        #if this destination doesnt exist, it will create it
        self.destination_dir = "/tmp/temp"

        self.file = {'filedata': SimpleUploadedFile(self.filename, 'test')}
        self.upload_controller = UploadFileController(self.session)
        # create an instance of the controller. It is used in all test methods.
        self.process_controller = ProcessFileController(self.session, account_id=account.id)

    def test_mock_double_click_upload_form(self):
        # This mock tests ticket #4067
        self._attach_permissions_to_group(self.group, 'create_file', 'view_file', 'update_file')
        myfile = self._create_uploaded_file('test_file.pdf', 'Australians do not have sex. Australians mate.')
        f0 = UploadFileController().get_hash_temp_files(myfile.file_hash)[0]
        f1 = self.process_controller.save_files_from_state({'hashes': myfile.file_hash, 'category': u'ticket_4067'})[0]
        self.assertNotEqual(f0, f1)
        f2 = UploadFileController().get_hash_temp_files(myfile.file_hash)[0]
        self.assertEqual(f1, f2)
        f3 = self.process_controller.save_files_from_state({'hashes': myfile.file_hash, 'category': u'ticket_4067'})[0]
        self.assertEqual(f2, f3)

    def test_save_from_state(self):
        """
        """
        self._attach_permissions_to_group(self.group, 'create_file', 'view_file')
        tmp_file = self.upload_controller.save_temp_files(self.file)[0]

        self.process_controller.save_files_from_state({'hashes': tmp_file.file_hash, 'category': u'mycategory'})
        file = self.process_controller.get_file_by_hash(tmp_file.file_hash)

        self.assertEqual(file.file_name, self.filename)
        self.assertEqual(file.category.label, u'mycategory')

    def test_save_and_update_category(self):
        """
        """
        self.assertRaises(PermissionDeniedException, self.process_controller.save_or_update_file_category, None, 'test', None)

        self._attach_permissions_to_group(self.group, 'create_file')

        fc = self.process_controller.save_or_update_file_category(None, u'test', None)
        self.assertEqual(fc.label, u'test')

        self.assertRaises(PermissionDeniedException, self.process_controller.save_or_update_file_category, fc.id, 'tested')

        self._attach_permissions_to_group(self.group, 'update_file')

        fc = self.process_controller.save_or_update_file_category(fc.id, u'tested')
        self.assertEqual(fc.label, u'tested')

    def test_save_files_from_temp(self):
        """
        tests saving(processing) temp uploaded files
        """
        self._attach_permissions_to_group(self.group, 'create_file', 'view_file')
        fc = self.process_controller.save_or_update_file_category(None, u'test', None)

        #first we need a tmp uploaded file
        tmp_file = self.upload_controller.save_temp_files(self.file)[0]

        #now save them
        file = self.process_controller.save_files_from_temp([tmp_file], fc)[0]
        self.assertEqual(file.file_name, self.filename)

        # lets see if the file is in the right destination
        self.assertTrue(os.path.exists(os.path.join(settings.FILE_REPOSITORY, file.file_hash)))

        # check if the temp file was also removed
        self.assertEqual(len(self.upload_controller.get_temp_files(id = tmp_file.id)), 0)

        # test if removed from the destination
        self.assertFalse(os.path.exists(os.path.join(settings.TMP_FILE_REPOSITORY, file.file_hash)))

        # i remove here the uploaded file from the destination cuz tearDown
        # won't know about its hash
        os.remove(os.path.join(settings.FILE_REPOSITORY, file.file_hash))

    def test_save_or_update_file(self):
        """
        test the method for saving or updating a file
        (this has nothing to do with the temp uploaded files)
        """

        hash = sha(str(random.random())).hexdigest()[:8]

        #create a source file
        source_path = os.path.join(settings.TMP_FILE_REPOSITORY,  hash)
        f = open(source_path, 'w')
        f.write("a")
        f.close()

        self._attach_permissions_to_group(self.group, 'create_file', 'view_file', 'update_file')

        fc = self.process_controller.save_or_update_file_category(None, u'test', None)

        file = self.process_controller.save_or_update_file(id=None,
                                            file_name=self.filename,
                                            file_hash=hash,
                                            remove_source = False,
                                            category = fc)
        self.assertEqual(file.file_name, self.filename)
        self.assertEqual(file.file_hash, hash)
        # lets see if the file is in the right destination
        self.assertTrue(os.path.exists(os.path.join(settings.FILE_REPOSITORY, hash)))

        # check that the source file was not removed
        self.assertTrue(os.path.exists(source_path))

        # now update the name
        ufilename = "test_file_update.txt"
        ufile = self.process_controller.save_or_update_file(id=file.id,
                                            file_name=ufilename,
                                            file_hash=hash,
                                            remove_source = True,
                                            category = fc)
        self.assertEqual(ufile.file_name, ufilename)
        self.assertEqual(ufile.file_hash, hash)
        # lets see if the file is in the right destination
        self.assertTrue(os.path.exists(os.path.join(settings.FILE_REPOSITORY, hash)))

        # i remove here the saved file from the destination and the source file
        os.remove(os.path.join(settings.FILE_REPOSITORY, hash))

    def test_save_or_update_file_from_content(self):
        """
        test the method for saving or updating a file from a stream
        (this has nothing to do with the temp uploaded files)
        """
        self._attach_permissions_to_group(self.group, 'create_file', 'view_file', 'update_file')
        fc = self.process_controller.save_or_update_file_category(None, u'test', None)

        hash = sha(str(random.random())).hexdigest()[:8]

        file = self.process_controller.save_or_update_file_from_content(id=None,
                                            file_name=self.filename,
                                            file_hash=hash,
                                            data="some simple test data",
                                            category = fc)
        self.assertEqual(file.file_name, self.filename)
        self.assertEqual(file.file_hash, hash)

        # lets see if the file is0 in the right destination
        self.assertTrue(os.path.exists(os.path.join(settings.FILE_REPOSITORY, hash)))

        # now update the name
        ufilename = "test_file_update.txt"
        ufile = self.process_controller.save_or_update_file_from_content(id=file.id,
                                            file_name=ufilename,
                                            file_hash=hash,
                                            data="some simple test data",
                                            category=fc)
        self.assertEqual(ufile.file_name, ufilename)
        self.assertEqual(ufile.file_hash, hash)

        # lets see if the file is in the right destination
        self.assertTrue(os.path.exists(os.path.join(settings.FILE_REPOSITORY, hash)))

        # i remove here the saved file from the destination and the source file
        os.remove(os.path.join(settings.FILE_REPOSITORY, hash))

    def test_save_or_update_file_from_content_with_object(self):
        """
        test the method for saving or updating a file from a stream with the object parameter
        (this has nothing to do with the temp uploaded files)
        """
        file_obj = ExcelFile()

        self._attach_permissions_to_group(self.group, 'create_file', 'view_file')
        fc = self.process_controller.save_or_update_file_category(None, u'test', None)

        hash = sha(str(random.random())).hexdigest()[:8]

        file = self.process_controller.save_or_update_file_from_content(id=None,
                                            file_name=self.filename,
                                            file_hash=hash,
                                            data="some simple test data",
                                            file_obj=file_obj,
                                            category=fc)
        self.assertEqual(file.file_name, self.filename)
        self.assertEqual(file.file_hash, hash)

        # lets see if the file is in the right destination
        self.assertTrue(os.path.exists(os.path.join(settings.FILE_REPOSITORY, hash)))
        os.remove(os.path.join(settings.FILE_REPOSITORY, hash))

    def test_delete_files(self):
        """
        test removing a temp file
        """

        hash = sha(str(random.random())).hexdigest()[:8]

        #create a source file
        source_path = os.path.join(settings.TMP_FILE_REPOSITORY, hash)
        f = open(source_path, 'w')
        f.write("a")
        f.close()

        self._attach_permissions_to_group(self.group, 'create_file', 'view_file', 'delete_file')

        fc = self.process_controller.save_or_update_file_category(None, u'test', None)

        #creaate file to delete
        file = self.process_controller.save_or_update_file(id=None,
                                            file_name=self.filename,
                                            file_hash=hash,
                                            remove_source = True,
                                            category=fc)

        raise SkipTest('ticket #1863')

        self.process_controller.delete_files([file])

        # test if removed from db
        self.assertEqual(len(self.process_controller.get_files(id = file.id)), 0)

        # removal of OS file should be done by the File Objects Garbage Collector callback.


class TestCase(iqportal_unittest.IQPortalTestCase):

    def setUp(self):

        self.connect_db()

        self.account = self._create_active_account()

    def test_import_file(self):
        # create csv file
        headers = ['column1', 'column2', 'column3']
        row1 = ['row11', 'row12', 'row13']
        row2 = ['row21', 22, 'row23']

        import csv
        with open('test_mock_file.csv', 'wb') as f:
            wr = csv.writer(f, dialect=csv.excel, delimiter=',')
            wr.writerow(headers)
            wr.writerow(row1)
            wr.writerow(row2)

        # import file
        mfc = MockFileController(self.sa_session, account_id=self.account.id)
        rv, failures = mfc.import_file('test_mock_file.csv', None)

        # assert data is saved
        headers.insert(0, 'Line number')
        self.assertEqual(len(rv), 1)
        self.assertEqual(len(failures), 1)
        self.assertTrue(all(val in row1 for val in rv[0]))
        self.assertTrue(all(h in headers for h in mfc._file_headers))

        # create failure file
        file_path, file_name = mfc.save_failure_file('test_mock_file.csv', failures)
        rows = mfc.get_rows(file_path)

        self.assertEqual(len(rows), 1)
        self.assertEqual(rows[0][4], 'Column should be a string')

        os.remove('test_mock_file.csv')
        os.remove(file_path)

    def test_save_file_for_background_import(self):
        # create csv file
        headers = ['column1', 'column2', 'column3']
        row1 = ['row11', 'row12', 'row13']
        row2 = ['row21', 22, 'row23']

        self._attach_permissions_to_group(self.account.default_group, 'create_file', 'view_file')

        import csv
        with open('test_mock_file.csv', 'wb') as f:
            wr = csv.writer(f, dialect=csv.excel, delimiter=',')
            wr.writerow(headers)
            wr.writerow(row1)
            wr.writerow(row2)

        with file('test_mock_file.csv') as f:
            data = f.read()
        os.remove('test_mock_file.csv')
        myfile = self._create_uploaded_file('test_mock_file.csv', data)

        # import file
        mfc = MockFileController(self.sa_session, account_id=self.account.id)
        mfc.MAX_RECORDS_FOR_INSTANT_IMPORT = 1
        rv, failures = mfc.import_file(myfile, None, file_name='test_mock_file')

        # get the saved file
        f = self.sa_session.query(ImportFile).join(ImportFileType)\
            .filter(and_(ImportFileType.name == 'basesoft.web.test', ImportFile.status == ImportFile.STATUS_NEW))\
            .order_by(ImportFile.last_updated.desc()).first()
        self.assertEqual(f.file_name, 'test_mock_file.csv')
        self.assertEqual(f.settings, '{"file_name": "test_mock_file"}')

        import time
        time.sleep(1)

        # Trigger task
        ImportFileTask().execute()

        f = self.sa_session.query(File).order_by(File.id.desc()).first()

        fi = self.sa_session.query(ImportFile).join(ImportFileType)\
            .filter(and_(ImportFileType.name == 'basesoft.web.test', ImportFile.status == ImportFile.STATUS_SUCCESS))\
            .order_by(ImportFile.last_updated.desc()).first()
        self.assertEqual(fi.failure_file.id, f.id)

        self.assertEqual(f.file_name, 'failures_test_mock_file.csv')
        file_path = os.path.join(settings.FILE_REPOSITORY, f.file_hash)
        rows = mfc.get_rows(file_path)

        # failure file has two lines - headers and 1 failed row
        self.assertEqual(len(rows), 2)
        self.assertEqual(rows[1][4], 'Column should be a string')

        # check notification is created
        nc = NotificationController(self.sa_session, account_id=self.account.id)
        notif = nc.get_unread_notifications()
        self.assertEqual(len(notif), 1)
        self.assertEqual(notif[0].callback, '{"js_module": "test123", "parameters": []}')

    def test_stop_import_after_number_empty_rows(self):
        # create csv file
        headers = ['column1', 'column2', 'column3']
        row1 = ['row11', 'row12', 'row13']
        row2 = ['row21', 'row33', 'row23']
        row3 = ['', '', '']
        row4 = ['', '', '']
        row5 = ['', '', '']

        self._attach_permissions_to_group(self.account.default_group, 'create_file', 'view_file')

        import csv
        with open('test_mock_file.csv', 'wb') as f:
            wr = csv.writer(f, dialect=csv.excel, delimiter=',')
            wr.writerow(headers)
            wr.writerow(row1)
            wr.writerow(row2)
            wr.writerow(row3)
            wr.writerow(row4)
            wr.writerow(row5)

        with file('test_mock_file.csv') as f:
            data = f.read()
        os.remove('test_mock_file.csv')
        myfile = self._create_uploaded_file('test_mock_file.csv', data)

        # import file
        mfc = MockFileController(self.sa_session, account_id=self.account.id)
        mfc.MAX_RECORDS_FOR_INSTANT_IMPORT = 10
        rv, failures = mfc.import_file(myfile, None, file_name='test_mock_file')

        # we shouldn't have failure lines
        self.assertEqual(len(failures), 0)
