# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.
from decimal import Decimal
from twisted.trial.unittest import SkipTest
from basesoft.util import number

from basesoft.web.contrib import i18n
from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.contrib.i18n.controllers import I18nController
from basesoft.web.core.exceptions import PermissionDeniedException, PublicSecurityException

class TestCase(iqportal_unittest.IQPortalTestCase):
    def setUp(self):
        self.connect_db()

        self.account = self._create_active_account()
        self.group = self.account.groups[0].group

        # create an instance of the controller. It is used in all test methods.
        self.controller = I18nController(self.session, account_id=self.account.id)

    def test_domain_country_inactive(self):
        self._attach_permissions_to_group(self.group, 'manage_domain_country_mapping')

        self.controller.save_domain_country(10, active=False)

        c = self.controller.get_domain_countries(10)
        self.assertTrue(len(c) == 0)

    def test_domain_country_no_access(self):
        self.assertRaises(PermissionDeniedException, self.controller.save_domain_country, 1, 'text')

    def test_domain_country_name_change(self):
        self._attach_permissions_to_group(self.group, 'manage_domain_country_mapping')
        self.controller.save_domain_country(1, 'local name')

        c, l = self.controller.get_domain_countries(1).pop()
        self.assertTrue(l is not None)
        self.assertEqual(l.name, 'local name')

    def test_domain_country_name_change_revert(self):
        self._attach_permissions_to_group(self.group, 'manage_domain_country_mapping')
        self.controller.save_domain_country(1, 'local name')

        c, l = self.controller.get_domain_countries(1).pop()
        self.assertTrue(l is not None)

        self.controller.save_domain_country(1, c.name)

        c, l = self.controller.get_domain_countries(1).pop()
        self.assertTrue(l is None)

    def test_domain_country_active_change(self):
        self._attach_permissions_to_group(self.group, 'manage_domain_country_mapping')
        self.controller.save_domain_country(1, active=False)

        c, l = self.controller.get_domain_countries(1, include_inactive=True).pop()
        self.assertTrue(l is not None)
        self.assertFalse(l.active)

        self.controller.save_domain_country(1, active=True)

        c, l = self.controller.get_domain_countries(1).pop()
        self.assertTrue(l is None)

    def test_domain_language_alias_no_access(self):
        self.assertRaises(PermissionDeniedException, self.controller.save_domain_language_alias, 1, 'text')

    def test_domain_language_alias(self):
        self._attach_permissions_to_group(self.group, 'manage_domain_language_aliases')

        self.controller.save_domain_language_alias(1, 'Unknown language')

        language = self.controller.get_language_by_alias('Unknown language')
        self.assertEqual(1, language.id)

    def test_domain_language_duplicate_alias(self):
        self._attach_permissions_to_group(self.group, 'manage_domain_language_aliases')

        self.controller.save_domain_language_alias(1, 'Unknown language')

        self.assertRaises(PublicSecurityException, self.controller.save_domain_language_alias, 2, 'Unknown language')

    def test_domain_language_alias_same_as_original(self):
        self._attach_permissions_to_group(self.group, 'manage_domain_language_aliases')

        language = self.controller.get_language_by_id(1)
        self.controller.save_domain_language_alias(1, language.name)

        # we expect no alias to have been saved
        self.assertTrue(len(self.controller.get_language_aliases_by_language_id(1))==0)

    def test_decimal_separators(self):
        raise SkipTest('skippy the bush kangarooooooo')

        account = self._create_active_account()
        account.set_config('basesoft.web.contrib.locale', 'nl_NL.utf8')

        d = number.get_decimal('3,1415', account)
        self.assertEqual(d, Decimal('3.1415'))

        account.set_config('basesoft.web.contrib.locale', 'en_GB.utf8')

        d = number.get_decimal('3.1415', account)
        self.assertEqual(d, Decimal('3.1415'))

        account.set_config('basesoft.web.contrib.i18n.number.decimal.separator', i18n.DECIMAL_SEPARATOR_COMMA)
        account.set_config('basesoft.web.contrib.i18n.number.thousands.separator', i18n.THOUSANDS_SEPARATOR_DOT)

        d = number.get_decimal('3,1415', account)
        self.assertEqual(d, Decimal('3.1415'))

        d = number.get_decimal('3.000,1415', account)
        self.assertEqual(d, Decimal('3000.1415'))

        self.assertRaises(number.DecimalConversionException, number.get_decimal, '3,000.1415', account)
        self.assertRaises(number.DecimalConversionException, number.get_decimal, '3,000,1415', account)
        self.assertRaises(number.DecimalConversionException, number.get_decimal, '3.0400.1415', account)
