# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.
from django.http import HttpRequest

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.contrib.license.models import Permission
from basesoft.web.contrib.account.decorators import assert_user_logged_in, assert_user_not_logged_in
from twisted.trial.unittest import SkipTest
from basesoft.web.contrib.iconbar.models import IconBarIcon

# define some icons used for testing:
class Icon1(IconBarIcon):
    """
    An icon that we use for testing. This is shown only for logged out users.
    """

    icon_index = 2

    @assert_user_not_logged_in
    def assert_permissions(self, request):
        """
        This icon is only shown if the user is logged out
        """
        pass

class Icon2(IconBarIcon):
    """
    An icon that we use for testing. This is shown only for logged in users
    """

    icon_index = 4

    @assert_user_logged_in
    def assert_permissions(self, request):
        """
        This icon is only shown if the user is logged in
        """
        pass

class Icon3(IconBarIcon):
    """
    An icon that we use for testing. This is shown all the time
    """

    icon_index = 1
    group = 'stacked'

    def assert_permissions(self, request):
        """
        This icon is only shown all the time
        """
        pass

class Icon4(IconBarIcon):
    """
    An icon that we use for testing. This is shown only for logged in users that
    have permissions: 'test_perm_read1', 'test_perm_write1'
    """

    icon_index = 3
    group = 'stacked'

    @assert_user_logged_in
    def assert_permissions(self, request):
        """
        This icon is only shown if the user is logged out
        """
        pc = self._get_permission_controller(request)
        pc.assert_permissions('test_perm_read1', 'test_perm_write1')

class Icon5(IconBarIcon):
    """
    An icon that we use for testing. This is shown only for logged in users that
    have permissions: 'test_perm_read2', 'test_perm_write2'
    """

    icon_index = 0

    @assert_user_logged_in
    def assert_permissions(self, request):
        """
        This icon is only shown if the user is logged out
        """
        pc = self._get_permission_controller(request)
        pc.assert_permissions('test_perm_read2', 'test_perm_write2')


# define a mock request object that contains the information that we need
# we need this 'request' object because the decorators use it to check if a
# user is logged in or not
class MockRequest(HttpRequest):
    """
    We use this mock class to pass in the information to the assert_permissions
    methods when we do the check/add to iconbar (this normally happens in the
    context processor)
    """
    pass

class IconbarTestCase(iqportal_unittest.IQPortalTestCase):
    """
    This will test the icon bar.
    """

    def setUp(self):
        """
        The script for reseting the database and creating all the tables
        """

        raise SkipTest('FIXME: enable test: it currnetly fails cuz of iconbar refactor')

        self.connect_db()

        # create all icon objects
        self.icon_list = [Icon1(), Icon2(), Icon3(), Icon4(), Icon5()]

    def test_create_iconbar_order(self):
        """
        Check that the iconbar widget is created and after addding the icons
        with no permission checks we have them in the iconbar list of icons, and
        then test the order of the icons after render is called
        """

        iconbar = IconBar()
        for icon in self.icon_list:
            iconbar.add_icons(icon) # add each icon without checks

        added_icons = iconbar.icon_list

        # check if they have the proper types
        # the list contains now dictionaries with the group information and the list of icons for that group
        # each item in the icon list has an object with the following keys:
        # {'group_index': the_group_index,
        #  'group': name_of_the_group,
        #  'icons': [icon_object_1, icon_object_2, ...]}
        assert isinstance(added_icons[0]['icons'][0], Icon1)
        assert isinstance(added_icons[1]['icons'][0], Icon2)
        assert isinstance(added_icons[2]['icons'][0], Icon3) # icons 3 and 4 belong to the same stack
        assert isinstance(added_icons[2]['icons'][1], Icon4) # so they're in the same list
        assert isinstance(added_icons[3]['icons'][0], Icon5)

        # this creates the widget and causes the render to sort the icons
        iconbar.as_html()

        # this is the list of icons after the widget sorts them
        ordered_icons = iconbar.widget.icons

        # check the order by looking at the types of the icons
        # ther order should be according to the icon_index we set up above
        assert isinstance(ordered_icons[0]['icons'][0], Icon5)
        assert isinstance(ordered_icons[1]['icons'][0], Icon3) # the stack comes second, because of the smallest index
        assert isinstance(ordered_icons[1]['icons'][1], Icon4) # between the stack icons
        assert isinstance(ordered_icons[2]['icons'][0], Icon1)
        assert isinstance(ordered_icons[3]['icons'][0], Icon2)

    def test_add_icons_logged_out_user(self):
        """
        Test what icons the logged out user sees. They should see only icons 1,3
        """
        # setup a mock request that contains some information about the user
        # we want a request that contains information about a guest user
        request = MockRequest()
        request.session = {}
        request.Session = self.session

        iconbar = IconBar()
        for icon in self.icon_list:
            if icon.has_permissions(request): # we check each icons' permission
                iconbar.add_icons(icon)

        # we should have in the list of icons only icons 1 and 3
        added_icons = iconbar.icon_list
        assert isinstance(added_icons[0]['icons'][0], Icon1)
        assert isinstance(added_icons[1]['icons'][0], Icon3)
        self.assertEqual(len(added_icons), 2, "we should have only 2 objects in the list of added icons")
        self.assertEqual(len(added_icons[0]['icons']), 1, "the first list of icons should have only one icon")
        self.assertEqual(len(added_icons[1]['icons']), 1, "the second list of icons should have only one icon")

    def test_add_icons_logged_in_user_no_permission(self):
        """
        Test what icons are shown to a logged in user that has no permissions.
        The user should see only icons 2 and 3
        """
        # create an account
        active_account = self._create_active_account()

        # setup a mock request that contains some information about the user
        # that's logged in
        request = MockRequest()
        request.session = {'user_id': active_account.id }
        request.Session = self.session

        iconbar = IconBar()
        for icon in self.icon_list:
            if icon.has_permissions(request): # we check each icons' permission
                iconbar.add_icons(icon)

        # we should have in the list of icons only icons 1 and 3
        added_icons = iconbar.icon_list
        assert isinstance(added_icons[0]['icons'][0], Icon2)
        assert isinstance(added_icons[1]['icons'][0], Icon3)
        self.assertEqual(len(added_icons), 2, "we should have only 2 objects in the list of added icons")
        self.assertEqual(len(added_icons[0]['icons']), 1, "the first list of icons should have only one icon")
        self.assertEqual(len(added_icons[1]['icons']), 1, "the second list of icons should have only one icon")

    def test_add_icons_logged_in_user_with_permission(self):
        """
        Test and see if icons are added properly for logged in users, based on
        what permissions they have
        """
        # add the test permissions that we'll use:
        permissions = ['test_perm_read1',
                       'test_perm_read2',
                       'test_perm_write1',
                       'test_perm_write2']
        for permission in permissions:
            try:
                self.session.query(Permission).filter(Permission.name==permission).one()
            except:
                self.session.add(Permission(name=permission))
        self.session.flush()

        # we'll test for each configuration if the user that has the given
        # permissions sees only the given icons.
        # icon_count represents how many icons should each object from the iconbar list have
        config_to_test = [
            {'permissions': [u'test_perm_read1', u'test_perm_write1'],
             'icon_count': [1, 2], 'combination': 1},

            {'permissions': [u'test_perm_read2', u'test_perm_write2'],
             'icon_count': [1, 1, 1], 'combination': 2},

            {'permissions': [u'test_perm_read1', u'test_perm_write2'],
             'icon_count': [1, 1], 'combination': 3},

            {'permissions': [u'test_perm_read1', u'test_perm_write1',
                             u'test_perm_read2', u'test_perm_write2'],
             'icon_count': [1, 2, 1], 'combination': 4},
        ]

        # setup a mock request that contains some information about the user
        # that's logged in
        request = MockRequest()
        request.Session = self.session

        for config in config_to_test:
            perm = config['permissions']
            icon_count = config['icon_count']

            # create a new account
            active_account = self._create_active_account()

            # set the id of the 'logged' user
            request.session = {'user_id': active_account.id }

            # add the proper permissions to the current account
            group = active_account.groups[0].group
            self._attach_permissions_to_group(group, *perm)

            # create the iconbar
            iconbar = IconBar()
            for icon in self.icon_list:
                if icon.has_permissions(request): # we check the permission
                    iconbar.add_icons(icon)

            added_icons = iconbar.icon_list

            if config['combination'] == 1:
                assert isinstance(added_icons[0]['icons'][0], Icon2)
                assert isinstance(added_icons[1]['icons'][0], Icon3)
                assert isinstance(added_icons[1]['icons'][1], Icon4)
            if config['combination'] == 2:
                assert isinstance(added_icons[0]['icons'][0], Icon2)
                assert isinstance(added_icons[1]['icons'][0], Icon3)
                assert isinstance(added_icons[2]['icons'][0], Icon5)
            if config['combination'] == 3:
                assert isinstance(added_icons[0]['icons'][0], Icon2)
                assert isinstance(added_icons[1]['icons'][0], Icon3)
            if config['combination'] == 4:
                assert isinstance(added_icons[0]['icons'][0], Icon2)
                assert isinstance(added_icons[1]['icons'][0], Icon3)
                assert isinstance(added_icons[1]['icons'][1], Icon4)
                assert isinstance(added_icons[2]['icons'][0], Icon5)

            self.assertEqual(len(added_icons), len(icon_count), "wrong icon count")

            for i in range(len(added_icons)):
                self.assertEqual(len(added_icons[i]['icons']), icon_count[i], "wrong icon count for group object")
