# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.
from twisted.trial.unittest import SkipTest
from basesoft.web.contrib.account.controllers import AccountController

from basesoft.web.test.unit import iqportal_unittest

from basesoft.web.contrib.license.controllers import LicenseController
from basesoft.web.contrib.license.models import License, Role, Permission
from basesoft.web.core.exceptions import SecurityException,\
    PermissionDeniedException, PublicUserException
from basesoft.web.core.util.controllers import LicenseConfigController, StateManager, PermissionController
from basesoft.web.contrib.notification.controllers import NotificationController
from basesoft.web.contrib.contact.models import Company
from basesoft.web.contrib.dashboard.controllers import DashboardStateController
from basesoft.web.test.unit.test_dashboard_controller import get_state
from basesoft.web.contrib.dashboard.models import Dashboard

class TestCase(iqportal_unittest.IQPortalTestCase):
    """
    This will test the License Controller.
    All individual *public* methods should be tested separately.
    """

    def setUp(self):
        """
        The script for reseting the db and creating all the tables
        """
        self.connect_db()

        self.account = self._create_active_account()
        self.group = self.account.groups[0].group
        # create an instance of the controller. It is used in all test methods.
        self.controller = LicenseController(self.session, account_id=self.account.id)

    def _get_license_state(self):
        """
        """
        from basesoft.web.contrib.contact.models import Person
        return {'license': {
                    'name': u'test_license_from_state',
                    'expiration_date': u'2050-01-01',
                    'is_default': True,
                    'polymorphic_type': 'License',
                    },
                'domain': {'name': self._uniquestring('DM')},
                'contact_person': {
                    'first_name': u'Marten',
                    'last_name': u'Koedam',
                    'gender': Person.MALE,
                    'email': u'marten@basesoft.com',
                    'position': u'developer',
                    'department': u'development',
                    'phone': '+31206847087',
                    'birth_date': '1981-10-30',
                    'address_data': self._get_address_data()
                    },
                'company': {
                    'name': self._uniquestring('CM'),
                    'vat_no': 'NL 8104.49.493.B01',
                    'website': u'http://www.basesoft.com',
                    },
                'company_address': {
                    'address': self._get_address_data(),
                    'is_invoice': True,
                    'is_delivery': False
                    },
                'company_bank_account': {
                    'bank_account': {
                        'number': self._uniquestring('99'),
                        'iban': self._uniquestring('NL'),
                        'bic':'BBBBNLNH'},
                    'is_default': True
                    },
                'group': {'name': u'admins'},
                'permissions': [1, 2],
                'usage_options': []
                }

    def _get_address_data(self):
        return {'country': '155',
                'postcode': "1017 CH",
                'street': u" Herengracht 564",
                'city': u"Amsterdam",
                'state': u"Noord-Holland"}

    def test_validate_default_group(self):
        self._attach_permissions_to_group(self.group, 'view_all_domains')
        self.assertFalse(self.controller.validate_default_group(self.group.license.id))
        self.assertTrue(self.controller.validate_default_group(self.group.license.id, self.group.id))

    def test_save_privilege_hook_from_state(self):
        """
        This test the saving of a privilege hook.
        """

        state = {
            'domain_id': self.group.license.domain.id,
            'object': 'Contact',
            'domain_privileges': [],
            'license_privileges': [1, 2, 4],
            'group_privileges': []
        }

        self.assertRaises(PermissionDeniedException, self.controller.save_privilege_hook_from_state, state)
        self._attach_permissions_to_group(self.group, 'hook_manager', 'view_all_domains')

        poh = self.controller.save_privilege_hook_from_state(state)
        self.assertEqual(poh.license_privileges, 7)

    def test_save_privilege_delegation(self):
        """
        """
        from basesoft.web.contrib.contact.models import Contact, ContactGroup
        contact = Contact()
        cg = ContactGroup()
        cg.group = self.group
        cg.privilege = Contact.PRIV_VIEW
        cg.group_has_privilege = True
        contact.groups.append(cg)

        cg = ContactGroup()
        cg.group = self.group
        cg.privilege = Contact.PRIV_UPDATE
        cg.group_has_privilege = True
        contact.groups.append(cg)

        self.session.add(contact)
        self.session.flush()

        self._attach_permissions_to_group(self.group, 'view_license')

        self.assertRaises(PermissionDeniedException, self.controller.save_privilege_delegation, 'Contact', contact.id, self.group.id, [Contact.PRIV_UPDATE], True)

        self._attach_permissions_to_group(self.group, 'distribute_group_permissions')

        self.assertRaises(PermissionDeniedException, self.controller.save_privilege_delegation, 'Contact', contact.id, self.group.id, [Contact.PRIV_UPDATE], True)

        cg = ContactGroup()
        cg.group = self.group
        cg.privilege = Contact.PRIV_DELEGATE_PRIVILEGES
        cg.group_has_privilege = True
        contact.groups.append(cg)
        self.session.add(contact)
        self.session.flush()

        self.controller.save_privilege_delegation('Contact', contact.id, self.group.id, [Contact.PRIV_UPDATE], True)
        self.assertTrue(self.controller.has_privileges_on_object(contact, Contact.PRIV_UPDATE, Contact.PRIV_VIEW))

        self.controller.save_privilege_delegation('Contact', contact.id, self.group.id, [Contact.PRIV_VIEW], True, license_has_privilege=True)

    def _add_account_to_new_license(self):
        from basesoft.web.contrib.account.models import GroupAccount
        from basesoft.web.contrib.account.controllers import AccountController

        # create new license with new group (needs sysadmin account)
        ac = AccountController(self.session)
        account = ac.get_account_by('sysadmin')
        lc = LicenseController(self.session, None, account.id)
        license2 = self._create_license(domain=self.domain)
        g = lc.save_or_update_group(None, 'test_group2', license2, is_default=True)

        # make the link between account and the new group.
        ga = GroupAccount()
        ga.is_default = 1
        ga.active = True
        ga.account = self.account
        ga.group = g

        self.session.add(ga)
        self.session.flush()

        return [license2, g]

    def test_get_licenses(self):
        """
        Test the get_licenses method, with and without the correct rights.
        """
        # don't have view domain (to get licenses for current domain)
        self.assertRaises(PermissionDeniedException, self.controller.get_licenses)
        self._attach_permissions_to_group(self.group, 'view_domain')
        current = self.controller.get_licenses()
        self.assertEqual(type(current), list)
        # now get licenses from all other domains with correct permission
        self._attach_permissions_to_group(self.group, 'view_all_domains')
        licenses = self.controller.get_licenses()
        self.assertEqual(type(licenses), list)
        self.assertNotEqual(len(licenses), len(current))

        # now add this account to another license
        license1 = self.license
        license2, g = self._add_account_to_new_license()

        self._revoke_permissions_from_group(self.group, 'view_all_domains', 'view_domain')
        # no permission for any license
        self.assertRaises(PermissionDeniedException, self.controller.get_licenses)

        self._attach_permissions_to_group(self.group, 'view_license')
        self.controller = LicenseController(self.session, account_id=self.account.id)
        licenses = self.controller.get_licenses()
        # only for one license the account has view_license
        self.assertEqual(len(licenses), 1)

        self._attach_permissions_to_group(g, 'view_license')
        licenses = self.controller.get_licenses()
        self.assertEqual(len(licenses), 2)

    def test_get_account_licenses_with_permssions(self):
        """
        Gets licenses for an account that is attached to multiple licenses
        """
        license1 = self.license
        license2, g = self._add_account_to_new_license()

        self.controller = LicenseController(self.session, account_id=self.account.id)
        # we should have 2 active groups for this account now
        self.assertEqual(len(self.controller._active_account_groups), 2)

        licenses = self.controller.get_account_licenses_with_permssions('view_license')
        # none of the account's licenses has tat permission
        self.assertEqual(len(licenses), 0)

        self._attach_permissions_to_group(self.group, 'view_license')
        licenses = self.controller.get_account_licenses_with_permssions('view_license')
        # only the first license has the perm
        self.assertEqual(len(licenses), 1)
        self.assertEqual(licenses[0].id, license1.id)

        self._attach_permissions_to_group(g, 'view_license')
        licenses = self.controller.get_account_licenses_with_permssions('view_license')

        # both licenses have the perm
        self.assertEqual(len(licenses), 2)
        self.assertEqual(licenses[0].id, license1.id)
        self.assertEqual(licenses[1].id, license2.id)

    def test_assert_privilege_delegation(self):
        """
        This tests whether or not the current group has the rights to delegate privileges on the provided object
        """
        from basesoft.web.contrib.contact.models import Contact
        contact = Contact()
        contact.group = self.group
        contact.group_privileges = Contact.all_privileges
        contact.license_privileges = 0
        contact.domain_privileges = 0

        # add the contact
        self.session.add(contact)
        self.session.flush()

        # we dont have permission yet
        self.assertRaises(PermissionDeniedException, self.controller.assert_privilege_delegation, 'Contact', contact.id)

        self._attach_permissions_to_group(self.group, 'distribute_group_permissions')
        self.controller.assert_privilege_delegation('Contact', contact.id)

        # test if exception is raised if it is someone elses Contact.
        account = self._create_active_account()

        contact = Contact()
        contact.group = account.groups[0].group
        contact.group_privileges = Contact.all_privileges
        contact.license_privileges = 0
        contact.domain_privileges = 0

        # add the contact
        self.session.add(contact)
        self.session.flush()

        # the current user has no rights whatsoever on the provided contact.
        self.assertRaises(PermissionDeniedException, self.controller.assert_privilege_delegation, 'Contact', contact.id)

    def test_validate_permissions(self):
        """
        Test the validate permissions method, for access to the requested permissions.
        """
        self._attach_permissions_to_group(self.group, 'view_all_permissions')
        perms = self.controller.get_permissions()
        self._revoke_permissions_from_group(self.group, 'view_all_permissions')
        self.assertRaises(PermissionDeniedException, self.controller.validate_permissions, perms)
        perm = perms.pop()
        self.assertRaises(PermissionDeniedException, self.controller.validate_permissions, [perm])
        self._attach_permissions_to_group(self.group, 'view_all_permissions')
        self.assertEqual(True, self.controller.validate_permissions([perm]))

    def test_get_permissions(self):
        """
        Test the get_permissions method, with and without the correct rights.
        """
        self.assertRaises(PermissionDeniedException, self.controller.get_permissions)
        self._attach_permissions_to_group(self.group, 'view_license')
        # can view only license's permissions
        perms = self.controller.get_permissions()
        self.assertEqual(len(perms), len(self.group.license.permissions))

        self._attach_permissions_to_group(self.group, 'view_domain')
        # can view only domains's permissions
        perms = self.controller.get_permissions()

        self._attach_permissions_to_group(self.group, 'view_all_permissions')
        # can view all permissions
        perms = self.controller.get_permissions()
        self.assertEqual(type(perms), list)

    def test_get_usage_options(self):
        """
        Test the get_usage_options method, with and without the correct rights.
        """
        self.assertRaises(PermissionDeniedException, self.controller.get_usage_options)
        self._attach_permissions_to_group(self.group, 'view_license')
        # can view only license's usage options
        self.controller.get_usage_options()

        self._attach_permissions_to_group(self.group, 'view_domain')
        # can view only domains's usage options
        self.controller.get_usage_options()

        self._attach_permissions_to_group(self.group, 'view_all_permissions')
        # can view all usage options
        opts = self.controller.get_usage_options()
        self.assertEqual(type(opts), list)

    def test_save_or_update_usage_option(self):
        """
        This tests saving and updating of usage options
        """

        name = self._uniquestring('UO')
        desc = u'Total Number of Surveys to be created.'

        usage_option = self.controller.save_or_update_usage_option(None, name, desc)
        self.assertEqual(usage_option.name, name)
        self.assertEqual(usage_option.description, desc)

        new_desc = u'The Number of Surveys allowed to create with the License'
        update_usage_option = self.controller.save_or_update_usage_option(
                                              id=usage_option.id,
                                              name=name,
                                              description=new_desc
                                              )

        self.assertEqual(update_usage_option.description, new_desc)
        self.assertEqual(update_usage_option.name, name)

    def test_save_or_update_domain(self):
        """
        This will test the creation and the update of a domain.
        """
        name = self._uniquestring('d')

        self.assertRaises(PermissionDeniedException, self.controller.save_or_update_domain, None, name)

        self._attach_permissions_to_group(self.group, 'create_domain')
        domain = self.controller.save_or_update_domain(None, name)
        self.assertEqual(domain.name, name)

        name = self._uniquestring('dd')

        self.assertRaises(PermissionDeniedException, self.controller.save_or_update_domain, domain.id, name)

        self._attach_permissions_to_group(self.group, 'update_domain')
        updated_domain = self.controller.save_or_update_domain(id=domain.id,
                                                               name=name
                                                               )

        self.assertEqual(updated_domain.name, name)
        self.assertEqual(updated_domain.id, domain.id)

    def test_save_or_update_license(self):
        """
        This will test the creation and the update of a license.
        """
        from basesoft.web.contrib.contact.models import Person
        self._attach_permissions_to_group(self.group, 'create_domain', 'create_license', 'view_license')

        name = 'Test License'
        domain = self.controller.save_or_update_domain(id=None,
                                                       name=self._uniquestring('D'))
        contact = Person()
        license = self.controller.save_or_update_license(id=None,
                                                         name=name,
                                                         domain=domain,
                                                         polymorphic_type=License,
                                                         contact=contact,
                                                         company=None,
                                                         expiration_date=None)

        self.assertEqual(license.name, name)
        self.assertEqual(license.domain, domain)
        self.assertEqual(license.company.contact_person, contact)

        self._attach_permissions_to_group(self.group, 'update_license')
        new_contact = Person()
        new_name = 'Test Updated License'

        updated_license = self.controller.save_or_update_license(id=license.id,
                                                                 name=new_name,
                                                                 domain=domain,
                                                                 polymorphic_type=License,
                                                                 contact=new_contact,
                                                                 company=None,
                                                                 expiration_date=None)

        self.assertEqual(updated_license.name, new_name)
        self.assertEqual(updated_license.domain, domain)
        self.assertEqual(updated_license.company.contact_person, new_contact)


    def test_license_with_invoice_same_as_contact(self):
        license_state = self._get_license_state()
        license_state['invoice_person'] = license_state['contact_person']
        license_state['invoice_person']['is_same_person_as'] = True

        self._attach_permissions_to_group(self.group,
                                  'create_domain',
                                  'update_domain',
                                  'create_license',
                                  'update_license',
                                  'view_license',
                                  'view_all_domains', # because it creates a new domain and has to load it back
                                  'view_all_permissions',
                                  'view_all_usage_options',
                                  'distribute_license_permissions',
                                  'distribute_group_permissions')
        license = self.controller.save_license_from_state(license_state)
        license_id = license.id
        self.sa_session.expire(license)

        license = self.controller.get_licenses(license_id).pop()

        self.assertEqual(license.company.contact_person, license.company.invoice_person)

    def test_save_or_update_license_from_state(self):
        """
        """
        license_state = self._get_license_state()

        self.assertRaises(SecurityException,
                          self.controller.save_license_from_state,
                          license_state)
        self._attach_permissions_to_group(self.group,
                                          'create_domain',
                                          'update_domain',
                                          'create_license',
                                          'update_license',
                                          'view_license',
                                          'view_all_domains', # because it creates a new domain and has to load it back
                                          'view_all_permissions',
                                          'view_all_usage_options',
                                          'distribute_license_permissions',
                                          'distribute_group_permissions')

        license = self.controller.save_license_from_state(license_state)

        self.assertEqual(license.domain.name, license_state['domain']['name'])
        self.assertEqual(license.company.name, license_state['company']['name'])
        self.assertEqual(license.company.contact_person.first_name,
                         license_state['contact_person']['first_name'])

        self.assertEqual(license.permissions,
                         self.controller.get_permissions(license_state['permissions']))

        license_state = self._get_license_state()
        license_state['dbID'] = license.id
        license_state['append_to_group'] = license.default_group.id
        license_state['permissions'] = [1, 3]
        license = self.controller.save_license_from_state(license_state)

        perms = self.controller.get_permissions(license_state['permissions'])
        self.assertEqual(license.id, license_state['dbID'])
        self.assertEqual(license.permissions, perms)
        self.assertEqual(license.default_group.permissions, perms)

    def test_save_or_update_group(self):
        """
        This tests the creation and the update of a group.
        """

        from basesoft.web.contrib.contact.models import Person
        self._attach_permissions_to_group(self.group, 'create_domain', 'create_license', 'view_license')

        domain = self.controller.save_or_update_domain(id=None,
                                                       name=self._uniquestring('TD'))
        contact = Person()
        license = self.controller.save_or_update_license(id=None,
                                                         name='Test License',
                                                         domain=domain,
                                                         polymorphic_type=License,
                                                         contact=contact,
                                                         company=None,
                                                         expiration_date=None)

        group = self.controller.save_or_update_group(id=None,
                                                     name=u'Test Group',
                                                     license=license)
        self.assertEqual(group.name, u'Test Group')
        self.assertEqual(group.license, license)

        updated_group = self.controller.save_or_update_group(id=group.id,
                                                             name=u'Test Updated Group',
                                                             license=license)

        self.assertEqual(updated_group.name, u'Test Updated Group')
        self.assertEqual(updated_group.license, license)
        self.assertEqual(group.id, updated_group.id)


    def test_add_permissions_to_group(self):
        """
        This will test the distribution of permissions to groups.
        """
        from basesoft.web.contrib.license.models import Permission
        account = self._create_active_account()
        group = account.groups[0].group

        lc = LicenseController(self.session, account_id=account.id)

        permissions = self.session.query(Permission).filter(
                          Permission.name.in_(['view_license'])).all()

        self.assertTrue(len(permissions)>0)
        # test with incorrect permissions.
        self.assertRaises(PermissionDeniedException, lc.add_permissions_to_group,
                          group, permissions)

        # add permissions to group
        self._attach_permissions_to_group(group,
                                          'distribute_group_permissions',
                                          'distribute_license_permissions')

        group_perms = len(group.permissions)

        # test with license permissions not in place
        self.assertRaises(SecurityException, lc.add_permissions_to_group,
                          group, permissions)

        # add permissions to the license.
        lc.add_permissions_to_license(group.license, permissions)

        # add permissions to the group.
        lc.add_permissions_to_group(group, permissions)
        self.assertEqual(group_perms, len(group.permissions)-len(permissions))

    def test_save_permissions_to_license(self):
        """
        This test the saving of permissions on a license. The logic is
        distributed to the add and revoke methods based on the available
        information. The user needs to have the correct rights to add/revoke
        permissions.
        """
        from basesoft.web.contrib.license.models import Permission
        account = self._create_active_account()
        group = account.groups[0].group

        lc = LicenseController(self.session, account_id=account.id)

        permissions = self.session.query(Permission).filter(
                          Permission.name.in_(['view_license'])).all()

        self.assertTrue(len(permissions)>0)
        # test with incorrect permissions.
        self.assertRaises(PermissionDeniedException, lc.save_permissions_to_license,
                          group.license, permissions)

        self._attach_permissions_to_group(group,
                                          'distribute_license_permissions')

        permissions = group.license.permissions + permissions
        # add permissions to the license.
        lc.save_permissions_to_license(group.license, permissions)
        self.assertEqual(group.license.permissions, permissions)

        permissions = self.session.query(Permission).filter(
                          Permission.name.in_(['distribute_license_permissions'])).all()

        self.assertRaises(PermissionDeniedException, lc.save_permissions_to_license,
                          group.license, permissions)

        self._attach_permissions_to_group(group, 'view_license', 'distribute_group_permissions')
        lc.save_permissions_to_license(group.license, permissions)
        self.assertEqual(len(group.license.permissions), len(permissions))

    def test_add_permissions_to_license(self):
        """
        This will test the distribution of permissions to licenses
        """
        from basesoft.web.contrib.license.models import Permission
        account = self._create_active_account()
        group = account.groups[0].group

        lc = LicenseController(self.session, account_id=account.id)

        permissions = self.session.query(Permission).filter(
                          Permission.name.in_(['view_license'])).all()

        self.assertTrue(len(permissions)>0)
        # test with incorrect permissions.
        self.assertRaises(PermissionDeniedException, lc.add_permissions_to_license,
                          group.license, permissions)

        # add permissions to group so we can test the actual method.
        self._attach_permissions_to_group(group,
                                          'distribute_license_permissions')

        license_perms = len(group.license.permissions)
        # add permissions to the license.
        lc.add_permissions_to_license(group.license, permissions)
        self.assertEqual(license_perms, len(group.license.permissions)-len(permissions))

    def test_revoke_permissions_from_group(self):
        """
        This will test the revoking of permissions on a group.
        """
        from basesoft.web.contrib.license.models import Permission
        account = self._create_active_account()
        group = account.groups[0].group

        lc = LicenseController(self.session, account_id=account.id)
        permissions = self.session.query(Permission).filter(
                          Permission.name.in_(['view_license'])).all()

        self.assertTrue(len(permissions)>0)
        # test with incorrect permissions.
        self.assertRaises(PermissionDeniedException, lc.revoke_permissions_from_group,
                          group, permissions)

        self._attach_permissions_to_group(group, 'distribute_group_permissions',
                                          'view_license')
        group_perms = len(group.permissions)

        lc.revoke_permissions_from_group(group, permissions)
        self.assertEqual(group_perms, len(group.permissions)+len(permissions))


    def test_revoke_permissions_from_license(self):
        """
        This will test the revoking of permissions on a license.
        """
        from basesoft.web.contrib.license.models import Permission
        account = self._create_active_account()
        group = account.groups[0].group

        lc = LicenseController(self.session, account_id=account.id)
        permissions = self.session.query(Permission).filter(
                          Permission.name.in_(['view_license'])).all()

        self.assertTrue(len(permissions)>0)
        # test with incorrect permissions.
        self.assertRaises(PermissionDeniedException, lc.revoke_permissions_from_license,
                          group.license, permissions)

        self._attach_permissions_to_group(group, 'distribute_group_permissions',
                                          'distribute_license_permissions',
                                          'view_license')

        # store the lengths
        license_perms = len(group.license.permissions)
        group_perms = len(group.permissions)
        lc.revoke_permissions_from_license(group.license, permissions)
        # make sure the old lengths are the current lengths + the length of the
        # permissions revoked.
        self.assertEqual(license_perms, len(group.license.permissions)+len(permissions))
        self.assertEqual(group_perms, len(group.permissions)+len(permissions))

    def test_distribute_usage_options(self):
        """
        This will test the distribution of usage options and its credits to
        Licenses.
        """
        from sqlalchemy.sql import and_
        from decimal import Decimal
        from basesoft.web.contrib.license.models import LicenseUsage

        account = self._create_active_account()
        group = account.groups[0].group
        self._attach_permissions_to_group(group,
                                          'distribute_license_permissions',
                                          'view_all_usage_options')

        # we need a license controller do create some stuff.
        lc = LicenseController(self.session, account_id=account.id)

        # create a few usage options to test with.
        uo1 = self.controller.save_or_update_usage_option(None, self._uniquestring('One'), None).id
        uo2 = self.controller.save_or_update_usage_option(None, self._uniquestring('Two'), None).id

        # make a few mock usage options. needed for credits.
        mocks = [{'usage_option_id': uo1, 'credits': 4, "price_per_unit": 15.20},
                 {'usage_option_id': uo2, 'credits': 10, "price_per_unit": 5.20}]

        # add them to the license.
        lc.save_or_update_license_usage_options(group.license,mocks)

        # query the license usage options
        luos = self.session.query(LicenseUsage).filter(
                    and_(LicenseUsage.license_id==group.license.id,
                    LicenseUsage.usage_option_id.in_([uo1, uo2]))).order_by(
                        LicenseUsage.usage_option_id).all()

        # we need to have to license usage options attached.
        self.assertEqual(len(luos),2)
        credits = [10, 4]
        prices = [Decimal("5.20"), Decimal("15.20")]
        for luo in luos:
            self.assertEqual(luo.credits, credits.pop())
            self.assertEqual(luo.price_per_unit, prices.pop())

        # we up the credits and validate this.
        lc.add_usage_options_to_license(group.license,
                                        [{'usage_option_id': uo1, 'credits': 20, "price_per_unit": 15.2}])

        luos = self.session.query(LicenseUsage).filter(
                    and_(LicenseUsage.license_id==group.license.id,
                    LicenseUsage.usage_option_id.in_([uo1, uo2]))).order_by(
                        LicenseUsage.usage_option_id).all()

        credits = [10,24]
        for luo in luos:
            self.assertEqual(luo.credits, credits.pop())

    def test_permission_role(self):
        """
        Test the permission roles as the defined in #380
        """
        ps = self.sa_session.query(Permission).filter(Permission.name.in_(['create_mock_mail', 'view_mock_mail'])).all()

        rn = self._uniquestring('test role')

        self.assertRaises(PermissionDeniedException, self.controller.save_or_update_role, None, Role.ROLE_TYPE_MANUAL, rn, ps, None)

        self._attach_permissions_to_group(self.group, 'manage_roles', 'create_roles')

        r = self.controller.save_or_update_role(None, Role.ROLE_TYPE_MANUAL, rn, ps, {'basesoft.test.mock.config': ('False', True)})

        role = self.sa_session.query(Role).filter(Role.id==r.id).one()

        self.assertEqual(role.name, rn)
        self.assertEqual(role.permissions, ps)
        self.assertEqual(len(role.configs), 1)
        self.assertTrue(role.configs[0].is_license)

        rn = self._uniquestring('test role update')

        ps = ps + self.sa_session.query(Permission).filter(Permission.name=='update_mock_mail').all()
        r = self.controller.save_or_update_role(role.id, Role.ROLE_TYPE_MANUAL, rn, ps, {})

        role = self.sa_session.query(Role).filter(Role.id==role.id).one()

        self.assertEqual(role.name, rn)
        self.assertEqual(role.permissions, ps)
        self.assertEqual(len(role.configs), 0)

    def test_role_with_dashboard(self):

        # create a dashboard.
        self._attach_permissions_to_group(self.group, 'save_dashboard', 'view_dashboard', 'manage_roles', 'create_roles')
        manager = StateManager(None, self.account.id, None)
        dc = DashboardStateController(self.sa_session, manager)

        state = get_state()
        _map = dc.save_state(state)
        dashboard = dc.get_dashboards(_map[state['dashboard']['domID']]).pop()

        ps = self.sa_session.query(Permission).filter(Permission.name.in_(['view_dashboard', 'create_mock_mail', 'view_mock_mail'])).all()
        rn = self._uniquestring('test role')

        r = self.controller.save_or_update_role(None, Role.ROLE_TYPE_MANUAL, rn, ps, None, dashboard=dashboard)

        role = self.sa_session.query(Role).filter(Role.id==r.id).one()

        self.assertEqual(role.name, rn)
        for p in role.permissions:
            self.assertTrue(p in ps)

        self.assertEqual(role.dashboard, dashboard)

        # Force add of automated Role.
        rn = self._uniquestring('automated role')
        r = Role()
        r.name = rn
        r.permissions = ps
        r.required_permission = 'manage_roles'
        r.role_type = Role.ROLE_TYPE_AUTOMATIC
        self.sa_session.add(r)
        self.sa_session.flush()

        role = self.sa_session.query(Role).filter(Role.id==r.id).first()

        r = self.controller.save_or_update_role(role.id, Role.ROLE_TYPE_AUTOMATIC, rn, ps, None, dashboard=dashboard)
        self.assertEqual(r.dashboard, dashboard)

    def test_apply_role(self):

        ac = self._create_active_account()
        domain = ac.default_group.license.domain

        # the account should not have this permission to begin with.
        c = PermissionController(self.sa_session, account_id=ac.id)
        self.assertFalse(c.has_permissions('view_mock_mail'))

        role = self.controller.get_role_by_name('Unit Test Automated Role')
        self.assertRaises(PermissionDeniedException, self.controller.apply_role, role.id, domain.id, 'License')

        ac.default_group.license.role = role
        self.sa_session.add(ac.default_group.license)

        self._attach_permissions_to_group(self.group, 'apply_roles', 'manage_roles', 'manage_iqportal_roles', 'view_all_domains', 'distribute_license_permissions', 'distribute_group_permissions')

        self.controller.apply_role(role.id, domain.id, apply_to_default_group=False)

        # we didnt apply to group
        c = PermissionController(self.sa_session, account_id=ac.id)
        self.assertFalse(c.has_permissions('view_mock_mail'))

        self.controller.apply_role(role.id, domain.id, apply_to_default_group=True)

        c = PermissionController(self.sa_session, account_id=ac.id)
        self.assertTrue(c.has_permissions('view_mock_mail'))

    def test_apply_role_dashboard_reference(self):
        self._attach_permissions_to_group(self.group, 'create_roles', 'apply_roles', 'manage_roles', 'view_all_domains',
                                          'distribute_license_permissions', 'distribute_group_permissions', 'view_dashboard')

        permissions = self.sa_session.query(Permission).filter(Permission.name.in_(['view_dashboard'])).all()

        dashboard = False

        # cant set privs for dashboard if no dashboard is supplied
        self.assertRaises(PublicUserException, self.controller.save_or_update_role, None, Role.ROLE_TYPE_MANUAL,
                          self._uniquestring('role'), permissions, None, dashboard=dashboard,
                          dashboard_privileges=Dashboard.PRIV_VIEW)

        # create a dashboard.
        self._attach_permissions_to_group(self.group, 'save_dashboard', 'view_dashboard', 'manage_roles', 'create_roles')
        manager = StateManager(None, self.account.id, None)
        dc = DashboardStateController(self.sa_session, manager)

        state = get_state()
        _map = dc.save_state(state)
        dashboard = dc.get_dashboards(_map[state['dashboard']['domID']]).pop()

        role = self.controller.save_or_update_role(None, Role.ROLE_TYPE_MANUAL, self._uniquestring('role'), permissions,
                                                   None, dashboard=dashboard, dashboard_privileges=Dashboard.PRIV_VIEW)

        self.assertEqual(role.dashboard, dashboard)
        self.assertEqual(role.dashboard_privileges & Dashboard.PRIV_VIEW, Dashboard.PRIV_VIEW)

        ac = self._create_active_account()

        c = LicenseController(self.sa_session, account_id=ac.id)

        self.assertFalse(c.has_privileges_on_object(dashboard, Dashboard.PRIV_VIEW))

        self.controller.grant_from_role(role, ac.default_group.license, add_dashboard=True)

        self.assertTrue(c.has_privileges_on_object(dashboard, Dashboard.PRIV_VIEW))

    def test_apply_role_revoke_permissions(self):
        ac = self._create_active_account()
        domain = ac.default_group.license.domain
        role = self.controller.get_role_by_name('Unit Test Automated Role')
        ac.default_group.license.role = role
        self.sa_session.add(ac.default_group.license)

        self._attach_permissions_to_group(ac.default_group, 'view_mock_orders')
        self._attach_permissions_to_group(self.group, 'apply_roles', 'manage_roles', 'manage_iqportal_roles', 'view_all_domains', 'distribute_license_permissions', 'distribute_group_permissions')

        self.controller.apply_role(role.id, domain.id, revoke_permissions=True)

        c = PermissionController(self.sa_session, account_id=ac.id)
        self.assertFalse(c.has_permissions('view_mock_orders'))

    def test_new_account_with_role_configs(self):
        ps = self.sa_session.query(Permission).filter(Permission.name.in_(['create_mock_mail', 'view_mock_mail'])).all()
        rn = self._uniquestring('test role')
        self._attach_permissions_to_group(self.group, 'manage_roles', 'create_roles', 'apply_roles', 'view_all_domains', 'distribute_license_permissions', 'distribute_group_permissions')
        r = self.controller.save_or_update_role(None, Role.ROLE_TYPE_MANUAL, rn, ps, {'basesoft.account.use.iconbar': ('False', False)})

        account = self._create_active_account()
        self.controller.grant_from_role(r, account.default_group.license, link_role=True)
        self.assertEqual(account.default_group.license.role, r)
        # the original account has not had its configs updated
        self.assertTrue(account.get_config('basesoft.account.use.iconbar'))

        ac = AccountController(self.sa_session)
        newaccount = ac.create_account(self._uniquestring('iii'), 'test_password')
        ac.save_or_update_group_account(account.default_group, newaccount, is_default=True)
        aa = ac.save_account_activation(newaccount)
        aa.activation_stage = aa.ACTIVATION_STAGE_PENDING
        self.sa_session.add(aa)

        ac.activate_account(aa.activation_key)

        self.assertFalse(newaccount.get_config('basesoft.account.use.iconbar'))

        # now reapply roel
        self.controller.apply_role(r.id, account.default_group.license.domain.id, apply_account_configs=True)
        self.assertFalse(account.get_config('basesoft.account.use.iconbar'))

    def test_license_config(self):
        """
        This will test updating the license config by an account which has access for the configs.
        """
        from basesoft.web.contrib.license.rpc import get_configs

        configs = get_configs()

        self.assertRaises(PermissionDeniedException, self.controller.save_license_configs, configs, {'basesoft.test.mock.config': True})

        # now we attach the right permissions so we can actually save the config
        self._attach_permissions_to_group(self.group, 'manage_mock_configs', 'manage_license_configs')

        lcc = LicenseConfigController(self.sa_session, scope=self.group.license)

        self.assertEqual(int(lcc.get('basesoft.test.mock.config')), 0)

        self.controller.save_license_configs(configs, {'basesoft.test.mock.config': True})

        self.assertTrue(lcc.get('basesoft.test.mock.config'))

    def test_license_masterdata_change(self):
        """
        This will test the process of master data changes. See also ticket #3773.
        """

        # create a license within the current domain
        account = self._create_active_account(license=self._create_license(domain=self.group.license.domain))
        group = account.groups[0].group
        license = group.license
        original_name = license.company.name
        lc = LicenseController(self.sa_session, account_id=account.id)

        md = {
            'company': {
                'name': 'Updated company name',
                'vat_no': 'NL UPD VAT NO',
            },
            'invoice_person_data': {
                'first_name': 'Fri',
                'last_name': 'Tuurvet',
                'gender': 'male',
                'email': 'invoice@frituur.vet'
            }
        }

        # verify that master data can't be changed by themselves.
        self.assertRaises(PermissionDeniedException, lc.update_master_data, license.id, md)

        # give the group the rights to alter their own license master data.
        self._attach_permissions_to_group(group, 'view_license', 'update_license_masterdata')

        # update the master data.
        license = lc.update_master_data(license.id, md)

        # verify that the company and temporary company have different fields.
        self.assertEqual(license.company.name, original_name)
        self.assertEqual(license.temp_company.name, md['company']['name'])
        self.assertEqual(license.temp_company.invoice_person.first_name, md['invoice_person_data']['first_name'])

        # check with test account that approval/rejection is not possible due to insufficient permissions.
        self.assertRaises(PermissionDeniedException, self.controller.reject_master_data_change, license.id, 'sorry cant approve')

        # grant permissions to test account.
        self._attach_permissions_to_group(self.group, 'license_masterdata_change_approval', 'view_all_domains', 'create_notification')

        nc = NotificationController(self.sa_session, account_id=account.id)
        ncount = nc.get_unread_notifications_count()

        # reject the change request.
        self.controller.reject_master_data_change(license.id, 'sorry cant approve')
        self.assertEqual(ncount+1, nc.get_unread_notifications_count())

        license = self.controller.get_licenses(license.id).pop()
        self.assertEqual(license.temp_company.is_temp, Company.TEMPORARY_STATUS_REJECTED)

        # store number of notifications for test account.
        nc = NotificationController(self.sa_session, account_id=self.account.id)
        ncount = nc.get_unread_notifications_count()

        # change some additional fields and re apply change.
        md = {
            'invoice_person_data': {
                'first_name': 'Second update'
            }
        }

        license = lc.update_master_data(license.id, md)

        # verify original changes and new change
        self.assertEqual(license.temp_company.invoice_person.first_name, 'Second update')
        self.assertEqual(license.temp_company.invoice_person.last_name, 'Tuurvet')
        self.assertEqual(license.temp_company.name, 'Updated company name')

        # verify receiving of notification.
        self.assertEqual(ncount+1, nc.get_unread_notifications_count())

        # approve the change to master data.
        self.controller.approve_master_data_change(license.id)

        # verify the new company details.
        license = self.controller.get_licenses(license.id).pop()
        self.assertEqual(license.company.name, 'Updated company name')
        self.assertEqual(license.company.invoice_person.first_name, 'Second update')
        self.assertEqual(license.company.invoice_person.last_name, 'Tuurvet')

        # verify that the temporary company is gone.
        self.assertEqual(license.temp_company, None)

    def test_temp_company_garbage_collector(self):

        from basesoft.web.contrib.contact.schedule import TempCompanyGarbageCollectorTask

        account = self._create_active_account(license=self._create_license(domain=self.group.license.domain))
        group = account.groups[0].group
        license = group.license
        original_name = license.company.name
        lc = LicenseController(self.sa_session, account_id=account.id)

        md = {
            'company': {
                'name': 'Updated company name',
                'vat_no': 'NL UPD VAT NO',
            },
            'invoice_person_data': {
                'first_name': 'Fri',
                'last_name': 'Tuurvet',
                'gender': 'male',
                'email': 'invoice@frituur.vet'
            }
        }

        # verify that master data can't be changed by themselves.
        self.assertRaises(PermissionDeniedException, lc.update_master_data, license.id, md)

        # give the group the rights to alter their own license master data.
        self._attach_permissions_to_group(group, 'view_license', 'update_license_masterdata')

        # update the master data.
        license = lc.update_master_data(license.id, md)

        temp_comp_id = license.temp_company.id

        self._attach_permissions_to_group(self.group, 'license_masterdata_change_approval', 'view_all_domains', 'create_notification')

        TempCompanyGarbageCollectorTask().execute()
        r = self.sa_session.query(Company).filter(Company.id==temp_comp_id).first()

        self.assertEqual(r, license.temp_company)

        self.controller.approve_master_data_change(license.id)
        self.sa_session.expire(license)
        self.assertEqual(license.temp_company, None)

        TempCompanyGarbageCollectorTask().execute()
        r = self.sa_session.query(Company).filter(Company.id==temp_comp_id).first()

        self.assertEqual(r, None)

    def test_ticket_4013(self):
        account = self._create_active_account(license=self._create_license(domain=self.group.license.domain))
        group = account.groups[0].group
        license = group.license
        lc = LicenseController(self.sa_session, account_id=account.id)

        md = {
            'company': {
                'name': 'Updated company name',
                'vat_no': 'NL UPD VAT NO',
            },
            'invoice_person_data': {
                'is_same_person_as': True
            }
        }

        # give the group the rights to alter their own license master data.
        self._attach_permissions_to_group(group, 'view_license', 'update_license_masterdata')

        # update the master data.
        license = lc.update_master_data(license.id, md)
        self.assertEqual(license.temp_company.contact_person, license.temp_company.invoice_person)

        md = {
            'invoice_person_data': {
                'first_name': 'Fri',
                'last_name': 'Tuurvet',
                'gender': 'male',
                'email': 'invoice@frituur.vet'
            }
        }
        license = lc.update_master_data(license.id, md)
        self.assertNotEqual(license.temp_company.contact_person, license.temp_company.invoice_person)

    def test_disable_login_for_permission(self):
        ac = self._create_active_account()
        self._attach_permissions_to_group(ac.default_group, 'view_all_domains', 'view_all_permissions')

        p = self.sa_session.query(Permission).filter(Permission.name=='view_mock_orders').first()
        p.disable_login = False # make sure logins are possible
        self.sa_session.add(p)
        self.sa_session.flush()
        lc = LicenseController(self.sa_session, account_id=ac.id)

        # create another account
        ac2 = self._create_active_account()
        self._attach_permissions_to_group(ac2.default_group, 'view_mock_orders')
        self.assertTrue(lc.assert_no_disabled_permissions(ac2))

        lc.permission_toggle_disable_login(p.id)

        self.assertRaises(PublicUserException, lc.assert_no_disabled_permissions, ac2)

        # reset permission status
        p.disable_login = False
        self.sa_session.add(p)
        self.sa_session.flush()
