# -*- coding: utf-8 -*-
# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

import datetime
import exceptions

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.contrib.i18n import models as i18n_models
from basesoft.web.contrib.mailing.controllers import MailBaseController
from basesoft.web.core.exceptions import PublicSystemException, PermissionDeniedException
from basesoft.web.contrib.mailing.models import Mail, Mailing, MailResult, MailGroup
from basesoft.web.contrib.contact.models import Company, ContactListContact
from basesoft.web.contrib.contact.controllers import ContactController

from basesoft.web.test.models import MockMail, MockMailing
from basesoft.web.test.mailing import MailController, MockMailingTask

from django.conf import settings

from basesoft import logging
from twisted.trial.unittest import SkipTest
logger = logging.getLogger('basesoft.web.test.unit')

class MailTestCase(iqportal_unittest.IQPortalTestCase):
    """
    """
    def setUp(self):
        self.connect_db()

        self.account = self._create_active_account()
        self.group = self.account.groups[0].group

        self.locale = self.session.query(i18n_models.Locale).filter_by(
            locale='en_GB').one()
        self.locale_nl = self.session.query(i18n_models.Locale).filter_by(
            locale='nl_NL').one()

        self.controller = MailController(self.session, self.locale,
                                         account_id=self.account.id)

    def get_state(self):
        return {
            "mail": {
                "is_wiki": False,
                "mail_from_text": u"MockMailer B.V.",
                "name": u"Some email",
                "objectName": u"MockMail",
                "dbID": None,

                "locales": {
                    'nl_NL': {
                        'subject': u"Mijn test onderworpen",
                        'content': u"Hallo <b>wereld !</b>",
                    },
                    'en_GB': {
                        'subject': u'My test subject',
                        'content': u"Hello <b>world !</b>",
                    },
                },

                "fallbackLocale": {'value': u'en_GB', 'name': u'English' },
            }
        }

    def assertLocales(self, obj, state):
        """
        Helps with asserting locales in an obj
        """
        count = 0 # keeps track of the number of texts in the state

        # start asserting text values
        for locale_str in state:
            for field_name in state[locale_str]:
                for locale_inst in obj.locales:
                    if locale_inst.locale.locale == locale_str:
                        self.assertEquals( getattr(locale_inst, field_name) , state[locale_str][field_name] )
            count = count + 1

        # the length of the locales should be the same as the number of text items tested
        self.assertEquals(count, len(obj.locales) )

    def test_load_state(self):

        self._attach_permissions_to_group(self.group, 'create_mail', 'create_mock_mail')

        state = self.get_state()

        # first save a new state
        mail = self.controller.save_state(state, self.group)

        # test without permission
        self.assertRaises(PermissionDeniedException, self.controller.load_by_mail_id, mail.id)

        # add permissions
        self._attach_permissions_to_group(self.group, 'view_mail', 'view_mock_mail')

        # then load it again
        loaded_state = self.controller.load_by_mail_id(mail.id)

        # update the id in our state
        state['mail']['dbID'] = mail.id

        # now state should be the same as the loaded_state
        try:
            self.assertEquals(state, loaded_state)
        except exceptions.AssertionError, e:
            from django.utils import simplejson
            logger.info('diff failed! check the difference at http://tlrobinson.net/projects/js/jsondiff/')

            from basesoft.web.core.jsonrpc.dispatcher import JsonRpcJSONEncoder

            logger.debug('state: %s' % simplejson.dumps(state, cls=JsonRpcJSONEncoder))
            logger.debug('loaded_state: %s' % simplejson.dumps(loaded_state, cls=JsonRpcJSONEncoder))
            raise e

    def test_save_update_state(self):
        """
        Tests the updating of a mail using a state that has the dbID set
        """

        self._attach_permissions_to_group(self.group, 'create_mail', 'view_mail', 'create_mock_mail', 'view_mock_mail')

        # first save a new mail and get the response
        # (we save it only with view privileges so we can test later)
        state = self.get_state()
        state_response = self.controller.save_state(state, self.group)

        # load the state. this has the dbID set
        loaded_state = self.controller.load_by_mail_id(state_response.id)

        # verify that the canvas id is there
        self.assertNotEquals(loaded_state['mail']['dbID'], None)

        # now we change the name a bit
        loaded_state['mail']['name'] = u'Some updated mail'
        loaded_state['mail']['mail_from_text'] = u'MockMailer2 B.V.'
        loaded_state['mail']['locales']['en_GB']['subject'] = u'My updated test subject'

        # add a new locale
        loaded_state['mail']['locales']['fr_FR'] = {}
        loaded_state['mail']['locales']['fr_FR']['subject'] = u'Test subject - french'
        loaded_state['mail']['locales']['fr_FR']['content'] = u'Content - french'

        # call save again, first without permission
        self.assertRaises(PermissionDeniedException, self.controller.save_state, loaded_state, self.group)

        # add the proper permission
        self._attach_permissions_to_group(self.group, 'update_mail', 'update_mock_mail')

        updated_state_response = self.controller.save_state(loaded_state, self.group)

        # now query it and verify
        mail = self.session.query(Mail).get(updated_state_response.id)
        self.assertEquals(mail.name, u'Some updated mail')
        self.assertEquals(mail.mail_from_text, u'MockMailer2 B.V.')

        self.assertLocales(mail, {
            'nl_NL': {
                'subject': u"Mijn test onderworpen",
                'content': u"Hallo <b>wereld !</b>",
            },
            'en_GB': {
                'subject': u'My updated test subject',
                'content': u"Hello <b>world !</b>",
            },
            'fr_FR': {
                'subject': u'Test subject - french',
                'content': u"Content - french",
            },
        })

        # delete a locale
        loaded_state = self.controller.load_by_mail_id(updated_state_response.id)
        del loaded_state['mail']['locales']['fr_FR']
        updated_state_response = self.controller.save_state(loaded_state, self.group)

        # now query it and verify
        mail = self.session.query(Mail).get(updated_state_response.id)
        self.assertEquals(mail.name, u'Some updated mail')

        self.assertLocales(mail, {
            'nl_NL': {
                'subject': u"Mijn test onderworpen",
                'content': u"Hallo <b>wereld !</b>",
            },
            'en_GB': {
                'subject': u'My updated test subject',
                'content': u"Hello <b>world !</b>",
            }
        })

    def test_save_new_state(self):
        """
        Test the creation of a mail in multiple locales.
        No need to test privileges because there is no object in create mode.
        """
        # test create without permission
        initial_state = self.get_state()
        self.assertRaises(PermissionDeniedException, self.controller.save_state, initial_state, self.group)

        # test create with permissions
        self._attach_permissions_to_group(self.group, 'create_mail', 'create_mock_mail')

        saved_mail = self.controller.save_state(initial_state, self.group)

        # assert the first mail
        mail = self.session.query(MockMail).get(saved_mail.id)
        self.assertEquals(mail.name, u"Some email")
        self.assertEquals(mail.fallback_locale.locale, 'en_GB')

        self.assertLocales(mail, {

            'nl_NL': {
                'subject': u"Mijn test onderworpen",
                'content': u"Hallo <b>wereld !</b>",
            },
            'en_GB': {
                'subject': u'My test subject',
                'content': u"Hello <b>world !</b>",
            },
        })

    def test_save_new_state_wiki(self):
        initial_state = self.get_state()
        initial_state['mail']['is_wiki'] = True

        initial_state['mail']['locales']['nl_NL']['subject_wiki'] = u"Mijn test onderworpen"
        initial_state['mail']['locales']['nl_NL']['content_wiki'] = u"Hallo '''wereld !'''"
        initial_state['mail']['locales']['en_GB']['subject_wiki'] = u"My test subject"
        initial_state['mail']['locales']['en_GB']['content_wiki'] = u"Hello '''world !'''"

        self.assertRaises(PermissionDeniedException, self.controller.save_state, initial_state, self.group)

        # test create with permissions
        self._attach_permissions_to_group(self.group, 'create_mail', 'create_mock_mail')

        saved_mail = self.controller.save_state(initial_state, self.group)

        # assert the first mail
        mail = self.session.query(MockMail).get(saved_mail.id)
        self.assertEquals(mail.name, u"Some email")
        self.assertEquals(mail.is_wiki, True)
        self.assertEquals(mail.fallback_locale.locale, 'en_GB')

        self.assertLocales(mail, {

            'nl_NL': {
                'subject': u"Mijn test onderworpen",
                'content': u"Hallo '''wereld !'''",
            },
            'en_GB': {
                'subject': u'My test subject',
                'content': u"Hello '''world !'''",
            },
        })

    def test_delete_mail_locale(self):
        """
        Test the deleting of a locale from a mail.
        No PERMISSION distinction between locale deleting or full deleting
        """
        self._attach_permissions_to_group(self.group, 'create_mail', 'view_mail', 'create_mock_mail', 'view_mock_mail')
        mail = self.controller.save_state(self.get_state(), self.group)

        # test delete of a locale without permission
        self.assertRaises(PermissionDeniedException, self.controller.delete_mail_locales, mail)

        # test delete of a locale without privileges
        self._attach_permissions_to_group(self.group, 'delete_mail', 'delete_mock_mail')
        self.assertRaises(PermissionDeniedException, self.controller.delete_mail_locales, mail)

        # test delete of a locale with permissions and privileges
        mail_obj = self.session.query(Mail).filter(Mail.id==mail.id).one()
        mg = MailGroup()
        mg.group = self.group
        mg.privilege = Mail.PRIV_DELETE
        mg.group_has_privilege = True
        mail_obj.groups.append(mg)
        self.session.add(mail_obj)
        self.session.flush()

        self.assertEquals(len(self.controller.get_mail_locales(mail)), 2)

        # delete the english locale
        self.controller.delete_mail_locales(mail, self.locale)
        self.assertEquals(len(self.controller.get_mail_locales(mail)), 1)

        # delete all locales left (the dutch one)
        self.controller.delete_mail_locales(mail)
        self.assertEquals(len(self.controller.get_mail_locales(mail)), 0)

    def test_delete_mail(self):
        """
        Test the deletion of a mail
        """
        self._attach_permissions_to_group(self.group, 'create_mail', 'view_mail', 'create_mock_mail', 'view_mock_mail')

        mail = self.controller.save_state(self.get_state(), self.group)

        # test delete without permission
        self.assertRaises(PermissionDeniedException, self.controller.delete_mail, mail.id)

        # test delete without 'delete_mock_mail' permission
        self._attach_permissions_to_group(self.group, 'delete_mail')
        self.assertRaises(PermissionDeniedException, self.controller.delete_mail, mail.id)

        # test delete without privileges
        self._attach_permissions_to_group(self.group, 'delete_mock_mail')
        self.controller.delete_mail(mail.id)
        left_mails = self.controller.get_mails(mail_type='MockMail')
        self.assertEqual(left_mails, [mail])

        # test delete with permissions and privileges
        mail2_state = {
            "mail": {
                "is_wiki": False,
                "mail_from_text": "MockMailer",
                "name": u"Another email",
                "objectName": u"MockMail",
                "dbID": None,

                "locales": {
                    'nl_NL': {
                        'subject': u"Mijn tweede onderwerp",
                        'content': u"Bye <b>wereld !</b>",
                    },
                    'en_GB': {
                        'subject': u"My second subject",
                        'content': u"Bye <b>world !</b>",
                    },
                },

                "fallbackLocale": {'value': u'en_GB', 'name': u'English' },
            }
        }

        # add another email that has privileges for deleting
        mail2 = self.controller.save_state(mail2_state, self.group)

        # this should fail because we don't have privileges for deleting mail
        self.controller.delete_mail([mail.id, mail2.id])
        left_mails = self.controller.get_mails(mail_type='MockMail')
        self.assertEquals(len(left_mails), 2)

        # add the delete privilege to mail too
        mail_obj = self.session.query(Mail).filter(Mail.id==mail.id).one()
        mg = MailGroup()
        mg.group = self.group
        mg.privilege = Mail.PRIV_DELETE
        mg.group_has_privilege = True
        mail_obj.groups.append(mg)
        self.session.add(mail_obj)
        self.session.flush()

        self.controller.delete_mail(mail.id) # delete only one mail

        # test get_mails with the mail_type parameter also
        left_mails = self.controller.get_mails(mail_type='MockMail')
        self.assertEquals(len(left_mails), 1)
        mail2_obj = left_mails[0]
        self.assertEquals(mail2_obj.name, u"Another email")

        # same test, but without the mail_type. should work the same because we only have mock mails
        left_mails = self.controller.get_mails()
        self.assertEquals(len(left_mails), 1)
        mail2_obj = left_mails[0]
        self.assertEquals(mail2_obj.name, u"Another email")

    def test_mail_types(self):
        """
        Test the checking of mail types
        """
        self.assertEquals(self.controller.get_all_mail_types(), [])

        self._attach_permissions_to_group(self.group, 'view_mock_mail')

        new_controller = MailController(self.controller.sa_session, self.controller.locale, self.controller.account_id)

        self.assertEquals(new_controller.get_all_mail_types(), [MockMail])

    def test_mail_replacement_fields(self):
        """
        Test if the replacement fields are retrieved correctly for a given mail type
        """
        default_placeholders = Mail.replacement_fields()
        nr_keys = len(default_placeholders)

        bc = MailBaseController(self.controller.sa_session, self.controller.locale, self.controller.account_id)

        # should work with the MailController for the MockMail
        rf = self.controller.get_replacement_fields('MockMail')
        assert rf.has_key('test.field.mockmail')
        assert rf['test.field.mockmail'] == 'Test field only for MockMail'
        assert len(rf) == nr_keys + 2

        # and also with the MailBaseController
        rf = bc.get_replacement_fields('MockMail')
        assert rf.has_key('test.field.mockmail')
        assert rf['test.field.mockmail'] == 'Test field only for MockMail'
        assert len(rf) == nr_keys + 2

        # should fail if we provide some inexistent type
        self.assertRaises(PublicSystemException, self.controller.get_replacement_fields, 'BogusMail')

    def test_mail_fallback(self):
        """
        Tests all language fallback related stuff in mails
        """

        self._attach_permissions_to_group(self.group, 'create_mail', 'create_mock_mail')

        mail_state = {
            "mail": {
                "mail_from_text": "MockMailer",
                "name": u"Another email",
                "objectName": u"MockMail",
                "dbID": None,

                "locales": {
                    'nl_NL': {
                        'subject': u"Mijn tweede onderwerp",
                        'content': 'Mijn tweede content',
                    },
                    'en_GB': {
                        'subject': u"My second subject",
                        'content': u'My second content',
                    },
                },

                "fallbackLocale": {'value': u'nl_NL'},
            }
        }

        mail = self.controller.save_state(mail_state, self.group)

        locale_en = self.session.query(i18n_models.Locale).filter_by(locale='en_GB').one()
        locale_nl = self.session.query(i18n_models.Locale).filter_by(locale='nl_NL').one()
        locale_de = self.session.query(i18n_models.Locale).filter_by(locale='de_DE').one()

        # english and dutch subject can be retrieved directly
        self.assertEqual(mail.get_locale_mail(locale_en).subject, 'My second subject')
        self.assertEqual(mail.get_locale_mail(locale_nl).subject, 'Mijn tweede onderwerp')

        # german subject should fallback to dutch
        self.assertEqual(mail.get_locale_mail(locale_de).subject, 'Mijn tweede onderwerp')

        # it should also be possible to get the mail with a None locale.. we expect fallback to be returned
        self.assertEqual(mail.get_locale_mail(None).subject, 'Mijn tweede onderwerp')


class MailingTestcase(iqportal_unittest.IQPortalTestCase):
    """
    tests mailing related stuff
    """

    def setUp(self):

        self.connect_db()

        self.account = self._create_active_account()
        self.group = self.account.groups[0].group

        self.locale = self.session.query(i18n_models.Locale).filter_by(locale='en_GB').one()
        self.locale_nl = self.session.query(i18n_models.Locale).filter_by(locale='nl_NL').one()

        # permissions
        self._attach_permissions_to_group(self.group,
            'create_mail', 'create_mock_mail',
            'link_to_contact_list', 'create_contact', 'create_contact_list',
        )

        # controllers
        self.mail_controller = MailController(self.session, self.locale, account_id=self.account.id)
        self.contact_controller = ContactController(self.session, account_id=self.account.id)


    def create_mail(self, name, locales=None, fallback_locale=u'en_GB'):
        """
        helper method for creating mails.. returns the new created mail model object
        """

        mail_state = {
            "mail": {
                "mail_from_text": "MockMailer",
                "name": name,
                "objectName": u"MockMail",
                "dbID": None,
                "locales": {},
                "fallbackLocale": {'value': fallback_locale },
            }
        }

        if locales is None:
            locales = ['nl_NL', 'en_GB']

        for locale in locales:
            mail_state['mail']['locales'][locale] = {
                'subject': '%s: subject: %s' % (name, locale),
                # add unicode character (see ticket #1389)
                'content': u'%s: enqu\u00eate content: %s' % (name, locale),
            }

        return self.mail_controller.save_state(mail_state, self.group)

    def create_contact_list(self, cl_name, contacts=None):
        """
        creates a contact list for the given contacts
        """

        if contacts is None:
            contacts = [
                {'last_name': u'Contact Name', 'email': u'testing@mail.clusternet'}
            ]

        cs = []
        for c in contacts:
            has_company = False
            if c.has_key('company'):
                has_company = True
                company_name = c['company']
                del(c['company'])

            contact = self.contact_controller.save_or_update_contact(None, c, None, self.group.id)
            cs.append(contact)

            if has_company:
                pass
                # add a company for this contact
                company = Company()
                company.contact_person = contact.person
                company.website = u'http://www.google.com'
                company.vat_no = 'RO1234567'
                company.name = company_name

                contact.company = company
                self.session.add(contact)
                self.session.flush()

        cl = self.contact_controller.save_or_update_contact_list(None, cl_name, self.group.id)
        self.contact_controller.link_contacts(cs, cl)

        return cl

    def create_mailing(self, mail, contact_lists, status, send_date):
        """
        Builds and returns a mailing for the given parameters
        """
        mailing = MockMailing()
        mailing.mail = mail
        mailing.mail_from_text = u'Test Mailing'
        mailing.status = status
        mailing.contact_lists = contact_lists
        mailing.send_date = send_date
        self.session.add(mailing)
        self.session.flush()

        return mailing

    def test_simple_mailing(self):
        """
        Sends out one mail to one contact list with one contact.. keep it simple here.
        """

        # test if mailing task builds a correct plain_text out of a html mail
        task = MockMailingTask()
        html = '%s%s%s' % (
               'Hello <strong>GÜNTHER</strong><br> click link:<br>',
               '<a class = "ggg" href="http://nicoleta.dev.cluj:8000/iqportal.tnt.response/display/368f6f7f/" style="color:#DE#">link label</a>',
               '<br><br>logo:<br><img  src="http://192.168.1.11:8000/site_media/basesoft.web.core/themes/icecream/media/arrow-down.png">')
        text1 = "Hello GÜNTHER\n click link:\nhttp://nicoleta.dev.cluj:8000/iqportal.tnt.response/display/368f6f7f/\n\nlogo:\n"
        text2 = task.get_plain_text(html)
        self.assertEquals(text1, text2)

        # create mailing
        mail = self.create_mail(u'mail 1')
        contact_list = self.create_contact_list(u'Contact List 1')
        mailing = self.create_mailing(mail, [contact_list], Mailing.STATUS_SCHEDULED, datetime.datetime.now())

        # now a mailing is ready to be sent out.. execute the task like the
        # scheduler would do
        task.execute()
        self.session.expire(mailing)

        mailing = self.session.query(MockMailing).filter(MockMailing.id==mailing.id).first()

        # expect that the status and the result of the mailing is sent
        self.assertEquals(mailing.status, Mailing.STATUS_SENT)

        if settings.EMAIL_HOST is not None:
            self.assertEquals(mailing.results[0].status, MailResult.STATUS_ACCEPTED)

    def test_mailing_future(self):
        """
        Tests that a mailing in the future wont be send out
        """
        mail = self.create_mail(u'mail 1')
        contact_list = self.create_contact_list(u'Contact List 1')

        # schedule for tomorrow
        today = datetime.datetime.now()
        tomorrow = today+datetime.timedelta(days=1)
        mailing = self.create_mailing(mail, [contact_list], Mailing.STATUS_SCHEDULED, tomorrow)

        # execute
        MockMailingTask().execute()
        mailing = self.session.query(MockMailing).filter(MockMailing.id==mailing.id).first()

        # mail shouldnt have been sent yet
        self.assertEquals(mailing.status, Mailing.STATUS_SCHEDULED)

    def test_ignore_contact(self):
        """
        Test that ignores sending to a contact that returns 'false' in task.format_contact_addr_to method.
        The mock test task is configured to ignore sending to people that are called 'Kees'
        """
        mail = self.create_mail(u'mail 1')
        contact_list = self.create_contact_list(u'Contact List 1', contacts=[
            {'last_name': u'Jan', 'email': u'testing@mail.clusternet'},
            {'last_name': u'Kees', 'email': u'noreply@iqportalmail.com'},
            {'last_name': u'Piet', 'email': u'testing@mail.clusternet'},
        ])

        mailing = self.create_mailing(mail, [contact_list], Mailing.STATUS_SCHEDULED, datetime.datetime.now())
        MockMailingTask().execute()

        self.session.expire(mailing)

        mailing = self.session.query(MockMailing).filter(MockMailing.id==mailing.id).first()

        self.assertEquals(mailing.status, Mailing.STATUS_SENT)
        self.assertEquals(len(mailing.results), 3)
        kees = False
        for i in mailing.results:
            if i.contact.person.last_name == 'Kees' and i.status == MailResult.STATUS_IGNORED:
                kees = True
        if not kees:
            self.fail('Kees does not have status Ignored.')

    def test_mail_settings(self):
        """
        See ticket #3168
        """

        # create a contact list
        mail = self.create_mail(u'mail settings text 1')
        contact_list = self.create_contact_list(u'Contact List 1', contacts=[
            {'last_name': u'Jan', 'email': u'testing@mail.clusternet'},
            {'last_name': u'Fred', 'email': u'testing@mail.clusternet'},
            {'last_name': u'Piet', 'email': u'testing@mail.clusternet'},
        ])

        for clc in contact_list.contact_list_contacts:
            # set the mail_settings for one of them to CONTACT_RECEIVES_NO_MAIL
            if clc.contact.person.last_name == 'Fred':
                clc.mail_settings = ContactListContact.CONTACT_RECEIVES_NO_MAIL
                self.session.add(clc)
                break

        # send the mailing.
        mailing = self.create_mailing(mail, [contact_list], Mailing.STATUS_SCHEDULED, datetime.datetime.now())
        MockMailingTask().execute()

        # verify that the contact is not in MailResult
        mailing = self.session.query(MockMailing).filter(MockMailing.id==mailing.id).first()

        self.assertEquals(mailing.status, Mailing.STATUS_SENT)
        self.assertEquals(len(mailing.results), 2)

    def test_replacement_regexp(self):
        """
        """

        import re
        regexp = re.compile(Mail.regexp)

        test_good_replacement_images = (
            '<img class="replacement-field" title="contact.last_name" src="/basesoft.web.core/rte/make_image/Contact%3ALast%20Name"/>',
            '<img  class="replacement-field" title="account.activation.link" src="/basesoft.web.core/rte/make_image/Account:Activation%20Link"/>',
            '<img class="replacement-field" title="account.activation.link" src="/basesoft.web.core/rte/make_image/Account:Activation%20Link"/>',
            '<img title="contact.last_name" src="/basesoft.web.core/rte/make_image/Contact%3ALast%20Name" class="replacement-field"/>',
            '<img class="replacement-field" title=account.activation.link src="/basesoft.web.core/rte/make_image/Account:Activation%20Link"/>',
            '<img class="replacement-field" title=account.activation.link src=/basesoft.web.core/rte/make_image/Account:Activation%20Link/>',
            '<img  class="replacement-field " title=wave.link src="basesoft.web.core/rte/make_image/Wave%3AWave%20link" width=89 height=12>',
            '<img  style="font-family: yui-tmp;" class="replacement-field " title="person.last_name" src="/basesoft.web.core/rte/make_image/Person%3ALast%20name">',
            '<img  title="iquest.survey.link" class="replacement-field " style="line-height: 1.22;" src="/basesoft.web.core/rte/make_image/Survey%3ASurvey%20link">',
        )

        test_bad_replacement_images = (
            '<img title="contact.last_name" src="/basesoft.web.core/rte/make_image/Contact%3ALast%20Name" />',
            '<img src="/basesoft.web.contrib.file/view/cf13aa20_basesoft.jpg"/>',
        )

        def replace_field(match_obj):
            return 'matched'

        for rfi in test_good_replacement_images:
            html = re.sub(regexp, replace_field, rfi)
            self.assertEqual(html, 'matched')

        for rfi in test_bad_replacement_images:
            html = re.sub(regexp, replace_field, rfi)
            self.assertEqual(html, rfi)

    def test_embed_regexp(self):
        import re
        regexp = re.compile(Mail.regexp_embed)

        test_good_embed = (
            '<img  src="basesoft.web.contrib.file/view/8854b1a4_TNT%20Sure%20we%20can.jpg">',
            '<img  src="https://iqportal.basesoft.com/basesoft.web.contrib.file/view/f1a83233_nbbu%20logo.jpg" alt="-">',
            '<img  style="width: 307px; height: 61px;" src="https://iqportal.basesoft.com/basesoft.web.contrib.file/view/c00e012d_LOGO%20newcom%20RC%20klein.jpg" alt="-">',
        )

        def replace_field(match_obj):
            return 'cid:image1'

        for rfi in test_good_embed:
            html = re.sub(regexp, replace_field, rfi)
            self.assertEqual(html, 'cid:image1')

    def test_replacement_fields_wiki(self):
        """
        This test should have the same result as the non-wiki variant
        """
        mail_state = {
            "mail": {
                "is_wiki": True,
                "mail_from_text": "MockMailer",
                "name": u"Another email",
                "objectName": u"MockMail",
                "dbID": None,

                "locales": {
                    'nl_NL': {
                        'subject_wiki': u"test_replacement_fields",
                        'content_wiki': u"""Geachte @Person:person.last_name, [[BR]][[BR]]
                                  Klikkie niet op de volgende link svp: [[BR]][[BR]]
                                  Company Name @Company:company.name,
                                  @Mock:test.field.link""",
                    }
                },

                "fallbackLocale": {'value': u'nl_NL'},
            }
        }

        company_name = unicode(self._uniquestring('Henks-R-US %s')[0:39])

        mail = self.mail_controller.save_state(mail_state, self.group)
        contact_list = self.create_contact_list(u'Contact List 1', contacts=[
            {'last_name': u'Jan', 'email': u'testing@mail.clusternet'},
            {'last_name': u'Henk', 'email': u'testing@mail.clusternet', 'company': company_name},
            {'last_name': u'Piet', 'email': u'testing@mail.clusternet'},
        ])

        self.create_mailing(mail, [contact_list], Mailing.STATUS_SCHEDULED, datetime.datetime.now())

        http_host = self._uniquestring('http_host')[:40]
        license = contact_list.contacts[0].groups[0].group.license
        license.http_host = http_host
        self.session.add(license)
        self.session.flush()

        t = MockMailingTask()
        t.execute()

        # assert 3 messages were sent
        self.assertEquals(len(t.sent_messages), 3)

        # check if in the mime-message there is a html version that contains the replaced text for contact-last-name for each sent mail
        for name,i in [('Jan',0), ('Henk',1), ('Piet',2)]:
            mime_msg = t.sent_messages[i]
            html = mime_msg.get_payload(0).as_string()

            self.assertTrue( html.find('Geachte %s,' % name) > 0)
            if name == 'Henk': # company replacement only works for ComposeMail see ticket #2770
                pass
            else:
                # check the company was substituted with an empty string
                self.assertTrue( html.find('Company Name ,') > 0)

            # assert http_host
            self.assertTrue( html.find(http_host) > 0)

    def test_replacement_fields(self):

        mail_state = {
            "mail": {
                "mail_from_text": "MockMailer",
                "name": u"Another email",
                "objectName": u"MockMail",
                "dbID": None,

                "locales": {
                    'nl_NL': {
                        'subject': u"test_replacement_fields",
                        'content': u"""Geachte <img class="replacement-field" src="/basesoft.web.core/rte/make_image/Contact:Last%20Name" title="person.last_name"/>, <br /><br />
                                  Klikkie niet op de volgende link svp: <br /><br />
                                  Company Name <img class="replacement-field" src="/basesoft.web.core/rte/make_image/Contact:Company:Name" title="contact.company.name"/>,
                                  <img class="replacement-field" src="/basesoft.web.core/rte/make_image/Linky" title="test.field.link"/>""",
                    },
                    'en_GB': {
                        'subject': u"test_replacement_fields",
                        'content': u"""Dear <img class="replacement-field" src="/basesoft.web.core/rte/make_image/Contact:Last%20Name" title="person.last_name"/>, <br /><br />
                                  Please dont clicky the following link: <br /><br />
                                  Company Name <img class="replacement-field" src="/basesoft.web.core/rte/make_image/Contact:Company:Name" title="contact.company.name"/>,
                                  <img class="replacement-field" src="/basesoft.web.core/rte/make_image/Linky" title="test.field.link"/>""",
                    },
                },

                "fallbackLocale": {'value': u'nl_NL'},
            }
        }

        company_name = unicode(self._uniquestring('Henks-R-US %s')[0:39])

        mail = self.mail_controller.save_state(mail_state, self.group)
        contact_list = self.create_contact_list(u'Contact List 1', contacts=[
            {'last_name': u'Jan', 'email': u'testing@mail.clusternet'},
            {'last_name': u'Henk', 'email': u'testing@mail.clusternet', 'company': company_name},
            {'last_name': u'Piet', 'email': u'testing@mail.clusternet'},
        ])

        self.create_mailing(mail, [contact_list], Mailing.STATUS_SCHEDULED, datetime.datetime.now())

        http_host = self._uniquestring('http_host')[:40]
        license = contact_list.contacts[0].groups[0].group.license
        license.http_host = http_host
        self.session.add(license)
        self.session.flush()

        t = MockMailingTask()
        t.execute()

        # assert 3 messages were sent
        self.assertEquals(len(t.sent_messages), 3)

        # check if in the mime-message there is a html version that contains the replaced text for contact-last-name for each sent mail
        for name,i in [('Jan',0), ('Henk',1), ('Piet',2)]:
            mime_msg = t.sent_messages[i]
            html = mime_msg.get_payload(0).as_string()

            self.assertTrue( html.find('Geachte %s,' % name) > 0)
            if name == 'Henk': # company replacement only works for ComposeMail see ticket #2770
                pass
            else:
                # check the company was substituted with an empty string
                self.assertTrue( html.find('Company Name ,') > 0)

            # assert http_host
            self.assertTrue( html.find(http_host) > 0)

    def test_replacement_with_images(self):

        mail_state = {
            "mail": {
                "mail_from_text": "MockMailer",
                "name": u"Another email",
                "objectName": u"MockMail",
                "dbID": None,

                "locales": {
                    'en_GB': {
                        'subject': u"test_replacement_fields",
                        'content': u'test1<br><img src="https://www.nbbu.nl/sites/all/themes/nbbu/images/nbbu_logo_left.jpg">'+
                        '<br>test2<br>dear <img  class="replacement-field " title="person.last_name" '+
                        'src="/basesoft.web.core/rte/make_image/Person%3ALast%20name">,<br>end mail'
                    },
                },

                "fallbackLocale": {'value': u'en_GB'},
            }
        }

        contact = self.contact_controller.save_or_update_contact(None, {'last_name': u'Jan', 'email': u'testing@mail.clusternet'}, None)
        mail = self.mail_controller.save_state(mail_state, self.group)
        html = mail.get_html_mail(contact.person, {})
        shouldbe = u'test1<br><img src="https://www.nbbu.nl/sites/all/themes/nbbu/images/nbbu_logo_left.jpg"><br>test2<br>dear Jan,<br>end mail'
        self.assertEqual(html, shouldbe)

# below is for running the test from eclipse
if __name__ == '__main__':
    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_compose.py', 'basesoft.web.test.unit.test_mailing.MailingTestcase.test_ignore_contact']
    run()
    sys.exit()
