# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.
import datetime
from basesoft.web.core.util.controllers import AccountConfigController

from django.conf import settings
from django.utils import simplejson

from basesoft.util.virtual_date import set_date
from basesoft.web.contrib.i18n.models import Locale

from basesoft.web.contrib.license.models import License
from basesoft.web.contrib.notification.mailing import MailController
from basesoft.web.contrib.notification.schedule import NotificationReminderMailTask
from basesoft.web.core.jsonrpc.dispatcher import json_object_hook

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.core.exceptions import PermissionDeniedException

from basesoft.web.contrib.notification.controllers import NotificationController
from basesoft.web.contrib.notification.models import Notification, NotificationReminderMail


class NotificationControllerTestCase(iqportal_unittest.IQPortalTestCase):
    """
    This tests the Notification Controller.
    """
    def setUp(self):
        """
        """
        self.connect_db()

        self.account = self._create_active_account()
        self.group = self.account.groups[0].group
        self.controller = NotificationController(self.session, account_id=self.account.id)
        self._attach_permissions_to_group(self.group, 'view_notification')

    def test_create_system_notification(self):
        """
        tests sending system notifications
        """
        subject = u'test subject'
        text = u'test text'

        # just send to this group
        notif = self.controller.set_notification(subject, text)
        id = self.controller.create_system_notification(notif, self.group, None, True).id
        n = self.controller.get_notifications(id)[0]

        self.assertEqual(n.subject, subject)
        self.assertEqual(n.text, text)
        self.assertEqual(n.sender, None)
        self.assertEqual(len(n.groups), 1)
        self.assertEqual(n.groups[0].privilege, n.PRIV_VIEW)
        self.assertEqual(n.groups[0].group_has_privilege, True)
        self.assertEqual(n.groups[0].license_has_privilege, False)
        self.assertEqual(n.groups[0].domain_has_privilege, False)

        # create another one for the whole domain
        notif = self.controller.set_notification(subject, text)
        id = self.controller.create_system_notification(notif, self.group, self.group, False, False, True).id
        n = self.controller.get_notifications(id)[0]

        self.assertEqual(len(n.groups), 1)
        self.assertEqual(n.sender, self.group)
        self.assertEqual(n.groups[0].privilege, n.PRIV_VIEW)
        self.assertEqual(n.groups[0].group_has_privilege, False)
        self.assertEqual(n.groups[0].license_has_privilege, False)
        self.assertEqual(n.groups[0].domain_has_privilege, True)

    def test_create_notification(self):
        """
        tests sending user notifications
        """
        self._attach_permissions_to_group(self.group, 'create_notification')

        # first create some recipients

        # create another group in same license
        recip1 = self._create_active_account(self.group.license).default_group
        self._attach_permissions_to_group(recip1, 'view_notification')

        # one in another license, but same domain
        l = self._create_license(domain=self.group.license.domain)
        recip2 = self._create_active_account(l).default_group
        self._attach_permissions_to_group(recip2, 'view_notification')

        # and one in another domain
        recip3 = self._create_active_account().default_group
        self._attach_permissions_to_group(recip3, 'view_notification')

        subject = u'test subject'
        text = u'test text'

        notif = self.controller.set_notification(subject, text)
        # cannot send to another group
        # not in his license
        self.assertRaises(PermissionDeniedException,
                          self.controller.create_notification, notif, recip1, True, False, False)
        # not in another license
        self.assertRaises(PermissionDeniedException,
                          self.controller.create_notification, notif, recip2, True, False, False)
        # and also not in another domain
        self.assertRaises(PermissionDeniedException,
                          self.controller.create_notification, notif, recip3, True, False, False)

        self._attach_permissions_to_group(self.group, 'view_license')
        # now can send to groups in his license
        id = self.controller.create_notification(notif, recip1, True, False, False).id
        n = self.session.query(Notification).filter(Notification.id==id).one()
        self.assertEqual(len(n.groups), 1)
        self.assertEqual(n.sender, self.group)

        # should also be able to send to all it's own license
        notif = self.controller.set_notification(subject, text)
        id = self.controller.create_notification(notif, [self.group, recip1], False, True, False).id
        n = self.session.query(Notification).filter(Notification.id==id).one()
        # two groups from the same license, so just one entry
        self.assertEqual(len(n.groups), 1)
        self.assertEqual(n.sender, self.group)

        # cannot sent to all its own domain
        self.assertRaises(PermissionDeniedException,
                          self.controller.create_notification, notif, self.group, False, False, True)
        # cannot send to another license, but same domain
        self.assertRaises(PermissionDeniedException,
                          self.controller.create_notification, notif, recip2, True, False, False)
        # cannot send to a group in another domain
        self.assertRaises(PermissionDeniedException,
                          self.controller.create_notification, notif, recip3, True, False, False)

        self._attach_permissions_to_group(self.group, 'view_domain')
        # not can send to groups in all his domain
        notif = self.controller.set_notification(subject, text)
        id = self.controller.create_notification(notif, [recip1, recip2], True, False, False).id
        n = self.session.query(Notification).filter(Notification.id==id).one()
        # sent to two groups
        self.assertEqual(len(n.groups), 2)
        self.assertEqual(n.sender, self.group)

        notif = self.controller.set_notification(subject, text)
        id = self.controller.create_notification(notif, [self.group, recip2], False, False, True).id
        n = self.session.query(Notification).filter(Notification.id==id).one()
        # even tho recipients is a list, still only one entry in group because they are both for the same domain
        self.assertEqual(len(n.groups), 1)
        self.assertEqual(n.sender, self.group)

        # cannot send to another domain
        self.assertRaises(PermissionDeniedException,
                          self.controller.create_notification, notif, recip3, True, False, False)

        self._attach_permissions_to_group(self.group, 'view_all_domains')
        # should be able to send anywhere
        notif = self.controller.set_notification(subject, text)
        id = self.controller.create_notification(notif, [recip3, self.group], False, False, True).id
        n = self.session.query(Notification).filter(Notification.id==id).one()
        self.assertEqual(len(n.groups), 2)
        self.assertEqual(n.sender, self.group)


    def test_load_notification(self):
        """
        Tests loading of a notification by a recipient (so also marking as read)
        """
        self._attach_permissions_to_group(self.group, 'view_license', 'create_notification')

        subject = u'test subject'
        text = u'test text'

        notif = self.controller.set_notification(subject, text)
        notification = self.controller.create_notification(notif, self.group, True, False, False)

        self.assertEqual(notification.sender, self.group)

        n = self.controller.load_notification(notification.id, True)
        self.assertEqual(n['id'], notification.id)
        self.assertEqual(n['is_read'], True)

        # now test mark notification as archived
        self.controller.mark_as_archived(notification.id)
        self.assertEqual(notification.archived, True)

    def test_notification_callback(self):

        subject = 'NOTIF TEST'
        text = 'This is a test for notifications'
        callback = {
            'js_module': 'iqreport_manager',
        }
        self._attach_permissions_to_group(self.group, 'view_license', 'create_notification')

        notification = self.controller.set_notification(subject, text, callback=callback)
        notif = self.controller.create_notification(notification, [self.group])

        n = self.sa_session.query(Notification).filter(Notification.id == notif.id).one()
        c = simplejson.loads(n.callback, object_hook=json_object_hook)
        self.assertEqual(c, callback)
        self.assertEqual(n.subject, subject)

    def test_notification_reminder_mail_weekly(self):
        self.sa_session.execute('UPDATE tblNotification SET is_read = 1')

        # set virtual date to a Monday of a week in the middle of the month.
        set_date(datetime.date(2013, 1, 21))

        self._attach_permissions_to_group(self.group, 'view_license', 'create_notification')

        # create a notification reminder mail
        self._setup_reminder_mail()

        mail = self.controller.get_reminder_mail()
        self.assertNotEqual(mail, None)

        # create a notification
        notification = self.controller.set_notification('test', 'subj')
        notification = self.controller.create_notification(notification, [self.group])

        # verify notification/account is in the list of recipients
        n = self.controller.get_unread_notifications()
        self.assertTrue(notification in n)

        # trigger the notification mail task
        cnt = NotificationReminderMailTask().execute()
        self.assertEqual(1, cnt)

    def test_notification_reminder_mail_monthly(self):
        self.sa_session.execute('UPDATE tblNotification SET is_read = 1')

        # update account config
        ac = AccountConfigController(self.sa_session, account_id=self.account.id)
        ac.set('basesoft.web.contrib.notification.reminder.mail', NotificationReminderMail.SEND_MONTHLY)

        # set virtual date to a Monday of a week in the middle of the month.
        set_date(datetime.date(2013, 1, 21))

        self._attach_permissions_to_group(self.group, 'view_license', 'create_notification')

        # create a notification reminder mail
        self._setup_reminder_mail()

        mail = self.controller.get_reminder_mail()
        self.assertNotEqual(mail, None)

        # create a notification
        notification = self.controller.set_notification('test', 'subj')
        notification = self.controller.create_notification(notification, [self.group])

        # verify notification/account is in the list of recipients
        n = self.controller.get_unread_notifications()
        self.assertTrue(notification in n)

        # trigger the notification mail task
        cnt = NotificationReminderMailTask().execute()
        self.assertEqual(0, cnt)

        # now set virtual date to the first Monday of the month.
        set_date(datetime.date(2013, 1, 7))

        cnt = NotificationReminderMailTask().execute()
        self.assertEqual(1, cnt)

    def _setup_reminder_mail(self):
        for mail in self.session.query(NotificationReminderMail).all():
            self.session.delete(mail)

        # flush to commit to db
        self.session.flush()

        locale = self.session.query(Locale).filter_by(locale='en_GB').one()

        # mail controller
        self._attach_permissions_to_group(self.group, 'create_mail', 'manage_notification_reminder_mail')
        controller = MailController(self.session, locale, account_id=self.account.id)

        # notification reminder mail
        mail_state = {
            "mail": {
                "mail_from_text": "MockMailer",
                "name": u"NotificationReminderMail",
                "objectName": u"NotificationReminderMail",
                "dbID": None,

                "locales": {
                    'en_GB': {
                        'subject_wiki': u"Unread notifications",
                        'content_wiki': u'Hello @person:first_name, please check your notifications: @notification:subjects  Go here yo, @notification:link'
                    },
                },
                "is_wiki": True,
                "fallbackLocale": {'value': u'en_GB'},
            }
        }

        owner = self.sa_session.query(License).filter(License.name==settings.SITE_OWNER_LICENSE_NAME).one()

        mail = controller.save_state(mail_state, self.group)
        controller.grant_privilege_to(mail, NotificationReminderMail.PRIV_USE_FOR_SENDING, group=True, target_group=owner.default_group, check_own_privs=False, world=True)
