# -*- coding: utf-8 -*-
'''
Created on Oct 2, 2013

@author: marten
'''

from decimal import Decimal
import datetime

import time

from sqlalchemy import func, and_

from basesoft.util.virtual_date import today
from basesoft.web.contrib.order.rule.actions import RuleActionController
from basesoft.web.contrib.order.rule.conditions import RuleConditionController

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.contrib.order.models import Order, NewInvoiceMail,\
    PaymentMethod, PaymentMethodBankTransfer, OrderPaymentStatus,\
    PaymentMethodOgone, OrderRule, OrderLineRule, OrderRuleCondition, OrderRuleAction, TaxClass, OrderLine
from basesoft.web.core.exceptions import PublicUserException, PermissionDeniedException
from basesoft.web.contrib.order.controllers import AbstractOrderController, TaxController, GLAccountController,\
    PaymentController, BankTransferPaymentController, OgonePaymentController
from basesoft.web.test.models import MockOrder, MockOrderLine, MockOrder2
from basesoft.web.contrib.i18n.models import Country, Locale
from basesoft.web.contrib.order.mailing import MailController
from basesoft.web.contrib.order.schedule import NewInvoiceMailTask
from basesoft.web.core.util.controllers import LicenseConfigController, transaction_decorator
from basesoft.util.virtual_date import now

from django.conf import settings
from django.http import HttpRequest
from basesoft.web.contrib.order import views as payment_method_ogone_views
from basesoft.web.core.util.db import Session

class MockOrderController(AbstractOrderController):

    _instance_class = MockOrder

    def save_or_update_order(self, _id, lines, **kwargs):
        return self._save_or_update_order(_id, lines, **kwargs)

    # rule mock stuff

    def _get_available_rule_conditions(self, order_type):

        conditions = super(MockOrderController, self)._get_available_rule_conditions(order_type)
        conditions.append(
            RuleConditionMockQuantityGt(self),
        )
        return conditions

    def _get_available_rule_actions(self, order_type):

        actions = super(MockOrderController, self)._get_available_rule_actions(order_type)
        actions.append(
            RuleActionMockAllFree(self),
        )
        return actions

class MockOrder2Controller(AbstractOrderController):

    _instance_class = MockOrder2

    def save_or_update_order(self, _id, lines, **kwargs):
        return self._save_or_update_order(_id, lines, **kwargs)

class RuleConditionMockQuantityGt(RuleConditionController):
    """
    mock rule condition that checks if all line quantities are above a certain number
    """

    name = 'mock_quantity_gte'
    label = 'Quantity over X'
    type = OrderRuleCondition.TYPE_INTEGER

    def matches(self, order, rule, condition):
        return sum([x.quantity for x in order.lines]) >= condition.value_int

class RuleActionMockAllFree(RuleActionController):
    """
    mock rule action that negates all existing lines. the final total amount is 0
    """

    name = 'mock_all_free'
    label = 'All free!'
    type = OrderRuleAction.TYPE_NONE

    def execute(self, order, rule, action, dry_run):
        for line_i in range(len(order.lines)):
            tax_class = self._order_controller._get_privileged_object(TaxClass, where=and_(TaxClass.code == order.lines[line_i].tax_class_code))[0]
            self._order_controller._save_or_update_order_line(order, len(order.lines), order.merchant, action.full_description,
                                            quantity=order.lines[line_i].quantity * -1, price=order.lines[line_i].price_per_unit,
                                            polymorphic_type=OrderLineRule, gl_account_code=action.gl_account_code,
                                            tax_class = tax_class, dry_run=dry_run)


class OrderControllerTestCase(iqportal_unittest.IQPortalTestCase):
    """
    basesoft.web.test.unit.test_order_controller.OrderControllerTestCase
    """

    def setUp(self):
        self.connect_db()

        self.merchant = self._create_license()
        self.ma = self._create_active_account(self.merchant)
        self.locale = self.session.query(Locale).filter_by(locale='en_GB').one()
        self._add_address_and_bank_account_to_company(self.merchant.company)

        self.account_license = self._create_license(domain=self.merchant.default_group.license.domain)
        self._add_address_and_bank_account_to_company(self.account_license.company)
        self.account = self._create_active_account(license=self.account_license)

        self.group = self.account.groups[0].group
        self.controller = MockOrderController(self.sa_session, account_id=self.account.id)

    def _get_new_invoice_mail_state(self):
        return {
            "mail": {
                "mail_from_text": "MockMailer",
                "name": u"New Invoice Mail",
                "objectName": u"NewInvoiceMail",
                "dbID": None,

                "locales": {
                    'en_GB': {
                        'subject': u"New Invoice for Order",
                        'content': u'Dear <img  src="/basesoft.web.core/rte/make_image/Person%3ALast%20Name" title="person.last_name" class="replacement-field "> a new invoice ... etc.<img  class="replacement-field " title="order.id" src="/basesoft.web.core/rte/make_image/Order%3AOrder%20ID"> '
                    },
                },
                "fallbackLocale": {'value': u'en_GB'},
            }
        }

    def _setup_tax_and_gl_account(self):
        self._attach_permissions_to_group(self.account.groups[0].group, 'manage_tax_classes', 'manage_country_tax', 'manage_gl_accounts', 'view_domain', 'view_license')

        c = TaxController(self.sa_session, account_id = self.account.id)
        tax_class = c.save_tax_class_state(None, {'name': 'Administrative Fee Tax', 'code': 'MOCK_ORDER_TAX_CLASS'})
        tcc = c.save_country_tax_state(None, {
            'country_id': 155,
            'tax_class_id': tax_class.id,
            'tax_rates': [{
                'start_date': today(),
                'rate': Decimal(21, 0)
            }]
        })

        c = GLAccountController(self.sa_session, account_id = self.account.id)
        ga = c.save_gl_account_state(None, {'merchant_id': self.merchant.id, 'code': 'MOCK_ORDER_GL_ACCOUNT', 'account_number': 'GL1234'})

        return tax_class, tcc, ga

    def test_minimum_invoiceable_amount(self):
        self._setup_tax_and_gl_account()

        lcc = LicenseConfigController(self.sa_session, scope=self.merchant)
        lcc.set('basesoft.web.contrib.order.minimum_invoiceable_amount', 100)

        state = {
            'merchant': self.merchant,
        }
        lines = [{
            'description': 'Some random product',
            'polymorphic_type': MockOrderLine,
            'quantity': 1,
            'price': Decimal('49.99')
        }]
        order = self.controller.save_or_update_order(None, lines, **state)
        self.assertEqual(order.invoiced_amount, Decimal(0,0))

        order.status = Order.STATUS_SUBMITTED
        self.sa_session.add(order)

        self.sa_session.expire(order)
        self.assertRaises(PermissionDeniedException, self.controller.get_unexported_orders_for_merchant, self.merchant.id)
        self._attach_permissions_to_group(self.account.default_group, 'view_mock_orders')
        self.controller.grant_privilege_to(order, [Order.PRIV_EXPORT], True, check_own_privs=False)

        orders = [o.id for o in self.controller.get_unexported_orders_for_merchant(self.merchant.id)]
        self.assertTrue(order.id not in orders)

    def test_simple_order(self):

        self._setup_tax_and_gl_account()

        state = {
            'merchant': self.merchant,
        }
        lines = [{
            'description': 'Some random product',
            'quantity': 2,
            'price': Decimal('299.99')
        }, {
            'description': 'Some other product',
            'polymorphic_type': MockOrderLine,
            'quantity': 7,
            'price': Decimal('15.99')
        }]

        total_line_1 = 2*Decimal('299.99')
        total_line_2 = 7*Decimal('15.99')
        total = total_line_1+total_line_2

        total_line_1_inc = total_line_1
        total_line_2_inc = total_line_2*Decimal('1.21')
        total_inc = total_line_1_inc+total_line_2_inc

        order = self.controller.save_or_update_order(None, lines, **state)
        self.assertEqual(len(order.lines), 2)

        self.assertEqual(order.merchant, self.merchant)
        self.assertEqual(order.lines[0].line_amount_excluding_vat, total_line_1)
        self.assertEqual(order.lines[1].line_amount_excluding_vat, total_line_2)
        self.assertEqual(order.total_amount_excluding_vat, total)

        self.assertEqual(order.lines[0].line_amount_including_vat, total_line_1_inc)
        self.assertEqual(order.lines[1].line_amount_including_vat, total_line_2_inc)
        self.assertEqual(order.total_amount_including_vat, total_inc)

        self.assertEqual(order.lines[0].vat_rate, 0)
        self.assertEqual(order.lines[1].vat_rate, Decimal('21.0'))

        self.assertEqual(order.lines[0].gl_account_number, None)
        self.assertEqual(order.lines[1].gl_account_number, 'GL1234')

    def test_remove_line(self):
        # test on Mock order for ticket #4076
        self._setup_tax_and_gl_account()

        lines = [{
            'description': 'Some random product',
            'polymorphic_type': MockOrderLine,
            'quantity': 2,
            'price': Decimal('299.99')
        }, {
            'description': 'Some other product',
            'polymorphic_type': MockOrderLine,
            'quantity': 7,
            'price': Decimal('15.99')
        }]

        order = self.controller.save_or_update_order(None, lines, merchant=self.merchant)
        self.controller.submit_order(order)
        self.assertRaises(PublicUserException, self.controller.remove_line, order, order.lines[0].id)

        order.status = MockOrder.STATUS_PENDING
        self.sa_session.add(order)
        self.sa_session.flush()

        self.controller.remove_line(order, order.lines[0].id)
        order = self.controller.get(order.id).pop()
        self.assertEqual(len(order.lines), 1)

    def test_remove_line_large_order(self):
        self._setup_tax_and_gl_account()

        lines = []
        for x in range(1,1000):
            lines.append({
                'description': 'My lines %s' % x,
                'polymorphic_type': MockOrderLine,
                'quantity': 1,
                'price': Decimal('1.00')*x
            })
        order = self.controller.save_or_update_order(None, lines, merchant=self.merchant)
        order.status = MockOrder.STATUS_PENDING
        self.sa_session.add(order)
        self.sa_session.flush()

        self.controller.logger.info('START REMOVE LINE')
        self.controller.remove_line(order, order.lines[0].id)
        self.controller.logger.info('END REMOVE LINE')
        order = self.controller.get(order.id).pop()
        self.assertEqual(len(order.lines), 998)

    def test_max_line_quantity(self):
        self._setup_tax_and_gl_account()

        state = {
            'merchant': self.merchant,
        }
        lines = [{
            'description': 'Some random product',
            'quantity': 2,
            'price': Decimal('299.99')
        }, {
            'description': 'Some other product',
            'polymorphic_type': MockOrderLine,
            'quantity': MockOrderLine.MAX_LINE_QUANTITY+1,
            'price': Decimal('15.99')
        }]

        self.assertRaises(PublicUserException, self.controller.save_or_update_order, None, lines, **state)

    def test_order_invoice(self):

        # add a new invoice mail.
        self._attach_permissions_to_group(self.ma.default_group, 'create_mail', 'view_mail', 'create_invoice_mail', 'view_invoice_mail', 'manage_gl_accounts', 'view_domain', 'view_license')

        controller = MailController(self.session, self.locale, account_id=self.ma.id)
        mail = controller.save_state(self._get_new_invoice_mail_state())
        controller.grant_privilege_to(mail, NewInvoiceMail.PRIV_USE_FOR_SENDING, group=True, target_group=self.merchant.default_group, check_own_privs=False, world=True)

        self._setup_tax_and_gl_account()

        state = {
            'merchant': self.merchant
        }

        lines = [{
            'description': 'Some random product',
            'quantity': 2,
            'price': Decimal('299.99')
        }, {
            'description': 'Some other product',
            'polymorphic_type': MockOrderLine,
            'quantity': 7,
            'price': Decimal('15.99')
        }]

        order = self.controller.save_or_update_order(None, lines, **state)
        order.export_date = datetime.datetime.now().date() # set as exported so we can attach an invoice
        self.sa_session.add(order)
        self.sa_session.flush()

        NewInvoiceMailTask().execute()
        self.sa_session.expire(order)
        self.assertEqual(order.invoice_stage, None) # email was not sent as there is no invoice attached

        # grant invoice priv
        self.controller.grant_privilege_to(order, [Order.PRIV_ADD_INVOICE], True, check_own_privs=False)
        self.controller.attach_invoice_to_order(order.id, 'test_invoice.txt', 'You have to pay A LOT!', False)

        NewInvoiceMailTask().execute()
        self.sa_session.expire(order)
        self.assertEqual(order.invoice_stage, Order.INVOICE_STAGE_SENT)

    def test_currency(self):

        country_nl = self.sa_session.query(Country).filter(Country.id == 155).one()

        currency_code = self.sa_session.query(func.iqGetCurrencyCodeForCountry(country_nl.id, datetime.date(2013, 1, 1))).one()[0]
        self.assertEqual(currency_code, 'EUR')

        currency_code = self.sa_session.query(func.iqGetCurrencyCodeForCountry(country_nl.id, datetime.date(1999, 1, 1))).one()[0]
        self.assertEqual(currency_code, 'NLG')

    def test_different_countries_no_vat(self):
        # Test for ticket #3953
        self._setup_tax_and_gl_account()

        state = {
            'merchant': self.merchant
        }

        lines = [{
            'description': 'Some random product',
            'quantity': 2,
            'price': Decimal('299.99')
        }, {
            'description': 'Some other product',
            'polymorphic_type': MockOrderLine,
            'quantity': 7,
            'price': Decimal('15.99')
        }]

        order = self.controller.save_or_update_order(None, lines, **state)
        self.assertTrue(order.total_amount_excluding_vat!=order.total_amount_including_vat)

        # merchant from Country NL
        # client from Country NOT NL
        self.account_license.company.addresses[0].address.country = self.sa_session.query(Country).filter(Country.id!=155).first()
        self.sa_session.add(self.account_license.company)
        self.sa_session.flush()

        # assert total_amount_excl_vat == total_amount_incl_vat
        state = {
            'merchant': self.merchant
        }

        lines = [{
            'description': 'Some random product',
            'quantity': 2,
            'price': Decimal('299.99')
        }, {
            'description': 'Some other product',
            'polymorphic_type': MockOrderLine,
            'quantity': 7,
            'price': Decimal('15.99')
        }]

        c = MockOrderController(self.sa_session, account_id=self.account.id)

        order = c.save_or_update_order(None, lines, **state)
        self.assertTrue(order.total_amount_excluding_vat==order.total_amount_including_vat)

    def test_resubmit_with_payment(self):
        """
        Test case for ticket #4024
        """
        accountant_license = self._create_license(domain=self.merchant.default_group.license.domain)
        accountant_account = self._create_active_account(license=accountant_license)
        self._attach_permissions_to_group(accountant_account.default_group, 'complete_orders')
        self._attach_permissions_to_group(self.ma.default_group, 'manage_payment_methods')
        self._attach_permissions_to_group(self.account.default_group, 'pay_orders')

        pc = PaymentController(self.sa_session, account_id=self.ma.id)
        pm1 = pc.save_or_update_payment_method_for_merchant(PaymentMethodBankTransfer, label='bank transfer')

        self._setup_tax_and_gl_account()

        lines = [{
            'description': 'Some random product',
            'quantity': 2,
            'price': Decimal('299.99')
        }, {
            'description': 'Some other product',
            'polymorphic_type': MockOrderLine,
            'quantity': 7,
            'price': Decimal('15.99')
        }]

        order = self.controller.save_or_update_order(None, lines, merchant=self.merchant)
        self.controller.submit_order(order)

        credit = self.controller._credit_order(order)

        lines = [{
            'description': 'Some random product',
            'quantity': 3,
            'price': Decimal('299.99')
        }, {
            'description': 'Some other product',
            'polymorphic_type': MockOrderLine,
            'quantity': 5,
            'price': Decimal('15.99')
        }]
        self.controller._instance_class.auto_submit = False # To reproduce ticket we need to skip auto-approve
        resubmit = self.controller.save_or_update_order(None, lines, merchant=self.merchant, parent_order=credit)
        self.assertEqual(resubmit.status, Order.STATUS_NEW)
        pc = PaymentController(self.sa_session, account_id=self.account.id, payment_method=pm1)
        pc.submit_payment(resubmit.id, 'marky mark and the funky bunch.')

        c = self.controller.get(credit.id).pop()
        self.assertEqual(c.status, Order.STATUS_SUBMITTED)

        # give privileges to the account to complete the credit (normally done by a hook)
        pc._grant_or_revoke(credit, (Order.PRIV_COMPLETE, ), True, group=True, target_group=accountant_account.default_group, check_own_privs=False)

        # let the accountnat complete the order
        accountant_pc = PaymentController(self.sa_session, account_id=accountant_account)
        time.sleep(1); accountant_pc.complete_payment(credit.id, "credit iz paid")
        self.assertEqual(credit.get_current_payment_status(), OrderPaymentStatus.PAYMENT_COMPLETED)

    def test_bank_payment_method(self):

        # setup an accountant user for completing orders
        accountant_license = self._create_license(domain=self.merchant.default_group.license.domain)
        accountant_account = self._create_active_account(license=accountant_license)
        self._attach_permissions_to_group(accountant_account.default_group, 'complete_orders')
        self._attach_permissions_to_group(self.ma.default_group, 'manage_payment_methods')

        # setup two payment methods
        pc = PaymentController(self.sa_session, account_id=self.ma.id)
        pm1 = pc.save_or_update_payment_method_for_merchant(PaymentMethodBankTransfer, label='bank transfer')
        pm2 = pc.save_or_update_payment_method_for_merchant(PaymentMethodOgone, label='online payment', psp_id='psp1', sha_in='aap', sha_out='tuut')

        pc = PaymentController(self.sa_session, account_id=self.account.id)

        # expect 2 payment methods to be available
        payment_methods = pc.get_payment_methods(self.merchant)
        self.assertEqual(len(payment_methods), 2)

        #
        # create an order
        #

        self._setup_tax_and_gl_account()

        state = {
            'merchant': self.merchant
        }

        lines = [{
            'description': 'Some random product',
            'quantity': 2,
            'price': Decimal('299.99')
        }, {
            'description': 'Some other product',
            'polymorphic_type': MockOrderLine,
            'quantity': 7,
            'price': Decimal('15.99')
        }]

        order = self.controller.save_or_update_order(None, lines, **state)
        order.export_date = datetime.datetime.now().date() # set as exported so we can attach an invoice
        self.sa_session.add(order)
        self.sa_session.flush()

        # assert current order has payment pending status
        self.assertEqual(order.get_current_payment_status(), OrderPaymentStatus.PAYMENT_PENDING)

        # give privileges to the account to complete the order (normally done by a hook)
        pc._grant_or_revoke(order, (Order.PRIV_COMPLETE, ), True, group=True, target_group=accountant_account.default_group, check_own_privs=False)

        # add permissions to user to pay the order
        self._attach_permissions_to_group(self.account.default_group, 'pay_orders')

        # use the abstract payment controller to get the specific payment controller for bank transfer
        pc = PaymentController(self.sa_session, account_id=self.account.id)
        self.assertEqual( pc.get_payment_controller(pm1.id), BankTransferPaymentController )

        # instanciate the user payment specific controller
        user_pc = pc.get_payment_controller(pm1.id)(self.sa_session, account_id=self.account.id, payment_method=pm1)

        # the user marks the order as paid
        user_pc.submit_payment(order.id, "transfer description")
        self.assertEqual(order.get_current_payment_status(), OrderPaymentStatus.PAYMENT_SUBMITTED)

        # the order should have status SUBMITTED (auto)
        self.assertEqual(order.status, Order.STATUS_SUBMITTED)

        # assert that the accountant can not complete or reject the order using pm2
        accountant_pc = PaymentController(self.sa_session, account_id=accountant_account, payment_method=pm2)
        self.assertRaises(PublicUserException, accountant_pc.reject_payment, order.id, "")
        self.assertRaises(PublicUserException, accountant_pc.complete_payment, order.id, "")

        # let the accountant reject the order
        accountant_pc = PaymentController(self.sa_session, account_id=accountant_account, payment_method=pm1)
        time.sleep(1); accountant_pc.reject_payment(order.id, "i r never received any")
        self.assertEqual(order.get_current_payment_status(), OrderPaymentStatus.PAYMENT_REJECTED)

        # let the user pay again
        time.sleep(1); user_pc.submit_payment(order.id, "transfering from other account")
        self.assertEqual(order.get_current_payment_status(), OrderPaymentStatus.PAYMENT_SUBMITTED)

        # let the accountnat complete the order
        time.sleep(1); accountant_pc.complete_payment(order.id, "all good")
        self.assertEqual(order.get_current_payment_status(), OrderPaymentStatus.PAYMENT_COMPLETED)

        # assert that its not possible to change completed order
        self.assertRaises(Exception, accountant_pc.reject_payment, order.id, "")

    def test_bank_payment_method_pending_to_approved(self):

        # setup an accountant user for completing orders
        accountant_license = self._create_license(domain=self.merchant.default_group.license.domain)
        accountant_account = self._create_active_account(license=accountant_license)
        self._attach_permissions_to_group(accountant_account.default_group, 'complete_orders')
        self._attach_permissions_to_group(self.ma.default_group, 'manage_payment_methods')

        # setup two payment methods
        pc = PaymentController(self.sa_session, account_id=self.ma.id)
        pm1 = pc.save_or_update_payment_method_for_merchant(PaymentMethodBankTransfer, label='bank transfer')

        pc = PaymentController(self.sa_session, account_id=self.account.id)

        self._setup_tax_and_gl_account()

        state = {
            'merchant': self.merchant
        }

        lines = [{
            'description': 'Some random product',
            'quantity': 2,
            'price': Decimal('299.99')
        }, {
            'description': 'Some other product',
            'polymorphic_type': MockOrderLine,
            'quantity': 7,
            'price': Decimal('15.99')
        }]

        order = self.controller.save_or_update_order(None, lines, **state)
        order.export_date = datetime.datetime.now().date() # set as exported so we can attach an invoice
        self.sa_session.add(order)
        self.sa_session.flush()

        # assert current order has payment pending status
        self.assertEqual(order.get_current_payment_status(), OrderPaymentStatus.PAYMENT_PENDING)

        # give privileges to the account to complete the order (normally done by a hook)
        pc._grant_or_revoke(order, (Order.PRIV_COMPLETE, ), True, group=True, target_group=accountant_account.default_group, check_own_privs=False)

        # add permissions to user to pay the order
        self._attach_permissions_to_group(self.account.default_group, 'pay_orders')

        # let the accountant complete the order
        accountant_pc = PaymentController(self.sa_session, account_id=accountant_account, payment_method=pm1)
        time.sleep(1); accountant_pc.complete_payment(order.id, "all good")

        # order payment status should be complketed
        self.assertEqual(order.get_current_payment_status(), OrderPaymentStatus.PAYMENT_COMPLETED)


    def test_ogone_payment_method(self):

        self._attach_permissions_to_group(self.account.default_group, 'manage_tax_classes', 'manage_country_tax', 'manage_payment_methods')

        # setup ogone payment method
        psp_id = self._uniquestring('psp_id')

        pc = PaymentController(self.sa_session, account_id=self.account.id)
        pm = pc.save_or_update_payment_method_for_merchant(PaymentMethodOgone, merchant=self.merchant, label='online payment', **{
            'psp_id': psp_id,
            'sha_in': 'Bier1234',
            'sha_out': 'Tuut1234'
        })

        # create an order
        self._setup_tax_and_gl_account()

        state = {
            'merchant': self.merchant
        }

        lines = [{
            'description': 'Some random product',
            'quantity': 2,
            'price': Decimal('299.99')
        }, {
            'description': 'Some other product',
            'polymorphic_type': MockOrderLine,
            'quantity': 7,
            'price': Decimal('15.99')
        }]

        order = self.controller.save_or_update_order(None, lines, **state)
        order.export_date = datetime.datetime.now().date() # set as exported so we can attach an invoice
        self.sa_session.add(order)
        self.sa_session.flush()

        self._attach_permissions_to_group(self.account.default_group, 'view_mock_orders')

        # get ogone payment method controller
        payment_methods = pc.get_payment_methods(merchant=self.merchant)
        self.assertEqual(len(payment_methods), 1)

        request = HttpRequest()
        request.META['HTTP_REFERER'] = 'http://iqportal.com/nl/showrooms/Regio_Rotterdam/'

        pc = PaymentController(self.sa_session, account_id=self.account.id)
        pc = pc.get_payment_controller(payment_methods[0].id)(self.sa_session, account_id=self.account.id, payment_method=payment_methods[0], request=request)

        # assert vars that will be sent to the ogone psp
        vars = pc.get_payment_vars(order.id)
        self.assertEqual(vars['fields']['CN'], 'TestLastName')
        self.assertEqual(vars['fields']['ACCEPTURL'], 'http://iqportal.com/basesoft.web.contrib.order/payment_method/ogone/%s/accept/' % psp_id)

        # initial status must be pending
        self.assertEqual(order.get_current_payment_status(), OrderPaymentStatus.PAYMENT_PENDING)

        #
        # ogone replying
        #

        # ogone server replies with payment canceled
        request = HttpRequest()
        request.Session = Session
        request.session = {}

        request.POST = {
            'ACCEPTANCE': 'test123',
            'BRAND': 'VISA',
            'CARDNO': 'XXXXXXXXXXXX1111',
            'CN': 'InvoiceFirstName InvoiceLastName',
            'NCERROR': '0',
            'PAYID': '26024396',
            'STATUS': '1', # canceled by customer
            'AMOUNT': '453.75',
            'CURRENCY': 'EUR',
            'orderID': order.id,
        }

        request.POST['SHASIGN'] = OgonePaymentController.ogone_hash(request.POST, pm.sha_out )
        time.sleep(1); payment_method_ogone_views.payment_method_ogone_direct_cancel(request, psp_id)

        # status must still be pending
        self.assertEqual(order.get_current_payment_status(), OrderPaymentStatus.PAYMENT_PENDING)

        # ogone server replies with payment uncertain
        request = HttpRequest()
        request.Session = Session
        request.session = {}

        request.POST = {
            'ACCEPTANCE': 'test123',
            'BRAND': 'VISA',
            'CARDNO': 'XXXXXXXXXXXX1111',
            'CN': 'InvoiceFirstName InvoiceLastName',
            'NCERROR': '0',
            'PAYID': '26024396',
            'STATUS': '2',
            'AMOUNT': '453.75',
            'CURRENCY': 'EUR',
            'orderID': order.id,
        }

        request.POST['SHASIGN'] = OgonePaymentController.ogone_hash(request.POST, pm.sha_out)
        time.sleep(1); payment_method_ogone_views.payment_method_ogone_direct_accept(request, psp_id)

        # status status is submitted (awaiting approval)
        self.assertEqual(order.get_current_payment_status(), OrderPaymentStatus.PAYMENT_SUBMITTED)

        # ogone server replies with payment OK
        request = HttpRequest()
        request.Session = Session
        request.session = {}

        request.POST = {
            'ACCEPTANCE': 'test123',
            'BRAND': 'VISA',
            'CARDNO': 'XXXXXXXXXXXX1111',
            'CN': 'InvoiceFirstName InvoiceLastName',
            'ED': '0115',
            'IP': '80.28.174.236',
            'NCERROR': '0',
            'PAYID': '26024396',
            'PM': 'CreditCard',
            'STATUS': '9',
            'TRXDATE': '12/11/13',
            'AMOUNT': '453.75',
            'CURRENCY': 'EUR',
            'orderID': order.id,
        }

        request.POST['SHASIGN'] = OgonePaymentController.ogone_hash(request.POST, pm.sha_out)

        # after the shasign is made, change orderid back to lowercase.. ogone seems to do this too
        del request.POST['ORDERID']
        request.POST['orderID'] = order.id

        # accept the payment
        time.sleep(1); payment_method_ogone_views.payment_method_ogone_direct_accept(request, psp_id)

        # status status is completed
        self.assertEqual(order.get_current_payment_status(), OrderPaymentStatus.PAYMENT_COMPLETED)

    def test_ogone_hash(self):

        sha_key = 'BIER'
        fields = {
            'CN': u'€  Customer Name',
            'EMPTY_VALUE': '',
            'NONE_VALUE': None
        }

        sha_sign = OgonePaymentController.ogone_hash(fields, sha_key)

        # assert that the sha_sign is generated without only the CN
        self.assertEqual(sha_sign, '91F4FFFF5F79C7E985A16CC64772606C15D96C6F')

        # assert that the field list is modified
        self.assertEqual(fields.keys(), ['CN'])


    def test_ogone_hash_response(self):
        """
        tests the hashing for a ogone response
        """

        fields = {
            'ACCEPTANCE': 'test123',
            'BRAND': 'VISA',
            'CARDNO': 'XXXXXXXXXXXX1111',
            'CN': u"InvoiceFirstName InvoiceLastName",
            'ED': '0115',
            'IP': '80.28.174.236',
            'NCERROR': '0',
            'PAYID': '26024396',
            'PM': 'CreditCard',
            #'SHASIGN': '494971E540384E52C6343638B3B6E57DC04D8366',
            'STATUS': '9',
            'TRXDATE': '12/11/13',
            'AMOUNT': '453.75',
            'CURRENCY': 'EUR',
            'ORDERID': '3',
        }
        self.assertEqual('494971E540384E52C6343638B3B6E57DC04D8366', OgonePaymentController.ogone_hash(fields, 'getoutthere98!'))

    def test_zero_payment(self):
        # create an order
        self._setup_tax_and_gl_account()

        state = {
            'merchant': self.merchant
        }

        lines = [{
            'description': 'Some random product',
            'quantity': 0,
            'price': Decimal('299.99')
        }, {
            'description': 'Some other product',
            'polymorphic_type': MockOrderLine,
            'quantity': 0,
            'price': Decimal('15.99')
        }]

        order = self.controller.save_or_update_order(None, lines, **state)
        self.controller.submit_payment(order.id, None)

        self.sa_session.expire(order)
        self.assertEqual(order.status, Order.STATUS_SUBMITTED)
        self.assertEqual(order.get_current_payment_status(), OrderPaymentStatus.PAYMENT_COMPLETED)

        # now we test with another which has quantity, it should raise an exception
        state = {
            'merchant': self.merchant
        }

        lines = [{
            'description': 'Some random product',
            'quantity': 2,
            'price': Decimal('299.99')
        }, {
            'description': 'Some other product',
            'polymorphic_type': MockOrderLine,
            'quantity': 1,
            'price': Decimal('15.99')
        }]

        order = self.controller.save_or_update_order(None, lines, **state)
        self.assertRaises(PublicUserException, self.controller.submit_payment, order.id, None)

    #
    # rule tests start here
    #

    def _make_rule(self, name, **kwargs):

        self.controller.save_rule(None, {
            'order_type': kwargs.get('order_type', 'MockOrder'),
            'name': name,
            'enabled': kwargs.get('enabled', True),
            'execute_order': kwargs.get('execute_order', None),
            'description': kwargs.get('description', name),
            'start_date': kwargs.get('start_date', now()),
            'end_date': kwargs.get('end_date', None),
            'match_all': kwargs.get('match_all', True),
            'stop_processing': kwargs.get('stop_processing', False),
            'conditions': kwargs.get('conditions', []),
            'actions': kwargs.get('actions', [])
        })

    def _make_rule_order(self, **kwargs):

        state = {
            'merchant': self.merchant,
        }
        lines = [{
            'description': 'Some product',
            'polymorphic_type': MockOrderLine,
            'quantity': kwargs.get('quantity', 1),
            'price': kwargs.get('price', Decimal('99.9')),
        }]
        order = self.controller.save_or_update_order(None, lines, **state)

        return order

    def test_rule_fixed_discount(self):
        """
        basesoft.web.test.unit.test_order_controller.OrderControllerTestCase.test_rule_fixed_discount
        """


        self._attach_permissions_to_group(self.account.groups[0].group, 'view_mock_orders', 'manage_order_rules')
        tax_class, tcc, ga = self._setup_tax_and_gl_account()

        # simple rule that matches the current merchant and applies a fixed discount amount
        self._make_rule('rule 1',
            match_all = False,
            description='rule description',
            conditions =  [{
                'condition': 'merchant',
                'operator': None, 'value_text': None, 'value_bool': None,
                'value_int': self.merchant.id,
                'value_decimal': None,
            }, {
                'condition': 'merchant',
                'operator': None, 'value_text': None, 'value_bool': None,
                'value_int': 9999999, # testing 'match any' by supplying non existing merchant
                'value_decimal': None,
            }],
            actions = [{
                'action': 'fixed_amount_discount',
                'description': 'action description',
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': 10,
                'gl_account_code': ga.code,
                'tax_class_id': tax_class.id,
            }]
        )

        order = self._make_rule_order()
        self.controller.submit_order(order)

        self.assertEqual(len(order.lines), 2)
        self.assertEqual(order.lines[1].__class__, OrderLineRule )
        self.assertEqual(order.lines[1].quantity, -1 )
        self.assertEqual(order.lines[1].price_per_unit, 10 )
        self.assertEqual(order.lines[1].line_amount_excluding_vat, Decimal('-10') )
        self.assertEqual(order.lines[1].line_amount_including_vat, Decimal('-12.1') )
        self.assertEqual(order.lines[1].gl_account_number, 'GL1234' )
        self.assertEqual(order.lines[1].vat_rate, Decimal('21.0') )
        self.assertEqual(order.lines[1].description, 'rule description / action description' )
        self.assertEqual(order.total_amount_excluding_vat, Decimal('89.9') )
        self.assertEqual(order.total_amount_including_vat, Decimal('108.779') ) # 21%

        self._attach_permissions_to_group(self.account.groups[0].group, 'view_order_log')
        order_log = self.controller.get_order_log(order.id)
        self.assertTrue(order_log.find('Processing 2 conditions'))

    def test_rule_relative_discount(self):
        """
        basesoft.web.test.unit.test_order_controller.OrderControllerTestCase.test_rule_relative_discount
        """

        self._attach_permissions_to_group(self.account.groups[0].group, 'view_mock_orders', 'manage_order_rules')
        self._setup_tax_and_gl_account()

        self._make_rule('rule 1',
            match_all=True,
            conditions =  [{
                'condition': 'merchant',
                'operator': None,
                'value_text': None, 'value_bool': None,
                'value_int': self.merchant.id,
                'value_decimal': None,
            }, {
                'condition': 'merchant',
                'operator': None,
                'value_text': None, 'value_bool': None,
                'value_int': self.merchant.id, # testing 'match all' by supplying two matching conditions
                'value_decimal': None,
            }],
            actions = [{
                'action': 'relative_amount_discount',
                'description': None,
                'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': Decimal('7.5'), # 7.5% discount
                'gl_account_code': None, # notice: no tax/gl account in this rule
                'tax_class_id': None,
            }]
        )

        order = self._make_rule_order()
        self.controller.submit_order(order)

        self.assertEqual(order.lines[1].__class__, OrderLineRule )
        self.assertEqual(order.lines[1].quantity, -1)
        self.assertEqual(order.lines[1].price_per_unit, Decimal('7.49250') ) # 99.9 * 0.075
        self.assertEqual(order.lines[1].line_amount_excluding_vat, Decimal('-7.49250') )
        self.assertEqual(order.lines[1].line_amount_including_vat, Decimal('-9.06592') )
        self.assertEqual(order.lines[1].gl_account_number, None)
        self.assertEqual(order.lines[1].vat_rate, Decimal('0') )
        self.assertEqual(order.total_amount_excluding_vat, Decimal('92.4075') ) # 99.9 - 7.49250
        self.assertEqual(order.total_amount_including_vat, Decimal('111.81308') )

    def test_rule_full_discount_with_submit(self):
        """
        """
        accountant_license = self._create_license(domain=self.merchant.default_group.license.domain)
        accountant_account = self._create_active_account(license=accountant_license)

        self._attach_permissions_to_group(self.account.groups[0].group, 'view_mock_orders', 'manage_order_rules')
        self._attach_permissions_to_group(accountant_account.default_group, 'complete_orders')
        self._attach_permissions_to_group(self.ma.default_group, 'manage_payment_methods')
        self._attach_permissions_to_group(self.account.default_group, 'pay_orders')

        self._setup_tax_and_gl_account()

        self._make_rule('full discount',
            match_all=True,
            conditions = [],
            actions = [{
                'action': 'relative_amount_discount',
                'description': None,
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': Decimal('100'),
                'gl_account_code': 'MOCK_ORDER_GL_ACCOUNT',
                'tax_class_id': None,
            }, ]
        )

        lines = [{
            'description': 'Some random product',
            'quantity': 2,
            'price': Decimal('10')
        }, {
            'description': 'Some other product',
            'polymorphic_type': MockOrderLine,
            'quantity': 5,
            'price': Decimal('20')
        }]

        # save and pay order without a payment method
        order = self.controller.save_or_update_order(None, lines, merchant=self.merchant)
        self.controller.submit_payment(order.id, None)


    def test_rule_discount_penalty(self):
        """
        Tests the ridiculous rule where the total amount is the same as it started with.

        Order starts with a line with 99.9 euries. Then the following rules apply:
          * first give 10 discount (total = 89.9)
          * then give 50% discount (total = 44.95)
          * give 100% penalty (total = 89.9)
          * give 10 penalty (total = 99.9)
        """

        self._attach_permissions_to_group(self.account.groups[0].group, 'view_mock_orders', 'manage_order_rules')
        self._setup_tax_and_gl_account()

        self._make_rule('rule 1',
            match_all=True,
            conditions =  [],
            actions = [ {
                'action': 'fixed_amount_discount',
                'description': None,
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': Decimal('10'),
                'gl_account_code': None,
                'tax_class_id': None,
            }, {
                'action': 'relative_amount_discount',
                'description': None,
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': Decimal('50'),
                'gl_account_code': None,
                'tax_class_id': None,
            }, {
                'action': 'relative_amount_penalty',
                'description': None,
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': Decimal('100'),
                'gl_account_code': None,
                'tax_class_id': None,
            }, {
                'action': 'fixed_amount_penalty',
                'description': None,
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': Decimal('10'),
                'gl_account_code': None,
                'tax_class_id': None,
            },]
        )

        order = self._make_rule_order()
        self.controller.submit_order(order)

        self.assertEqual(len(order.lines), 5)
        self.assertEqual(order.total_amount_excluding_vat, Decimal('99.9') )

    def test_rule_execute_order(self):

        self._attach_permissions_to_group(self.account.groups[0].group, 'view_mock_orders', 'manage_order_rules')
        self._setup_tax_and_gl_account()

        # rule 1 has execute_order 2

        self._make_rule('rule 1',
            execute_order = 2,
            actions = [{
                'action': 'relative_amount_discount',
                'description': None,
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': 50,
                'gl_account_code': None,
                'tax_class_id': None,
            }]
        )

        # rule 2 has a lower execute_order, so should be executed first
        self._make_rule('rule 2',
            execute_order = 1,
            actions = [{
                'action': 'relative_amount_discount',
                'description': None,
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': 10,
                'gl_account_code': None,
                'tax_class_id': None,
            }]

        )
        order = self._make_rule_order()
        self.controller.submit_order(order)

        # rule 1
        self.assertEqual(order.lines[1].price_per_unit, Decimal('9.99') ) # 99.9 * 0.10

        # rule 2
        self.assertEqual(order.lines[2].price_per_unit, Decimal('44.955') ) # (99.9 - 9.99) * 0.50

        self.assertEqual(order.total_amount_excluding_vat, Decimal('44.955') ) # 99.9 - 9.99 - 44.955
        self.assertEqual(order.total_amount_including_vat, Decimal('54.39555') )


    def test_rule_custom_condition_and_action(self):

        self._attach_permissions_to_group(self.account.groups[0].group, 'view_mock_orders', 'manage_order_rules')
        tax_class, tcc, ga = self._setup_tax_and_gl_account()

        self._make_rule('rule 1',
            match_all = False,

            # condition; when ordering more than 666 products...
            conditions =  [{
                'condition': 'mock_quantity_gte',
                'operator': None, 'value_text': None, 'value_bool': None,
                'value_int': 666,
                'value_decimal': None,
            }],

            # action: give away all products for free!
            actions = [{
                'action': 'mock_all_free',
                'description': None,
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None, 'value_decimal': None,
                'tax_class_id': tax_class.id,
                'gl_account_code': None
            }]
        )

        order = self._make_rule_order(quantity=666)
        self.controller.submit_order(order)

        self.assertEqual(len(order.lines), 2)
        self.assertEqual(order.lines[1].quantity, -666 )
        self.assertEqual(order.lines[1].price_per_unit, Decimal('99.9') )
        self.assertEqual(order.lines[1].line_amount_excluding_vat, Decimal('-66533.4') )
        self.assertEqual(order.lines[1].line_amount_including_vat, Decimal('-80505.414') )

        self.assertEqual(order.total_amount_excluding_vat, Decimal('0') )
        self.assertEqual(order.total_amount_including_vat, Decimal('0') )

    def test_absolute_discount_total_amount_less_than_zero(self):
        self._attach_permissions_to_group(self.account.groups[0].group, 'view_mock_orders', 'manage_order_rules')
        self._setup_tax_and_gl_account()

        self._make_rule('rule 1',
            match_all=True,
            conditions = [{
                'condition': 'client_country',
                'operator': None, 'value_text': None, 'value_bool': None,
                'value_int': self.account_license.company.addresses[0].address.country.id,
                'value_decimal': None,
            }],
            actions = [{
                'action': 'fixed_amount_discount',
                'description': 'action description',
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': 10000,  # get ridiculous discount
                'gl_account_code': None,
                'tax_class_id': None,
            }]
        )

        order = self._make_rule_order()
        self.controller.submit_order(order)

        # the action should not have been applied, even if the condition matched
        self.assertEqual(len(order.lines), 1)
        self.assertEqual(order.total_amount_excluding_vat, Decimal('99.9'))
        self.assertEqual(order.total_amount_including_vat, Decimal('120.87900'))

    def test_rule_not_applied(self):

        self._attach_permissions_to_group(self.account.groups[0].group, 'view_mock_orders', 'manage_order_rules')
        self._setup_tax_and_gl_account()

        # test rule condition not matching 'any' condition
        self._make_rule('rule 1',
            match_all = False,
            conditions =  [{
                'condition': 'merchant',
                'operator': None, 'value_text': None, 'value_bool': None,
                'value_int': 9999998, # non existing merchant
                'value_decimal': None,
            }, {
                'condition': 'merchant',
                'operator': None, 'value_text': None, 'value_bool': None,
                'value_int': 9999999, # non existing merchant
                'value_decimal': None,
            }],
            actions = [{
                'action': 'fixed_amount_discount',
                'description': None,
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': 10,
                'gl_account_code': None, 'tax_class_id': None,
            }]

        )

        # test rule condition not matches all
        self._make_rule('rule 2',
            match_all = True,
            conditions =  [{
                'condition': 'merchant',
                'operator': None, 'value_text': None, 'value_bool': None,
                'value_int': self.merchant.id,
                'value_decimal': None,
            }, {
                'condition': 'merchant',
                'operator': None, 'value_text': None, 'value_bool': None,
                'value_int': 9999999, # non existing merchant
                'value_decimal': None,
            }],
            actions = [{
                'action': 'fixed_amount_discount',
                'description': None,
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': 10,
                'gl_account_code': None, 'tax_class_id': None,
            }]
        )

        # test rule not active
        self._make_rule('rule 3',
            enabled = False,
            actions = [{
                'action': 'fixed_amount_discount',
                'description': None,
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': 10,
                'gl_account_code': None, 'tax_class_id': None,
            }]
        )

        # Rule 4 shouldnt execute because of start date in future
        self._make_rule('rule 4',
            start_date = now() + datetime.timedelta(days=1),
            actions = [{
                'action': 'fixed_amount_discount',
                'description': None,
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': 10,
                'gl_account_code': None, 'tax_class_id': None,
            }]
        )

        # Rule 5 shouldnt execute because of stop date in past
        self._make_rule('rule 5',
            end_date = now() - datetime.timedelta(days=1),
            actions = [{
                'action': 'fixed_amount_discount',
                'description': None,
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': 10,
                'gl_account_code': None, 'tax_class_id': None,
            }]
        )

        # Rule 6 does nothing, but sets a 'stop processing'
        self._make_rule('rule 6',
            stop_processing=True,
        )

        # Rule 7 shouldn't execute because Rule 8 has stop processing
        self._make_rule('rule 7',
            actions = [{
                'action': 'fixed_amount_discount',
                'description': None,
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': 10,
                'gl_account_code': None, 'tax_class_id': None,
            }]
        )

        order = self._make_rule_order()
        self.controller.submit_order(order)

        # assert only 1 product line got charged with the product amount
        self.assertEqual(len(order.lines), 1)
        self.assertEqual(order.total_amount_excluding_vat, Decimal('99.9') )
        self.assertEqual(order.total_amount_including_vat, Decimal('120.879') )

    def test_rule_client_country(self):

        self._attach_permissions_to_group(self.account.groups[0].group, 'view_mock_orders', 'manage_order_rules')
        self._setup_tax_and_gl_account()

        self._make_rule('rule 1',
            match_all=True,
            conditions = [{
                'condition': 'client_country',
                'operator': None, 'value_text': None, 'value_bool': None,
                'value_int': self.account_license.company.addresses[0].address.country.id,
                'value_decimal': None,
            }],
            actions = [{
                'action': 'fixed_amount_discount',
                'description': 'action description',
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': 10,
                'gl_account_code': None,
                'tax_class_id': None,
            }]
        )

        order = self._make_rule_order()
        self.controller.submit_order(order)

        # if there are 2 lines it means the conditions matched
        self.assertEqual(len(order.lines), 2)

    def test_rule_additional_conditions(self):

        self._attach_permissions_to_group(self.account.groups[0].group, 'view_mock_orders', 'manage_order_rules')
        self._setup_tax_and_gl_account()

        self._make_rule('rule 1',
            match_all=True,
            conditions = [{
                'condition': 'client',
                'operator': None, 'value_text': None, 'value_bool': None,
                'value_int': self.account_license.id,
                'value_decimal': None,
            }, {
                'condition': 'auto_submitted',
                'operator': None, 'value_text': None,
                'value_bool': None,
                'value_int': Order.SUBMIT_DONE_BY_USER,
                'value_decimal': None,
            }, {
                'condition': 'total_amount_excl_vat',
                'operator': OrderRuleCondition.OPERATOR_GTE,
                'value_text': None,
                'value_bool': None,
                'value_int': None,
                'value_decimal': 50,
            }, {
                'condition': 'total_amount_incl_vat',
                'operator': OrderRuleCondition.OPERATOR_GTE, 'value_text': None,
                'value_bool': None,
                'value_int': None,
                'value_decimal': 50,
            },
            ],
            actions = [{
                'action': 'fixed_amount_discount',
                'description': 'action description',
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': 10,
                'gl_account_code': None,
                'tax_class_id': None,
            }]
        )

        order = self._make_rule_order()
        self.controller.submit_order(order)

        # if there are 2 lines it means the conditions matched
        self.assertEqual(len(order.lines), 2)


    def test_rule_is_resubmit(self):
        """
        """

        self._attach_permissions_to_group(self.account.groups[0].group, 'view_mock_orders', 'manage_order_rules')
        self._setup_tax_and_gl_account()

        self._make_rule('rule 1',
            match_all=True,
            conditions = [{
                'condition': 'is_resubmit',
                'operator': None, 'value_text': None, 'value_bool': True,
                'value_int': None, 'value_decimal': None,
            }],
            actions = [{
                'action': 'fixed_amount_discount',
                'description': 'action description',
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': 10,
                'gl_account_code': None,
                'tax_class_id': None,
            }]
        )

        # make normal order
        order = self.controller.save_or_update_order(None, [{
            'description': 'Pretty product',
            'quantity': 1,
            'price': Decimal('99.99')
        }], merchant=self.merchant)
        self.controller.submit_order(order)

        # expect that the rule didnt apply
        self.assertEqual(len(order.lines), 1)

        # credit the order
        credit = self.controller._credit_order(order)

        # expect that the rule didnt apply
        self.assertEqual(len(credit.lines), 1)

        # resubmit (changes 1 quantity)
        resubmit = self.controller.save_or_update_order(None, [{
            'description': 'Some random product',
            'quantity': 2,
            'price': Decimal('99.99')
        }], merchant=self.merchant, parent_order=credit)
        self.assertEqual(resubmit.status, Order.STATUS_NEW)
        self.controller.submit_order(resubmit)

        # roel shouldve added a line
        self.assertEqual(len(resubmit.lines), 2)

    def test_rule_minimum_amount(self):

        self._attach_permissions_to_group(self.account.groups[0].group, 'view_mock_orders', 'manage_order_rules')
        self._setup_tax_and_gl_account()

        self._make_rule('rule 1',
            match_all=True,
            conditions = [],
            actions = [{
                'action': 'minimum_amount',
                'description': '',
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': 999,
                'gl_account_code': None,
                'tax_class_id': None,
            }]
        )

        # order that will get the minimum amount applied
        order = self._make_rule_order()
        self.controller.submit_order(order)
        self.assertEqual(order.total_amount_excluding_vat, Decimal('999') )

        # order that is already high enough
        order = self._make_rule_order(price=Decimal('1000'))
        self.controller.submit_order(order)
        self.assertEqual(order.total_amount_excluding_vat, Decimal('1000') )

    def test_rule_dry_run(self):

        self._attach_permissions_to_group(self.account.groups[0].group, 'view_mock_orders', 'manage_order_rules')
        self._setup_tax_and_gl_account()

        self._make_rule('rule 1',
            match_all=True,
            conditions = [],
            actions = [{
                'action': 'minimum_amount',
                'description': '',
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': 999,
                'gl_account_code': None,
                'tax_class_id': None,
            }]
        )

        # make order (unsubmitted)
        order = self._make_rule_order()
        order_id = order.id
        self.sa_session.expunge(order)

        # load the state with dry-run enabled
        order_state = self.controller.load_order_state(order_id, submit_dry_run=True)
        self.assertEqual(order_state['total_amount_excluding_vat'], Decimal('999') )

        # re-load the order, make sure the rules haven't yet been executed
        order = self.controller.get(order_id)[0]
        self.assertEqual(order.total_amount_excluding_vat, Decimal('99.9') )

        # finally submit the order
        self.controller.submit_order(order)
        self.assertEqual(order.total_amount_excluding_vat, Decimal('999') )


    def test_resubmit_with_rule(self):
        """
        """
        accountant_license = self._create_license(domain=self.merchant.default_group.license.domain)
        accountant_account = self._create_active_account(license=accountant_license)

        self._attach_permissions_to_group(self.account.groups[0].group, 'view_mock_orders', 'manage_order_rules')
        self._attach_permissions_to_group(accountant_account.default_group, 'complete_orders')
        self._attach_permissions_to_group(self.ma.default_group, 'manage_payment_methods')
        self._attach_permissions_to_group(self.account.default_group, 'pay_orders')

        pc = PaymentController(self.sa_session, account_id=self.ma.id)
        pm1 = pc.save_or_update_payment_method_for_merchant(PaymentMethodBankTransfer, label='bank transfer')

        self._setup_tax_and_gl_account()

        self._make_rule('penalty',
            match_all=True,
            conditions = [],
            actions = [ {
                'action': 'fixed_amount_penalty',
                'description': None,
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': Decimal('10.0'),
                'gl_account_code': 'MOCK_ORDER_GL_ACCOUNT',
                'tax_class_id': None,
            }, ]
        )

        lines = [{
            'description': 'Some random product',
            'quantity': 2,
            'price': Decimal('10')
        }, {
            'description': 'Some other product',
            'polymorphic_type': MockOrderLine,
            'quantity': 5,
            'price': Decimal('20')
        }]

        # submit order
        order = self.controller.save_or_update_order(None, lines, merchant=self.merchant)
        self.controller.submit_order(order)
        self.assertEquals(len(order.lines), 3)
        self.assertEquals(order.total_amount_excluding_vat, 130)

        # credit order, expect all lines (including rule) to be credited
        credit = self.controller._credit_order(order)
        self.assertEquals(credit.total_amount_excluding_vat, -130)
        self.assertEquals(len(credit.lines), 3)
        self.assertEqual(credit.lines[2].gl_account_number, 'GL1234')

        # resubmit
        lines = [{
            'description': 'Some random product',
            'quantity': 3,
            'price': Decimal('10')
        }, {
            'description': 'Some other product',
            'polymorphic_type': MockOrderLine,
            'quantity': 5,
            'price': Decimal('20')
        }]

        resubmit = self.controller.save_or_update_order(None, lines, merchant=self.merchant, parent_order=credit)
        self.assertEquals(len(credit.lines), 3)
        self.assertEquals(resubmit.total_amount_excluding_vat, Decimal('130'))


    def test_rule_simulate(self):

        self._setup_tax_and_gl_account()

        self._attach_permissions_to_group(self.account.groups[0].group, 'view_mock_orders', 'manage_order_rules',
                                          'manage_order_simulations')

        self._make_rule('penalty',
            match_all=True,
            conditions = [],
            actions = [ {
                'action': 'fixed_amount_penalty',
                'description': None,
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': Decimal('10.0'),
                'gl_account_code': None,
                'tax_class_id': None,
            }, ]
        )

        fields = self.controller.get_simulate_order_fields('MockOrder')

        self.assertEquals(len(fields), 2)
        self.assertEquals(unicode(fields['merchant'].label), 'Merchant')
        self.assertEquals(fields['scope'].label, 'Group')

        order_state = {
            'polymorphic_type': 'MockOrder',

            'merchant': self.merchant.id,
            'scope': self.ma.default_group.id,

            'lines': [
                {
                    'polymorphic_type': 'MockOrderLine',
                    'description': 'line 1',
                    'quantity': 10,
                    'price': Decimal('1'),
                }

            ],
        }

        # process the simulate order
        order_state = self.controller.simulate_submit(order_state)

        self.assertEqual(len(order_state['lines']), 2)
        self.assertEqual(order_state['total_amount_excluding_vat'], Decimal('20') )

    def test_rule_presistent(self):
        """
        """

        self._attach_permissions_to_group(self.account.groups[0].group, 'view_mock_orders', 'manage_order_rules')
        self._setup_tax_and_gl_account()

        # normal (non-persistent) rule only applies to normal orders (not order)
        self._make_rule('rule1 np/nr',
            match_all = True,
            conditions = [
                 {
                 'condition': 'is_resubmit',
                 'operator': None, 'value_text': None, 'value_bool': False,
                 'value_int': None, 'value_decimal': None,
             }
            ],
            actions = [{
                'action': 'fixed_amount_discount',
                'description': 'action description',
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': 10,
                'gl_account_code': None,
                'tax_class_id': None,
            }]
        )

        # persistent rule that applies on normal order (not resubmit)
        self._make_rule('rule2 p/nr',
            match_all = True,
            conditions = [
                 {
                 'condition': 'is_resubmit',
                 'operator': None, 'value_text': None, 'value_bool': False,
                 'value_int': None, 'value_decimal': None,
             }
            ],
            actions = [{
                'action': 'fixed_amount_discount',
                'description': 'action description',
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': 10,
                'gl_account_code': None,
                'tax_class_id': None,
                'is_persistent': True,
            }]
        )

        # persistent rule that applies on resubmit
        self._make_rule('rule3 p/r',
            match_all = True,
            conditions = [
                 {
                 'condition': 'is_resubmit',
                 'operator': None, 'value_text': None, 'value_bool': True,
                 'value_int': None, 'value_decimal': None,
             }
            ],
            actions = [{
                'action': 'fixed_amount_discount',
                'description': 'action description',
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': 10,
                'gl_account_code': None,
                'tax_class_id': None,
                'is_persistent': True,
            }]
        )

        # make normal order
        order = self.controller.save_or_update_order(None, [{
            'description': 'Pretty product',
            'quantity': 10,
            'price': Decimal('50')
        }], merchant=self.merchant)
        self.controller.submit_order(order)

        # expect 3 lines (normal, rule1, rule2)
        self.assertEqual(len(order.lines), 3)

        # expect extra line to have the is_persistent flag set
        self.assertTrue(isinstance(order.lines[0], OrderLine)) # 500
        self.assertTrue(isinstance(order.lines[1], OrderLineRule)) # -10
        self.assertTrue(isinstance(order.lines[2], OrderLineRule)) # -10
        self.assertFalse(order.lines[1].is_persistent)
        self.assertTrue(order.lines[2].is_persistent)
        self.assertEquals(order.total_amount_excluding_vat, Decimal('480'))

        #
        # order 1
        #

        credit = self.controller._credit_order(order)
        self.assertEqual(len(credit.lines), 3)

        order = self.controller.save_or_update_order(None, [{
            'description': 'Some random product',
            'quantity': 11,
            'price': Decimal('50')
        }], merchant=self.merchant, parent_order=credit)
        self.controller.submit_order(order)

        # order 1 should have 3 lines
        # - order line
        # - matching rule (rule 3)
        # - persistent rule (rule 2) (inherited from 'normal' order)
        self.assertEqual(len(order.lines), 3)
        self.assertTrue(isinstance(order.lines[0], OrderLine))     # 550
        self.assertTrue(isinstance(order.lines[1], OrderLineRule)) # -10
        self.assertTrue(order.lines[1].is_persistent)
        self.assertTrue(isinstance(order.lines[2], OrderLineRule)) # -10
        self.assertTrue(order.lines[2].is_persistent)
        self.assertEquals(order.total_amount_excluding_vat, Decimal('530'))

        #
        # order 2
        #

        credit = self.controller._credit_order(order)
        self.assertEqual(len(credit.lines), 3)

        order = self.controller.save_or_update_order(None, [{
            'description': 'Some random product',
            'quantity': 12,
            'price': Decimal('50')
        }], merchant=self.merchant, parent_order=credit)
        self.controller.submit_order(order)

        # order should have 4 lines:
        # - order line
        # - matching rule (rule 3)
        # - persistent line from order 1
        # - another persistent line from order 1
        self.assertEqual(len(order.lines), 4)
        self.assertTrue(isinstance(order.lines[0], OrderLine))     # 600
        self.assertTrue(isinstance(order.lines[1], OrderLineRule)) # -10
        self.assertTrue(isinstance(order.lines[2], OrderLineRule)) # -10
        self.assertTrue(isinstance(order.lines[3], OrderLineRule)) # -10
        self.assertEquals(order.total_amount_excluding_vat, Decimal('570.0'))



    def test_persistent_resubmit_zero(self):

        self._attach_permissions_to_group(self.account.groups[0].group, 'view_mock_orders', 'manage_order_rules')
        self._setup_tax_and_gl_account()

        self._make_rule('persistent roel',
            match_all=True,
            conditions=[
                {
                    'condition': 'mock_quantity_gte',
                    'operator': OrderRuleCondition.OPERATOR_GT, 'value_text': None, 'value_bool': None, 'value_decimal': None,
                    'value_int': 10,
                }
            ],
            actions=[{
                'action': 'fixed_amount_penalty',
                'description': 'action description',
                'operator': None, 'value_text': None, 'value_bool': None, 'value_int': None,
                'value_decimal': 100,
                'gl_account_code': None,
                'tax_class_id': None,
                'is_persistent': True,
            }]
        )

        order = self.controller.save_or_update_order(None, [{
            'description': 'Pretty product',
            'quantity': 10,
            'price': Decimal('50')
        }], merchant=self.merchant)
        self.controller.submit_order(order)

        credit = self.controller._credit_order(order)

        order = self.controller.save_or_update_order(None, [{
            'description': 'Some random product',
            'quantity': 0,
            'price': Decimal('50')
        }], merchant=self.merchant, parent_order=credit)

        # dry-run the resubmit before resubmitting
        order_state = self.controller.load_order_state(order.id, submit_dry_run=True)

        # assert that there are 2 lines in the state
        self.assertEqual(len(order_state['lines']), 2)

        # real run
        self.controller.submit_order(order)


# TODO: add test for payment method thats archived

class TestTaxController(iqportal_unittest.IQPortalTestCase):

    def setUp(self):
        self.connect_db()

    def test_set_tax(self):

        account1 = self._create_active_account()
        self._attach_permissions_to_group(account1.groups[0].group, 'manage_tax_classes', 'manage_country_tax')

        country = self.sa_session.query(Country).filter(Country.name=='Netherlands').one()

        # create a tax class
        c = TaxController(self.sa_session, account_id = account1)
        tax_class = c.save_tax_class_state(None, {'name': 'tax class 1', 'code': 'TC1'})

        # assert saving second tax for same domain class gives error
        self.assertRaises(PublicUserException, c.save_tax_class_state, None, {'name': 'tax class 1', 'code': 'TC1'})

        # create a tax country rate
        tax_country = c.save_country_tax_state(None, {
            'tax_class_id': tax_class.id,
            'country_id': country.id,
            'tax_rates': [{'start_date': today(), 'rate': Decimal(12, 34)}]
        })

        # assert current fee
        self.assertEquals(tax_country.get_current_rate(), Decimal(12, 35))

        # assert other user in same domain can see the current tax rate
        license2 = self._create_license(domain=account1.groups[0].group.license.domain)
        account2 = self._create_active_account(license=license2)
        c = TaxController(self.sa_session, account_id = account2)
        rate = c.get_current_rate(country, tax_class.code)
        self.assertEquals(rate, Decimal(12, 35))


# below is for running the test from eclipse
if __name__ == '__main__':

    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_order_controller.py', 'basesoft.web.test.unit.test_order_controller.OrderControllerTestCase.test_bank_payment_method_pending_to_approved']
    run()
    sys.exit()

