# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.
from twisted.web.xmlrpc import Fault
from basesoft.util.virtual_date import set_date, now
from basesoft.web.contrib.scheduler.controllers import SchedulerServerController
from basesoft.web.core.exceptions import PublicSystemException
from basesoft.web.test.schedule import CronTask, LongTask

import datetime

import subprocess, os, commands, time, signal
import xmlrpclib

import basesoft
from django.conf import settings
from basesoft.web.test.unit.iqportal_unittest import IQPortalTestCase
from basesoft.scheduler.task import SIGNAL_STOP

from basesoft import logging
logger = logging.getLogger('basesoft.web.test.unit')

class TestSchedulerServer(IQPortalTestCase):

    def setUp(self):

        # inital datetime
        set_date(datetime.date(2013, 11, 17)) # set to sunday

        self._start_scheduler()
        self.client = xmlrpclib.Server(settings.SCHEDULER_SERVER_XMLRPC, allow_none=True)

    def test_echo(self):
        """
        simple connect test
        """
        self.assertEquals(self.client.echo('hello'), ['hello']) # echo returns all args so always a list

    def _get_task_id(self, name):
        for task in self.client.list_registered_tasks():
            if task['name'] == name:
                return task['task_id']
        return None

    def _get_process_id(self, name):
        for process in self.client.list_processes():
            if process['name'] == name:
                return process['process_id']
        return None

    def test_tasks(self):
        """
        Tests the xmlrpc methods for task management with the two test tasks
        """
        sleepy_id = self._get_task_id('basesoft.web.test.schedule.SleepyTask')
        ticky_id = self._get_task_id('basesoft.web.test.schedule.TickyTask')

        self.client.enable_task(sleepy_id)
        self.client.enable_task(ticky_id)

        self.assertTrue(sleepy_id is not None)
        self.assertTrue(ticky_id is not None)

        # now start the sleepy task
        self.client.start_task(sleepy_id, None)

        # get running task results
        sleepy_process_id = self._get_process_id('basesoft.web.test.schedule.SleepyTask')
        ticky_process_id = self._get_process_id('basesoft.web.test.schedule.TickyTask')

        # verify sleepy is running
        self.assertTrue(sleepy_process_id is not None)

        # verify that the ticky is not running
        self.assertTrue(ticky_process_id is None)

        # now also start the tickytask and wait a bit so ticky starts tickin
        ticky_process_id = self.client.start_task(ticky_id, None)
        time.sleep(4)

        # now ticky's status should be somewhere between 0 and 60
        status = self.client.get_process_status(ticky_process_id)
        self.assertTrue(status > 0 and status < 60)

        # sleep again and assert that status is higher than before
        time.sleep(1)
        self.assertTrue(self.client.get_process_status(ticky_process_id) > status)

        # kill ticky and verify its dead
        status = self.client.send_process_signal(ticky_process_id, SIGNAL_STOP)
        time.sleep(1)

        # shouldnt be possible to get process information anymore
        self.assertRaises(xmlrpclib.Fault, self.client.xmlrpc_get_process_status, ticky_process_id)

        # assert the result of this task is killed
        results = self.client.get_task_results(ticky_id)
        self.assertEqual(len(results), 1)
        self.assertEqual(results[0]['result'], 'killed by SIGNAL_STOP')

    def test_bad_task(self):

        # start task
        task_id = self._get_task_id('basesoft.web.test.schedule.BadTask')
        self.assertTrue(task_id is not None)
        self.client.enable_task(task_id)
        self.client.start_task(task_id, None)

        # sleep a second so the task can run and finish
        time.sleep(1)

        # get results for the task
        results = self.client.get_task_results(task_id)
        self.assertEqual(len(results), 1)
        result = results[0]

        # failure flag must be true and the exception message must be in result
        self.assertTrue(result['failure'])
        self.assertTrue(result['result'].find('grr!') >= 0)

    def test_db_task(self):
        """
        Tests task with db interaction by executing the task and retrieving its results
        """
        task_id = self._get_task_id('basesoft.web.test.schedule.DbTask')
        self.client.enable_task(task_id)
        self.client.start_task(task_id, None)

        # sleep a second so the task can run and finish
        time.sleep(2)

        # get results for the task
        results = self.client.get_task_results(task_id)
        self.assertEqual(len(results), 1)
        result = results[0]

        # assert result
        self.assertFalse(result['failure'])
        self.assertEquals(result['result'], 'all good!')

    def test_no_concurrent_task(self):
        """
        Tests that, by default, we cannot schedule a task that is already running or in the queue to run
        """
        task_id = self._get_task_id('basesoft.web.test.schedule.LongTask')
        self.client.enable_task(task_id)
        self.client.start_task(task_id, None)
        self.assertRaises(xmlrpclib.Fault, self.client.start_task, task_id, None)
        time.sleep(5) # wait for task to be done

    def test_concurrent_task(self):
        """
        Tests that, by default, we cannot schedule a task that is already running or in the queue to run
        """
        task_id = self._get_task_id('basesoft.web.test.schedule.DuplicateTask')
        self.client.enable_task(task_id)
        self.client.start_task(task_id, {'stuff': 'stuff'})
        self.client.start_task(task_id, {'stuff': 'more stuff'})

        #wait
        time.sleep(2)

        # get results for the task
        results = self.client.get_task_results(task_id)
        self.assertEqual(len(results), 2)
        result = results[1]

        # assert result
        self.assertFalse(result['failure'])
        self.assertEquals(result['result'], 'more stuff')


    def test_task_goes_to_queue(self):
        """
        Tests that by default we only have X number of tasks running, and additional tasks are queued and later executed.
        """
        queue_hog_task_id = self._get_task_id('basesoft.web.test.schedule.LongDuplicateTask')
        self.client.enable_task(queue_hog_task_id)
        task_id = self._get_task_id('basesoft.web.test.schedule.DuplicateTask')
        self.client.enable_task(task_id)
        self.client.start_task(queue_hog_task_id, None)
        self.client.start_task(queue_hog_task_id, None)
        self.client.start_task(queue_hog_task_id, None)
        self.client.start_task(queue_hog_task_id, None)
        self.client.start_task(queue_hog_task_id, None)
        self.client.start_task(task_id, {'stuff': 'I was in the queue!'})

        #wait a moment
        time.sleep(2)

        #make sure it hasn't run yet (with MAX_CONCURRENT_TASKS set to 5 it should not have)
        results = self.client.get_task_results(task_id)
        self.assertEquals(len(results), 0)

        # but it is currently running
        processes = self.client.list_processes()
        running = False
        for i in processes:
            if i['name'] == 'basesoft.web.test.schedule.DuplicateTask':
                running = True
        self.assertTrue(running)

        #wait for the queue hogs
        time.sleep(10)

        # get results for the task
        results = self.client.get_task_results(task_id)
        self.assertEqual(len(results), 1)
        result = results[0]

        # assert result
        self.assertFalse(result['failure'])
        self.assertEquals(result['result'], 'I was in the queue!')

    def test_queued_task_type(self):
        task_id = self._get_task_id('basesoft.web.test.schedule.DuplicateQueueableTask')
        self.assertTrue(task_id is not None)
        self.client.enable_task(task_id)
        self.client.start_task(task_id, None)
        self.client.start_task(task_id, None)
        self.client.start_task(task_id, None)
        self.assertRaises(Fault, self.client.start_task, task_id, None)

        processes = self.client.list_processes()
        running = False
        for i in processes:
            if i['name'] == 'basesoft.web.test.schedule.DuplicateQueueableTask':
                running = True
        self.assertTrue(running)

        tasks = self.client.list_task_queue_tasks()
        self.assertTrue(tasks.has_key('basesoft.web.test.schedule.DuplicateQueueableTask'))
        self.assertEqual(tasks['basesoft.web.test.schedule.DuplicateQueueableTask'], 2)

        # sleep for 6 seconds, the queue should now have one less item.
        time.sleep(6)
        tasks = self.client.list_task_queue_tasks()
        self.assertTrue(tasks.has_key('basesoft.web.test.schedule.DuplicateQueueableTask'))
        self.assertEqual(tasks['basesoft.web.test.schedule.DuplicateQueueableTask'], 1)

        time.sleep(6)
        tasks = self.client.list_task_queue_tasks()
        self.assertTrue(tasks.has_key('basesoft.web.test.schedule.DuplicateQueueableTask'))
        self.assertEqual(tasks['basesoft.web.test.schedule.DuplicateQueueableTask'], 0)

        time.sleep(6)


    def test_cron_task(self):

        task_id = self._get_task_id('basesoft.web.test.schedule.CronTask')
        self.assertTrue(task_id is not None)
        self.client.enable_task(task_id)

        self.assertEquals(len(self.client.get_task_results(task_id)), 0)

        # wait till after next run (less than a minute)
        s = CronTask.INTERVAL.next(now=now()) + 2
        logger.info('sleeping %s seconds' % s)
        time.sleep(s)

        # expect 1 task finished
        self.assertEquals(len(self.client.get_task_results(task_id)), 1)
        # expect result (task counter) to be 1
        self.assertEquals(self.client.get_task_results(task_id)[0]['result'], '1')

        # time before 2nd run (less than a minute)
        s = CronTask.INTERVAL.next(now=now()) + 2

        # set virtual date to monday, after the next run we dont expect it to run for a week
        # we have to do this before the next task is finished because it will be scheduled again after
        set_date(datetime.date(2013, 11, 18))

        # wait till after next run
        logger.info('sleeping %s seconds' % s)
        time.sleep(s)

        # task count should be 2 now with second task having 2 as result
        self.assertEquals(len(self.client.get_task_results(task_id)), 2)
        self.assertEquals(self.client.get_task_results(task_id)[1]['result'], '2')

        # it monday now.. wait a minute, we dont expect the task to have ran again
        logger.info('sleeping 60 seconds')
        time.sleep(60)
        self.assertEquals(len(self.client.get_task_results(task_id)), 2)

    def test_task_not_enabled(self):
        ssc = SchedulerServerController()
        self.assertRaises(PublicSystemException, ssc.system_start_task, 'basesoft.web.test.schedule.DisabledTask')
