# coding=utf-8
# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.
import os
import datetime
import codecs

from basesoft.web.contrib.contact.models import Person

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.contrib.ticketing.controllers import TicketController, TicketMailController
from basesoft.web.contrib.ticketing.models import Ticket

class TicketStub(object):
    """
    A stub used for testing. This will replace the xmlrpclib.ServerProxy the controller uses
    """
    def __init__(self, create_ticket=False, create_type=False):
        """
        Create the ticket stub + initializing some other objects also.
        @param create_ticket: if true, create an ticket object stub as well
        @param create_type: if true, create a type object on the stub as well (which is also a stub)
        """
        if create_ticket:
            self.ticket = TicketStub(create_type=True)

        if create_type:
            self.type = TicketStub()

        self.tickets = {} # a dictionary with ticket info, used by the stub methods

        self._attachments = {}

    def get_ticket_types(self):
        return ['defect', 'enhancement', 'task']

    def create(self, summary, description, attributes, notify=False):
        """
        stub for create
        """
        available_id = 1
        while self.tickets.has_key(str(available_id)):
            available_id = available_id + 1

        available_id = str(available_id)

        info = {
            'summary': summary,
            'description': description,
            'status': u'new',
            'resolution': u'',
            'type': self.get_ticket_types()[0],
            'comments': []
        }

        for k in attributes.keys():
            info[k] = attributes[k]

        self.tickets[available_id] = info

        return available_id

    def update(self, id, comment, attributes, notify=False):
        """
        stub for update
        """
        id = str(id)

        attr = self.tickets[id]
        if comment != '':
            attr['comments'].append(comment)

        for k in attributes.keys():
            attr[k] = attributes[k]

        self.tickets[id] = attr

    def get(self, id):
        """
        stub for get
        """
        return [id, None, None, self.tickets[str(id)]] # None for the dates, we don't care about that

    def changeLog(self, id):
        """
        stub for the changelog method, used for getting the comments in our case
        """
        comments = self.tickets[str(id)]['comments']

        ret = []
        # changelog is a line that contains example of what has changed. eg:
        # [<DateTime '20101021T11:12:10' at 9d5318c>, 'Mircea Uifalean', 'comment', '1', 'some comment', 1]
        # [<DateTime '20101021T11:12:10' at 9d531cc>, 'Mircea Uifalean', 'component', '', 'Core', 1]
        for c in comments:
            ret.append([None, '', 'comment', '', c, 1])

        return ret

    def listAttachments(self, ticket_id):

        if not self._attachments.has_key(ticket_id):
            self._attachments[ticket_id] = []

        return self._attachments[ticket_id]

    def putAttachment(self, ticket_id, file_name, account_name, content, replace):

        if not self._attachments.has_key(ticket_id):
            self._attachments[ticket_id] = []

        # ['inline.png', 'test_account_ffeab0be1b356bb1810bd9755c', 21916, <DateTime '20150327T11:10:40' at 5bb2878>, 'Marten Koedam']
        o = [file_name, account_name, 100, datetime.datetime.now(), account_name]

        for i in range(len(self._attachments[ticket_id])):

            if self._attachments[ticket_id][i][0] == file_name:
                self._attachments[ticket_id][i] = o
                return

        self._attachments[ticket_id].append(o)


class TicketingTestCase(iqportal_unittest.IQPortalTestCase):
    """
    Test the ticketing controller
    """

    def setUp(self):
        """
        Create the account and a controller
        """
        self.connect_db()
        account = self._create_active_account()
        self.account = account
        self.group = account.groups[0].group

        stub = TicketStub(create_ticket=True)
        self.controller = TicketController(self.session, account_id=account.id, server=stub)

    def test_controller(self):
        # give the permissions and onwards with the tests
        self._attach_permissions_to_group(self.group, 'create_ticket', 'view_ticket', 'delete_ticket',
                                          'update_ticket', 'create_ticket_comment', 'view_ticket_comment')

        state = {
            'id': None,
            'summary': u'ticket summary',
            'description': u'line1\nline2',
            'type': u'defect'
        }

        # this does the save and also loads the state back
        ticket_state = self.controller.save_or_update_ticket(state)

        for k in state.keys():
            if k == 'id':
                continue
            if k == 'summary':
                state[k] = '[%s] %s' % (self.group.license.domain.name, state[k])
            self.assertEqual(ticket_state[k],  state[k])

        self.assertEqual(ticket_state['trac_ticket_id'],  1) # first one we create with this test/controller/server
        self.assertEqual(ticket_state['status'],  'new')

        self.controller.get_tickets(ticket_state['id']).pop() # Ticket object
        self.assertDeepEqual(ticket_state, self.controller.load_ticket_state(ticket_state['id']))

        ticket_state['summary'] = u'updated summary'
        ticket_state['comment'] = u'first comment'
        ticket_state = self.controller.save_or_update_ticket(ticket_state)
        self.assertEqual( ticket_state['comments'][0], "<p>\nfirst comment <br />\n</p>\n<hr class=\"wiki\" />\n<p>\nReported by: %s <br />\n</p>\n" % self.account.name)


    def test_ticket_privileges(self):
        self._attach_permissions_to_group(self.group, 'create_ticket', 'view_ticket', 'delete_ticket',
                                  'update_ticket', 'create_ticket_comment', 'view_ticket_comment')

        state = {
            'id': None,
            'summary': u'ticket summary',
            'description': u'line1\nline2',
            'type': u'defect'
        }

        ticket_state = self.controller.save_or_update_ticket(state)
        ticket = self.controller.get_tickets(ticket_state['id']).pop() # Ticket object

        self.controller.revoke_privilege_from(ticket, Ticket.PRIV_ADD_COMMENT, group=True, check_own_privs=False)
        ticket_state['comment'] = u'My comment is not saved'
        ticket_state = self.controller.save_or_update_ticket(ticket_state)

        self.assertFalse(ticket_state['privileges']['comment'])
        self.assertEqual(ticket_state['comments'], [])

        summary = ticket_state['summary']
        self.controller.revoke_privilege_from(ticket, Ticket.PRIV_UPDATE, group=True, check_own_privs=False)
        ticket_state['summary'] = u'I cannot do this!'
        ticket_state = self.controller.save_or_update_ticket(ticket_state)

        self.assertFalse(ticket_state['privileges']['update'])
        self.assertEqual(ticket_state['summary'], summary)



class TicketingMailTestCase(iqportal_unittest.IQPortalTestCase):
    """
    todo: test attachments
    """


    def setUp(self):
        """
        Create the account and a controller
        """
        self.connect_db()

        create_ticket_person = Person()
        create_ticket_person.email = self._uniquestring('create_ticket') + '@mail-test.basesoft.com'
        self.account_create_ticket = self._create_active_account( person=create_ticket_person)

        self._attach_permissions_to_group(self.account_create_ticket.default_group, 'create_ticket', 'view_ticket', 'delete_ticket',
                                  'update_ticket', 'create_ticket_comment', 'view_ticket_comment')

        #self._ticket_server = None
        self._ticket_server = TicketStub(create_ticket=True)


    def test_create_ticket_from_email(self):

        raw_email = codecs.open(os.path.join(os.path.dirname(os.path.realpath(__file__)), 'files', 'ticket_email_complex.txt'), 'r', 'utf-8').read()

        tmc = TicketMailController(server=self._ticket_server)
        ticket_state = tmc.process_email(raw_email, self.account_create_ticket.person.email)

        self.assertEqual(len(ticket_state['attachments']), 3)
        self.assertTrue(ticket_state['description'].find(u'¿cuántos años tienes?') )


    def test_reply_ticket_from_email(self):

        state = {
            'id': None,
            'summary': u'ticket for reply',
            'description': u'line1\nline2',
            'type': u'Task'
        }

        tc = TicketController(self.sa_session, account_id=self.account_create_ticket.id, server=self._ticket_server)
        ticket_state = tc.save_or_update_ticket(state)

        raw_email = codecs.open(os.path.join(os.path.dirname(os.path.realpath(__file__)), 'files', 'ticket_email_complex.txt'), 'r', 'utf-8').read()

        # update subject to contain ticket id
        raw_email = raw_email.replace('Subject: new ticket:', 'Subject: Re: [Trac Sandbox] #%s:' % ticket_state['trac_ticket_id'])

        tmc = TicketMailController(server=self._ticket_server)
        ticket_state = tmc.process_email(raw_email, self.account_create_ticket.person.email)

        self.assertEqual(len(ticket_state['attachments']), 3)


    def test_forward_email_to_ticket(self):
        """
        """

        ticket_admin_person = Person()
        ticket_admin_person.email = self._uniquestring('owner') + '@mail-test.basesoft.com'
        ticket_admin_account = self._create_active_account( person=ticket_admin_person)

        self._attach_permissions_to_group(ticket_admin_account.default_group, 'create_ticket', 'view_ticket', 'delete_ticket',
                                  'update_ticket', 'create_ticket_comment', 'view_ticket_comment')

        forward_person = Person()
        forward_person.email = self._uniquestring('forward_person') + '@mail-test.basesoft.com'
        account_forward = self._create_active_account( person=forward_person)

        self._attach_permissions_to_group(account_forward.default_group, 'ticket_admin')

        raw_email = codecs.open(os.path.join(os.path.dirname(os.path.realpath(__file__)), 'files', 'ticket_email_forward.txt'), 'r', 'utf-8').read()

        raw_email = raw_email.replace(
            'From: 	Danielle de Kluijver <danielle.de.kluijver@teeentee.com>',
            'From: 	Danielle de Kluijver <%s>' % ticket_admin_person.email)

        tmc = TicketMailController(server=self._ticket_server)
        ticket_state = tmc.process_email(raw_email, forward_person.email)

        # this email has two inline screenshots and 1 small signature. expect only two images saved
        self.assertEqual(len(ticket_state['attachments']), 2)

        # assert that the forwarded_account has view privs
        tc = TicketController(self.sa_session, account_id = ticket_admin_account.id, server=self._ticket_server)
        ticket = tc.get_tickets(ticket_state['id']).pop()
        self.assertTrue(ticket is not None)

