'''
Created on Apr 17, 2013

@author: marten

This test all features related to the Contact import of Arvato.

Tickets: #3677, #3678 (TODO), #3705 (TODO)
'''
import os
import shutil


from datetime import datetime, timedelta
from django.conf import settings

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.contrib.file.models import FileTmp
from basesoft.web.contrib.contact.schedule import ImportContactsTask, AutomatedImportTask
from basesoft.web.contrib.i18n.models import Locale, Country, Language
from basesoft.web.contrib.license.models import License
from basesoft.web.contrib.contact.controllers import ContactController
from basesoft.web.core.util.controllers import LicenseConfigController

from iqportal.client.arvato.schedule import ArvatoImportXml
from iqportal.client.arvato.controllers import ArvatoImportContactController
from iqportal.client.arvato.models import ArvatoContact, ArvatoPerson
from iqportal.iquest.core.mailing import MailController
from iqportal.iquest.test.unit import iquest_unittest
from iqportal.iquest.core.models import SurveyPublication
from basesoft.web.contrib.compose.models import ComposePublicationContactList
from iqportal.iquest.core.schedule import CreatePublicationInviteesTask, PublicationMailing
from basesoft.web.contrib.mailing.models import MailResult
from basesoft.web.contrib.i18n.controllers import I18nController
from basesoft.web.contrib.notification.controllers import NotificationController

def get_field_mapping(tc, account):
    tc._attach_permissions_to_group(account.default_group, 'use_field_mapping_presets', 'manage_field_mapping_presets')

    cc = ContactController(tc.sa_session, account_id=account.id, contact_type='ArvatoContact')

    mapping = [
        {"field": "ContactGender", "target": {cc.COLUMN_IS_PERSON_INFO: 'gender'}},
        {"field": "ContactFirstname", "target": {cc.COLUMN_IS_PERSON_INFO: 'first_name'}},
        {"field": "ContactLastname", "target": {cc.COLUMN_IS_PERSON_INFO: 'last_name'}},
        {"field": "ContactEmail", "target": {cc.COLUMN_IS_PERSON_INFO: 'email'}},
        {"field": "ContactLanguage", "target": {cc.COLUMN_IS_PERSON_INFO: 'preferred_locale'}},
        {"field": "ContactCountry", "target": {cc.COLUMN_IS_PERSON_INFO: 'country_iso_3166_a2'}},

        {"field": "CaseId", "target": {cc.COLUMN_IS_CONTACT: 'case_id'}},
        {"field": "ContactChannel", "target": {cc.COLUMN_IS_CONTACT: 'channel'}},
        {"field": "ContactCompany", "target": {cc.COLUMN_IS_COMPANY_INFO: 'name'}},
        {"field": "CaseCreated", "target": {cc.COLUMN_IS_CONTACT: 'case_created'}},
        {"field": "CaseCreatedTeam", "target": {cc.COLUMN_IS_CONTACT: 'case_created_team'}},
        {"field": "CaseModified", "target": {cc.COLUMN_IS_NOT_IMPORTED: ''}},
        {"field": "CaseModifiedTeam", "target": {cc.COLUMN_IS_NOT_IMPORTED: ''}},
        {"field": "CaseStatus", "target": {cc.COLUMN_IS_CONTACT: 'case_status'}},
        {"field": "Complaint", "target": {cc.COLUMN_IS_CONTACT: 'complaint'}},

        {"field": "Taxonomy1", "target": {cc.COLUMN_IS_NOT_IMPORTED: ''}},
        {"field": "Taxonomy2", "target": {cc.COLUMN_IS_NOT_IMPORTED: ''}},
        {"field": "Taxonomy3", "target": {cc.COLUMN_IS_NOT_IMPORTED: ''}},

    ]

    unique_fields = [
        '%s+case_id' % cc.COLUMN_IS_CONTACT,
        '%s+name' % cc.COLUMN_IS_COMPANY_INFO,
    ]

    return cc.save_or_update_field_mapping(None, 'test1', 'ArvatoContact', mapping, unique_fields)

def smorgle_xml_file(tc, _license, fn='contact_import.xml'):
    # create a virtual ftp directory for this test
    dest = os.path.join(settings.TMP_FILE_REPOSITORY, 'arvato_ftp')
    if not os.path.exists(dest):
        os.mkdir(dest)

    # copy the xml testfile to the ftp where it will be picked up by the task
    xml_file = os.path.join(os.path.dirname(__file__), 'arvato', fn)
    shutil.copy(xml_file, dest)

    # run the task
    ArvatoImportXml().execute()

    # assert that an .csv is created in the contact import dir for the arvato license
    csv_file = os.path.join(settings.CONTACT_IMPORT_DIR, unicode(_license.id), '%s.csv' % fn)
    tc.assertTrue(os.path.exists(csv_file))

class ArvatoContactTestCase(iqportal_unittest.IQPortalTestCase):

    def setUp(self):

        self.connect_db()

        self.account = self._create_active_account()

        self.group = self.account.groups[0].group

        self._attach_permissions_to_group(self.group,
            'view_contact',
            'update_contact',
            'create_contact',
            'arvato_contact_access',
            'import_contacts',
            'view_file',
            'create_file'
        )

        self._file_hash = self._uniquestring('f')[:8]

        # create the file
        self.filepointer = os.path.join(settings.TMP_FILE_REPOSITORY, self._file_hash)
        self._f = open(self.filepointer, 'w')
        self._f.write(
            "%s\n%s\n%s\n%s\n%s" % (
                "ContactGender;ContactFirstname;ContactLastname;ContactCompany;ContactEmail;ContactLanguage;ContactCountry;CaseId;ContactChannel;CaseCreated;CaseCreatedTeam;CaseStatus;Complaint",
                "male;Sebastian;Ewald;;se.arvato@mail-test.basesoft.com;de;DE;46285;Inbound Mail;18.11.2013 19:11:00;Service Center;closed;N",
                "male;Julian;Pfeifer;;jp.arvato@mail-test.basesoft.com;nl;BE;46281;Inbound Mail;18.11.2013 18:44:21;Service Center;closed;N",
                "male;Stefan;Szwedziak;;ss.arvato@mail-test.basesoft.com;de;DE;46278;Inbound Mail;18.11.2013 18:19:24;Service Center;closed;N",
                "male;Nils;Weismann;RTCom;nw.arvato@mail-test.basesoft.com;fr;BE;46277;Inbound Mail;18.11.2013 18:12:30;Service Center;closed;N",
            )
        )

        self._f.close()

        self.controller = ArvatoImportContactController(self.filepointer, 'test.csv', self.session, self.account.id, 'ArvatoContact')
        self.controller.set_file_lines()

        self.controller.field_mappings = [
            {"file_header": "ContactGender", "column": "gender", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "ContactFirstname", "column": "first_name", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "ContactLastname", "column": "last_name", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "ContactCompany", "column": "name", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "ContactEmail", "column": "email", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "ContactLanguage", "column": "preferred_locale", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "ContactCountry", "column": "country_iso_3166_a2", "target": self.controller.COLUMN_IS_PERSON_INFO},

            {"file_header": "CaseId", "column": "case_id", "target": self.controller.COLUMN_IS_CONTACT},
            {"file_header": "ContactChannel", "column": "channel", "target": self.controller.COLUMN_IS_CONTACT},
            {"file_header": "CaseCreated", "column": "case_created", "target": self.controller.COLUMN_IS_CONTACT},
            {"file_header": "CaseCreatedTeam", "column": "case_created_team", "target": self.controller.COLUMN_IS_CONTACT},
            {"file_header": "CaseStatus", "column": "case_status", "target": self.controller.COLUMN_IS_CONTACT},
            {"file_header": "Complaint", "column": "complaint", "target": self.controller.COLUMN_IS_CONTACT},
        ]

        self.unique_fields = [{"column": "case_id", "target": self.controller.COLUMN_IS_CONTACT},
                              {"column": "email", "target": self.controller.COLUMN_IS_PERSON_INFO}]


    def _assert_contacts(self, contacts):

        de_locale = self.sa_session.query(Locale).filter(Locale.locale=='de_DE').one()
        benl_locale = self.sa_session.query(Locale).filter(Locale.locale=='nl_BE').one()
        befr_locale = self.sa_session.query(Locale).filter(Locale.locale=='fr_BE').one()

        self.assertEqual(len(contacts), 4)

        self.assertTrue(isinstance(contacts[0], ArvatoContact))
        self.assertEqual(contacts[0].case_id, 46285)
        self.assertEqual(contacts[0].channel, 'Inbound Mail')
        self.assertEqual(contacts[0].case_created, datetime(2013, 11, 18, 19, 11, 0))
        self.assertEqual(contacts[0].case_created_team, 'Service Center')
        self.assertEqual(contacts[0].case_status, 'closed')
        self.assertEqual(contacts[0].complaint, 'N')
        self.assertTrue(isinstance(contacts[0].person, ArvatoPerson))
        self.assertEqual(contacts[0].person.preferred_locale, de_locale)

        self.assertTrue(isinstance(contacts[1], ArvatoContact))
        self.assertTrue(isinstance(contacts[1].person, ArvatoPerson))
        self.assertEqual(contacts[1].person.preferred_locale, benl_locale)

        self.assertTrue(isinstance(contacts[2], ArvatoContact))
        self.assertTrue(isinstance(contacts[2].person, ArvatoPerson))
        self.assertEqual(contacts[2].person.preferred_locale, de_locale)

        self.assertTrue(isinstance(contacts[3], ArvatoContact))
        self.assertTrue(isinstance(contacts[3].person, ArvatoPerson))
        self.assertEqual(contacts[3].person.preferred_locale, befr_locale)

    def test_simple_import(self):
        """
        This tests the "interface" import for Arvato contacts (#3677)
        """
        contacts = self.controller.import_contacts(unique_fields=self.unique_fields)
        self._assert_contacts(contacts)

    def test_uniqueness(self):
        """
        This tests if the unique fields for Arvato contacts work as intended.
        """
        contacts = self.controller.import_contacts(unique_fields=self.unique_fields)
        ids = [c.id for c in contacts]
        contacts = self.controller.import_contacts(unique_fields=self.unique_fields)
        self.assertEqual(len(contacts), 4)
        for c in contacts:
            self.assertTrue(c.id in ids)

    def test_background_import_arvato_contact(self):
        """
        This tests the background import for Arvato contacts (#3677)
        """

        f = FileTmp()
        f.file_name = 'uploaded_test_file.csv'
        f.file_hash = self._file_hash
        self.sa_session.add(f)
        self.sa_session.flush()

        import_settings = {
            'separator': None,
            'encoding': None,
            'field_mappings': self.controller.field_mappings,
            'import_row1': False,
            'contact_type': 'ArvatoContact',
            'contact_list': None,
            'updated_rows': None,
            'excluded_rows': None,
            'unique_fields': self.unique_fields,
            'account_id': self.account.id,
            'group_id': self.group.id,
            'do_validation': True
        }
        self.controller.save_file([f], import_settings)

        ImportContactsTask().execute()

        contacts = self.controller.get_contacts()
        self._assert_contacts(contacts)


class ArvatoXmlTestCase(iqportal_unittest.IQPortalTestCase):

    def setUp(self):
        self.connect_db()
        if not os.path.exists(settings.ARVATO_LOCAL_PATH):
            os.mkdir(settings.ARVATO_LOCAL_PATH)

    def test_xml_import_including_belgian(self):
        # create a virtual ftp directory for this test
        dest = os.path.join(settings.TMP_FILE_REPOSITORY, 'arvato_ftp')
        if not os.path.exists(dest):
            os.mkdir(dest)

        # assert that an .csv is created in the contact import dir for the arvato license
        license = self.sa_session.query(License).filter(License.name == settings.ARVATO_LICENSE_NAME).one()

        # reset tblDomainCountry for arvato
        self.sa_session.execute('DELETE FROM tblDomainCountry WHERE domain_id = :domain_id', {'domain_id': license.domain.id})
        self.sa_session.execute('DELETE FROM tblDomainLanguage WHERE domain_id = :domain_id', {'domain_id': license.domain.id})

        self.account = self._create_active_account(license)

        self.group = self.account.groups[0].group

        self._attach_permissions_to_group(self.group,
            'view_contact',
            'update_contact',
            'create_contact',
            'arvato_contact_access',
            'import_contacts',
            'view_file',
            'create_file',
            'automated_imports',
            'create_contact_list',
            'view_contact_list',
            'link_to_contact_list',
            'view_notification',
            'manage_domain_language_aliases',
        )

        i18nc = I18nController(self.sa_session, account_id=self.account.id)
        i18nc.save_domain_language_alias(self.sa_session.query(Language).filter(Language.iso_639_1=='en').first().id, 'Belgian')

        smorgle_xml_file(self, license, 'contact_import_including_belgian.xml')

        # assert that the xml file is gone from the ftp
        copied_xml_file = os.path.join(dest, 'contact_import_including_belgian.xml')
        self.assertFalse(os.path.exists(copied_xml_file), 'exists: %s' % copied_xml_file)

        # add field mapping preset
        fm = get_field_mapping(self, self.account)
        cc = ContactController(self.sa_session, account_id=self.account.id, contact_type='ArvatoContact')

        cl = cc.save_or_update_contact_list(None, self._uniquestring('cl'))

        lcc = LicenseConfigController(self.sa_session, scope=license)
        lcc.set('contact.import.default.field.mapping.id', fm.id)
        lcc.set('contact.import.default.contact.list.id', cl.id)

        t = AutomatedImportTask()
        t.execute()

        # verify file has been added to the background import queue
        it = ImportContactsTask()
        it.execute()

        # we should have 2 newly created contacts for the current license, this includes the belgian language LMFAO
        self.assertEqual(len(cc.get_contacts()),2)
        self.assertEqual(len(cc.get_contacts(None, contact_list_id=cl.id)),2)

    def test_xml_import_including_belgian_failure(self):
        # create a virtual ftp directory for this test
        dest = os.path.join(settings.TMP_FILE_REPOSITORY, 'arvato_ftp')
        if not os.path.exists(dest):
            os.mkdir(dest)

        # assert that an .csv is created in the contact import dir for the arvato license
        license = self.sa_session.query(License).filter(License.name == settings.ARVATO_LICENSE_NAME).one()

        # reset tblDomainCountry for arvato
        self.sa_session.execute('DELETE FROM tblDomainCountry WHERE domain_id = :domain_id', {'domain_id': license.domain.id})
        self.sa_session.execute('DELETE FROM tblDomainLanguage WHERE domain_id = :domain_id', {'domain_id': license.domain.id})

        self.account = self._create_active_account(license)

        self.group = self.account.groups[0].group

        self._attach_permissions_to_group(self.group,
            'view_contact',
            'update_contact',
            'create_contact',
            'arvato_contact_access',
            'import_contacts',
            'view_file',
            'create_file',
            'automated_imports',
            'create_contact_list',
            'view_contact_list',
            'link_to_contact_list',
            'view_notification',
        )

        # get current unread nofication count
        nc = NotificationController(self.sa_session, account_id=self.account.id)
        cnt = nc.get_unread_notifications_count()

        smorgle_xml_file(self, license, 'contact_import_including_belgian.xml')

        self.assertEqual(cnt+1, nc.get_unread_notifications_count())

        # assert that the xml file is gone from the ftp
        copied_xml_file = os.path.join(dest, 'contact_import_including_belgian.xml')
        self.assertFalse(os.path.exists(copied_xml_file), 'exists: %s' % copied_xml_file)

        # add field mapping preset
        fm = get_field_mapping(self, self.account)
        cc = ContactController(self.sa_session, account_id=self.account.id, contact_type='ArvatoContact')

        cl = cc.save_or_update_contact_list(None, self._uniquestring('cl'))

        lcc = LicenseConfigController(self.sa_session, scope=license)
        lcc.set('contact.import.default.field.mapping.id', fm.id)
        lcc.set('contact.import.default.contact.list.id', cl.id)

        t = AutomatedImportTask()
        t.execute()

        # verify file has been added to the background import queue
        it = ImportContactsTask()
        it.execute()

        # we should have 1 newly created contact for the current license, this excludes the belgian language LMFAO
        self.assertEqual(len(cc.get_contacts()),1)
        self.assertEqual(len(cc.get_contacts(None, contact_list_id=cl.id)),1)

    def test_xml_import(self):

        # create a virtual ftp directory for this test
        dest = os.path.join(settings.TMP_FILE_REPOSITORY, 'arvato_ftp')
        if not os.path.exists(dest):
            os.mkdir(dest)

        # assert that an .csv is created in the contact import dir for the arvato license
        license = self.sa_session.query(License).filter(License.name == settings.ARVATO_LICENSE_NAME).one()

        # reset tblDomainCountry for arvato
        self.sa_session.execute('DELETE FROM tblDomainCountry WHERE domain_id = :domain_id', {'domain_id': license.domain.id})

        self.account = self._create_active_account(license)

        self.group = self.account.groups[0].group

        self._attach_permissions_to_group(self.group,
            'view_contact',
            'update_contact',
            'create_contact',
            'arvato_contact_access',
            'import_contacts',
            'view_file',
            'create_file',
            'automated_imports',
            'create_contact_list',
            'view_contact_list',
            'link_to_contact_list',
            'view_notification',
        )

        # get current unread nofication count
        nc = NotificationController(self.sa_session, account_id=self.account.id)
        cnt = nc.get_unread_notifications_count()

        smorgle_xml_file(self, license)

        self.assertEqual(cnt+1, nc.get_unread_notifications_count())

        # assert that the xml file is gone from the ftp
        copied_xml_file = os.path.join(dest, 'contact_import.xml')
        self.assertFalse(os.path.exists(copied_xml_file), 'exists: %s' % copied_xml_file)

        # add field mapping preset
        fm = get_field_mapping(self, self.account)
        cc = ContactController(self.sa_session, account_id=self.account.id, contact_type='ArvatoContact')

        cl = cc.save_or_update_contact_list(None, self._uniquestring('cl'))

        lcc = LicenseConfigController(self.sa_session, scope=license)
        lcc.set('contact.import.default.field.mapping.id', fm.id)
        lcc.set('contact.import.default.contact.list.id', cl.id)

        t = AutomatedImportTask()
        t.execute()

        # verify file has been added to the background import queue
        it = ImportContactsTask()
        it.execute()

        # we should have 22 newly created contacts for the current license, this excludes 3 with UK country name
        self.assertEqual(len(cc.get_contacts()),22)
        self.assertEqual(len(cc.get_contacts(None, contact_list_id=cl.id)),22)

    def test_xml_import_including_UK(self):

        # create a virtual ftp directory for this test
        dest = os.path.join(settings.TMP_FILE_REPOSITORY, 'arvato_ftp')
        if not os.path.exists(dest):
            os.mkdir(dest)

        # assert that an .csv is created in the contact import dir for the arvato license
        license = self.sa_session.query(License).filter(License.name == settings.ARVATO_LICENSE_NAME).one()

        # reset tblDomainCountry for arvato
        self.sa_session.execute('DELETE FROM tblDomainCountry WHERE domain_id = :domain_id', {'domain_id': license.domain.id})

        self.account = self._create_active_account(license)

        self.group = self.account.groups[0].group

        self._attach_permissions_to_group(self.group,
            'view_contact',
            'update_contact',
            'create_contact',
            'arvato_contact_access',
            'import_contacts',
            'view_file',
            'create_file',
            'automated_imports',
            'create_contact_list',
            'view_contact_list',
            'link_to_contact_list',
            'manage_domain_country_mapping'
        )

        i18nc = I18nController(self.sa_session, account_id=self.account.id)
        i18nc.save_domain_country(self.sa_session.query(Country).filter(Country.iso_3166_a2=='GB').first().id, 'UK')

        smorgle_xml_file(self, license)

        # assert that the xml file is gone from the ftp
        copied_xml_file = os.path.join(dest, 'contact_import.xml')
        self.assertFalse(os.path.exists(copied_xml_file), 'exists: %s' % copied_xml_file)

        # add field mapping preset
        fm = get_field_mapping(self, self.account)
        cc = ContactController(self.sa_session, account_id=self.account.id, contact_type='ArvatoContact')

        cl = cc.save_or_update_contact_list(None, self._uniquestring('cl'))

        lcc = LicenseConfigController(self.sa_session, scope=license)
        lcc.set('contact.import.default.field.mapping.id', fm.id)
        lcc.set('contact.import.default.contact.list.id', cl.id)

        t = AutomatedImportTask()
        t.execute()

        # verify file has been added to the background import queue
        it = ImportContactsTask()
        it.execute()

        # get current unread nofication count

        # we should have 22 newly created contacts for the current license, this excludes 3 with UK country name
        self.assertEqual(len(cc.get_contacts()),25)
        self.assertEqual(len(cc.get_contacts(None, contact_list_id=cl.id)),25)

class ArvatorCsvTestCase(iqportal_unittest.IQPortalTestCase):
    def setUp(self):
        self.connect_db()

    def test_import_csv(self):


        _license = self.sa_session.query(License).filter(License.name==settings.ARVATO_LICENSE_NAME).one()
        a = self._create_active_account(license=_license)
        self.account = a
        g = a.default_group
        self.group = g

        dest = os.path.join(settings.CONTACT_IMPORT_DIR, '%s' % _license.id)
        if not os.path.exists(dest):
            os.mkdir(dest)

        csv_file = os.path.join(os.path.dirname(__file__), 'arvato', 'contacts.csv')
        shutil.copy(csv_file, dest)

        # Grant the correct permissions.
        permissions = [
            'view_mail', 'create_mail', 'view_iquest_mail', 'create_iquest_mail', 'schedule_iquest_mail', # mailing
            'view_file', 'create_file', # files
            'view_contact', 'update_contact', 'create_contact', 'import_contacts', 'automated_imports', 'arvato_contact_access',  # contacts
            'create_contact_list', 'view_contact_list', 'link_to_contact_list', # contact lists
            'view_survey', 'create_survey', # survey
            'view_survey_publication', 'create_survey_publication', 'schedule_survey_publication', # survey publication
            'use_field_mapping_presets', 'manage_field_mapping_presets',
        ]
        self._attach_permissions_to_group(g, *permissions)

        cc = ContactController(self.sa_session, account_id=self.account.id, contact_type='ArvatoContact')

        mapping = [{"field": "ContactGender", "target": {"1": "gender"}}, {"field": "ContactFirstname", "target": {"1": "first_name"}}, {"field": "ContactLastname", "target": {"1": "last_name"}}, {"field": "ContactEmail", "target": {"1": "email"}}, {"field": "ContactLanguage", "target": {"1": "preferred_locale"}}, {"field": "ContactCountry", "target": {"1": "country_iso_3166_a2"}}, {"field": "CaseId", "target": {"4": "case_id"}}, {"field": "ContactChannel", "target": {"4": "channel"}}, {"field": "ContactCompany", "target": {"2": "name"}}, {"field": "CaseCreated", "target": {"4": "case_created"}}, {"field": "CaseCreatedTeam", "target": {"4": "case_created_team"}}, {"field": "CaseModified", "target": {"0": ""}}, {"field": "CaseModifiedTeam", "target": {"0": ""}}, {"field": "CaseStatus", "target": {"4": "case_status"}}, {"field": "Complaint", "target": {"4": "complaint"}}, {"field": "Taxonomy1", "target": {"0": ""}}, {"field": "Taxonomy2", "target": {"0": ""}}, {"field": "Taxonomy3", "target": {"0": ""}}]

        unique_fields = [
            '%s+case_id' % cc.COLUMN_IS_CONTACT,
            '%s+name' % cc.COLUMN_IS_COMPANY_INFO,
        ]

        fm = cc.save_or_update_field_mapping(None, 'test1', 'ArvatoContact', mapping, unique_fields)
        lcc = LicenseConfigController(self.sa_session, scope=_license)
        lcc.set('contact.import.default.field.mapping.id', fm.id) # expire after 1 day

        # Create an empty contact list.
        cc = ContactController(self.sa_session, account_id=a.id, contact_type='Contact')
        cl = cc.save_or_update_contact_list(None, self._uniquestring('cl'))

        # add at least one contact to list
        contact = cc.save_or_update_contact(None, {'first_name': 'Arvato', 'email': 'arvato@mail-test.basesoft.com'})
        cc.link_contacts([contact], cl)

        cc = ContactController(self.sa_session, account_id=a.id, contact_type='ArvatoContact')

        # Set contact list as default in License Config.
        lcc.set('contact.import.default.contact.list.id', cl.id)

        # Trigger Automated Import task - this adds the file to the import queue
        t = AutomatedImportTask()
        t.execute()

        # Trigger import
        it = ImportContactsTask()
        it.execute()

class ArvatoFullCycleTestCase(iquest_unittest.IQuestTestCase):
    """
    Test case to test the whole cycle of Arvato
    """
    def setUp(self):
        self.connect_db()
        if not os.path.exists(settings.ARVATO_LOCAL_PATH):
            os.mkdir(settings.ARVATO_LOCAL_PATH)

    def test_arvato_cycle(self):
        # Create an Arvato license account.
        _license = self.sa_session.query(License).filter(License.name==settings.ARVATO_LICENSE_NAME).one()
        a = self._create_active_account(license=_license)

        # reset tblDomainCountry for arvato
        self.sa_session.execute('DELETE FROM tblDomainCountry WHERE domain_id = :domain_id', {'domain_id': _license.domain.id})

        self.account = a
        g = a.default_group
        self.group = g

        # Grant the correct permissions.
        permissions = [
            'view_mail', 'create_mail', 'view_iquest_mail', 'create_iquest_mail', 'schedule_iquest_mail', # mailing
            'view_file', 'create_file', # files
            'view_contact', 'update_contact', 'create_contact', 'import_contacts', 'automated_imports', 'arvato_contact_access',  # contacts
            'create_contact_list', 'view_contact_list', 'link_to_contact_list', # contact lists
            'view_survey', 'create_survey', # survey
            'view_survey_publication', 'create_survey_publication', 'schedule_survey_publication', # survey publication
            'manage_domain_country_mapping',
        ]
        self._attach_permissions_to_group(g, *permissions)

        # update domain country, set UK
        i18nc = I18nController(self.sa_session, account_id=self.account.id)
        i18nc.save_domain_country(self.sa_session.query(Country).filter(Country.iso_3166_a2=='GB').first().id, 'UK')

        # get field mapping
        fm = get_field_mapping(self, a)

        # Add the license configs for Arvato.
        lcc = LicenseConfigController(self.sa_session, scope=_license)
        lcc.set('basesoft.web.response.invitee_expiry_days', 1) # expire after 1 day
        lcc.set('contact.import.default.field.mapping.id', fm.id) # expire after 1 day

        locale = self.sa_session.query(Locale).filter(Locale.locale=='en_GB').one()
        self.locale = locale
        # Create an IQuest Invitation mail.
        mc = MailController(self.session, locale, account_id=a.id)
        mail = mc.save_state({
            "mail": {
                "name": 'Arvato Invitation Email',
                "objectName": u"IQuestMail",
                "mail_from_text": "Arvato Test",
                "dbID": None,
                "locales": {
                    'en_GB': {
                        'subject': u"Arvato customer satisfaction invite test",
                        'content': u'Dear <img class="replacement-field" src="Contact:Name" title="person.last_name"/> \
                            <br /><br /> \
                            Clicky on the link:.<br/>\
                            <img class="replacement-field" src="Survey:Link" title="iquest.survey.link"/>',
                    },
                    'de_DE': {
                        'subject': u"Arvato deutsch invitation test",
                        'content': u'Guten tag <img class="replacement-field" src="Contact:Name" title="person.last_name"/> \
                            <br /><br /> \
                            Clicky wiemahl on ze link:.<br/>\
                            <img class="replacement-field" src="Survey:Link" title="iquest.survey.link"/>',
                    },
                },
                "fallbackLocale": {'value': u'en_GB' },
            }
        }, g)

        # Create an empty contact list.
        cc = ContactController(self.sa_session, account_id=a.id, contact_type='Contact')
        cl = cc.save_or_update_contact_list(None, self._uniquestring('cl'))

        # add at least one contact to list
        contact = cc.save_or_update_contact(None, {'first_name': 'Arvato', 'email': 'arvato@mail-test.basesoft.com'})
        cc.link_contacts([contact], cl)

        cc = ContactController(self.sa_session, account_id=a.id, contact_type='ArvatoContact')

        # Set contact list as default in License Config.
        lcc.set('contact.import.default.contact.list.id', cl.id)

        sd = datetime.now()
        # Create a Survey and publish it with the contact list attached
        mailings = [{
            "dbID": None,
            "domID": u'publish_1-mailing_1',
            "mail_id": mail.id,
            "send_date": sd,
            "contact_lists": [cl.id],
            'mail_from_text': 'Arvato'
        }]
        survey_id = self.publish_survey(SurveyPublication.STATUS_SCHEDULED, begin_date=sd-timedelta(seconds=5), contact_lists=[cl.id], anonymous=False, mailings=mailings)

        # Trigger mailing
        pm = PublicationMailing()
        pm.execute()

        self.assertEqual(self.sa_session.query(MailResult).filter(MailResult.mailing==self._publication.mailings[0]).count(), 1)

        self._publication.mailings[0].sending_grace_period = None
        self.sa_session.add(self._publication.mailings[0])
        self.sa_session.flush()

        dest = os.path.join(settings.TMP_FILE_REPOSITORY, 'arvato_ftp')
        if not os.path.exists(dest):
            os.mkdir(dest)

        # Get the contacts from the XML file
        smorgle_xml_file(self, _license)

        # Trigger Automated Import task - this adds the file to the import queue
        t = AutomatedImportTask()
        t.execute()

        # Trigger import
        it = ImportContactsTask()
        it.execute()

        # Verify status of publication contact list
        cpcl = self.sa_session.query(ComposePublicationContactList).filter(ComposePublicationContactList.publication_id==self._publication.id).one()
        self.assertEqual(cpcl.contact_list, cl)
        self.assertEqual(cpcl.is_processed, ComposePublicationContactList.STATUS_REPROCESS)

        # Trigger creation of new invitees
        cpit = CreatePublicationInviteesTask()
        cpit.execute()

        # Verify process status
        self.sa_session.expire(cpcl)
        self.assertEqual(cpcl.is_processed, ComposePublicationContactList.STATUS_PROCESSED)

        # Verify number of invitees
        self.sa_session.expire(self._publication)
        self.assertEqual(len(self._publication.invitees), 26) # 25 from xml + 1 default.

        # Trigger mailing
        pm = PublicationMailing()
        pm.execute()

        # Verify emails are sent (check db status).
        self.assertEqual(self.sa_session.query(MailResult).filter(MailResult.mailing==self._publication.mailings[0]).count(), 26)

        # Drink beer!!!
















