# Copyright (c) 2012 BaseSoft B.V.
# See LICENSE for details.
import os

from sqlalchemy.sql.expression import and_
from sqlalchemy.orm.exc import NoResultFound

from django.conf import settings

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.contrib.account.models import Account, Config
from basesoft.web.contrib.contact.models import Contact, ContactList

from iqportal.client.test.controllers import FmeTestController
from iqportal.client.core.models import FMEContact,FMEPerson, FMECompany
from iqportal.client.core.controllers import FMEImportContactController
from basesoft.web.contrib.contact.schedule import ImportContactsTask
from basesoft.web.contrib.file.models import FileTmp
from basesoft.web.contrib.file.controllers import UploadFileController
from basesoft.web.contrib.notification.controllers import NotificationController
from basesoft.web.contrib.license.models import LicenseConfig

class FMEContactImportTestCase(iqportal_unittest.IQPortalTestCase):

    def setUp(self):
        """
        The script for reseting the db and creating all the tables
        """
        self.connect_db()

        account = self.session.query(Account).filter(Account.name=='sysadmin').one()
        # we populate necessary fme account data
        c = FmeTestController(self.session, account_id=account.id)
        c.populate_fme_domain()
        self.feda_license = c.feda_license
        self.fedet_license = c.fedet_license
        self.account = c.admin_account

        self.group = self.account.groups[0].group

        self._file_headers = ["CN","COMPANY","NAME","CL","PN","LAST_UPDATE", "UITGROEI","EMAIL"]
        self._contact1 = ["10065","Yokogawa Europe B.V.","Van der Geer","FEDA: fmecl1","18116249","2011-01-29","1","bitbucket@iqportalmail.com"]
        self._contact2 = ["10102","Eaton Industries B.V.","Oomen","FEDET: fmecl2","1552329","2011-01-29", "","bitbucket@iqportalmail.com"]

        self._file_hash = self._uniquestring('f')[:8]

        # create the file
        self.filepointer = os.path.join(settings.TMP_FILE_REPOSITORY, self._file_hash)
        self._f = open(self.filepointer, 'w')
        self._f.write("%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s" % (
                    ','.join(self._file_headers),
                    ','.join(self._contact1),
                    ','.join(self._contact2),
                    "10102,Eaton Industries B.V.,Meulenbroek,FEDET: fmecl3,18483154,2011-01-29,,bitbucket@iqportalmail.com",
                    "10102,Eaton Industries B.V.,Van der Geer,FEDET: fmecl3,18116249,2011-01-29,,bitbucket@iqportalmail.com",
                    "10173,Honeywell B.V.,Vega,FEDA: fmecl4,1670613,2011-01-29,,bitbucket@iqportalmail.com",
                    "10173,Honeywell B.V.,Van de Zwaard,FEDA: fmecl5,1887140,2011-01-29,,bitbucket@iqportalmail.com",
                    "10173,Honeywell B.V.,Van der Poel,FEDA: fmecl6,2488331,2011-01-29,1,bitbucket@iqportalmail.com",
                    "10173,Honeywell B.V.,De Breuk,FEDA: fmecl6,18464084,2011-01-29,,bitbucket@iqportalmail.com",

                    "10102,Eaton Industries B.V.,Oomen,FEDA: fmecl7,1552329,2011-01-29,,bitbucket@iqportalmail.com",
        ))
        self._f.close()

        self.controller = FMEImportContactController(self.filepointer, 'test.csv', self.session, self.account.id, 'FMEContact')
        self.controller.set_file_lines()
        self.controller.field_mappings = [
            {"file_header": "CN", "column": "company_number", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "COMPANY", "column": "name", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "NAME", "column": "last_name", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "CL", "column": "contact_list_name", "target": self.controller.COLUMN_IS_CONTACT_LIST},
            {"file_header": "PN", "column": "person_number", "target": self.controller.COLUMN_IS_PERSON_INFO}
        ]
        self.unique_fields = [{"column": "company_number", "target": self.controller.COLUMN_IS_COMPANY_INFO},
                              {"column": "person_number", "target": self.controller.COLUMN_IS_PERSON_INFO}]

    def _check_privs(self, contact):
        has_feda_priv = False
        has_fedet_priv = False
        for cgo in contact.groups:
            if cgo.group.id == self.feda_license.default_group.id and cgo.license_has_privilege == 1 and\
            cgo.privilege in Contact.all_privileges:
                has_feda_priv = True
            if cgo.group.id == self.fedet_license.default_group.id and cgo.license_has_privilege == 1 and\
            cgo.privilege in Contact.all_privileges:
                has_fedet_priv = True
        return has_feda_priv, has_fedet_priv

    def test_background_import(self):

        f = FileTmp()
        f.file_name = 'uploaded_test_file.csv'
        f.file_hash = self._file_hash
        self.sa_session.add(f)
        self.sa_session.flush()

        self.controller.field_mappings.append({"file_header": "LAST_UPDATE", "column": "do_not_import", "target": 0})
        self.controller.field_mappings.append({"file_header": "UITGROEI", "column": "mail_settings", "target": self.controller.COLUMN_IS_CONTACT_LIST_CONTACT})
        self.controller.field_mappings.append({"file_header": "EMAIL", "column": "contact_email", "target": self.controller.COLUMN_IS_CONTACT})

        import_settings = {
            'separator': None,
            'encoding': None,
            'field_mappings': self.controller.field_mappings,
            'import_row1': False,
            'contact_type': 'FMEContact',
            'contact_list': None,
            'updated_rows': None,
            'excluded_rows': None,
            'unique_fields': self.unique_fields,
            'account_id': self.account.id,
            'group_id': self.group.id,
            'do_validation': True
        }
        self.controller.save_file([f], import_settings)

        self._attach_permissions_to_group(self.group, 'view_notification')

        nc = NotificationController(self.sa_session, account_id=self.account.id)
        cnt = len(nc.get_notifications())

        ImportContactsTask().execute()

        notifications = len(nc.get_notifications())
        self.assertEqual(cnt+1, notifications)

    def test_background_import_failure_file(self):
        f = FileTmp()
        f.file_name = 'uploaded_test_file.csv'
        f.file_hash = self._file_hash
        self.sa_session.add(f)
        self.sa_session.flush()

        c = self.sa_session.query(Config).filter(Config.name=='file.import.failure.email.notification.address').one()
        lc = self.sa_session.query(LicenseConfig).filter(and_(LicenseConfig.config==c, LicenseConfig.license==self.group.license)).first()
        if lc is None:

            lc = LicenseConfig()
            lc.config = c
            lc.license = self.group.license
            lc.value = 'import-failure@mail-test.basesoft.com'
            self.sa_session.add(lc)

        self.controller.field_mappings.append({"file_header": "LAST_UPDATE", "column": "do_not_import", "target": 0})
        self.controller.field_mappings.append({"file_header": "UITGROEI", "column": "contact_email", "target": self.controller.COLUMN_IS_CONTACT_LIST_CONTACT})
        self.controller.field_mappings.append({"file_header": "EMAIL", "column": "do_not_import", "target": 0})

        import_settings = {
            'separator': None,
            'encoding': None,
            'field_mappings': self.controller.field_mappings,
            'import_row1': False,
            'contact_type': 'FMEContact',
            'contact_list': None,
            'updated_rows': None,
            'excluded_rows': None,
            'unique_fields': self.unique_fields,
            'account_id': self.account.id,
            'group_id': self.group.id,
            'do_validation': True
        }
        self.controller.save_file([f], import_settings)

        self._attach_permissions_to_group(self.group, 'view_notification')

        nc = NotificationController(self.sa_session, account_id=self.account.id)
        cnt = len(nc.get_notifications())

        ImportContactsTask().execute()

        notifications = len(nc.get_notifications())
        self.assertEqual(cnt+1, notifications)

    def test_simple_import(self):
        """
        Just use the file provided for import.
        """

        contacts = self.controller.import_contacts(unique_fields=self.unique_fields)

        # we have 3 distinct companies and 7 distinct persons, one person belongs to 2 companies and one contact belongs
        # to 2 contact lists from 2 different licenses
        # so we have 8 contacts
        companies = []
        persons = []
        distinct_contacts = []
        for c in contacts:
            if c.id not in distinct_contacts:
                distinct_contacts.append(c.id)
            if c.company.id not in companies:
                companies.append(c.company.id)
            if c.person.id not in persons:
                persons.append(c.person.id)
        self.assertEqual(len(distinct_contacts), 8)
        self.assertEqual(len(companies), 3)
        self.assertEqual(len(persons), 7)

        # check imported contact lists
        cls = self.session.query(ContactList).filter(ContactList.name.in_(['fmecl1', 'fmecl2' ,'fmecl3' ,'fmecl4',
                                                                           'fmecl5', 'fmecl6', 'fmecl7'])).all()
        self.assertEqual(len(cls), 7)

        # check if all contacts and contact lists are saved for the correct license

        # we have one contact in both licenses
        ctc = None
        for c in contacts:
            if c.person.person_number == 1552329 and c.company.company_number == 10102:
                ctc = c

        for cl in cls:

            if cl.name in ['fmecl3', 'fmecl6']:
                self.assertEqual(len(cl.contacts), 2)
            else:
                self.assertEqual(len(cl.contacts), 1)

            if cl.name in ['fmecl2', 'fmecl3']:
                for go in cl.groups:
                    self.assertEqual(go.group_id, self.fedet_license.default_group.id)
                for c in cl.contacts:
                    has_feda_priv, has_fedet_priv = self._check_privs(c)
                    self.assertTrue(has_fedet_priv)
                    # only that one contact has both
                    if c.id == ctc.id: self.assertTrue(has_feda_priv)
                    else: self.assertFalse(has_feda_priv)
            else:
                for go in cl.groups:
                    self.assertEqual(go.group_id, self.feda_license.default_group.id)
                for c in cl.contacts:
                    has_feda_priv, has_fedet_priv = self._check_privs(c)
                    self.assertTrue(has_feda_priv)
                    # only that one contact has both
                    if c.id == ctc.id: self.assertTrue(has_fedet_priv)
                    else: self.assertFalse(has_fedet_priv)

        # same import won't change anything
        contacts = self.controller.import_contacts(unique_fields=self.unique_fields)

        companies = []
        persons = []
        distinct_contacts = []
        for c in contacts:
            if c.id not in distinct_contacts:
                distinct_contacts.append(c.id)
            if c.company.id not in companies:
                companies.append(c.company.id)
            if c.person.id not in persons:
                persons.append(c.person.id)
        self.assertEqual(len(distinct_contacts), 8)
        self.assertEqual(len(companies), 3)
        self.assertEqual(len(persons), 7)

        # change one person_number: we should get new person, same company and new contact
        # update third contact in the file
        updated_row = {'row_index': 2,
                       'row_values':{'company': {'company_number': '10102',
                                                 'name': 'Eaton Industries B.V.'
                                                 },
                                     'last_name': 'Meulenbroek new',
                                     'contact_list_name': 'FEDET: fmecl8',
                                     'person_number': '123'
                                     }}
        contacts = self.controller.import_contacts(updated_rows=[updated_row],unique_fields=self.unique_fields)
        distinct_contacts = []
        for c in contacts:
            if c.id not in distinct_contacts:
                distinct_contacts.append(c.id)
        self.assertEqual(len(distinct_contacts), 8)
        self.assertEqual(self.session.query(FMEContact).count(), 9)
        self.assertEqual(self.session.query(FMECompany).count(), 3)
        self.assertEqual(self.session.query(FMEPerson).count(), 8)

        # change contact list and license (use the contact created above so we don't break asserts): should find the contact
        # and link it to the new contact list and add privs for that contact for the cl license

        # contact that should be updated
        contact = None
        for c in contacts:
            if c.person.person_number == 123 and c.company.company_number == 10102:
                contact = c
                break

        updated_row = {'row_index': 2,
                       'row_values':{'company': {'company_number': '10102',
                                                 # also update the name of the company and last name
                                                 'name': 'Eaton Industries B.V. update'
                                                },
                                     'last_name': 'Meulenbroek update',
                                     'contact_list_name': 'FEDA: fmecl9', # other license
                                     'person_number': '123'
                                     }}
        # lets only import that row
        excluded_rows = [0,1,3,4,5,6,7,8]
        contacts = self.controller.import_contacts(updated_rows=[updated_row], excluded_rows=excluded_rows,
                                                   unique_fields=self.unique_fields)
        self.assertEqual(len(contacts), 1)
        c = contacts[0]
        self.assertEqual(c.id, contact.id)
        # company and last name should be updated
        self.assertEqual(c.person.last_name, 'Meulenbroek update')
        self.assertEqual(c.company.name, 'Eaton Industries B.V. update')
        # this contact didn't have privs on feda, but now it should have on both licenses
        has_feda_priv, has_fedet_priv = self._check_privs(c)
        self.assertTrue(has_feda_priv)
        self.assertTrue(has_fedet_priv)

        # it should now belong to 2 contact lists
        self.assertEqual(len(c.contact_lists), 2)
        cl_names = [cl.name for cl in c.contact_lists]
        assert 'fmecl8' in cl_names
        assert 'fmecl9' in cl_names

    def test_with_client_last_update(self):
        import datetime
        # See ticket #3355, apparently FME contacts behave differently than normal Contacts. Make sure it works here too.
        self.controller.field_mappings.append({"file_header": "LAST_UPDATE", "column": "client_last_update", "target": self.controller.COLUMN_IS_CONTACT})

        contacts = self.controller.import_contacts(unique_fields=self.unique_fields)
        for c in contacts:
            self.assertEqual(c.client_last_update, datetime.date(2011,01,29))

        # VERIFY dates

        self._contact1[5] = '2012-05-05' # should updte
        self._contact2[5] = '2010-10-10' # should errorrrr...

        f = open(self.filepointer, 'w')
        f.write("%s\n%s\n%s" % (
                    ','.join(self._file_headers),
                    ','.join(self._contact1),
                    ','.join(self._contact2),
        ))
        f.close()
        self.controller.set_file_lines()

        rv = self.controller.validate(None, None, self.unique_fields)
        self.assertEqual(len(rv), 1)

