# Copyright (c) 2012 BaseSoft B.V.
# See LICENSE for details.

from datetime import datetime

from sqlalchemy.sql import and_

from basesoft.web.contrib.account.models import Account
from basesoft.web.core.util.controllers import StateManager
from basesoft.web.contrib.compose.models import ComposeInvitee, ComposePublication

from iqportal.iqreport.core.schedule import ReportPublicationCacheTask,\
    PublishReportsTask, CreatePublicationInviteesTask
from iqportal.client.test.controllers import FmeTestController
from iqportal.iqube.core.controllers import ProjectController
from iqportal.iquest.report.schedule import AggregateResponsesTask
from iqportal.iquest.core.models import Survey
from iqportal.iqube.test.unit.test_configurable_widget_reporters import AbstractConfigurableWidgetReporterTestCase
from iqportal.iqreport.compose.controllers import ComposeController
from iqportal.iquest.report.models import ReportIQuestPublication, ReportIQuestPublicationXMLACache
from iqportal.iquest.report.controllers import IQuestReportPublicationStateController
from iqportal.iqreport.core.controllers import ReportPublicationStateController
from iqportal.iqube.core.models import DataSource

class ReportPublicationTestCase(AbstractConfigurableWidgetReporterTestCase):
    """
    This test is based on an IQuest datasource (FME)
    """
    DATASOURCE = 'IQuest'

    def setUp(self):
        """
        The script for reseting the db and creating all the tables
        """
        self.connect_db()

        account = self.session.query(Account).filter(Account.name=='sysadmin').one()
        # we populate necessary fme account data
        c = FmeTestController(self.session, account_id=account.id)
        c.populate_fme_domain()
        self._fmeTestController = c
        self.feda_license = c.feda_license
        self.fedet_license = c.fedet_license
        account = c.admin_account

        self.group = account.groups[0].group
        self.account = account
        self._attach_permissions_to_group(self.group,
            'create_individual_reports',
            'create_report_publication',
            'schedule_report_publication',
            'update_report_publication',
        )

        self.set_datasource()

        survey_id = c.populate_survey()

        self._contact_lists = [cl.id for cl in c.get_contact_lists(self.account)]

        pc = ProjectController(self.sa_session, account_id=self.account.id)
        for x in range(2):
            publication_id = c.publish_survey(survey_id, label=x, additional_state={'contact_lists': self._contact_lists}, anonymous=False)
            invitees = self._get_invitees(publication_id, 3)

            for i in invitees:  # 3 respondents
                c.populate_responses(publication_id, seed='bier'*i.id*x, invitee_hash=i.invitee_hash)

        # aggregator
        pc.flush_schema(self.datasource.id)
        AggregateResponsesTask().execute()
        self.survey = self.sa_session.query(Survey).filter(Survey.id==survey_id).one()

        # start django for iqube to connect for roles info
        self._start_django()


    def _get_invitees(self, publication_id, limit=None):
        invitees = self.sa_session.query(ComposeInvitee).filter(ComposeInvitee.publication_id==publication_id).all()

        if limit is None:
            return invitees
        return invitees[0:limit]

    def _get_default_qa_applied_config(self, anonymous=False):

        reporter, applied_config = self._get_configurable_widget_defaults('Default QA Item')

        qas = self.survey.itemQAs
        locale_id = self.survey.fallback_locale.id
        sid = self.survey.id
        pid = self.survey.publications[0].id
        first = qas[0]

        aid = first.answers[0].id

        dimensions = {
            'Question': ['[Question].[%s].[%s].[%s]' % (first.id, locale_id, q.id) for q in first.questions],
            'Survey': ['[Survey].[%s].[%s]' % (sid, pid)],
            'Answer': ['[Answer].[%s].[%s].[%s]' % (first.id, locale_id, aid)],
            'Measures': ["[Measures].[Responses Int Sum]", "[Measures].[Responses Float Sum]"]
        }

        applied_config['show_response_measure'] = True
        applied_config['axes_0'] = 'slicers'
        applied_config['axes_1'] = 'rows'
        applied_config['axes_2'] = 'columns'
        applied_config['axes_3'] = 'slicers'
        applied_config['column_order'] = ['Answer']
        applied_config['row_order'] = ['Question']
        applied_config['measure_order'] = ['response']

        return reporter.config['id'], self._smorgle_applied_config(applied_config, dimensions, 'VirtualResponse'), reporter

    def _get_growth_applied_config(self, anonymous=False):
        reporter, applied_config = self._get_configurable_widget_defaults('Growth Item')

        qas = self.survey.itemQAs
        locale_id = self.survey.fallback_locale.id
        sid = self.survey.id
        first = qas[0]

        aid = first.answers[0].id

        dimensions = {
            'Survey': ['[Survey].[%s].[%s]' % (sid, p.id) for p in self.survey.publications],
            'Question': ['[Question].[%s].[%s].[%s]' % (first.id, locale_id, q.id) for q in first.questions],
            'Answer': ['[Answer].[%s].[%s].[%s]' % (first.id, locale_id, aid)],
            'Measures': ["[Measures].[Responses Int Sum]", "[Measures].[Responses Float Sum]"]
        }

        applied_config['show_response_measure'] = True
        applied_config['show_growth_measure'] = True
        if anonymous:
            applied_config['show_individual_measure'] = False
        else:
            applied_config['show_individual_measure'] = True
        applied_config['axes_0'] = 'rows'
        applied_config['axes_1'] = 'columns'
        applied_config['axes_2'] = 'columns'
        applied_config['axes_3'] = 'columns'
        applied_config['column_order'] = ['Question', 'Answer', 'Measures']
        applied_config['row_order'] = ['Survey']
        applied_config['measure_order'] = ['response', 'growth', 'individual']

        return reporter.config['id'], self._smorgle_applied_config(applied_config, dimensions, 'VirtualResponse'), reporter

    def _get_result_item_state(self):
        # use the (applied) config from one of the ConfigurableWidgets.
        bitbucket, applied_config, r =  self._get_default_qa_applied_config(True)
        config = r.config

        return {
            "objectName": u'ReportItemResult',
            'dbID': None,
            'domID': 'result-item1',
            'title': u'test',
            'showTitle': True,
            'config': config,
            'appliedConfig': applied_config,
            'tableConfig': None,
            'graphConfig': None
        }

    def _get_report_state(self, anonymous=False, include_result_item=False):
        item1_id, item1_ac, r =  self._get_default_qa_applied_config(anonymous)
        item2_id, item2_ac, r = self._get_growth_applied_config(anonymous)

        items = [{
            'objectName': u'ReportItemDataSourceWidget',
            'loaderName': 'configurable_report_item',
            'requires': [u'level_filter_fancy_select', u'iq_fancyselect'],
            'datasource_widget_id': item1_id,
            'title': u'1. Handel (in-/verkoop zonder toegevoegde waarde)',
            'dbID': None,
            'domID': 'i1',
            'tableConfig': {u'slicersExpanded': False, u'isTransposed': False},
            'appliedConfig': item1_ac,
            'config': {},
            'graphConfig': {},
            'showTitle': 1
        }, {
            'objectName': u'ReportItemDataSourceWidget',
            'loaderName': 'configurable_report_item',
            'requires': [u'level_filter_fancy_select', u'iq_fancyselect'],
            'datasource_widget_id': item2_id,
            'title': u'Click to change title',
            'dbID': None,
            'domID': 'i2',
            'tableConfig': {u'slicersExpanded': False, u'isTransposed': False},
            'appliedConfig': item2_ac,
            'config': {},
            'graphConfig': {},
            'showTitle': 1
        }]

        # if we are to include the result item in the cache, we replace the default QA one, as they are the same...easier to validate this way.
        if include_result_item:
            items[0] = self._get_result_item_state()

        state = {
            'report': {
                'objectName': 'Report',
                'name': 'test',
                'fallbackLocale': {'locale': 'en_GB', 'name': 'English'},
                'dbID': None,
                'domID': 'r1',
                'availableLocales': [],
                'theme': {'cssClass': u'iqreport-basesoft', 'name': u'BaseSoft', 'module': u'iqreport_canvas_theme_basesoft', 'graph_config': None},
                'has_active_publications': False,
                "locales": {
                    'en_GB': {
                        u'label': u'Test report'
                    }
                },
                'children': {
                    'blocks': [{
                        'dbID': None,
                        'domID': 'b1',
                        'children': {
                            'columns': [{
                                'dbID': None,
                                'domID': 'c1',
                                'children': {
                                    'items': items
                                },
                                'objectName': u'ReportBlockColumn',
                                'width': None
                            }]
                        },
                        'objectName': u'ReportBlock',
                        'pageEnd': False
                    }]
                }
            }
        }
        return state

    def _publish_report(self, label, anonymous=True, result_item=False):

        state = self._get_report_state(anonymous=anonymous,include_result_item=result_item)

        # we create a simple IQuest based report.
        cc = ComposeController(self.sa_session, self.survey.fallback_locale, account_id=self.account.id)
        map = cc.save_state(state)
        report_id = map['r1']

        publish_state = {
            'report_id': report_id,
            "dbID": None,
            "domID": u'publish_1',
            "begin_date": datetime.now(),
            "end_date": None,
            "status": ReportIQuestPublication.STATUS_SCHEDULED,
            "label": u"publication %s" % label,
            "show_page_nrs": True,
            "anonymous": anonymous,
            "contact_lists": [],
            "mailings": [], # no mailings necessary,
            'viewer_print': False,
            'survey_publication_id': self.survey.publications[-1].id
        }
        if not anonymous:
            publish_state['contact_lists'] = self._contact_lists

        # save publication
        publication_controller = IQuestReportPublicationStateController(self.sa_session, StateManager(self.survey.fallback_locale, self.account.id, None))
        publication = publication_controller.save(publish_state)

        # publish the report like the scheduler would do
        PublishReportsTask().execute()
        CreatePublicationInviteesTask().execute()

        # reload publication for invitees:
        return self.sa_session.query(ComposePublication).filter(ComposePublication.id==publication.id).one()

    def test_anonymous_cache(self):
        """
        Simple anonymous test case (anonymous=global cache only)
        """
        publication = self._publish_report('johnny')
        ReportPublicationCacheTask().execute()
        self.sa_session.flush()
        self.sa_session.expire(publication)

        # now that we have the cache generated, lets verify there are two entries for this publication.
        r = self.sa_session.query(ReportIQuestPublicationXMLACache) \
            .filter(ReportIQuestPublicationXMLACache.report_publication==publication) \
            .order_by(ReportIQuestPublicationXMLACache.id.desc()) \
            .all()
        self.assertEqual(len(r), 2)
        cache_id = r[0].id
        cache_xmla = r[0].xmla_response

        # insert new responses
        invitees = self._get_invitees(publication.survey_publication.id, 1)
        for i in invitees:  # 1 respondents
            self._fmeTestController.populate_responses(publication.survey_publication.id, seed='bier'*i.id*123, invitee_hash=i.invitee_hash)

        # rerun aggregator and reset mondrian cache
        # FIXME: check why test fails if mondrian cache is on
        AggregateResponsesTask().execute()
        from django.conf import settings
        ds = self.session.query(DataSource).filter(DataSource.name=='%s.IQuest' % settings.SETTINGS_MODULE).first()
        pc = ProjectController(self.sa_session)
        pc.clear_cache(ds.id)

        # reset cache for this publication
        pc = ReportPublicationStateController(self.sa_session, StateManager(None, self.account.id, None))
        recache_ids = pc.reset_cache(publication.id)
        self.assertEquals(len(recache_ids), 1)
        self.assertEquals(recache_ids[0], publication.id)
        ReportPublicationCacheTask().execute()
        self.sa_session.expire(publication)

        # cache should be different because of new responses
        r = self.sa_session.query(ReportIQuestPublicationXMLACache).filter(ReportIQuestPublicationXMLACache.id == cache_id).one()
        self.assertNotEqual(cache_xmla, r.xmla_response)

    def test_individual_cache(self):
        """
        Individual cache test case
        """
        publication = self._publish_report('johnny-indiv', anonymous=False)

        ReportPublicationCacheTask().execute()

        # we should have 4 cache entries (1 for the first non-individual widget + 3 individuals for the last)
        r = self.sa_session.query(ReportIQuestPublicationXMLACache).filter(ReportIQuestPublicationXMLACache.report_publication==publication).all()
        self.assertEqual(len(r), 4)

        r = self.sa_session.query(ReportIQuestPublicationXMLACache).filter(and_(
            ReportIQuestPublicationXMLACache.report_publication==publication,
            ReportIQuestPublicationXMLACache.company!=None
        )).all()
        self.assertEqual(len(r), 3)

