# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.

from django.conf import settings

from basesoft.web.test.selenium.selenium_testcase import SeleniumTestCase

from iqportal.iqube.core.models import MdxMethod
from iqportal.iqube.compose.controllers import MdxCalculatedMemberController
from twisted.trial.unittest import SkipTest

class TestCase(SeleniumTestCase):
    def setUp(self):
        super(TestCase, self).setUp()
        self.connect_db()

        self.account = self._create_active_account()
        self.group = self.account.default_group
        self._attach_permissions_to_group(self.group, 'use_raw_mdx', 'view_dashboard', 'view_calculated_member', 'update_calculated_member', 'view_datasource', 'create_calculated_member')

        # we create a controller to grant privs
        self.controller = MdxCalculatedMemberController(self.sa_session, account_id=self.account.id)

    def test_named_set(self):

        # we want topcount
        top_count = self.sa_session.query(MdxMethod).filter(MdxMethod.name == 'TopCount').first()
        self.controller.grant_privilege_to(top_count, MdxMethod.PRIV_VIEW, group=True, target_group=self.group, check_own_privs=False)
        # and we want crossjoin
        crossjoin = self.sa_session.query(MdxMethod).filter(MdxMethod.name == 'Crossjoin').first()
        self.controller.grant_privilege_to(crossjoin, MdxMethod.PRIV_VIEW, group=True, target_group=self.group, check_own_privs=False)

        # we login
        self.login(self.account.name, 'test_password')
        self.wait_for_loading()

        # open the compose
        self.click_iconbar('mdx-calculated-member-designer')
        self.wait_for_loading()

        self.click_palette('Set functions', 2)

        # drag top count
        self.drag_item_from_palette_by_offset('Top count', xoffset=400, yoffset=300)
        self.wait_for_loading()

        # drag the crossjoin so we can connect it to the top count
        self.drag_item_from_palette_by_offset('Crossjoin', xoffset=200, yoffset=100)
        self.wait_for_loading()

        # connect the 5th terminal created (4 from the Top Count + 1 for crossjoin source)
        # to the 2nd terminal (source from Top Count + 1)
        self.connect_jsplumb_terminals(5,2)

        self.click_palette('Save', 0)
        self.save_dialog_input('Wicked calculation')
        self.wait_for_loading()

        mcmc = MdxCalculatedMemberController(self.sa_session, account_id=self.account.id)
        cm = mcmc.get_calculated_member().pop()
        self.assertEqual(mcmc.get_parsed(cm.id), 'TopCount(Crossjoin($var,$var),$var,$var)')

    def test_time_member(self):
        if not hasattr(settings, 'DB_CONNSTR_FOODMART'):
            raise SkipTest('No Foodmart datasource available')

        # we want Year To Date
        ytd = self.sa_session.query(MdxMethod).filter(MdxMethod.name == 'YTD').first()
        self.controller.grant_privilege_to(ytd, MdxMethod.PRIV_VIEW, group=True, target_group=self.group, check_own_privs=False)

        # we login
        self.login(self.account.name, 'test_password')
        self.wait_for_loading()

        # open the compose
        self.click_iconbar('mdx-calculated-member-designer')
        self.wait_for_loading()

        self.click_palette('Time functions', 2)

        # drag year to date
        self.drag_item_from_palette_by_offset('Year to date', xoffset=500, yoffset=400)
        self.wait_for_loading()

        self.click_palette('Datasource items', 2)

        # drag datasource item
        self.drag_item_from_palette_by_offset('Member expression (time)', xoffset=300, yoffset=200)
        self.wait_for_loading()

        self.click_button(panel_name='dialog-question', text='Yes')
        self.wait_for_loading()

        values = {'datasource_id': ('select', '%s.FoodMart' % settings.SETTINGS_MODULE)}
        self.fill_form(panel_name='iqube-datasource-cube-panel', values=values)
        self.click_button(panel_name='iqube-datasource-cube-panel', text='Continue')
        self.wait_for_loading()

        self.select_compose_module_floating(item_index=2)

        #configure the member expression
        self.click_palette('Configure', 1)
        self.wait_for_loading()

        self.fill_form(panel_name='iqube-mdx-expression-selection', values={'dimension': ('select', 'Time')})
        self.wait_for_loading()

        self.fill_form(panel_name='iqube-mdx-expression-selection', values={'selection': ('fancy-select', ['1997'])})
        self.click_button(panel_name='iqube-mdx-expression-selection', text='Use selection')
        self.wait_for_loading()

        # connect the 3trd terminal created (2 from the Year To Date + 1 for time member expression source)
        # to the 2nd terminal (source from Year To Date + 1)
        self.connect_jsplumb_terminals(3,2)

        #save
        self.click_palette('Save', 0)
        self.wait_for_loading()
        self.save_dialog_input('Time calculation')
        self.wait_for_loading()

        mcmc = MdxCalculatedMemberController(self.sa_session, account_id=self.account.id)
        cm = mcmc.get_calculated_member().pop()
        self.assertEqual(mcmc.get_parsed(cm.id), 'YTD([Time.Weekly].[1997])')

    def test_nested_calculated_member_end_point(self):
        # we want topcount
        top_count = self.sa_session.query(MdxMethod).filter(MdxMethod.name == 'TopCount').first()
        self.controller.grant_privilege_to(top_count, MdxMethod.PRIV_VIEW, group=True, target_group=self.group, check_own_privs=False)

        state = {
            'objectName': 'MdxMethodExceptItem',
            'loaderName': 'iqube_mdx_method_except_item',
            'return_type': 'set',
            'method_id': top_count.id,
            'inputs': [None, None, None],
        }

        cm = self.controller.save_or_update_calculated_member(None, 'My Named Set', state)

        # we login
        self.login(self.account.name, 'test_password')
        self.wait_for_loading()

        # open the compose
        self.click_iconbar('mdx-calculated-member-designer')
        self.wait_for_loading()

        self.click_palette('Miscellaneous', 2)

        # drag top count
        self.drag_item_from_palette_by_offset('Calculated member', xoffset=400, yoffset=300)
        self.wait_for_loading()

        # select
        self.fill_form(panel_name='iqube-mdx-cm-selection', values={
            'calculated_member_id': ('select', 'My Named Set')
        })
        self.click_button(panel_name='iqube-mdx-cm-selection', text='Use calculation')
        self.wait_for_loading()

        self.click_palette('Save', 0)
        self.save_dialog_input('Wicked calculation')
        self.wait_for_loading()

        cm = self.controller.get_calculated_member().pop()
        self.assertEqual(self.controller.get_parsed(cm.id), 'TopCount($var,$var,$var)')

    def test_nested_calculated_member(self):
        # we want topcount
        top_count = self.sa_session.query(MdxMethod).filter(MdxMethod.name == 'TopCount').first()
        self.controller.grant_privilege_to(top_count, MdxMethod.PRIV_VIEW, group=True, target_group=self.group, check_own_privs=False)
        # and we want crossjoin
        crossjoin = self.sa_session.query(MdxMethod).filter(MdxMethod.name == 'Crossjoin').first()
        self.controller.grant_privilege_to(crossjoin, MdxMethod.PRIV_VIEW, group=True, target_group=self.group, check_own_privs=False)

        state = {
            'objectName': 'MdxMethodExceptItem',
            'loaderName': 'iqube_mdx_method_except_item',
            'return_type': 'set',
            'method_id': top_count.id,
            'inputs': [None, None, None],
        }

        self.controller.save_or_update_calculated_member(None, 'My Named Set', state)

        # we login
        self.login(self.account.name, 'test_password')
        self.wait_for_loading()

        # open the compose
        self.click_iconbar('mdx-calculated-member-designer')
        self.wait_for_loading()

        self.click_palette('Miscellaneous', 2)

        # drag top count
        self.drag_item_from_palette_by_offset('Calculated member', xoffset=400, yoffset=300)
        self.wait_for_loading()

        # select
        self.fill_form(panel_name='iqube-mdx-cm-selection', values={
            'calculated_member_id': ('select', 'My Named Set')
        })
        self.click_button(panel_name='iqube-mdx-cm-selection', text='Use calculation')
        self.wait_for_loading()

        self.click_palette('Set functions', 2)

        # drag the crossjoin so we can connect it to the top count
        self.drag_item_from_palette_by_offset('Crossjoin', xoffset=200, yoffset=100)
        self.wait_for_loading()

        # connect the 5th terminal created (4 from the Top Count + 1 for crossjoin source)
        # to the 2nd terminal (source from Top Count + 1)
        self.connect_jsplumb_terminals(5,2)

        self.click_palette('Save', 0)
        self.save_dialog_input('Wicked calculation')
        self.wait_for_loading()

        cm = self.controller.get_calculated_member().pop()
        self.assertEqual(self.controller.get_parsed(cm.id), 'TopCount(Crossjoin($var,$var),$var,$var)')

    def test_iif_expression_set(self):
        # we want iif
        top_count = self.sa_session.query(MdxMethod).filter(MdxMethod.name == 'Iif').first()
        self.controller.grant_privilege_to(top_count, MdxMethod.PRIV_VIEW, group=True, target_group=self.group, check_own_privs=False)
        # and we want crossjoin
        crossjoin = self.sa_session.query(MdxMethod).filter(MdxMethod.name == 'Crossjoin').first()
        self.controller.grant_privilege_to(crossjoin, MdxMethod.PRIV_VIEW, group=True, target_group=self.group, check_own_privs=False)

        # we login
        self.login(self.account.name, 'test_password')
        self.wait_for_loading()

        # open the compose
        self.click_iconbar('mdx-calculated-member-designer')
        self.wait_for_loading()

        self.click_palette('Generic functions', 2)

        # drag top count
        self.drag_item_from_palette_by_offset('Iif', xoffset=400, yoffset=300)
        self.wait_for_loading()

        # select
        self.fill_form(panel_name='iqube-mdx-type-selection', values={
            'return_type': ('select', 'Set')
        })
        self.click_button(panel_name='iqube-mdx-type-selection', text='Use return type')
        self.wait_for_loading()

        self.click_palette('Set functions', 2)

        # drag the crossjoin so we can connect it to the top count
        self.drag_item_from_palette_by_offset('Crossjoin', xoffset=200, yoffset=100)
        self.wait_for_loading()

        # connect the 5th terminal created (4 from the Top Count + 1 for crossjoin source)
        # to the 3nd terminal (source from Iif + 1)
        self.connect_jsplumb_terminals(5,3)

        self.click_palette('Save', 0)
        self.save_dialog_input('Wicked calculation')
        self.wait_for_loading()

        mcmc = MdxCalculatedMemberController(self.sa_session, account_id=self.account.id)
        cm = mcmc.get_calculated_member().pop()
        self.assertEqual(mcmc.get_parsed(cm.id), 'Iif($var,Crossjoin($var,$var),$var)')

    def test_iif_expression_member(self):

        if not hasattr(settings, 'DB_CONNSTR_FOODMART'):
            raise SkipTest('No Foodmart datasource available')

        # we want iif
        top_count = self.sa_session.query(MdxMethod).filter(MdxMethod.name == 'Iif').first()
        self.controller.grant_privilege_to(top_count, MdxMethod.PRIV_VIEW, group=True, target_group=self.group, check_own_privs=False)

        # we login
        self.login(self.account.name, 'test_password')
        self.wait_for_loading()

        # open the compose
        self.click_iconbar('mdx-calculated-member-designer')
        self.wait_for_loading()

        self.click_palette('Generic functions', 2)

        # drag top count
        self.drag_item_from_palette_by_offset('Iif', xoffset=400, yoffset=300)
        self.wait_for_loading()

        # select
        self.fill_form(panel_name='iqube-mdx-type-selection', values={
            'return_type': ('select', 'Member')
        })
        self.click_button(panel_name='iqube-mdx-type-selection', text='Use return type')
        self.wait_for_loading()

        self.click_palette('Datasource items', 2)

        # drag datasource item
        self.drag_item_from_palette_by_offset('Member expression', xoffset=300, yoffset=200)
        self.wait_for_loading()

        self.click_button(panel_name='dialog-question', text='Yes')
        self.wait_for_loading()

        values = {'datasource_id': ('select', '%s.FoodMart' % settings.SETTINGS_MODULE)}
        self.fill_form(panel_name='iqube-datasource-cube-panel', values=values)
        self.click_button(panel_name='iqube-datasource-cube-panel', text='Continue')
        self.wait_for_loading()

        self.select_compose_module_floating(item_index=2)

        #configure the member expression
        self.click_palette('Configure', 1)
        self.wait_for_loading(stable_time=2)

        self.fill_form(panel_name='iqube-mdx-expression-selection', values={'dimension': ('select', 'Product')})
        self.wait_for_loading()

        self.fill_form(panel_name='iqube-mdx-expression-selection', values={'selection': ('fancy-select', ['Food'])})
        self.click_button(panel_name='iqube-mdx-expression-selection', text='Use selection')
        self.wait_for_loading()

        # connect the 5th terminal created (4 from the Top Count + 1 for crossjoin source)
        # to the 3nd terminal (source from Iif + 1)
        self.connect_jsplumb_terminals(5,3)

        self.click_palette('Save', 0)
        self.wait_for_loading()
        self.save_dialog_input('Wicked calculation')
        self.wait_for_loading()

        mcmc = MdxCalculatedMemberController(self.sa_session, account_id=self.account.id)
        cm = mcmc.get_calculated_member().pop()
        self.assertEqual(mcmc.get_parsed(cm.id), 'Iif($var,[Product].[Food],$var)')

    def test_format_string_presets(self):
        self._attach_permissions_to_group(self.group, 'manage_format_strings')

        self.login(self.account.name, 'test_password')
        self.wait_for_loading()

        # open the compose
        self.click_iconbar('mdx-format-string-manager', 'mdx-calculated-member-designer')
        self.wait_for_loading()

        self.click_menu(panel_name='mdx-format-string-manager-panel', text='New format string')
        self.wait_for_loading()

        self.click_fieldset(panel_name='mdx-format-string-designer', title='Advanced settings')
        self.fill_form(panel_name='mdx-format-string-designer', values={
            'pre_character': '$',
            'number_of_decimals': 2,
            'operator': ('select', 'Smaller than'),
            'compare_value': 1000,
            'left_text_style': ('select', 'Bold'),
        })

        self.click_button(panel_name='mdx-format-string-designer', text='Save as preset')
        self.wait_for_loading()

        self.save_dialog_input(input_text='My first format string')
        self.wait_for_loading()

        self.assert_cq(panel_name='mdx-format-string-manager-panel', row_index=0, data={
            'name': 'My first format string',
            'format_string': "IIF({value}<1000, '|$#,##0.00|font-weight: bold', '$#,##0.00')"
        })

        self.click_cq(panel_name='mdx-format-string-manager-panel', row_index=0, col_name='name')
        self.wait_for_loading()

        self.click_fieldset(panel_name='mdx-format-string-designer', title='Advanced settings')
        self.assert_form(panel_name='mdx-format-string-designer', values={
            'pre_character': '$',
            'number_of_decimals': '2',
            'operator': ('select', 'Smaller than'),
            'compare_value': '1000',
            'left_text_style': ('select', 'Bold'),
        })

        self.fill_form(panel_name='mdx-format-string-designer', values={
            'number_of_decimals': 3,
            'right_text_style': ('select', 'Italic'),
        })

        self.click_button(panel_name='mdx-format-string-designer', text='Save as preset')
        self.wait_for_loading()

        self.click_button(panel_name='mdx-format-string-preset-update-request-panel', text='Update preset')
        self.save_dialog_input(input_text='My first updated format string')
        self.wait_for_loading()

        self.assert_cq(panel_name='mdx-format-string-manager-panel', row_index=0, data={
            'name': 'My first updated format string',
            'format_string': "IIF({value}<1000, '|$#,##0.000|font-weight: bold', '|$#,##0.000|font-style: italic')"
        })

    def test_repeaters(self):
        # we want iif
        iif = self.sa_session.query(MdxMethod).filter(MdxMethod.name == 'Iif').first()
        self.controller.grant_privilege_to(iif, MdxMethod.PRIV_VIEW, group=True, target_group=self.group, check_own_privs=False)

        self.login(self.account.name, 'test_password')
        self.wait_for_loading()

        # open the compose
        self.click_iconbar('mdx-calculated-member-designer')
        self.wait_for_loading()

        self.click_palette('Miscellaneous', 2)

        # drag top count
        self.drag_item_from_palette_by_offset('Repeater', xoffset=400, yoffset=300)
        self.wait_for_loading()

        # select
        self.fill_form(panel_name='iqube-mdx-type-selection', values={
            'return_type': ('select', 'Numeric')
        })
        self.click_button(panel_name='iqube-mdx-type-selection', text='Use return type')
        self.wait_for_loading()

        #drag Iif numeric
        self.click_palette('Generic functions', 2)

        self.drag_item_from_palette_by_offset('Iif', xoffset=500, yoffset=500)
        self.wait_for_loading()

        # select
        self.fill_form(panel_name='iqube-mdx-type-selection', values={
            'return_type': ('select', 'Numeric')
        })
        self.click_button(panel_name='iqube-mdx-type-selection', text='Use return type')
        self.wait_for_loading()
        #connect
        self.connect_jsplumb_terminals(1,5)
        self.connect_jsplumb_terminals(1,6)
        #save
        self.click_palette('Save', 0)
        self.save_dialog_input('Repeater test')
        self.wait_for_loading()
        #load state
        cm = self.controller.get_calculated_member().pop()
        cm_state = self.controller.load_state(cm.id)
        #refresh page
        self.refresh_page()
        self.wait_for_loading()
        #save and load again to assert that the state is the same
        self.click_palette('Save', 0)
        self.wait_for_loading()
        cm2 = self.controller.get_calculated_member().pop()
        cm_state1 = self.controller.load_state(cm2.id)

        self.assertDeepEqual(cm_state, cm_state1)
        self.assertDeepEqual(cm_state['children']['inputs'], cm_state1['children']['inputs'])
