# -*- coding: utf-8 -*-

# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.
from django.conf import settings
from django.utils import simplejson

from basesoft.web.test.selenium.selenium_testcase import SeleniumTestCase
from basesoft.web.contrib.dashboard.models import Dashboard
from iqportal.iqube.core.models import MdxMethod
from iqportal.iqube.compose.controllers import MdxCalculatedMemberController
from iqportal.iqreport.core.models import Report

class TestCase(SeleniumTestCase):
    def setUp(self):
        super(TestCase, self).setUp()
        self.connect_db()

    def _select_dimension(self, dimension_name, dimension_index, members, axis='Rows'):
        # make some selection.
        self.click_fieldset(panel_name='config-widget-panel', title=dimension_name)
        # wait for the members to be loaded
        self.wait_for_loading()

        # select a few options
        self.fill_form(panel_name='config-widget-panel', values={
            'dimension_%d' % dimension_index: ('fancy-select', members),
            'axes_%d' % dimension_index: ('select', axis)
        })

    def _add_result_item(self):
        # drag Result widget to dashboard
        self.drag_item_from_palette('Results', block_index=0, column_index=0)
        self.wait_for_loading()

        # select datasource and cube
        self.fill_form(panel_name='iqube-datasource-cube-panel', values={
            'datasource_id': ('select', '%s.FoodMart' % settings.SETTINGS_MODULE)
        })
        self.wait_for_loading()

        self.fill_form(panel_name='iqube-datasource-cube-panel', values={
            'cube': ('select', 'Sales')
        })
        self.click_button(panel_name='iqube-datasource-cube-panel', text='Continue')
        self.wait_for_loading()

    def test_link_mode(self):

        # create account with permissions
        account = self._create_active_account()
        group = account.default_group
        self._attach_permissions_to_group(group, 'view_dashboard', 'save_dashboard', 'pivot_table', 'view_datasource', 'link_result_widgets', 'item_result')

        # open dashboard
        self.login(account.name, 'test_password')
        self.wait_for_loading()

        self.expand_layout_pane(0)

        # set mode to compose
        self.click_palette('Mode')
        self.click_palette('Compose')
        self.wait_for_loading()

        self.click_palette('IQReport', palette_index=2)

        self._add_result_item()
        self._select_dimension('Education Level', 8, ('Education Level (Members)', ))

        self.click_button(panel_name='config-widget-panel', text='Apply and close')
        self.wait_for_loading()

        # add second widget
        self._add_result_item()
        self._select_dimension('Promotion Media', 5, ('Promotion Media (Members)', ))

        self.click_button(panel_name='config-widget-panel', text='Apply and close')
        self.wait_for_loading()

        # switch to MDXLink mode
        self.click_palette('Link')
        self.wait_for_loading()

        # connect Widget A to Widget B (Slicers)
        self.connect_jsplumb_terminals(1, 8)

        # switch to Compose mode
        self.click_palette('Compose')
        self.wait_for_loading()

        db_name = self._uniquestring('db')

        # save the dashboard
        self.click_palette('Save the dashboard')
        self.fill_form(panel_name='dashboard-settings-panel', values={
            'name': db_name
        })
        self.click_button(panel_name='dashboard-settings-panel', text='OK')
        self.wait_for_loading()

        # query the linked_items column and assert the connection.
        db = self.sa_session.query(Dashboard).filter(Dashboard.name==db_name).first()
        widgets = db.tabs[0].blocks[0].columns[0].widgets
        links = simplejson.loads(widgets[0].linked_items)
        self.assertEqual({str(widgets[1].id): 3}, links)

    def test_link_mode_iqreport(self):

        # create account with permissions
        account = self._create_active_account()
        group = account.default_group
        self._attach_permissions_to_group(group, 'view_dashboard', 'view_report', 'create_report', 'pivot_table', 'view_datasource', 'link_result_widgets', 'item_result')

        # open dashboard
        self.login(account.name, 'test_password')
        self.wait_for_loading()

        self.click_iconbar('icon-iqreport')
        self.wait_for_loading()

        self.expand_layout_pane(0)

        self._add_result_item()
        self._select_dimension('Education Level', 8, ('Education Level (Members)', ))

        self.click_button(panel_name='config-widget-panel', text='Apply and close')
        self.wait_for_loading()

        # add second widget
        self._add_result_item()
        self._select_dimension('Promotion Media', 5, ('Promotion Media (Members)', ))

        self.click_button(panel_name='config-widget-panel', text='Apply and close')
        self.wait_for_loading()

        # click tab to switch to MDXLink mode
        self.click_tab('Link mode')
        self.wait_for_loading()

        # connect Widget A to Widget B (Slicers)
        self.connect_jsplumb_terminals(1, 8)

        # switch to Compose mode
        self.click_tab('Compose')
        self.wait_for_loading()

        #import pdb;pdb.set_trace()

        db_name = self._uniquestring('db')

        # save the dashboard
        self.click_palette('Save', palette_index=0)
        self.wait_for_loading()

        # save form with a name and click 'ok' button
        self.save_dialog_input(db_name)
        self.wait_for_loading()

        # query the linked_items column and assert the connection.
        r = self.sa_session.query(Report).filter(Report.name==db_name).first()
        widgets = r.blocks[0].columns[0].items
        links = simplejson.loads(widgets[0].linked_items)
        self.assertEqual({str(widgets[1].id): 3}, links)

    def test_mdx_method_wizard_with_logical_and_string_expression(self):
        # create account with permissions
        account = self._create_active_account()
        group = account.default_group
        self._attach_permissions_to_group(group, 'view_dashboard', 'save_dashboard', 'pivot_table', 'view_datasource',
                                          'item_result','view_calculated_member', 'create_calculated_member', 'update_calculated_member')

        controller = MdxCalculatedMemberController(self.sa_session, account_id=account.id)
        m = self.sa_session.query(MdxMethod).filter(MdxMethod.name=='Iif').first()
        controller.grant_privilege_to(m, MdxMethod.PRIV_VIEW, group=True, target_group=group, check_own_privs=False)

        state = {
            'objectName': 'MdxMethodIifItem',
            'loaderName': 'iqube_mdx_method_iif_item',
            'return_type': 'string',
            'method_id': m.id,
            'inputs': [None, None, {
                'objectName': 'MdxStringInputItem',
                'loaderName': 'iqube_mdx_string_input_item',
                'return_type': 'string',
                'value': 'Failure!'
            }]
        }
        cm = controller.save_or_update_calculated_member(None, 'Use me', state)
        cm = controller.get_calculated_member(cm.id).pop()

        # open dashboard
        self.login(account.name, 'test_password')
        self.wait_for_loading()

        self.expand_layout_pane(0)

        # set mode to compose
        self.click_palette('Mode')
        self.click_palette('Compose')
        self.wait_for_loading()

        self.click_palette('IQReport', palette_index=2)

        self._add_result_item()
        self.click_fieldset(panel_name='config-widget-panel', title='Widget settings')
        self.click_fieldset(panel_name='config-widget-panel', title='Table settings')
        self._select_dimension('Measures', 12, ('Use me', ), axis='Columns')

        self.click_button_by_id('cm-configure-12')
        self.wait_for_loading()

        self.click_button(panel_name='mdx-formula-wizard', text=u'Next »')
        self.wait_for_loading()

        self.fill_form(panel_name='mdx-formula-wizard', form_name='logical_expression_page_0', values={
            'selection': ('checkbox2', True)
        })
        self.click_button(panel_name='mdx-formula-wizard', text=u'Next »')
        self.wait_for_loading()

        self.fill_form(panel_name='mdx-formula-wizard', form_name='string_expression_page_1', values={
            'selection': 'Success!'
        })
        self.click_button(panel_name='mdx-formula-wizard', text=u'Finish')
        self.wait_for_loading()

        self.click_button(panel_name='config-widget-panel', text='Apply and close')
        self.wait_for_loading()

        self.assert_cube_table(1, value=['266,773', 'Success!'], css_selector='table.basesoft-cubetable', has_row_headers=False)

    def test_mdx_parallel_period_method(self):
        # create account with permissions
        account = self._create_active_account()
        group = account.default_group
        self._attach_permissions_to_group(group, 'view_dashboard', 'save_dashboard', 'pivot_table', 'view_datasource',
                                          'item_result','view_calculated_member', 'create_calculated_member', 'update_calculated_member')

        controller = MdxCalculatedMemberController(self.sa_session, account_id=account.id)
        m = self.sa_session.query(MdxMethod).filter(MdxMethod.name=='ParallelPeriod').first()
        controller.grant_privilege_to(m, MdxMethod.PRIV_VIEW, group=True, target_group=group, check_own_privs=False)

        state = {
            'objectName': 'MdxMethodItem',
            'loaderName': 'iqube_mdx_method_item',
            'return_type': 'string',
            'method_id': m.id,
            'inputs': [None, {
                'objectName': 'MdxNumericalInputItem',
                'loaderName': 'iqube_mdx_count_input_item',
                'return_type': 'count',
                'value': 3
            }, None]
        }
        cm = controller.save_or_update_calculated_member(None, 'Use me', state)
        cm = controller.get_calculated_member(cm.id).pop()

        # open dashboard
        self.login(account.name, 'test_password')
        self.wait_for_loading()

        self.expand_layout_pane(0)

        # set mode to compose
        self.click_palette('Mode')
        self.click_palette('Compose')
        self.wait_for_loading()

        self.click_palette('IQReport', palette_index=2)

        self._add_result_item()
        self.click_fieldset(panel_name='config-widget-panel', title='Widget settings')
        self.click_fieldset(panel_name='config-widget-panel', title='Table settings')
        self._select_dimension('Measures', 12, ('Use me', ), axis='Columns')

        self.click_button_by_id('cm-configure-12')
        self.wait_for_loading()

        self.fill_form(panel_name='mdx-formula-wizard', form_name='datasource_page', values={
            'label': 'ParallelPeriodCM'
        })
        self.click_button(panel_name='mdx-formula-wizard', text=u'Next »')
        self.wait_for_loading()

        self.fill_form(panel_name='mdx-formula-wizard', form_name='level_expression_page_0', values={
            'dimension': ('select', 'Time')
        })

        self.fill_form(panel_name='mdx-formula-wizard', form_name='level_expression_page_0', values={
            'selection': ('select', 'Week')
        })
        self.click_button(panel_name='mdx-formula-wizard', text=u'Next »')
        self.wait_for_loading()

        self.fill_form(panel_name='mdx-formula-wizard', form_name='member_expression_page_1', values={
            'dimension': ('select', 'Time')
        })
        self.wait_for_loading()

        self.fill_form(panel_name='mdx-formula-wizard', form_name='member_expression_page_1', values={
            'selection': ('fancy-select', ['1997'])
        })

        #click Filter selection
        self.click_button(panel_name='mdx-formula-wizard', form_name='member_expression_page_1', text='Filter selection')
        self.wait_for_loading()

        self.fill_form(panel_name='mdx-formula-wizard', form_name='member_expression_page_1', values={
            'selection': ('fancy-select', ['10'])
        })
        self.click_button(panel_name='mdx-formula-wizard', text=u'Finish')
        self.wait_for_loading()

        self.click_button(panel_name='config-widget-panel', text='Apply and close')
        self.wait_for_loading()

        self.assert_cube_table(1, value=['266,773','5,516'], css_selector='table.basesoft-cubetable', has_row_headers=False)


    def test_mdx_wizard_last_page_validation(self):
        # create account with permissions
        account = self._create_active_account()
        group = account.default_group
        self._attach_permissions_to_group(group, 'view_dashboard', 'save_dashboard', 'pivot_table', 'view_datasource',
                                          'item_result','view_calculated_member', 'create_calculated_member', 'update_calculated_member')

        controller = MdxCalculatedMemberController(self.sa_session, account_id=account.id)
        m = self.sa_session.query(MdxMethod).filter(MdxMethod.name=='Crossjoin').first()
        controller.grant_privilege_to(m, MdxMethod.PRIV_VIEW, group=True, target_group=group, check_own_privs=False)

        state = {
            'objectName': 'MdxMethodItem',
            'loaderName': 'iqube_mdx_method_item',
            'return_type': 'set',
            'method_id': m.id,
            'inputs': [None, None]
        }
        cm = controller.save_or_update_calculated_member(None, 'Test me', state)
        cm = controller.get_calculated_member(cm.id).pop()

        # open dashboard
        self.login(account.name, 'test_password')
        self.wait_for_loading()

        self.expand_layout_pane(0)

        # set mode to compose
        self.click_palette('Mode')
        self.click_palette('Compose')
        self.wait_for_loading()

        self.click_palette('IQReport', palette_index=2)

        self._add_result_item()
        self.click_fieldset(panel_name='config-widget-panel', title='Widget settings')
        self.click_fieldset(panel_name='config-widget-panel', title='Table settings')
        self.click_fieldset(panel_name='config-widget-panel', title='Named sets')
        self.fill_form(panel_name='config-widget-panel', values={
            'named_sets': ('fancy-select', ['Test me'])
        })

        self.click_button_by_id('cm-configure-named_sets')
        self.wait_for_loading()

        self.fill_form(panel_name='mdx-formula-wizard', form_name='datasource_page', values={
            'axis': ('select', 'Rows')
        })
        self.click_button(panel_name='mdx-formula-wizard', text=u'Next »')
        self.wait_for_loading()

        self.fill_form(panel_name='mdx-formula-wizard', form_name='set_expression_page_0', values={
            'dimension': ('select', 'Product'),
        })
        self.wait_for_loading()
        self.fill_form(panel_name='mdx-formula-wizard', form_name='set_expression_page_0', values={
            'selection': ('fancy-select', ['Non-Consumable'])
        })

        self.click_button(panel_name='mdx-formula-wizard', text=u'Next »')

        self.click_button(panel_name='mdx-formula-wizard', text=u'Finish')
        self.wait_for_loading(assert_no_errors=False)
        self.assert_growl_errors_present()
