# Copyright (c) 2012 BaseSoft B.V.
# See LICENSE for details.

from sqlalchemy import func

from basesoft.web.test.unit import iqportal_unittest

from iqportal.iqube.xmla.mdx import MdxParser
from iqportal.iqube.core.models import DataSource, MdxMethod
from iqportal.iqube.xmla.grammar import MemberCalculation
from twisted.trial.unittest import SkipTest
from django.conf import settings
from basesoft.web.core.exceptions import PublicUserException

class TestCase(iqportal_unittest.IQPortalTestCase):

    def setUp(self):
        """
        ds = 'IQuest'
        datasource = self.sa_session.query(DataSource).filter(func.substr(DataSource.name, -(len(ds))) == ds).all()
        if len(datasource) == 0:
            raise Exception('No %s datasource' % ds)
        self.datasource = datasource[0]
        self.client = self.datasource.connect(account_id=self.account.id)
        """
        self.parser = MdxParser()

    def test_simple_grouped_grammar(self):
        member_string = "[Survey].[MySurveys] AS [Survey].[1].[1]+[Survey].[1].[2]"
        calculation = self.parser.get_dict(member_string)

        self.assertEqual(calculation, {
            'calculation': [{
                'left': {'name': '[Survey].[1].[1]', 'description':'[Survey].[1].[1]'},
                'operator': '+',
                'right': {'name': '[Survey].[1].[2]', 'description':'[Survey].[1].[2]'}
            }],
            'hierarchy': 'Survey',
            'member_name': 'MySurveys'
        })

    def test_nested_calculation(self):
        member_string = "[Survey].[MySurveys] AS '([Survey].[1].[1]/[Survey].[1].[2])*100'"
        calculation = self.parser.get_dict(member_string)

        self.assertEqual(calculation, {
            'hierarchy': 'Survey',
            'calculation': [{
                'operator': '*',
                'right': {'name': '100', 'description': '100'},
                'left': {
                    'operator': '/',
                    'right': {'name': '[Survey].[1].[2]', 'description': '[Survey].[1].[2]'},
                    'left': {'name': '[Survey].[1].[1]', 'description': '[Survey].[1].[1]'}
                }
            }], 'member_name': 'MySurveys'
        })

    def test_simple_function(self):
        member_string = "[Measures].[Average] AS 'AVG([MySet])'"
        calculation = self.parser.parse(member_string)

        self.assertEqual(calculation.name[0], 'Measures')
        self.assertEqual(calculation.name[1], 'Average')

        parsed = self.parser.get_dict(member_string, None, 'test')
        self.assertDeepEqual({
            'hierarchy': 'Measures', 'calculation': [
                {'name': 'avg', 'description': 'avg'},
                {'name': '[MySet]', 'description': '[MySet]'}
            ], 'member_name': 'Average'}, parsed)

    def test_aggregate_function(self):
        member_string = "[Measures].[Agg] AS 'AGGREGATE({[Member].[1], [Member].[2]})'"

        #print MdxParser().get_dict(member_string, None, 'test')

        calculation = self.parser.parse(member_string)

        self.assertEqual(calculation.name[0], 'Measures')
        self.assertEqual(calculation.name[1], 'Agg')

        parsed = self.parser.get_dict(member_string, None, 'test')
        self.assertDeepEqual({
            'hierarchy': 'Measures', 'calculation': [
                {'name': 'aggregate', 'description': 'aggregate'},
                [{'name': '[Member].[1]', 'description': '[Member].[1]'},
                {'name': '[Member].[2]', 'description': '[Member].[2]'}]
            ], 'member_name': 'Agg'}, parsed)

    def test_simple_topcount(self):
        member_string = """[Some Set Oi] AS TopCount({[Promotion Media].[Media Type].Members},10,[Measures].[Unit Sales])"""

        parsed = self.parser.get_dict(member_string, None, 'test')
        self.assertDeepEqual({
            'hierarchy': 'FORMULA', 'calculation': [
                {'name': 'topcount', 'description': 'topcount'},
                {'name': '[Promotion Media].[Media Type].Members', 'description': '[Promotion Media].[Media Type] (Members)'},
                {'name': '10', 'description': '10'},
                {'name': '[Measures].[Unit Sales]', 'description': '[Measures].[Unit Sales]'}
            ], 'member_name': 'Some Set Oi'}, parsed)

    def test_nested_function(self):
        member_string =  """[Extrmee] AS TopCount(
            Crossjoin(
                {[Education Level].[Education Level].Members},{[Marital Status].[M],[Marital Status].[S]}
            ),
            5,
            [Measures].[Unit Sales]
        )"""

        parsed = self.parser.get_dict(member_string, None, 'test')
        self.assertDeepEqual({
            'hierarchy': 'FORMULA', 'calculation': [
                {'name': 'topcount', 'description': 'topcount'},
                {'name': 'crossjoin', 'description': 'crossjoin'},
                {'name': '[Education Level].[Education Level].Members', 'description': '[Education Level].[Education Level] (Members)'},
                [{'name': '[Marital Status].[M]', 'description': '[Marital Status].[M]'},
                 {'name': '[Marital Status].[S]', 'description': '[Marital Status].[S]'}],
                {'name': '5', 'description': '5'},
                {'name': '[Measures].[Unit Sales]', 'description': '[Measures].[Unit Sales]'}
            ], 'member_name': 'Extrmee'}, parsed)

    def test_nested_function_with_calculation(self):
        member_string =  """[Extrmee] AS TopCount(
            Crossjoin(
                {[Education Level].[Education Level].Members},{[Marital Status].[M],[Marital Status].[S]}
            ),
            5,
            ([Measures].[Unit Sales]/[Measures].[Customer Count])
        )"""

        parsed = self.parser.get_dict(member_string, None, 'test')
        self.assertDeepEqual({
            'hierarchy': 'FORMULA', 'calculation': [
                {'name': 'topcount', 'description': 'topcount'},
                {'name': 'crossjoin', 'description': 'crossjoin'},
                {'name': '[Education Level].[Education Level].Members', 'description': '[Education Level].[Education Level] (Members)'},
                [{'name': '[Marital Status].[M]', 'description': '[Marital Status].[M]'},
                 {'name': '[Marital Status].[S]', 'description': '[Marital Status].[S]'}],
                {'name': '5', 'description': '5'},
                {'operator': '/',
                 'right': {'name': '[Measures].[Customer Count]', 'description': '[Measures].[Customer Count]'},
                 'left': {'name': '[Measures].[Unit Sales]', 'description': '[Measures].[Unit Sales]'}
                }
            ], 'member_name': 'Extrmee'}, parsed)
