# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.

from django.conf import settings
from django.utils import simplejson

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.core.exceptions import PublicSecurityException, PermissionDeniedException, PublicUserException

from iqportal.iqube.compose.controllers import MdxCalculatedMemberController, MdxFormatStringController
from iqportal.iqube.core.models import MdxMethod, MdxCalculatedMember, DataSource, MdxDataSourceCalculatedMember
from iqportal.iqube.xmla.client import XMLAClient
from twisted.trial.unittest import SkipTest
from iqportal.iqube.core.controllers import ConfigurableWidgetController


class TestCase(iqportal_unittest.IQPortalTestCase):
    def setUp(self):
        self.connect_db()
        account = self._create_active_account()
        self.account = account
        self.group = account.default_group
        self.controller = MdxCalculatedMemberController(self.sa_session, account_id=account.id)
        self._attach_permissions_to_group(self.group, 'view_calculated_member')

    def _grant_method_privs(self):
        for m in self.sa_session.query(MdxMethod).all():
            self.controller.grant_privilege_to(m, MdxMethod.PRIV_VIEW, group=True, target_group=self.group, check_own_privs=False)

    def _get_method(self, name):
        return self.sa_session.query(MdxMethod).filter(MdxMethod.name==name).first()

    def test_save_simple_named_set(self):

        m = self._get_method('TopCount')
        state = {
            'objectName': 'MdxMethodItem',
            'loaderName': 'iqube_mdx_method_item',
            'return_type': 'set',
            'method_id': m.id,
            'inputs': [None, None, None],
        }

        # we do not have the permission.
        self.assertRaises(PermissionDeniedException, self.controller.save_or_update_calculated_member, None, 'My Named Set', state)
        self._attach_permissions_to_group(self.group, 'create_calculated_member')

        # we do not have the priv on the method.
        self.assertRaises(PublicSecurityException, self.controller.save_or_update_calculated_member, None, 'My Named Set', state)
        self._grant_method_privs()

        cm = self.controller.save_or_update_calculated_member(None, 'My Named Set', state)
        self.assertEqual(cm.type, MdxCalculatedMember.NAMED_SET)
        self.assertEqual(cm.input_count, 3)

        self.assertEqual(self.controller.get_parsed(cm.id, ['?', '?', '?']), 'TopCount(?,?,?)')

    def test_load_simple_state(self):

        # set the correct rights (this is verified in another test case)
        self._attach_permissions_to_group(self.group, 'create_calculated_member')
        self._grant_method_privs()

        m = self._get_method('TopCount')
        state = {
            'objectName': 'MdxMethodItem',
            'loaderName': 'iqube_mdx_method_item',
            'return_type': 'set',
            'method_id': m.id,
            'inputs': [None, None, None],
        }

        cm = self.controller.save_or_update_calculated_member(None, 'My Named Set', state)

        state = self.controller.load_state(cm.id)
        self.assertDeepEqual(state, {"dimension": None, "dbID": cm.id, "format_string": None, "input_count": 3, "type": 1, "name": "My Named Set", "children": {"inputs": [None, None, None], "loaderName": "iqube_mdx_method_item", "method_id": m.id, "return_type": "set", "objectName": "MdxMethodItem"}})

    def test_except_method_including_all(self):
        self._attach_permissions_to_group(self.group, 'create_calculated_member')
        self._grant_method_privs()

        m = self._get_method('Except')
        state = {
            'objectName': 'MdxMethodExceptItem',
            'loaderName': 'iqube_mdx_method_except_item',
            'return_type': 'set',
            'method_id': m.id,
            'inputs': [None, None, 'ALL'],
        }

        cm = self.controller.save_or_update_calculated_member(None, 'My Named Set', state)
        self.assertEqual(cm.type, MdxCalculatedMember.NAMED_SET)
        self.assertEqual(cm.input_count, 2)

        self.assertEqual(self.controller.get_parsed(cm.id, ['?', '?']), 'Except(?,?,ALL)')

    def test_nested_named_set_including_operators(self):

        # set the correct rights (this is verified in another test case)
        self._attach_permissions_to_group(self.group, 'create_calculated_member')
        self._grant_method_privs()

        m = self._get_method('TopCount')
        state = {
            'objectName': 'MdxMethodItem',
            'loaderName': 'iqube_mdx_method_item',
            'return_type': 'set',
            'method_id': m.id,
            'inputs': [None, None, {
                'objectName': 'MdxOperatorItem',
                'loaderName': 'iqube_mdx_operator_item',
                'return_type': 'numeric',
                'operator': '*', # multiply operator
                'inputs': [None, {
                    'objectName': 'MdxNumericalInputItem',
                    'loaderName': 'iqube_mdx_numerical_input_item',
                    'return_type': 'numeric',
                    'value': 1000
                }]
            }],
        }
        cm = self.controller.save_or_update_calculated_member(None, 'My Named Set', state)
        cm = self.controller.get_calculated_member(cm.id).pop()

        self.assertEqual(cm.type, MdxCalculatedMember.NAMED_SET)
        self.assertEqual(cm.input_count, 3)

        self.assertEqual(self.controller.get_parsed(cm.id, ['?', '?', '?']), 'TopCount(?,?,(?*1000))')

    def test_tuple_expression(self):
        self._attach_permissions_to_group(self.group, 'create_calculated_member')
        self._grant_method_privs()

        state = {
            'objectName': 'MdxTupleExpressionItem',
            'loaderName': 'iqube_mdx_tuple_expression_item',
            'return_type': 'member',
            'is_tuple': True,
            'inputs': [None, None],
        }
        cm = self.controller.save_or_update_calculated_member(None, 'My Tuple', state)
        cm = self.controller.get_calculated_member(cm.id).pop()

        self.assertEqual(cm.type, MdxCalculatedMember.CALCULATED_MEMBER)
        self.assertEqual(cm.input_count, 2)

        self.assertEqual(self.controller.get_parsed(cm.id, ['?', '?']), '(?,?)')

    def test_simple_property_on_set(self):
        self._attach_permissions_to_group(self.group, 'create_calculated_member')
        self._grant_method_privs()

        m = self._get_method('TopCount')
        state = {
            'objectName': 'MdxSetPropertyItem',
            'loaderName': 'iqube_mdx_set_property_item',
            'return_type': 'numeric',
            'value': 'CurrentOrdinal',
            'is_property': True,
            'inputs': [{
                'objectName': 'MdxMethodItem',
                'loaderName': 'iqube_mdx_method_item',
                'return_type': 'set',
                'method_id': m.id,
                'inputs': [None, None]
            }],
        }
        cm = self.controller.save_or_update_calculated_member(None, 'My Property', state)
        cm = self.controller.get_calculated_member(cm.id).pop()

        self.assertEqual(cm.type, MdxCalculatedMember.CALCULATED_MEMBER)
        self.assertEqual(cm.input_count, 2)

        self.assertEqual(self.controller.get_parsed(cm.id, ['?', '?']), 'TopCount(?,?).CurrentOrdinal')

    def test_complex_property_on_set(self):
        self._attach_permissions_to_group(self.group, 'create_calculated_member')
        self._grant_method_privs()

        m = self._get_method('TopCount')
        state = {
            'objectName': 'MdxSetPropertyItem',
            'loaderName': 'iqube_mdx_set_property_item',
            'return_type': 'numeric',
            'value': 'Item',
            'is_property': True,
            'inputs': [{
                'objectName': 'MdxMethodItem',
                'loaderName': 'iqube_mdx_method_item',
                'return_type': 'set',
                'method_id': m.id,
                'inputs': [None, None]
            }, None],
        }
        cm = self.controller.save_or_update_calculated_member(None, 'My Property', state)
        cm = self.controller.get_calculated_member(cm.id).pop()

        self.assertEqual(cm.type, MdxCalculatedMember.CALCULATED_MEMBER)
        self.assertEqual(cm.input_count, 3)

        self.assertEqual(self.controller.get_parsed(cm.id, ['?', '?', '?']), 'TopCount(?,?).Item(?)')

    def test_complex_property_on_set_no_input(self):
        self._attach_permissions_to_group(self.group, 'create_calculated_member')
        self._grant_method_privs()

        m = self._get_method('TopCount')
        state = {
            'objectName': 'MdxSetPropertyItem',
            'loaderName': 'iqube_mdx_set_property_item',
            'return_type': 'numeric',
            'value': 'Item',
            'is_property': True,
            'inputs': [None, None],
        }
        cm = self.controller.save_or_update_calculated_member(None, 'My Property', state)
        cm = self.controller.get_calculated_member(cm.id).pop()

        self.assertEqual(cm.type, MdxCalculatedMember.CALCULATED_MEMBER)
        self.assertEqual(cm.input_count, 2)

        self.assertEqual(self.controller.get_parsed(cm.id, ['?', '?']), '?.Item(?)')

    def test_use_of_filter_and_logical_expression(self):
        # set the correct rights (this is verified in another test case)
        self._attach_permissions_to_group(self.group, 'create_calculated_member')
        self._grant_method_privs()

        m = self._get_method('Filter')
        state = {
            'objectName': 'MdxMethodItem',
            'loaderName': 'iqube_mdx_method_item',
            'return_type': 'set',
            'method_id': m.id,
            'inputs': [None, {
                'objectName': 'MdxComparatorItem',
                'loaderName': 'iqube_mdx_comparator_item',
                'return_type': 'logical',
                'comparator': '>',
                'inputs': [None, {
                    'objectName': 'MdxNumericalInputItem',
                    'loaderName': 'iqube_mdx_numerical_input_item',
                    'return_type': 'numeric',
                    'value': 1000
                }]
            }],
        }
        cm = self.controller.save_or_update_calculated_member(None, 'My Named Set', state)
        cm = self.controller.get_calculated_member(cm.id).pop()

        self.assertEqual(cm.type, MdxCalculatedMember.NAMED_SET)
        self.assertEqual(cm.input_count, 2)

        self.assertEqual(self.controller.get_parsed(cm.id, ['?', '2000']), 'Filter(?,(2000>1000))')

    def test_use_of_axis_expression(self):
        self._attach_permissions_to_group(self.group, 'create_calculated_member')
        self._grant_method_privs()

        m1 = self._get_method('Sum')
        m2 = self._get_method('Axis')
        state = {
            'objectName': 'MdxMethodItem',
            'loaderName': 'iqube_mdx_method_item',
            'return_type': 'numeric',
            'method_id': m1.id,
            'inputs': [{
                'objectName': 'MdxMethodAxisItem',
                'loaderName': 'iqube_mdx_method_axis_item',
                'return_type': 'set',
                'method_id': m2.id,
                'value': 1
            }, None]
        }
        cm = self.controller.save_or_update_calculated_member(None, 'My CM', state)
        cm = self.controller.get_calculated_member(cm.id).pop()

        self.assertEqual(cm.type, MdxCalculatedMember.CALCULATED_MEMBER)
        self.assertEqual(cm.input_count, 1)

        self.assertEqual(self.controller.get_parsed(cm.id, [None]), [('[Axis1]', u'Axis(1)'), u'Sum([Axis1])'])
        _with = self.controller.get_with_statement('[Sum over axis]', MdxCalculatedMember.CALCULATED_MEMBER, self.controller.get_parsed(cm.id, [None]), None)
        self.assertEqual(_with, ('[Measures].[Sum over axis]', [u"SET [Axis1] AS 'Axis(1)'", u'MEMBER [Measures].[Sum over axis] AS Sum([Axis1])'], '[Measures]'))

    def test_with_datasource(self):

        host, port = settings.MONDRIAN_XMLA_SERVER
        client = XMLAClient(host, port, settings.MONDRIAN_XMLA_PATH, '%s.FoodMart' % settings.SETTINGS_MODULE)
        if not client.test_connection():
            raise SkipTest('No Mondrian connection available')

        # set the correct rights (this is verified in another test case)
        self._attach_permissions_to_group(self.group, 'view_datasource', 'view_calculated_member', 'create_calculated_member')
        self._grant_method_privs()


        ds = self.sa_session.query(DataSource).filter(DataSource.name=='%s.FoodMart' % settings.SETTINGS_MODULE).first()

        # create a calculation which uses the expression objects and datasource/cube.
        m = self._get_method('TopCount')
        state = {
            'objectName': 'MdxMethodItem',
            'loaderName': 'iqube_mdx_method_item',
            'return_type': 'set',
            'method_id': m.id,
            'inputs': [{
                'objectName': 'MdxDataSourceExpressionItem',
                'loaderName': 'iqube_datasource_expression_item',
                'return_type': 'set',
                'selection': [simplejson.dumps(x) for x in client.get_level_members('Sales', '[Store].[Store Country]')[:2]] # US and Canada?
            }, None, None]
        }
        # save the calculated member
        cm = self.controller.save_or_update_calculated_member(None, 'My Named Set', state, datasource_id=ds.id, cube='Sales')
        cm = self.controller.get_calculated_member(cm.id).pop()

        # assert the polymorphic type
        self.assertTrue(isinstance(cm, MdxDataSourceCalculatedMember))

        # assert parsed.
        self.assertEqual(self.controller.get_parsed(cm.id, ['?', '?']), 'TopCount({[Store].[Canada],[Store].[Mexico]},?,?)')

    def test_nested_calculated_members(self):
        self._attach_permissions_to_group(self.group, 'create_calculated_member')
        self._grant_method_privs()

        # Create a top count CM
        m = self._get_method('TopCount')
        state = {
            'objectName': 'MdxMethodItem',
            'loaderName': 'iqube_mdx_method_item',
            'return_type': 'set',
            'method_id': m.id,
            'inputs': [None, None, {
                'objectName': 'MdxOperatorItem',
                'loaderName': 'iqube_mdx_operator_item',
                'return_type': 'numeric',
                'operator': '*', # multiply operator
                'inputs': [None, {
                    'objectName': 'MdxNumericalInputItem',
                    'loaderName': 'iqube_mdx_numerical_input_item',
                    'return_type': 'numeric',
                    'value': 1000
                }]
            }],
        }
        cm = self.controller.save_or_update_calculated_member(None, 'My nested CM', state)
        cm = self.controller.get_calculated_member(cm.id).pop()

        # now we create a second member
        m = self._get_method('Crossjoin')
        state = {
            'objectName': 'MdxMethodItem',
            'loaderName': 'iqube_mdx_method_item',
            'return_type': 'set',
            'method_id': m.id,
            'inputs': [{
                'objectName': 'MdxCalculatedMemberItem',
                'loaderName': 'iqube_calculated_member_item',
                'calculated_member_id': cm.id,
                'inputs': [None, None, None]
            }, None],
        }
        cm = self.controller.save_or_update_calculated_member(None, 'My epic CM', state)
        cm = self.controller.get_calculated_member(cm.id).pop()

        self.assertEqual(cm.type, MdxCalculatedMember.NAMED_SET)
        self.assertEqual(cm.input_count, 4)

        self.assertEqual(self.controller.get_parsed(cm.id, ['A', 'B', 'C', 'D']), 'Crossjoin(TopCount(A,B,(C*1000)),D)')

    def test_nested_calculated_member_recursion_failure(self):
        self._attach_permissions_to_group(self.group, 'create_calculated_member', 'update_calculated_member')
        self._grant_method_privs()

        # Create a top count CM
        m = self._get_method('TopCount')
        state = {
            'objectName': 'MdxMethodItem',
            'loaderName': 'iqube_mdx_method_item',
            'return_type': 'set',
            'method_id': m.id,
            'inputs': [None, None, {
                'objectName': 'MdxOperatorItem',
                'loaderName': 'iqube_mdx_operator_item',
                'return_type': 'numeric',
                'operator': '*', # multiply operator
                'inputs': [None, {
                    'objectName': 'MdxNumericalInputItem',
                    'loaderName': 'iqube_mdx_numerical_input_item',
                    'return_type': 'numeric',
                    'value': 1000
                }]
            }],
        }
        cm = self.controller.save_or_update_calculated_member(None, 'My nested CM', state)
        cm = self.controller.get_calculated_member(cm.id).pop()

        # now we create a second member
        m = self._get_method('Crossjoin')
        state = {
            'objectName': 'MdxMethodItem',
            'loaderName': 'iqube_mdx_method_item',
            'return_type': 'set',
            'method_id': m.id,
            'inputs': [{
                'objectName': 'MdxCalculatedMemberItem',
                'loaderName': 'iqube_calculated_member_item',
                'calculated_member_id': cm.id,
                'inputs': [None, None, None]
            }, None],
        }
        ncm = self.controller.save_or_update_calculated_member(None, 'My epic CM', state)
        ncm = self.controller.get_calculated_member(ncm.id).pop()

        loaded = self.controller.load_state(cm.id)['children']

        loaded['inputs'][0] = {
            'objectName': 'MdxCalculatedMemberItem',
            'loaderName': 'iqube_calculated_member_item',
            'calculated_member_id': ncm.id,
            'inputs': []
        }

        # cant use recursion!
        self.assertRaises(PublicUserException, self.controller.save_or_update_calculated_member, cm.id, 'NESTED FAILURE', loaded)

    def test_nested_calculated_member_end_point(self):
        self._attach_permissions_to_group(self.group, 'create_calculated_member', 'update_calculated_member')
        self._grant_method_privs()

        # Create a top count CM
        m = self._get_method('TopCount')
        state = {
            'objectName': 'MdxMethodItem',
            'loaderName': 'iqube_mdx_method_item',
            'return_type': 'set',
            'method_id': m.id,
            'inputs': [None, None, None]
        }
        cm = self.controller.save_or_update_calculated_member(None, 'My nested CM', state)
        cm = self.controller.get_calculated_member(cm.id).pop()

        m = self._get_method('Crossjoin')
        state = {
            'objectName': 'MdxCalculatedMemberItem',
            'loaderName': 'iqube_calculated_member_item',
            'calculated_member_id': cm.id,
            'return_type': 'set',
            'inputs': [{
                'objectName': 'MdxMethodItem',
                'loaderName': 'iqube_mdx_method_item',
                'return_type': 'set',
                'method_id': m.id,
                'inputs': [None, None],
            }, None, None]
        }
        ncm = self.controller.save_or_update_calculated_member(None, 'My epic CM', state)
        ncm = self.controller.get_calculated_member(ncm.id).pop()

        self.assertEqual(self.controller.get_parsed(ncm.id, ['A', 'B', 'C', 'D']), 'TopCount(Crossjoin(A,B),C,D)')

    def test_iif_expression(self):
        self._attach_permissions_to_group(self.group, 'create_calculated_member', 'update_calculated_member')
        self._grant_method_privs()

        m = self._get_method('Iif')
        state = {
            'objectName': 'MdxMethodIifItem',
            'loaderName': 'iqube_mdx_method_iif_item',
            'return_type': 'set',
            'method_id': m.id,
            'inputs': [{
                'objectName': 'MdxComparatorItem',
                'loaderName': 'iqube_mdx_comparator_item',
                'return_type': 'logical',
                'comparator': '>',
                'inputs': [None, {
                    'objectName': 'MdxNumericalInputItem',
                    'loaderName': 'iqube_mdx_numerical_input_item',
                    'return_type': 'numeric',
                    'value': 1000
                }]
            }, None, None]
        }
        cm = self.controller.save_or_update_calculated_member(None, 'My nested CM', state)
        cm = self.controller.get_calculated_member(cm.id).pop()

        self.assertEqual(self.controller.get_parsed(cm.id, ['A', 'B', 'C']), 'Iif((A>1000),B,C)')

    def test_iif_expression_string(self):
        self._attach_permissions_to_group(self.group, 'create_calculated_member', 'update_calculated_member')
        self._grant_method_privs()

        m = self._get_method('Iif')
        state = {
            'objectName': 'MdxMethodIifItem',
            'loaderName': 'iqube_mdx_method_iif_item',
            'return_type': 'string',
            'method_id': m.id,
            'inputs': [None, {
                'objectName': 'MdxStringInputItem',
                'loaderName': 'iqube_mdx_string_input_item',
                'return_type': 'string',
                'value': 'Success!'
            }, {
                'objectName': 'MdxStringInputItem',
                'loaderName': 'iqube_mdx_string_input_item',
                'return_type': 'string',
                'value': 'Failure!'
            }]
        }
        cm = self.controller.save_or_update_calculated_member(None, 'My nested CM', state)
        cm = self.controller.get_calculated_member(cm.id).pop()

        self.assertEqual(self.controller.get_parsed(cm.id, ['A']), "Iif(A,'Success!','Failure!')")

    def test_save_and_update_format_string_preset(self):

        c = MdxFormatStringController(self.sa_session, account_id=self.account.id)
        self.assertRaises(PermissionDeniedException, c.save_or_update_format_string, None, 'Test', '#.##0.00')

        self._attach_permissions_to_group(self.group, 'manage_format_strings')

        fmt = c.save_or_update_format_string(None, 'Test', '#.##0.00')

        loaded = c.get_format_string(fmt.id)
        self.assertEqual(fmt.format_string, loaded.format_string)

        updated = c.save_or_update_format_string(fmt.id, 'Test Update', '#.###')
        self.assertEqual(fmt.id, updated.id)
        self.assertEqual(updated.format_string, '#.###')
        self.assertEqual(updated.name, 'Test Update')

    def test_repeater_expression(self):
        self._attach_permissions_to_group(self.group, 'create_calculated_member', 'update_calculated_member')

        state = {
            'repeaters': [{
                'repeaterID': 'repeater-1',
                'return_type': 'numeric',
                'loaderName': 'iqube_mdx_repeater_item',
                'inputs': [{
                    'objectName': 'MdxNumericalInputItem',
                    'loaderName': 'iqube_mdx_numerical_input_item',
                    'return_type': 'numeric',
                    'value': 1000
                }]
            }],
            'objectName': 'MdxOperatorItem',
            'loaderName': 'iqube_mdx_operator_item',
            'return_type': 'numeric',
            'operator': '/',
            'inputs': [{
                'objectName': 'MdxOperatorItem',
                'loaderName': 'iqube_mdx_operator_item',
                'return_type': 'numeric',
                'operator': '-',
                'inputs': [{
                    'repeater': 'repeater-1'
                }, {
                    'objectName': 'MdxNumericalInputItem',
                    'loaderName': 'iqube_mdx_numerical_input_item',
                    'return_type': 'numeric',
                    'value': 700
                }]
            }, {
                'repeater': 'repeater-1'
            }]
        }

        cm = self.controller.save_or_update_calculated_member(None, 'My repeater CM', state)
        cm = self.controller.get_calculated_member(cm.id).pop()
        parsed_cm = self.controller.get_parsed(cm.id)
        self.assertEqual(parsed_cm, [('[Measures].[repeater-1]', '1000'), '(([Measures].[repeater-1]-700)/[Measures].[repeater-1])'])

        #test get_with_statement
        _type = MdxCalculatedMember.CALCULATED_MEMBER
        self.assertEqual(self.controller.get_with_statement('[My repeater]', _type, parsed_cm, None), ('[Measures].[My repeater]' , ["MEMBER [Measures].[repeater-1] AS '1000'", 'MEMBER [Measures].[My repeater] AS (([Measures].[repeater-1]-700)/[Measures].[repeater-1])'], '[Measures]'))

    def test_nested_repeater_expression(self):
        self._attach_permissions_to_group(self.group, 'create_calculated_member', 'update_calculated_member')

        state = {
            'repeaters': [{
                'repeaterID': 'repeater-1',
                'return_type': 'numeric',
                'loaderName': 'iqube_mdx_repeater_item',
                'inputs': [{
                    'objectName': 'MdxNumer>>>icalInputItem',
                    'loaderName': 'iqube_mdx_numerical_input_item',
                    'return_type': 'numeric',
                    'value': 1000
                }]
            }],
            'objectName': 'MdxOperatorItem',
            'loaderName': 'iqube_mdx_operator_item',
            'return_type': 'numeric',
            'operator': '/',
            'inputs': [{
                'repeaters': [{
                    'repeaterID': 'repeater-2',
                    'return_type': 'numeric',
                    'loaderName': 'iqube_mdx_repeater_item',
                    'inputs': [{
                        'objectName': 'MdxNumericalInputItem',
                        'loaderName': 'iqube_mdx_numerical_input_item',
                        'return_type': 'numeric',
                        'value': 700
                    }]
            }],
                'objectName': 'MdxOperatorItem',
                'loaderName': 'iqube_mdx_operator_item',
                'return_type': 'numeric',
                'operator': '-',
                'inputs': [{
                        'objectName': 'MdxOperatorItem',
                        'loaderName': 'iqube_mdx_operator_item',
                        'return_type': 'numeric',
                        'operator': '+',
                        'inputs': [{
                                'repeater': 'repeater-1'
                            }, {
                                'repeater': 'repeater-2'
                            }]
                        }, {
                            'repeater': 'repeater-2'
                        }]
                }, {
                    'repeater': 'repeater-1'
                }]
        }

        cm = self.controller.save_or_update_calculated_member(None, 'My repeater CM', state)
        cm = self.controller.get_calculated_member(cm.id).pop()
        parsed_cm = self.controller.get_parsed(cm.id)
        self.assertEqual(parsed_cm, [('[Measures].[repeater-1]', '1000'), ('[Measures].[repeater-2]', '700'), '((([Measures].[repeater-1]+[Measures].[repeater-2])-[Measures].[repeater-2])/[Measures].[repeater-1])'])

        #test get_with_statement
        _type = MdxCalculatedMember.CALCULATED_MEMBER
        self.assertEqual(self.controller.get_with_statement('[My repeater]', _type, parsed_cm, None), ('[Measures].[My repeater]' , ["MEMBER [Measures].[repeater-1] AS '1000'", "MEMBER [Measures].[repeater-2] AS '700'", 'MEMBER [Measures].[My repeater] AS ((([Measures].[repeater-1]+[Measures].[repeater-2])-[Measures].[repeater-2])/[Measures].[repeater-1])'], '[Measures]'))


    def test_validate_repeater(self):

        self._attach_permissions_to_group(self.group, 'create_calculated_member', 'update_calculated_member')

        state = {
            'repeaters': [{
                'repeaterID': 'repeater-1',
                'return_type': 'numeric',
                'loaderName': 'iqube_mdx_repeater_item',
                'inputs': [{
                    'repeater': 'repeater-2'
                }]
            }, {
                'repeaterID': 'repeater-2',
                'return_type': 'numeric',
                'loaderName': 'iqube_mdx_repeater_item',
                'inputs': [{
                    'repeater': 'repeater-1'
                }]
            }],
            'objectName': 'MdxOperatorItem',
            'loaderName': 'iqube_mdx_operator_item',
            'return_type': 'numeric',
            'operator': '/',
            'inputs': [{
                'objectName': 'MdxOperatorItem',
                'loaderName': 'iqube_mdx_operator_item',
                'return_type': 'numeric',
                'operator': '-',
                'inputs': [{
                        'objectName': 'MdxOperatorItem',
                        'loaderName': 'iqube_mdx_operator_item',
                        'return_type': 'numeric',
                        'operator': '+',
                        'inputs': [{
                                'repeater': 'repeater-1'
                            }, {
                                'repeater': 'repeater-2'
                        }]
                    }, {
                    'repeater': 'repeater-2'
                }]
            }, {
                'repeater': 'repeater-1'
            }]
        }

        self.assertRaises(PublicUserException, self.controller._validate_state, state)

    def test_with_statement_repeater_wrong_order(self):

        self._attach_permissions_to_group(self.group, 'create_calculated_member', 'update_calculated_member')

        state = {
            'repeaters': [{
                'repeaterID': 'repeater-1',
                'return_type': 'numeric',
                'loaderName': 'iqube_mdx_repeater_item',
                'inputs': [{
                    'repeater': 'repeater-2'
                }]
            }, {
                'repeaterID': 'repeater-2',
                'return_type': 'numeric',
                'loaderName': 'iqube_mdx_repeater_item',
                'inputs': [None]
            }],
            'objectName': 'MdxOperatorItem',
            'loaderName': 'iqube_mdx_operator_item',
            'return_type': 'numeric',
            'operator': '/',
            'inputs': [{
                'objectName': 'MdxOperatorItem',
                'loaderName': 'iqube_mdx_operator_item',
                'return_type': 'numeric',
                'operator': '-',
                'inputs': [{
                        'objectName': 'MdxOperatorItem',
                        'loaderName': 'iqube_mdx_operator_item',
                        'return_type': 'numeric',
                        'operator': '+',
                        'inputs': [{
                                'repeater': 'repeater-1'
                            }, {
                                'repeater': 'repeater-2'
                        }]
                    }, {
                    'repeater': 'repeater-2'
                }]
            }, {
                'repeater': 'repeater-1'
            }]
        }

        cm = self.controller.save_or_update_calculated_member(None, 'My repeater CM', state)
        cm = self.controller.get_calculated_member(cm.id).pop()
        parsed_cm = self.controller.get_parsed(cm.id, [200])

        #test get_with_statement
        _type = MdxCalculatedMember.CALCULATED_MEMBER
        self.assertEqual(self.controller.get_with_statement('[My repeater]', _type, parsed_cm, None),
                         ('[Measures].[My repeater]' , ['MEMBER [Measures].[repeater-2] AS \'200\'', 'MEMBER [Measures].[repeater-1] AS \'[Measures].[repeater-2]\'', 'MEMBER [Measures].[My repeater] AS ((([Measures].[repeater-1]+[Measures].[repeater-2])-[Measures].[repeater-2])/[Measures].[repeater-1])'], '[Measures]'))


    def test_validate_repeater_deep_recursion(self):

        self._attach_permissions_to_group(self.group, 'create_calculated_member', 'update_calculated_member')

        state = {
            'repeaters': [{
                'repeaterID': 'repeater-1',
                'return_type': 'numeric',
                'loaderName': 'iqube_mdx_repeater_item',
                'inputs': [{
                    'repeater': 'repeater-2'
                }]
            }, {
                'repeaterID': 'repeater-2',
                'return_type': 'numeric',
                'loaderName': 'iqube_mdx_repeater_item',
                'inputs': [{
                    'repeater': 'repeater-3'
                }]
            }, {
                'repeaterID': 'repeater-3',
                'return_type': 'numeric',
                'loaderName': 'iqube_mdx_repeater_item',
                'inputs': [{
                    'repeater': 'repeater-1'
                }]
            }],
            'objectName': 'MdxOperatorItem',
            'loaderName': 'iqube_mdx_operator_item',
            'return_type': 'numeric',
            'operator': '/',
            'inputs': [{
                'objectName': 'MdxOperatorItem',
                'loaderName': 'iqube_mdx_operator_item',
                'return_type': 'numeric',
                'operator': '-',
                'inputs': [{
                        'objectName': 'MdxOperatorItem',
                        'loaderName': 'iqube_mdx_operator_item',
                        'return_type': 'numeric',
                        'operator': '+',
                        'inputs': [{
                                'repeater': 'repeater-1'
                            }, {
                                'repeater': 'repeater-2'
                        }]
                    }, {
                    'repeater': 'repeater-2'
                }]
            }, {
                'repeater': 'repeater-1'
            }]
        }

        self.assertRaises(PublicUserException, self.controller._validate_state, state)

    def test_validate_repeater_nested_recursion(self):

        self._attach_permissions_to_group(self.group, 'create_calculated_member', 'update_calculated_member')

        state = {
            'repeaters': [{
                'repeaterID': 'repeater-1',
                'return_type': 'numeric',
                'loaderName': 'iqube_mdx_repeater_item',
                'inputs': [{
                    'repeater': 'repeater-2'
                }]
            }, {
                'repeaterID': 'repeater-2',
                'return_type': 'numeric',
                'loaderName': 'iqube_mdx_repeater_item',
                'inputs': [{
                    'objectName': 'MdxOperatorItem',
                    'loaderName': 'iqube_mdx_operator_item',
                    'return_type': 'numeric',
                    'operator': '-',
                    'inputs': [{
                            'objectName': 'MdxOperatorItem',
                            'loaderName': 'iqube_mdx_operator_item',
                            'return_type': 'numeric',
                            'operator': '+',
                            'inputs': [{
                                    'repeater': 'repeater-1'
                                }, None]
                        }, None]
                }, None]
            }],
            'objectName': 'MdxOperatorItem',
            'loaderName': 'iqube_mdx_operator_item',
            'return_type': 'numeric',
            'operator': '/',
            'inputs': []
        }

        self.assertRaises(PublicUserException, self.controller._validate_state, state)

    def test_state_from_mdx(self):
        self._grant_method_privs()
        mdx = """
            WITH MEMBER [Measures].[Test] AS '1000*500'
        """
        state = self.controller.state_from_mdx(mdx)

        valid = {
            'datasource_id': None,
            'cube': None,
            'repeaters': [],
            'children': {
                'objectName': 'MdxOperatorItem',
                'loaderName': 'iqube_mdx_operator_item',
                'return_type': 'numeric',
                'operator': '*', # multiply operator
                "posXY": [450, 650],
                'inputs': [{
                    'objectName': 'MdxNumericalInputItem',
                    'loaderName': 'iqube_mdx_numerical_input_item',
                    'return_type': 'numeric',
                    'value': 1000,
                    "posXY": [325, 450]
                }, {
                    'objectName': 'MdxNumericalInputItem',
                    'loaderName': 'iqube_mdx_numerical_input_item',
                    'return_type': 'numeric',
                    'value': 500,
                    "posXY": [575, 450]
                }]
            }
         }

        self.assertDeepEqual(valid, state)

        mdx = "WITH MEMBER [Test] AS TopCount(Crossjoin({[Measures].[None]},{[Measures].[None]}),[Measures].[None],(1000*500))"
        state = self.controller.state_from_mdx(mdx)

        valid = {
            'datasource_id': None,
            'cube': None,
            'repeaters': [],
            'children': {
                'method_id': 4,
                'objectName': 'MdxMethodItem',
                'posXY': [450, 650],
                'return_type': 'set',
                'loaderName': 'iqube_mdx_method_item',
                'inputs': [{
                    'posXY': [199, 450],
                    'loaderName': 'iqube_mdx_method_item',
                    'method_id': 8,
                    'return_type': 'set',
                    'objectName': 'MdxMethodItem',
                    'inputs': [None, None]
                }, None, {
                    'objectName': 'MdxOperatorItem',
                    'loaderName': 'iqube_mdx_operator_item',
                    'operator': '*',
                    'return_type': 'numeric',
                    'posXY': [699, 450],
                    'inputs': [{
                        'posXY': [574, 250],
                        'value': 1000,
                        'loaderName': 'iqube_mdx_numerical_input_item',
                        'return_type': 'numeric',
                        'objectName': 'MdxNumericalInputItem'
                    }, {
                        'posXY': [824, 250],
                        'value': 500,
                        'loaderName': 'iqube_mdx_numerical_input_item',
                        'return_type': 'numeric',
                        'objectName': 'MdxNumericalInputItem'
                    }],
                }],
            }
        }

        mdx = "WITH MEMBER [Test] AS TopCount(Crossjoin({[Marital Status].[M],[Marital Status].[S]},{[Store].[Mexico]}),5,[Measures].[Unit Sales])"
        state = self.controller.state_from_mdx(mdx)

    def test_state_from_mdx_no_priv(self):
        mdx = "WITH MEMBER [Test] AS TopCount({[Measures].[None]},5,[Measures].[None])"
        self.assertRaises(PermissionDeniedException, self.controller.state_from_mdx, mdx)

    def test_state_from_mdx_repeaters(self):
        self._grant_method_privs()
        mdx = "WITH SET [MySet] AS 'CROSSJOIN({[Marital Status].[M],[Marital Status].[S]}, {[Store].[Mexico]})'\nMEMBER [Measures].[Top] AS TopCount([MySet],5,[Measures].[None])"
        state = self.controller.state_from_mdx(mdx)
        self.assertEqual(len(state['repeaters']), 1)

    def test_mdx_auto_configure(self):
        cube_name = 'Sales'
        datasource = self.sa_session.query(DataSource).filter(DataSource.name == '%s.FoodMart' % settings.SETTINGS_MODULE).first()

        self._attach_permissions_to_group(self.group, 'manage_mdx_methods', 'view_datasource', 'create_calculated_member', 'view_calculated_member', 'update_calculated_member')
        # grant privilege to mdx method
        agg = self.sa_session.query(MdxMethod).filter(MdxMethod.name == 'Aggregate').first()
        self.controller.grant_privilege_to(agg, MdxMethod.PRIV_VIEW, group=True, target_group=self.group, check_own_privs=False)

        state = {
            'objectName': 'MdxMethodItem',
            'loaderName': 'iqube_mdx_method_item',
            'return_type': 'numeric',
            'method_id': agg.id,
            'inputs': [None]
        }

        cm = self.controller.save_or_update_calculated_member(None, 'CMagg', state, datasource_id=datasource.id, cube = cube_name, dimension='Time')

        #auto configure should not be done on a calculated member with open inputs (raise exception)
        self.assertRaises(PublicUserException, self.controller.auto_configure_mdx_function, cm.id)

        #add datasource item
        new_state = {
            'objectName': 'MdxMethodItem',
            'loaderName': 'iqube_mdx_method_item',
            'return_type': 'numeric',
            'method_id': agg.id,
            'inputs': [{"inputs": [],
                        "selection": [
                                      {"dimension_unique_name": "[Time]", "member_unique_name": "[Time.Weekly].[1997].[1]", "depth": "2", "level_unique_name": "[Time.Weekly].[Week]", "member_name": "1", "hierarchy_unique_name": "[Time.Weekly]"},
                                      {"dimension_unique_name": "[Time]", "member_unique_name": "[Time.Weekly].[1997].[2]", "depth": "2", "level_unique_name": "[Time.Weekly].[Week]", "member_name": "2", "hierarchy_unique_name": "[Time.Weekly]"},
                                      {"dimension_unique_name": "[Time]", "member_unique_name": "[Time.Weekly].[1997].[3]", "depth": "2", "level_unique_name": "[Time.Weekly].[Week]", "member_name": "3", "hierarchy_unique_name": "[Time.Weekly]"},
                                      {"dimension_unique_name": "[Time]", "member_unique_name": "[Time.Weekly].[1997].[4]", "depth": "2", "level_unique_name": "[Time.Weekly].[Week]", "member_name": "4", "hierarchy_unique_name": "[Time.Weekly]"}
                                      ],
                        'objectName': 'MdxDataSourceExpressionItem',
                        'loaderName': 'iqube_datasource_expression_item',
                        'return_type': 'set',
                        "type": "set",
                        "dimension": "[Time.Weekly].[Year]"
                       }]
        }

        mcm = self.controller.save_or_update_calculated_member(cm.id, 'CMAggTime', new_state, datasource_id=datasource.id, cube = cube_name, dimension='Time')
        formula_conf = self.controller.auto_configure_mdx_function(mcm.id)
        #auto_configure should not raise exception
        formula_config = {
            'calculated_member_id': mcm.id,
            'replacements': [],
            'settings': {
                'label': mcm.label
            }
        }
        self.assertDeepEqual(formula_config, formula_conf)

    def test_auto_configure_dimension_options(self):

        self._attach_permissions_to_group(self.group, 'create_calculated_member')

        state = {
            'objectName': 'MdxOperatorItem',
            'loaderName': 'iqube_mdx_operator_item',
            'return_type': 'numeric',
            'operator': '*', # multiply operator
            'inputs': [{
                'objectName': 'MdxNumericalInputItem',
                'loaderName': 'iqube_mdx_numerical_input_item',
                'return_type': 'numeric',
                'value': 5
            }, {
                'objectName': 'MdxNumericalInputItem',
                'loaderName': 'iqube_mdx_numerical_input_item',
                'return_type': 'numeric',
                'value': 1000
            }]
        }
        cm = self.controller.save_or_update_calculated_member(None, 'My CM', state)

        cube_name = 'Sales'
        datasource = self.sa_session.query(DataSource).filter(DataSource.name == '%s.FoodMart' % settings.SETTINGS_MODULE).first()

        config = {
            "available_on_rows": False,
            "show_named_sets": False,
            "has_select_as_group": False,
            "available_as_split_by": False,
            "required": True,
            "available_as_slicer": True,
            "caption": "Measures",
            "fancy_select_module": "iq_fancyselect",
            "available_on_columns": False,
            "show_calculated_members": True,
            "children": [
                {
                    "unique_name": "[Measures].[MeasuresLevel]",
                    "has_children_available": False,
                    "caption": "MeasuresLevel",
                    "has_all_member_available": False,
                    "show_members": True,
                    "method_argument": "[Measures].[MeasuresLevel]",
                    "method": "get_level_members"
                }
            ]
        }

        values = [simplejson.dumps(cm.id)] # auto configured cm.

        controller = ConfigurableWidgetController(self.session, account_id=self.account.id)
        options = controller.get_dimension_options(datasource, cube_name, config, values)
        members = options['Calculated members']
        self.assertEqual(len(members), 2) # one for the selected value and one for the available CM.
        self.assertNotEqual(members[0]['value'], members[1]['value'])

# below is for running the test from eclipse
if __name__ == '__main__':
    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_compose.py', 'iqportal.iqube.test.unit.test_compose.TestCase.test_save_simple_named_set']
    run()
    sys.exit()


