# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

from django.conf import settings

from sqlalchemy.orm.exc import NoResultFound

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.core.exceptions import PermissionDeniedException
from iqportal.iqube.core.controllers import DataSourceController, IQubeRoleController,\
    IQubeRole, ItemResultController
from iqportal.iqube.core.models import DataSource


class DataSourceTestCase(iqportal_unittest.IQPortalTestCase):
    """
    """
    def setUp(self):
        """
        """
        self.connect_db()

        account = self._create_active_account()
        self.group = account.groups[0].group

        self._attach_permissions_to_group(self.group, 'view_datasource')
        self.controller = DataSourceController(self.session, account_id=account.id)

        self.db = {"protocol": "mysql",
                   "user": "buildbot",
                   "password": "buildbot",
                   "host": "testorix.dev.ams",
                   "port": "3306",
                   "db_name": "iqportal_buildbot_unit"}


        # delete any existing datasources for this module
        try:
            ds = self.session.query(DataSource).filter(DataSource.name=='%s_test' % settings.SETTINGS_MODULE).one()
        except NoResultFound:
            pass
        else:
            self.session.delete(ds)
            self.session.flush()


    def test_save_or_update(self):
        """
        """
        name = '%s_test' % settings.SETTINGS_MODULE
        descr = name + " descr"

        self.assertRaises(PermissionDeniedException, self.controller.save_or_update_datasource,
                          None, name, descr, name,
                          self.db["protocol"], self.db["user"], self.db["password"], self.db["host"], \
                          self.db["port"], self.db["db_name"])

        self._attach_permissions_to_group(self.group, 'create_datasource', 'view_datasource')

        ds = self.controller.save_or_update_datasource(None, name, descr, name, \
                        self.db["protocol"], self.db["user"], self.db["password"], self.db["host"], \
                        self.db["port"], self.db["db_name"])
        self.assertEqual(ds.name, name)
        self.assertEqual(ds.mondrian_project, name)

    def test_get_datasources(self):
        """
        """
        self._attach_permissions_to_group(self.group, 'create_datasource', 'view_datasource')

        name = '%s_test' % settings.SETTINGS_MODULE
        descr = name + " descr"

        ds = self.controller.save_or_update_datasource(None, name, descr, name, \
                        self.db["protocol"], self.db["user"], self.db["password"], self.db["host"], \
                        self.db["port"], self.db["db_name"])

        ds2 = self.controller.get_datasources(ds.id).pop()
        self.assertEqual(ds, ds2)

    def test_load_datasource_state(self):
        """
        """
        self._attach_permissions_to_group(self.group, 'create_datasource', 'view_datasource')

        name = '%s_test' % settings.SETTINGS_MODULE
        descr = name + " descr"

        ds = self.controller.save_or_update_datasource(None, name, descr, name, \
                        self.db["protocol"], self.db["user"], self.db["password"], self.db["host"], \
                        self.db["port"], self.db["db_name"])

        ds2 = self.controller.load_datasource_state(ds.id)
        self.assertEqual(ds2['name'], name)
        self.assertEqual(ds2['protocol'], self.db["protocol"])
        self.assertEqual(ds2['user'], self.db["user"])
        self.assertEqual(ds2['password'], self.db["password"])
        self.assertEqual(ds2['host'], self.db["host"])
        self.assertEqual(ds2['port'], self.db["port"])
        self.assertEqual(ds2['database'], self.db["db_name"])

    def test_connection(self):
        """
        """
        from sqlalchemy.exc import OperationalError
        self.assertRaises(OperationalError, self.controller.test_connection, 'mysql', 'something', 'else', 'somehost', '3306', 'db')

        self.controller.test_connection(self.db["protocol"], self.db["user"], self.db["password"], self.db["host"], \
                                            self.db["port"], self.db["db_name"])


class DataSourceRoleTestCase(iqportal_unittest.IQPortalTestCase):
    def setUp(self):
        self.connect_db()

        self.account = self._create_active_account()
        self.group = self.account.groups[0].group
        self._attach_permissions_to_group(self.group, 'view_datasource')

    def test_simple_policy(self):
        ds = self.session.query(DataSource).filter(DataSource.name=='%s.IQuest' % settings.SETTINGS_MODULE).first()
        rc = IQubeRoleController(self.sa_session, self.account, ds, 'Response')

        rc.grant_cube(IQubeRole.ACCESS_ALL)
        rc.grant_hierarchy('[Survey]', IQubeRole.ACCESS_CUSTOM, rollup_policy=IQubeRole.ROLLUP_POLICY_PARTIAL)
        rc.grant_member('[Survey].[10]', IQubeRole.ACCESS_NONE)
        rc.grant_member('[Survey].[11]', IQubeRole.ACCESS_ALL)
        rc.grant_member('[Survey].[11].[1]', IQubeRole.ACCESS_NONE)

        dsrs = rc.to_list()

        self.assertEqual(len(dsrs), 5)
        self.assertEqual(dsrs[1]['access'], IQubeRole.ACCESS_CUSTOM)
        self.assertEqual(dsrs[1]['rollup_policy'], IQubeRole.ROLLUP_POLICY_PARTIAL)
        self.assertEqual(dsrs[2]['unique_name'], '[Survey].[10]')
        self.assertEqual(dsrs[2]['access'], IQubeRole.ACCESS_NONE)
        self.assertEqual(dsrs[3]['unique_name'], '[Survey].[11]')
        self.assertEqual(dsrs[3]['access'], IQubeRole.ACCESS_ALL)
        self.assertEqual(dsrs[4]['unique_name'], '[Survey].[11].[1]')
        self.assertEqual(dsrs[4]['access'], IQubeRole.ACCESS_NONE)

class ItemResultTestCase(iqportal_unittest.IQPortalTestCase):

    def setUp(self):
        self.connect_db()

        self.account = self._create_active_account()
        self.group = self.account.groups[0].group
        self._attach_permissions_to_group(self.group, 'view_datasource')


    def test_dynamic_configurable_widget_config(self):
        ic = ItemResultController(self.sa_session, account_id=self.account.id)

        ds = self.session.query(DataSource).filter(DataSource.name=='%s.FoodMart' % settings.SETTINGS_MODULE).one()
        result = ic.make_configurable_widget_config(ds.id, 'Sales')

        config = result['config']
        applied_filters = result['applied_filters']

        self.assertEqual(len(config['dimensions']), 13)

        self.assertEqual(applied_filters['dimension_12'], [u'{"dimension_unique_name": "[Measures]", "member_unique_name": "[Measures].[Unit Sales]", "depth": "0", "level_unique_name": "[Measures].[MeasuresLevel]", "member_name": "Unit Sales", "hierarchy_unique_name": "[Measures]"}'])





