# -*- coding: utf-8 -*-

# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.

from datetime import datetime

from django.utils import simplejson
from django.conf import settings

from twisted.trial.unittest import SkipTest

from basesoft.web.contrib.account.models import Account
from basesoft.web.test.selenium.selenium_testcase import SeleniumTestCase
from basesoft.web.core.util.controllers import StateManager
from basesoft.web.contrib.contact.controllers import ContactController
from basesoft.web.core.jsonrpc.dispatcher import json_object_hook, JsonRpcJSONEncoder

from iqportal.client.test.controllers import FmeTestController
from iqportal.iquest.compose.controllers import ComposeController, PublicationStateController
from iqportal.iquest.core import schedule
from iqportal.iquest.core import models as iquest_models
from iqportal.iquest.response.controllers import ParticipantController

from iqportal.iqube.core.controllers import DataSourceController, ProjectController, MdxMethodController
from iqportal.iqube.core.models import DataSource, MdxMethod
from iqportal.iquest.report.schedule import AggregateResponsesTask
from iqportal.iquest.report.client import XMLAClient
from iqportal.iqube.compose.controllers import MdxCalculatedMemberController
from iqportal.iquest.core.models import Survey

class TestCase(SeleniumTestCase):
    """
    """

    def setUp(self):
        """
        """
        super(TestCase, self).setUp()

        self.connect_db()

        #create an account so we can check permission with i
        self.account = self._create_active_account()

        # create the necessary controllers
        self.group = self.account.groups[0].group

        host, port = settings.MONDRIAN_XMLA_SERVER
        self.mdx_client = XMLAClient(host, port, settings.MONDRIAN_XMLA_PATH, '%s.IQuest' % settings.SETTINGS_MODULE)
        if not self.mdx_client.test_connection():
            raise SkipTest('No Mondrian connection available')

        from basesoft.web.contrib.i18n import models as i18n_models
        self.locale = self.session.query(i18n_models.Locale).filter_by(locale='en_GB').one()

        self.datasource_controller = DataSourceController(self.session, account_id=self.account.id)

        self.compose_controller = ComposeController(self.session, self.locale, account_id=self.account.id)

        self.publication_controller = PublicationStateController(self.session, StateManager(self.locale, self.account.id, None))
        self.project_controller = ProjectController(self.session, account_id=self.account.id)

        self.mdx_controller = MdxMethodController(self.session, account_id = self.account.id)
        # we create a controller to grant privs
        self.controller = MdxCalculatedMemberController(self.sa_session, account_id=self.account.id)

        # set permissions for the user before login
        self._set_permissions()

        c = FmeTestController(self.sa_session, account_id=self.sa_session.query(Account).filter(Account.name=='sysadmin').one().id)
        c.grant_configurable_widget_privs(self.group)

    def test_decimals_to_format_string_backwards_compatibility(self):
        """
        This tests the compatibility between the original "number of decimals" fields for the Measures settings
        versus the new setup of Format Strings.
        """
        survey_id = self.create_survey()

        publication_id = self.publish_survey(survey_id, anonymous=True)
        self.save_response(survey_id, publication_id, answer_q1='yes', answer_q2='yes')
        self.save_response(survey_id, publication_id, answer_q1='no', answer_q2='yes')
        self.save_response(survey_id, publication_id, answer_q1='no', answer_q2='no')

        name = '%s.IQuest ' % settings.SETTINGS_MODULE

        ds = self.sa_session.query(DataSource).filter(DataSource.name==name).one()

        self.project_controller.flush_schema(ds.id)
        agg = AggregateResponsesTask()

        # run the aggregators for the new responses
        agg.populate()

        # login as that account
        self.login(self.account.name, 'test_password')
        self.wait_for_loading()

        #click IQ Report icon
        self.click_iconbar('icon-iqreport')
        self.wait_for_loading()
        widget = 'IQ Report - IQuest'
        #open pallete for widgets
        self.click_palette(widget, palette_index=2)
        #drag widget to dashboard
        self.drag_item_from_palette('Default QA Item', block_index=0, column_index=0)
        self.wait_for_loading()
        #select survey, publication and QA from the widget
        self.fill_form(panel_name='config-widget-panel', values= {'survey': ('select', 'My Response Survey')})
        self.wait_for_loading()

        self.fill_form(panel_name='config-widget-panel', values= {'publication': ('select', 'Latest publication')})
        self.wait_for_loading()

        self.fill_form(panel_name='config-widget-panel', values= {'qa': ('select', 'Do you like cows?')})
        self.wait_for_loading()

        # click Apply and close
        self.click_button(panel_name='config-widget-panel', text='Apply and close')
        self.wait_for_loading()

        # save the default QA in the report.
        self.click_palette('Save as..', palette_index=0)
        self.wait_for_loading()

        # save report with a name and click 'ok' button
        self.save_dialog_input('Decimals check')
        self.wait_for_loading()

        from iqportal.iqreport.compose.controllers import ComposeController as ReportComposeController

        rc = ReportComposeController(self.sa_session, account_id=self.account.id)
        report = rc.get_reports().pop()

        item = report.blocks[0].columns[0].items[0]

        # load the item from the database and manually change them to have decimal fields set.
        applied_config = simplejson.loads(item.applied_config, object_hook=json_object_hook)

        applied_config['show_respondents_measure'] = True
        applied_config['show_response_measure'] = True
        applied_config['show_average_measure'] = True
        applied_config['average_decimals'] = 1
        applied_config['response_decimals'] = 3

        # delete these entries for testing purposes:
        del applied_config['average_format_string']
        del applied_config['response_format_string']

        item.applied_config = simplejson.dumps(applied_config, ensure_ascii=False, cls=JsonRpcJSONEncoder)
        self.sa_session.add(item)
        self.sa_session.flush()

        # reload the report by navigating away and back
        self.click_iconbar('icon-home')
        self.wait_for_loading()

        self.click_iconbar('icon-iqreport')
        self.wait_for_loading()

        self.select_compose_module(block_index=0, column_index=0, item_index=0)

        self.wait_for_loading()
        # open the settings panel and open the Measures settings fieldset
        self.click_cube_table_icon(button_title='Settings', block_index=0, column_index=0)
        self.wait_for_loading()

        self.click_fieldset(panel_name='config-widget-panel', title='Measure selection')

        # assert that the correct measures are selected
        self.assert_form(panel_name='config-widget-panel', values={
            'show_respondents_measure': ('checkbox', 'true'),
            'show_response_measure': ('checkbox', 'true'),
            'show_average_measure': ('checkbox', 'true'),
        })

        # verify that the Format String for the selected measures have the correct number of decimals set.
        self.assert_form(panel_name='config-widget-panel', values={
            'average_format_string': '#,##0.0',
            'response_format_string': '#,##0.000',
        })

    def test_default_qa_item(self):

        survey_id = self.create_survey()

        publication_id = self.publish_survey(survey_id, anonymous=True)
        self.save_response(survey_id, publication_id, answer_q1='yes', answer_q2='yes')
        self.save_response(survey_id, publication_id, answer_q1='no', answer_q2='yes')
        self.save_response(survey_id, publication_id, answer_q1='no', answer_q2='no')

        name = '%s.IQuest ' % settings.SETTINGS_MODULE

        ds = self.sa_session.query(DataSource).filter(DataSource.name==name).one()

        self.project_controller.flush_schema(ds.id)
        agg = AggregateResponsesTask()

        # run the aggregators for the new responses
        agg.populate()

        # login as that account
        self.login(self.account.name, 'test_password')
        self.wait_for_loading()

        #click IQ Report icon
        self.click_iconbar('icon-iqreport')
        self.wait_for_loading()
        widget = 'IQ Report - IQuest'
        #open pallete for widgets
        self.click_palette(widget, palette_index=2)
        #drag widget to dashboard
        self.drag_item_from_palette('Default QA Item', block_index=0, column_index=0)
        self.wait_for_loading()
        #select survey, publication and QA from the widget
        self.fill_form(panel_name='config-widget-panel', values= {'survey': ('select', 'My Response Survey')})
        self.wait_for_loading()

        self.fill_form(panel_name='config-widget-panel', values= {'publication': ('select', 'Latest publication')})
        self.wait_for_loading()

        self.fill_form(panel_name='config-widget-panel', values= {'qa': ('select', 'Do you like cows?')})
        self.wait_for_loading()

        # click Apply and close
        self.click_button(panel_name='config-widget-panel', text='Apply and close')
        self.wait_for_loading()

        #assert table data
        self.assert_cube_table(1, ['1','2'], 'table.basesoft-cubetable', col_index=2)

    def test_iquest_report_mdx_calculated_measure(self):
        self._attach_permissions_to_group(self.group, 'view_calculated_member', 'create_calculated_member')

        controller = MdxCalculatedMemberController(self.sa_session, account_id=self.account.id)

        state = {
            'objectName': 'MdxOperatorItem',
            'loaderName': 'iqube_mdx_operator_item',
            'return_type': 'numeric',
            'operator': '*', # multiply operator
            'inputs': [None, None]
        }
        # create new CM (aggregate)
        cm = controller.save_or_update_calculated_member(None, 'Something silly', state)

        survey_id = self.create_survey()

        publication_id = self.publish_survey(survey_id, anonymous=True)
        self.save_response(survey_id, publication_id, answer_q1='yes', answer_q2='yes')
        self.save_response(survey_id, publication_id, answer_q1='no', answer_q2='yes')
        self.save_response(survey_id, publication_id, answer_q1='no', answer_q2='no')

        name = '%s.IQuest ' % settings.SETTINGS_MODULE

        ds = self.sa_session.query(DataSource).filter(DataSource.name==name).one()

        self.project_controller.flush_schema(ds.id)
        agg = AggregateResponsesTask()

        # run the aggregators for the new responses
        agg.populate()

        # login as that account
        self.login(self.account.name, 'test_password')
        self.wait_for_loading()

        self.expand_layout_pane(0)

        # set mode to compose
        self.click_palette('Mode')
        self.click_palette('Compose')
        self.wait_for_loading()

        widget = 'IQ Report - IQuest'
        #open pallete for widgets
        self.click_palette(widget, palette_index=2)
        #drag widget to dashboard
        self.drag_item_from_palette('Default QA Item', block_index=0, column_index=0)
        self.wait_for_loading()
        #select survey, publication and QA from the widget
        self.fill_form(panel_name='config-widget-panel', values= {'survey': ('select', 'My Response Survey')})
        self.wait_for_loading()

        self.fill_form(panel_name='config-widget-panel', values= {'publication': ('select', 'Latest publication')})
        self.wait_for_loading()

        self.fill_form(panel_name='config-widget-panel', values= {'qa': ('select', 'Do you like cows?')})
        self.wait_for_loading()

        self.click_fieldset(panel_name='config-widget-panel', title='Measure selection')
        self.click_fieldset(panel_name='config-widget-panel', title='Calculated members')
        self.wait_for_loading()

        self.fill_form(panel_name='config-widget-panel', values={
            'calculated_measures': ('fancy-select', ('Something silly', ))
        })
        self.click_button_by_id('cm-configure-calculated_measures')
        self.wait_for_loading()

        self.fill_form(panel_name='mdx-formula-wizard', form_name='datasource_page', values={
            'axis': ('select', 'Columns'),# TODO: this should be unavailable in this case!
            'label': 'My modified label'
        })

        self.click_button(panel_name='mdx-formula-wizard', text=u'Next »')
        self.wait_for_loading()

        self.fill_form(panel_name='mdx-formula-wizard', form_name='measure_expression_page_0', values={
            'selection': ('select', 'Respondents')
        })
        self.click_button(panel_name='mdx-formula-wizard', text=u'Next »')
        self.wait_for_loading()

        self.fill_form(panel_name='mdx-formula-wizard', form_name='measure_expression_page_1', values={
            'selection': ('select', 'Companies')
        })
        self.click_button(panel_name='mdx-formula-wizard', text=u'Finish')
        self.wait_for_loading()

        # click Apply and close
        self.click_button(panel_name='config-widget-panel', text='Apply and close')
        self.wait_for_loading()

        #assert table data (the companies are 0 with the test data and something*0 is always 0 so thats why every other column 0
        self.assert_cube_table(1, ['1','0','2','0'], 'table.basesoft-cubetable', col_index=2)

    def test_iquest_report_mdx_calculated_member_compose(self):
        # this test is just to use the IQube compose in conjuction with the Datasource set items.

        self._attach_permissions_to_group(self.group, 'use_raw_mdx', 'view_calculated_member', 'create_calculated_member')

        # we create a controller to grant privs
        controller = MdxCalculatedMemberController(self.sa_session, account_id=self.account.id)
        # we want topcount
        top_count = self.sa_session.query(MdxMethod).filter(MdxMethod.name == 'TopCount').first()
        controller.grant_privilege_to(top_count, MdxMethod.PRIV_VIEW, group=True, target_group=self.group, check_own_privs=False)

        survey_id = self.create_survey()

        publication_id = self.publish_survey(survey_id, anonymous=True)
        self.save_response(survey_id, publication_id, answer_q1='yes', answer_q2='yes')
        self.save_response(survey_id, publication_id, answer_q1='yes', answer_q2='no')
        self.save_response(survey_id, publication_id, answer_q1='yes', answer_q2='no')
        self.save_response(survey_id, publication_id, answer_q1='no', answer_q2='no')

        name = '%s.IQuest ' % settings.SETTINGS_MODULE

        ds = self.sa_session.query(DataSource).filter(DataSource.name==name).one()

        survey = self.sa_session.query(Survey).filter(Survey.id==survey_id).one()
        item = survey.itemQAs[0]
        item_id = item.id
        questions = item.questions
        qid1 = questions[0].id
        qid2 = questions[1].id

        self.project_controller.flush_schema(ds.id)
        agg = AggregateResponsesTask()

        # run the aggregators for the new responses
        agg.populate()

        # login as that account
        self.login(self.account.name, 'test_password')
        self.wait_for_loading()

        # open the compose
        self.click_iconbar('mdx-calculated-member-designer', 'icon-datasource-manager')
        self.wait_for_loading(stable_time=2)

        # collapse actions
        self.click_palette_title('Actions')

        self.click_palette('Set functions', 2)

        # open datasource items
        self.click_palette('Datasource items', 2)

        # drag top count
        self.drag_item_from_palette_by_offset('Top count', xoffset=400, yoffset=300)
        self.wait_for_loading()

        # drag set
        self.drag_item_from_palette_by_offset('Set expression', xoffset=200, yoffset=100)
        self.wait_for_loading()

        self.click_button(panel_name='dialog-question', text='Yes')
        self.wait_for_loading()

        self.fill_form(panel_name='iqube-datasource-cube-panel', values={
            'datasource_id': ('select', 'value=%d' % ds.id)
        })
        self.wait_for_loading()

        self.fill_form(panel_name='iqube-datasource-cube-panel', values={
            'cube': ('select', 'VirtualResponse')
        })

        self.click_button(panel_name='iqube-datasource-cube-panel', text='Continue')
        self.wait_for_loading()

        self.fill_form(panel_name='iqube-compose-datasource-constraints', values={
            'survey': ('select', 'value=%d' % survey_id)
        })
        self.click_button(panel_name='iqube-compose-datasource-constraints', text='Continue')
        self.wait_for_loading()

        # connect the terminals
        self.connect_jsplumb_terminals(5,2)

        target = "//div[contains(@class, 'compose-container-items')]/div[3]/div[contains(@class, 'hd')]//span[contains(@class, 'configure')]"
        self._driver.find_element_by_xpath(target).click()
        self.wait_for_loading()

        # select questions
        self.fill_form(panel_name='iqube-mdx-expression-selection', values={
            'dimension': ('select', 'Question')
        })
        self.wait_for_loading()

        # select a few questions
        self.fill_form(panel_name='iqube-mdx-expression-selection', values={
            'selection': ('fancy-select', ('Like cows?', 'Have many cows?'))
        })
        self.click_button(panel_name='iqube-mdx-expression-selection', text='Use selection')

        self.click_palette('Save', 0)
        self.wait_for_loading()
        self.save_dialog_input('IQuest Top Count')
        self.wait_for_loading()

        mcmc = MdxCalculatedMemberController(self.sa_session, account_id=self.account.id)
        cm = mcmc.get_calculated_member().pop()

        self.assertEqual(simplejson.loads(cm.constraints)['survey'], str(survey_id))
        self.assertEqual(mcmc.get_parsed(cm.id), 'TopCount({[Question].[%d].[%d].[%d],[Question].[%d].[%d].[%d]},$var,$var)' % (
            item_id, 1, qid1,
            item_id, 1, qid2
        ))


    # test "Multi QA Item" widget
    def test_multi_qa_item(self):
        # set permissions for the user before login
        self._set_permissions()

        survey_id = self.create_survey()

        publication1_id = self.publish_survey(survey_id, anonymous=True)
        publication2_id = self.publish_survey(survey_id, anonymous=True, label='Publication 2')

        # populate data / responses
        self._populate_responses(survey_id, publication1_id, publication2_id)

        # flush mondrian results
        name = '%s.IQuest ' % settings.SETTINGS_MODULE

        ds = self.sa_session.query(DataSource).filter(DataSource.name==name).one()

        agg = AggregateResponsesTask()
        self.project_controller.flush_schema(ds.id)

        # run the aggregators for the new responses
        agg.populate()

        # login as that account
        self.login(self.account.name, 'test_password')

        # open palette panel
        self.expand_layout_pane(0)

        # set mode to compose
        self.click_palette('Mode')
        self.click_palette('Compose')
        self.wait_for_loading()

        # select "Multi QA Item" and drag to dashboard
        self.click_palette('IQ Report - IQuest', palette_index=2)
        self.drag_item_from_palette('Multi QA Item', block_index=0, column_index=0)
        self.wait_for_loading(times=2)

        # select "Table settings"
        self.click_fieldset(panel_name='config-widget-panel', title='Table settings')

        # select checkbox "Row colors"
        self.fill_form(panel_name='config-widget-panel', values= {
            'rowColors': ('checkbox', 'rowColors')
        })

        # select survey in "filter panel" -> Presets > Latest publucation > select
        self.fill_form(panel_name='config-widget-panel', values= {
            'survey': ('select', 'My Response Survey')
        })
        self.wait_for_loading()

        # select "Publications" -> wait until selected
        self.fill_form(panel_name='config-widget-panel', values= {
            'publication': ('fancy-select', ['publication 1'])
        })
        self.wait_for_loading()

        # select "Select QA" -> wait until selected > 1. Handel (in/verk..... > select button
        self.fill_form(panel_name='config-widget-panel', values= {
            'qa': ('fancy-select', ['Do you like cows?'])
        })
        self.wait_for_loading()

        # select "Questions" -> wait until selected > select all button
        self.fill_form(panel_name='config-widget-panel', values= {
            'question': ('fancy-select', ['Like cows?'])
        })
        self.wait_for_loading()

        # select "Answers" -> wait until selected > Order intake kEuro ... & Waarvan Nederand
        self.fill_form(panel_name='config-widget-panel', values= {
            'answer': ('fancy-select', ['Yes', 'No.. bleh'])
        })
        self.wait_for_loading()

        # click Apply and close
        self.click_button(panel_name='config-widget-panel', text='Apply and close')
        self.wait_for_loading()

        # assert table data of Multi QA Item table
        self.assert_cube_table(1, [3, 2], 'table.basesoft-cubetable', col_index=3)


    def test_multi_qa_item_reload_select_as_group(self):
        """
        Test to verify that Select as Group properly loads in the fancy select. (#4520)
        """
        # set permissions for the user before login
        self._set_permissions()

        survey_id = self.create_survey()

        publication1_id = self.publish_survey(survey_id, anonymous=True)
        publication2_id = self.publish_survey(survey_id, anonymous=True, label='Publication 2')

        # populate data / responses
        self._populate_responses(survey_id, publication1_id, publication2_id)

        # flush mondrian results
        name = '%s.IQuest ' % settings.SETTINGS_MODULE

        ds = self.sa_session.query(DataSource).filter(DataSource.name==name).one()

        agg = AggregateResponsesTask()
        self.project_controller.flush_schema(ds.id)

        # run the aggregators for the new responses
        agg.populate()

        # login as that account
        self.login(self.account.name, 'test_password')

        self.click_iconbar('icon-iqreport')
        self.wait_for_loading()

        # select "Multi QA Item" and drag to dashboard
        self.click_palette('IQ Report - IQuest', palette_index=2)
        self.drag_item_from_palette('Multi QA Item', block_index=0, column_index=0)
        self.wait_for_loading(times=2)

        # select "Table settings"
        self.click_fieldset(panel_name='config-widget-panel', title='Table settings')

        # select checkbox "Row colors"
        self.fill_form(panel_name='config-widget-panel', values= {
            'rowColors': ('checkbox', 'rowColors')
        })

        # select survey in "filter panel" -> Presets > Latest publucation > select
        self.fill_form(panel_name='config-widget-panel', values= {
            'survey': ('select', 'My Response Survey')
        })
        self.wait_for_loading()

        # select "Publications" -> wait until selected
        self.fill_form(panel_name='config-widget-panel', values= {
            'publication': ('fancy-select', ['publication 1'])
        })
        self.wait_for_loading()

        # select "Select QA" -> wait until selected > 1. Handel (in/verk..... > select button
        self.fill_form(panel_name='config-widget-panel', values= {
            'qa': ('fancy-select', ['Do you like cows?'])
        })
        self.wait_for_loading()

        # select "Questions" -> wait until selected > select all button
        self.fill_form(panel_name='config-widget-panel', values= {
            'question': ('fancy-select', ['Like cows?'])
        })
        self.wait_for_loading()

        self.select_as_group(panel_name='config-widget-panel', el_name='answer', values=['Yes'], name='testgroup')
        self.wait_for_loading()

        # click Apply and close
        self.click_button(panel_name='config-widget-panel', text='Apply and close')
        self.wait_for_loading()

        # assert table data of Multi QA Item table
        self.assert_cube_table(1, [3], 'table.basesoft-cubetable', col_index=3)

        self.click_palette('Save as..', palette_index=0)
        self.wait_for_loading()

        # save form with a name and click 'ok' button
        self.save_dialog_input('My Report Yo!')
        self.wait_for_loading()

        self.click_iconbar('icon-home')
        self.wait_for_loading()

        self.click_iconbar('icon-iqreport') # go back for reload of report
        self.wait_for_loading()

        self.select_compose_module(block_index=0, column_index=0, item_index=0)
        self.wait_for_loading()
        # open the settings panel and open the Measures settings fieldset
        self.click_cube_table_icon(button_title='Settings', block_index=0, column_index=0)
        self.wait_for_loading()

        self.fill_form(panel_name='config-widget-panel', values= {
            'answer': ('fancy-select', ['testgroup'])
        }, assert_=True)

    # test minimum respondents given on a Multi QA Item table (3 is shown / 2 is not shown)
    def test_multi_qa_item_minimum_respondents(self):
        # set permissions for the user before login
        self._set_permissions()

        survey_id = self.create_survey()

        publication1_id = self.publish_survey(survey_id, anonymous=True, minimum_respondents=3)

        # populate data / responses
        self._populate_responses(survey_id, publication1_id)

        # flush mondrian results
        name = '%s.IQuest ' % settings.SETTINGS_MODULE

        ds = self.sa_session.query(DataSource).filter(DataSource.name==name).one()

        agg = AggregateResponsesTask()
        self.project_controller.flush_schema(ds.id)

        # run the aggregators for the new responses
        agg.populate()

        # login as that account
        self.login(self.account.name, 'test_password')

        # open palette panel
        self.expand_layout_pane(0)

        # set mode to compose
        self.click_palette('Mode')
        self.click_palette('Compose')
        self.wait_for_loading()

        # select "Multi QA Item" and drag to dashboard
        self.click_palette('IQ Report - IQuest', palette_index=2)
        self.drag_item_from_palette('Multi QA Item', block_index=0, column_index=0)
        self.wait_for_loading(times=2)

        # select "Table settings"
        self.click_fieldset(panel_name='config-widget-panel', title='Table settings')

        # select checkbox "Row colors"
        self.fill_form(panel_name='config-widget-panel', values= {
            'rowColors': ('checkbox', 'rowColors')
        })

        # select survey in "filter panel" -> Presets > Latest publucation > select
        self.fill_form(panel_name='config-widget-panel', values= {
            'survey': ('select', 'My Response Survey')
        })
        self.wait_for_loading()

        # select "Publications" -> wait until selected
        self.fill_form(panel_name='config-widget-panel', values= {
            'publication': ('fancy-select', ['publication 1'])
        })
        self.wait_for_loading()

        # select "Select QA" -> wait until selected > 1. Handel (in/verk..... > select button
        self.fill_form(panel_name='config-widget-panel', values= {
            'qa': ('fancy-select', ['Do you like cows?'])
        })
        self.wait_for_loading()

        # select "Questions" -> wait until selected > select all button
        self.fill_form(panel_name='config-widget-panel', values= {
            'question': ('fancy-select', ['Like cows?'])
        })
        self.wait_for_loading()

        # select "Answers" -> wait until selected > Order intake kEuro ... & Waarvan Nederand
        self.fill_form(panel_name='config-widget-panel', values= {
            'answer': ('fancy-select', ['Yes', 'No.. bleh'])
        })
        self.wait_for_loading()

        # click Apply and close
        self.click_button(panel_name='config-widget-panel', text='Apply and close')
        self.wait_for_loading()

        # click on "review with filtered data" button
        self.click_cube_table_icon('Review with filtered data', block_index=0, column_index=0, item_index=0)
        self.wait_for_loading()

        # assert table data of Multi QA Item table
        self.assert_cube_table(1, [3,'-'], 'table.basesoft-cubetable', col_index=3)

    # test percentage maximum market share given on a Multi QA Item table
    def test_multi_qa_item_maximum_market_share(self):
        # set permissions for the user before login
        self._set_permissions()

        survey_id = self.create_survey()

        publication1_id = self.publish_survey(survey_id, anonymous=True, maximum_market_share=18)

        # populate data / responses
        self._populate_responses(survey_id, publication1_id)

        # flush mondrian results
        name = '%s.IQuest ' % settings.SETTINGS_MODULE

        ds = self.sa_session.query(DataSource).filter(DataSource.name==name).one()

        agg = AggregateResponsesTask()
        self.project_controller.flush_schema(ds.id)

        # run the aggregators for the new responses
        agg.populate()

        # login as that account
        self.login(self.account.name, 'test_password')

        # open palette panel
        self.expand_layout_pane(0)

        # set mode to compose
        self.click_palette('Mode')
        self.click_palette('Compose')
        self.wait_for_loading()

        # select "Multi QA Item" and drag to dashboard
        self.click_palette('IQ Report - IQuest', palette_index=2)
        self.drag_item_from_palette('Multi QA Item', block_index=0, column_index=0)
        self.wait_for_loading(times=2)

        # select survey in "filter panel" -> Presets > Latest publucation > select
        self.fill_form(panel_name='config-widget-panel', values= {
            'survey': ('select', 'My Response Survey')
        })
        self.wait_for_loading()

        # select "Publications" -> wait until selected
        self.fill_form(panel_name='config-widget-panel', values= {
            'publication': ('fancy-select', ['publication 1'])
        })
        self.wait_for_loading()

        # select "Select QA" -> wait until selected > 1. Handel (in/verk..... > select button
        self.fill_form(panel_name='config-widget-panel', values= {
            'qa': ('fancy-select', ['But how many cows do you have?'])
        })
        self.wait_for_loading()

        # select "Questions" -> wait until selected > select all button
        self.fill_form(panel_name='config-widget-panel', values= {
            'question': ('fancy-select', ['-'])
        })
        self.wait_for_loading()

        # select "Answers" -> wait until selected > Order intake kEuro ... & Waarvan Nederand
        self.fill_form(panel_name='config-widget-panel', values= {
            'answer': ('fancy-select', ['Number of cows'])
        })
        self.wait_for_loading()

        # click Apply and close
        self.click_button(panel_name='config-widget-panel', text='Apply and close')
        self.wait_for_loading()

        # click on "review with filtered data" button
        self.click_cube_table_icon('Review with filtered data', block_index=0, column_index=0, item_index=0)
        self.wait_for_loading()

        # assert table data of Multi QA Item table
        self.assert_cube_table(1, [''], 'table.basesoft-cubetable', col_index=3)

    # select individual preferences to get results out of Multi QA Items
    def test_multi_qa_item_show_individual_measure(self):
        # set permissions for the user before login
        self._set_permissions()

        survey_id = self.create_survey()
        cl = self.set_up_contactlists()
        publication1_id = self.publish_survey(survey_id, anonymous=False, contact_lists=[cl.id])

        # populate data / responses
        publication = self.session.query(iquest_models.SurveyPublication).get(publication1_id)
        self.assertEqual(len(publication.invitees), 2)

        invitee_1 = publication.invitees[0]
        invitee_2 = publication.invitees[1]

        # save invitee 1
        self.save_response(survey_id, publication1_id, answer_q3=500, invitee_hash=invitee_1.invitee_hash)

        # save invitee 2
        self.save_response(survey_id, publication1_id, answer_q3=800, invitee_hash=invitee_2.invitee_hash)

        # flush mondrian results
        name = '%s.IQuest ' % settings.SETTINGS_MODULE

        ds = self.sa_session.query(DataSource).filter(DataSource.name==name).one()

        agg = AggregateResponsesTask()
        self.project_controller.flush_schema(ds.id)

        # run the aggregators for the new responses
        agg.populate()

        # in Multi QA Item widget show field "individual selections" in "Measure selection" field
        # added "create_individual_reports" to "this._attach_permissions_to_group" shows the appropiate field

        # login as that account
        self.login(self.account.name, 'test_password')

        # click on IQReport iconbar
        self.click_iconbar('icon-iqreport')
        #self.click_menu('iqreport', text='IQReport')
        self.wait_for_loading()

        # open palette panel
        self.expand_layout_pane(0)

        # select "Multi QA Item" and drag to dashboard
        self.click_palette('IQ Report - IQuest', palette_index=2)
        self.drag_item_from_palette('Multi QA Item', block_index=0, column_index=0)
        self.wait_for_loading(times=2)

        # select survey in "filter panel" -> Presets > Latest publucation > select
        self.fill_form(panel_name='config-widget-panel', values= {
            'survey': ('select', 'My Response Survey')
        })
        self.wait_for_loading()

        # select "Publications" -> wait until selected
        self.fill_form(panel_name='config-widget-panel', values= {
            'publication': ('fancy-select', ['publication 1'])
        })
        self.wait_for_loading()

        # select "Select QA" -> wait until selected > 1. Handel (in/verk..... > select button
        self.fill_form(panel_name='config-widget-panel', values= {
            'qa': ('fancy-select', ['But how many cows do you have?'])
        })
        self.wait_for_loading()

        # select "Questions" -> wait until selected > select all button
        self.fill_form(panel_name='config-widget-panel', values= {
            'question': ('fancy-select', ['-'])
        })
        self.wait_for_loading()

        # select "Answers" -> wait until selected > Order intake kEuro ... & Waarvan Nederand
        self.fill_form(panel_name='config-widget-panel', values= {
            'answer': ('fancy-select', ['Number of cows'])
        })
        self.wait_for_loading()

        # select "Individual response" on "Measure selection" field
        self.click_fieldset(panel_name='config-widget-panel', title='Measure selection')

        # select checkbox "Individual response"
        self.fill_form(panel_name='config-widget-panel', values= {
            'show_individual_measure': ('checkbox', 'show_individual_measure')
        })

        # click Apply and close
        self.click_button(panel_name='config-widget-panel', text='Apply and close')
        self.wait_for_loading()

        # save report and asign it a name from the actions panel
        # open palette panel
        self.expand_layout_pane(0)

        self.click_palette('Save as..', palette_index=0)
        self.wait_for_loading()

        # save form with a name and click 'ok' button
        self.save_dialog_input('Saved Form Name')
        self.wait_for_loading()

        # click "Preview run" to open IQPortal Report
        self.click_palette('Preview run', palette_index=0)
        self.wait_for_loading()

        # click "Report preview tools"
        self.click_fieldset(panel_name='report-panel', title='Report preview tools')
        self.wait_for_loading()

        # select "Contact list" -> wait until selected
        self.fill_form(panel_name='report-panel', values={
            'contact_list': ('select', 'Contact List 1')
        })
        self.wait_for_loading()

        # select "Company uno" -> wait until selected
        self.fill_form(panel_name='report-panel', values={
            'company': ('select', 'Company uno')
        })

        #switch to frame to be able to select the table data
        self._driver._switch_to.frame('iqreport-viewer-iframe')
        css = "body table.basesoft-cubetable tbody tr:nth-child(1) td:nth-child(4)"
        self.wait_for_selector(css)

        # check if individual result equals 500
        self.assert_cube_table(1, ['500'], 'table.basesoft-cubetable', col_index=4)

        #switch back to body to select another company
        self._driver._switch_to.default_content()

        # select "Company zwei" -> wait until selected
        self.fill_form(panel_name='report-panel', values={
            'company': ('select', 'Company zwei')
        })

        #back to frame
        self._driver._switch_to.frame('iqreport-viewer-iframe')
        self.wait_for_selector(css)

        # check if individual result equals 800
        self.assert_cube_table(1, ['800'], 'table.basesoft-cubetable', col_index=4)

    # run preview runner test
    def test_preview_runner(self):
        # set permissions for the user before login
        self._set_permissions()

        survey_id = self.create_survey()

        publication1_id = self.publish_survey(survey_id, anonymous=True)
        publication2_id = self.publish_survey(survey_id, anonymous=True, label='Publication 2')

        # populate data / responses
        self._populate_responses(survey_id, publication1_id, publication2_id)

        # flush mondrian results
        name = '%s.IQuest ' % settings.SETTINGS_MODULE

        ds = self.sa_session.query(DataSource).filter(DataSource.name==name).one()

        agg = AggregateResponsesTask()
        self.project_controller.flush_schema(ds.id)

        # run the aggregators for the new responses
        agg.populate()

        # login as that account
        self.login(self.account.name, 'test_password')

        # click on IQReport iconbar
        self.click_iconbar('icon-iqreport')
        self.wait_for_loading()

        # open Actions panel / Create a new Report
        self.click_palette('Create a new Report', palette_index=0)
        self.wait_for_loading()

        # select "Multi QA Item" and drag to dashboard
        self.click_palette('IQ Report - IQuest', palette_index=2)
        self.drag_item_from_palette('Multi QA Item', block_index=0, column_index=0)
        self.wait_for_loading()

        # select "Table settings"
        self.click_fieldset(panel_name='config-widget-panel', title='Table settings')

        # select checkbox "Row colors"
        self.fill_form(panel_name='config-widget-panel', values= {
            'rowColors': ('checkbox', 'rowColors')
        })

        # select survey in "filter panel" -> Presets > Latest publucation > select
        self.fill_form(panel_name='config-widget-panel', values= {
            'survey': ('select', 'My Response Survey')
        })
        self.wait_for_loading()

        # select "Publications" -> wait until selected
        self.fill_form(panel_name='config-widget-panel', values= {
            'publication': ('fancy-select', ['publication 1'])
        })
        self.wait_for_loading()

        # select "Select QA" -> wait until selected > 1. Handel (in/verk..... > select button
        self.fill_form(panel_name='config-widget-panel', values= {
            'qa': ('fancy-select', ['Do you like cows?'])
        })
        self.wait_for_loading()

        # select "Questions" -> wait until selected > select all button
        self.fill_form(panel_name='config-widget-panel', values= {
            'question': ('fancy-select', ['Like cows?'])
        })
        self.wait_for_loading()

        # select "Answers" -> wait until selected > Order intake kEuro ... & Waarvan Nederand
        self.fill_form(panel_name='config-widget-panel', values= {
            'answer': ('fancy-select', ['Yes', 'No.. bleh'])
        })
        self.wait_for_loading()

        # click Apply and close
        self.click_button(panel_name='config-widget-panel', text='Apply and close')
        self.wait_for_loading()

        # save report
        self.click_palette('Save as..', palette_index=0)
        self.wait_for_loading()

        # save report with a name and click 'ok' button
        self.save_dialog_input('Saved Report Name')
        self.wait_for_loading()

        # click "Preview run" to open IQPortal Report
        self.click_palette('Preview run', palette_index=0)
        self.wait_for_loading()

        # assert table data of Multi QA Item table
        self.assert_cube_table(1, [3, 2], 'table.basesoft-cubetable', col_index=3)

    def test_calculated_member_wizard_hook_page(self):
        # set permissions for the user before login
        self._set_permissions()
        # we want topcount
        top_count = self.sa_session.query(MdxMethod).filter(MdxMethod.name == 'TopCount').first()
        self.controller.grant_privilege_to(top_count, MdxMethod.PRIV_VIEW, group=True, target_group=self.group, check_own_privs=False)

        survey_id = self.create_survey()

        publication1_id = self.publish_survey(survey_id, anonymous=True)
        publication2_id = self.publish_survey(survey_id, anonymous=True)

        # populate data / responses
        self._populate_responses(survey_id, publication1_id, publication2_id)

        # flush mondrian results
        ds_name = '%s.IQuest' % settings.SETTINGS_MODULE

        ds = self.sa_session.query(DataSource).filter(DataSource.name==ds_name).one()

        agg = AggregateResponsesTask()
        self.project_controller.flush_schema(ds.id)

        # run the aggregators for the new responses
        agg.populate()

        # login as that account
        self.login(self.account.name, 'test_password')

        # click on CM Designer iconbar
        self.click_iconbar('mdx-calculated-member-designer', 'icon-datasource-manager')
        self.wait_for_loading()

        self.click_palette('Set functions', 2)

        # drag top count
        self.drag_item_from_palette_by_offset('Top count', xoffset=400, yoffset=300)
        self.wait_for_loading()

        #check if we can reselect the selected survey when we dont have predefined datasource
        self.click_palette('Preview', 0)

        self.fill_form(panel_name='mdx-formula-wizard', form_name='datasource_page', values={'datasource_id': ('select', ds_name)})
        self.wait_for_loading()

        self.fill_form(panel_name='mdx-formula-wizard', form_name='datasource_page', values={'cube': ('select', 'Response')})
        self.wait_for_loading()

        self.fill_form(panel_name='mdx-formula-wizard', form_name='measure_page', values={'selection': ('select', 'Response Responses Int Sum')})
        self.click_button(panel_name='mdx-formula-wizard', text=u'Next »')
        self.wait_for_loading()

        self.fill_form(panel_name='mdx-formula-wizard', values={'survey': ('select', 'My Response Survey')})
        self.click_button(panel_name='mdx-formula-wizard', text=u'Next »')
        self.wait_for_loading()
        #here we should be able to go back and select a survey again
        self.click_button(panel_name='mdx-formula-wizard', text=u'« Back')
        self.wait_for_loading()

        self.fill_form(panel_name='mdx-formula-wizard', values={'survey': ('select', 'My Response Survey')})
        self.click_button(panel_name='mdx-formula-wizard', text=u'Next »')
        self.wait_for_loading()
        self.click_button(panel_name='mdx-formula-wizard', text=u'Cancel')
        self.wait_for_loading()
        #then leave wizard and do the check with predefined datasource
        self.click_palette('Datasource items', 2)

        # drag datasource item
        self.drag_item_from_palette_by_offset('Set expression', xoffset=300, yoffset=70)

        self.click_button(panel_name='dialog-question', text='Yes')
        self.wait_for_loading()

        self.fill_form(panel_name='iqube-datasource-cube-panel', values={'datasource_id': ('select', ds_name)})
        self.click_button(panel_name='iqube-datasource-cube-panel', text='Continue')
        self.wait_for_loading()

        self.fill_form(panel_name='iqube-compose-datasource-constraints', values={'survey': ('select', 'My Response Survey')})
        self.click_button(panel_name='iqube-compose-datasource-constraints', text='Continue')
        self.wait_for_loading()


        # connect the 5th terminal created (4 from the Top Count + 1 for set expression source)
        self.connect_jsplumb_terminals(5,2)
        self.wait_for_loading()

        self.select_compose_module_floating(item_index=2)
        #configure the set expression
        self.click_palette('Configure', 1)
        self.wait_for_loading()

        self.fill_form(panel_name='iqube-mdx-expression-selection', values={'dimension': ('select', 'Answer')})
        self.wait_for_loading()

        self.fill_form(panel_name='iqube-mdx-expression-selection', values={'selection': ('fancy-select', ['Yes', 'No.. bleh', 'Little', 'Medium'])})
        self.click_button(panel_name='iqube-mdx-expression-selection', text='Use selection')
        self.wait_for_loading()

        self.click_palette('Preview', 0)
        self.wait_for_loading()

        self.fill_form(panel_name='mdx-formula-wizard', form_name='measure_page', values={'selection': ('select', 'FilteredResponse Responses Int Sum')})
        self.click_button(panel_name='mdx-formula-wizard', text=u'Next »')
        self.wait_for_loading()

        #here the hook page should have been skipped
        #so we can select a numeric input for the top count formula
        self.fill_form(panel_name='mdx-formula-wizard', form_name='numeric_expression_page_0', values={'selection': '3'})
        self.click_button(panel_name='mdx-formula-wizard', text=u'Next »')
        self.wait_for_loading()

        self.fill_form(panel_name='mdx-formula-wizard', form_name='measure_expression_page_1', values={'selection': ('select', 'FilteredResponse Responses Int Sum')})
        self.click_button(panel_name='mdx-formula-wizard', text=u'Finish')
        self.wait_for_loading()

    def create_survey(self):
        """
        Helper method for creating a survey state
        """

        state = {
            "survey": {
                "name": u"My Response Survey",
                "objectName": u"Survey",
                "dbID": None,
                "domID": u"iquest-canvas-dom-id",

                "availableLocales": ['en_GB'],

                "fallbackLocale": {'locale': u'en_GB', 'name': u'English' },

                "theme": {
                    "module": u"iquest_survey_theme_basesoft",
                    "name": u"BaseSoft",
                    "cssClass": u"survey-basesoft",
                },

                "children": {
                    "blocks": [

                        # First block contains only one column
                        {
                            "objectName": u"SurveyBlock",
                            "dbID":None,
                            "domID": u"yui-gen42",
                            "pageEnd": False,
                            "children": {
                                "columns": [{
                                    "children": {
                                        'items': []
                                    },
                                    "objectName": u"SurveyBlockColumn",
                                    "dbID": None,
                                    "domID": u"yui-gen44",
                                    "width":100
                                }],
                            }
                        },

                        {
                            "objectName":u"SurveyBlock",
                            "dbID":None,
                            "domID":u"yui-gen59",
                            "pageEnd": False,

                            "children": {
                                "columns":[{
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen61",
                                    "width":50,

                                    "children": {
                                        "items":[

                                        # first question is a simple single choice question
                                        {
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"qa-1",
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
                                            "trailingQuestions": False,
                                            "type": iquest_models.SurveyItemQA.TYPE_SINGLE_CHOICE,
                                            "layout": None,
                                            "showTitle": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],

                                            "locales": {
                                                'en_GB': {
                                                    u'label': u'Do you like cows?'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-1-1",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Like cows?'
                                                        }
                                                    },
                                                },{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-1-2",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Have many cows?'
                                                        }
                                                    },
                                                }],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-1-1",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Yes'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-1-2",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'No.. bleh'
                                                        }
                                                    },
                                                }]
                                            }
                                        },

                                        # second question is a matrix question
                                        {
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"qa-2",
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
                                            "trailingQuestions": False,
                                            "type": iquest_models.SurveyItemQA.TYPE_SINGLE_CHOICE,
                                            "layout": None,
                                            "showTitle": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],

                                            "locales": {
                                                'en_GB': {
                                                    u'label': u'Do you like ..?'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-2-1",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Dog'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-2-2",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Cat'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-2-3",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Cow'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-2-4",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Chicken'
                                                        }
                                                    },
                                                },
                                                ],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-2-1",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Little'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-2-2",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Medium'
                                                        }
                                                    },
                                                }, {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-2-3",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'A Lot'
                                                        }
                                                    },
                                                }]
                                            }
                                        },

                                        # third question is an numeric open question
                                        {
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"qa-3",
                                            "validateMarketShare": True,
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
                                            "trailingQuestions": False,
                                            "type": iquest_models.SurveyItemQA.TYPE_SINGLE_TEXT_INPUT,
                                            "layout": None,
                                            "showTitle": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],

                                            "locales": {
                                                'en_GB': {
                                                    u'label': u'But how many cows do you have?'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-3-1",
                                                    "locales": {},
                                                }],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-3-1",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Number of cows'
                                                        }
                                                    },
                                                },
                                            ]}
                                        },

                                        # fourth is float matrix
                                        {
                                            "objectName":u"SurveyItemQA",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"qa-4",
                                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
                                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
                                            "trailingQuestions": False,
                                            "type": iquest_models.SurveyItemQA.TYPE_SINGLE_TEXT_INPUT,
                                            "layout": None,
                                            "showTitle": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],

                                            "locales": {
                                                'en_GB': {
                                                    u'label': u'How much?'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-4-1",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Dog'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-4-2",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Cat'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-4-3",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Cow'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "domID":u"q-4-4",
                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Chicken'
                                                        }
                                                    },
                                                },
                                                ],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-4-1",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Yesterday'
                                                        }
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-4-2",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Today'
                                                        }
                                                    },
                                                }, {
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None,
                                                    "domID":u"a-4-3",

                                                    "locales": {
                                                        'en_GB': {
                                                            u'label': u'Tomorrow'
                                                        }
                                                    },
                                                }]
                                            }
                                        },

                                        ]
                                    }
                                }

                                ]
                            }
                        },
                    ]
                }
            }
        }

        # save the survey
        map = self.compose_controller.save_state(state)
        survey_id = map['iquest-canvas-dom-id']


        return survey_id

    def publish_survey(self, survey_id, status=None, begin_date=None, end_date=None, contact_lists=None, anonymous=False,
                       minimum_respondents=None, maximum_market_share=None, show_individual_measure=None, label="publication 1"):

        if contact_lists is None:
            contact_lists = []

        if status is None:
            status = iquest_models.SurveyPublication.STATUS_SCHEDULED

        if begin_date is None:
            begin_date = datetime.now()

        publish_state = {
            'survey_id': survey_id,
            "dbID": None,
            "domID": u'publish_1',
            "begin_date": begin_date,
            "end_date": end_date,
            "status": status,
            "label": label,
            "show_page_nrs": True,
            "anonymous": anonymous,
            "contact_lists": contact_lists,
            "viewer_print": False,
            "minimum_respondents": minimum_respondents,
            "maximum_market_share": maximum_market_share,
            "show_individual_measure": show_individual_measure,
            "mailings": [] # no mailings necessary
        }

        # save publication
        publication = self.publication_controller.save(publish_state)

        # publish the surveys like the scheduler would do
        schedule.PublishSurveysTask().execute()
        schedule.CreatePublicationInviteesTask().execute()

        self.session.expire(publication)
        return publication.id


    def save_response(self, survey_id, publication_id, answer_q1=None, answer_q2=None, answer_q3=None, answer_q4=None, invitee_hash=None):
        # get some vars
        survey = self.session.query(iquest_models.Survey).get(survey_id)

        publication = self.session.query(iquest_models.SurveyPublication).get(publication_id)
        assert publication.survey.id == survey_id

        # for now assume anonymous survey so get that hash
        invitee_hash = invitee_hash if invitee_hash is not None else publication.invitees[0].invitee_hash

        # init the controller
        controller = ParticipantController(self.session, self.locale, invitee_hash)

        # instanciate survey by invitee
        state = controller.load_survey_state()
        self.assertTrue(isinstance(state['survey']['dbID'], (long, int)))


        # q1 (do you like cows? yes/no)
        if answer_q1 is not None:
            responses = []
            q1_a_id = {'yes': survey.itemQAs[0].answers[0].id, 'no': survey.itemQAs[0].answers[1].id}[answer_q1]
            responses.append({'q': survey.itemQAs[0].questions[0].id, 'a': q1_a_id, 'v': 1})
            controller.update_responses([{'item_id': survey.itemQAs[0].id, 'responses': responses}])

        if answer_q2 is not None:

            responses = []
            for i in range(len(answer_q2)):
                for j in range(len(answer_q2[i])):

                    q_id = survey.itemQAs[1].questions[i].id
                    a_id = survey.itemQAs[1].answers[j].id

                    if answer_q2[i][j]:
                        responses.append({'q': q_id, 'a': a_id, 'v': 1})

            controller.update_responses([{'item_id': survey.itemQAs[1].id, 'responses': responses}])

        if answer_q3 is not None:
            responses = []
            q_id = survey.itemQAs[2].questions[0].id
            a_id = survey.itemQAs[2].answers[0].id
            responses.append({'q': q_id, 'a': a_id, 'v': answer_q3}, )
            controller.update_responses([{'item_id': survey.itemQAs[2].id, 'responses': responses}])

        if answer_q4 is not None:

            responses = []
            for i in range(len(answer_q4)):
                for j in range(len(answer_q4[i])):

                    q_id = survey.itemQAs[3].questions[i].id
                    a_id = survey.itemQAs[3].answers[j].id

                    responses.append({'q': q_id, 'a': a_id, 'v': answer_q4[i][j]})

            controller.update_responses([{'item_id': survey.itemQAs[3].id, 'responses': responses}])

    # helper method to set permissions before login
    def _set_permissions(self):
        self._attach_permissions_to_group(self.group,
            'view_dashboard', 'save_dashboard',

            'fme_contact_access', 'iquest_nps_access',

            'create_survey', 'view_survey', 'save_survey', 'delete_survey', 'update_survey',
            'create_survey_publication', 'view_survey_publication', 'save_survey_publication', 'delete_survey_publication',
            'update_survey_publication',
            'view_survey_publication_response', 'schedule_survey_publication',
            'view_iquest_pool_category', 'create_iquest_pool_category',

            'create_report', 'view_report', 'update_report', 'manage_mdx_methods',
            'view_datasource', 'datasource_manager', 'view_datasource_widget',

            'use_iquest_time_dimension', 'use_iquest_person_dimensions', 'use_iquest_company_dimensions',

            'view_mail', 'view_contact_list',
            'create_individual_reports', 'view_contact',
            'use_raw_mdx', 'view_calculated_member', 'create_calculated_member'
            )

    # helper method to populate responses
    def _populate_responses(self, survey_id, publication1_id=None, publication2_id=None):
        if publication1_id is not None:
            # q1: 3 yes, 2 nope
            for i in range(3):
                self.save_response(survey_id, publication1_id, answer_q1='yes')
            for i in range(2):
                self.save_response(survey_id, publication1_id, answer_q1='no')

            # q2
            for i in range(8):
                self.save_response(survey_id, publication1_id, answer_q2 = [
                    [i in (1,), 0, i  > 2],
                    [i < 5, 0, i > 5],
                    [i in (1,2,3,4), 0, i  > 4],
                    [i < 3, 0, i >= 3],
                ])

            # q3: number of cows
            for i in range(10):
                self.save_response(survey_id, publication1_id, answer_q3=i*3)

            # q4: how much
            for i in range(10):
                self.save_response(survey_id, publication1_id, answer_q4 = [
                    [i*10.5, 0, i*5.3],
                    [i*2.6, 0, i*1.8],
                    [i*78.2, 0, 0],
                    [0, 0, 0],
                ])

        if publication2_id is not None:
            # responses publication 2
            # q2: 3 yes, 2 nope
            for i in range(3):
                self.save_response(survey_id, publication2_id, answer_q1='yes')
            for i in range(2):
                self.save_response(survey_id, publication2_id, answer_q1='no')

    def set_up_contactlists(self):
        """
        Creates 2 contacts and 1 contact lists
        """
        self._attach_permissions_to_group(self.group, 'link_to_contact_list', 'create_contact', 'create_contact_list',
                                          'view_survey', 'create_survey_publication', 'view_contact_list',
                                          'schedule_survey_publication')
        contact_controller = ContactController(self.session, account_id=self.account.id)

        # 2 contacts.. first two have a person with locale set.. last is just a contact
        c1 = contact_controller.save_or_update_contact(None, {'last_name': 'de Vries'}, {'name': 'Company uno'}, self.group.id)
        c2 = contact_controller.save_or_update_contact(None, {'last_name': 'the Freeze'},  {'name': 'Company zwei'}, self.group.id)

        cl1 = contact_controller.save_or_update_contact_list(None, u'Contact List 1', self.group.id)
        contact_controller.link_contacts([c1, c2], cl1)

        return cl1

# below is for running the test from eclipse
if __name__ == '__main__':
    from twisted.scripts.trial import run
    import sys
    sys.argv = ['trial.py', 'iqportal.iquest.test.selenium.test_iquest_report']
    run()
    sys.exit()
