# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

import exceptions
from datetime import datetime

from basesoft.web.contrib.i18n import models as i18n_models

from iqportal.iquest.test.unit import iquest_unittest
from iqportal.iquest.compose.controllers import ComposeController
from iqportal.iquest.core import models as iquest_models
from iqportal.iquest.response import models as response_models
from iqportal.iquest.response.controllers import BranchController, ParticipantController

from basesoft import logging
logger = logging.getLogger('basesoft.web.test.unit')

class BranchingTestCase(iquest_unittest.IQuestTestCase):
    """
    """
    def setUp(self):
        iquest_unittest.IQuestTestCase.setUp(self, ['create_survey', 'view_survey', 'update_survey',
            'create_survey_publication', 'schedule_survey_publication',
            'view_conditional_branch', 'create_conditional_branch',
            'view_iquest_pool_category', 'create_iquest_pool_category'])

        locale = self.session.query(i18n_models.Locale).filter_by(locale='nl_NL').one()

        # controller
        self.controller = ComposeController(self.session, locale, account_id=self.account.id)

    def _update_module_ids(self, state, id_map, stripDomID=False):

        if state['objectName'] == 'SurveyItemQA':
            for b in state['branching']:
                for i in ['question', 'answer', 'block']:
                    b[i]['dbID'] = None
                    if b[i]['domID'] is not None:
                        b[i]['dbID'] = id_map[b[i]['domID']]
                    if stripDomID:
                        del b[i]['domID']

        iquest_unittest.IQuestTestCase._update_module_ids(self, state, id_map, stripDomID)

    def yui_test_branching_survey(self):
        """
        Creates a survey for the yui branching test and returns information about it
        """

        state = self._get_state()

        state_response = self.controller.save_state(state)

        self._update_module_ids(state['survey'], state_response)

        return state['survey']['dbID']

    def yui_test_branching_survey_dropdown(self):
        state = self._get_state()
        state['survey']['children']['blocks'][0]['children']['columns'][0]['children']['items'][0]['layout'] = 2

        state_response = self.controller.save_state(state)

        self._update_module_ids(state['survey'], state_response)

        return state['survey']['dbID']

    def yui_test_branching_survey_slider(self):
        state = self._get_state()
        state['survey']['children']['blocks'][0]['children']['columns'][0]['children']['items'][0]['layout'] = 1

        state_response = self.controller.save_state(state)

        self._update_module_ids(state['survey'], state_response)

        return state['survey']['dbID']

    def yui_test_publish_branching_survey(self):

        state = self._get_state()
        self.publish_survey(iquest_models.SurveyPublication.STATUS_SCHEDULED, datetime.now(), anonymous=True, state=state)

        return self._publication

    def test_invalid_response_deletion_single_choice(self):
        """
        If the participant answers a branched question, goes to the next page, etc. then decides to use
        the previous button, changes the answer of the branched question, all responses that follow
        from the flow should be trashed.
        """

        state = self._get_state()

        # publish a survey
        survey_id = self.publish_survey(iquest_models.SurveyPublication.STATUS_SCHEDULED, datetime.now(), anonymous=True, state=state)
        survey = self.session.query(iquest_models.Survey).get(survey_id)

        items = survey.itemQAs

        # create a response array, for all 4 QA's, they are all single choice, for this we don't care which block/column they are in.
        self.assertEqual(len(items), 4)

        responses = []
        for qa in items:
            responses.append({
                'item_id': qa.id,
                'responses': [{
                    'q': qa.questions[0].id,
                    'a': qa.answers[0].id,
                    'v': True
                }]
            })

        pc = ParticipantController(self.session, None, invitee_hash=self._publication.anonymous_hash)

        # need to save responses per each page so that filters can be checked per page and skipped in case of a skipped block
        for b in survey.blocks:
            pc.update_responses(responses, blocks=[b.id])

        resp = self.session.query(response_models.SurveyQAResponse).filter(
                response_models.SurveyQAResponse.participant_id==pc._participant.id).all()

        # we added 4 responses, but only 3 should be saved because of the second item QA based branching
        self.assertEqual(len(resp), 3)

        # add response for the first item branching: should skip second block, so just 3 responses saved
        responses[0]['responses'][0]['a'] = items[0].answers[1].id
        pc.update_responses(responses)
        resp = self.session.query(response_models.SurveyQAResponse).filter(
                response_models.SurveyQAResponse.participant_id==pc._participant.id).all()
        self.assertEqual(len(resp), 3)

    def test_invalid_response_deletion_multiple_choice(self):
        """
        Test the removal of impossible responses on a multiple choice based branching survey.
        """

        state = self._get_multiple_choice_state()
        # publish a survey
        survey_id = self.publish_survey(iquest_models.SurveyPublication.STATUS_SCHEDULED, datetime.now(), anonymous=True, state=state)
        survey = self.session.query(iquest_models.Survey).get(survey_id)

        # create a response array, for all 4 QA's, they are all single choice, for this we don't care which block/column they are in.
        self.assertEqual(len(survey.itemQAs), 5)

        responses = []
        for qa in survey.itemQAs:
            responses.append({
                'item_id': qa.id,
                'responses': [{
                    'q': qa.questions[0].id,
                    'a': qa.answers[1].id,
                    'v': True
                }]
            })
            if len(responses) == 1:
                responses[0]['responses'].append({
                    'q': qa.questions[0].id,
                    'a': qa.answers[0].id,
                    'v': True
                })

        # we have 4 responses, for each QA item one response.
        del responses[1]
        self.assertEqual(len(responses), 4)

        bc = BranchController(self.session, responses)
        delete_responses, skip_items = bc.filter_invalid_responses(self._publication)

        # 2 items should be skip and 4 answers should be deleted
        self.assertEqual(len(skip_items), 2)
        self.assertEqual(len(delete_responses), 4)

    def test_save_and_load_branching(self):
        """
        """
        state = self._get_state()
        state_response = self.controller.save_state(state)

        self._update_module_ids(state['survey'], state_response)

        stripDomID = True
        self._update_module_ids(state['survey'], state_response, stripDomID)

        loaded_state = self.controller.load_state(state['survey']['dbID'])

        # now state should be the same as the loaded_state
        try:
            self.assertEquals(state, loaded_state)
        except exceptions.AssertionError, e:
            from django.utils import simplejson
            logger.info('diff failed! check the difference at http://tlrobinson.net/projects/js/jsondiff/')

            from basesoft.web.core.jsonrpc.dispatcher import JsonRpcJSONEncoder

            logger.info('state: %s' % simplejson.dumps(state, cls=JsonRpcJSONEncoder))
            logger.info('loaded_state: %s' % simplejson.dumps(loaded_state, cls=JsonRpcJSONEncoder))

            raise e

    def test_update_branching(self):
        """
        """
        state = self._get_state()
        state_response = self.controller.save_state(state)

        self._update_module_ids(state['survey'], state_response)
        id = state['survey']['dbID']

        item = state['survey']['children']['blocks'][2]['children']['columns'][0]['children']['items'][0]
        item['branching'] = [{
            'question': {'domID': 'yui-gen015', 'dbID': None},
            'answer': {'domID': 'yui-gen016', 'dbID': None},
            'block': {'domID': 'yui-gen001', 'dbID': None}
        }]
        state_response = self.controller.save_state(state)
        self._update_module_ids(state['survey'], state_response)

        loaded_state = self.controller.load_state(state['survey']['dbID'])
        branching = loaded_state['survey']['children']['blocks'][2]['children']['columns'][0]['children']['items'][0]['branching']

        self.assertTrue(branching[0]['question']['dbID'] is not None)
        self.assertTrue(branching[0]['answer']['dbID'] is not None)
        self.assertTrue(branching[0]['block']['dbID'] is not None)

    def _get_state(self):
        """
        """
        return {

            "survey": {
                'alter_published_survey': False,
                "name": u"IQPortalTestCase Branching Survey",
                "objectName": u"Survey",
                "dbID": None,
                "domID": u"iquest-canvas-dom-id",
                "has_active_publications": False,
                "availableLocales": ['nl_NL'],
                "alter_published_survey": False,

                "theme": {
                    "module": u"iquest_survey_theme_basesoft",
                    "name": u"BaseSoft",
                    "cssClass": u"survey-basesoft",
                },

                "fallbackLocale": {'locale': u'nl_NL', 'name': u'Dutch' },

                "children": {
                    "blocks": [{
                        "objectName": u"SurveyBlock",
                        "dbID":None,
                        "domID": u"yui-gen001",
                        "pageEnd": True,
                        "children": {
                            "columns": [{
                                "children": {
                                    'items': [{
                                        "objectName":u"SurveyItemQA",
                                        "dbID":None,
                                        "poolItemID": None,
                                        "domID":u"yui-gen002",
                                        "questionsAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
                                        "answersAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
                                        "trailingQuestions": False,
                                        "type": iquest_models.SurveyItemQA.TYPE_SINGLE_CHOICE,
                                        "layout": None,
                                        "showTitle": True,
                                        "sum_questions": False,
                                        "sum_answers": False,
                                        "layoutAttributes": {},
                                        'required': False,
                                        "filters": [],
                                        "weight_factors": [],
                                        "alwaysShowAnswerLabels": False,
                                        'alwaysShowQuestionLabels': False,
                                        "branching": [{
                                            'question': {'domID': "yui-gen003", 'dbID': None},
                                            'answer': {'domID': "yui-gen004", 'dbID': None},
                                            'block': {'domID': "yui-gen007", 'dbID': None}
                                        },{
                                            'question': {'domID': "yui-gen003", 'dbID': None},
                                            'answer': {'domID': "yui-gen005", 'dbID': None},
                                            'block': {'domID': "yui-gen013", 'dbID': None}
                                        }],

                                        "locales": {
                                            'nl_NL': {
                                                u'label': u'Wat denkt u van Fritesaus?'
                                            }
                                        },

                                        "children": {
                                            "questions":[{
                                                "objectName":u"SurveyQuestion",
                                                "dbID":None,
                                                "domID":u"yui-gen003",
                                                "poolElementID":None,
                                                "locales": {},
                                            }],

                                            "answers":[{
                                                "objectName":u"SurveyAnswer",
                                                "dbID":None,
                                                "poolElementID":None,
                                                "domID":u"yui-gen004",

                                                "locales": {
                                                    'nl_NL': {
                                                        u'label': u'Goed',
                                                        u'trailing_label': None
                                                    }
                                                },
                                            },
                                            {
                                                "objectName":u"SurveyAnswer",
                                                "dbID":None,
                                                "poolElementID":None,
                                                "domID":u"yui-gen005",

                                                "locales": {
                                                    'nl_NL': {
                                                        u'label': u'Ik vind Freddie beter.',
                                                        u'trailing_label': None
                                                    }
                                                },
                                            }]
                                        }
                                    }]
                                },
                                "objectName": u"SurveyBlockColumn",
                                "dbID": None,
                                "domID": u"yui-gen006",
                                "width":100
                            }],
                        }
                    },{
                        "objectName": u"SurveyBlock",
                        "dbID":None,
                        "domID": u"yui-gen007",
                        "pageEnd": True,
                        "children": {
                            "columns": [{
                                "children": {
                                    'items': [{
                                        "objectName":u"SurveyItemQA",
                                        "dbID":None,
                                        "poolItemID": None,
                                        "domID":u"yui-gen008",
                                        "questionsAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
                                        "answersAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
                                        "trailingQuestions": False,
                                        "type": iquest_models.SurveyItemQA.TYPE_SINGLE_CHOICE,
                                        "layout": None,
                                        "showTitle": True,
                                        "sum_questions": False,
                                        "sum_answers": False,
                                        "layoutAttributes": {},
                                        'required': False,
                                        "filters": [],
                                        "weight_factors": [],
                                        "alwaysShowAnswerLabels": False,
                                        'alwaysShowQuestionLabels': False,
                                        "branching": [{
                                                'question': {'domID': None, 'dbID': None},
                                                'answer': {'domID': None, 'dbID': None},
                                                'block': {'domID': "yui-gen019", 'dbID': None}
                                        }],

                                        "locales": {
                                            'nl_NL': {
                                                u'label': u'QA level branch?',
                                            }
                                        },

                                        "children": {
                                            "questions":[{
                                                "objectName":u"SurveyQuestion",
                                                "dbID":None,
                                                "poolElementID":None,
                                                "domID":u"yui-gen009",
                                                "locales": {},
                                            }],

                                            "answers":[{
                                                "objectName":u"SurveyAnswer",
                                                "dbID":None,
                                                "poolElementID":None,
                                                "domID":u"yui-gen010",

                                                "locales": {
                                                    'nl_NL': {
                                                        u'label': u'Oke dan',
                                                        u'trailing_label': None
                                                    }
                                                },
                                            },
                                            {
                                                "objectName":u"SurveyAnswer",
                                                "dbID":None,
                                                "poolElementID":None,
                                                "domID":u"yui-gen011",

                                                "locales": {
                                                    'nl_NL': {
                                                        u'label': u'Bah',
                                                        u'trailing_label': None
                                                    }
                                                },
                                            }]
                                        }
                                    }]
                                },
                                "objectName": u"SurveyBlockColumn",
                                "dbID": None,
                                "domID": u"yui-gen012",
                                "width":100
                            }],
                        }
                    },{
                        "objectName": u"SurveyBlock",
                        "dbID":None,
                        "domID": u"yui-gen013",
                        "pageEnd": True,
                        "children": {
                            "columns": [{
                                "children": {
                                    'items': [{
                                        "objectName":u"SurveyItemQA",
                                        "dbID":None,
                                        "poolItemID": None,
                                        "domID":u"yui-gen014",
                                        "questionsAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
                                        "answersAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
                                        "trailingQuestions": False,
                                        "type": iquest_models.SurveyItemQA.TYPE_SINGLE_CHOICE,
                                        "layout": None,
                                        "showTitle": True,
                                        "sum_questions": False,
                                        "sum_answers": False,
                                        "layoutAttributes": {},
                                        'required': True,
                                        "filters": [],
                                        "weight_factors": [],
                                        "alwaysShowAnswerLabels": False,
                                        'alwaysShowQuestionLabels': False,
                                        "branching": [],

                                        "locales": {
                                            'nl_NL': {
                                                u'label': u'Geen enkele branch?',
                                            }
                                        },

                                        "children": {
                                            "questions":[{
                                                "objectName":u"SurveyQuestion",
                                                "dbID":None,
                                                "poolElementID":None,
                                                "domID":u"yui-gen015",
                                                "locales": {},
                                            }],

                                            "answers":[{
                                                "objectName":u"SurveyAnswer",
                                                "dbID":None,
                                                "poolElementID":None,
                                                "domID":u"yui-gen016",

                                                "locales": {
                                                    'nl_NL': {
                                                        u'label': u'Hehe, eindelijk',
                                                        u'trailing_label': None
                                                    }
                                                },
                                            },
                                            {
                                                "objectName":u"SurveyAnswer",
                                                "dbID":None,
                                                "poolElementID":None,
                                                "domID":u"yui-gen017",

                                                "locales": {
                                                    'nl_NL': {
                                                        u'label': u'Ik vind dat branchen wel wat hebben',
                                                        u'trailing_label': None
                                                    }
                                                },
                                            }]
                                        }
                                    }]
                                },
                                "objectName": u"SurveyBlockColumn",
                                "dbID": None,
                                "domID": u"yui-gen018",
                                "width":100
                            }],
                        }
                    },{
                        "objectName": u"SurveyBlock",
                        "dbID":None,
                        "domID": u"yui-gen019",
                        "pageEnd": True,
                        "children": {
                            "columns": [{
                                "children": {
                                    'items': [{
                                        "objectName":u"SurveyItemQA",
                                        "dbID":None,
                                        "poolItemID": None,
                                        "domID":u"yui-gen020",
                                        "questionsAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
                                        "answersAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
                                        "trailingQuestions": False,
                                        "type": iquest_models.SurveyItemQA.TYPE_SINGLE_CHOICE,
                                        "layout": None,
                                        "showTitle": True,
                                        "sum_questions": False,
                                        "sum_answers": False,
                                        "layoutAttributes": {},
                                        'required': False,
                                        "filters": [],
                                        "weight_factors": [],
                                        "alwaysShowAnswerLabels": False,
                                        'alwaysShowQuestionLabels': False,
                                        "branching": [],

                                        "locales": {
                                            'nl_NL': {
                                                u'label': u'En de laatste vraag?',
                                            }
                                        },

                                        "children": {
                                            "questions":[{
                                                "objectName":u"SurveyQuestion",
                                                "dbID":None,
                                                "poolElementID":None,
                                                "domID":u"yui-gen021",
                                                "locales": {},
                                            }],

                                            "answers":[{
                                                "objectName":u"SurveyAnswer",
                                                "dbID":None,
                                                "poolElementID":None,
                                                "domID":u"yui-gen022",

                                                "locales": {
                                                    'nl_NL': {
                                                        u'label': u'Uiteraard',
                                                        u'trailing_label': None
                                                    }
                                                },
                                            },
                                            {
                                                "objectName":u"SurveyAnswer",
                                                "dbID":None,
                                                "poolElementID":None,
                                                "domID":u"yui-gen023",

                                                "locales": {
                                                    'nl_NL': {
                                                        u'label': u'Zal wel moeten',
                                                        u'trailing_label': None
                                                    }
                                                },
                                            }]
                                        }
                                    }]
                                },
                                "objectName": u"SurveyBlockColumn",
                                "dbID": None,
                                "domID": u"yui-gen024",
                                "width":100
                            }],
                        }
                    }]
                }
            }
        }

    def _get_multiple_choice_state(self):

        state = self._get_state()
        item = state['survey']['children']['blocks'][0]['children']['columns'][0]['children']['items'][0]
        item['type'] = iquest_models.SurveyItemQA.TYPE_MULTIPLE_CHOICE

        item = state['survey']['children']['blocks'][2]['children']['columns'][0]['children']['items'][0]
        item['branching'] = [{
            'question': {'domID': 'yui-gen015', 'dbID': None},
            'answer': {'domID': 'yui-gen017', 'dbID': None},
            'block': {'domID': "yui-gen100", 'dbID': None}
        }]

        state['survey']['children']['blocks'].append({
            "objectName": u"SurveyBlock",
            "dbID":None,
            "domID": u"yui-gen100",
            "pageEnd": True,
            "children": {
                "columns": [{
                    "children": {
                        'items': [{
                            "objectName":u"SurveyItemQA",
                            "dbID":None,
                            "poolItemID": None,
                            "domID":u"yui-gen101",
                            "questionsAlign": iquest_models.SurveyItemQA.ALIGN_TOP,
                            "answersAlign": iquest_models.SurveyItemQA.ALIGN_LEFT,
                            "trailingQuestions": False,
                            "type": iquest_models.SurveyItemQA.TYPE_SINGLE_CHOICE,
                            "layout": None,
                            "showTitle": True,
                            "layoutAttributes": {},
                            'required': False,
                            "filters": [],
                            "weight_factors": [],
                            "branching": [],

                            "locales": {
                                'nl_NL': {
                                    u'label': u'En de laatste vraag?',
                                }
                            },

                            "children": {
                                "questions":[{
                                    "objectName":u"SurveyQuestion",
                                    "dbID":None,
                                    "poolElementID":None,
                                    "domID":u"yui-gen102",
                                    "locales": {},
                                }],

                                "answers":[{
                                    "objectName":u"SurveyAnswer",
                                    "dbID":None,
                                    "poolElementID":None,
                                    "domID":u"yui-gen103",

                                    "locales": {
                                        'nl_NL': {
                                            u'label': u'Uiteraard',
                                        }
                                    },
                                },
                                {
                                    "objectName":u"SurveyAnswer",
                                    "dbID":None,
                                    "poolElementID":None,
                                    "domID":u"yui-gen104",

                                    "locales": {
                                        'nl_NL': {
                                            u'label': u'Zal wel moeten',
                                        }
                                    },
                                }]
                            }
                        }]
                    },
                    "objectName": u"SurveyBlockColumn",
                    "dbID": None,
                    "domID": u"yui-gen105",
                    "width":100
                }],
            }
        })
        return state

