# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from django.forms.util import ValidationError

from basesoft.web.core.exceptions import SecurityException,\
    PermissionDeniedException
from basesoft.web.core.util.controllers import StateManager

from iqportal.iquest.test.unit.iquest_unittest import IQuestTestCase
from iqportal.iquest.core.qa_filters.controllers import IQuestQAFilterController, SurveyItemQAFilterController
from iqportal.iquest.core import models as iquest_models
from iqportal.iquest.core.models import IQuestQAFilter

class ItemQAFilterTest(IQuestTestCase):

    def setUp(self):
        IQuestTestCase.setUp(self, ['create_survey', 'view_survey', 'create_survey_publication'])
        self.controller = IQuestQAFilterController(self.session, account_id=self.account.id)

    def test_numerical_filter(self):
        """
        Test a few combinations of the numerical filter.
        """
        from iqportal.iquest.core.qa_filters.filters import Numeric
        nf = Numeric({'v': 'not_numeric'})
        self.assertRaises(ValidationError, nf.validate, domain='int')
        self.assertRaises(ValidationError, nf.validate, domain='float')

        nf = Numeric([{'v': 200}, {'v': 400}])
        self.assertRaises(ValidationError, nf.validate, minValue=300)

        nf = Numeric([{'v': 20.5}, {'v': 10.4}])
        self.assertRaises(ValidationError, nf.validate, domain='int')

        nf = Numeric([{'v': 200}, {'v': 400}])
        self.assertTrue(nf.validate(minValue=200, maxValue=400, domain='int'))

        nf = Numeric([{'v': 0.1}, {'v': 0.7}])
        self.assertTrue(nf.validate(sumValue=0.8, domain='float'))

        nf = Numeric([{'v': 0.2}, {'v': 0.7}])
        self.assertRaises(ValidationError, nf.validate, sumValue=0.8, domain='float')

    def test_text_filters(self):
        """
        Test text filters.
        """
        from iqportal.iquest.core.qa_filters.filters import Text, AlphaNumeric, Email
        tf = Text({'v': 'I should not fail!'})
        self.assertTrue(tf.validate())

        tf = Text({'v': 'three'})
        self.assertRaises(ValidationError, tf.validate, maxLength=3)

        tf = Text({'v': 'five'})
        self.assertRaises(ValidationError, tf.validate, minLength=5)

        anf = AlphaNumeric({'v': 'abcd1234'})
        self.assertTrue(anf.validate())

        anf = AlphaNumeric({'v': 'a!cd1234'})
        self.assertRaises(ValidationError, anf.validate)

        ef = Email({'v': 'marten@basesoft.com'})
        self.assertTrue(ef.validate())

        ef = Email({'v': 'marten_koedam@office.basesoft.com'})
        self.assertTrue(ef.validate())

        ef = Email({'v': 'marten@basesoft_solutions.com'})
        self.assertRaises(ValidationError, ef.validate)

    def test_filters(self):
        """
        Test whether the creating/updating/deleting of filters works properly
        """
        # add view permissions
        self._attach_permissions_to_group(self.group, 'view_itemqa_filter')
        # this should work properly
        self.controller.get_filters()

        # test create. first time it should fail without permissions
        self.assertRaises(PermissionDeniedException, self.controller.save_or_update_filter, name=u'Test Text')
        # add the proper permission
        self._attach_permissions_to_group(self.group, 'create_itemqa_filter')
        # now we should be able to add a new filter
        test_filter = self.controller.save_or_update_filter(name=u"Test Text")
        filters = self.controller.get_filters(test_filter.id)
        self.assertEquals(len(filters), 1) # we should have our filter

        # try to update without permissions
        self.assertRaises(PermissionDeniedException, self.controller.save_or_update_filter, id=test_filter.id, name=u'New name')
        # once we add the permissions we should be able to modify the filter we just added
        self._attach_permissions_to_group(self.group, 'update_itemqa_filter')
        test_filter = self.controller.save_or_update_filter(name=u"New Name", id=test_filter.id)
        filters = self.controller.get_filters(test_filter.id)
        self.assertEquals(len(filters), 1) # we should have our updated filter
        self.assertEquals(filters[0].name, u"New Name") # we should have our updated filter

        # try to delete without permissions
        self.assertRaises(PermissionDeniedException, self.controller.delete_filters, test_filter)

        self._attach_permissions_to_group(self.group, 'delete_itemqa_filter')
        fid = test_filter.id # store the id so we can test later if it still exists
        self.controller.delete_filters(test_filter)
        filters = self.controller.get_filters(fid)
        self.assertEquals(len(filters), 0) # list should be empty since we deleted the filter

    def test_itemqa_filters(self):
        """
        Test if attaching filters to itemQAs works ok
        """
        controller = SurveyItemQAFilterController(self.session, None, self.account.id, StateManager(None, None, None))
        # create a survey
        survey_id = self.publish_survey(iquest_models.SurveyPublication.STATUS_NEW)
        survey = self.session.query(iquest_models.Survey).get(survey_id)

        # get one of the items from the survey
        item_qa = survey.blocks[2].columns[0].items[0]
        self.assertTrue(isinstance(item_qa, iquest_models.SurveyItemQA))

        # try to get the list of filters that are attached to item_qa. first without permissions
        self.assertRaises(PermissionDeniedException, controller.get_item_filters, item_qa.id)
        # add view permissions and try again
        self._attach_permissions_to_group(self.group, 'view_itemqa_filter')
        controller.get_item_filters(item_qa.id)

        # test create
        # first it should fail because we don't have proper objects
        self.assertRaises(AssertionError, controller.save_or_update_item_filter, None, None, None, None)
        # add create permission, so we can add some filter and attributes
        self._attach_permissions_to_group(self.group, 'create_itemqa_filter')

        # add the necessary objects
        # create a filter and a couple of attributes so we can attach them to our itemQA
        filter = self.controller.save_or_update_filter(name=u"Test Text")

        # now it should work since we have all we need
        # the item_qa needs to have values that have min_lenth 1 and max_length 5 (validation is not done here)
        sif1 = controller.save_or_update_item_filter(None, filter, item_qa, attributes={'minLength': 1, 'maxLength': 5})
        self.session.flush()
        item_filters = controller.get_item_filters(item_id=item_qa.id)
        self.assertEquals(len(item_filters), 1)

        # add another attribute so we can test get_item_filters with question_id param
        controller.save_or_update_item_filter(None, filter, item_qa, attributes={'regexp': "abc"}, question_id=item_qa.questions[0].id)
        item_filters = controller.get_item_filters(item_id=item_qa.id, question_id=item_qa.questions[0].id, answer_id=None)
        self.assertEquals(len(item_filters), 1)

        # update an item filter
        controller.save_or_update_item_filter(sif1.id, filter, item_qa, attributes={'minLength': 2, 'maxLength': 5})
        changed_item_filter = controller.get_item_filters(id=sif1.id, item_id=item_qa.id, question_id=None).pop()
        for attr in changed_item_filter.attributes:
            if attr.attribute == 'minLength':
                self.assertEquals(attr.value, u"2") # we should have our updated value (unicode)
            else:
                self.assertEquals(attr.value, u"5") # we should have our updated value (unicode)

        # test the deletion of item filters
        id = sif1.id # store the id so we can test later if it still exists
        controller.delete_item_filters(sif1)

        item_filters = controller.get_item_filters(item_qa.id)
        self.assertEquals(len(item_filters), 1) # we should still have sif2

