# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from datetime import datetime

from sqlalchemy import and_

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.contrib.contact.controllers import ContactController

from iqportal.iquest.core import schedule
from iqportal.iquest.core.models import Survey, SurveyPublication
from iqportal.iquest.compose.controllers import ComposeController, PublicationStateController
from iqportal.iquest.core.mailing import MailController
from basesoft.web.core.util.controllers import StateManager
from basesoft.web.contrib.mailing.models import Mailing
from basesoft.web.contrib.compose.models import ComposePublicationContactList
from basesoft.web.core.exceptions import PublicUserException

class MailingTest(iqportal_unittest.IQPortalTestCase):
    """
    """

    def setUp(self):
        """
        """
        #self.reset_db()
        self.connect_db()

        #create an account so we can check permission with it
        self.account = self._create_active_account()

        # create the necessary controllers
        self.group = self.account.groups[0].group

        # initialise compose controller
        from basesoft.web.contrib.i18n import models as i18n_models
        self.locale = self.session.query(i18n_models.Locale).filter_by(locale='en_GB').one()
        self.survey_controller = ComposeController(self.session, self.locale, account_id=self.account.id)

        # init the schedule state controller
        self.publication_controller = PublicationStateController(self.session, StateManager(self.locale, self.account.id, None))

        # give current group access to create surveys
        self._attach_permissions_to_group(self.group, 'create_survey', 'create_survey_publication',
                                          'schedule_survey_publication', 'view_survey',
                                          'schedule_iquest_mail')

        # create some contact lists and mails
        self.setUpContactLists()
        self.setUpMails()

    def setUpContactLists(self):
        """
        Creates 3 contacts and 2 contact lists
        """
        self._attach_permissions_to_group(self.group, 'link_to_contact_list', 'create_contact', 'create_contact_list',
                                          'view_contact_list')
        contact_controller = ContactController(self.session, account_id=self.account.id)

        c1 = contact_controller.save_or_update_contact(None, {'last_name': u'Contact c1', 'email': u'testing@mail.clusternet'}, None, self.group.id)
        c2 = contact_controller.save_or_update_contact(None, {'last_name': u'Contact c2', 'email': u'testing@mail.clusternet'}, {'name': 'Contact Company c2'}, self.group.id)
        c3 = contact_controller.save_or_update_contact(None, {'last_name': u'Contact c3', 'email': u'testing@mail.clusternet'}, None, self.group.id)

        self.cl1 = contact_controller.save_or_update_contact_list(None, 'Contact List 1', self.group.id)
        contact_controller.link_contacts([c1, c2], self.cl1)

        self.cl2 = contact_controller.save_or_update_contact_list(None, 'Contact List 2', self.group.id)
        contact_controller.link_contacts([c2, c3], self.cl2)

    def get_mail_state(self, name):
        return {
            "mail": {
                "mail_from_text": "MockMailer",
                "name": name,
                "objectName": u"IQuestMail",
                "dbID": None,
                "locales": {
                    'nl_NL': {
                        'subject': u"iquest uitnodiging",
                        'content': u'Geachte <img class="replacement-field" src="Contact:Name" title="person.last_name"/> \
                            (<img class="replacement-field" src="Company:Name" title="company.name"/>) \
                            <br /><br /> \
                            Klikie op de ze link:.<br/>\
                            <img class="replacement-field" src="Survey:Link" title="iquest.survey.link"/>',
                    },
                    'en_GB': {
                        'subject': u"iquest invite",
                        'content': u'Dear <img class="replacement-field" src="Contact:Name" title="person.last_name"/> \
                            (<img class="replacement-field" src="Company:Name" title="company.name"/>) \
                            <br /><br /> \
                            Clicky on ze link:.<br/>\
                            <img class="replacement-field" src="Survey:Link" title="iquest.survey.link"/>',
                    },
                },
                "fallbackLocale": {'value': u'en_GB' },
            }
        }

    def setUpMails(self):

        self._attach_permissions_to_group(self.group, 'create_mail', 'create_iquest_mail')
        controller = MailController(self.session, self.locale, account_id=self.account.id)
        self.mail1 = controller.save_state(self.get_mail_state(u'IQuest Mail 1'), self.group)

    def _publish_survey(self, status, begin_date, end_date=None, contact_lists=None, anonymous=False, mailing_contact_lists=None, return_publication=False):
        """
        Helper method for creating a survey state
        """
        if contact_lists is None:
            contact_lists = []
        if mailing_contact_lists is None:
            mailing_contact_lists = contact_lists

        survey_state = {
            "survey": {
                "name": u"My Pretty Survey",
                "objectName": u"Survey",
                "dbID": None,
                "domID": u"iquest-canvas-dom-id",

                "theme": {
                    "module": u"iquest_survey_theme_basesoft",
                    "name": u"BaseSoft",
                    "cssClass": u"survey-basesoft",
                },

                "availableLocales": ['en_GB'],

                "fallbackLocale": {'locale': u'en_GB', 'name': u'English' },
                "children": {'blocks': []}
            }
        }

        survey_map = self.survey_controller.save_state(survey_state)
        survey_id = survey_map['iquest-canvas-dom-id']

        publish_state = {
            'survey_id': survey_id,
            "dbID": None,
            "domID": u'publish_1',
            "begin_date": begin_date,
            "end_date": end_date,
            "status": status,
            "label": u"publication 1",
            "show_page_nrs": True,
            "anonymous": anonymous,
            "contact_lists": contact_lists,
            "viewer_print": False,
            "mailings": [{
                "dbID": None,
                "domID": u'publish_1-mailing_1',
                "mail_id": self.mail1.id,
                "send_date": begin_date,
                "contact_lists": mailing_contact_lists,
                'mail_from_text': 'mail from text'
            }]
        }

        # save publication
        publication = self.publication_controller.save(publish_state)

        if return_publication:
            return publication
        return survey_id

    def test_mailing_anonymous(self):

        # create anonymous survey and publish
        survey_id = self._publish_survey(SurveyPublication.STATUS_SCHEDULED, datetime.now(), contact_lists=[self.cl1.id, self.cl2.id], anonymous=True)
        self.session.query(Survey).get(survey_id)

        rv = schedule.PublishSurveysTask().execute()
        # 1 publication set online
        self.assertEqual(len(rv), 1)
        schedule.CreatePublicationInviteesTask().execute()
        # now it should send emails because invitees have been created
        rv = schedule.PublishSurveysTask().execute()
        # no publications set online
        self.assertEqual(len(rv), 0)

        # notice that we cant really assert stuff here since mails go out.. but if there is no error till here
        # all 'should' be good cuz mailing is pretty good tested in the mailing tests

    def test_mailing_non_anonymous(self):

        survey_id = self._publish_survey(SurveyPublication.STATUS_SCHEDULED, datetime.now(), contact_lists=[self.cl1.id, self.cl2.id])
        self.session.query(Survey).get(survey_id)

        rv = schedule.PublishSurveysTask().execute()
        # 1 publication set online
        self.assertEqual(len(rv), 1)
        schedule.CreatePublicationInviteesTask().execute()
        # now it should send emails because invitees have been created
        rv = schedule.PublishSurveysTask().execute()
        # no publications set online
        self.assertEqual(len(rv), 0)

        # notice that we cant really assert stuff here since mails go out.. but if there is no error till here
        # all 'should' be good cuz mailing is pretty good tested in the mailing tests

    def test_additional_contacts_after_publication(self):
        survey_id = self._publish_survey(SurveyPublication.STATUS_SCHEDULED, datetime.now(), contact_lists=[self.cl1.id, self.cl2.id])
        survey = self.session.query(Survey).get(survey_id)

        rv = schedule.PublishSurveysTask().execute()
        # 1 publication set online
        self.assertEqual(len(rv), 1)
        schedule.CreatePublicationInviteesTask().execute()

        publication = survey.publications[0]
        self.assertEqual(len(publication.invitees), 3)

        cl = publication.contact_lists[0]
        cc = ContactController(self.sa_session, account_id=self.account.id)

        contact_details = {'first_name': 'Barry',
                           'last_name': 'Hay',
                           'email': 'barry.hay@mail-test.basesoft.com'}
        #insert
        contact =  cc.save_or_update_contacts_from_state(contacts_state = [contact_details])[0]
        cc.link_contacts([contact], cl)

        schedule.CreatePublicationInviteesTask().execute()
        self.sa_session.expire(survey)
        survey = self.session.query(Survey).get(survey_id)

        publication = survey.publications[0]
        self.assertEqual(len(publication.invitees), 4)

    def test_mailing_replacement_fields(self):

        survey_id = self._publish_survey(SurveyPublication.STATUS_SCHEDULED, datetime.now(), contact_lists=[self.cl1.id, self.cl2.id])
        self.session.query(Survey).get(survey_id)

        rv = schedule.PublishSurveysTask().execute()
        # 1 publication set online
        self.assertEqual(len(rv), 1)
        schedule.CreatePublicationInviteesTask().execute()
        # now it should send emails because invitees have been created
        t = MockPublicationMailing()
        t.execute()

        # we should have 3 sent messages
        self.assertEqual(len(t.sent_messages), 3)

        # one of them should have a company replacement.
        for result in t.sent_messages:
            c, mime_msg = result
            html = mime_msg.get_payload(0).as_string()
            if c.person.last_name == 'Contact c2':
                self.assertTrue(html.find('(Contact Company c2)'))
            else:
                self.assertTrue(html.find('()'))


    def test_mailing_exception(self):
        """
        Test some of the exception in the publication mailing
        """
        # make a publication with more mailing contact lists than there are on attached to the publication
        self.assertRaises(PublicUserException, self._publish_survey, SurveyPublication.STATUS_SCHEDULED, datetime.now(), contact_lists=[self.cl1.id],
                                         anonymous=False, mailing_contact_lists=[self.cl1.id, self.cl2.id])

        # Can no longer add contact list which are not part of the invitees

    def test_no_mailing_exception(self):
        # now test again with anonymous is True, this should throw no exceptions whatsoever
        self._publish_survey(SurveyPublication.STATUS_SCHEDULED, datetime.now(), contact_lists=[self.cl1.id],
                                         anonymous=True, mailing_contact_lists=[self.cl1.id, self.cl2.id])
        rv = schedule.PublishSurveysTask().execute()
        # 1 publication set online
        self.assertEqual(len(rv), 1)
        schedule.CreatePublicationInviteesTask().execute()
        schedule.PublishSurveysTask().execute()

        # notice that we cant really assert stuff here since mails go out.. but if there is no error till here
        # all 'should' be good cuz mailing is pretty good tested in the mailing tests

    def test_mailing_grace_period_0_days(self):
        # test that new contacts do not reset the mailing status if the grace period is 0
        publication = self._publish_survey(SurveyPublication.STATUS_SCHEDULED, datetime.now(), contact_lists=[self.cl1.id],
                                         anonymous=True, mailing_contact_lists=[self.cl1.id, self.cl2.id], return_publication=True)

        rv = schedule.PublishSurveysTask().execute()
        # 1 publication set online
        self.assertEqual(len(rv), 1)
        schedule.CreatePublicationInviteesTask().execute()
        schedule.PublishSurveysTask().execute()

        self.sa_session.expire(publication)
        self.assertEqual(publication.mailings[0].status, Mailing.STATUS_SENT)
        self.assertEqual(publication.mailings[0].sending_grace_period, 0)

        # add contacts to the contact lists cl1
        cl = publication.contact_lists[0]
        cc = ContactController(self.sa_session, account_id=self.account.id)

        contact_details = {'first_name': 'Barry',
                           'last_name': 'Hay',
                           'email': 'barry.hay@mail-test.basesoft.com'}
        #insert
        contact =  cc.save_or_update_contacts_from_state(contacts_state = [contact_details])[0]
        cc.link_contacts([contact], cl)

        cpcl = self.sa_session.query(ComposePublicationContactList).filter(and_(ComposePublicationContactList.publication==publication, ComposePublicationContactList.contact_list==cl)).first()
        self.assertEqual(cpcl.is_processed, ComposePublicationContactList.STATUS_REPROCESS)

        schedule.CreatePublicationInviteesTask().execute()

        # the creation of new invitees should NOT have reset the mailing to scheduled.
        self.sa_session.expire(publication)
        self.assertEqual(publication.mailings[0].status, Mailing.STATUS_SENT)

    def test_mailing_grace_period_1_day(self):
        # test that new contacts do reset the mailing status if the grace period is 1 day
        publication = self._publish_survey(SurveyPublication.STATUS_SCHEDULED, datetime.now(), contact_lists=[self.cl1.id],
                                         anonymous=True, mailing_contact_lists=[self.cl1.id, self.cl2.id], return_publication=True)

        rv = schedule.PublishSurveysTask().execute()
        # 1 publication set online
        self.assertEqual(len(rv), 1)
        schedule.CreatePublicationInviteesTask().execute()
        schedule.PublishSurveysTask().execute()

        self.sa_session.expire(publication)
        self.assertEqual(publication.mailings[0].status, Mailing.STATUS_SENT)

        publication.mailings[0].sending_grace_period = 1
        self.sa_session.add(publication.mailings[0])
        self.sa_session.flush()

        # add contacts to the contact lists cl1
        cl = publication.contact_lists[0]
        cc = ContactController(self.sa_session, account_id=self.account.id)

        contact_details = {'first_name': 'Barry',
                           'last_name': 'Hay',
                           'email': 'barry.hay@mail-test.basesoft.com'}
        #insert
        contact =  cc.save_or_update_contacts_from_state(contacts_state = [contact_details])[0]
        cc.link_contacts([contact], cl)

        import time
        time.sleep(1)

        cpcl = self.sa_session.query(ComposePublicationContactList).filter(and_(ComposePublicationContactList.publication==publication, ComposePublicationContactList.contact_list==cl)).first()
        self.assertEqual(cpcl.is_processed, ComposePublicationContactList.STATUS_REPROCESS)

        schedule.CreatePublicationInviteesTask().execute()

        # the creation of new invitees should have reset the mailing to scheduled.
        self.sa_session.expire(publication)
        self.assertEqual(publication.mailings[0].status, Mailing.STATUS_SCHEDULED)

    def test_mailing_grace_period_none(self):
        # test that new contacts do reset the mailing status if the grace period is None
        publication = self._publish_survey(SurveyPublication.STATUS_SCHEDULED, datetime.now(), contact_lists=[self.cl1.id],
                                         anonymous=True, mailing_contact_lists=[self.cl1.id, self.cl2.id], return_publication=True)

        rv = schedule.PublishSurveysTask().execute()
        # 1 publication set online
        self.assertEqual(len(rv), 1)
        schedule.CreatePublicationInviteesTask().execute()
        schedule.PublishSurveysTask().execute()

        self.sa_session.expire(publication)
        self.assertEqual(publication.mailings[0].status, Mailing.STATUS_SENT)

        publication.mailings[0].sending_grace_period = None
        self.sa_session.add(publication.mailings[0])
        self.sa_session.flush()

        # add contacts to the contact lists cl1
        cl = publication.contact_lists[0]
        cc = ContactController(self.sa_session, account_id=self.account.id)

        contact_details = {'first_name': 'Barry',
                           'last_name': 'Hay',
                           'email': 'barry.hay@mail-test.basesoft.com'}
        #insert
        contact =  cc.save_or_update_contacts_from_state(contacts_state = [contact_details])[0]
        cc.link_contacts([contact], cl)

        import time
        time.sleep(1)

        cpcl = self.sa_session.query(ComposePublicationContactList).filter(and_(ComposePublicationContactList.publication==publication, ComposePublicationContactList.contact_list==cl)).first()
        self.assertEqual(cpcl.is_processed, ComposePublicationContactList.STATUS_REPROCESS)

        schedule.CreatePublicationInviteesTask().execute()

        # the creation of new invitees should have reset the mailing to scheduled.
        self.sa_session.expire(publication)
        self.assertEqual(publication.mailings[0].status, Mailing.STATUS_SCHEDULED)

class MockPublicationMailing(schedule.PublicationMailing):
    sent_messages = None

    def _build_message(self, contact, mailing, mail_from, license):
        msg, result = schedule.PublicationMailing._build_message(self, contact, mailing, mail_from, license)

        if self.sent_messages is None:
            self.sent_messages = []

        self.sent_messages.append((contact, msg))

        return msg, result

# below is for running the test from eclipse
if __name__ == '__main__':
    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_compose.py', 'iqportal.iquest.test.unit.test_schedule.ScheduleTest.test_load_save']
    run()
    sys.exit()
