# Copyright (c) 2012 BaseSoft B.V.
# See LICENSE for details.

from sqlalchemy import func, create_engine, and_
from datetime import datetime, timedelta

from django.utils import simplejson
from django.conf import settings

from iqportal.iquest.test.unit import iquest_unittest
from iqportal.iquest.core.models import SurveyPublication, Survey

from iqportal.iqube.core.controllers import ConfigurableWidgetController, ProjectController, MondrianMemberSerialize
from iqportal.iqube.core.models import DataSource, MdxMethod
from iqportal.iquest.report.schedule import AggregateResponsesTask

from twisted.trial.unittest import SkipTest
from iqportal.iqube.compose.controllers import MdxCalculatedMemberController

class IQuestReportWidgetTestCase(iquest_unittest.IQuestTestCase):

    def setUp(self):
        self.permissions = ['create_survey', 'view_survey', 'update_survey', 'create_survey_publication',
                            'schedule_survey_publication', 'view_survey_publication_response', 'view_datasource']
        iquest_unittest.IQuestTestCase.setUp(self, self.permissions)
        self.survey_id = self.publish_survey(SurveyPublication.STATUS_SCHEDULED, datetime.now(), anonymous=True)
        self.survey = self.session.query(Survey).get(self.survey_id)

        self.p1 = self._publication

        self.publish_survey(SurveyPublication.STATUS_SCHEDULED, datetime.now(), anonymous=True, state=self._survey_state, label='publication 2')
        self.p2 = self._publication

        self.controller = ConfigurableWidgetController(self.session, account_id=self.account.id)

        datasource = self.session.query(DataSource).filter(func.substr(DataSource.name, -6) == 'IQuest').all()
        if len(datasource) == 0:
            raise Exception('No IQuest datasource')
        self.datasource = datasource[0]

        pc = ProjectController(self.session)
        pc.flush_schema(self.datasource.id)

        self.cube = 'VirtualResponse'

    def _get_survey_dimension_cfg(self):
        """
        Basic Survey dimension config
        """
        return {
            "available_on_rows": True,
            "show_named_sets": False,
            "has_select_as_group": False,
            "available_as_split_by": False,
            "required": False,
            "available_as_slicer": True,
            "caption": "Survey",
            "fancy_select_module": "iq_fancyselect",
            "available_on_columns": True,
            "show_calculated_members": False,
            "children": [
                {
                    "unique_name": "[Survey].[Survey]",
                    "has_children_available": False,
                    "caption": "Survey",
                    "has_all_member_available": False,
                    "show_members": True,
                    "method_argument": "[Survey].[Survey]",
                    "method": "get_level_members"
                },
                {
                    "unique_name": "[Survey].[Publication]",
                    "has_children_available": False,
                    "caption": "Publication",
                    "has_all_member_available": False,
                    "show_members": True,
                    "method_argument": "[Survey].[Publication]",
                    "method": "get_level_members"
                }
            ]
        }

    def test_get_dimension_options_with_selected_cm(self):

        self._start_django()

        self._attach_permissions_to_group(self.group, 'view_datasource', 'view_calculated_member', 'create_calculated_member')
        for m in self.sa_session.query(MdxMethod).all():
            self.controller.grant_privilege_to(m, MdxMethod.PRIV_VIEW, group=True, target_group=self.group, check_own_privs=False)

        measure_cfg = {
            "available_on_rows": False,
            "show_named_sets": False,
            "has_select_as_group": False,
            "available_as_split_by": False,
            "required": True,
            "available_as_slicer": True,
            "caption": "Measures",
            "fancy_select_module": "iq_fancyselect",
            "available_on_columns": False,
            "show_calculated_members": True,
            "children": [
                {
                    "unique_name": "[Measures].[MeasuresLevel]",
                    "has_children_available": False,
                    "caption": "MeasuresLevel",
                    "has_all_member_available": False,
                    "show_members": True,
                    "method_argument": "[Measures].[MeasuresLevel]",
                    "method": "get_level_members"
                }
            ]
        }

        options = self.controller.get_dimension_options(self.datasource, self.cube, measure_cfg)
        self.assertEqual(len(options.keys()), 2)

        cmc = MdxCalculatedMemberController(self.sa_session, account_id=self.account.id)

        ds = self.datasource
        client = ds.connect(self.sa_session, self.account.id)

        # create a calculation which uses the expression objects and datasource/cube.
        m = self.sa_session.query(MdxMethod).filter(MdxMethod.name=='Aggregate').first()
        state = {
            'objectName': 'MdxMethodItem',
            'loaderName': 'iqube_mdx_method_item',
            'return_type': 'numeric',
            'method_id': m.id,
            'inputs': [{
                'objectName': 'MdxDataSourceExpressionItem',
                'loaderName': 'iqube_datasource_expression_item',
                'return_type': 'set',
                'selection': [simplejson.dumps(x) for x in client.get_level_members('VirtualResponse', '[Survey].Members')[:1]]
            }]
        }
        # create new CM (aggregate)
        cm = cmc.save_or_update_calculated_member(None, 'My Named Set', state, datasource_id=ds.id, cube='Sales')

        selected = MondrianMemberSerialize({
            'calculated_member_id': cm.id,
            'replacements': [],
            'settings': {'label': 'My Test'}
        })

        # get dimension options for measures
        options = self.controller.get_dimension_options(self.datasource, self.cube, measure_cfg, values=[selected])
        self.assertEqual(len(options['Calculated members']), 1)
        self.assertEqual(options['Calculated members'][0]['label'], 'My Test')
        self.assertEqual(options['Calculated members'][0]['value'], selected)

    def test_get_dimension_options(self):

        # start django for iqube to connect for roles info
        self._start_django()

        survey_cfg = self._get_survey_dimension_cfg()

        options = self.controller.get_dimension_options(self.datasource, self.cube, survey_cfg)

        keys = options.keys()
        self.assertEqual(len(keys), 2)
        self.assertEqual(keys[0], 'Survey')
        self.assertEqual(keys[1], 'Publication')

        self.assertEqual(len(options['Survey']), 1)
        v = simplejson.loads(options['Survey'][0]['value'])
        self.assertEqual(v["member_unique_name"], '[Survey].[%s]' % self.survey_id)

        self.assertEqual(len(options['Publication']), 2)
        v = simplejson.loads(options['Publication'][0]['value'])
        self.assertEqual(v["member_unique_name"], '[Survey].[%s].[%s]' % (self.survey_id, self.p1.id))
        v = simplejson.loads(options['Publication'][1]['value'])
        self.assertEqual(v["member_unique_name"], '[Survey].[%s].[%s]' % (self.survey_id, self.p2.id))

        original_account = self.account

        raise SkipTest('SKIP testing user with privileges only on certain publications of a survey since that needs changes')

        # now get another account/group, add privs for it for this survey and create a publication for it
        self.account = self._create_active_account()
        self.group = self.account.groups[0].group
        self._attach_permissions_to_group(self.group, *self.permissions)
        self.controller.grant_privilege_to(self.survey, Survey.all_privileges, group=True, target_group=self.group,
                                           check_own_privs=False)

        self.publish_survey(SurveyPublication.STATUS_SCHEDULED, datetime.now(), anonymous=True, state=self._survey_state)
        p3 = self._publication

        pc = ProjectController(self.session)
        pc.flush_schema(self.datasource.id)

        controller = ConfigurableWidgetController(self.session, account_id=self.account.id)
        options = controller.get_dimension_options(self.datasource, self.cube, survey_cfg)

        keys = options.keys()
        self.assertEqual(len(keys), 2)
        self.assertEqual(keys[0], 'Survey')
        self.assertEqual(keys[1], 'Publication')

        self.assertEqual(len(options['Survey']), 1)
        v = simplejson.loads(options['Survey'][0]['value'])
        self.assertEqual(v["member_unique_name"], '[Survey].[%s]' % self.survey_id)

        self.assertEqual(len(options['Publication']), 1)
        v = simplejson.loads(options['Publication'][0]['value'])
        self.assertEqual(v["member_unique_name"], '[Survey].[%s].[%s]' % (self.survey_id, p3.id))

        # original accpunt still has the same access
        controller = ConfigurableWidgetController(self.session, account_id=original_account.id)
        options = controller.get_dimension_options(self.datasource, self.cube, survey_cfg)

        keys = options.keys()
        self.assertEqual(len(keys), 2)
        self.assertEqual(keys[0], 'Survey')
        self.assertEqual(keys[1], 'Publication')

        self.assertEqual(len(options['Survey']), 1)
        v = simplejson.loads(options['Survey'][0]['value'])
        self.assertEqual(v["member_unique_name"], '[Survey].[%s]' % self.survey_id)

        self.assertEqual(len(options['Publication']), 2)
        v = simplejson.loads(options['Publication'][0]['value'])
        self.assertEqual(v["member_unique_name"], '[Survey].[%s].[%s]' % (self.survey_id, self.p1.id))
        v = simplejson.loads(options['Publication'][1]['value'])
        self.assertEqual(v["member_unique_name"], '[Survey].[%s].[%s]' % (self.survey_id, self.p2.id))

        # lets make another account with the same permissions, but no privs on any survey or publication
        account = self._create_active_account()
        group = account.groups[0].group
        self._attach_permissions_to_group(group, *self.permissions)

        pc = ProjectController(self.session)
        pc.flush_schema(self.datasource.id)

        controller = ConfigurableWidgetController(self.session, account_id=account.id)
        options = controller.get_dimension_options(self.datasource, self.cube, survey_cfg)

        self.assertEqual(len(options), 0)

    def test_aggregators(self):
        raise SkipTest('SKIP test - not all queries have order by statements so can randomly fail')

        from basesoft.web.contrib.i18n.models import Locale
        locales = self.session.query(Locale).filter(Locale.locale.in_(['en_GB', 'nl_NL', 'de_DE'])).order_by(Locale.id).all()
        self.assertEqual(len(locales), 3, 'Not enough locales to test with')
        locale_en = locales[0]
        locale_nl = locales[1]
        locale_de = locales[2]

        # we know the created survey has 2 available locales, 3 itemQAs, each with 1 question and 2 answers (3 elements)

        items = self.survey.itemQAs

        item_ids = []
        item_elements = []
        for item in items:
            item_elements = item_elements + item.elements
            item_ids.append(str(item.id))

        # a new survey was created, run the aggregator and check that all the new items are in the locale agg tables
        agg = AggregateResponsesTask()
        agg.populate()

        db_engine_ro = create_engine(settings.DB_CONNSTR_READONLY, echo=False)

        # keep trac of the fallback translation
        fallbacks = {}
        s = ','.join([str(i) for i in item_ids])

        # item QA locales
        r = db_engine_ro.execute("SELECT item_id, locale_id, label FROM aggSurveyItemQALocale where item_id in (" +\
                                    s + ")")
        rows = r.fetchall()
        # 2 locales and 3 items = 6 rows
        self.assertEqual(len(rows), 2*len(items))
        i = 0
        for item in items:
            for il in item.locales:
                if il.locale_id == locale_nl.id:
                    fallbacks[item.id] = il.label
                self.assertEqual(rows[i], (il.item_id, il.locale_id, il.label))
                i = i + 1
        items_rows_before = rows

        # item QA elements locales
        r = db_engine_ro.execute("SELECT item_id, item_element_id, locale_id, label FROM aggSurveyItemQAElementLocale where item_id in (" +\
                                    s + ")")
        rows = r.fetchall()
        # 2 locales, 3 items with 3 elements each = 18 rows
        self.assertEqual(len(rows), 2*len(item_elements))
        i = 0
        for item_element in item_elements:
            # questions don't have translation
            if item_element.is_question:
                self.assertEqual(rows[i], (item_element.item_id, item_element.id, locale_en.id, '-'))
                i = i + 1
                self.assertEqual(rows[i], (item_element.item_id, item_element.id, locale_nl.id, '-'))
                i = i + 1
            else:
                for il in item_element.locales:
                    self.assertEqual(rows[i], (item_element.item_id, il.item_element_id, il.locale_id, il.label))
                    i = i + 1

        # now lets add another locale to this survey and a translation ------------------------------------------------
        from iqportal.iquest.core.models import SurveyLocale, SurveyItemQALocale
        sl = SurveyLocale()
        sl.survey = self.survey
        sl.locale = locale_de
        self.session.add(sl)

        qal = SurveyItemQALocale()
        qal.item = items[0]
        qal.locale = locale_de
        qal.label = 'german qa'
        # make sure it's the last one when selecting it from agg
        qal.last_updated = datetime.now() + timedelta(seconds=5)
        self.session.add(qal)
        self.session.flush()

        self.survey.stale = 1
        self.session.add(self.survey)
        self.session.flush()
        agg.populate()

        r = db_engine_ro.execute("SELECT item_id, locale_id, label FROM aggSurveyItemQALocale where item_id in (" +\
                                    s + ") order by last_updated, item_id")
        rows = r.fetchall()
        # 3 locales and 3 items = 9 rows
        self.assertEqual(len(rows), 3*len(items))

        # non german translation rows should be the same
        i = 0
        for row in items_rows_before:
            self.assertEqual(rows[i], row)
            i = i + 1
        # rest are the german translations
        i = len(rows) - len(items)
        for item in items:
            if item.id != items[0].id:
                self.assertEqual(rows[i], (item.id, locale_de.id, fallbacks[item.id]))
                i = i + 1
        # last one is the new german one
        self.assertEqual(rows[len(rows)-1], (items[0].id, locale_de.id, 'german qa'))
        items_rows_before = rows

        # lets modify a translation -----------------------------------------------------------------------------------
        qal = self.session.query(SurveyItemQALocale).filter(and_(SurveyItemQALocale.item==items[0],
                                                                 SurveyItemQALocale.locale==locale_en)).first()
        qal.label = 'update english'
        self.session.add(qal)
        # make sure it's the last one when selecting it from agg
        qal.last_updated = datetime.now() + timedelta(seconds=10)
        self.session.flush()
        self.survey.stale = 1
        self.session.add(self.survey)
        self.session.flush()
        agg.populate()

        r = db_engine_ro.execute("SELECT item_id, locale_id, label FROM aggSurveyItemQALocale where item_id in (" +\
                                    s + ") order by last_updated, item_id")
        rows = r.fetchall()
        # nothing was added, just updated
        self.assertEqual(len(rows), 3*len(items))
        # rows should be like before except for the updated English translation (last one)
        for i in range(0, len(rows)-1):
            self.assertEqual(rows[i], items_rows_before[i+1])
        # last one is the updated english one
        self.assertEqual(rows[len(rows)-1], (items[0].id, locale_en.id, 'update english'))

        # modify again this translation, but to '' --------------------------------------------------------------------
        # (a translation cannot be removed by user, no point in testing that)
        qal = self.session.query(SurveyItemQALocale).filter(and_(SurveyItemQALocale.item==items[0],
                                                                 SurveyItemQALocale.locale==locale_en)).first()
        qal.label = ''
        self.session.add(qal)
        self.session.flush()
        self.survey.stale = 1
        self.session.add(self.survey)
        self.session.flush()
        agg.populate()

        r = db_engine_ro.execute("SELECT item_id, locale_id, label FROM aggSurveyItemQALocale where item_id in (" +\
                                    s + ") order by last_updated, item_id")
        rows = r.fetchall()
        # nothing was added, just updated
        self.assertEqual(len(rows), 3*len(items))
        found_it = False
        for row in rows:
            if row[0] == items[0].id and row[1] == locale_en.id:
                self.assertEqual(row, (items[0].id, locale_en.id, fallbacks[items[0].id]))
                found_it = True
        self.assertTrue(found_it)

        # change translation for fallback -----------------------------------------------------------------------------
        # (so in the agg, both fallback and the locale with no translation have to be updated)
        qal = self.session.query(SurveyItemQALocale).filter(and_(SurveyItemQALocale.item==items[1],
                                                                 SurveyItemQALocale.locale==locale_nl)).first()
        qal.label = 'second item nl update'
        self.session.add(qal)
        self.session.flush()
        self.survey.stale = 1
        self.session.add(self.survey)
        self.session.flush()
        agg.populate()

        r = db_engine_ro.execute("SELECT item_id, locale_id, label FROM aggSurveyItemQALocale where item_id in (" +\
                                    s + ") order by last_updated, item_id")
        rows = r.fetchall()
        # nothing was added, just updated
        self.assertEqual(len(rows), 3*len(items))
        found_nl = False
        found_de = False
        for row in rows:
            if row[0] == items[1].id and row[1] == locale_nl.id:
                self.assertEqual(row, (items[1].id, locale_nl.id, 'second item nl update'))
                found_nl = True
            if row[0] == items[1].id and row[1] == locale_de.id:
                self.assertEqual(row, (items[1].id, locale_de.id, 'second item nl update'))
                found_de = True
        self.assertTrue(found_nl)
        self.assertTrue(found_de)

class IQuestReportNPSAggregationTestCase(iquest_unittest.IQuestTestCase):

    def setUp(self):
        self.permissions = ['create_survey', 'view_survey', 'update_survey', 'create_survey_publication',
                            'schedule_survey_publication', 'view_survey_publication_response', 'iquest_nps_access']
        iquest_unittest.IQuestTestCase.setUp(self, self.permissions)
        from iqportal.iquest.core.models import SurveyItemQA
        self.state = {
            "survey": {
                'alter_published_survey': False,
                "name": u"IQPortalTestCase Survey",
                "objectName": u"Survey",
                "dbID": None,
                "domID": u"iquest-canvas-dom-id",
                "has_active_publications": False,

                "availableLocales": ['nl_NL', 'en_GB'],

                "theme": {
                    "module": u"iquest_survey_theme_basesoft",
                    "name": u"BaseSoft",
                    "cssClass": u"survey-basesoft",
                },

                "fallbackLocale": {'locale': u'nl_NL', 'name': u'Dutch' },

                "children": {
                    "blocks": [

                        # just one block
                        {
                            "objectName": u"SurveyBlock",
                            "dbID":None,
                            "domID":u"yui-gen31",
                            "pageEnd": False,

                            "children": {
                                "columns": [
                                    {
                                    "children": {
                                        "items":[{
                                            "objectName":u"ItemQANPS",
                                            "dbID":None,
                                            "poolItemID": None,
                                            "domID":u"yui-gen77",
                                            "questionsAlign": SurveyItemQA.ALIGN_LEFT,
                                            "answersAlign": SurveyItemQA.ALIGN_BOTTOM,
                                            "trailingQuestions": False,
                                            "type": SurveyItemQA.TYPE_SINGLE_CHOICE,
                                            "layout": None,
                                            "showTitle": True,
                                            "layoutAttributes": {},
                                            "required": False,
                                            "filters": [],
                                            "branching": [],

                                            "locales": {
                                                'nl_NL': {
                                                    u'label': u'Would you nl?',
                                                },
                                                'en_GB': {
                                                    u'label': u'Would you?'
                                                }
                                            },

                                            "children": {
                                                "questions":[{
                                                    "objectName":u"SurveyQuestion",
                                                    "dbID":None,
                                                    "poolElementID": None,
                                                    "domID":u"yui-gen205",
                                                    "locales": {},
                                                }],

                                                "answers":[{
                                                    "objectName":u"SurveyAnswer",
                                                    "dbID":None, "poolElementID": None, "domID":u"yui-gen79",
                                                    "locales": {
                                                        'nl_NL': {u'label': u'No nl', u'trailing_label': None},
                                                        'en_GB': {u'label': u'No', u'trailing_label': None}
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer", "dbID":None, "domID":u"yui-gen84",
                                                    "locales": {
                                                        'nl_NL': {},
                                                        'en_GB': {}
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer", "dbID":None, "domID":u"yui-gen85",
                                                    "locales": {
                                                        'nl_NL': {},
                                                        'en_GB': {}
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer", "dbID":None, "domID":u"yui-gen86",
                                                    "locales": {
                                                        'nl_NL': {},
                                                        'en_GB': {}
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer", "dbID":None, "domID":u"yui-gen87",
                                                    "locales": {
                                                        'nl_NL': {},
                                                        'en_GB': {}
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer", "dbID":None, "domID":u"yui-gen88",
                                                    "locales": {
                                                        'nl_NL': {u'label': u'Maybe nl', u'trailing_label': None},
                                                        'en_GB': {u'label': u'Maybe', u'trailing_label': None}
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer", "dbID":None, "domID":u"yui-gen89",
                                                    "locales": {
                                                        'nl_NL': {},
                                                        'en_GB': {}
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer", "dbID":None, "domID":u"yui-gen90",
                                                    "locales": {
                                                        'nl_NL': {},
                                                        'en_GB': {}
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer", "dbID":None, "domID":u"yui-gen91",
                                                    "locales": {
                                                        'nl_NL': {},
                                                        'en_GB': {}
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer", "dbID":None, "domID":u"yui-gen92",
                                                    "locales": {
                                                        'nl_NL': {},
                                                        'en_GB': {}
                                                    },
                                                },
                                                {
                                                    "objectName":u"SurveyAnswer", "dbID":None, "domID":u"yui-gen93",
                                                    "locales": {
                                                        'nl_NL': {u'label': u'Yes nl', u'trailing_label': None},
                                                        'en_GB': {u'label': u'Yes', u'trailing_label': None}
                                                    },
                                                }
                                                ]
                                            }
                                        }]
                                    },
                                    "objectName":u"SurveyBlockColumn",
                                    "dbID":None,
                                    "domID":u"yui-gen53",
                                    "width":100
                                    }
                                ],
                            }
                        }
                    ]
                }
            }
        }
        self.survey_id = self.publish_survey(SurveyPublication.STATUS_SCHEDULED, datetime.now(), anonymous=True, state=self.state)
        self.survey = self.session.query(Survey).get(self.survey_id)
        self.p1 = self._publication

        self.controller = ConfigurableWidgetController(self.session, account_id=self.account.id)

        datasource = self.session.query(DataSource).filter(func.substr(DataSource.name, -6) == 'IQuest').all()
        if len(datasource) == 0:
            raise Exception('No IQuest datasource')
        self.datasource = datasource[0]

        pc = ProjectController(self.session)
        pc.flush_schema(self.datasource.id)

        self.cube = 'Response'

    def test_nps_aggregators(self):

        from basesoft.web.contrib.i18n.models import Locale
        locales = self.session.query(Locale).filter(Locale.locale.in_(['en_GB', 'nl_NL', 'de_DE'])).order_by(Locale.id).all()
        self.assertEqual(len(locales), 3, 'Not enough locales to test with')
        locale_en = locales[0]
        locale_nl = locales[1]
        locale_de = locales[2]

        # we know the created survey has 2 available locales, 1 itemQA with 1 question and 11 answers (12 elements)

        items = self.survey.itemQAs

        item_ids = []
        item_elements = []
        for item in items:
            item_elements = item_elements + item.elements
            item_ids.append(str(item.id))

        # a new survey was created, run the aggregator and check that all the new items are in the locale agg tables
        agg = AggregateResponsesTask()
        agg.populate()

        db_engine_ro = create_engine(settings.DB_CONNSTR_READONLY, echo=False)

        # keep trac of the fallback translation
        fallbacks = {}
        s = ','.join([str(i) for i in item_ids])

        # item QA locales
        r = db_engine_ro.execute("SELECT item_id, locale_id, label FROM aggSurveyItemQALocale where item_id in (" +\
                                    s + ")")
        rows = r.fetchall()
        # 2 locales and 1 item = 2 rows
        self.assertEqual(len(rows), 2*len(items))
        i = 0
        for item in items:
            for il in item.locales:
                if il.locale_id == locale_nl.id:
                    fallbacks[item.id] = il.label
                self.assertEqual(rows[i], (il.item_id, il.locale_id, il.label))
                i = i + 1
        items_rows_before = rows

        # item QA elements locales
        r = db_engine_ro.execute("SELECT item_id, item_element_id, locale_id, label FROM aggSurveyItemQAElementLocale where item_id in (" +\
                                    s + ") order by item_element_id, locale_id")
        rows = r.fetchall()
        # 2 locales, 1 item with 12 elements = 24 rows
        self.assertEqual(len(rows), 2*len(item_elements))
        i = 0
        for item_element in item_elements:
            # questions don't have translation
            if item_element.is_question:
                self.assertEqual(rows[i], (item_element.item_id, item_element.id, locale_en.id, '-'))
                i = i + 1
                self.assertEqual(rows[i], (item_element.item_id, item_element.id, locale_nl.id, '-'))
                i = i + 1
            else:
                if item_element.position == 0:
                    self.assertEqual(rows[i], (item_element.item_id, item_element.id, locale_en.id, 'No'))
                    i = i + 1
                    self.assertEqual(rows[i], (item_element.item_id, item_element.id, locale_nl.id, 'No nl'))
                    i = i + 1
                elif item_element.position == 5:
                    self.assertEqual(rows[i], (item_element.item_id, item_element.id, locale_en.id, 'Maybe'))
                    i = i + 1
                    self.assertEqual(rows[i], (item_element.item_id, item_element.id, locale_nl.id, 'Maybe nl'))
                    i = i + 1
                elif item_element.position == 10:
                    self.assertEqual(rows[i], (item_element.item_id, item_element.id, locale_en.id, 'Yes'))
                    i = i + 1
                    self.assertEqual(rows[i], (item_element.item_id, item_element.id, locale_nl.id, 'Yes nl'))
                    i = i + 1
                else:
                    self.assertEqual(rows[i], (item_element.item_id, item_element.id, locale_en.id, str(item_element.position)))
                    i = i + 1
                    self.assertEqual(rows[i], (item_element.item_id, item_element.id, locale_nl.id, str(item_element.position)))
                    i = i + 1
