# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from datetime import datetime, timedelta

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.contrib.contact.controllers import ContactController
from basesoft.web.core.exceptions import SecurityException, PublicUserException,\
    PermissionDeniedException

from basesoft.web.core.util.controllers import StateManager

from iqportal.iquest.core import schedule
from iqportal.iquest.core.models import Survey, SurveyPublication
from iqportal.iquest.compose.controllers import ComposeController, PublicationStateController, \
    SurveyVariableStateController
from iqportal.iquest.core.mailing import MailController

class ScheduleTest(iqportal_unittest.IQPortalTestCase):
    """
    """

    def setUp(self):
        """
        """
        self.connect_db()

        #create an account so we can check permission with it
        self.account = self._create_active_account()

        # create the necessary controllers
        self.group = self.account.groups[0].group

        # initialise compose controller
        from basesoft.web.contrib.i18n import models as i18n_models
        self.locale = self.session.query(i18n_models.Locale).filter_by(locale='en_GB').one()
        self.survey_controller = ComposeController(self.session, self.locale, account_id=self.account.id)

        # init the schedule state controller
        self.publication_controller = PublicationStateController(self.session, StateManager(self.locale, self.account.id, None))

        # give current group access to create surveys
        self._attach_permissions_to_group(self.group, 'create_survey', 'view_survey', 'schedule_iquest_mail',
                                          'view_contact_list',
                                          'create_survey_variable_set', 'view_survey_variable_set')

        # create some contact lists and mails
        self.setUpContactLists()
        self.setUpMails()

    def setUpContactLists(self):
        """
        Creates 3 contacts and 2 contact lists
        """
        self._attach_permissions_to_group(self.group, 'link_to_contact_list', 'create_contact', 'create_contact_list')
        contact_controller = ContactController(self.session, account_id=self.account.id)

        person = {'first_name': u'John',
                'middle_name': u'',
                'last_name': u'Smith',
                'title': u'mr',
                'gender': None,
                'email': u'john@mail-test.basesoft.com',
                'position': u'assistent',
                'department': u'marketing',
                'phone': u'0040123456789',
                'mobile_phone': '',
                'fax': '',
                'birth_date': '1978-10-10',
                'address_data': {'country': '181',
                                 'postcode': u"400295",
                                 'street': u" Bihorului 10",
                                 'city': u"Cluj Napoca",
                                 'state': u"Cluj"
                                }
               }

        c1 = contact_controller.save_or_update_contact(None, person, None, self.group.id)
        c2 = contact_controller.save_or_update_contact(None, person, None, self.group.id)
        c3 = contact_controller.save_or_update_contact(None, person, None, self.group.id)
        c4 = contact_controller.save_or_update_contact(None, None, None, self.group.id)

        self.cl1 = contact_controller.save_or_update_contact_list(None, u'Contact List 1', self.group.id)
        contact_controller.link_contacts([c1, c2], self.cl1)

        self.cl2 = contact_controller.save_or_update_contact_list(None, u'Contact List 2', self.group.id)
        contact_controller.link_contacts([c2, c3], self.cl2)

        # contact list that doesn't have any contact with email address
        self.cl3 = contact_controller.save_or_update_contact_list(None, u'Contact List 3', self.group.id)
        contact_controller.link_contacts([c4], self.cl3)


    def get_mail_state(self, name):
        return {
            "mail": {
                "mail_from_text": "MockMailer",
                "name": name,
                "objectName": u"IQuestMail",
                "dbID": None,
                "locales": {
                    'nl_NL': {
                        'subject': u'dutch mail subject',
                        'content': u'dutch mail content'
                    },
                },
                "fallbackLocale": {'value': u'en_GB', 'name': u'English' },
            }
        }

    def setUpMails(self):

        self._attach_permissions_to_group(self.group, 'create_mail', 'create_iquest_mail')
        controller = MailController(self.session, self.locale, account_id=self.account.id)
        self.mail1 = controller.save_state(self.get_mail_state(u'IQuest Mail 1'), self.group)

    def _publish_survey(self, status, begin_date, end_date=None, contact_lists=None, anonymous=False):
        """
        Helper method for creating a survey state
        """
        if contact_lists is None:
            contact_lists = []

        survey_state = {
            "survey": {
                "name": u"My Pretty Survey",
                "objectName": u"Survey",
                "dbID": None,
                "domID": u"iquest-canvas-dom-id",

                "theme": {
                    "module": u"iquest_survey_theme_basesoft",
                    "name": u"BaseSoft",
                    "cssClass": u"survey-basesoft",
                },

                "availableLocales": ['en_GB'],

                "fallbackLocale": {'locale': u'en_GB', 'name': u'English' },
                "children": {'blocks': []}
            }
        }

        survey_map = self.survey_controller.save_state(survey_state)
        survey_id = survey_map['iquest-canvas-dom-id']

        # set some variable sets for the survey
        vs_controller = SurveyVariableStateController(self.session, StateManager(self.locale, self.account.id, None))
        vs_id = vs_controller.save({'dbID': None,
                                    'domID': None,
                                    'survey_id': survey_id,
                                    'name': u'test variable set'})

        publish_state = {
            'survey_id': survey_id,
            "dbID": None,
            "domID": u'publish_1',
            "begin_date": begin_date,
            "end_date": end_date,
            "exit_url": None,
            "label": u"publication 1",
            "show_page_nrs": True,
            "variable_set_id": vs_id,
            "anonymous": anonymous,
            "contact_lists": contact_lists,
            "status": status,
            "mailings": [{
                "dbID": None,
                "domID": u'publish_1-mailing_1',
                "mail_id": self.mail1.id,
                "mail_from_text": u'Test IQuest',
                "send_date": begin_date,
                "contact_lists": contact_lists
            }]
        }

        # save publication
        self._publication = self.publication_controller.save(publish_state)

        return publish_state

    def test_load_save_no_rights(self):

        begin_date = datetime(2009, 4, 16, 14, 3)

        # try to save
        self.assertRaises(PermissionDeniedException,
                          self._publish_survey,
                          status = SurveyPublication.STATUS_NEW,
                          begin_date = begin_date)

        self._attach_permissions_to_group(self.group, 'create_survey_publication')
        state = self._publish_survey(SurveyPublication.STATUS_NEW, begin_date)

        publication = self._publication
        # try to load
        self.assertRaises(PermissionDeniedException,
                          self.publication_controller.load,
                          id = publication.id)

    def test_load_save(self):
        """
        Simple test of what goes in must come out
        """

        self._attach_permissions_to_group(self.group, 'create_survey_publication', 'view_survey_publication')

        begin_date = datetime(2009, 4, 16, 14, 3)

        # set survey publications number limit to 1 for this license so we can test the usage options check
        self._set_license_usage(self.group.license, 'max_number_of_survey_publications', 1)
        # first save is ok
        state = self._publish_survey(SurveyPublication.STATUS_NEW, begin_date)
        # second one should raise an exception cuz the maximum limit of total survey publications was already reached
        self.assertRaises(PermissionDeniedException, self._publish_survey, SurveyPublication.STATUS_NEW, begin_date)

        # assert the first publication
        survey = self.session.query(Survey).get(state['survey_id'])
        self.assertEqual(len(survey.publications), 1)

        # assert loaded publication state
        publication = survey.publications[0]
        state = self.publication_controller.load(publication.id)

        expected_state = {
            'dbID': publication.id,
            'domID': None,
            'begin_date': begin_date,
            'end_date': None,
            'exit_url': None,
            'label': 'publication 1',
            "show_page_nrs": True,
            'show_publication_label': True,
            'anonymous': False,
            "variable_set_id": survey.variable_sets[0].id,
            'anonymous_hash': publication.anonymous_hash,
            'contact_lists': [],
            'viewer_print': False,
            'viewer_pdf': False,
            'minimum_respondents': None,
            'multi_session_setting': 1,
            'show_compose_label': True,
            'show_end_message': 0,
            'maximum_market_share': None,
            'minimum_responses_for_respondent': 1,
            'mailings': [{
                'mail_id': publication.mailings[0].mail.id,
                'mail_from_text': u'Test IQuest',
                'dbID': state['mailings'][0]['dbID'], # impossible to know this one.. get it from the saved state
                'domID': None,
                'send_date': begin_date,
                'contact_lists': [],
                'status': 0L,
                'filter':0L,
                'mail_name': 'IQuest Mail 1'
            }],
            'status': SurveyPublication.STATUS_NEW,
            'survey_id': state['survey_id'],
            'polymorphic_type': publication.polymorphic_type
        }

        self.assertEqual(state, expected_state)

    def test_status_new(self):
        """
        The scheduler should not change the status of survey in NEW state
        """

        self._attach_permissions_to_group(self.group, 'create_survey_publication')

        # save a new survey and leave the state in STATUS_NEW
        state = self._publish_survey(SurveyPublication.STATUS_NEW, datetime.now())

        # normally we would wait for the scheduler to publish the survey but here just call it directly
        # so it will do its work.
        schedule.PublishSurveysTask().execute()

        # assert that the survey state didnt change
        publication = self.session.query(Survey).get(state['survey_id']).publications[0]
        self.assertEquals(publication.status, SurveyPublication.STATUS_NEW)

    def test_schedule_publication_no_rights(self):
        """
        Tests setting the status of a publication to scheduled from new without having the required rights
        """

        self._attach_permissions_to_group(self.group, 'create_survey_publication')
        state = self._publish_survey(SurveyPublication.STATUS_NEW, datetime.now())
        publication_id = self.session.query(Survey).get(state['survey_id']).publications[0].id
        state['dbID'] = publication_id
        state['status'] = SurveyPublication.STATUS_SCHEDULED

        self.assertRaises(PermissionDeniedException,
                          self.publication_controller.save,
                          state = state)

    def test_schedule_publication(self):
        """
        Tests setting the status of a publication to scheduled from new
        """
        self._attach_permissions_to_group(self.group, 'create_survey_publication', 'schedule_survey_publication',
                                          'update_survey_publication')

        state = self._publish_survey(SurveyPublication.STATUS_NEW, datetime.now())
        publication_id = self._publication.id
        state['dbID'] = publication_id
        state['status'] = SurveyPublication.STATUS_SCHEDULED

        publication = self.publication_controller.save(state)

        self.assertEquals(publication.id, publication_id)
        self.assertEquals(publication.status, SurveyPublication.STATUS_SCHEDULED)

    def test_update_online_publication(self):
        """
        In case the publication is ONLINE the user can only update end_date, label, show_page_nrs,
        add new mailings and, in case the existing mailings have send_date > now, edit the existing mailings
        """
        self._attach_permissions_to_group(self.group, 'create_survey_publication', 'update_survey_publication',
                                          'schedule_survey_publication')
        begin_date = datetime(2009, 8, 16, 14, 3)
        state = self._publish_survey(SurveyPublication.STATUS_ONLINE, begin_date)
        publication_id = self._publication.id
        state['dbID'] = publication_id

        # end_date can be modified
        end_date = datetime(2010, 9, 1, 4, 3)
        state['end_date'] = end_date
        publication = self.publication_controller.save(state)
        self.assertEquals(publication.id, publication_id)
        self.assertEquals(publication.end_date, end_date)

        state['begin_date'] = datetime.now()
        self.assertRaises(PublicUserException, self.publication_controller.save, state = state)
        state['anonymous'] = True
        self.assertRaises(PublicUserException, self.publication_controller.save, state = state)
        # this will raise an exception cuz the mailings date is not after the time that the publication went online.
        state['mailings'][0]['mail_id'] = 1
        self.assertRaises(PublicUserException, self.publication_controller.save, state = state)

    def test_adding_additional_contacts_to_online_publication(self):
        """
        We should be able to add additional contacts to an online publication. New invitees have to
        be created for the contacts that are new.
        Note, contact lists cannot be removed from an online publication.
        """
        self._attach_permissions_to_group(self.group, 'create_survey_publication', 'update_survey_publication',
                                          'view_survey_publication', 'schedule_survey_publication')
        begin_date = datetime(2009, 8, 16, 14, 3)
        state = self._publish_survey(SurveyPublication.STATUS_SCHEDULED, begin_date, contact_lists=[self.cl1.id])
        publication_id = self._publication.id

        schedule.PublishSurveysTask().execute() # make sure it gets put online.
        schedule.CreatePublicationInviteesTask().execute()

        state['dbID'] = publication_id
        state['status'] = SurveyPublication.STATUS_ONLINE
        state['contact_lists'] = [self.cl1.id, self.cl2.id]
        publication = self.publication_controller.save(state)
        schedule.CreatePublicationInviteesTask().execute()
        invitee_contacts = [i.contact for i in publication.invitees]
        for contact in self.cl1.contacts:
            self.assertTrue(contact in invitee_contacts)
        for contact in self.cl2.contacts:
            self.assertTrue(contact in invitee_contacts)

        state['contact_lists'] = [self.cl2.id]
        self.assertRaises(PublicUserException, self.publication_controller.save, state=state)

    def test_update_finished_publication(self):
        """
        A finished publication cannot be updated anymore
        """
        self._attach_permissions_to_group(self.group, 'create_survey_publication', 'update_survey_publication',
                                         'schedule_survey_publication')
        state = self._publish_survey(SurveyPublication.STATUS_FINISHED, datetime.now())
        publication_id = self._publication.id
        state['dbID'] = publication_id
        # try to update
        self.assertRaises(PublicUserException, self.publication_controller.save, state = state)

    def test_mark_as_archived(self):
        """
        Marking a publication as deleted.
        """
        self._attach_permissions_to_group(self.group, 'create_survey_publication', 'view_survey_publication', 'schedule_survey_publication')
        state = self._publish_survey(SurveyPublication.STATUS_SCHEDULED, datetime.now())
        publication = self._publication

        self.assertRaises(PermissionDeniedException, self.publication_controller.mark_as_archived, SurveyPublication, publication.id)

        self._attach_permissions_to_group(self.group, 'update_survey_publication')

        self.publication_controller.mark_as_archived(SurveyPublication, publication.id)

        pub = self.session.query(Survey).get(state['survey_id']).publications
        self.assertTrue(len(pub)==0)

    def test_status_scheduled(self):
        """
        The scheduler should set the status of the survey to ONLINE
        """
        self._attach_permissions_to_group(self.group, 'create_survey_publication', 'schedule_survey_publication')
        today = datetime.now()
        tomorrow = today+timedelta(days=1)

        state = self._publish_survey(SurveyPublication.STATUS_SCHEDULED, datetime.now(), tomorrow)

        # publish the surveys with the scheduler method
        schedule.PublishSurveysTask().execute()

        # check to see if the status was updated to online
        publication = self._publication
        self.assertEquals(publication.status, SurveyPublication.STATUS_ONLINE)

        # trigger the finish surveys task
        schedule.FinishSurveysTask().execute()

        # check to see if the status is still online, cuz the end date was tomorrow
        survey = self.session.query(Survey).get(state['survey_id'])
        publication = survey.publications[0]
        self.assertEquals(publication.status, SurveyPublication.STATUS_ONLINE)

        # manually update the publication to end now
        publication.end_date = datetime.now()
        self.session.add(publication)
        self.session.flush()

        # trigger the finish surveys method
        schedule.FinishSurveysTask().execute()

        # check to see if the status was changed to finished.. expire the publication cuz it was updated in another thread
        survey = self.session.query(Survey).get(state['survey_id'])
        publication = survey.publications[0]
        self.session.expire(publication)

        # assert finished status
        self.assertEquals(publication.status, SurveyPublication.STATUS_FINISHED)

    def test_single_contact_list(self):
        """
        This test if the contact from the attached contact list are added to the publication.
        """
        self._attach_permissions_to_group(self.group, 'create_survey_publication', 'schedule_survey_publication')

        # attaching a contact list with no email address, should raise an exception
        self.assertRaises(PublicUserException,
                          self._publish_survey,
                          status=SurveyPublication.STATUS_SCHEDULED,
                          begin_date=datetime.now(),
                          contact_lists=[self.cl3.id])

        state = self._publish_survey(SurveyPublication.STATUS_SCHEDULED, datetime.now(), contact_lists=[self.cl1.id])

        schedule.PublishSurveysTask().execute()
        schedule.CreatePublicationInviteesTask().execute()

        publication = self._publication

        self.assertEquals(len(publication.invitees), len(self.cl1.contacts))

    def test_overlapping_contact_list(self):
        """
        This tests if the distinct contact of the attached contact lists are added to the publication.
        """
        self._attach_permissions_to_group(self.group, 'create_survey_publication', 'schedule_survey_publication')
        state = self._publish_survey(SurveyPublication.STATUS_SCHEDULED, datetime.now(), contact_lists=[self.cl1.id, self.cl2.id])

        schedule.PublishSurveysTask().execute()
        schedule.CreatePublicationInviteesTask().execute()

        survey = self.session.query(Survey).get(state['survey_id'])

        compare = self.cl1.contacts
        compare += [c for c in self.cl2.contacts if c not in compare]

        self.assertEquals(len(survey.publications[0].invitees), len(compare) )

    def test_anonymous(self):
        """
        If a survey is anonymous we expect one anonymous invitee to be created and entries for all contact in the contact lists. When
        a survey is not anonymous only the contacts from the contact lists are used.
        """

        self._attach_permissions_to_group(self.group, 'create_survey_publication', 'schedule_survey_publication')
        # publish anonymous
        state = self._publish_survey(SurveyPublication.STATUS_SCHEDULED, datetime.now(), contact_lists=[self.cl1.id, self.cl2.id], anonymous=True)
        schedule.PublishSurveysTask().execute()
        schedule.CreatePublicationInviteesTask().execute()
        survey = self.session.query(Survey).get(state['survey_id'])

        # assert that all the contacts are included (no doubles) plus the anonymous invite
        compare = self.cl1.contacts
        compare += [c for c in self.cl2.contacts if c not in compare]
        self.assertEquals(len(survey.publications[0].invitees), len(compare) + 1)

        # check if the hash for the anonymous invitee is the same as it was set when the publication was created
        i_hash = ""
        for invitee in survey.publications[0].invitees:
            if invitee.contact_id is None:
                i_hash = invitee.invitee_hash
                break
        a_hash = survey.publications[0].anonymous_hash
        self.assertEquals(a_hash, i_hash)


# below is for running the test from eclipse
if __name__ == '__main__':
    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_compose.py', 'iqportal.iquest.test.unit.test_schedule.ScheduleTest.test_load_save']
    run()
    sys.exit()
