# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.
import os
from iqportal.tnt.core.controllers import TntController

from sqlalchemy.sql.expression import and_
from sqlalchemy.orm.exc import NoResultFound
from basesoft.web.core.util.db import Session

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.contrib.i18n.models import Country
from basesoft.web.contrib.account.models import Account
from basesoft.web.contrib.contact.models import Person, Contact

from iqportal.tnt.clm.models import TntCLMCustomer, TntCLMLocale, TntCLMCompany, TntCLMContact
from iqportal.tnt.clm.controllers import TntCLMContactController
from iqportal.tnt.test.controllers import TntWaveController
from iqportal.tnt.core.models import TntDepot, TntDepotAlias
from basesoft.web.contrib.contact.controllers import ImportContactController
from twisted.trial.unittest import SkipTest

class MockSimpleContactController(TntCLMContactController):
    """
    Mock object to test protected methods of the SimpleContactController
    """
    def save_or_update_company(self, id, company_data):
        return self._save_or_update_company(id, company_data)

class TntCLMCompanyControllerTestCase(iqportal_unittest.IQPortalTestCase):
    """
    iqportal.tnt.test.unit.test_clm_company_controller.TntCLMCompanyControllerTestCase
    """

    def setUp(self):
        self.connect_db()
        self.account = self._create_active_account()
        self.group = self.account.groups[0].group

        # we populate necessary tnt account data
        account = self.session.query(Account).filter(Account.name=='sysadmin').one()
        c = TntWaveController(self.session, account_id=account.id)
        license, account = c.populate_tnt_account()

        self.controller = MockSimpleContactController(self.session, account_id=self.account.id)

    def test_save_or_update_company(self):
        company_data =  {'name': u'clm company',
                        'vat_no': u'NL 8104.49.493.B01',
                        'website': u'http://www.basesoft.com',
                        'polymorphic_type': 'TntCLMCompany',
                        'contact_person_data': {'email': u'john@mail-test.basesoft.com'},
                        'subsector': 1,
                        'imp_code': 2,
                        'revenue_band': u'A',
                        'locale_id': 1,
                        'account_number': 1234567890,
                        }
        company = self.controller.save_or_update_company(None,company_data)

        for key in company_data:
            if key not in ["contact_person_data", "polymorphic_type", "account_number"]:
                self.assertEqual(getattr(company, key),
                                 company_data[key])
                continue
            for k in company_data["contact_person_data"]:
                if(k != 'address_data'):
                    self.assertEqual(unicode(getattr(company.contact_person, k)),
                        company_data["contact_person_data"][k])


class TntCLMContactImportTestCase(iqportal_unittest.IQPortalTestCase):
    """
    iqportal.tnt.test.unit.test_clm_company_controller.TntCLMContactImportTestCase
    """

    def setUp(self):
        self.connect_db()

        account = self.session.query(Account).filter(Account.name=='sysadmin').one()
        # we populate necessary tnt account data
        c = TntWaveController(self.session, account_id=account.id)
        license, account = c.populate_tnt_account()
        self.account = account

        self.group = account.groups[0].group

        # create the file
        self.filepointer = "/tmp/test_import_contacts.csv"
        f = open(self.filepointer, 'w')
        f.write('%s\n%s\n%s' % (
            '"CONTACTPERSON","EMAIL","LANGUAGECODE","REVENUE","GENDER","LOCATION","COUNTRY","ACCOUNTNR","DEPOTCODE","COMPANYNAME", "ENTITY"',
            '"Bart-jan Baard","bartjan@mail-test.basesoft.com","EN","B","M","I","GB","84773653","CW4","Baard International","Express"',
            '"Bart-jan Baard","bartjan@mail-test.basesoft.com","EN","B","M","I","GB","84773653","CW4","Baard International","Express"',
            )
        )
        f.close()

        self.controller = ImportContactController(self.filepointer, 'test.csv', self.session, account.id, 'TntCLMContact')
        self.controller.set_file_lines()
        self.controller.field_mappings = [
            {"file_header": "CONTACTPERSON", "column": "last_name", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "EMAIL", "column": "email", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "LANGUAGECODE", "column": "language", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "REVENUE", "column": "revenue_band", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "GENDER", "column": "gender", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "LOCATION", "column": "location", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "COUNTRY", "column": "company_country_iso_3166_a2", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "ACCOUNTNR", "column": "account_number", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "DEPOTCODE", "column": "depot", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "COMPANYNAME", "column": "name", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "ENTITY", "column": "entity", "target": self.controller.COLUMN_IS_COMPANY_INFO},
       ]

    def _create_customer(self, account_no, country):

        country = self.session.query(Country).filter(Country.iso_3166_a2==country).one()
        try:
            customer = self.session.query(TntCLMCustomer).filter(
                and_(
                    TntCLMCustomer.account_number==account_no,
                    TntCLMCustomer.country==country
                )
            ).one()
        except NoResultFound:
            customer = TntCLMCustomer()
            customer.country = country
            customer.account_number = account_no
            self.session.add(customer)
            self.session.flush()

        return customer

    def _create_company(self, revenue, depot, locale, name, customer):

        depot = self.session.query(TntDepot).filter(TntDepot.default_code==depot).all()
        if len(depot) == 0:
            depot = self.session.query(TntDepot).join(TntDepotAlias).filter(TntDepotAlias.code==depot).all()
        depot = depot[0]

        locale = self.session.query(TntCLMLocale).filter(TntCLMLocale.label==locale).one()
        try:
            company = self.session.query(TntCLMCompany).filter(
                and_(
                    TntCLMCompany.depot==depot,
                    TntCLMCompany.revenue_band==revenue,
                    TntCLMCompany.locale==locale,
                    TntCLMCompany.customer==customer,
                )
            ).one()
        except NoResultFound:
            company = TntCLMCompany()
            company.name = name
            company.depot = depot
            company.revenue_band = revenue
            company.locale = locale
            company.customer = customer
            self.session.add(company)
            self.session.flush()

        return company

    def _create_person(self, last_name, email, create_new=False):
        if create_new:
            person = None
        else:
            person = self.session.query(Person).filter(and_(Person.last_name==last_name, Person.email==email)).first()
        if person is None:
            person = Person()
            person.last_name = last_name
            person.email = email
            self.session.add(person)
            self.session.flush()
        return person

    def _create_contact(self, person, company, controller=None):
        if controller is None:
            controller = self.controller
        try:
            contact = self.session.query(TntCLMContact).filter(and_(TntCLMContact.person==person, TntCLMContact.company==company)).one()
        except NoResultFound:
            contact = TntCLMContact()
            contact.person = person
            contact.company = company
            controller.save_default_privilege_object_group(contact)
            self.session.add(contact)
            self.session.flush()

        return contact

    def test_simple_import(self):
        """
        Just use the file provided for import.
        """
        contacts = self.controller.import_contacts()
        # 2 rows in the file, 2 contacts processed
        self.assertEqual(len(contacts),2)
        # but just one distinct
        ids = []
        for c in contacts:
            if c.id not in ids:
                ids.append(c.id)
        self.assertEqual(len(ids),1)

    def test_import_entity(self):
        # create the file
        self.filepointer = "/tmp/test_import_contacts.csv"
        f = open(self.filepointer, 'w')
        f.write('%s\n%s\n%s' % (
            '"CONTACTPERSON","EMAIL","LANGUAGECODE","REVENUE","GENDER","LOCATION","COUNTRY","ACCOUNTNR","DEPOTCODE","COMPANYNAME","ENTITY"',
            '"Bart-jan 1","bartjan1@mail-test.basesoft.com","EN","B","M","I","GB","84773653","CW4","Baard International1","Express"',
            '"Bart-jan 2","bartjan2@mail-test.basesoft.com","EN","B","M","D","GB","84773650","CW4","Baard International2","Express"',
            )
        )
        f.close()

        self.controller = ImportContactController(self.filepointer, 'test.csv', self.session, self.account.id, 'TntCLMContact')
        self.controller.set_file_lines()
        self.controller.field_mappings = [
            {"file_header": "CONTACTPERSON", "column": "last_name", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "EMAIL", "column": "email", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "LANGUAGECODE", "column": "language", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "REVENUE", "column": "revenue_band", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "GENDER", "column": "gender", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "LOCATION", "column": "location", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "COUNTRY", "column": "company_country_iso_3166_a2", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "ACCOUNTNR", "column": "account_number", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "DEPOTCODE", "column": "depot", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "COMPANYNAME", "column": "name", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "ENTITY", "column": "entity", "target": self.controller.COLUMN_IS_COMPANY_INFO},
        ]

        #print self.controller.validate()
        contacts = self.controller.import_contacts()
        # 2 rows in the file, 2 contacts processed
        self.assertEqual(len(contacts),2)
        # 2 distinct
        ids = []
        for c in contacts:
            if c.id not in ids:
                ids.append(c.id)

        self.assertEqual(len(ids), 2)
        # each should have different locales even though same service and country
        self.assertEqual(contacts[0].company.locale.label, 'GBENI')
        self.assertEqual(contacts[1].company.locale.label, 'GBEND')

    def test_existing_customer(self):
        """
        Create a Customer before using the import file.
        """
        customer = self._create_customer("84773653", "GB")

        contacts = self.controller.import_contacts()
        ids = []
        for c in contacts:
            if c.id not in ids:
                ids.append(c.id)
        self.assertEqual(len(ids),1)
        for c in contacts:
            if c.company.customer == customer:
                return
        self.fail('No contact in result for customer %s' % customer)

    def test_existing_customer_and_company(self):
        """
        Create a Customer and Company before using the import file.
        """
        customer = self._create_customer("84773653", "GB")
        company = self._create_company("B", "CW4", "GBENI", "Baard International", customer)

        contacts = self.controller.import_contacts()
        ids = []
        for c in contacts:
            if c.id not in ids:
                ids.append(c.id)
        self.assertEqual(len(ids),1)
        for c in contacts:
            if c.company == company and c.company.customer == customer:
                return
        self.fail('No contact in result for customer %s and company %s' % (customer, company))

    def test_existing_contact(self):
        """
        Create a Customer, Company, Person and Contact entry before using the import file.
        """

        customer = self._create_customer("84773653", "GB")
        company = self._create_company("B", "CW4", "GBENI", "Baard International", customer)
        person = self._create_person("Bart-jan Baard", "bartjan@mail-test.basesoft.com")
        contact = self._create_contact(person, company)

        contacts = self.controller.import_contacts()
        ids = []
        for c in contacts:
            if c.id not in ids:
                ids.append(c.id)
        self.assertEqual(len(ids),1)
        for c in contacts:
            if c == contact and c.company == company and c.person == person:
                return
        self.fail('No contact in result for contact %s' % contact)

    def test_ticket_3377(self):
        # This will test that a person/company unique information for TNT only looks for persons/companies this group has access to.

        # make a controller with a different licensee.
        account = self._create_active_account()
        controller = MockSimpleContactController(self.session, account_id=account.id)

        # save the same person information
        customer = self._create_customer("84773653", "GB")
        company = self._create_company("B", "CW4", "GBENI", "Baard International", customer)
        person = self._create_person("Bart-jan Baard", "bartjan@mail-test.basesoft.com", True)
        self._create_contact(person, company, controller)

        contacts = self.session.query(Contact).join(Person, Contact.person_id==Person.id).filter(and_(Person.last_name=="Bart-jan Baard", Person.email=="bartjan@mail-test.basesoft.com")).all()
        not_my_persons = []
        for c in contacts:
            if not self.controller.has_privileges_on_object(c, Contact.PRIV_VIEW):
                not_my_persons.append(c.person)

        # import contacts defined in the setUp
        my_contacts = self.controller.import_contacts()
        for c in my_contacts:
            self.assertTrue(c.person not in not_my_persons)


    def test_duplicate_account_nr_in_import(self):
        """
        Tests fix for #4762
        """

        self.sa_session.begin()

        # create a spanish customer to ensure this country/account_nr combination doesnt exist yet

        customer = self._create_customer("84773650", "ES")
        company = self._create_company("B", "CW4", "ESESI", "Baard International", customer)
        person = self._create_person("Bart-jan 2", "bartjan@mail-test.basesoft.com")
        contact = self._create_contact(person, company)

        # create the file
        self.filepointer = "/tmp/test_import_contacts.csv"
        f = open(self.filepointer, 'w')
        f.write('%s\n%s\n%s\n%s' % (
            '"CONTACTPERSON","EMAIL","LANGUAGECODE","REVENUE","GENDER","LOCATION","COUNTRY","ACCOUNTNR","DEPOTCODE","COMPANYNAME","ENTITY"',
            '"Bart-jan 1","bartjan1@mail-test.basesoft.com","EN","B","M","I","GB","84773653","CW4","Baard International1","Express"',
            '"Bart-jan 2","bartjan2@mail-test.basesoft.com","EN","B","M","D","GB","0","CW4","Baard International2","Express"',
            '"Bart-jan 2","bartjan2@mail-test.basesoft.com","EN","B","M","D","GB","00","CW4","Baard International2","Express"', # duplicate line
            )
        )
        f.close()

        self.controller = ImportContactController(self.filepointer, 'test.csv', self.session, self.account.id, 'TntCLMContact')
        self.controller.set_file_lines()
        self.controller.field_mappings = [
            {"file_header": "CONTACTPERSON", "column": "last_name", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "EMAIL", "column": "email", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "LANGUAGECODE", "column": "language", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "REVENUE", "column": "revenue_band", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "GENDER", "column": "gender", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "LOCATION", "column": "location", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "COUNTRY", "column": "company_country_iso_3166_a2", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "ACCOUNTNR", "column": "account_number", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "DEPOTCODE", "column": "depot", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "COMPANYNAME", "column": "name", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "ENTITY", "column": "entity", "target": self.controller.COLUMN_IS_COMPANY_INFO},
        ]

        # call validate() to check if the file as errors. Expect to get an error about the '0' account numbers
        v = self.controller.validate()
        self.assertEqual(len(v), 1)
        self.assertEqual( v[0]['errors']['account_number'], ['Not a valid integer'] )

    def test_same_depot_code_in_different_country(self):
        # insert/update CW4 depot in BR (CW4 is also linked to GB)

        britain = self.sa_session.query(Country).filter(Country.iso_3166_a2=='GB').first()
        country = self.sa_session.query(Country).filter(Country.iso_3166_a2=='BR').first()

        c = TntController(self.sa_session, account_id=self.account.id)

        c.save_depot_state({
            'dbID': None,
            'default_code': u'CW4',
            'label': u'Same same but different',
            'country': country.id, # id of the country: Brazil
            'service': None,
            'aliases': []
        })

        # import a few contacts.
        self.filepointer = "/tmp/test_import_contacts.csv"
        f = open(self.filepointer, 'w')
        f.write('%s\n%s\n%s' % (
            '"CONTACTPERSON","EMAIL","LANGUAGECODE","REVENUE","GENDER","LOCATION","COUNTRY","ACCOUNTNR","DEPOTCODE","COMPANYNAME","ENTITY"',
            '"Bart-jan 1","bartjan1@mail-test.basesoft.com","EN","B","M","I","GB","84773653","CW4","Baard International1","Express"',
            '"Bart-jan 2","bartjan2@mail-test.basesoft.com","PT","B","M","I","BR","0","CW4","Mine mine","Express"',
            )
        )
        f.close()

        # verify country/depot per contact.
        self.controller = ImportContactController(self.filepointer, 'test.csv', self.session, self.account.id, 'TntCLMContact')
        self.controller.set_file_lines()
        self.controller.field_mappings = [
            {"file_header": "CONTACTPERSON", "column": "last_name", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "EMAIL", "column": "email", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "LANGUAGECODE", "column": "language", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "REVENUE", "column": "revenue_band", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "GENDER", "column": "gender", "target": self.controller.COLUMN_IS_PERSON_INFO},
            {"file_header": "LOCATION", "column": "location", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "COUNTRY", "column": "company_country_iso_3166_a2", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "ACCOUNTNR", "column": "account_number", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "DEPOTCODE", "column": "depot", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "COMPANYNAME", "column": "name", "target": self.controller.COLUMN_IS_COMPANY_INFO},
            {"file_header": "ENTITY", "column": "entity", "target": self.controller.COLUMN_IS_COMPANY_INFO},
        ]

        contacts = self.controller.import_contacts()
        self.assertEqual(len(contacts), 2)

        if contacts[0].company.locale.country == country:
            br = contacts[0].company
            gb = contacts[1].company
        else:
            br = contacts[1].company
            gb = contacts[0].company

        self.assertEqual(br.depot.country, country)
        self.assertEqual(br.locale.country, country)
        self.assertEqual(gb.depot.country, britain)
        self.assertEqual(gb.locale.country, britain)

    def tearDown(self):
        """
        Remove the test file
        """
        os.remove(self.filepointer)
