# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.contrib.i18n.models import Country
from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.core.exceptions import SecurityException, PublicUserException,\
    PermissionDeniedException

from basesoft.web.core.util.controllers import StateManager
from iqportal.tnt.test.controllers import TntWaveController
from iqportal.tnt.clm.models import TntCLMLocale, TntCLMWaveType, TntCLMWaveLocaleRevision
from iqportal.tnt.clm.controllers import WaveController, TntCLMAttributeController, TntCLMScaleAnswerController,\
    TntCLMLocaleController, TntCLMAccountController
from twisted.trial.unittest import SkipTest

from django.conf import settings

class WaveControllerTestCase(iqportal_unittest.IQPortalTestCase):
    """
    """
    def setUp(self):
        self.connect_db()
        self.account = self._create_active_account()
        self.group = self.account.groups[0].group

        self.controller = WaveController(self.session, account_id=self.account.id)

        self.generator = TntWaveController(self.session, account_id=self.account.id)
        self.wave_id = self.generator.create_wave_36()

    def test_merge_attributes(self):
        wave_33_id = self.generator.create_wave_33()
        self._attach_permissions_to_group(self.group, 'view_tnt_clm_wave', 'create_tnt_clm_attribute', 'view_tnt_clm_attribute', 'merge_tnt_clm_attribute')

        wave33 = self.controller.get_waves(wave_33_id).pop()
        wave36 = self.controller.get_waves(self.wave_id).pop()

        controller = TntCLMAttributeController(self.session, StateManager(None, self.account.id, None))

        self.assertRaises(PublicUserException, controller.merge, wave36.attributes[0].attribute.id, wave33.attributes[0].attribute.id, wave36.attributes[0].attribute.id)

        a_id = wave33.attributes[1].attribute.id
        controller.merge(wave36.attributes[0].attribute.id, *[wave33.attributes[1].attribute.id, wave36.attributes[0].attribute.id])
        self.assertTrue(len(controller.get_attributes(a_id))==0)

        wave33 = self.controller.get_waves(wave_33_id).pop()
        wave36 = self.controller.get_waves(self.wave_id).pop()
        self.assertEqual(wave33.attributes[1].attribute.id, wave36.attributes[0].attribute.id)

    def test_get_wave_state_default(self):
        state = self.controller.get_wave_state(self.wave_id)

        self.assertEqual(state['type'], u'clm_wave_36')
        self.assertEqual(len(state['wave']['attributes']), 39)
        self.assertEqual(len(state['wave']['scale_answers']), 6)

    def test_get_wave_state_locale(self):
        locale = self.session.query(TntCLMLocale).filter(TntCLMLocale.label=='NLNLD').one()
        self.generator.translate(self.wave_id, locale, skip_attribute=(10,15,))

        state = self.controller.get_wave_state(self.wave_id, locale.id)

        self.assertEqual(state['type'], u'clm_wave_36')
        self.assertEqual(len(state['wave']['attributes']), 37)
        self.assertEqual(len(state['wave']['scale_answers']), 6)

    def test_clone_wave(self):
        locale = self.session.query(TntCLMLocale).filter(TntCLMLocale.label=='NLNLD').one()
        self.generator.translate(self.wave_id, locale, skip_attribute=(10,15,))

        newwavepolly = self.controller.clone_wave(self.wave_id, locales=None)

        self.assertNotEqual(self.wave_id, newwavepolly.id)

        state = self.controller.get_wave_state(newwavepolly.id)

        self.assertEqual(state['type'], u'clm_wave_36')
        self.assertEqual(len(state['wave']['attributes']), 39)
        self.assertEqual(len(state['wave']['scale_answers']), 6)

        state = self.controller.get_wave_state(newwavepolly.id, locale.id)
        self.assertEqual(state['type'], u'clm_wave_36')
        self.assertEqual(len(state['wave']['attributes']), 37)
        self.assertEqual(len(state['wave']['scale_answers']), 6)

    def test_save_chinese_and_arabic(self):
        self._attach_permissions_to_group(self.group, 'view_tnt_clm_wave', 'translate_tnt_clm_wave')


        def _translate(localeCode, values):

            state = self.controller.get_wave_state(self.wave_id)

            locale = self.session.query(TntCLMLocale).filter(TntCLMLocale.label==localeCode).one()

            form_values = {'locale': locale.id, 'wave_label': '[%s] wave name' % localeCode, 'opt_out_text': '[%s] opt out' % localeCode}
            sa_len = len(state['wave']['scale_answers'])

            for sa in state['wave']['scale_answers']:
                form_values['scale_answer_%s' % sa['id']] = values['scale_answers'][0]
                del values['scale_answers'][0]

            attr_len = len(values['attributes'])

            for attr in state['wave']['attributes']:
                if len(values['attributes'])==0:
                    break
                form_values['attribute_%s' % attr['id']] = values['attributes'][0]
                del values['attributes'][0]

            self.controller.save_translation(self.wave_id, locale.id, form_values)
            state = self.controller.get_wave_state(self.wave_id, locale.id)
            self.assertEqual(state['type'], u'clm_wave_36')
            self.assertEqual(state['wave']['opt_out_text'], u'[%s] opt out' % localeCode)
            self.assertEqual(len(state['wave']['attributes']), attr_len)
            self.assertEqual(len(state['wave']['scale_answers']), sa_len)

        # chinese
        _translate('CNCNI', self.generator.get_chinese_translations())

        # arabic
        _translate('SAARI', self.generator.get_arabic_translations())


    def test_save_translation(self):
        self._attach_permissions_to_group(self.group, 'view_tnt_clm_wave', 'translate_tnt_clm_wave')
        state = self.controller.get_wave_state(self.wave_id)

        locale = self.session.query(TntCLMLocale).filter(TntCLMLocale.label=='NLNLD').one()

        # as we get them back from the form.
        form_values = {
            'locale': locale.id,
            'opt_out_text': '[NLNLD] opt out',
            'header_text': '[NLNLD] header text',
            'sub_header_text': '[NLNLD] sub header text',
            'closure_text': '[NLNLD] closure text',
        }
        for sa in state['wave']['scale_answers']:
            form_values['scale_answer_%s' % sa['id']] = '[NLNLD] scale answer %s' % sa['id']
        for attr in state['wave']['attributes']:
            form_values['attribute_%s' % attr['id']] = '[NLNLD] attribute %s' % attr['id']
            if attr.has_key('category'):
                form_values['category_%s' % attr['category']['id']] = '[NLNLD] category %s' % attr['category']['id']
            if not attr.has_key('answers'):
                continue
            for answer in attr['answers']:
                form_values['answer_%s' % answer['id']] = '[NLNLD] answer %s' % answer['id']

        self.controller.save_translation(self.wave_id, locale.id, form_values)
        #see what latest revision is
        r = self.session.query(TntCLMWaveLocaleRevision).filter((TntCLMWaveLocaleRevision.locale==locale) & \
                (TntCLMWaveLocaleRevision.wave_id==self.wave_id) & \
                (TntCLMWaveLocaleRevision.account==self.account)).one()

        state = self.controller.get_wave_state(self.wave_id, locale.id)
        self.assertEqual(state['revision'], r.id)
        self.assertEqual(state['type'], u'clm_wave_36')
        self.assertEqual(state['wave']['opt_out_text'], u'[NLNLD] opt out')
        self.assertEqual(state['wave']['header_text'], u'[NLNLD] header text')
        self.assertEqual(state['wave']['sub_header_text'], u'[NLNLD] sub header text')
        self.assertEqual(state['wave']['closure_text'], u'[NLNLD] closure text')
        self.assertEqual(len(state['wave']['attributes']), 39)
        self.assertEqual(len(state['wave']['scale_answers']), 6)

    def test_set_wave_attributes_priorities(self):
        self._attach_permissions_to_group(self.group, 'view_tnt_clm_wave', 'translate_tnt_clm_wave', )
        state = self.controller.get_wave_state(self.wave_id)

        locale = self.session.query(TntCLMLocale).filter(TntCLMLocale.label=='NLNLD').one()

        # as we get them back from the form.
        form_values = {'locale': locale.id, 'wave_label': '[NLNLD] wave name', 'opt_out_text': '[NLNLD] opt out'}
        for sa in state['wave']['scale_answers']:
            form_values['scale_answer_%s' % sa['id']] = '[NLNLD] scale answer %s' % sa['id']
        for attr in state['wave']['attributes']:
            form_values['attribute_%s' % attr['id']] = '[NLNLD] attribute %s' % attr['id']
            if attr.has_key('category'):
                form_values['category_%s' % attr['category']['id']] = '[NLNLD] category %s' % attr['category']['id']
            if not attr.has_key('answers'):
                continue
            for answer in attr['answers']:
                form_values['answer_%s' % answer['id']] = '[NLNLD] answer %s' % answer['id']

        self.controller.save_translation(self.wave_id, locale.id, form_values)

        priorities = {state['wave']['attributes'][0]['id']: 1}
        self.controller.set_wave_attributes_priorities(self.wave_id, locale.id, priorities)

        state = self.controller.get_wave_state(self.wave_id, locale.id)
        self.assertEqual(len(state['wave']['attributes']), 39)
        self.assertEqual(state['wave']['attributes'][0]['priority_trigger_at'], 1)
        self.assertEqual(state['wave']['attributes'][0]['priority_is_global'], False)
        self.assertEqual(state['wave']['attributes'][2]['priority_trigger_at'], None)
        self.assertEqual(state['wave']['attributes'][2]['priority_is_global'], False)

        self._attach_permissions_to_group(self.group, 'set_wave_global_attribute_priority')

        priorities = {
            state['wave']['attributes'][1]['id']: 3,
            state['wave']['attributes'][2]['id']: 4
        }

        self.controller.set_wave_attributes_priorities(self.wave_id, locale.id, priorities, priority_global=True)
        state = self.controller.get_wave_state(self.wave_id, locale.id)
        self.assertEqual(len(state['wave']['attributes']), 39)
        self.assertEqual(state['wave']['attributes'][0]['priority_trigger_at'], 1)
        self.assertEqual(state['wave']['attributes'][0]['priority_is_global'], False)
        self.assertEqual(state['wave']['attributes'][1]['priority_trigger_at'], 3)
        self.assertEqual(state['wave']['attributes'][1]['priority_is_global'], True)
        self.assertEqual(state['wave']['attributes'][2]['priority_trigger_at'], 4)
        self.assertEqual(state['wave']['attributes'][2]['priority_is_global'], True)

    def test_translations_statuses(self):
        self._attach_permissions_to_group(self.group, 'view_tnt_clm_wave', 'translate_tnt_clm_wave')
        state = self.controller.get_wave_state(self.wave_id)

        locale = self.session.query(TntCLMLocale).filter(TntCLMLocale.label=='NLNLD').one()

        # as we get them back from the form.
        form_values = {'locale': locale.id, 'wave_label': '[NLNLD] wave name', 'opt_out_text': '[NLNLD] opt out'}
        for sa in state['wave']['scale_answers']:
            form_values['scale_answer_%s' % sa['id']] = '[NLNLD] scale answer %s' % sa['id']
        for attr in state['wave']['attributes']:
            form_values['attribute_%s' % attr['id']] = '[NLNLD] attribute %s' % attr['id']
            if attr.has_key('category'):
                form_values['category_%s' % attr['category']['id']] = '[NLNLD] category %s' % attr['category']['id']
            if not attr.has_key('answers'):
                continue
            for answer in attr['answers']:
                form_values['answer_%s' % answer['id']] = '[NLNLD] answer %s' % answer['id']

        self.controller.save_translation(self.wave_id, locale.id, form_values)

        # request for approval -> status PENDING
        self.controller.set_wave_translation_status(self.wave_id, locale.id, TntCLMWaveLocaleRevision.STATUS_PENDING)
        self.controller.send_status_notification(self.wave_id, TntCLMWaveLocaleRevision.STATUS_PENDING, locale)

        # translations cannot be modified while pending
        self.assertRaises(PublicUserException, self.controller.save_translation, self.wave_id, locale.id, form_values)

        # reject - with insufficient perms
        self.assertRaises(PermissionDeniedException, self.controller.set_wave_translation_status, self.wave_id, locale.id,
                          TntCLMWaveLocaleRevision.STATUS_REJECTED)
        self._attach_permissions_to_group(self.group, 'approve_tnt_clm_translations')

        # set to rejected
        self.controller.set_wave_translation_status(self.wave_id, locale.id, TntCLMWaveLocaleRevision.STATUS_REJECTED)

        # now it should be possible to modify
        form_values['header_text'] = '[NLNLD] wave name update'
        self.controller.save_translation(self.wave_id, locale.id, form_values)

        #request for approval again
        self.controller.set_wave_translation_status(self.wave_id, locale.id, TntCLMWaveLocaleRevision.STATUS_PENDING)

        # now approve - it'll be directly ACTIVE APPROVED
        lra1 = self.controller.set_wave_translation_status(self.wave_id, locale.id, TntCLMWaveLocaleRevision.STATUS_ACTIVE_APPROVED)

        # translations can still be modified after approved
        form_values['header_text'] = '[NLNLD] wave name update2'
        self.controller.save_translation(self.wave_id, locale.id, form_values)

        #request for approval again
        self.controller.set_wave_translation_status(self.wave_id, locale.id, TntCLMWaveLocaleRevision.STATUS_PENDING)

        # now approve again - it'll be directly ACTIVE APPROVED
        lra2 = self.controller.set_wave_translation_status(self.wave_id, locale.id, TntCLMWaveLocaleRevision.STATUS_ACTIVE_APPROVED)

        # all previous ACTIVE_APPROVED should be changed to APPROVED
        lrs = self.controller.get_wave_locale_revisions(self.wave_id, locale.id)
        for lr in lrs:
            if lr.id == lra1.id:
                self.assertEqual(lr.status, TntCLMWaveLocaleRevision.STATUS_APPROVED)
            elif lr.id == lra2.id:
                self.assertEqual(lr.status, TntCLMWaveLocaleRevision.STATUS_ACTIVE_APPROVED)
            else:
                self.assertNotEqual(lr.status, TntCLMWaveLocaleRevision.STATUS_ACTIVE_APPROVED)


    def test_wave_creation_unnamed_categories(self):
        # this test the bug whereby attributes in separate categories with an empty label were not reloaded correctly.
        self._attach_permissions_to_group(self.group, 'view_tnt_clm_wave', 'update_tnt_clm_wave', 'translate_tnt_clm_wave')
        state = self.controller.get_wave_setup(self.wave_id)

        #there is one category with 7 attributes, and 31 attributes with no category
        self.assertEqual(len(state['wave_attributes']), 2)
        self.assertEqual(len(state['wave_attributes'][0]['attributes']), 32)
        self.assertEqual(state['wave_attributes'][0]['id'], None)
        self.assertEqual(len(state['wave_attributes'][1]['attributes']), 7)
        self.assertEqual(state['wave_attributes'][1]['category_name'], u'General')

        state['name'] = u'test wave update'
        state['code'] = u'test code update'
        state['wave_attributes'][0]['category_name'] = u''
        # remove one attribute from it
        state['wave_attributes'][0]['attributes'].pop()
        # update the name of the existing categ
        state['wave_attributes'][1]['category_name'] = u'General update'

        # add a new category, also with an empty label
        state['wave_attributes'].append({})
        state['wave_attributes'][2]['id'] = None
        state['wave_attributes'][2]['category_name'] = u''
        # get the last attribute of the General category as attribute for the new category
        state['wave_attributes'][2]['attributes'] = [state['wave_attributes'][1]['attributes'].pop()]

        self.controller.save_or_update_wave(self.wave_id, state['name'], state['code'], state['default_locale'],\
                                            state['type'], state['wave_attributes'], [])

        new_state = self.controller.get_wave_setup(self.wave_id)

        # we expect 3 categories.
        self.assertEqual(len(new_state['wave_attributes']), 3)
        # first one with an attribute less than before
        self.assertEqual(len(new_state['wave_attributes'][0]['attributes']), 31)
        # Still with an empty label for categoruy name
        self.assertEqual(new_state['wave_attributes'][0]['category_name'], u'')
        # second category has 6  attributes (one was moved to the new category)
        self.assertEqual(len(new_state['wave_attributes'][1]['attributes']), 6)
        # update label
        self.assertEqual(new_state['wave_attributes'][1]['category_name'], u'General update')
        # one attribute in last category
        self.assertEqual(len(new_state['wave_attributes'][2]['attributes']), 1)
        # also empty label for category
        self.assertEqual(new_state['wave_attributes'][2]['category_name'], u'')
        # ids for both categories with an empty label should NOT be equal
        self.assertNotEqual(new_state['wave_attributes'][0]['id'], new_state['wave_attributes'][2]['id'])

    def test_update_wave(self):
        self._attach_permissions_to_group(self.group, 'view_tnt_clm_wave', 'update_tnt_clm_wave', 'translate_tnt_clm_wave')
        state = self.controller.get_wave_setup(self.wave_id)

        #there is one category with 7 attributes, and 31 attributes with no category
        self.assertEqual(len(state['wave_attributes']), 2)
        self.assertEqual(len(state['wave_attributes'][0]['attributes']), 32)
        self.assertEqual(state['wave_attributes'][0]['id'], None)
        self.assertEqual(len(state['wave_attributes'][1]['attributes']), 7)
        self.assertEqual(state['wave_attributes'][1]['category_name'], u'General')

        state['name'] = u'test wave update'
        state['code'] = u'test code update'
        # add all attributes without category to a new category
        state['wave_attributes'][0]['category_name'] = u'New category'
        # remove one attribute from it
        state['wave_attributes'][0]['attributes'].pop()
        # update the name of the existing categ
        state['wave_attributes'][1]['category_name'] = u'General update'

        self.controller.save_or_update_wave(self.wave_id, state['name'], state['code'], state['default_locale'],\
                                            state['type'], state['wave_attributes'], [])
        new_state = self.controller.get_wave_setup(self.wave_id)
        self.assertEqual(len(new_state['wave_attributes']), 2)
        self.assertEqual(len(new_state['wave_attributes'][0]['attributes']), 31)
        self.assertEqual(new_state['wave_attributes'][0]['category_name'], u'New category')
        self.assertEqual(len(new_state['wave_attributes'][1]['attributes']), 7)
        self.assertEqual(new_state['wave_attributes'][1]['category_name'], u'General update')

    def test_update_wave_ticket_3143(self):
        self._attach_permissions_to_group(self.group, 'view_tnt_clm_wave', 'update_tnt_clm_wave', 'translate_tnt_clm_wave')


        newwavepolly = self.controller.clone_wave(self.wave_id, locales=None)

        wsetup = self.controller.get_wave_setup(newwavepolly.id)

        locale = self.session.query(TntCLMLocale).filter(TntCLMLocale.label=='XXENI').one()

        state = self.controller.get_wave_state(newwavepolly.id, locale.id)['wave']
        no_of_answer_map = {}
        for attr in state['attributes']:
            if attr.has_key('answers'):
                no_of_answer_map[attr['id']] = len(attr['answers'])

        #there is one category with 7 attributes, and 31 attributes with no category
        self.assertEqual(len(wsetup['wave_attributes']), 2)
        self.assertEqual(len(wsetup['wave_attributes'][0]['attributes']), 32)
        self.assertEqual(wsetup['wave_attributes'][0]['id'], None)
        self.assertEqual(len(wsetup['wave_attributes'][1]['attributes']), 7)
        self.assertEqual(wsetup['wave_attributes'][1]['category_name'], u'General')

        # remove one attribute from it
        last = wsetup['wave_attributes'][0]['attributes'].pop()
        second_last = wsetup['wave_attributes'][0]['attributes'].pop()
        wsetup['wave_attributes'].append({
            'attributes': [second_last, last],
            'category_name': '',
            'id': ''
        })
        self.sa_session.flush()
        self.sa_session.expire(newwavepolly)

        self.controller.save_or_update_wave(newwavepolly.id, wsetup['name'], wsetup['code'], wsetup['default_locale'],\
                                            wsetup['type'], wsetup['wave_attributes'], [])

        new_wsetup = self.controller.get_wave_setup(self.wave_id)
        self.assertEqual(len(new_wsetup['wave_attributes']), 2)
        self.assertEqual(len(new_wsetup['wave_attributes'][0]['attributes']), 32)
        self.assertEqual(new_wsetup['wave_attributes'][0]['category_name'], u'')
        self.assertEqual(len(new_wsetup['wave_attributes'][1]['attributes']), 7)
        self.assertEqual(new_wsetup['wave_attributes'][1]['category_name'], u'General')

        new_state = self.controller.get_wave_state(newwavepolly.id, locale.id)['wave']
        for attr in new_state['attributes']:
            if attr.has_key('answers'):
                self.assertEqual(no_of_answer_map[attr['id']], len(attr['answers']))

    def test_save_pdf(self):

        if settings.FONTS_PATH is None:
            raise SkipTest('FIXME, #3858')

        # english
        locale = self.session.query(TntCLMLocale).filter(TntCLMLocale.label=='XXENI').one()
        self.controller.export_wave_pdf('/tmp/tnt_xxeni.pdf', self.wave_id, locale.id)

        self.test_save_chinese_and_arabic()

        # chinese
        locale = self.session.query(TntCLMLocale).filter(TntCLMLocale.label=='CNCNI').one()
        self.controller.export_wave_pdf('/tmp/tnt_cnci.pdf', self.wave_id, locale.id)

        # arabic
        locale = self.session.query(TntCLMLocale).filter(TntCLMLocale.label=='SAARI').one()
        self.controller.export_wave_pdf('/tmp/tnt_saari.pdf', self.wave_id, locale.id)

    def test_save_pdf_39c(self):

        if settings.FONTS_PATH is None:
            raise SkipTest('FIXME, #3858')

        # wave 39c
        wave_39c_id = self.generator.create_wave_39c()

        # english
        locale = self.session.query(TntCLMLocale).filter(TntCLMLocale.label=='XXENI').one()
        self.controller.export_wave_pdf('/tmp/tnt_xxeni.pdf', wave_39c_id, locale.id)


class TntCLMAttributeControllerTestCase(iqportal_unittest.IQPortalTestCase):
    """
    This tests the tnt clm attribute controller
    """

    def setUp(self):
        """
        The script for reseting the db and creating the account+controller stuff
        """
        self.connect_db()

        self.account = self._create_active_account()
        self.group = self.account.groups[0].group

        self.controller = TntCLMAttributeController(self.session, StateManager(None, self.account.id, None))

    def _get_attribute_state(self):
        """
        """
        return {'dbID': None,
                'domID': None,
                'type': 2,
                'default_label': u"test attrubute",
                'priority_trigger_at': 1,
                'locales': {"GBEND": {"label": u"en attribute", "priority_trigger_at": 2}},
                'answers': [
                    {'dbID': None,
                     'domID': None,
                     'default_label': u"a1 default",
                     'position': 1,
                     'locales':{"GBEND": {"label": u"en a1"}}
                     },
                    {'dbID': None,
                     'domID': None,
                     'default_label': u"a2 default",
                     'position': 2,
                     'locales':{"GBEND": {"label": u"en a2"}}
                     },
                ]
               }

    def test_save_and_load_attribute(self):
        state = self._get_attribute_state()

        self._attach_permissions_to_group(self.group, 'create_tnt_clm_attribute', 'view_tnt_clm_attribute')
        attribute_id = self.controller.save_attribute(state)

        lstate = self.controller.load_attribute(attribute_id)
        state['dbID'] = lstate['dbID']
        for i in range(0, len(lstate['answers'])):
            state['answers'][i]['dbID'] = lstate['answers'][i]['dbID']
        self.assertDeepEqual(lstate, state)

        #update attr label and one answer's label and locale
        state['default_label'] = u"updated attr label"
        state['answers'][0]['default_label'] = u"updated a1 default"
        state['answers'][0]['locales']['GBEND']['label'] = u"updated en a1"

        self._attach_permissions_to_group(self.group, 'update_tnt_clm_attribute')
        attribute_id = self.controller.save_attribute(state)
        nstate = self.controller.load_attribute(attribute_id)

        self.assertEqual(nstate['default_label'],  u"updated attr label")
        self.assertEqual(nstate['answers'][0]['default_label'], u"updated a1 default")
        self.assertEqual(nstate['answers'][0]['locales']['GBEND']['label'], u"updated en a1")

class TntCLMAccountControllerTestCase(iqportal_unittest.IQPortalTestCase):
    def setUp(self):
        self.connect_db()

        self.account = self._create_active_account()
        self.group = self.account.groups[0].group

        self.controller = TntCLMAccountController(self.session, account_id=self.account.id)
        self.lc = TntCLMLocaleController(self.session, account_id=self.account.id)
        #self.cc = TntController(self.session, account_id=self.account.id)

    def test_save_account_from_state(self):
        """
        Test the saving of new CLM accounts.
        """

        state = {
            'person': {
                'first_name': u'Account FN',
                'gender': u'male',
                'last_name': u'Account LN',
                'email': '%s@mail-test.basesoft.com' % self._uniquestring('random'),
                'address_data': {
                    'city': u'My City',
                    'state': u'My State',
                    'street': u'232 near the high school',
                    'postcode': u'90210',
                    'country': 204} # Spain
            },
            'account': {
                'account_name': self._uniquestring('myaccountyo')[0:39],
                'groups': [self.group.id],
                'default_group': self.group.id
            }
        }
        geolevels = []

        # get some random locales
        locale_objects = self.lc.get_clm_locales_by('label', ['KRKRI', 'EGARI', 'NLNLD'])
        locales = [l.id for l in locale_objects]

        self._attach_permissions_to_group(self.group, 'view_datasource', 'create_account', 'view_license', 'view_domain')

        # save the account
        account = self.controller.save_account_from_state(state, geolevels, locales)

        # verify locales.
        [self.assertTrue(locale in locale_objects) for locale in account.locales]

class TntCLMLocaleControllerTestCase(iqportal_unittest.IQPortalTestCase):
    def setUp(self):
        self.connect_db()

        self.account = self._create_active_account()
        self.group = self.account.groups[0].group

        self.controller = TntCLMLocaleController(self.session, account_id=self.account.id)

    def test_save_or_update_locale(self):

        country_id = self.session.query(Country).first().id
        self.assertRaises(PermissionDeniedException, self.controller.save_or_update_locale, None, 'AAAAA', 'test locale', 'EN', 'ltr', country_id)

        self._attach_permissions_to_group(self.group, 'create_tnt_clm_locale')

        locale = self.controller.save_or_update_locale(None, 'AAAAA', 'test_locale', 'EN', 'ltr', country_id, location="D", location_2015="I", service='Postal')
        self.assertEqual(locale.country.id, country_id)
        self.assertEqual(locale.location, 'D')
        self.assertEqual(locale.location_2015, 'I')

class TntCLMScaleAnswerControllerTestCase(iqportal_unittest.IQPortalTestCase):
    """
    This tests the tnt clm scale answer controller
    """

    def setUp(self):
        self.connect_db()

        self.account = self._create_active_account()
        self.group = self.account.groups[0].group

        self.controller = TntCLMScaleAnswerController(self.session, StateManager(None, self.account.id, None))

    def test_scale_answers(self):

        # create a wave type for this test
        wt = TntCLMWaveType()
        wt.module = u'clm_wave_test'
        wt.label = u'clm_wave_test'

        self.controller.save_default_privilege_object_group(wt)
        self.session.add(wt)

        # give the permissions and onwards with the tests
        self._attach_permissions_to_group(self.group, 'create_tnt_clm_scale_answer', 'view_tnt_clm_scale_answer',
                                          'delete_tnt_clm_scale_answer', 'update_tnt_clm_scale_answer')
        scale_answers_ids = [] # we keep here the ids of the scale answers we create
        scale_answers_positions = [] # we keep here the positions of the answers
        for i in range(5):

            # create 5 scale answers
            state = {
                'dbID': None,
                'domID': None,
                'type': int(wt.id),
                'default_label': u"test scale answer %s" % i,
                'locales': {"GBEND": {"label": u"en translation for scale answer %s" % i}}
            }
            sa_id = self.controller.save_scale_answer(state)
            sa = self.controller.get_scale_answers(sa_id)[0]
            scale_answers_ids.append(sa.id)
            scale_answers_positions.append(sa.position)

        # test the first scale answer
        load_state = self.controller.load_scale_answer(scale_answers_ids[0])
        save_state = {
            'dbID': scale_answers_ids[0],
            'domID': None,
            'type': int(wt.id),
            'default_label': u"test scale answer 0",
            'locales': {"GBEND": {"label": u"en translation for scale answer 0"}}
        }
        self.assertDeepEqual(load_state, save_state)

        #update the first scale answer
        save_state['default_label'] = u"updated scale answer"
        sa_id = self.controller.save_scale_answer(save_state)
        updated_state = self.controller.load_scale_answer(scale_answers_ids[0])

        self.assertEqual(updated_state['default_label'],  u"updated scale answer")

        # test the reordering of the elements
        scale_answers = self.controller.get_scale_answers(scale_answers_ids)
        self.assertEqual([e.position for e in scale_answers], scale_answers_positions)

        # move the second scale answer up one position
        sa = self.controller.get_scale_answers(scale_answers_ids[1])[0]
        self.controller.move_scale_answer(sa.id, True)

        scale_answers = self.controller.get_scale_answers(scale_answers_ids)
        self.assertEqual(scale_answers[0].position, scale_answers_positions[1])
        self.assertEqual(scale_answers[1].position, scale_answers_positions[0])

        # move the 3rd scale answer down one position
        sa = self.controller.get_scale_answers(scale_answers_ids[2])[0]
        self.controller.move_scale_answer(sa.id, False)

        scale_answers = self.controller.get_scale_answers(scale_answers_ids)
        self.assertEqual(scale_answers[2].position, scale_answers_positions[3])
        self.assertEqual(scale_answers[3].position, scale_answers_positions[2])

# below is for running the test from eclipse
if __name__ == '__main__':
    from twisted.scripts.trial import run
    import sys
    sys.argv = ['test_js_lint.py', 'iqportal.tnt.test.unit.test_clm_controller.WaveControllerTestCase.test_save_pdf']
    run()
    sys.exit()
