# -*- coding: utf-8 -*-
# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

import exceptions

from django.conf import settings

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.contrib.i18n import models as i18n_models
from basesoft.web.core.exceptions import SecurityException,\
    PermissionDeniedException
from basesoft.web.contrib.mailing.models import Mail, MailGroup

from iqportal.tnt.clm.mailing import MailController
from iqportal.tnt.clm.models import TntCLMLocale, TntCLMWaveMailLocale, TntCLMWaveMail
from iqportal.tnt.clm.controllers import TntCLMMailBaseController

from basesoft import logging
from twisted.trial.unittest import SkipTest
logger = logging.getLogger('iqportal.tnt.test.unit')

class MailTestCase(iqportal_unittest.IQPortalTestCase):
    """
    """
    def setUp(self):
        self.connect_db()

        self.account = self._create_active_account()
        self.group = self.account.groups[0].group

        self.locale = self.session.query(i18n_models.Locale).filter_by(locale='en_GB').one()
        self.locale_nl = self.session.query(i18n_models.Locale).filter_by(locale='nl_NL').one()

        self.tnt_clm_locale = self.session.query(TntCLMLocale).filter_by(label='GBEND').one()
        self.tnt_clm_locale_nl = self.session.query(TntCLMLocale).filter_by(label='NLNLD').one()

        self.controller = MailController(self.session, self.locale, account_id=self.account.id)

    def get_state(self):
        return {
            "mail": {
                "mail_from_text": "MockTntMailer",
                "name": u"Some tnt wave email",
                "objectName": u"TntCLMWaveMail",
                "dbID": None,

                "is_wiki": 0,
                "locales": {
                    'GBEND': {
                        'subject': u"Mijn test onderworpen",
                        'content': u"Hallo <b>wereld !</b>",
                        'link_label': None,
                        'status': TntCLMWaveMailLocale.STATUS_OPEN,
                    },
                    'NLNLD': {
                        'subject': u'My test subject',
                        'content': u"Hello <b>world !</b>",
                        'link_label': None,
                        'status': TntCLMWaveMailLocale.STATUS_OPEN,
                    },
                },

                "fallbackLocale": {'value': u'GBEND', 'name': u'English' },
            }
        }

    def assertLocales(self, obj, state):
        """
        Helps with asserting locales in an obj
        """
        count = 0 # keeps track of the number of texts in the state

        # start asserting text values
        for locale_str in state:
            for field_name in state[locale_str]:
                for locale_inst in obj.locales:
                    if locale_inst.locale.locale == locale_str:
                        self.assertEquals( getattr(locale_inst, field_name) , state[locale_str][field_name] )
            count = count + 1

        # the length of the locales should be the same as the number of text items tested
        self.assertEquals(count, len(obj.locales) )

    def test_load_state(self):

        self._attach_permissions_to_group(self.group, 'create_mail', 'create_tnt_wave_mail')

        state = self.get_state()

        # first save a new state
        mail = self.controller.save_state(state, self.group)

        # test without permission
        self.assertRaises(PermissionDeniedException, self.controller.load_by_mail_id, mail.id)

        # add permissions
        self._attach_permissions_to_group(self.group, 'view_mail', 'view_tnt_wave_mail')

        # then load it again
        loaded_state = self.controller.load_by_mail_id(mail.id)

        # update the id in our state
        state['mail']['dbID'] = mail.id

        # now state should be the same as the loaded_state
        try:
            self.assertEquals(state, loaded_state)
        except exceptions.AssertionError, e:
            from django.utils import simplejson
            logger.info('diff failed! check the difference at http://tlrobinson.net/projects/js/jsondiff/')

            from basesoft.web.core.jsonrpc.dispatcher import JsonRpcJSONEncoder

            logger.debug('state: %s' % simplejson.dumps(state, cls=JsonRpcJSONEncoder))
            logger.debug('loaded_state: %s' % simplejson.dumps(loaded_state, cls=JsonRpcJSONEncoder))
            raise e

    def test_save_update_state(self):
        """
        Tests the updating of a mail using a state that has the dbID set
        """

        self._attach_permissions_to_group(self.group, 'create_mail', 'view_mail',
                                          'create_tnt_wave_mail', 'view_tnt_wave_mail')

        # first save a new mail and get the response
        # (we save it only with view privileges so we can test later)
        state = self.get_state()
        state_response = self.controller.save_state(state, self.group)

        # load the state. this has the dbID set
        loaded_state = self.controller.load_by_mail_id(state_response.id)

        # verify that the canvas id is there
        self.assertNotEquals(loaded_state['mail']['dbID'], None)

        # now we change the name a bit
        loaded_state['mail']['name'] = u'Some updated mail'
        loaded_state['mail']['locales']['GBEND']['subject'] = u'My updated test subject'

        # add a new locale
        loaded_state['mail']['locales']['ROROI'] = {}
        loaded_state['mail']['locales']['ROROI']['subject'] = u'Subiect'
        loaded_state['mail']['locales']['ROROI']['content'] = u'Continut mesaj'

        # call save again, first without permission
        self.assertRaises(PermissionDeniedException, self.controller.save_state, loaded_state, self.group)

        # add the proper permission
        self._attach_permissions_to_group(self.group, 'update_mail', 'update_tnt_wave_mail')

        updated_state_response = self.controller.save_state(loaded_state, self.group)

        # now query it and verify
        mail = self.session.query(Mail).get(updated_state_response.id)
        self.assertEquals(mail.name, u'Some updated mail')

        self.assertLocales(mail, {
            'NLNLD': {
                'subject': u"Mijn test onderworpen",
                'content': u"Hallo <b>wereld !</b>",
            },
            'GBEND': {
                'subject': u'My updated test subject',
                'content': u"Hello <b>world !</b>",
            },
            'ROROI': {
                'subject': u'Subiect',
                'content': u"Continut mesaj",
            },
        })

    def test_save_new_state(self):
        """
        Test the creation of a mail in multiple locales.
        No need to test privileges because there is no object in create mode.
        """
        # test create without permission
        initial_state = self.get_state()
        self.assertRaises(PermissionDeniedException, self.controller.save_state, initial_state, self.group)

        # test create with permissions
        self._attach_permissions_to_group(self.group, 'create_mail', 'create_tnt_wave_mail')

        saved_mail = self.controller.save_state(initial_state, self.group)

        # assert the first mail
        mail = self.session.query(TntCLMWaveMail).get(saved_mail.id)
        self.assertEquals(mail.name, u"Some tnt wave email")
        self.assertEquals(mail.fallback_locale.label, 'GBEND')

        self.assertLocales(mail, {

            'NLNLD': {
                'subject': u"Mijn test onderworpen",
                'content': u"Hallo <b>wereld !</b>",
            },
            'GBEND': {
                'subject': u'My test subject',
                'content': u"Hello <b>world !</b>",
            },
        })

    def test_delete_mail_locale(self):
        """
        Test the deleting of a locale from a mail.
        No PERMISSION distinction between locale deleting or full deleting
        """
        self._attach_permissions_to_group(self.group, 'create_mail', 'view_mail',
                                          'create_tnt_wave_mail', 'view_tnt_wave_mail')
        mail = self.controller.save_state(self.get_state(), self.group)

        # test delete of a locale without permission
        self.assertRaises(PermissionDeniedException, self.controller.delete_mail_locales, mail)

        # test delete of a locale without privileges
        self._attach_permissions_to_group(self.group, 'delete_mail', 'delete_tnt_wave_mail')
        self.assertRaises(PermissionDeniedException, self.controller.delete_mail_locales, mail)

        # test delete of a locale with permissions and privileges
        mail_obj = self.session.query(Mail).filter(Mail.id==mail.id).one()
        mg = MailGroup()
        mg.group = self.group
        mg.privilege = Mail.PRIV_DELETE
        mg.group_has_privilege = True
        mail_obj.groups.append(mg)
        self.session.add(mail_obj)
        self.session.flush()

        self.assertEquals(len(self.controller.get_mail_locales(mail)), 2)

        # delete the english locale
        self.controller.delete_mail_locales(mail, self.tnt_clm_locale)
        self.assertEquals(len(self.controller.get_mail_locales(mail)), 1)

        # delete all locales left (the dutch one)
        self.controller.delete_mail_locales(mail)
        self.assertEquals(len(self.controller.get_mail_locales(mail)), 0)

    def test_delete_mail(self):
        """
        Test the deletion of a mail
        """
        self._attach_permissions_to_group(self.group, 'create_mail', 'view_mail',
                                          'create_tnt_wave_mail', 'view_tnt_wave_mail')

        mail = self.controller.save_state(self.get_state(), self.group)

        # test delete without permission
        self.assertRaises(PermissionDeniedException, self.controller.delete_mail, mail.id)

        # test delete without 'delete_tnt_wave_mail' permission
        self._attach_permissions_to_group(self.group, 'delete_mail')
        self.assertRaises(PermissionDeniedException, self.controller.delete_mail, mail.id)

        # test delete without privileges
        self._attach_permissions_to_group(self.group, 'delete_tnt_wave_mail')
        self.controller.delete_mail(mail.id)
        left_mails = self.controller.get_mails(mail_type='TntCLMWaveMail')
        self.assertEqual(left_mails, [mail])

        # test delete with permissions and privileges
        mail2_state = {
            "mail": {
                "mail_from_text": "MockTntMailer",
                "name": u"Another email",
                "objectName": u"TntCLMWaveMail",
                "dbID": None,

                "locales": {
                    'NLNLD': {
                        'subject': u"Mijn tweede onderwerp",
                        'content': u"Bye <b>wereld !</b>",
                    },
                    'GBEND': {
                        'subject': u"My second subject",
                        'content': u"Bye <b>world !</b>",
                    },
                },

                "fallbackLocale": {'value': u'GBEND', 'name': u'English' },
            }
        }

        # add another email that has privileges for deleting
        mail2 = self.controller.save_state(mail2_state, self.group)

        # this should fail because we don't have privileges for deleting mail
        self.controller.delete_mail([mail.id, mail2.id])
        left_mails = self.controller.get_mails(mail_type='TntCLMWaveMail')
        self.assertEquals(len(left_mails), 2)

        # add the delete privilege to mail too
        mail_obj = self.session.query(Mail).filter(Mail.id==mail.id).one()
        mg = MailGroup()
        mg.group = self.group
        mg.privilege = Mail.PRIV_DELETE
        mg.group_has_privilege = True
        mail_obj.groups.append(mg)
        self.session.add(mail_obj)
        self.session.flush()

        self.controller.delete_mail(mail.id) # delete only one mail

        # test get_mails with the mail_type parameter also
        left_mails = self.controller.get_mails(mail_type='TntCLMWaveMail')
        self.assertEquals(len(left_mails), 1)
        mail2_obj = left_mails[0]
        self.assertEquals(mail2_obj.name, u"Another email")

        # same test, but without the mail_type. should work the same because we only have mock mails
        left_mails = self.controller.get_mails()
        self.assertEquals(len(left_mails), 1)
        mail2_obj = left_mails[0]
        self.assertEquals(mail2_obj.name, u"Another email")

    def test_clone_mail(self):
        self._attach_permissions_to_group(self.group, 'create_mail', 'create_tnt_wave_mail')
        initial_state = self.get_state()

        mail = self.controller.save_state(initial_state, self.group)
        new_mail = self.controller.duplicate_mail(mail)
        self.assertNotEqual(mail.id, new_mail.id)
        self.assertEqual(mail.name, new_mail.name)

    def test_mail_types(self):
        """
        Test the checking of mail types
        """
        self.assertEquals(self.controller.get_all_mail_types(), [])

        self._attach_permissions_to_group(self.group, 'view_tnt_wave_mail')

        new_controller = MailController(self.controller.sa_session, self.controller.locale, self.controller.account_id)

        self.assertEquals(new_controller.get_all_mail_types(), [TntCLMWaveMail])

    def test_mail_fallback(self):
        """
        Tests all language fallback related stuff in mails
        """

        self._attach_permissions_to_group(self.group, 'create_mail', 'create_tnt_wave_mail')

        mail_state = {
            "mail": {
                "mail_from_text": "MockTntMailer",
                "name": u"Another email",
                "objectName": u"TntCLMWaveMail",
                "dbID": None,

                "locales": {
                    'NLNLD': {
                        'subject': u"Mijn tweede onderwerp",
                        'content': 'Mijn tweede content',
                    },
                    'GBEND': {
                        'subject': u"My second subject",
                        'content': u'My second content',
                    },
                },

                "fallbackLocale": {'value': u'NLNLD'},
            }
        }

        mail = self.controller.save_state(mail_state, self.group)

        locale_en = self.session.query(TntCLMLocale).filter_by(label='GBEND').one()
        locale_nl = self.session.query(TntCLMLocale).filter_by(label='NLNLD').one()
        locale_ro = self.session.query(TntCLMLocale).filter_by(label='ROROI').one()

        # english and dutch subject can be retrieved directly
        self.assertEqual(mail.get_locale_mail(locale_en, TntCLMWaveMailLocale).subject, 'My second subject')
        self.assertEqual(mail.get_locale_mail(locale_nl, TntCLMWaveMailLocale).subject, 'Mijn tweede onderwerp')

        # romanian subject should fallback to dutch
        self.assertEqual(mail.get_locale_mail(locale_ro, TntCLMWaveMailLocale).subject, 'Mijn tweede onderwerp')

        # it should also be possible to get the mail with a None locale.. we expect fallback to be returned
        self.assertEqual(mail.get_locale_mail(None, TntCLMWaveMailLocale).subject, 'Mijn tweede onderwerp')

    def test_save_pdf(self):

        if settings.FONTS_PATH is None:
            raise SkipTest('FIXME, #3858')

        self._attach_permissions_to_group(self.group, 'create_mail', 'create_tnt_wave_mail',
                                                      'view_mail', 'view_tnt_wave_mail',
                                                      'update_mail', 'update_tnt_wave_mail')

        import iqportal

        image = 'file://%s' % iqportal.__path__[0] + '/../basesoft/web/contrib/basesoft/media/75x75/apps/preferences-desktop-wallpaper.png'

        state = self.get_state()
        state['mail']['locales']['XXENI'] = {
            'subject': u"Some test subject for pdf",
            'content': u'<img  style="width: 75px; height: 75px;" src="' + image + u'"><br>Hello world aaa & < > bbb ! This is your link: <img class="replacement-field" title="wave.link" src="/basesoft.web.core/rte/make_image/Wave%3AWave%20link"> . <span style="text-decoration: underline;">And now <strong>just</strong> some dummy</span> <span style="text-decoration: line-through;">text to fill up<br/> some space...</span><br><br><strong>Lorem ipsum dolor sit amet, consectetur adipisicing elit,</strong> <em>sed do eiusmod tempor incididunt ut labore et dolore magna aliqua</em>. Ut <span style="color: #bc6e6e;">enim </span>ad <span style="background-color: #ffdfbf;">minim </span>veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat.<br><div style="text-align: right;">test some right alignment<br><strong>with bold too...</strong><br></div><br><div style="text-align: center;">some align center<br></div><div style="text-align: justify;">some aaa aaa aaa aaa aaa aaa aaa aaa aaa aaa aaa aaa aaa align justify<br></div><br>',
            'link_label': u'wave link'
        }

        state['mail']['locales']['CNCNI'] = {
            'subject': u"是否容易联系上您要找的TNT工作人员",
            'content': u'<img  style="width: 75px; height: 75px;" src="' + image + u'"> 同重量和大小物品的能力递处理您各类不同重量和大小物品的能力递处理您和大小物品的能力递处理您各类不同重量和大小物品的能力递处理您各类不同重量和大小物品的能力递处理您各类不同重量和</span>大小同重量和大小物品的能力递处理您各类不同重量和大小物品的能力递处理您各类不同重量和大小物品的能力递处理您各类不同重量和大小物品的能力 <img  class="replacement-field " title="wave.link" src="/basesoft.web.core/rte/make_image/Wave%3AWave%20link">',# <br> <img style="width: 457px; height: 89px;" src="site_media/iqportal.tnt.clm/img/logo.jpg">',
            'link_label': u'比我要求的差'
        }

        state['mail']['locales']['SAARI'] = {
            'subject': u"أقل بكثير من المتوقع",
            'content': u'<img  style="width: 457px; height: 89px;" src="' + image + u'">سهولة <span style="text-decoration: line-through;">الحصول على<br>&nbsp;اتصال مع ا</span><span style="text-decoration: underline;">لشخص المطلوب في مادة تي ان تي</span><strong>اتصال مع الشخص</strong><br><strong>المطلوب</strong><br>&nbsp;في مادة تي ان تي اكسبريسسهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريسسهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريسسهولة الحصول على اتصال مع الشخص المطلوب في ماادة تي ان تي اكسسهولة الحصول على اتصال مع الشخص المطلوب <span style="background-color: #ffff00;">في مادة تي ان </span>تي اكسب<span style="color: #ff0000;">ريسسهولة الح</span>صول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريسسهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريسسهولة الحصول على اتصال مع الشخص المطلوب في مادة تي ان تي اكسبريسسهولة الحصول على اتصال مع الشخص المطلوب في&nbsp; <img  class="replacement-field " title="wave.link" src="/basesoft.web.core/rte/make_image/Wave%3AWave%20link"> ماادة تي ان تي اكس<br><br><ul><li> مع الشخص المطلوب في مادة تي ان تي اكسبريس</li></ul>',
            'link_label': None
        }

        # first save a new state
        mail = self.controller.save_state(state, self.group)

        clm_mail_controller = TntCLMMailBaseController(self.session, self.locale, account_id=self.account.id)
        # english
        locale = self.session.query(TntCLMLocale).filter(TntCLMLocale.label=='XXENI').one()
        clm_mail_controller.export_mail_pdf('%stnt_clm_wave_mail_xxeni.pdf' % settings.WRITABLE_DIRECTORY, mail.id, locale.id)

        # chinese
        locale = self.session.query(TntCLMLocale).filter(TntCLMLocale.label=='CNCNI').one()
        clm_mail_controller.export_mail_pdf('%stnt_clm_wave_mail_cnci.pdf' % settings.WRITABLE_DIRECTORY, mail.id, locale.id)

        # arabic
        locale = self.session.query(TntCLMLocale).filter(TntCLMLocale.label=='SAARI').one()
        clm_mail_controller.export_mail_pdf('%stnt_clm_wave_mail_saari.pdf' % settings.WRITABLE_DIRECTORY, mail.id, locale.id)
