# Copyright (c) 2012 BaseSoft B.V.
# See LICENSE for details.

from sqlalchemy import func
from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.contrib.account.models import Account

from iqportal.tnt.test.controllers import TntWaveController
from iqportal.tnt.clm.models import TntCLMAttribute
from iqportal.tnt.core.models import TntGeolevel, TntGeolevelAccount

from iqportal.iqube.core.controllers import ConfigurableWidgetController, ProjectController
from iqportal.iqube.core.db_sync import AbstractConfigurableWidget
from iqportal.iqube.core.models import DataSource

from iqportal.tnt.report import db_sync as report_sync

from django.utils import simplejson
from iqportal.iqube.test.unit.test_configurable_widget_reporters import AbstractConfigurableWidgetReporterTestCase

class TNTDimensionsTestCase(iqportal_unittest.IQPortalTestCase):

    def setUp(self):
        self.connect_db()

        account = self.session.query(Account).filter(Account.name=='sysadmin').one()
        # we populate necessary tnt account data
        c = TntWaveController(self.session, account_id=account.id)
        license, account = c.populate_tnt_account()

        self.account = account
        self.group = account.groups[0].group

        self.controller = ConfigurableWidgetController(self.session, account_id=self.account.id)

        datasource = self.session.query(DataSource).filter(func.substr(DataSource.name, -7) == 'TNT-CLM').all()
        if len(datasource) == 0:
            raise Exception('No TNT-CLM datasource')
        self.datasource = datasource[0]

        report_sync.sync(self.session)
        pc = ProjectController(self.session)
        pc.flush_schema(self.datasource.id)

        self.cube = 'TntCLMVirtualCube'

    def _get_attribute_dimension_cfg(self):
        """
        Basic Attribute dimension config
        """
        return {
            "available_on_rows": True,
            "show_named_sets": False,
            "has_select_as_group": False,
            "available_as_split_by": False,
            "required": False,
            "available_as_slicer": True,
            "caption": "Attribute",
            "fancy_select_module": "iq_fancyselect",
            "available_on_columns": True,
            "show_calculated_members": True,
            "children": [
                {
                    "unique_name": "[Attribute].[Type]",
                    "has_children_available": False,
                    "caption": "Type",
                    "has_all_member_available": False,
                    "show_members": True,
                    "method_argument": "[Attribute].[Type]",
                    "method": "get_level_members"
                },
                {
                    "unique_name": "[Attribute].[Attribute]",
                    "has_children_available": False,
                    "caption": "Attribute",
                    "has_all_member_available": False,
                    "show_members": True,
                    "method_argument": "[Attribute].[Attribute]",
                    "method": "get_level_members"
                }
            ]
        }

    def _get_geolevel_dimension_cfg(self):
        """
        Basic Geolevel dimension config
        """
        return {
            "available_on_rows": True,
            "show_named_sets": False,
            "has_select_as_group": False,
            "available_as_split_by": False,
            "required": True,
            "available_as_slicer": False,
            "caption": "Geolevel",
            "fancy_select_module": "level_filter_fancy_select",
            "available_on_columns": True,
            "show_calculated_members": False,
            "children": [
                {
                    "unique_name": "[Geolevel].[Business Unit]",
                    "has_children_available": False,
                    "caption": "Business Unit",
                    "has_all_member_available": False,
                    "show_members": True,
                    "method_argument": "[Geolevel].[Business Unit]",
                    "method": "get_level_members"
                },
                {
                    "unique_name": "[Geolevel].[Region]",
                    "has_children_available": False,
                    "caption": "Region",
                    "has_all_member_available": False,
                    "show_members": True,
                    "method_argument": "[Geolevel].[Region]",
                    "method": "get_level_members"
                },
                {
                    "unique_name": "[Geolevel].[Sub Region]",
                    "has_children_available": False,
                    "caption": "Sub Region",
                    "has_all_member_available": False,
                    "show_members": True,
                    "method_argument": "[Geolevel].[Sub Region]",
                    "method": "get_level_members"
                },
                {
                    "unique_name": "[Geolevel].[Country]",
                    "has_children_available": False,
                    "caption": "Country",
                    "has_all_member_available": False,
                    "show_members": True,
                    "method_argument": "[Geolevel].[Country]",
                    "method": "get_level_members"
                },
                {
                    "unique_name": "[Geolevel].[Depot]",
                    "has_children_available": False,
                    "caption": "Depot",
                    "has_all_member_available": False,
                    "show_members": False,
                    "method_argument": "[Geolevel].[Depot]",
                    "method": "get_level_members"
                }
            ]
        }

    def test_get_dimension_options(self):
        attribute_cfg = self._get_attribute_dimension_cfg()

        attributes = self.session.query(TntCLMAttribute).all()
        types = []
        for a in attributes:
            if a.type not in types:
                types.append(a.type)

        options = self.controller.get_dimension_options(self.datasource, self.cube, attribute_cfg)

        keys = options.keys()
        self.assertEqual(len(keys), 3)
        self.assertEqual(keys[0], 'Type')
        self.assertEqual(keys[1], 'Attribute')
        self.assertEqual(keys[2], 'Calculated members')

        self.assertEqual(len(options['Type']), len(types))
        self.assertEqual(len(options['Attribute']), len(attributes))
        self.assertEqual(len(options['Calculated members']), 1)

        # lets change a cfg a bit
        attribute_cfg['children'][1]['show_members'] = False
        attribute_cfg['children'][1]['has_children_available'] = True
        attribute_cfg['show_calculated_members'] = False

        options = self.controller.get_dimension_options(self.datasource, self.cube, attribute_cfg)

        keys = options.keys()
        self.assertEqual(len(keys), 2)
        self.assertEqual(keys[0], 'Type')
        # we don't have Attribute members, only the All option
        self.assertEqual(keys[1], 'All (Attribute)')

        self.assertEqual(len(options['Type']), len(types))
        self.assertEqual(len(options['All (Attribute)']), 1)
        v = simplejson.loads(options['All (Attribute)'][0]['value'])
        self.assertEqual(v['member_unique_name'], "[Attribute].[Attribute].Members")

        # lets say we saved a widget with a filter build with the 'Use .Children' option
        filters = ['[Attribute].[3].Children']
        c = AbstractConfigurableWidget(self.session)
        c._ds = self.datasource
        c.cube = self.cube
        values = [simplejson.dumps(c.get_modified_filter(f), ensure_ascii=False) for f in filters]

        options = self.controller.get_dimension_options(self.datasource, self.cube, attribute_cfg, values)
        self.assertEqual(len(options['Type']), len(types) + 1)
        v = simplejson.loads(options['Type'][len(types)]['value'])
        self.assertEqual(v['member_unique_name'], "[Attribute].[3].Children")
        self.assertEqual(options['Type'][len(types)]['label'], "Text (Children)")

        # now lets filter this dimension
        attribute_cfg = self._get_attribute_dimension_cfg()
        # make available only Scale attributes
        attribute_cfg['filters'] = {"members": ["[Attribute].[1]"], "children": 1}

        options = self.controller.get_dimension_options(self.datasource, self.cube, attribute_cfg)

        scale_attributes = []
        for a in attributes:
            if a.type == 1:
                scale_attributes.append(a)

        keys = options.keys()
        self.assertEqual(len(keys), 3)
        self.assertEqual(keys[0], 'Type')
        self.assertEqual(keys[1], 'Attribute')
        self.assertEqual(keys[2], 'Calculated members')

        self.assertEqual(len(options['Type']), 1)
        self.assertEqual(len(options['Attribute']), len(scale_attributes))
        self.assertEqual(len(options['Calculated members']), 1)

    def test_get_dimension_options_geolevel(self):

        geolevel_cfg = self._get_geolevel_dimension_cfg()

        account_id = self.account.id

        self.controller = ConfigurableWidgetController(self.session, account_id=account_id)

        # currently selected is a BU, Sub Region and a Depot..
        # the BU is available in default selection without filter. The SR is not but its level name is.. and the Depot is not available as level
        # because they are selected, we expect them to be in available options
        values = [
            # business unit
            """{"dimension_unique_name": "[Geolevel]", "member_unique_name": "[Geolevel].[1]", "depth": "1", "level_unique_name": "[Geolevel].[Business Unit]", "member_name": "1", "hierarchy_unique_name": "[Geolevel]"}""",
            # subregion
            """{"dimension_unique_name": "[Geolevel]", "member_unique_name": "[Geolevel].[1].[0].[27]", "depth": "3", "level_unique_name": "[Geolevel].[Sub Region]", "member_name": "27", "hierarchy_unique_name": "[Geolevel]"}""",

            #subregion .children
            """{"dimension_unique_name":"[Geolevel]", "member_unique_name":"[Geolevel].[1].[0].[27].Children","hierarchy_unique_name":"[Geolevel]","depth":"3","member_name":"27","level_unique_name":"[Geolevel].[Sub Region]"}""",

            # depot
            """{"dimension_unique_name": "[Geolevel]", "member_unique_name": "[Geolevel].[1].[0].[27].[11].[4]", "depth": "5", "level_unique_name": "[Geolevel].[Depot]", "member_name": "4", "hierarchy_unique_name": "[Geolevel]"}""",
        ]
        options = self.controller.get_dimension_options(self.datasource, self.cube, geolevel_cfg, values)

        self.assertTrue(options.has_key('Depot'))
        self.assertEquals(len(options['Depot']), 1)

        self.assertEquals(options['Depot'][0]['label'], 'ARO')
        self.assertEquals(options['Depot'][0]['value'], """{"dimension_unique_name": "[Geolevel]", "member_unique_name": "[Geolevel].[1].[0].[27].[11].[4]", "depth": "5", "level_unique_name": "[Geolevel].[Depot]", "member_name": "4", "hierarchy_unique_name": "[Geolevel]"}""" )

        self.assertEquals(options['Sub Region'][3]['label'], 'AM - South America 09')
        self.assertEquals(options['Sub Region'][4]['label'], 'AM - South America 09 (Children)') # children are added directly after there parent


    def test_get_dimensions_options_select_as_group(self):
        """
        tests bug #3820
        """

        self.controller = ConfigurableWidgetController(self.session, account_id=self.account.id)

        attribute_cfg = self._get_attribute_dimension_cfg()
        filters = ["MEMBER [Attribute].[Drivers] AS 'AGGREGATE({[Attribute].[1].[23],[Attribute].[1].[24]})'"]

        c = AbstractConfigurableWidget(self.session)
        c._ds = self.datasource
        c.cube = self.cube
        values = [simplejson.dumps(c.get_modified_filter(f), ensure_ascii=False) for f in filters]

        options = self.controller.get_dimension_options(self.datasource, self.cube, attribute_cfg, values)

        self.assertEqual(options['Custom'][0]['label'], u'Drivers')
        self.assertEqual(options['Custom'][0]['value'], u'"MEMBER [Attribute].[Drivers] AS \'AGGREGATE({[Attribute].[1].[23],[Attribute].[1].[24]})\'"')


    def test_get_dimensions_options_members(self):
        self.controller = ConfigurableWidgetController(self.session, account_id=self.account.id)

        attribute_cfg = {"available_on_rows":True,"show_named_sets":False,"has_select_as_group":False,"available_as_split_by":False,"required":True,"available_as_slicer":False,"caption":"Geolevel","fancy_select_module":"level_filter_fancy_select","available_on_columns":True,"show_calculated_members":False,"children":[{"unique_name":"[Geolevel].[Business Unit]","has_children_available":True,"caption":"Business Unit","has_all_member_available":False,"show_members":True,"method_argument":"[Geolevel].[Business Unit]","method":"get_level_members"},{"unique_name":"[Geolevel].[Region]","has_children_available":True,"caption":"Region","has_all_member_available":False,"show_members":True,"method_argument":"[Geolevel].[Region]","method":"get_level_members"},{"unique_name":"[Geolevel].[Sub Region]","has_children_available":True,"caption":"Sub Region","has_all_member_available":False,"show_members":True,"method_argument":"[Geolevel].[Sub Region]","method":"get_level_members"},{"unique_name":"[Geolevel].[Country]","has_children_available":True,"caption":"Country","has_all_member_available":False,"show_members":True,"method_argument":"[Geolevel].[Country]","method":"get_level_members"},{"unique_name":"[Geolevel].[Depot]","has_children_available":True,"caption":"Depot","has_all_member_available":False,"show_members":False,"method_argument":"[Geolevel].[Depot]","method":"get_level_members"}]}
        filters = ['{"hierarchy_unique_name": "[Geolevel]", "dimension_unique_name": "[Geolevel]", "level_unique_name": "[Geolevel].[Country]", "member_unique_name": "[Geolevel].[Country].Members"}']

        options = self.controller.get_dimension_options(self.datasource, self.cube, attribute_cfg, filters)
        self.assertEqual(options['All (Country)'][0]['label'], u'Country (Members)')

    def test_role_get_dimension_options(self):
        # get geolevel dimension, most representative for testing roles
        geolevel_cfg = self._get_geolevel_dimension_cfg()

        # get an account that has permission only on a specific geolevel
        region = self.session.query(TntGeolevel).filter(func.substr(TntGeolevel.name, -7) == 'Benelux').first()
        account_id = self.session.query(TntGeolevelAccount).filter(TntGeolevelAccount.geolevel==region).first().account_id

        self.controller = ConfigurableWidgetController(self.session, account_id=account_id)

        options = self.controller.get_dimension_options(self.datasource, self.cube, geolevel_cfg)

        keys = options.keys()
        self.assertEqual(len(keys), 3)
        self.assertEqual(keys[0], 'Region')
        self.assertEqual(keys[1], 'Sub Region')
        self.assertEqual(keys[2], 'Country')

        self.assertEqual(len(options['Region']), 1)
        self.assertEqual(len(options['Sub Region']), 2)
        self.assertEqual(len(options['Country']), 3)

        geolevel_cfg['filters'] = {"members": ["[Geolevel].[Region].[1]"], "children": 2}
        # user has no permission on the region set in filters, so we should get nothing back
        options = self.controller.get_dimension_options(self.datasource, self.cube, geolevel_cfg)
        self.assertEqual(len(options), 0)


    def test_level_descendants(self):

        self.controller = ConfigurableWidgetController(self.session, account_id=self.account.id)

        geolevel_cfg = self._get_geolevel_dimension_cfg()

        # the members to get descendents for
        filtered = [
            """{"dimension_unique_name": "[Geolevel]", "member_unique_name": "[Geolevel].[1]", "depth": "1", "level_unique_name": "[Geolevel].[Business Unit]", "member_name": "1", "hierarchy_unique_name": "[Geolevel]"}""",
        ]

        # currently selected members
        selected = [
            """{"dimension_unique_name": "[Geolevel]", "member_unique_name": "[Geolevel].[1]", "depth": "1", "level_unique_name": "[Geolevel].[Business Unit]", "member_name": "1", "hierarchy_unique_name": "[Geolevel]"}""",
            """{"member_unique_name":"[Geolevel].[1].[0].[27].Children","hierarchy_unique_name":"[Geolevel]","depth":"3","member_name":"27","level_unique_name":"[Geolevel].[Sub Region]","dimension_unique_name":"[Geolevel]"}""",
        ]

        descendents = self.controller.get_level_descendants(self.datasource, self.cube, geolevel_cfg, filtered, selected)

        # expect 5 option groups
        self.assertEquals(len(descendents.keys()), 5)

        # expect the sub region options to contain the selected .children
        self.assertTrue(descendents.has_key('Sub Region'))
        self.assertTrue(len(descendents['Sub Region']), 4)
        self.assertTrue(descendents['Sub Region'][3]['label'], 'AM - South America 09 (Children)')
        self.assertTrue(descendents['Sub Region'][3]['value'], """{"member_unique_name":"[Geolevel].[1].[0].[27].Children","hierarchy_unique_name":"[Geolevel]","depth":"3","member_name":"27","level_unique_name":"[Geolevel].[Sub Region]","dimension_unique_name":"[Geolevel]"}""")

    def test_syncdb_tnt_widget_ticket_3775(self):
        from iqportal.tnt.report.configurable_widgets.satisfaction_kpi_graph_widget import SatisfactionKPIgraphWidget

        widget = SatisfactionKPIgraphWidget(self.sa_session)
        af = widget.modify_applied_filters(widget.applied_filters())
        self.assertEqual(af['dimension_2'], [u'{"hierarchy_unique_name": "[Geolevel]", "dimension_unique_name": "[Geolevel]", "level_unique_name": "[Geolevel].[Country]", "member_unique_name": "[Geolevel].[Country].Members", "member_name": "11"}'])


class TntCLMConfigurableWidgetsTestCase(AbstractConfigurableWidgetReporterTestCase):
    DATASOURCE = 'TNT-CLM'

    def setUp(self):
        """
        The script for reseting the db and creating all the tables
        """
        self.connect_db()

        account = self.session.query(Account).filter(Account.name=='sysadmin').one()
        # we populate necessary tnt account data
        c = TntWaveController(self.session, account_id=account.id)
        license, account = c.populate_tnt_account()

        self.account = account
        self.group = account.groups[0].group

        self.controller = ConfigurableWidgetController(self.session, account_id=self.account.id)

        self.set_datasource()

        report_sync.sync(self.session)
        pc = ProjectController(self.session)
        pc.flush_schema(self.datasource.id)

        self.cube = 'TntCLMVirtualCube'

    def test_default_reporter(self):
        reporter, applied_config = self._get_configurable_widget_defaults('Satisfaction Table (new)')

        mdx = self._get_mdx(reporter.get_mdx_config(applied_config))
        self.assertMdx("""
        SELECT NON EMPTY {[KPI].[No. of Answers],[KPI].[Dissatisfied %],[KPI].[Meeting Cust. Exp. %],[KPI].[Exceeding Cust. Exp. %],[KPI].[NCS Score],[KPI].[Mean Score],[KPI].[OES],[KPI].[No. of Answers]} ON COLUMNS,
            NON EMPTY {[Wave].[1].Children} ON ROWS
        FROM [TntCLMVirtualCube]
        WHERE CROSSJOIN({[Attribute].[Overall satisfaction]}, {[Service].[6]})
        """, mdx[0])

    def test_diff_and_avg_reporter(self):
        reporter, applied_config = self._get_configurable_widget_defaults('Diff and Avg')

        applied_config = self._reset_config(applied_config)

        # only belgium
        dimensions = {
            'Geolevel': ['[Geolevel].[Country].[22]'],
            'Wave': ['[Wave].Children']
        }
        applied_config = self._smorgle_applied_config(applied_config, dimensions, self.cube)

        mdx = self._get_mdx(reporter.get_mdx_config(applied_config))
        self.assertMdx(u"""
        WITH set [set_avg] as {[Geolevel].[Country].[22]} MEMBER [Geolevel].[Avg] as ' avg([set_avg]) ) ',
        SOLVE_ORDER=3 set [set_diff] as {[Wave].[All Waves].Children} MEMBER [Wave].[Diff] as ' IIf([set_diff].Item([set_diff].Count - 1) > 0,
        [set_diff].Item([set_diff].Count - 1) - [set_diff].Item([set_diff].Count - 2), null ) ', SOLVE_ORDER=3 SELECT {[set_diff], [Wave].[Diff]} ON COLUMNS,
        {[set_avg], [Geolevel].[Avg]} ON ROWS FROM [TntCLMVirtualCube] WHERE CROSSJOIN(CROSSJOIN({[Attribute].[Overall satisfaction]}, {[KPI].[NCS Score]}), {[Service].[6]})
        """, mdx[0])

    def test_general_attributes_reporter(self):
        reporter, applied_config = self._get_configurable_widget_defaults('General Attributes')

        applied_config = self._reset_config(applied_config)

        # only belgium
        dimensions = {
            'Wave': ['[Wave].Children']
        }
        applied_config = self._smorgle_applied_config(applied_config, dimensions, self.cube)

        # lets test the default settings first
        mdx = self._get_mdx(reporter.get_mdx_config(applied_config))
        self.assertMdx("""
        WITH SET [All General Attributes (Members)] AS 'ORDER({[Attribute].[2].Children,[Attribute].[4].Children,[Attribute].[6].Children}, [Attribute].[Attribute].CurrentMember.Caption, BASC)'
        SELECT NON EMPTY {[Wave].[All Waves].Children} ON COLUMNS,
            NON EMPTY CROSSJOIN({[All General Attributes (Members)]}, {[Answer].[Answer].Members}) ON ROWS
        FROM [TntCLMVirtualCube] WHERE {[Service].[6]}
        """, mdx[0])

        # now we move a single attribute to the slicer axis (this is used for graphs)
        reporter, applied_config = self._get_configurable_widget_defaults('General Attributes')
        applied_config = self._reset_config(applied_config)

        attribute_index = applied_config['dimensions']['Attribute']
        applied_config['dimension_%s' % attribute_index] = ['[Attribute].[2].[31]']
        applied_config['axes_%s' % attribute_index] = 'slicers'
        applied_config['row_order'] = ['Answer']

        dimensions = {
            'Wave': ['[Wave].Children']
        }

        applied_config = self._smorgle_applied_config(applied_config, dimensions, self.cube)
        mdx = self._get_mdx(reporter.get_mdx_config(applied_config))
        self.assertMdx("""
        SELECT NON EMPTY {[Wave].[All Waves].Children} ON COLUMNS,
            NON EMPTY {[Answer].[Answer].Members} ON ROWS
        FROM [TntCLMVirtualCube]
        WHERE CROSSJOIN({[Attribute].[2].[31]}, {[Service].[6]})
        """, mdx[0])

    def test_nps_reporter(self):
        reporter, applied_config = self._get_configurable_widget_defaults('NPS KPI')

        applied_config = self._reset_config(applied_config)

        # only business unit Northern Europe
        dimensions = {
            'Geolevel': ['[Geolevel].[Business Unit].[3]'],
            'Wave': []
        }
        applied_config = self._smorgle_applied_config(applied_config, dimensions, self.cube)

        mdx = self._get_mdx(reporter.get_mdx_config(applied_config))
        self.assertMdx("""
        SELECT NON EMPTY {[Answer].[% NPS],[Answer].[NPS KPI Promotors %],[Answer].[NPS KPI Detractor %]} ON COLUMNS,
        NON EMPTY {[Geolevel].[Business Unit].[3]} ON ROWS FROM [TntCLMVirtualCube] WHERE {[Service].[6]}
        """, mdx[0])





