# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

import os
import subprocess

from datetime import datetime, timedelta
from random import randint

from django.conf import settings

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.contrib.contact.controllers import ContactController
from basesoft.web.contrib.account.models import Account

from iqportal.tnt.test.controllers import TntWaveController
from iqportal.tnt.response.models import TntCLMWaveResponse
from iqportal.tnt.clm.models import TntCLMWave, TntCLMLocale
from iqportal.tnt.clm.controllers import WaveController
from iqportal.tnt.response.controllers import ResponseController, ParticipantController,\
    PriorityController, ResultsController, ManagerParticipantController
from twisted.trial.unittest import SkipTest

class ResponseControllerWave36TestCase(iqportal_unittest.IQPortalTestCase):

    def setUp(self):

        self.connect_db()

        #create an account so we can check permission with it
        account = self.session.query(Account).filter(Account.name=='sysadmin').one()
        # we populate necessary tnt account data
        c = TntWaveController(self.session, account_id=account.id)
        license, account = c.populate_tnt_account()

        self.account = account
        self.group = account.groups[0].group

        self.generator = TntWaveController(self.session, account_id=self.account.id)
        self.wave_id = self.generator.create_wave_36()
        self.controller = ResponseController(self.session)

        self.setUpContactLists()

    def setUpContactLists(self):
        """
        Creates contacts and contact lists
        """
        #self._attach_permissions_to_group(self.group, 'view_depot', 'link_to_contact_list', 'create_contact', 'create_contact_list')
        contact_controller = ContactController(self.session, account_id=self.account.id, contact_type='TntCLMContact')

        person = {'first_name': u'John',
                'middle_name': u'',
                'last_name': u'Smith',
                'title': u'mr',
                'gender': None,
                'email': u'john@mail-test.basesoft.com',
                'position': u'assistent',
                'department': u'marketing',
                'phone': u'0040123456789',
                'mobile_phone': '',
                'fax': '',
                'birth_date': '1978-10-10',
                'address_data': {'country': '181',
                                 'postcode': u"400295",
                                 'street': u" Bihorului 10",
                                 'city': u"Cluj Napoca",
                                 'state': u"Cluj"
                                }
               }

        company1 = {
            'depot': 'F01',
            'revenue_band': 'C',
            'company_type': 'TntCLMCompany',
            'account_number': '1234567890',
            'locale_id': self.session.query(TntCLMLocale).filter(TntCLMLocale.label=='NLNLI').one().id
        }
        company2 = {
            'depot': 'F01',
            'revenue_band': 'F',
            'company_type': 'TntCLMCompany',
            'account_number': '1234567891',
            'locale_id': self.session.query(TntCLMLocale).filter(TntCLMLocale.label=='NLNLI').one().id
        }
        company3 = {
            'depot': 'F01',
            'revenue_band': 'A',
            'company_type': 'TntCLMCompany',
            'account_number': '1234567892',
            'locale_id': self.session.query(TntCLMLocale).filter(TntCLMLocale.label=='NLNLI').one().id
        }

        # all TNT contacts have a company
        c1 = contact_controller.save_or_update_contact(None, person, company1, self.group.id)
        c2 = contact_controller.save_or_update_contact(None, person, company2, self.group.id)
        c3 = contact_controller.save_or_update_contact(None, person, company3, self.group.id)

        self.cl1 = contact_controller.save_or_update_contact_list(None, u'Contact List 1', self.group.id)
        contact_controller.link_contacts([c1, c2], self.cl1)

        self.cl2 = contact_controller.save_or_update_contact_list(None, u'Contact List 2', self.group.id)
        contact_controller.link_contacts([c2, c3], self.cl2)

    def test_save_or_update_invitees(self):
        self._attach_permissions_to_group(self.group, 'view_tnt_clm_wave')

        wc = WaveController(self.session, account_id=self.account.id)

        wave = wc.get_waves(self.wave_id).pop()
        wave.status = TntCLMWave.STATUS_ONLINE
        wave.contact_lists.append(self.cl1)
        self.session.add(wave)
        self.session.flush()

        self.controller.save_or_update_invitees(wave)
        self.assertEqual(len(wave.invitees), 2)

        # test additional invitees, no dupes!
        wave.contact_lists.append(self.cl2)
        self.session.add(wave)
        self.session.flush()

        self.controller.save_or_update_invitees(wave)
        self.assertEqual(len(wave.invitees), 3)

        for i in wave.invitees:
            self.assertEqual(i.contact.company.customer.country_id, 155) # NL

    def _get_responses(self):
        wc = WaveController(self.session, account_id=self.account.id)
        state = {}
        wave = wc.get_wave_state(self.wave_id)['wave']
        for a in wave['attributes']:
            state[a['id']] = {'type': a['type']}
            if a['type'] == 1: #scale
                state[a['id']]['value'] = 1 #select first radio
            if a['type'] == 2 or a['type'] == 6: #single choice
                state[a['id']]['value'] = a['answers'][0]['id'] # select first answer radio
            if a['type'] == 4: #multiple choice
                state[a['id']]['value'] = [a['answers'][0]['id'], a['answers'][1]['id']] # select first 2 answers
            if a['type'] == 3: #text
                if not a.has_key('answers'):
                    state[a['id']]['value'] = ' single answer'
                else:
                    state[a['id']]['value'] = {}
                    for answ in a['answers']:
                        state[a['id']]['value'][answ['id']] = 'answer %s' % answ['id']
            # type 4 (multiple choice) and 5 (range) not used for w36+
        return state

    def test_save_participant_responses(self):
        self._attach_permissions_to_group(self.group, 'view_tnt_clm_wave')

        wc = WaveController(self.session, account_id=self.account.id)

        wave = wc.get_waves(self.wave_id).pop()
        wave.status = TntCLMWave.STATUS_ONLINE
        wave.contact_lists.append(self.cl1)
        self.session.add(wave)
        self.session.flush()
        self.controller.save_or_update_invitees(wave)

        responses = self._get_responses()
        state = {'responses': responses, 'opt_out': False}

        pc = ParticipantController(self.session, wave.invitees[0].invitee_hash)

        pc.save_responses(state)
        saved_responses = pc.get_responses()
        self.assertDeepEqual(state, saved_responses)

        # 'remove' response for first attribute
        resp = {}
        cmp_state = {} # state to compare after saving new responses

        for a,v in responses.items():
            if v['type'] == 1:
                v['value'] = 3

            resp[a] = v
            cmp_state[a] = v

        new_state = {'responses': resp, 'opt_out': False}

        pc.save_responses(new_state)
        saved_responses = pc.get_responses()
        cmp_responses = {'responses': cmp_state, 'opt_out': False}
        self.assertDeepEqual(cmp_responses, saved_responses)


    def test_import_wave_responses(self):

        # create the file
        self.filepointer = "/tmp/test_import_wave_respondents.csv"
        f = open(self.filepointer, 'w')
        f.write("%s\n%s\n%s\n%s"
                % ("COUNTRY,DEPOT,ACCOUNTNR,EMAIL,REVENUE,ANONYMOUS,CHANNEL,LOCATION,VR1,VR2,VR3,VR4,"+
                   "VR5,VR6,VR7,VR8,VR9,VR10,VR11,VR12,VR13,VR14,VR15,VR16,VR17,VR18,VR19,"+
                   "VR20,VR21,VR22,VR23,VR24,VR25,VR26,VR27,VR28,VR29,VR30,VRA,VRB,VRC,VRD,"+
                   "VRE,VRF,VRG",

                   "NL,F01,1234567890,testing@mail.clusternet,C,N,Internet,,99,99,99,99,99,99,99,"+
                   "2,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,"+
                   "99,99,99,99,1,99,99,99,99,99,99",

                   "NL,F01,1234567891,testing@mail.clusternet,F,N,Internet,I,99,99,99,99,99,99,99,"+
                   "99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,"+
                   "99,99,99,99,99,99,99,99,99,99,99",

                   "NL,F01,1234567892,testing@mail.clusternet,A,N,Internet,I,99,99,99,99,99,1,99,"+
                   "99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,"+
                   "99,99,5,99,99,99,99,99,99,99,4")
                )
        f.close()

        self._attach_permissions_to_group(self.group, 'view_tnt_clm_wave')

        wc = WaveController(self.session, account_id=self.account.id)

        wave = wc.get_waves(self.wave_id).pop()
        wave.status = TntCLMWave.STATUS_ONLINE
        wave.contact_lists.append(self.cl1)
        wave.contact_lists.append(self.cl2)
        self.session.add(wave)
        self.session.flush()
        self.controller.save_or_update_invitees(wave)

        import_controller = ManagerParticipantController(self.session, account_id=self.account.id)
        ret = import_controller.import_responses(self.wave_id, 'test_import_wave_respondents.csv', '/tmp/', ",", False, 'csv')

        self.assertEqual(ret['imported'], 2)
        # one participant has no responses in the file
        self.assertEqual(ret['failures'], 1)



    def test_participant_lock(self):
        """
        Test for bug #3469. Fires two parallel requests to create a participant id
        """

        self._attach_permissions_to_group(self.group, 'view_tnt_clm_wave')

        wc = WaveController(self.session, account_id=self.account.id)

        wave = wc.get_waves(self.wave_id).pop()
        wave.status = TntCLMWave.STATUS_ONLINE
        wave.contact_lists.append(self.cl1)
        self.session.add(wave)
        self.session.flush()
        self.controller.save_or_update_invitees(wave)

        hash_ = wave.invitees[0].invitee_hash

        self._start_django()

        args = ['/usr/bin/curl', '--fail', '--silent', '--output', '/dev/null', 'http://'+settings.SELENIUM_DJANGO_ADDR+'/iqportal.tnt.response/display/%s/' % hash_]

        p1 = subprocess.Popen(args)
        p2 = subprocess.Popen(args)

        self.assertEqual(p1.wait(), 0, 'curl exit with errorcode')
        self.assertEqual(p2.wait(), 0, 'curl exit with errorcode')


class ResponseControllerWave40aTestCase(iqportal_unittest.IQPortalTestCase):

    def setUp(self):

        self.connect_db()

        #create an account so we can check permission with it
        account = self.session.query(Account).filter(Account.name=='sysadmin').one()
        # we populate necessary tnt account data
        c = TntWaveController(self.session, account_id=account.id)
        license, account = c.populate_tnt_account()

        self.account = account
        self.group = account.groups[0].group

        self.generator = TntWaveController(self.session, account_id=self.account.id)
        self.wave_id = self.generator.create_wave_40a()
        self.controller = ResponseController(self.session)

        self.setUpContactLists()

    def setUpContactLists(self):
        """
        Creates contacts and contact lists
        """
        #self._attach_permissions_to_group(self.group, 'view_depot', 'link_to_contact_list', 'create_contact', 'create_contact_list')
        contact_controller = ContactController(self.session, account_id=self.account.id, contact_type='TntCLMContact')

        person = {'first_name': u'John',
                'middle_name': u'',
                'last_name': u'Smith',
                'title': u'mr',
                'gender': None,
                'email': u'john@mail-test.basesoft.com',
                'position': u'assistent',
                'department': u'marketing',
                'phone': u'0040123456789',
                'mobile_phone': '',
                'fax': '',
                'birth_date': '1978-10-10',
                'address_data': {'country': '181',
                                 'postcode': u"400295",
                                 'street': u" Bihorului 10",
                                 'city': u"Cluj Napoca",
                                 'state': u"Cluj"
                                }
               }

        company1 = {
            'depot': 'F01',
            'revenue_band': 'C',
            'company_type': 'TntCLMCompany',
            'account_number': '1234567890',
            'locale_id': self.session.query(TntCLMLocale).filter(TntCLMLocale.label=='NLNLI').one().id
        }
        company2 = {
            'depot': 'F01',
            'revenue_band': 'F',
            'company_type': 'TntCLMCompany',
            'account_number': '1234567891',
            'locale_id': self.session.query(TntCLMLocale).filter(TntCLMLocale.label=='NLNLI').one().id
        }
        company3 = {
            'depot': 'F01',
            'revenue_band': 'A',
            'company_type': 'TntCLMCompany',
            'account_number': '1234567892',
            'locale_id': self.session.query(TntCLMLocale).filter(TntCLMLocale.label=='NLNLI').one().id
        }

        # all TNT contacts have a company
        c1 = contact_controller.save_or_update_contact(None, person, company1, self.group.id)
        c2 = contact_controller.save_or_update_contact(None, person, company2, self.group.id)
        c3 = contact_controller.save_or_update_contact(None, person, company3, self.group.id)

        self.cl1 = contact_controller.save_or_update_contact_list(None, u'Contact List 1', self.group.id)
        contact_controller.link_contacts([c1, c2], self.cl1)

        self.cl2 = contact_controller.save_or_update_contact_list(None, u'Contact List 2', self.group.id)
        contact_controller.link_contacts([c2, c3], self.cl2)

    def test_save_participant_responses(self):
        self._attach_permissions_to_group(self.group, 'view_tnt_clm_wave')

        wc = WaveController(self.session, account_id=self.account.id)

        wave = wc.get_waves(self.wave_id).pop()
        wave.status = TntCLMWave.STATUS_ONLINE
        wave.contact_lists.append(self.cl1)
        self.session.add(wave)
        self.session.flush()
        self.controller.save_or_update_invitees(wave)

        state = {}
        wave_state = wc.get_wave_state(self.wave_id)['wave']
        # wave 40a only has scale, nps and text
        for a in wave_state['attributes']:
            state[a['id']] = {'type': a['type']}
            if a['type'] == 1: #scale
                state[a['id']]['value'] = 1 #select first radio
            if a['type'] == 7: #nps
                state[a['id']]['value'] = a['answers'][0]['id'] # select first answer radio
            if a['type'] == 3: #text
                if not a.has_key('answers'):
                    state[a['id']]['value'] = ' single answer'
                else:
                    state[a['id']]['value'] = {}
                    for answ in a['answers']:
                        state[a['id']]['value'][answ['id']] = 'answer %s' % answ['id']
        state = {'responses': state, 'opt_out': False}

        pc = ParticipantController(self.session, wave.invitees[0].invitee_hash)

        pc.save_responses(state)
        saved_responses = pc.get_responses()
        self.assertDeepEqual(state, saved_responses)

        resp = {}
        cmp_state = {} # state to compare after saving new responses

        for a,v in state['responses'].items():
            if v['type'] == 1:
                v['value'] = 3
            if v['type'] == 1:
                v['value'] = 3

            resp[a] = v
            cmp_state[a] = v

        new_state = {'responses': resp, 'opt_out': False}

        pc.save_responses(new_state)
        saved_responses = pc.get_responses()
        cmp_responses = {'responses': cmp_state, 'opt_out': False}
        self.assertDeepEqual(cmp_responses, saved_responses)


    def test_response_export(self):

        rc = ResponseController(self.session)

        contact_list = self.generator.generateContactList(self.wave_id, 10)
        wc = WaveController(self.session, account_id=self.account.id)
        wave = wc.get_waves(self.wave_id).pop()
        wave.status = TntCLMWave.STATUS_ONLINE
        wave.contact_lists.append(contact_list)
        self.session.add(wave)
        self.session.flush()
        rc.save_or_update_invitees(wave)
        self.generator.generateResponses(self.wave_id)

        # use the ResponseController to export a xls
        rc = ResponseController(self.session, account_id=self.account.id)
        file_hash = rc.export_response_csv(self.wave_id)

        file_path = os.path.join(settings.TMP_FILE_REPOSITORY, 'tnt_wave_%s.xls' % file_hash)
        self.assertTrue(os.path.exists( file_path ))

        #print 'file_path: %s' % file_path


    def test_import_wave_responses(self):

        # create the file
        self.filepointer = "/tmp/test_import_wave_respondents.csv"
        f = open(self.filepointer, 'w')
        f.write("%s\n%s\n%s\n%s"
                % ("COUNTRY,DEPOT,ACCOUNTNR,EMAIL,REVENUE,ANONYMOUS,CHANNEL,LOCATION,"+
                   "VR1,VR2,VR3,VR4,VR5,VR6,VR7,VR8,VR9,VR10,VRA",

                   "NL,F01,1234567890,testing@mail.clusternet,C,N,Internet,,"+
                   "99,99,99,99,99,99,99,2,99,99,1",

                   "NL,F01,1234567891,testing@mail.clusternet,F,N,Internet,I,"+
                   "99,99,99,99,99,99,99,99,99,99,99",

                   "NL,F01,1234567892,testing@mail.clusternet,A,N,Internet,I,"+
                   "99,99,99,99,99,1,99,99,99,99,3")
                )
        f.close()

        self._attach_permissions_to_group(self.group, 'view_tnt_clm_wave')

        wc = WaveController(self.session, account_id=self.account.id)

        wave = wc.get_waves(self.wave_id).pop()
        wave.status = TntCLMWave.STATUS_ONLINE
        wave.contact_lists.append(self.cl1)
        wave.contact_lists.append(self.cl2)
        self.session.add(wave)
        self.session.flush()
        self.controller.save_or_update_invitees(wave)

        import_controller = ManagerParticipantController(self.session, account_id=self.account.id)
        ret = import_controller.import_responses(self.wave_id, 'test_import_wave_respondents.csv', '/tmp/', ",", False, 'csv')

        self.assertEqual(ret['imported'], 2)
        # one participant has no responses in the file
        self.assertEqual(ret['failures'], 1)

class PriorityExportTestCase(iqportal_unittest.IQPortalTestCase):

    def setUp(self):
        self.connect_db()

        account = self.session.query(Account).filter(Account.name=='sysadmin').one()
        # we populate necessary tnt account data
        c = TntWaveController(self.session, account_id=account.id)
        license, account = c.populate_tnt_account()

        self.account = account
        self.group = self.account.groups[0].group

        self.generator = TntWaveController(self.session, account_id=self.account.id)
        self.wave_id = self.generator.create_wave_36()
        self.controller = PriorityController(self.session)

    def test_CSV_export(self):
        """
        """
        rc = ResponseController(self.session)

        # generate a contact list with 10 contacts
        contact_list = self.generator.generateContactList(self.wave_id, 10)

        wc = WaveController(self.session, account_id=self.account.id)

        wave = wc.get_waves(self.wave_id).pop()
        wave.status = TntCLMWave.STATUS_ONLINE
        wave.contact_lists.append(contact_list)
        self.session.add(wave)
        self.session.flush()

        rc.save_or_update_invitees(wave)

        # save responses for a few invitees that would trigger the priority
        self.generator.generateResponses(self.wave_id, {'attribute_1': 2, 'attribute_33': 1})

        # create csv file
        self.controller.write_CSV()


class ResultsControllerTestCase(iqportal_unittest.IQPortalTestCase):

    def setUp(self):
        self.connect_db()

        account = self.session.query(Account).filter(Account.name=='sysadmin').one()
        # we populate necessary tnt account data
        c = TntWaveController(self.session, account_id=account.id)
        license, account = c.populate_tnt_account()

        self.account = account
        self.group = self.account.groups[0].group


        # create the necessary controllers
        self.generator = TntWaveController(self.session, account_id=self.account.id)

    def test_generate_respondents_chart(self):
        """
        Create a chart for the
        """
        self._attach_permissions_to_group(self.group, 'view_tnt_clm_wave', 'link_to_contact_list',
                                          'create_contact', 'create_contact_list')

        self.wave_id = self.generator.create_wave_36()

        # create more locales for the wave
        locales_label = ['XXENI', 'CNCNI', 'JPJPI', 'EGARI', 'DEDEI', 'NLNLI', 'BEFRI']
        locales = self.session.query(TntCLMLocale).filter(TntCLMLocale.label.in_(locales_label)).all()
        for l in locales:
            self.generator.translate(self.wave_id, l)

        rc = ResponseController(self.session)
        wc = WaveController(self.session, account_id=self.account.id)

        cl = self.generator.generateContactList(self.wave_id, 90)

        wave = wc.get_waves(self.wave_id).pop()
        wave.status = TntCLMWave.STATUS_ONLINE
        wave.contact_lists.append(cl)

        # start the wave one month ago, finish it yesterday
        now = datetime.now()
        wave.begin_date = now - timedelta(days=20)
        wave.end_date = now - timedelta(days=1)

        self.session.add(wave)
        self.session.flush()

        rc.save_or_update_invitees(wave)

        # save responses for a few invitees that would trigger the priority
        self.generator.generateResponses(self.wave_id)

        # change the date of the responses
        for invitee in wave.invitees:
            pc = ParticipantController(self.session, invitee.invitee_hash)
            wave_responses = self.session.query(TntCLMWaveResponse).filter(TntCLMWaveResponse.participant==pc.participant).all()
            # set the creation date randomly in the wave publish interval
            d = now - timedelta(days=randint(5, 19))
            # change the hour too
            d = d.replace(hour=randint(0, 23))
            for wr in wave_responses:
                wr.created = d
                self.session.add(wr)
        self.session.flush()

        controller = ResultsController(self.session, account_id=self.account.id)

        chart_data = controller.generate_respondents_result(self.wave_id)
        raise SkipTest('Skipped due to reportlab being deprecated')

