# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

from datetime import datetime

from basesoft.web.test.unit import iqportal_unittest

from iqportal.tnt.test.controllers import TntWaveController
from iqportal.tnt.clm import controllers, schedule
from iqportal.tnt.clm.models import TntCLMWave, TntCLMWaveLocaleRevision, TntCLMLocale

from iqportal.tnt.clm.mailing import MailController
from basesoft.web.contrib.contact.controllers import ContactController

class ScheduleTest(iqportal_unittest.IQPortalTestCase):
    def setUp(self):
        """
        """
        self.connect_db()

        #create an account so we can check permission with it
        self.account = self._create_active_account()

        # create the necessary controllers
        self.group = self.account.groups[0].group

        self.generator = TntWaveController(self.session, account_id=self.account.id)
        self.wave_id = self.generator.create_wave_36()
        self.controller = controllers.WaveController(self.session, account_id=self.account.id)

    def setUpContactLists(self):
        """
        Creates contacts and contact lists
        """
        self._attach_permissions_to_group(self.group, 'link_to_contact_list', 'create_contact', 'create_contact_list',\
                                          'view_contact_list')
        contact_controller = ContactController(self.session, account_id=self.account.id)

        person = {'first_name': u'John',
                'middle_name': u'',
                'last_name': u'Smith',
                'title': u'mr',
                'gender': None,
                'email': u'john@mail-test.basesoft.com',
                'position': u'assistent',
                'department': u'marketing',
                'phone': u'0040123456789',
                'mobile_phone': '',
                'fax': '',
                'birth_date': '1978-10-10',
                'address_data': {'country': '181',
                                 'postcode': u"400295",
                                 'street': u" Bihorului 10",
                                 'city': u"Cluj Napoca",
                                 'state': u"Cluj"
                                }
               }

        company = {
            'company_type': 'TntCLMCompany',
            'account_number': '1234567890',
            'locale_id': self.session.query(TntCLMLocale).filter(TntCLMLocale.label=='NLNLI').one().id
        }

        c1 = contact_controller.save_or_update_contact(None, person, company, self.group.id)
        c2 = contact_controller.save_or_update_contact(None, person, None, self.group.id)
        c3 = contact_controller.save_or_update_contact(None, person, None, self.group.id)

        self.cl1 = contact_controller.save_or_update_contact_list(None, u'Contact List 1', self.group.id)
        contact_controller.link_contacts([c1, c2], self.cl1)

        self.cl2 = contact_controller.save_or_update_contact_list(None, u'Contact List 2', self.group.id)
        contact_controller.link_contacts([c2, c3], self.cl2)

    def setUpMails(self):
        """
        Creates contacts and contact lists
        """
        self._attach_permissions_to_group(self.group, 'create_mail', 'create_tnt_wave_mail')
        mail_controller = MailController(self.session, account_id=self.account.id)

        state = {
            "mail": {
                "mail_from_text": "MockMailer",
                "name": u"Some tnt wave email",
                "objectName": u"TntCLMWaveMail",
                "dbID": None,

                "locales": {
                    'GBEND': {
                        'subject': u"Mijn test onderworpen",
                        'content': u"Hallo <b>wereld !</b>",
                        'link_label': None,
                    },
                    'NLNLD': {
                        'subject': u'My test subject',
                        'content': u"Hello <b>world !</b>",
                        'link_label': None,
                    },
                },

                "fallbackLocale": {'value': u'GBEND', 'name': u'English' },
            }
        }
        self.mail = mail_controller.save_state(state, self.group)

    def test_set_schedule_data(self):
        self._attach_permissions_to_group(self.group, 'view_tnt_clm_wave', 'update_tnt_clm_wave')

        self.setUpContactLists()
        self.setUpMails()

        begin_date = datetime(2009, 4, 16, 14, 3) # sometime in the past.

        contact_lists = [self.cl1.id, self.cl2.id]

        mailings = [{'dbID': None,
                    'domID': u'domID',
                    'mail_id': self.mail.id,
                    'mail_from_text': u'Test TNT',
                    'send_date': begin_date,
                    'contact_lists': [self.cl2.id]
                    }]
        self.controller.set_wave_schedule_data(self.wave_id, begin_date, None, contact_lists, mailings)
        state = self.controller.get_wave_schedule_data(self.wave_id)

        self.assertEqual(state['begin_date'], begin_date)

        self.assertEqual(len(state['contact_lists']), 2)
        self.assertEqual(state['contact_lists'][0], self.cl1.id)

        self.assertEqual(len(state['mailings']), 1)
        self.assertEqual(len(state['mailings'][0]['contact_lists']), 1)
        self.assertEqual(state['mailings'][0]['mail_id'], self.mail.id)
        self.assertEqual(state['mailings'][0]['contact_lists'][0], self.cl2.id)
        self.assertEqual(state['mailings'][0]['send_date'], begin_date)

    def test_publish_wave(self):
        self._attach_permissions_to_group(self.group, 'view_tnt_clm_wave', 'schedule_tnt_clm_wave',
                                          'translate_tnt_clm_wave', 'approve_tnt_clm_translations', 'update_tnt_clm_wave')

        begin_date = datetime(2009, 4, 16, 14, 3) # sometime in the past.
        self.controller.set_wave_schedule_data(self.wave_id, begin_date=begin_date)

        self.controller.schedule_wave(self.wave_id)
        t = schedule.PublishWaveTask()
        t.execute()

        # could not publish due to unapproved locale revisions.
        wave = self.controller.get_waves(self.wave_id).pop()
        self.assertEqual(wave.status, TntCLMWave.STATUS_SCHEDULED)

        for rv in self.controller.get_available_wave_locales(wave):
            self.controller.set_wave_translation_status(wave.id, rv.id, TntCLMWaveLocaleRevision.STATUS_PENDING)
            self.controller.set_wave_translation_status(wave.id, rv.id, TntCLMWaveLocaleRevision.STATUS_ACTIVE_APPROVED)

        schedule.PublishWaveTask().execute()
        self.assertEqual(wave.status, TntCLMWave.STATUS_ONLINE)

        wave.end_date = datetime(2010, 12, 12, 12, 12)
        self.session.add(wave)
        self.session.flush()

        # FINISH HIM!
        schedule.FinishWaveTask().execute()

        # FATALITY!
        self.session.expire(wave)
        self.assertEqual(wave.status, TntCLMWave.STATUS_FINISHED)


