# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

import datetime
import hashlib
import time

from copy import copy
from django.conf import settings

from sqlalchemy.orm.exc import NoResultFound

from twisted.trial.unittest import SkipTest

from basesoft.web.core.exceptions import PublicUserException
from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.contrib.account.models import Account

from iqportal.iqube.core.controllers import DataSourceController
from iqportal.tnt.report.reporters import TntCLMSatisfactionReporter,\
    TntCLMLoyaltyLeagueTableReport, TntCLMSatisfactionGraphsReporter,\
    TntCLMLoyaltyScoreReporter, TntCLMNCSLeagueTableReport, TntCLMActionPlanningReport
from iqportal.tnt.response.models import TntCLMWaveResponse,\
    TntCLMWaveParticipant, TntCLMWaveInvitee
from iqportal.tnt.report.schedule import AggregateResponsesTask
from iqportal.tnt.clm.controllers import TntCLMLocaleController
from basesoft.web.contrib.i18n.models import Country
from iqportal.tnt.report.controllers import TntCLMTntCLMWaveAggregator
from iqportal.tnt.test.controllers import TntWaveController
from iqportal.tnt.clm.models import TntCLMWave
from iqportal.tnt.response.controllers import ResponseController


class TntClmReportersTestCase(iqportal_unittest.IQPortalTestCase):

    _empty_filters = {
        'answers': [],
        'attributes': [],
        'depots': [],
        'geolevels': [],
        'services': [],
        'waves': [],
    }

    def setUp(self):

        self.connect_db()

        try:
            self.tnt_account = self.session.query(Account).filter(Account.name=='tnt_account_test').one()
        except NoResultFound:
            raise SkipTest("tnt_account_test not found.. run tnt_populate_responses.py")

        try:
            dsc = DataSourceController(self.session, account_id=self.tnt_account.id)
            ds = dsc.get_datasource_by('mondrian_project', '%s.TNT-CLM' % settings.SETTINGS_MODULE).pop()
        except IndexError:
            raise SkipTest("datasource not found.. run tnt_populate_responses.py")

        self.dsc = DataSourceController(self.session, account_id=self.tnt_account.id)

    def _execute_response(self, resp):
        """
        executes the response object like in iqportal/tnt/report/rpc.py
        """

        datasource_id = resp['datasource_id']
        mdx_info = resp['mdx_config']

        ds = self.dsc.get_datasources(datasource_id).pop()
        client = ds.connect(self.session, account_id=self.tnt_account.id)

        mdx = {}
        for key, value in mdx_info.iteritems():
            mdx[str(key)] = value

        if mdx.has_key('calculated_members'):
            del mdx['calculated_members']
        if mdx.has_key('result_full_headers'):
            del mdx['result_full_headers']

        mdx = client.build_select_statement(**mdx)
        result = client.execute_mdx(mdx)

        return mdx, result

    def test_satisfaction(self):

        r = TntCLMSatisfactionReporter(self.session, self.tnt_account)
        resp = r.get_mdx_config({}, "table")
        # no filters - should raise an exception cuz nothing on rows or columns
        self.assertRaises(PublicUserException, self._execute_response, resp)

    def test_loyalty_league_table(self):

        r = TntCLMLoyaltyLeagueTableReport(self.session, self.tnt_account)

        # splitby wave
        config = copy(self._empty_filters)
        config['splitby'] = 'waves'
        config['services'] = [
            "[Service].[Express]",
        ]

        resp = r.get_mdx_config(config, "table")
        mdx = self._execute_response(resp)[0]

        #print mdx

        self.assertMdx(mdx, """
            WITH MEMBER [KPI].[No. of Answers] AS '[KPI].[All KPIs]'

            MEMBER [KPI].[Rank] AS Rank(
                [Country].CurrentMember,
                Order(
                    {[Country].[All Countries].Children},
                    [KPI].[Score %],
                    BDESC
                ),
                [KPI].[Score %]
            )

                SET [Ordered Set] AS Filter(
                    Order({[Country].[All Countries].Children}, ([KPI].[Score %], {[Wave].[Code].Members}.Item(0) ), BDESC),
                    NOT ISEMPTY([KPI].[Score %]))

            SELECT NON EMPTY {[KPI].[Score %],[KPI].[Rank],[KPI].[No. of Answers]} ON COLUMNS,
                NON EMPTY {[Ordered Set]} ON ROWS
            FROM [TntCLMResponse]
            WHERE CROSSJOIN({[Position].[31]}, {[Service].[Express]})
        """)

        # splitby attribute
        config['splitby'] = 'attributes'

        resp = r.get_mdx_config(config, "table")
        mdx = self._execute_response(resp)[0]

        #print mdx

        self.assertMdx(mdx, """
            WITH MEMBER [KPI].[No. of Answers] AS '[KPI].[All KPIs]'

            MEMBER [KPI].[Rank] AS Rank(
                [Country].CurrentMember,
                Order(
                    {[Country].[All Countries].Children},
                    [KPI].[Score %],
                    BDESC
                ),
                [KPI].[Score %]
            )

                SET [Ordered Set] AS Filter(
                    Order({[Country].[All Countries].Children}, ([KPI].[Score %], {[Attribute].[Scale].Children}.Item(0) ), BDESC),
                    NOT ISEMPTY([KPI].[Score %]))

            SELECT NON EMPTY {[KPI].[Score %],[KPI].[Rank],[KPI].[No. of Answers]} ON COLUMNS,
                NON EMPTY {[Ordered Set]} ON ROWS
            FROM [TntCLMResponse]
            WHERE CROSSJOIN({[Position].[31]}, {[Service].[Express]})
        """)

    def _execute_and_check(self, resp):
        """
        Depending on Reporter and filters selected, executing mdx might raise an exception due to nothing on rows or columns
        """
        try:
            self._execute_response(resp)
        except PublicUserException, e:
            if e.message != 'At least one row and one column item are required':
                raise

    def test_simple_filters(self):
        """
        simple mdx test that runs a few filters against all reporters. we shouldnt get mdx errors at least
        """

        # FIXME: when putting TntCLMActionPlanningReport last in the list the test fails

        reporters = [TntCLMActionPlanningReport, TntCLMSatisfactionReporter, TntCLMSatisfactionGraphsReporter, TntCLMLoyaltyScoreReporter, TntCLMNCSLeagueTableReport]

        for Reporter in reporters:

            r = Reporter(self.session, self.tnt_account)

            # empty default filters
            config = copy(self._empty_filters)

            #print ' - default filters'
            resp = r.get_mdx_config(config, "table")
            self._execute_and_check(resp)

            # complex geolevels
            config['geolevels'] = [
                "MEMBER [Country].[AS - South-East Asia (07) wo India] AS '[Country].[ID]+[Country].[MY]+[Country].[PH]+[Country].[SG]+[Country].[TH]+[Country].[VN]'",
                "MEMBER [Country].[5. UK/IE BU] AS '[Country].[IE]+[Country].[GB]'"
            ]

            #print ' - geolevels'
            resp = r.get_mdx_config(config, "table")
            self._execute_and_check(resp)

            # add wave filter
            config['waves'] = [
                "[Wave].[Code].[W.1]",
                "[Wave].[Code].[W.2]"
            ]
            #print ' - waves'
            resp = r.get_mdx_config(config, "table")
            self._execute_and_check(resp)

            # add depot
            config['depots'] = [
                "[Depot].[BSB]",
            ]
            #print ' - depots'
            resp = r.get_mdx_config(config, "table")
            self._execute_and_check(resp)

            # services
            config['services'] = [
                "[Service].[Associates]",
            ]
            #print ' - services'
            resp = r.get_mdx_config(config, "table")
            self._execute_and_check(resp)

            # location
            config['services'] = [
                "MEMBER [Location].[Domestic] AS '[Location].[D]'",
            ]
            #print ' - location'
            resp = r.get_mdx_config(config, "table")
            self._execute_and_check(resp)

            # revenue_band
            config['revenue_band'] = [
                "[Revenue Band].[A]",
            ]
            #print ' - revenue band'
            resp = r.get_mdx_config(config, "table")
            self._execute_and_check(resp)

            # response_type
            config['response_type'] = [
                "MEMBER [Respondent Type].[Internet] AS '[Respondent Type].[I]'",
            ]
            #print ' - response type'
            resp = r.get_mdx_config(config, "table")
            self._execute_and_check(resp)



    def test_aggregator_run(self):
        """
        tests execution of an mdx statement, changes a response and verifies that the aggregator and clear cache gives different result
        """

        def _get_result():
            r = TntCLMSatisfactionReporter(self.session, self.tnt_account)
            config = copy(self._empty_filters)

            config['waves'] = [
                "[Wave].[Code].Members",
            ]
            resp = r.get_mdx_config(config, "table")
            mdx, result = self._execute_response(resp)

            return hashlib.sha1(unicode(result.get_data()).encode('utf-8')).hexdigest()

        # run aggregators to be sure what is in the db is same as mondrian
        AggregateResponsesTask().execute()

        # get first mdx result
        r1 = _get_result()

        # get a response for wave 4 where scale_answer 21 was selected
        response = self.sa_session.query(TntCLMWaveResponse) \
            .join(TntCLMWaveParticipant) \
            .join(TntCLMWaveInvitee) \
            .filter(TntCLMWaveInvitee.wave_id == 4) \
            .filter(TntCLMWaveResponse.wave_scale_answer_id == 21) \
            .order_by(TntCLMWaveResponse.id.desc()).all()[0]

        # update response and set answer=1
        time.sleep(1)
        response.last_updated = datetime.datetime.now()
        response.wave_scale_answer_id = 1
        self.sa_session.add(response)
        self.sa_session.flush()

        # trigger task that should update the aggregation tables and clear mondrian cache
        AggregateResponsesTask().execute()

        # get new result
        r2 = _get_result()

        # before asserting, restore db state to original
        time.sleep(1)
        response.wave_scale_answer_id = 21
        self.sa_session.add(response)
        self.sa_session.flush()
        AggregateResponsesTask().execute()

        # final assert, mdx result should be different
        self.assertNotEqual(r1, r2)

    def test_aggregator_on_location_2015(self):
        controller = TntCLMLocaleController(self.session, account_id=self.tnt_account.id)

        # we populate necessary tnt account data
        c = TntWaveController(self.session, account_id=self.tnt_account.id)
        country = self.sa_session.query(Country).filter(Country.name == 'Germany').first()
        locale = controller.save_or_update_locale(None, 'LOCAL', 'test_locale_location_2015', 'DE', 'ltr', country.id, location="D", location_2015="I", service='Postal')

        # make a wave
        wave_id = c.create_wave_36(wave_name='wave_random')
        wave = self.sa_session.query(TntCLMWave).filter(TntCLMWave.id == wave_id).one()
        c.translate(wave, locale, set_approved=True)

        agg = TntCLMTntCLMWaveAggregator()

        # add priv to view wave responses for the tnt accounts group
        c.grant_privilege_to(wave, [wave.PRIV_VIEW, wave.PRIV_VIEW_RESPONSE],
                                group=True, license=False, domain=False, target_group=self.tnt_account.groups[0].group)
        # make contact list
        contact_list = c.generateContactList(wave_id, 75)

        #set the wave online
        wave.status = TntCLMWave.STATUS_ONLINE
        wave.contact_lists.append(contact_list)
        self.sa_session.add(wave)
        self.sa_session.flush()
        agg.run(wave.id)

        # set wave invitees
        response_controller = ResponseController(self.sa_session)
        response_controller.save_or_update_invitees(wave)

        # generate some responses
        c.generateResponses(wave_id, None, False)

        # workaround for #2568.. re-run db_sync for tnt report so that aggTntCLMTntCLMWave gets filled for the aggregator
        from iqportal.tnt.report import db_sync
        db_sync.sync(self.sa_session)

        # run aggregator
        # dont refresh because the datasource doesnt exist yet.. iqubesync will run after this
        t = AggregateResponsesTask()
        t.execute(refresh=False)

        # check that agg table has location_2015 value for the new locale
        location_2015 = self.sa_session.execute('SELECT location, location_2015 FROM aggTntCLMFullWaveResponse \
                                                WHERE locale_id = %s AND wave_id = %s' % (locale.id, wave_id)).fetchone()
        self.assertEqual(location_2015[0], 'D')
        self.assertEqual(location_2015[1], 'I')

        location_2015 = self.sa_session.execute('SELECT location, location_2015 FROM aggTntCLMTrueWaveResponse \
                                                WHERE country_id = %s AND wave_id = %s' % (country.id, wave_id)).fetchone()
        self.assertEqual(location_2015[0], 'D')
        self.assertEqual(location_2015[1], 'I')

        location_2015 = self.sa_session.execute('SELECT location, location_2015 FROM aggTntCLMWaveResponse \
                                            WHERE wave_id = %s AND country_id = %s' % (wave_id, country.id)).fetchone()
        self.assertEqual(location_2015[0], "D")
        self.assertEqual(location_2015[1], 'I')