# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.test.unit import iqportal_unittest
from basesoft.web.core.exceptions import SecurityException,\
    PermissionDeniedException

from iqportal.tnt.core.controllers import TntController
from iqportal.tnt.core.models import TntGeolevel, TntDepot

class TntCoreControllerTestCase(iqportal_unittest.IQPortalTestCase):
    """
    This tests the tnt core controller
    """

    def setUp(self):
        """
        The script for reseting the db and creating the account+controller stuff
        """
        self.connect_db()

        self.account = self._create_active_account()
        self.group = self.account.groups[0].group

        self._attach_permissions_to_group(self.group, 'view_datasource')

        self.controller = TntController(self.session, account_id=self.account.id)

    def test_geolevels(self):
        """
        Tests the methods for geolevel manipulation
        """

        # define some test data
        BU = {
            'dbID': None,
            'name': u'Benelux',
            'typeName': u'TntBusinessUnit',
            'countries': [22, 155, 128], # ids of the countries: Belgium, Netherlands, Luxembourg
            'accounts': [self.account.id],
            'parent_geolevels': []
        }

        BU_RO = {
            'name': u'BeneluxRO',
            'typeName': u'TntBusinessUnit',
            'countries': [22, 155, 128, 181], # ids of the countries: Belgium, Netherlands, Luxembourg, Romania
            'accounts': [],
            'parent_geolevels': []
        }

        self.assertRaises(PermissionDeniedException, self.controller.get_geolevels) # we don't have view permission

        # create a geolevel
        self.assertRaises(PermissionDeniedException, self.controller.save_geolevel_state, BU) # we don't have create permission
        # add permissions; also need view_license so that it can link accounts to geolevel
        self._attach_permissions_to_group(self.group, 'create_geolevel', 'view_license')
        g = self.controller.save_geolevel_state(BU)

        # check get_geolevels
        self._attach_permissions_to_group(self.group, 'view_geolevel')
        results = self.controller.get_geolevels(g.id)
        self.assertEqual(len(results), 1)
        geo = results[0]
        self.assertEqual(geo.name, u'Benelux')
        self.assertEqual(len(geo.countries), 3)
        self.assertEqual(len(geo.accounts), 1)
        self.assertEqual(geo.accounts[0].id, self.account.id)
        self.assertEqual(len(geo.parent_geolevels), 0)
        self.assertEqual(len(geo.child_geolevels), 0)

        # test the update
        BU_RO['dbID'] = g.id
        self.assertRaises(PermissionDeniedException, self.controller.save_geolevel_state, BU_RO) # we don't have update permission
        self._attach_permissions_to_group(self.group, 'update_geolevel')
        updated_g = self.controller.save_geolevel_state(BU_RO)

        # check the state too
        state = self.controller.load_geolevel_state(updated_g.id)
        self.assertEqual(state['name'], u'BeneluxRO')
        self.assertEqual(state['dbID'], updated_g.id)
        self.assertEqual(state['typeName'], u'TntBusinessUnit')
        self.assertEqual(len(state['countries']), 4)
        for c in state['countries']:
            assert c in [22, 155, 128, 181]
        self.assertEqual(len(state['accounts']), 0)

        # check archiving
        self.controller.mark_as_archived(TntGeolevel, updated_g.id)
        results = self.controller.get_geolevels(updated_g.id)
        self.assertEqual(len(results), 0)

    def test_geolevel_hierarchy(self):
        self._attach_permissions_to_group(self.group, 'create_geolevel', 'view_geolevel', 'update_geolevel', 'view_license')

        BU = {
            'dbID': None,
            'name': u'Benelux',
            'typeName': u'TntBusinessUnit',
            'countries': [],
            'accounts': [],
            'parent_geolevels': []
        }
        bu = self.controller.save_geolevel_state(BU)

        REGION = {
            'dbID': None,
            'name': 'My Region',
            'typeName': u'TntRegion',
            'countries': [],
            'accounts': [],
            'parent_geolevels': [bu.id]
        }
        region = self.controller.save_geolevel_state(REGION)

        self.assertEqual(len(bu.child_geolevels), 1)
        self.assertEqual(len(region.child_geolevels), 0)

        self.assertEqual(len(bu.parent_geolevels), 0)
        self.assertEqual(len(region.parent_geolevels), 1)

        self.assertEqual(bu.child_geolevels[0].id, region.id)
        self.assertEqual(region.parent_geolevels[0].id, bu.id)

        SUBREGION = {
            'dbID': None,
            'name': 'My SubRegion',
            'typeName': u'TntSubregion',
            'countries': [],
            'accounts': [],
            'parent_geolevels': [region.id]
        }
        subregion = self.controller.save_geolevel_state(SUBREGION)
        self.assertEqual(subregion.parent_geolevels[0].id, region.id)

        # the following should error because one of the parent geolevels is child of another
        OTHER = {
            'dbID': None,
            'name': 'My something',
            'typeName': u'Other',
            'countries': [],
            'accounts': [],
            'parent_geolevels': [region.id, subregion.id]
        }
        self.assertRaises(SecurityException, self.controller.save_geolevel_state, OTHER)

    def test_depots(self):
        """
        Tests the methods for depot manipulation
        """

        # define some test data
        DEPOT = {
            'dbID': None,
            'default_code': u'HD',
            'label': u'Home Depot',
            'country': 22, # id of the country: Belgium
            'service': None,
            'aliases': [{'alias': u'Alias 1'}, {'alias': u'Alias 2'}]
        }

        DEPOT_RO = {
            'dbID': None,
            'default_code': u'RHD',
            'label': u'Romanian Home Depot',
            'country': 181, # id of Romania
            'service': None,
            'aliases': []
        }

        self.assertRaises(PermissionDeniedException, self.controller.get_depots) # we don't have view permission

        # create a depot
        self.assertRaises(PermissionDeniedException, self.controller.save_depot_state, DEPOT) # we don't have create permission
        self._attach_permissions_to_group(self.group, 'create_depot', 'depot_alias_manager')
        d = self.controller.save_depot_state(DEPOT)

        # check get_depots
        self._attach_permissions_to_group(self.group, 'view_depot')
        results = self.controller.get_depots(d.id) # check by id
        self.assertEqual(len(results), 1)
        d = results[0]
        self.assertEqual(len(d.aliases), 2)
        self.assertEqual([da.code for da in d.aliases], [u'Alias 1', u'Alias 2'])

        results = self.controller.get_depots(None, DEPOT["country"]) # check by country
        self.assertEqual(len(results), 1)

        depot = results[0]
        self.assertEqual(depot.label, u'Home Depot')
        self.assertEqual(depot.default_code, u'HD')
        self.assertEqual(depot.country.id, 22)

        # test the update
        DEPOT_RO['dbID'] = d.id
        self.assertRaises(PermissionDeniedException, self.controller.save_depot_state, DEPOT_RO) # we don't have update permission
        self._attach_permissions_to_group(self.group, 'update_depot')
        updated_depot = self.controller.save_depot_state(DEPOT_RO)

        # check the state too
        state = self.controller.load_depot_state(updated_depot.id)
        self.assertEqual(state['dbID'], updated_depot.id)
        self.assertEqual(state['label'], u'Romanian Home Depot')
        self.assertEqual(state['default_code'], u'RHD')
        self.assertEqual(state['country'], 181)

        # check archiving
        self.controller.mark_as_archived(TntDepot, updated_depot.id)
        results = self.controller.get_depots(updated_depot.id)
        self.assertEqual(len(results), 0)
